from mhcgnomes import Allele, parse


def test_Aona_DQA1_27_01():
    result = parse("Aona-DQA1*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQA1*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQA1*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQA1_27_02():
    result = parse("Aona-DQA1*27:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQA1*27:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQA1*27:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQA1_27_03():
    result = parse("Aona-DQA1*27:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQA1*27:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQA1*27:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQA1_27_04():
    result = parse("Aona-DQA1*27:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQA1*27:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQA1*27:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQA1_27_05():
    result = parse("Aona-DQA1*27:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQA1*27:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQA1*27:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_22_01_01():
    result = parse("Aona-DQB1*22:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*22:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DQB1*22:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_22_01_02():
    result = parse("Aona-DQB1*22:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*22:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DQB1*22:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_01():
    result = parse("Aona-DQB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_02():
    result = parse("Aona-DQB1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_03():
    result = parse("Aona-DQB1*23:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_04():
    result = parse("Aona-DQB1*23:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_05():
    result = parse("Aona-DQB1*23:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_06():
    result = parse("Aona-DQB1*23:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_07():
    result = parse("Aona-DQB1*23:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_08():
    result = parse("Aona-DQB1*23:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_09():
    result = parse("Aona-DQB1*23:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_10():
    result = parse("Aona-DQB1*23:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_11():
    result = parse("Aona-DQB1*23:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB1_23_12():
    result = parse("Aona-DQB1*23:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB1*23:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB1*23:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB2_01_01():
    result = parse("Aona-DQB2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DQB2_01_02():
    result = parse("Aona-DQB2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DQB2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DQB2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_01():
    result = parse("Aona-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_02():
    result = parse("Aona-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_03():
    result = parse("Aona-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_04():
    result = parse("Aona-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_05():
    result = parse("Aona-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_07():
    result = parse("Aona-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_08():
    result = parse("Aona-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_09():
    result = parse("Aona-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_11():
    result = parse("Aona-DRB1*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_12():
    result = parse("Aona-DRB1*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_13():
    result = parse("Aona-DRB1*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_14():
    result = parse("Aona-DRB1*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_15():
    result = parse("Aona-DRB1*03:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_16():
    result = parse("Aona-DRB1*03:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_01():
    result = parse("Aona-DRB3*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_02_01():
    result = parse("Aona-DRB3*06:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB3*06:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_03():
    result = parse("Aona-DRB3*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_04():
    result = parse("Aona-DRB3*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_05():
    result = parse("Aona-DRB3*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_06():
    result = parse("Aona-DRB3*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_07():
    result = parse("Aona-DRB3*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_08():
    result = parse("Aona-DRB3*06:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_09():
    result = parse("Aona-DRB3*06:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_10():
    result = parse("Aona-DRB3*06:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_11():
    result = parse("Aona-DRB3*06:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_12():
    result = parse("Aona-DRB3*06:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_01():
    result = parse("Aona-DRB*W013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_02():
    result = parse("Aona-DRB*W013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_01():
    result = parse("Aona-DRB*W018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_02():
    result = parse("Aona-DRB*W018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_01():
    result = parse("Aona-DRB*W029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_02():
    result = parse("Aona-DRB*W029:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_03():
    result = parse("Aona-DRB*W029:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W030_01_01():
    result = parse("Aona-DRB*W030:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W030:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB*W030:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB1_03_01():
    result = parse("Aoni-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB1_03_02():
    result = parse("Aoni-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DQB2_01_02():
    result = parse("Aotr-DQB2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DQB2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DQB2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB1_03_01():
    result = parse("Aotr-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB1_03_02():
    result = parse("Aotr-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB1_03_03():
    result = parse("Aotr-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB1_03_04():
    result = parse("Aotr-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB3_06_01():
    result = parse("Aotr-DRB3*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB3*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB3*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB3_06_02():
    result = parse("Aotr-DRB3*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB3*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB3*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB_W013_01():
    result = parse("Aotr-DRB*W013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB*W013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB*W013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB1_03_05():
    result = parse("Aotr-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB_W018_01():
    result = parse("Aotr-DRB*W018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB*W018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB*W018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB_W029_01():
    result = parse("Aotr-DRB*W029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB*W029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB*W029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_DRB_W029_02():
    result = parse("Aotr-DRB*W029:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-DRB*W029:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-DRB*W029:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_E_03_01():
    result = parse("Aotr-E*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-E*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-E*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_G_03_01():
    result = parse("Aotr-G*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-G*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-G*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_G_05_01():
    result = parse("Aotr-G*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-G*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-G*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_G_05_02():
    result = parse("Aotr-G*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-G*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aotr-G*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aotr_PS2_01():
    result = parse("Aotr-PS2*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aotr-PS2*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Aotr-PS2*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_B_01():
    result = parse("Atbe-B*01")
    assert result.__class__ is Allele, f'Expected parse("Atbe-B*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Atbe-B*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_DRB_W019_01():
    result = parse("Atbe-DRB*W019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atbe-DRB*W019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atbe-DRB*W019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_DRB_W038_01():
    result = parse("Atbe-DRB*W038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atbe-DRB*W038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atbe-DRB*W038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_DRB_W039_01():
    result = parse("Atbe-DRB*W039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atbe-DRB*W039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atbe-DRB*W039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_E_03_01():
    result = parse("Atbe-E*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atbe-E*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atbe-E*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_G_03_01():
    result = parse("Atbe-G*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atbe-G*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atbe-G*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_G_06_01():
    result = parse("Atbe-G*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atbe-G*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atbe-G*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atbe_G_03_02():
    result = parse("Atbe-G*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atbe-G*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atbe-G*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_E_03_01():
    result = parse("Atfu-E*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-E*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-E*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_E_03_02():
    result = parse("Atfu-E*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-E*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-E*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQA1_01_01():
    result = parse("Caja-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQA1_27_01():
    result = parse("Caja-DQA1*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQA1*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DQA1*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQB1_22_01_01():
    result = parse("Caja-DQB1*22:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQB1*22:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DQB1*22:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQB1_22_01_02():
    result = parse("Caja-DQB1*22:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQB1*22:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DQB1*22:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQB1_23_01():
    result = parse("Caja-DQB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DQB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQB1_23_02():
    result = parse("Caja-DQB1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQB1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DQB1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQB2_01_01():
    result = parse("Caja-DQB2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQB2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DQB2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DQB2_01_02():
    result = parse("Caja-DQB2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DQB2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DQB2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_01():
    result = parse("Caja-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_02():
    result = parse("Caja-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_03():
    result = parse("Caja-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_04():
    result = parse("Caja-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_05():
    result = parse("Caja-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_06():
    result = parse("Caja-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_07():
    result = parse("Caja-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_01_01():
    result = parse("Caja-DRB*W012:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_01():
    result = parse("Caja-DRB*W016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_04():
    result = parse("Caja-DRB*W016:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_05():
    result = parse("Caja-DRB*W016:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_06():
    result = parse("Caja-DRB*W016:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_07():
    result = parse("Caja-DRB*W016:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_08():
    result = parse("Caja-DRB*W016:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_09():
    result = parse("Caja-DRB*W016:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_10():
    result = parse("Caja-DRB*W016:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_11():
    result = parse("Caja-DRB*W016:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_12():
    result = parse("Caja-DRB*W016:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_13():
    result = parse("Caja-DRB*W016:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_14():
    result = parse("Caja-DRB*W016:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_15():
    result = parse("Caja-DRB*W016:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_E_03_01_01():
    result = parse("Caja-E*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-E*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-E*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_E_03_02():
    result = parse("Caja-E*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-E*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-E*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_02():
    result = parse("Caja-G*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_11_01_01():
    result = parse("Caja-G*11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*11:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_01():
    result = parse("Caja-G*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_01():
    result = parse("Caja-G*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_15_01():
    result = parse("Caja-G*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_PS2_01():
    result = parse("Caja-PS2*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-PS2*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-PS2*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB11_01_01():
    result = parse("Camo-DRB11*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB11*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB11*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB1_03_01():
    result = parse("Camo-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB1_03_02():
    result = parse("Camo-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB1_03_03():
    result = parse("Camo-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB1_03_04():
    result = parse("Camo-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB1_03_05():
    result = parse("Camo-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB3_05_01():
    result = parse("Camo-DRB3*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB3*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB3*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB3_05_02():
    result = parse("Camo-DRB3*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB3*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB3*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB3_05_03():
    result = parse("Camo-DRB3*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB3*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB3*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB3_05_04():
    result = parse("Camo-DRB3*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB3*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB3*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB3_06_01():
    result = parse("Camo-DRB3*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB3*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB3*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB3_07_01():
    result = parse("Camo-DRB3*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB3*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB3*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB_W014_01():
    result = parse("Camo-DRB*W014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB*W014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB*W014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB_W014_02():
    result = parse("Camo-DRB*W014:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB*W014:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB*W014:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Camo_DRB_W017_01():
    result = parse("Camo-DRB*W017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Camo-DRB*W017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Camo-DRB*W017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceap_DRB_W013_01():
    result = parse("Ceap-DRB*W013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceap-DRB*W013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceap-DRB*W013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceap_DRB_W013_02():
    result = parse("Ceap-DRB*W013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceap-DRB*W013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceap-DRB*W013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceap_DRB_W013_03():
    result = parse("Ceap-DRB*W013:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceap-DRB*W013:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceap-DRB*W013:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceap_DRB_W015_01():
    result = parse("Ceap-DRB*W015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceap-DRB*W015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceap-DRB*W015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceap_DRB_W015_02():
    result = parse("Ceap-DRB*W015:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceap-DRB*W015:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceap-DRB*W015:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceap_DRB_W032_01():
    result = parse("Ceap-DRB*W032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceap-DRB*W032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceap-DRB*W032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_01_01():
    result = parse("Cemi-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_01_02():
    result = parse("Cemi-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_01_03():
    result = parse("Cemi-DQA1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_01_04():
    result = parse("Cemi-DQA1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_01_05():
    result = parse("Cemi-DQA1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_05_01():
    result = parse("Cemi-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_20_01():
    result = parse("Cemi-DQA1*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_DQA1_20_02():
    result = parse("Cemi-DQA1*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-DQA1*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-DQA1*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cene_DQA1_26_01():
    result = parse("Cene-DQA1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cene-DQA1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cene-DQA1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cene_DQA1_26_02():
    result = parse("Cene-DQA1*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Cene-DQA1*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cene-DQA1*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Capy_DRB1_03_01():
    result = parse("Capy-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Capy-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Capy-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Capy_DRB_W012_01():
    result = parse("Capy-DRB*W012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Capy-DRB*W012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Capy-DRB*W012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQA1_01_01():
    result = parse("Chae-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQA1_01_02():
    result = parse("Chae-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQA1_05_01():
    result = parse("Chae-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQA1_05_02():
    result = parse("Chae-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQA1_26_01():
    result = parse("Chae-DQA1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQA1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQA1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQA1_26_02():
    result = parse("Chae-DQA1*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQA1*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQA1*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQB1_15_01():
    result = parse("Chae-DQB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQB1_18_01():
    result = parse("Chae-DQB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DQB1_18_02():
    result = parse("Chae-DQB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DQB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DQB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB1_03_01():
    result = parse("Chae-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB1_03_02():
    result = parse("Chae-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB1_03_04():
    result = parse("Chae-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB1_07_01():
    result = parse("Chae-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB1_07_02():
    result = parse("Chae-DRB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB1_07_03():
    result = parse("Chae-DRB1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB5_01_01():
    result = parse("Chae-DRB5*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB5*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB5*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB6_01_01N():
    result = parse("Chae-DRB6*01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB6*01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB6*01:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB_W006_01():
    result = parse("Chae-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB_W007_01():
    result = parse("Chae-DRB*W007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB*W007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB*W007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB_W007_02():
    result = parse("Chae-DRB*W007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB*W007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB*W007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_G_02_01():
    result = parse("Chae-G*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-G*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-G*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_G_02_02():
    result = parse("Chae-G*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-G*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-G*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cogu_DQA1_25_01():
    result = parse("Cogu-DQA1*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cogu-DQA1*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cogu-DQA1*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_01_01_01():
    result = parse("Gogo-A*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-A*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_02_01():
    result = parse("Gogo-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_04_01_01_01():
    result = parse("Gogo-A*04:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*04:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*04:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_05_01():
    result = parse("Gogo-A*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-A*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_01_01_01_01():
    result = parse("Gogo-B*01:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*01:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-B*01:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_01_02():
    result = parse("Gogo-B*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_01_03():
    result = parse("Gogo-B*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_02_01():
    result = parse("Gogo-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_03_01():
    result = parse("Gogo-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_04_01_01_01():
    result = parse("Gogo-B*04:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*04:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-B*04:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_05_01():
    result = parse("Gogo-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_05_02():
    result = parse("Gogo-B*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_01_01_01():
    result = parse("Gogo-C*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-C*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_01_01_02():
    result = parse("Gogo-C*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-C*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_01_03():
    result = parse("Gogo-C*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-C*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_02_01():
    result = parse("Gogo-C*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-C*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_02_02():
    result = parse("Gogo-C*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-C*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_02_03():
    result = parse("Gogo-C*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-C*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_02_04_01_01():
    result = parse("Gogo-C*02:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*02:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-C*02:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DMB_01_01_01():
    result = parse("Gogo-DMB*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DMB*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-DMB*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DMB_01_01_02():
    result = parse("Gogo-DMB*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DMB*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-DMB*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DMB_01_03():
    result = parse("Gogo-DMB*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DMB*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DMB*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DPB1_01():
    result = parse("Gogo-DPB1*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DPB1*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Gogo-DPB1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DPB1_02():
    result = parse("Gogo-DPB1*02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DPB1*02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Gogo-DPB1*02") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DPB1_03():
    result = parse("Gogo-DPB1*03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DPB1*03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Gogo-DPB1*03") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DPB1_04():
    result = parse("Gogo-DPB1*04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DPB1*04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Gogo-DPB1*04") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DPB1_05():
    result = parse("Gogo-DPB1*05")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DPB1*05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Gogo-DPB1*05") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA1_01_01():
    result = parse("Gogo-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA1_03_01():
    result = parse("Gogo-DQA1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA1_05_01():
    result = parse("Gogo-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA1_05_02():
    result = parse("Gogo-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA1_05_03():
    result = parse("Gogo-DQA1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA1_20_01():
    result = parse("Gogo-DQA1*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA1*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA1*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA2_01_01():
    result = parse("Gogo-DQA2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQA2_01_02():
    result = parse("Gogo-DQA2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQA2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQA2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQB1_02_02():
    result = parse("Gogo-DQB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQB1_05_02():
    result = parse("Gogo-DQB1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQB1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQB1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQB1_19_03():
    result = parse("Gogo-DQB1*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQB1*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQB1*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQB1_19_04():
    result = parse("Gogo-DQB1*19:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQB1*19:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQB1*19:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQB2_01_01():
    result = parse("Gogo-DQB2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQB2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQB2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DQB2_01_03():
    result = parse("Gogo-DQB2*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DQB2*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DQB2*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_01():
    result = parse("Gogo-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_08():
    result = parse("Gogo-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_10_01():
    result = parse("Gogo-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB3_01_01():
    result = parse("Gogo-DRB3*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB3*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB3*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB3_01_07():
    result = parse("Gogo-DRB3*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB3*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB3*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB3_04_01():
    result = parse("Gogo-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB5_01_01():
    result = parse("Gogo-DRB5*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB5*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB5*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB5_02_01():
    result = parse("Gogo-DRB5*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB5*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB5*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB5_04_03():
    result = parse("Gogo-DRB5*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB5*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB5*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB6_01_02N():
    result = parse("Gogo-DRB6*01:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB6*01:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB6*01:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB6_01_03N():
    result = parse("Gogo-DRB6*01:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB6*01:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB6*01:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB6_02_02N():
    result = parse("Gogo-DRB6*02:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB6*02:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB6*02:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB6_02_03N():
    result = parse("Gogo-DRB6*02:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB6*02:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB6*02:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB_W008_02():
    result = parse("Gogo-DRB*W008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB*W008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB*W008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_E_01_01_01():
    result = parse("Gogo-E*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-E*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-E*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_E_01_01_02():
    result = parse("Gogo-E*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-E*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-E*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_G_01_01():
    result = parse("Gogo-G*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-G*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-G*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_H_01():
    result = parse("Gogo-H*01")
    assert result.__class__ is Allele, f'Expected parse("Gogo-H*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Gogo-H*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_OKO_01_01():
    result = parse("Gogo-OKO*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-OKO*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-OKO*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_A_01():
    result = parse("Hyla-A*01")
    assert result.__class__ is Allele, f'Expected parse("Hyla-A*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Hyla-A*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_A_02():
    result = parse("Hyla-A*02")
    assert result.__class__ is Allele, f'Expected parse("Hyla-A*02") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Hyla-A*02") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_B_01():
    result = parse("Hyla-B*01")
    assert result.__class__ is Allele, f'Expected parse("Hyla-B*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Hyla-B*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQA1_01_01():
    result = parse("Hyla-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQA1_22_01():
    result = parse("Hyla-DQA1*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQA1*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQA1*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQA1_22_02():
    result = parse("Hyla-DQA1*22:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQA1*22:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQA1*22:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQA1_22_03():
    result = parse("Hyla-DQA1*22:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQA1*22:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQA1*22:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQA2_01_01():
    result = parse("Hyla-DQA2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQA2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQA2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQA2_01_02():
    result = parse("Hyla-DQA2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQA2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQA2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQB1_20_02():
    result = parse("Hyla-DQB1*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQB1*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQB1*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQB1_25_01():
    result = parse("Hyla-DQB1*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQB1*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQB1*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQB2_01_01():
    result = parse("Hyla-DQB2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQB2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQB2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hyla_DQB2_01_02():
    result = parse("Hyla-DQB2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hyla-DQB2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hyla-DQB2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Lero_G_03_01():
    result = parse("Lero-G*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Lero-G*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Lero-G*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Lero_G_03_02():
    result = parse("Lero-G*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Lero-G*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Lero-G*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Loat_DQA1_01_01():
    result = parse("Loat-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Loat-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Loat-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DPA1_02_01():
    result = parse("Maar-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQA1_01_01():
    result = parse("Maar-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQA1_01_02():
    result = parse("Maar-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQB1_06_01():
    result = parse("Maar-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQB1_06_02():
    result = parse("Maar-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQB1_06_03():
    result = parse("Maar-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQB1_06_04():
    result = parse("Maar-DQB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQB1_17_01():
    result = parse("Maar-DQB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DQB1_17_02():
    result = parse("Maar-DQB1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DQB1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DQB1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DRB1_03_01():
    result = parse("Maar-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DRB1_03_02():
    result = parse("Maar-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_DRB5_03_01():
    result = parse("Maar-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_I_01_01():
    result = parse("Maar-I*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-I*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-I*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_I_01_02():
    result = parse("Maar-I*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-I*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-I*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_I_01_03():
    result = parse("Maar-I*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-I*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-I*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_01():
    result = parse("Mafa-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_02():
    result = parse("Mafa-DPB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_01():
    result = parse("Mafa-DPB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_01_01():
    result = parse("Mafa-DQA1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_01():
    result = parse("Mafa-DQA1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_01_01():
    result = parse("Mafa-DQA1*26:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*26:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_01_01():
    result = parse("Mafa-DQB1*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_02():
    result = parse("Mafa-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_03():
    result = parse("Mafa-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_04():
    result = parse("Mafa-DQB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_06_01():
    result = parse("Mafa-DQB1*06:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_07_01():
    result = parse("Mafa-DQB1*06:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_08():
    result = parse("Mafa-DQB1*06:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_09():
    result = parse("Mafa-DQB1*06:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_10():
    result = parse("Mafa-DQB1*06:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_11_01():
    result = parse("Mafa-DQB1*06:11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:11:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_12_01():
    result = parse("Mafa-DQB1*06:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_13_01():
    result = parse("Mafa-DQB1*06:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_14():
    result = parse("Mafa-DQB1*06:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_15_01_01():
    result = parse("Mafa-DQB1*15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*15:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_15_02():
    result = parse("Mafa-DQB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_15_03_01():
    result = parse("Mafa-DQB1*15:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*15:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*15:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_16_01():
    result = parse("Mafa-DQB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_01():
    result = parse("Mafa-DQB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_02_01():
    result = parse("Mafa-DQB1*17:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_03():
    result = parse("Mafa-DQB1*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_04():
    result = parse("Mafa-DQB1*17:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_05():
    result = parse("Mafa-DQB1*17:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_01_01():
    result = parse("Mafa-DQB1*18:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*18:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_02():
    result = parse("Mafa-DQB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_03():
    result = parse("Mafa-DQB1*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_04():
    result = parse("Mafa-DQB1*18:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_05():
    result = parse("Mafa-DQB1*18:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_06():
    result = parse("Mafa-DQB1*18:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_08():
    result = parse("Mafa-DQB1*18:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_24_01():
    result = parse("Mafa-DQB1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_01_01():
    result = parse("Mafa-DRB1*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_02():
    result = parse("Mafa-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_03():
    result = parse("Mafa-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_04():
    result = parse("Mafa-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_05():
    result = parse("Mafa-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_06_01():
    result = parse("Mafa-DRB1*03:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_01():
    result = parse("Mafa-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_02_01():
    result = parse("Mafa-DRB1*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_01():
    result = parse("Mafa-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_02():
    result = parse("Mafa-DRB1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_01():
    result = parse("Mafa-DRB4*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_02():
    result = parse("Mafa-DRB4*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_01_01():
    result = parse("Mafa-DRB5*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB5*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_02():
    result = parse("Mafa-DRB5*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_01():
    result = parse("Mafa-DRB6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_01():
    result = parse("Mafa-DRB*W001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_02():
    result = parse("Mafa-DRB*W001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_03():
    result = parse("Mafa-DRB*W001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W020_01_01():
    result = parse("Mafa-DRB*W020:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W020:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W020:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_01():
    result = parse("Mafa-DRB*W002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W021_01():
    result = parse("Mafa-DRB*W021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W021_02():
    result = parse("Mafa-DRB*W021:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W021:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W021:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W021_03():
    result = parse("Mafa-DRB*W021:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W021:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W021:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W021_04_01():
    result = parse("Mafa-DRB*W021:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W021:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W021:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W025_01():
    result = parse("Mafa-DRB*W025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_01_01():
    result = parse("Mafa-DRB*W027:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W027:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_02():
    result = parse("Mafa-DRB*W027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_01():
    result = parse("Mafa-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_02():
    result = parse("Mafa-DRB*W003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W033_01_01():
    result = parse("Mafa-DRB*W033:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W033:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W033:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W036_01():
    result = parse("Mafa-DRB*W036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_01():
    result = parse("Mafa-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_02():
    result = parse("Mafa-DRB*W004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W005_01():
    result = parse("Mafa-DRB*W005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_01():
    result = parse("Mafa-DRB*W007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_01_01():
    result = parse("Mafa-E*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-E*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_02():
    result = parse("Mafa-E*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_01():
    result = parse("Mafa-G*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_02():
    result = parse("Mafa-G*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_03():
    result = parse("Mafa-G*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_04():
    result = parse("Mafa-G*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_01_01():
    result = parse("Mafa-I*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_01_01():
    result = parse("Mamu-AG*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-AG*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_02_01_01():
    result = parse("Mamu-AG*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-AG*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_02_01_02():
    result = parse("Mamu-AG*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-AG*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_02_02():
    result = parse("Mamu-AG*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-AG*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_03_01():
    result = parse("Mamu-AG*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-AG*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_03_02():
    result = parse("Mamu-AG*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-AG*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_01():
    result = parse("Mamu-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_08_01():
    result = parse("Mamu-DPB1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_08_02():
    result = parse("Mamu-DPB1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_05_01():
    result = parse("Mamu-DPB1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_01():
    result = parse("Mamu-DPB1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_22_01():
    result = parse("Mamu-DPB1*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_07_01():
    result = parse("Mamu-DPB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_07_02_01():
    result = parse("Mamu-DPB1*07:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*07:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPB1*07:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_06_02():
    result = parse("Mamu-DPB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_06_03():
    result = parse("Mamu-DPB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_15_01():
    result = parse("Mamu-DPB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_06_04():
    result = parse("Mamu-DPB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_04_01():
    result = parse("Mamu-DPB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_02_01():
    result = parse("Mamu-DPB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_03_01():
    result = parse("Mamu-DPB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_11_01():
    result = parse("Mamu-DPB1*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_02():
    result = parse("Mamu-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_04_01():
    result = parse("Mamu-DQA1*01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*01:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_05_01():
    result = parse("Mamu-DQA1*01:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*01:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_09():
    result = parse("Mamu-DQA1*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_06():
    result = parse("Mamu-DQA1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_07():
    result = parse("Mamu-DQA1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_08():
    result = parse("Mamu-DQA1*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_05_01():
    result = parse("Mamu-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_05_02():
    result = parse("Mamu-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_05_03():
    result = parse("Mamu-DQA1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_23_01():
    result = parse("Mamu-DQA1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_23_02():
    result = parse("Mamu-DQA1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_01_01():
    result = parse("Mamu-DQA1*24:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*24:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_02():
    result = parse("Mamu-DQA1*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_03():
    result = parse("Mamu-DQA1*24:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_04():
    result = parse("Mamu-DQA1*24:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_01():
    result = parse("Mamu-DQA1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_02_01():
    result = parse("Mamu-DQA1*26:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*26:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_03():
    result = parse("Mamu-DQA1*26:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_04():
    result = parse("Mamu-DQA1*26:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_01():
    result = parse("Mamu-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_02_01():
    result = parse("Mamu-DQB1*06:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_05():
    result = parse("Mamu-DQB1*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_06_01():
    result = parse("Mamu-DQB1*06:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_07():
    result = parse("Mamu-DQB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_08():
    result = parse("Mamu-DQB1*06:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_09():
    result = parse("Mamu-DQB1*06:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_10():
    result = parse("Mamu-DQB1*06:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_11_01():
    result = parse("Mamu-DQB1*06:11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:11:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_11_02():
    result = parse("Mamu-DQB1*06:11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:11:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_12():
    result = parse("Mamu-DQB1*06:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_13_01():
    result = parse("Mamu-DQB1*06:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_15_01_01():
    result = parse("Mamu-DQB1*15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*15:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_15_02():
    result = parse("Mamu-DQB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_15_03():
    result = parse("Mamu-DQB1*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_16_01():
    result = parse("Mamu-DQB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_16_02():
    result = parse("Mamu-DQB1*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_16_03():
    result = parse("Mamu-DQB1*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_02():
    result = parse("Mamu-DQB1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_03():
    result = parse("Mamu-DQB1*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_04():
    result = parse("Mamu-DQB1*17:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_05():
    result = parse("Mamu-DQB1*17:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_06_01():
    result = parse("Mamu-DQB1*17:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*17:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_07():
    result = parse("Mamu-DQB1*17:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_09():
    result = parse("Mamu-DQB1*17:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_01():
    result = parse("Mamu-DQB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_02():
    result = parse("Mamu-DQB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_03():
    result = parse("Mamu-DQB1*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_04():
    result = parse("Mamu-DQB1*18:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_07():
    result = parse("Mamu-DQB1*18:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_08():
    result = parse("Mamu-DQB1*18:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_09():
    result = parse("Mamu-DQB1*18:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_10():
    result = parse("Mamu-DQB1*18:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_11():
    result = parse("Mamu-DQB1*18:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_12():
    result = parse("Mamu-DQB1*18:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_13():
    result = parse("Mamu-DQB1*18:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_14():
    result = parse("Mamu-DQB1*18:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_24_01():
    result = parse("Mamu-DQB1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_01():
    result = parse("Mamu-DRA*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_02():
    result = parse("Mamu-DRA*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_03():
    result = parse("Mamu-DRA*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_04():
    result = parse("Mamu-DRA*01:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_05():
    result = parse("Mamu-DRA*01:02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_06():
    result = parse("Mamu-DRA*01:02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_07():
    result = parse("Mamu-DRA*01:02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_03_01():
    result = parse("Mamu-DRA*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_04_01():
    result = parse("Mamu-DRA*01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_04_02():
    result = parse("Mamu-DRA*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_05():
    result = parse("Mamu-DRA*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRA*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_03_01():
    result = parse("Mamu-DRB1*03:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*03:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_04():
    result = parse("Mamu-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_05():
    result = parse("Mamu-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_06():
    result = parse("Mamu-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_07():
    result = parse("Mamu-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_08():
    result = parse("Mamu-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_09():
    result = parse("Mamu-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_10():
    result = parse("Mamu-DRB1*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_11():
    result = parse("Mamu-DRB1*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_12():
    result = parse("Mamu-DRB1*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_13_01():
    result = parse("Mamu-DRB1*03:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*03:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_14():
    result = parse("Mamu-DRB1*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_15():
    result = parse("Mamu-DRB1*03:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_17():
    result = parse("Mamu-DRB1*03:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_18():
    result = parse("Mamu-DRB1*03:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_19():
    result = parse("Mamu-DRB1*03:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_21():
    result = parse("Mamu-DRB1*03:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_22():
    result = parse("Mamu-DRB1*03:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_23():
    result = parse("Mamu-DRB1*03:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_02():
    result = parse("Mamu-DRB1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_03():
    result = parse("Mamu-DRB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_04():
    result = parse("Mamu-DRB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_05():
    result = parse("Mamu-DRB1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_06_01():
    result = parse("Mamu-DRB1*04:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*04:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_07():
    result = parse("Mamu-DRB1*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_08():
    result = parse("Mamu-DRB1*04:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_07_01():
    result = parse("Mamu-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_07_02():
    result = parse("Mamu-DRB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_07_03_01():
    result = parse("Mamu-DRB1*07:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*07:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*07:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_07_03_02():
    result = parse("Mamu-DRB1*07:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*07:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*07:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_01():
    result = parse("Mamu-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_02_01():
    result = parse("Mamu-DRB1*10:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*10:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_03():
    result = parse("Mamu-DRB1*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_04():
    result = parse("Mamu-DRB1*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_05():
    result = parse("Mamu-DRB1*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_06():
    result = parse("Mamu-DRB1*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_07():
    result = parse("Mamu-DRB1*10:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_08():
    result = parse("Mamu-DRB1*10:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_01():
    result = parse("Mamu-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_02():
    result = parse("Mamu-DRB3*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_03():
    result = parse("Mamu-DRB3*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_04():
    result = parse("Mamu-DRB3*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_05():
    result = parse("Mamu-DRB3*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_06():
    result = parse("Mamu-DRB3*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_07():
    result = parse("Mamu-DRB3*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_08():
    result = parse("Mamu-DRB3*04:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_09():
    result = parse("Mamu-DRB3*04:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB4_01_01():
    result = parse("Mamu-DRB4*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB4*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB4*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB4_01_02():
    result = parse("Mamu-DRB4*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB4*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB4*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB4_01_03():
    result = parse("Mamu-DRB4*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB4*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB4*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_01():
    result = parse("Mamu-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_02():
    result = parse("Mamu-DRB5*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_03():
    result = parse("Mamu-DRB5*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_04():
    result = parse("Mamu-DRB5*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_05():
    result = parse("Mamu-DRB5*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_06():
    result = parse("Mamu-DRB5*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_07_01():
    result = parse("Mamu-DRB5*03:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB5*03:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_01N():
    result = parse("Mamu-DRB6*01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_02N():
    result = parse("Mamu-DRB6*01:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_03N():
    result = parse("Mamu-DRB6*01:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_05N():
    result = parse("Mamu-DRB6*01:05N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:05N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:05N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_06N():
    result = parse("Mamu-DRB6*01:06N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:06N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:06N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_07N():
    result = parse("Mamu-DRB6*01:07N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:07N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:07N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_08():
    result = parse("Mamu-DRB6*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_09N():
    result = parse("Mamu-DRB6*01:09N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:09N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:09N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_10():
    result = parse("Mamu-DRB6*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_11():
    result = parse("Mamu-DRB6*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_12_01():
    result = parse("Mamu-DRB6*01:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB6*01:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_13():
    result = parse("Mamu-DRB6*01:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_14():
    result = parse("Mamu-DRB6*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_15():
    result = parse("Mamu-DRB6*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_16():
    result = parse("Mamu-DRB6*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_17():
    result = parse("Mamu-DRB6*01:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_18():
    result = parse("Mamu-DRB6*01:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_19():
    result = parse("Mamu-DRB6*01:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_05_01():
    result = parse("Mamu-DRB6*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_01():
    result = parse("Mamu-DRB*W001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_02():
    result = parse("Mamu-DRB*W001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W020_01():
    result = parse("Mamu-DRB*W020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W020_02_01():
    result = parse("Mamu-DRB*W020:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W020:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W020:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W020_03():
    result = parse("Mamu-DRB*W020:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W020:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W020:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W002_01():
    result = parse("Mamu-DRB*W002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W021_01():
    result = parse("Mamu-DRB*W021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W021_02():
    result = parse("Mamu-DRB*W021:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W021:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W021:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W021_03():
    result = parse("Mamu-DRB*W021:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W021:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W021:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W021_04():
    result = parse("Mamu-DRB*W021:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W021:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W021:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_01():
    result = parse("Mamu-DRB*W025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_02():
    result = parse("Mamu-DRB*W025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_03():
    result = parse("Mamu-DRB*W025:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_04():
    result = parse("Mamu-DRB*W025:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_05_01():
    result = parse("Mamu-DRB*W025:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W025:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_07():
    result = parse("Mamu-DRB*W025:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W026_01():
    result = parse("Mamu-DRB*W026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W026_02():
    result = parse("Mamu-DRB*W026:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W026:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W026:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W026_03():
    result = parse("Mamu-DRB*W026:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W026:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W026:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_01():
    result = parse("Mamu-DRB*W027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_02():
    result = parse("Mamu-DRB*W027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W028_01():
    result = parse("Mamu-DRB*W028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_01():
    result = parse("Mamu-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_02():
    result = parse("Mamu-DRB*W003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_03_01():
    result = parse("Mamu-DRB*W003:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W003:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_04():
    result = parse("Mamu-DRB*W003:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_05():
    result = parse("Mamu-DRB*W003:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_06():
    result = parse("Mamu-DRB*W003:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_07():
    result = parse("Mamu-DRB*W003:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_08():
    result = parse("Mamu-DRB*W003:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_09():
    result = parse("Mamu-DRB*W003:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_10():
    result = parse("Mamu-DRB*W003:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W031_01():
    result = parse("Mamu-DRB*W031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_11():
    result = parse("Mamu-DRB*W003:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_12():
    result = parse("Mamu-DRB*W003:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_13():
    result = parse("Mamu-DRB*W003:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_01():
    result = parse("Mamu-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_02_01():
    result = parse("Mamu-DRB*W004:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W004:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_03():
    result = parse("Mamu-DRB*W004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_04():
    result = parse("Mamu-DRB*W004:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W004:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W005_01():
    result = parse("Mamu-DRB*W005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W005_02():
    result = parse("Mamu-DRB*W005:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W005:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W005:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_01():
    result = parse("Mamu-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_02():
    result = parse("Mamu-DRB*W006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_03():
    result = parse("Mamu-DRB*W006:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_04():
    result = parse("Mamu-DRB*W006:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_05():
    result = parse("Mamu-DRB*W006:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_06():
    result = parse("Mamu-DRB*W006:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_07():
    result = parse("Mamu-DRB*W006:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_08():
    result = parse("Mamu-DRB*W006:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_09_01():
    result = parse("Mamu-DRB*W006:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W006:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W007_01():
    result = parse("Mamu-DRB*W007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W007_02_01():
    result = parse("Mamu-DRB*W007:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W007:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W007:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_01_01():
    result = parse("Mamu-E*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-E*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_02():
    result = parse("Mamu-E*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_03():
    result = parse("Mamu-E*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_04():
    result = parse("Mamu-E*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_05():
    result = parse("Mamu-E*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_06():
    result = parse("Mamu-E*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_19():
    result = parse("Mamu-E*02:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_08():
    result = parse("Mamu-E*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_F_02_04_01():
    result = parse("Mamu-F*02:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-F*02:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-F*02:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_G_02_01():
    result = parse("Mamu-G*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-G*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-G*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_G_02_02():
    result = parse("Mamu-G*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-G*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-G*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_G_02_03():
    result = parse("Mamu-G*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-G*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-G*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_G_02_04():
    result = parse("Mamu-G*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-G*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-G*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_04():
    result = parse("Mamu-I*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_07():
    result = parse("Mamu-I*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_003_01():
    result = parse("Mane-A1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_032_01():
    result = parse("Mane-A1*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_004_01_01():
    result = parse("Mane-A1*004:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*004:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*004:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_083_01():
    result = parse("Mane-A1*083:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*083:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*083:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_18():
    result = parse("Mane-A2*05:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_009_01():
    result = parse("Mane-A1*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_006_01():
    result = parse("Mane-A1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A6_01_01_01():
    result = parse("Mane-A6*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A6*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A6*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_109_01():
    result = parse("Mane-B*109:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*109:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*109:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_122_01():
    result = parse("Mane-B*122:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*122:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*122:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_124_01():
    result = parse("Mane-B*124:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*124:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*124:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_017_01():
    result = parse("Mane-B*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_017_02():
    result = parse("Mane-B*017:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*017:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*017:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_016_02():
    result = parse("Mane-B*016:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*016:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*016:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_021_01():
    result = parse("Mane-B*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_107_03():
    result = parse("Mane-B*107:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*107:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*107:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_030_01():
    result = parse("Mane-B*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_01():
    result = parse("Mane-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_01():
    result = parse("Mane-DRB*W001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_02():
    result = parse("Mane-DRB*W001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_01():
    result = parse("Mane-DRB*W002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_02():
    result = parse("Mane-DRB*W002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_01_01():
    result = parse("Mane-I*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB1_03_01():
    result = parse("Masi-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB1_04_01():
    result = parse("Masi-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB4_01_01():
    result = parse("Masi-DRB4*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB4*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB4*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB_W001_01():
    result = parse("Masi-DRB*W001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB*W001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB*W001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB_W002_01():
    result = parse("Masi-DRB*W002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB*W002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB*W002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB_W003_01():
    result = parse("Masi-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB_W035_01():
    result = parse("Masi-DRB*W035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB*W035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB*W035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB_W004_01():
    result = parse("Masi-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masi_DRB_W004_02():
    result = parse("Masi-DRB*W004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masi-DRB*W004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masi-DRB*W004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_AG_01_01_01():
    result = parse("Paan-AG*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-AG*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-AG*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_AG_01_02():
    result = parse("Paan-AG*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-AG*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-AG*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_01_01():
    result = parse("Paan-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_02_01():
    result = parse("Paan-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_03_01():
    result = parse("Paan-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_01_01():
    result = parse("Paan-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_01():
    result = parse("Paan-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_A_03_01():
    result = parse("Pacy-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_A_03_02():
    result = parse("Pacy-A*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-A*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-A*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_A_01_01():
    result = parse("Pacy-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_A_02_01():
    result = parse("Pacy-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_B_05_01():
    result = parse("Pacy-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_B_01_01():
    result = parse("Pacy-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_B_02_01():
    result = parse("Pacy-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_B_04_01():
    result = parse("Pacy-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_01_01():
    result = parse("Pacy-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_01_02():
    result = parse("Pacy-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_01_03():
    result = parse("Pacy-DQA1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_05_01():
    result = parse("Pacy-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_05_02():
    result = parse("Pacy-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_23_01():
    result = parse("Pacy-DQA1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_26_01():
    result = parse("Pacy-DQA1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_26_02():
    result = parse("Pacy-DQA1*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_DQA1_26_03():
    result = parse("Pacy-DQA1*26:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-DQA1*26:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-DQA1*26:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pacy_E_02_01():
    result = parse("Pacy-E*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pacy-E*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pacy-E*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DPA1_02_01():
    result = parse("Paha-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DQA1_05_01():
    result = parse("Paha-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DQA1_05_02():
    result = parse("Paha-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DQA1_25_01():
    result = parse("Paha-DQA1*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DQA1*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DQA1*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DQB1_06_01():
    result = parse("Paha-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DQB1_06_02():
    result = parse("Paha-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DQB1_15_01():
    result = parse("Paha-DQB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DQB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DQB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_DRB1_03_01():
    result = parse("Paha-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_01_01():
    result = parse("Papa-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_02_01():
    result = parse("Papa-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_03_01():
    result = parse("Papa-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_04_01():
    result = parse("Papa-A*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_05_01():
    result = parse("Papa-A*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_01_01():
    result = parse("Papa-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_02_01():
    result = parse("Papa-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_03():
    result = parse("Papa-B*03")
    assert result.__class__ is Allele, f'Expected parse("Papa-B*03") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-B*03") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_04_01():
    result = parse("Papa-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_01_01_01():
    result = parse("Papa-C*03:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*03:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_01_01_02():
    result = parse("Papa-C*03:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*03:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DMB_01_01():
    result = parse("Papa-DMB*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DMB*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DMB*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DMB_01_02():
    result = parse("Papa-DMB*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DMB*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DMB*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DPB1_01():
    result = parse("Papa-DPB1*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DPB1*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-DPB1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DPB1_02():
    result = parse("Papa-DPB1*02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DPB1*02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-DPB1*02") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DPB1_03():
    result = parse("Papa-DPB1*03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DPB1*03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-DPB1*03") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DPB1_04():
    result = parse("Papa-DPB1*04")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DPB1*04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-DPB1*04") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DPB1_05():
    result = parse("Papa-DPB1*05")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DPB1*05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-DPB1*05") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DPB1_06():
    result = parse("Papa-DPB1*06")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DPB1*06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-DPB1*06") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DRB6_01_01():
    result = parse("Papa-DRB6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DRB6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DRB6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DRB6_01_02():
    result = parse("Papa-DRB6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DRB6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DRB6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_G_01_01():
    result = parse("Papa-G*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-G*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-G*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_H_01():
    result = parse("Papa-H*01")
    assert result.__class__ is Allele, f'Expected parse("Papa-H*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-H*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papp_DQA1_01_01():
    result = parse("Papp-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papp-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papp-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papp_DQA1_01_02():
    result = parse("Papp-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papp-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papp-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papp_DQA1_05_01():
    result = parse("Papp-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papp-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papp-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papp_DQA1_24_01():
    result = parse("Papp-DQA1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papp-DQA1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papp-DQA1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_01_01():
    result = parse("Patr-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_02_01():
    result = parse("Patr-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_03_01():
    result = parse("Patr-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_03_02():
    result = parse("Patr-A*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_04_01():
    result = parse("Patr-A*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_04_02():
    result = parse("Patr-A*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_04_04():
    result = parse("Patr-A*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_05_01():
    result = parse("Patr-A*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_06_01():
    result = parse("Patr-A*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_06_02():
    result = parse("Patr-A*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_07_01():
    result = parse("Patr-A*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_08_01_01_01():
    result = parse("Patr-A*08:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*08:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*08:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_08_02():
    result = parse("Patr-A*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_08_03():
    result = parse("Patr-A*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_09_01():
    result = parse("Patr-A*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_10_01():
    result = parse("Patr-A*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_11_01():
    result = parse("Patr-A*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_12_01():
    result = parse("Patr-A*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_13_01_01_01():
    result = parse("Patr-A*13:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*13:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*13:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_14_01():
    result = parse("Patr-A*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_15_01_01_01():
    result = parse("Patr-A*15:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*15:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*15:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_15_02():
    result = parse("Patr-A*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_16_01():
    result = parse("Patr-A*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_17_01_01_01():
    result = parse("Patr-A*17:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*17:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*17:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_17_02():
    result = parse("Patr-A*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_18_01():
    result = parse("Patr-A*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_01_01():
    result = parse("Patr-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_02_01():
    result = parse("Patr-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_03_01():
    result = parse("Patr-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_03_02():
    result = parse("Patr-B*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_04_01():
    result = parse("Patr-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_05_01():
    result = parse("Patr-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_05_02():
    result = parse("Patr-B*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_06_01():
    result = parse("Patr-B*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_07_01():
    result = parse("Patr-B*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_08_01():
    result = parse("Patr-B*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_08_02():
    result = parse("Patr-B*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_09_01():
    result = parse("Patr-B*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_10_01():
    result = parse("Patr-B*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_11_01():
    result = parse("Patr-B*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_11_02():
    result = parse("Patr-B*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_12_01():
    result = parse("Patr-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_12_02():
    result = parse("Patr-B*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_13_01():
    result = parse("Patr-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_14_01():
    result = parse("Patr-B*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_15_01():
    result = parse("Patr-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_16_01_01():
    result = parse("Patr-B*16:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*16:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-B*16:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_16_01_02():
    result = parse("Patr-B*16:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*16:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-B*16:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_16_02():
    result = parse("Patr-B*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_17_01():
    result = parse("Patr-B*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_17_02():
    result = parse("Patr-B*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_17_03():
    result = parse("Patr-B*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_18_01():
    result = parse("Patr-B*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_19_01():
    result = parse("Patr-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_20_01_01():
    result = parse("Patr-B*20:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*20:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-B*20:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_21_01():
    result = parse("Patr-B*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_22_01():
    result = parse("Patr-B*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_22_02():
    result = parse("Patr-B*22:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*22:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*22:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_01_01():
    result = parse("Patr-B*23:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-B*23:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_02():
    result = parse("Patr-B*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_03():
    result = parse("Patr-B*23:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*23:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_24_01():
    result = parse("Patr-B*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_24_02():
    result = parse("Patr-B*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_25_01():
    result = parse("Patr-B*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_26_01():
    result = parse("Patr-B*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_27_01():
    result = parse("Patr-B*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_28_01():
    result = parse("Patr-B*28:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*28:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*28:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_29_01():
    result = parse("Patr-B*29:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*29:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*29:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_30_01():
    result = parse("Patr-B*30:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*30:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*30:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_01_01():
    result = parse("Patr-C*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_02_01():
    result = parse("Patr-C*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_02_02():
    result = parse("Patr-C*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_02_03():
    result = parse("Patr-C*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_01():
    result = parse("Patr-C*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_02_01():
    result = parse("Patr-C*03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-C*03:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_02_02_01():
    result = parse("Patr-C*03:02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*03:02:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_04():
    result = parse("Patr-C*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_04_01():
    result = parse("Patr-C*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_05_01():
    result = parse("Patr-C*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_05_02():
    result = parse("Patr-C*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_06_01():
    result = parse("Patr-C*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_07_01():
    result = parse("Patr-C*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_08_01():
    result = parse("Patr-C*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_01_01_01():
    result = parse("Patr-C*09:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*09:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_01_01_02():
    result = parse("Patr-C*09:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*09:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_02_01_01():
    result = parse("Patr-C*09:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*09:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_03():
    result = parse("Patr-C*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_10_01():
    result = parse("Patr-C*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_11_01_01():
    result = parse("Patr-C*11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-C*11:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_12_01():
    result = parse("Patr-C*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_13_01():
    result = parse("Patr-C*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_13_02():
    result = parse("Patr-C*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DMB_01_01():
    result = parse("Patr-DMB*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DMB*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DMB*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DMB_01_02():
    result = parse("Patr-DMB*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DMB*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DMB*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DOB_01():
    result = parse("Patr-DOB*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DOB*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Patr-DOB*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_05():
    result = parse("Patr-DPA1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_03_01():
    result = parse("Patr-DPB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_03_02():
    result = parse("Patr-DPB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_03_03():
    result = parse("Patr-DPB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_03_04():
    result = parse("Patr-DPB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_01():
    result = parse("Patr-DPB1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_02():
    result = parse("Patr-DPB1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_03():
    result = parse("Patr-DPB1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_04():
    result = parse("Patr-DPB1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_05():
    result = parse("Patr-DPB1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_06():
    result = parse("Patr-DPB1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_07():
    result = parse("Patr-DPB1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_08():
    result = parse("Patr-DPB1*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_09():
    result = parse("Patr-DPB1*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_10():
    result = parse("Patr-DPB1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_11():
    result = parse("Patr-DPB1*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_12():
    result = parse("Patr-DPB1*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_02_01():
    result = parse("Patr-DPB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_13_01():
    result = parse("Patr-DPB1*01:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DPB1*01:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_14():
    result = parse("Patr-DPB1*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_15():
    result = parse("Patr-DPB1*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_01_01():
    result = parse("Patr-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_05_01():
    result = parse("Patr-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_05_02_01():
    result = parse("Patr-DQA1*05:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*05:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQA1*05:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_05_03():
    result = parse("Patr-DQA1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQA1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_20_01():
    result = parse("Patr-DQA1*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQA1*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_20_02():
    result = parse("Patr-DQA1*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQA1*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_20_03():
    result = parse("Patr-DQA1*20:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*20:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQA1*20:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA2_01_01():
    result = parse("Patr-DQA2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQA2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_03_02_01():
    result = parse("Patr-DQB1*03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQB1*03:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_03_03_01():
    result = parse("Patr-DQB1*03:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*03:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQB1*03:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_03_04():
    result = parse("Patr-DQB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_03_05():
    result = parse("Patr-DQB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_06_01():
    result = parse("Patr-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_06_02():
    result = parse("Patr-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_06_04():
    result = parse("Patr-DQB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_15_01():
    result = parse("Patr-DQB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_15_03():
    result = parse("Patr-DQB1*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB2_01_03():
    result = parse("Patr-DQB2*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB2*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB2*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_01():
    result = parse("Patr-DRB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_02():
    result = parse("Patr-DRB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_03():
    result = parse("Patr-DRB1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_04():
    result = parse("Patr-DRB1*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_05():
    result = parse("Patr-DRB1*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_13():
    result = parse("Patr-DRB1*02:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_02():
    result = parse("Patr-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_05():
    result = parse("Patr-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_06():
    result = parse("Patr-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_07_01():
    result = parse("Patr-DRB1*03:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DRB1*03:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_08():
    result = parse("Patr-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_09():
    result = parse("Patr-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_10():
    result = parse("Patr-DRB1*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_07_01():
    result = parse("Patr-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_07_02():
    result = parse("Patr-DRB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_10_01():
    result = parse("Patr-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_01_02():
    result = parse("Patr-DRB3*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_01():
    result = parse("Patr-DRB3*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_07():
    result = parse("Patr-DRB3*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_08():
    result = parse("Patr-DRB3*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_09():
    result = parse("Patr-DRB3*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_10():
    result = parse("Patr-DRB3*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_11():
    result = parse("Patr-DRB3*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_12():
    result = parse("Patr-DRB3*02:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_13():
    result = parse("Patr-DRB3*02:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_14():
    result = parse("Patr-DRB3*02:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_02_15():
    result = parse("Patr-DRB3*02:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*02:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*02:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB4_01_04N():
    result = parse("Patr-DRB4*01:04N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB4*01:04N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB4*01:04N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB4_01_05():
    result = parse("Patr-DRB4*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB4*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB4*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB4_01_06():
    result = parse("Patr-DRB4*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB4*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB4*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB4_01_07():
    result = parse("Patr-DRB4*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB4*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB4*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB4_02_01():
    result = parse("Patr-DRB4*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB4*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB4*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_01_01():
    result = parse("Patr-DRB5*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_01_02():
    result = parse("Patr-DRB5*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_01():
    result = parse("Patr-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_04():
    result = parse("Patr-DRB5*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_05():
    result = parse("Patr-DRB5*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_06():
    result = parse("Patr-DRB5*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_07():
    result = parse("Patr-DRB5*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_08():
    result = parse("Patr-DRB5*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_09():
    result = parse("Patr-DRB5*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_10():
    result = parse("Patr-DRB5*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_11():
    result = parse("Patr-DRB5*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_01_05N():
    result = parse("Patr-DRB6*01:05N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*01:05N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*01:05N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_01_06N():
    result = parse("Patr-DRB6*01:06N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*01:06N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*01:06N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_01_07N():
    result = parse("Patr-DRB6*01:07N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*01:07N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*01:07N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_01_09N():
    result = parse("Patr-DRB6*01:09N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*01:09N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*01:09N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_01_10N():
    result = parse("Patr-DRB6*01:10N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*01:10N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*01:10N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_03_01N():
    result = parse("Patr-DRB6*03:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*03:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*03:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_03_02N():
    result = parse("Patr-DRB6*03:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*03:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*03:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_03_03N():
    result = parse("Patr-DRB6*03:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*03:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*03:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_03_04N():
    result = parse("Patr-DRB6*03:04N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*03:04N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*03:04N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB6_03_05N():
    result = parse("Patr-DRB6*03:05N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB6*03:05N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB6*03:05N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB7_01_01N():
    result = parse("Patr-DRB7*01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB7*01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB7*01:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB_W008_01():
    result = parse("Patr-DRB*W008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB*W008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB*W008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_07_03():
    result = parse("Patr-DRB3*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_07_04():
    result = parse("Patr-DRB3*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_07_05():
    result = parse("Patr-DRB3*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_E_01_01_01_01():
    result = parse("Patr-E*01:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-E*01:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-E*01:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_E_01_01_01_02():
    result = parse("Patr-E*01:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-E*01:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-E*01:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_F_01_01_01():
    result = parse("Patr-F*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-F*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-F*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_H_01():
    result = parse("Patr-H*01")
    assert result.__class__ is Allele, f'Expected parse("Patr-H*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Patr-H*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_B_01():
    result = parse("Pipi-B*01")
    assert result.__class__ is Allele, f'Expected parse("Pipi-B*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Pipi-B*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_DRB3_05_01():
    result = parse("Pipi-DRB3*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-DRB3*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-DRB3*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_DRB_W012_01():
    result = parse("Pipi-DRB*W012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-DRB*W012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-DRB*W012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_DRB_W012_02():
    result = parse("Pipi-DRB*W012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-DRB*W012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-DRB*W012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_E_03_01():
    result = parse("Pipi-E*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-E*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-E*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_G_03_01():
    result = parse("Pipi-G*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-G*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-G*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_G_03_02():
    result = parse("Pipi-G*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-G*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-G*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_G_04_01():
    result = parse("Pipi-G*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-G*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-G*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Pipi_G_03_03():
    result = parse("Pipi-G*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Pipi-G*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Pipi-G*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_01_01():
    result = parse("Popy-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_02_01():
    result = parse("Popy-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_03_01():
    result = parse("Popy-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_01():
    result = parse("Popy-B*01")
    assert result.__class__ is Allele, f'Expected parse("Popy-B*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Popy-B*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_02():
    result = parse("Popy-B*02")
    assert result.__class__ is Allele, f'Expected parse("Popy-B*02") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Popy-B*02") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_03_01():
    result = parse("Popy-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_03_02():
    result = parse("Popy-B*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_04_01():
    result = parse("Popy-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_05_01():
    result = parse("Popy-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_06_01():
    result = parse("Popy-B*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_07_01():
    result = parse("Popy-B*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_01():
    result = parse("Popy-B*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_C_01_01_01():
    result = parse("Popy-C*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-C*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Popy-C*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_C_01_01_02():
    result = parse("Popy-C*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-C*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Popy-C*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_C_02_01():
    result = parse("Popy-C*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-C*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-C*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_C_02_02():
    result = parse("Popy-C*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-C*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-C*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_C_02_03():
    result = parse("Popy-C*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-C*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-C*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_C_03_01():
    result = parse("Popy-C*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-C*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-C*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DMB_02_01():
    result = parse("Popy-DMB*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DMB*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DMB*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DMB_02_02():
    result = parse("Popy-DMB*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DMB*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DMB*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DMB_02_03():
    result = parse("Popy-DMB*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DMB*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DMB*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DQA1_05_01():
    result = parse("Popy-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DQA1_21_01():
    result = parse("Popy-DQA1*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DQA1*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DQA1*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DQB1_06_03():
    result = parse("Popy-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DQB1_16_01():
    result = parse("Popy-DQB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DQB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DQB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DRB5_06_03():
    result = parse("Popy-DRB5*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DRB5*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DRB5*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DRB6_02_01N():
    result = parse("Popy-DRB6*02:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DRB6*02:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DRB6*02:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_DRB6_02_02N():
    result = parse("Popy-DRB6*02:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-DRB6*02:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-DRB6*02:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_E_01_01():
    result = parse("Popy-E*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-E*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-E*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_H_01():
    result = parse("Popy-H*01")
    assert result.__class__ is Allele, f'Expected parse("Popy-H*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Popy-H*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Safu_G_03_01():
    result = parse("Safu-G*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Safu-G*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Safu-G*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Safu_G_03_02():
    result = parse("Safu-G*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Safu-G*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Safu-G*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Safu_G_03_03():
    result = parse("Safu-G*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Safu-G*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Safu-G*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Safu_G_03_04():
    result = parse("Safu-G*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Safu-G*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Safu-G*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Safu_PS1_01():
    result = parse("Safu-PS1*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Safu-PS1*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Safu-PS1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Safu_PS2_01():
    result = parse("Safu-PS2*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Safu-PS2*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Safu-PS2*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sage_PS1_01():
    result = parse("Sage-PS1*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sage-PS1*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Sage-PS1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sage_PS2_01():
    result = parse("Sage-PS2*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sage-PS2*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Sage-PS2*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Samy_PS1_01():
    result = parse("Samy-PS1*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Samy-PS1*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Samy-PS1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Samy_PS2_01():
    result = parse("Samy-PS2*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Samy-PS2*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Samy-PS2*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DPB1_01_01():
    result = parse("Saoe-DPB1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DPB1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DPB1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DQA1_27_01():
    result = parse("Saoe-DQA1*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DQA1*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DQA1*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DQA1_27_02():
    result = parse("Saoe-DQA1*27:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DQA1*27:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DQA1*27:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DQA1_27_03():
    result = parse("Saoe-DQA1*27:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DQA1*27:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DQA1*27:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DQB1_22_01():
    result = parse("Saoe-DQB1*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DQB1*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DQB1*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DQB1_22_02():
    result = parse("Saoe-DQB1*22:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DQB1*22:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DQB1*22:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DQB1_23_01():
    result = parse("Saoe-DQB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DQB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DQB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DQB2_01_01():
    result = parse("Saoe-DQB2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DQB2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DQB2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_01():
    result = parse("Saoe-DRB11*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_02():
    result = parse("Saoe-DRB11*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_03():
    result = parse("Saoe-DRB11*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_04():
    result = parse("Saoe-DRB11*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_05():
    result = parse("Saoe-DRB11*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_06():
    result = parse("Saoe-DRB11*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_07N():
    result = parse("Saoe-DRB11*01:07N")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:07N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:07N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_08():
    result = parse("Saoe-DRB11*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_09():
    result = parse("Saoe-DRB11*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB11_01_10():
    result = parse("Saoe-DRB11*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB11*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB11*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB1_03_01():
    result = parse("Saoe-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB1_03_02():
    result = parse("Saoe-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB1_03_03N():
    result = parse("Saoe-DRB1*03:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB1*03:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB1*03:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB1_03_04N():
    result = parse("Saoe-DRB1*03:04N")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB1*03:04N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB1*03:04N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB1_03_05():
    result = parse("Saoe-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_01N():
    result = parse("Saoe-DRB3*05:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_02N():
    result = parse("Saoe-DRB3*05:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_03N():
    result = parse("Saoe-DRB3*05:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_05():
    result = parse("Saoe-DRB3*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_06():
    result = parse("Saoe-DRB3*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_07():
    result = parse("Saoe-DRB3*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_08():
    result = parse("Saoe-DRB3*05:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_09():
    result = parse("Saoe-DRB3*05:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_10():
    result = parse("Saoe-DRB3*05:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_11():
    result = parse("Saoe-DRB3*05:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_12():
    result = parse("Saoe-DRB3*05:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_13():
    result = parse("Saoe-DRB3*05:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_14():
    result = parse("Saoe-DRB3*05:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_15():
    result = parse("Saoe-DRB3*05:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_16():
    result = parse("Saoe-DRB3*05:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB3_05_17():
    result = parse("Saoe-DRB3*05:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB3*05:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB3*05:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB5_07_01():
    result = parse("Saoe-DRB5*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB5*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB5*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W012_01():
    result = parse("Saoe-DRB*W012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W012_03():
    result = parse("Saoe-DRB*W012:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W012:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W012:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W012_04():
    result = parse("Saoe-DRB*W012:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W012:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W012:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W012_05():
    result = parse("Saoe-DRB*W012:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W012:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W012:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_01():
    result = parse("Saoe-DRB*W022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_02():
    result = parse("Saoe-DRB*W022:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_03():
    result = parse("Saoe-DRB*W022:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_04():
    result = parse("Saoe-DRB*W022:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_05():
    result = parse("Saoe-DRB*W022:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_06():
    result = parse("Saoe-DRB*W022:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_07():
    result = parse("Saoe-DRB*W022:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_08():
    result = parse("Saoe-DRB*W022:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_09():
    result = parse("Saoe-DRB*W022:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_DRB_W022_10():
    result = parse("Saoe-DRB*W022:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-DRB*W022:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-DRB*W022:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_E_03_01():
    result = parse("Saoe-E*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-E*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-E*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_F_03_01():
    result = parse("Saoe-F*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-F*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-F*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_01():
    result = parse("Saoe-G*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_02():
    result = parse("Saoe-G*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_03():
    result = parse("Saoe-G*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_04():
    result = parse("Saoe-G*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_05():
    result = parse("Saoe-G*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_06():
    result = parse("Saoe-G*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_07():
    result = parse("Saoe-G*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_08():
    result = parse("Saoe-G*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_09():
    result = parse("Saoe-G*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_10():
    result = parse("Saoe-G*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_11():
    result = parse("Saoe-G*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_12Ps():
    result = parse("Saoe-G*03:12Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:12Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:12Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_13Ps():
    result = parse("Saoe-G*03:13Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:13Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:13Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_14Ps():
    result = parse("Saoe-G*03:14Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:14Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:14Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_16Ps():
    result = parse("Saoe-G*03:16Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:16Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:16Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_17Ps():
    result = parse("Saoe-G*03:17Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:17Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:17Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_18Ps():
    result = parse("Saoe-G*03:18Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:18Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:18Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_19Ps():
    result = parse("Saoe-G*03:19Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:19Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:19Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_G_03_20Ps():
    result = parse("Saoe-G*03:20Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-G*03:20Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Saoe-G*03:20Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_PS1_01():
    result = parse("Saoe-PS1*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Saoe-PS1*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Saoe-PS1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DPA1_05_01():
    result = parse("Sasc-DPA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DPA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DPA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DPA1_05_02():
    result = parse("Sasc-DPA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DPA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DPA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DPA1_06_01():
    result = parse("Sasc-DPA1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DPA1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DPA1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W012_01():
    result = parse("Sasc-DRB*W012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W012_02():
    result = parse("Sasc-DRB*W012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W012_03():
    result = parse("Sasc-DRB*W012:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W012:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W012:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W012_04():
    result = parse("Sasc-DRB*W012:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W012:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W012:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W014_01():
    result = parse("Sasc-DRB*W014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W014_02():
    result = parse("Sasc-DRB*W014:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W014:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W014:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W014_03():
    result = parse("Sasc-DRB*W014:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W014:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W014:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W014_04():
    result = parse("Sasc-DRB*W014:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W014:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W014:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W019_01():
    result = parse("Sasc-DRB*W019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W019_02():
    result = parse("Sasc-DRB*W019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_DRB_W019_03():
    result = parse("Sasc-DRB*W019:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-DRB*W019:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-DRB*W019:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasc_G_04_01():
    result = parse("Sasc-G*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasc-G*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasc-G*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_15():
    result = parse("Mamu-DQB1*18:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB1_10_01():
    result = parse("Mafu-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB1_03_01():
    result = parse("Mafu-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB1_03_02():
    result = parse("Mafu-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W001_01():
    result = parse("Mafu-DRB*W001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W001_02():
    result = parse("Mafu-DRB*W001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W025_01():
    result = parse("Mafu-DRB*W025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W004_01():
    result = parse("Mafu-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W031_01():
    result = parse("Mafa-DRB*W031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W060_01():
    result = parse("Mafu-DRB*W060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W003_03():
    result = parse("Mafu-DRB*W003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W004_02():
    result = parse("Mafu-DRB*W004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB3_04_01():
    result = parse("Mafu-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W006_01():
    result = parse("Mafu-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W006_02():
    result = parse("Mafu-DRB*W006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W025_02():
    result = parse("Mafu-DRB*W025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W006_03():
    result = parse("Mafu-DRB*W006:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W006:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W006:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W025_03():
    result = parse("Mafu-DRB*W025:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W025:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W025:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W003_01():
    result = parse("Mafu-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W003_02():
    result = parse("Mafu-DRB*W003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W060_02():
    result = parse("Mafu-DRB*W060:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W060:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W060:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W060_03():
    result = parse("Mafu-DRB*W060:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W060:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W060:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB1_10_02():
    result = parse("Mafu-DRB1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB1_10_03():
    result = parse("Mafu-DRB1*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB1*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB1*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB1_10_04():
    result = parse("Mafu-DRB1*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB1*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB1*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W061_01():
    result = parse("Mafu-DRB*W061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W061_02():
    result = parse("Mafu-DRB*W061:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W061:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W061:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W062_01():
    result = parse("Mafu-DRB*W062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_12():
    result = parse("Patr-DRB5*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_22_01():
    result = parse("Patr-A*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_14():
    result = parse("Patr-DRB1*02:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_17_03():
    result = parse("Patr-A*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_09_02():
    result = parse("Patr-A*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_35_01():
    result = parse("Patr-B*35:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*35:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*35:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_07_02():
    result = parse("Patr-B*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_02_04():
    result = parse("Patr-C*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_02_05():
    result = parse("Patr-C*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_05():
    result = parse("Patr-C*09:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*09:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_04():
    result = parse("Patr-C*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_04():
    result = parse("Mafa-DRB*W004:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_03():
    result = parse("Mafa-DRB*W027:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W027:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_03():
    result = parse("Mafa-DRB*W002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_04_01():
    result = parse("Mafa-DRB*W002:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W002:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_03():
    result = parse("Mafa-DRB*W004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_23():
    result = parse("Mafa-DRB1*03:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_10():
    result = parse("Mafa-DRB5*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_06():
    result = parse("Mafa-DRB1*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_07_01():
    result = parse("Mafa-DRB1*04:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_07_02():
    result = parse("Mafa-DRB1*04:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_03_01():
    result = parse("Mafa-DRB3*04:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB3*04:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_04():
    result = parse("Mafa-DRB3*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_05():
    result = parse("Mafa-DRB3*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_06():
    result = parse("Mafa-DRB3*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_04_01():
    result = parse("Mafa-DRB*W007:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W007:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_10():
    result = parse("Mafa-DRB*W007:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_04_02():
    result = parse("Mafa-DRB*W003:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W003:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_08():
    result = parse("Mafa-DRB*W003:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_09():
    result = parse("Mafa-DRB*W003:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W037_02():
    result = parse("Mafa-DRB*W037:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W037:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W037:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_05():
    result = parse("Mafa-DRB4*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_06():
    result = parse("Mafa-DRB4*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W053_02():
    result = parse("Mafa-DRB*W053:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W053:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W053:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_07():
    result = parse("Mafa-DRB3*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_05():
    result = parse("Mafa-DRB*W027:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W027:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W064_01():
    result = parse("Mafa-DRB*W064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_08():
    result = parse("Mafa-DRB5*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_09():
    result = parse("Mafa-DRB5*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W065_01():
    result = parse("Mafa-DRB*W065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_09():
    result = parse("Mafa-DRB1*10:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W026_01():
    result = parse("Mafa-DRB*W026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W026_02_01():
    result = parse("Mafa-DRB*W026:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W026:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W026:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_04():
    result = parse("Mafa-DRB*W027:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W027:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_16_02():
    result = parse("Mafa-DRB1*03:16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:16:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_09():
    result = parse("Mafa-DRB1*04:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_07_02_01():
    result = parse("Mafa-DRB1*07:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*07:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*07:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_07_02_02():
    result = parse("Mafa-DRB1*07:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*07:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*07:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_11():
    result = parse("Mafa-DRB5*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_04():
    result = parse("Mafa-DRB4*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_05():
    result = parse("Mafa-DRB1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_05():
    result = parse("Mafa-DRB*W002:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W002:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_12():
    result = parse("Mafa-DRB5*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_06():
    result = parse("Mafa-DRB*W027:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W027:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W025_03():
    result = parse("Mafa-DRB*W025:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W025:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W025:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_07_03():
    result = parse("Mafa-DRB1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W031_02():
    result = parse("Mafa-DRB*W031:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W031:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W031:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_06_01():
    result = parse("Mafa-DRB*W002:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W002:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_08():
    result = parse("Mafa-DRB4*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_09():
    result = parse("Mafa-DRB4*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_03():
    result = parse("Mafa-DRB*W007:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_10():
    result = parse("Mafa-DRB1*04:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W063_01():
    result = parse("Mafa-DRB*W063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W063_02():
    result = parse("Mafa-DRB*W063:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W063:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W063:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_11_01():
    result = parse("Mafa-DRB1*04:11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:11:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_08():
    result = parse("Mafa-DRB1*10:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_02_01():
    result = parse("Mamu-DPA1*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_03():
    result = parse("Mamu-DPA1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_04():
    result = parse("Mamu-DPA1*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_06_01_01():
    result = parse("Mamu-DPA1*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_01():
    result = parse("Mamu-DPA1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_05():
    result = parse("Mamu-DPA1*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_06():
    result = parse("Mamu-DPA1*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_14_01():
    result = parse("Mamu-DPB1*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_31():
    result = parse("Mafa-A2*05:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_05():
    result = parse("Mafa-DPB1*15:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_06():
    result = parse("Mafa-DPB1*15:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_04_01():
    result = parse("Mafa-DPB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_03():
    result = parse("Mafa-DPB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_02():
    result = parse("Mafa-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_15_01():
    result = parse("Mafa-DQA1*01:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_03_02():
    result = parse("Mafa-DQA1*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_04():
    result = parse("Mafa-DQA1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_05():
    result = parse("Mafa-DQA1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_01_01():
    result = parse("Mafa-DQA1*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_02_01():
    result = parse("Mafa-DQA1*05:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_03_01():
    result = parse("Mafa-DQA1*05:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_04_01():
    result = parse("Mafa-DQA1*05:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_23_01_01():
    result = parse("Mafa-DQA1*23:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*23:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*23:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_02_01():
    result = parse("Mafa-DQA1*24:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*24:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_02_02():
    result = parse("Mafa-DQA1*24:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*24:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_02():
    result = parse("Mafa-DQA1*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_15():
    result = parse("Mafa-DQB1*06:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_12_01():
    result = parse("Mafa-DRB1*03:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_13():
    result = parse("Mafa-DRB1*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_14():
    result = parse("Mafa-DRB1*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_15():
    result = parse("Mafa-DRB1*03:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_16_01():
    result = parse("Mafa-DRB1*03:16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:16:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_17_01():
    result = parse("Mafa-DRB1*03:17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:17:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_07_01():
    result = parse("Mafa-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_01():
    result = parse("Mafa-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_03():
    result = parse("Mafa-DRB4*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_01_02():
    result = parse("Mafa-DRB5*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB5*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_03():
    result = parse("Mafa-DRB5*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_04():
    result = parse("Mafa-DRB5*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_05():
    result = parse("Mafa-DRB5*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_06():
    result = parse("Mafa-DRB5*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_02():
    result = parse("Mafa-DRB6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_03_01():
    result = parse("Mafa-DRB*W003:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W003:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_04_01():
    result = parse("Mafa-DRB*W003:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W003:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_05():
    result = parse("Mafa-DRB*W003:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_06():
    result = parse("Mafa-DRB*W003:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_01():
    result = parse("Mafa-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_02():
    result = parse("Mafa-DRB*W007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W025_04():
    result = parse("Mafa-DRB*W025:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W025:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W025:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_02_01():
    result = parse("Mafa-DPB1*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_02():
    result = parse("Mafa-DPB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_03():
    result = parse("Mafa-DPB1*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_07_01():
    result = parse("Mafa-DPB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_10_01():
    result = parse("Mafa-DPB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_25_02():
    result = parse("Mafa-DPB1*25:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*25:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*25:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_20():
    result = parse("Mafa-A2*05:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_06_02():
    result = parse("Mafa-A2*05:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_18():
    result = parse("Mafa-A2*05:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_07():
    result = parse("Mafa-A2*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_19():
    result = parse("Mafa-A2*05:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_006_01_02():
    result = parse("Mafa-A1*006:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*006:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*006:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_04_01_01():
    result = parse("Mafa-A2*05:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_026_01_01_01():
    result = parse("Mamu-A1*026:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*026:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*026:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_02_02_01():
    result = parse("Mamu-A2*05:02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:02:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_03_01_01():
    result = parse("Mamu-A4*14:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_010_01():
    result = parse("Mamu-A1*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_011_02():
    result = parse("Mamu-A1*011:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*011:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*011:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_022_01_01_01():
    result = parse("Mamu-A1*022:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*022:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*022:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_01_01():
    result = parse("Mamu-A1*032:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*032:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_052_01():
    result = parse("Mamu-A1*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A7_01_03():
    result = parse("Mamu-A7*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A7*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_024_01_01_01():
    result = parse("Mamu-B*024:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*024:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*024:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_039_01_01_01():
    result = parse("Mamu-B*039:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*039:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*039:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_043_01_01_01():
    result = parse("Mamu-B*043:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*043:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*043:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_01_01_01():
    result = parse("Mamu-B*069:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*069:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_03_01_01():
    result = parse("Mamu-B*007:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*007:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_015_05_01_01():
    result = parse("Mamu-B*015:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*015:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*015:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_057_02():
    result = parse("Mamu-A1*057:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*057:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*057:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_04():
    result = parse("Mamu-A1*018:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_022_02():
    result = parse("Mamu-A1*022:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*022:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*022:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_011_03():
    result = parse("Mamu-A1*011:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*011:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*011:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_01_02_01():
    result = parse("Mamu-A2*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_03():
    result = parse("Mamu-A1*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_08():
    result = parse("Mamu-A1*003:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_02():
    result = parse("Mamu-B*069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_081_02():
    result = parse("Mamu-B*081:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*081:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*081:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_01():
    result = parse("Mamu-B*082:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_083_01_01_01():
    result = parse("Mamu-B*083:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*083:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*083:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_02_01_01():
    result = parse("Mamu-B*017:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*017:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_02_01_01():
    result = parse("Mamu-B*065:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*065:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_084_01():
    result = parse("Mamu-B*084:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*084:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*084:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_085_01():
    result = parse("Mamu-B*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_086_01():
    result = parse("Mamu-B*086:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*086:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*086:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_03_01_01():
    result = parse("Mamu-B*045:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*045:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_048_02():
    result = parse("Mamu-B*048:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*048:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*048:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_02():
    result = parse("Mamu-B*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_087_01_01_01():
    result = parse("Mamu-B*087:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*087:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*087:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_088_01_01_01():
    result = parse("Mamu-B*088:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*088:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*088:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_089_01_01():
    result = parse("Mamu-B*089:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*089:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*089:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_090_01():
    result = parse("Mamu-B*090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_091_01():
    result = parse("Mamu-B*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_18_01_01():
    result = parse("Mamu-I*01:18:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:18:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:18:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_01():
    result = parse("Mamu-I*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_16():
    result = parse("Mane-A2*05:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_21():
    result = parse("Mafa-A2*05:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_22():
    result = parse("Mafa-A2*05:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_23():
    result = parse("Mafa-A2*05:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_24():
    result = parse("Mafa-A2*05:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_25_01_01():
    result = parse("Mafa-A2*05:25:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:25:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:25:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_26():
    result = parse("Mafa-A2*05:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_27():
    result = parse("Mafa-A2*05:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_28():
    result = parse("Mafa-A2*05:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_29():
    result = parse("Mafa-A2*05:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_30():
    result = parse("Mafa-A2*05:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_05_04():
    result = parse("Mamu-DQA1*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_10():
    result = parse("Mamu-DQB1*17:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_DRB1_03_03():
    result = parse("Chae-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_07():
    result = parse("Mafa-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_08_01():
    result = parse("Mafa-DRB1*03:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_18():
    result = parse("Mafa-DRB1*03:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_19():
    result = parse("Mafa-DRB1*03:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_20():
    result = parse("Mafa-DRB1*03:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_21():
    result = parse("Mafa-DRB1*03:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_22():
    result = parse("Mafa-DRB1*03:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_04():
    result = parse("Mafa-DRB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_04():
    result = parse("Mafa-DRB1*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_05_01():
    result = parse("Mafa-DRB1*10:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*10:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_06():
    result = parse("Mafa-DRB1*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_07():
    result = parse("Mafa-DRB1*10:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_02_01():
    result = parse("Mafa-DRB3*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB3*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_07():
    result = parse("Mafa-DRB5*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_03():
    result = parse("Mafa-DRB6*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_04():
    result = parse("Mafa-DRB6*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_05():
    result = parse("Mafa-DRB6*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_04():
    result = parse("Mafa-DRB*W001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_02():
    result = parse("Mafa-DRB*W002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_07():
    result = parse("Mafa-DRB*W003:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_02():
    result = parse("Mafa-DRB*W006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_03_01():
    result = parse("Mafa-DRB*W006:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W006:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_03_02():
    result = parse("Mafa-DRB*W006:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W006:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_04():
    result = parse("Mafa-DRB*W006:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W033_03():
    result = parse("Mafa-DRB*W033:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W033:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W033:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W036_03():
    result = parse("Mafa-DRB*W036:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W036:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W036:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W049_01_01():
    result = parse("Mafa-DRB*W049:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W049:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W049:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W049_02():
    result = parse("Mafa-DRB*W049:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W049:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W049:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W053_01():
    result = parse("Mafa-DRB*W053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W054_01():
    result = parse("Mafa-DRB*W054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_09():
    result = parse("Mafa-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_10():
    result = parse("Mafa-DRB1*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_03():
    result = parse("Mafa-DRB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W025_02():
    result = parse("Mafa-DRB*W025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W036_02_01():
    result = parse("Mafa-DRB*W036:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W036:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W036:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W037_01():
    result = parse("Mafa-DRB*W037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W040_01_01():
    result = parse("Mafa-DRB*W040:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W040:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W040:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_01_01():
    result = parse("Mafa-DPB1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_03():
    result = parse("Mafa-DPB1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_05_01():
    result = parse("Mafa-DPB1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_04_01():
    result = parse("Mafa-DPB1*01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*01:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_05():
    result = parse("Mafa-DPB1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_06():
    result = parse("Mafa-DPB1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_01():
    result = parse("Mafa-DPB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_02():
    result = parse("Mafa-DPB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_01_01():
    result = parse("Mafa-DPB1*15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*15:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_04():
    result = parse("Mafa-DPB1*15:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_01():
    result = parse("Mafa-DPB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_02():
    result = parse("Mafa-DPB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_07_03_01():
    result = parse("Mafa-DPB1*07:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*07:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*07:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_25_01():
    result = parse("Mafa-DPB1*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_08_02():
    result = parse("Mafa-DPB1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_12_01():
    result = parse("Mafa-DPB1*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_12_02():
    result = parse("Mafa-DPB1*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_09_01_01():
    result = parse("Mafa-DPB1*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_09_02():
    result = parse("Mafa-DPB1*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_08_07():
    result = parse("Mafa-DPB1*08:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*08:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*08:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_05():
    result = parse("Mafa-DPB1*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_06_01():
    result = parse("Mafa-DPB1*06:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*06:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_18_01():
    result = parse("Mafa-DPB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_01():
    result = parse("Mafa-DPB1*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_02():
    result = parse("Mafa-DPB1*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_03():
    result = parse("Mafa-DPB1*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_04():
    result = parse("Mafa-DPB1*19:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*19:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_05():
    result = parse("Mafa-DPB1*19:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*19:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_06_01():
    result = parse("Mafa-DPB1*19:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*19:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_18_02():
    result = parse("Mafa-DPB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_03():
    result = parse("Mafa-DPB1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_03():
    result = parse("Mafa-DRB1*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_01():
    result = parse("Masp-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_02():
    result = parse("Masp-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_03():
    result = parse("Masp-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_04():
    result = parse("Masp-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_05():
    result = parse("Masp-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_04_01():
    result = parse("Masp-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_04_02():
    result = parse("Masp-DRB1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_04_03():
    result = parse("Masp-DRB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_04_04():
    result = parse("Masp-DRB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_04_05():
    result = parse("Masp-DRB1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB3_04_01():
    result = parse("Masp-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB3_04_02():
    result = parse("Masp-DRB3*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB3*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB3*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB5_03_01():
    result = parse("Masp-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB5_03_02():
    result = parse("Masp-DRB5*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB5*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB5*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB5_03_03():
    result = parse("Masp-DRB5*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB5*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB5*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB5_03_04():
    result = parse("Masp-DRB5*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB5*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB5*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB6_01_01():
    result = parse("Masp-DRB6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB6_01_02():
    result = parse("Masp-DRB6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB6_04_01():
    result = parse("Masp-DRB6*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB6*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB6*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB6_04_02():
    result = parse("Masp-DRB6*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB6*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB6*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB6_04_03():
    result = parse("Masp-DRB6*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB6*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB6*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB6_04_04():
    result = parse("Masp-DRB6*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB6*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB6*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W003_01():
    result = parse("Masp-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W003_02():
    result = parse("Masp-DRB*W003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W004_01():
    result = parse("Masp-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W025_01():
    result = parse("Masp-DRB*W025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W025_02():
    result = parse("Masp-DRB*W025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W025_03():
    result = parse("Masp-DRB*W025:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W025:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W025:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W025_04():
    result = parse("Masp-DRB*W025:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W025:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W025:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W026_01():
    result = parse("Masp-DRB*W026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W037_01():
    result = parse("Masp-DRB*W037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W051_01():
    result = parse("Masp-DRB*W051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W052_01():
    result = parse("Masp-DRB*W052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W048_01():
    result = parse("Paur-DRB*W048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W027_01():
    result = parse("Paur-DRB*W027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W055_01():
    result = parse("Paur-DRB*W055:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W055:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W055:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W056_01():
    result = parse("Paur-DRB*W056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB5_03_01():
    result = parse("Paur-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB1_03_01():
    result = parse("Paur-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB1_03_02():
    result = parse("Paur-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W036_01():
    result = parse("Paur-DRB*W036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W057_01():
    result = parse("Paur-DRB*W057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W058_01():
    result = parse("Paur-DRB*W058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W004_01():
    result = parse("Paur-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W003_01():
    result = parse("Paur-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W059_01():
    result = parse("Paur-DRB*W059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W056_02():
    result = parse("Paur-DRB*W056:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W056:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W056:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB1_03_03():
    result = parse("Paur-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W001_01():
    result = parse("Paur-DRB*W001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_11():
    result = parse("Mamu-DRB3*04:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_14():
    result = parse("Mamu-DRB*W003:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W003:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_10():
    result = parse("Mamu-DRB*W006:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W040_02():
    result = parse("Mamu-DRB*W040:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W040:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W040:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W040_01():
    result = parse("Mamu-DRB*W040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W037_01():
    result = parse("Mamu-DRB*W037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W036_02():
    result = parse("Mamu-DRB*W036:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W036:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W036:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W036_01():
    result = parse("Mamu-DRB*W036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_03():
    result = parse("Mamu-DRB*W027:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_09():
    result = parse("Mamu-DRB*W025:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_08():
    result = parse("Mamu-DRB*W025:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_06():
    result = parse("Mamu-DRB*W025:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W002_03():
    result = parse("Mamu-DRB*W002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W002_02():
    result = parse("Mamu-DRB*W002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_08():
    result = parse("Mamu-DRB5*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB4_01_04():
    result = parse("Mamu-DRB4*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB4*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB4*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_10():
    result = parse("Mamu-DRB3*04:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_09():
    result = parse("Mamu-DRB1*10:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_07_04():
    result = parse("Mamu-DRB1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_16():
    result = parse("Mamu-DRB1*04:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_15():
    result = parse("Mamu-DRB1*04:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_14():
    result = parse("Mamu-DRB1*04:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_12():
    result = parse("Mamu-DRB1*04:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_11():
    result = parse("Mamu-DRB1*04:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_10():
    result = parse("Mamu-DRB1*04:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_09():
    result = parse("Mamu-DRB1*04:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_25():
    result = parse("Mamu-DRB1*03:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_24():
    result = parse("Mamu-DRB1*03:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_20():
    result = parse("Mamu-DRB1*03:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_05_03():
    result = parse("Gogo-B*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_05():
    result = parse("Papa-B*05")
    assert result.__class__ is Allele, f'Expected parse("Papa-B*05") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-B*05") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_06():
    result = parse("Papa-B*06")
    assert result.__class__ is Allele, f'Expected parse("Papa-B*06") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Papa-B*06") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_07_01_01():
    result = parse("Papa-B*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Papa-B*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_02_03():
    result = parse("Patr-B*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_06_02():
    result = parse("Popy-B*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_09_01():
    result = parse("Popy-B*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_09():
    result = parse("Gogo-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_10():
    result = parse("Gogo-DRB1*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_11():
    result = parse("Gogo-DRB1*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_12():
    result = parse("Gogo-DRB1*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_13():
    result = parse("Gogo-DRB1*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_14():
    result = parse("Gogo-DRB1*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_10_02():
    result = parse("Gogo-DRB1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB3_01_08():
    result = parse("Gogo-DRB3*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB3*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB3*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB3_04_02():
    result = parse("Gogo-DRB3*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB3*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB3*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB5_05_04():
    result = parse("Gogo-DRB5*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB5*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB5*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB5_05_05():
    result = parse("Gogo-DRB5*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB5*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB5*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB5_05_06():
    result = parse("Gogo-DRB5*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB5*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB5*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB6_01_04():
    result = parse("Gogo-DRB6*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB6*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB6*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB6_02_04():
    result = parse("Gogo-DRB6*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB6*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB6*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_03_05():
    result = parse("Patr-DPB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_11():
    result = parse("Patr-DRB1*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_12():
    result = parse("Patr-DRB1*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoaz_DRB3_06_01():
    result = parse("Aoaz-DRB3*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoaz-DRB3*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoaz-DRB3*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoaz_DRB_W038_01():
    result = parse("Aoaz-DRB*W038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoaz-DRB*W038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoaz-DRB*W038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_17_01():
    result = parse("Aona-DRB1*03:17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB1*03:17:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_17_02():
    result = parse("Aona-DRB1*03:17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB1*03:17:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_18():
    result = parse("Aona-DRB1*03:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_19():
    result = parse("Aona-DRB1*03:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_21():
    result = parse("Aona-DRB1*03:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_22():
    result = parse("Aona-DRB1*03:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_23():
    result = parse("Aona-DRB1*03:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_24():
    result = parse("Aona-DRB1*03:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_25():
    result = parse("Aona-DRB1*03:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_26():
    result = parse("Aona-DRB1*03:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_27():
    result = parse("Aona-DRB1*03:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_13():
    result = parse("Aona-DRB3*06:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_14():
    result = parse("Aona-DRB3*06:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_15():
    result = parse("Aona-DRB3*06:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_16():
    result = parse("Aona-DRB3*06:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_18():
    result = parse("Aona-DRB3*06:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_20():
    result = parse("Aona-DRB3*06:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_21():
    result = parse("Aona-DRB3*06:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_22():
    result = parse("Aona-DRB3*06:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_23():
    result = parse("Aona-DRB3*06:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_24():
    result = parse("Aona-DRB3*06:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_03():
    result = parse("Aona-DRB*W013:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_04():
    result = parse("Aona-DRB*W013:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_05():
    result = parse("Aona-DRB*W013:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_06():
    result = parse("Aona-DRB*W013:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_07():
    result = parse("Aona-DRB*W013:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_08():
    result = parse("Aona-DRB*W013:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_09():
    result = parse("Aona-DRB*W013:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_10():
    result = parse("Aona-DRB*W013:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_11():
    result = parse("Aona-DRB*W013:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_12():
    result = parse("Aona-DRB*W013:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W013_13():
    result = parse("Aona-DRB*W013:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W013:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W013:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_03():
    result = parse("Aona-DRB*W018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_04():
    result = parse("Aona-DRB*W018:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_05():
    result = parse("Aona-DRB*W018:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_06():
    result = parse("Aona-DRB*W018:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_07():
    result = parse("Aona-DRB*W018:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_04():
    result = parse("Aona-DRB*W029:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_06():
    result = parse("Aona-DRB*W029:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_07():
    result = parse("Aona-DRB*W029:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_08():
    result = parse("Aona-DRB*W029:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_09():
    result = parse("Aona-DRB*W029:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W038_01():
    result = parse("Aona-DRB*W038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W038_02():
    result = parse("Aona-DRB*W038:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W038:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W038:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W041_01():
    result = parse("Aona-DRB*W041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W042_01():
    result = parse("Aona-DRB*W042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W044_01():
    result = parse("Aona-DRB*W044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W045_01():
    result = parse("Aona-DRB*W045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W046_01():
    result = parse("Aona-DRB*W046:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W046:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W046:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W046_02():
    result = parse("Aona-DRB*W046:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W046:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W046:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_01():
    result = parse("Aona-DRB*W047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_02():
    result = parse("Aona-DRB*W047:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_03():
    result = parse("Aona-DRB*W047:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_04_01():
    result = parse("Aona-DRB*W047:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB*W047:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_04_02():
    result = parse("Aona-DRB*W047:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB*W047:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_04_03():
    result = parse("Aona-DRB*W047:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB*W047:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_04_04():
    result = parse("Aona-DRB*W047:04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB*W047:04:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_05():
    result = parse("Aona-DRB*W047:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_06():
    result = parse("Aona-DRB*W047:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_07():
    result = parse("Aona-DRB*W047:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_08():
    result = parse("Aona-DRB*W047:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_09():
    result = parse("Aona-DRB*W047:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_10():
    result = parse("Aona-DRB*W047:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W047_11():
    result = parse("Aona-DRB*W047:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W047:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W047:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB1_03_03():
    result = parse("Aoni-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB1_03_04():
    result = parse("Aoni-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB1_03_05():
    result = parse("Aoni-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB1_03_06():
    result = parse("Aoni-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB1_03_07():
    result = parse("Aoni-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB3_06_01():
    result = parse("Aoni-DRB3*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB3*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB3*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_01():
    result = parse("Aoni-DRB*W013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_02():
    result = parse("Aoni-DRB*W013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_03():
    result = parse("Aoni-DRB*W013:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_04():
    result = parse("Aoni-DRB*W013:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_05():
    result = parse("Aoni-DRB*W013:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_06():
    result = parse("Aoni-DRB*W013:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_07():
    result = parse("Aoni-DRB*W013:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_08():
    result = parse("Aoni-DRB*W013:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W013_09():
    result = parse("Aoni-DRB*W013:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W013:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W013:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W018_01():
    result = parse("Aoni-DRB*W018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W029_01():
    result = parse("Aoni-DRB*W029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W029_02():
    result = parse("Aoni-DRB*W029:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W029:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W029:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W029_03():
    result = parse("Aoni-DRB*W029:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W029:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W029:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W029_04_01():
    result = parse("Aoni-DRB*W029:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W029:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aoni-DRB*W029:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W029_04_02():
    result = parse("Aoni-DRB*W029:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W029:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aoni-DRB*W029:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W038_01():
    result = parse("Aoni-DRB*W038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W042_01():
    result = parse("Aoni-DRB*W042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W043_01():
    result = parse("Aoni-DRB*W043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W043_02():
    result = parse("Aoni-DRB*W043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W043_03():
    result = parse("Aoni-DRB*W043:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W043:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W043:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W043_04():
    result = parse("Aoni-DRB*W043:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W043:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W043:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W043_05():
    result = parse("Aoni-DRB*W043:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W043:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W043:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W044_01():
    result = parse("Aoni-DRB*W044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aoni_DRB_W047_01():
    result = parse("Aoni-DRB*W047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aoni-DRB*W047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aoni-DRB*W047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB1_03_01():
    result = parse("Aovo-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB1_03_02():
    result = parse("Aovo-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB1_03_03():
    result = parse("Aovo-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB1_03_04():
    result = parse("Aovo-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W013_01_01():
    result = parse("Aovo-DRB*W013:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W013:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aovo-DRB*W013:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W013_01_02():
    result = parse("Aovo-DRB*W013:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W013:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aovo-DRB*W013:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W013_02():
    result = parse("Aovo-DRB*W013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W029_01():
    result = parse("Aovo-DRB*W029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W043_01():
    result = parse("Aovo-DRB*W043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W047_01():
    result = parse("Aovo-DRB*W047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W047_02():
    result = parse("Aovo-DRB*W047:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W047:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W047:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_11():
    result = parse("Caja-DRB1*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_12():
    result = parse("Caja-DRB1*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_14():
    result = parse("Caja-DRB1*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_08():
    result = parse("Caja-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_02_01():
    result = parse("Caja-DRB*W012:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_16():
    result = parse("Caja-DRB*W016:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_20():
    result = parse("Caja-DRB*W016:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_22():
    result = parse("Caja-DRB*W016:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_09():
    result = parse("Caja-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_10():
    result = parse("Caja-DRB1*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_17():
    result = parse("Caja-DRB*W016:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_18():
    result = parse("Caja-DRB*W016:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_23():
    result = parse("Caja-DRB*W016:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_24():
    result = parse("Caja-DRB*W016:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_01_01():
    result = parse("Mafa-DRA*01:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_02_01_01_01():
    result = parse("Mafa-DRA*02:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*02:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*02:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_06_01():
    result = parse("Mafa-DRA*01:01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:01:06:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_03_01():
    result = parse("Mafa-DRA*01:02:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:02:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_05_01():
    result = parse("Mafa-DRA*01:02:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:02:05:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_08():
    result = parse("Mafa-DRA*01:03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:08") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_08():
    result = parse("Mafa-DRA*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_07_01():
    result = parse("Mafa-DRA*01:02:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:02:07:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_04_02_01():
    result = parse("Mafa-DRA*01:04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:04:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_04_01():
    result = parse("Mafa-DRA*01:01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:01:04:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_08_01():
    result = parse("Mafa-DRA*01:02:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:02:08:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_02_01_03_01():
    result = parse("Mafa-DRA*02:01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*02:01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*02:01:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_07():
    result = parse("Mafa-DRA*01:01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:01:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_04_01_01():
    result = parse("Mafa-DRA*01:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_11_01():
    result = parse("Mafa-DRA*01:02:11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:02:11:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_04_03_01():
    result = parse("Mafa-DRA*01:04:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:04:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:04:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_02_01_02_01():
    result = parse("Mafa-DRA*02:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*02:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*02:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_01():
    result = parse("Mafa-DPA1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_02():
    result = parse("Mafa-DPA1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_13_01():
    result = parse("Mafa-DPA1*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_04_01():
    result = parse("Mafa-DPA1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_13_03():
    result = parse("Mafa-DPA1*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_20_01():
    result = parse("Mafa-DPB1*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_03():
    result = parse("Mafa-DQA1*24:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_06_01():
    result = parse("Mafa-DQA1*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_07_01():
    result = parse("Mafa-DQA1*01:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_08_01():
    result = parse("Mafa-DQA1*01:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_01_02():
    result = parse("Mafa-DQB1*06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_01():
    result = parse("Mafa-DRA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_028_04_01_01():
    result = parse("Mafa-B*028:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*028:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*028:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_14():
    result = parse("Mamu-DQB1*06:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_F_02_01():
    result = parse("Mamu-F*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-F*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-F*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_20():
    result = parse("Mamu-E*02:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_10():
    result = parse("Mamu-E*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_11():
    result = parse("Mamu-E*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_025_02_02_01():
    result = parse("Mamu-A1*025:02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*025:02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*025:02:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_092_01_01_01():
    result = parse("Mamu-B*092:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*092:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*092:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_02_01_01():
    result = parse("Mamu-B*079:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*079:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_086_01_01():
    result = parse("Mafa-A1*086:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*086:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*086:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_03_01():
    result = parse("Mafa-A4*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A4*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_078_02():
    result = parse("Mafa-A1*078:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*078:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*078:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_04():
    result = parse("Mafa-A1*010:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*010:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_05_01_01():
    result = parse("Mafa-A1*010:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*010:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_070_01_01_01():
    result = parse("Mafa-A1*070:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*070:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*070:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_03():
    result = parse("Mafa-A1*018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_087_01():
    result = parse("Mafa-A1*087:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*087:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*087:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_062_02():
    result = parse("Mafa-A1*062:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*062:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*062:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_066_03():
    result = parse("Mafa-A1*066:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*066:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*066:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_041_03_01():
    result = parse("Mafa-A1*041:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*041:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*041:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_124_01_01():
    result = parse("Mafa-B*124:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*124:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*124:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_104_03():
    result = parse("Mafa-B*104:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*104:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*104:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_02():
    result = parse("Mafa-B*098:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_01_01_01():
    result = parse("Mafa-B*011:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*011:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_075_01_01_01():
    result = parse("Mafa-B*075:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*075:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*075:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_01():
    result = parse("Mafa-B*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_136_02():
    result = parse("Mafa-B*136:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*136:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*136:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_004_03_01():
    result = parse("Mafa-B*004:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*004:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*004:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_004_01():
    result = parse("Mafa-B*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_03_01_01():
    result = parse("Mafa-B*137:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*137:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_076_01():
    result = parse("Mafa-B*076:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*076:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*076:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_07():
    result = parse("Mafa-B*013:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_047_01():
    result = parse("Mafa-B*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_153_01():
    result = parse("Mafa-B*153:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*153:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*153:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_154_01_01():
    result = parse("Mafa-B*154:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*154:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*154:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_02():
    result = parse("Mafa-B*045:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*045:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_004_02():
    result = parse("Mafa-B*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_155_01():
    result = parse("Mafa-B*155:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*155:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*155:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_078_01_02():
    result = parse("Mafa-A1*078:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*078:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*078:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_088_01():
    result = parse("Mafa-A1*088:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*088:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*088:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_02_01():
    result = parse("Mafa-A1*010:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*010:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_02():
    result = parse("Mafa-A1*022:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_01():
    result = parse("Mafa-A1*089:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_03():
    result = parse("Mafa-A1*060:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*060:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_06():
    result = parse("Mafa-A1*010:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*010:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_01_01():
    result = parse("Mafa-A1*090:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*090:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_016_01():
    result = parse("Mafa-B*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_01_01():
    result = parse("Mafa-B*101:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*101:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_076_03():
    result = parse("Mafa-B*076:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*076:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*076:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_02_01():
    result = parse("Mafa-B*048:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*048:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W026_04():
    result = parse("Mamu-DRB*W026:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W026:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W026:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_26():
    result = parse("Mamu-DRB1*03:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_17():
    result = parse("Mamu-DRB1*04:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_07_03_03():
    result = parse("Mamu-DRB1*07:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*07:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*07:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_32():
    result = parse("Mafa-A2*05:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_01_01_01():
    result = parse("Mafa-B*045:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*045:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_003_01():
    result = parse("Mafa-B*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_01_01_01():
    result = parse("Mafa-B*060:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_037_02():
    result = parse("Mafa-B*037:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*037:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*037:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_156_01():
    result = parse("Mafa-B*156:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*156:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*156:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_005_01N():
    result = parse("Mafa-B*005:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*005:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*005:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB5_03_02():
    result = parse("Paur-DRB5*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB5*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB5*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W028_01():
    result = parse("Paur-DRB*W028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_056_03():
    result = parse("Mamu-A1*056:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*056:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*056:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_091_01():
    result = parse("Mamu-A1*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_01():
    result = parse("Mamu-A1*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_063_01_01():
    result = parse("Mamu-A1*063:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*063:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*063:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_01_02():
    result = parse("Mamu-A1*032:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*032:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_02_02():
    result = parse("Mamu-A1*004:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*004:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_092_01():
    result = parse("Mamu-A1*092:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*092:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*092:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_085_01():
    result = parse("Mamu-A1*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_045_01():
    result = parse("Mamu-A1*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB6_01_01():
    result = parse("Paur-DRB6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB6_01_02():
    result = parse("Paur-DRB6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB6_01_03():
    result = parse("Paur-DRB6*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB6*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB6*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB_W053_01():
    result = parse("Paur-DRB*W053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB*W053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB*W053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paur_DRB6_01_04():
    result = parse("Paur-DRB6*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paur-DRB6*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paur-DRB6*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_071_02():
    result = parse("Mafa-A1*071:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*071:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*071:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_076_01():
    result = parse("Mafa-A1*076:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*076:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*076:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_02():
    result = parse("Mafa-A1*060:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*060:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_002_01_01():
    result = parse("Mafa-A1*002:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*002:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*002:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_077_01_01():
    result = parse("Mafa-A1*077:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*077:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*077:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_02():
    result = parse("Mafa-A6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_078_01_01():
    result = parse("Mafa-A1*078:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*078:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*078:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_009_01():
    result = parse("Mafa-A1*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_01():
    result = parse("Mafa-A1*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_032_02():
    result = parse("Mafa-A1*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_054_01():
    result = parse("Mafa-A1*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_02():
    result = parse("Mafa-A1*018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_079_01():
    result = parse("Mafa-A1*079:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*079:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*079:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_015_01_01_01():
    result = parse("Mafa-A1*015:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*015:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*015:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_045_01_01_01():
    result = parse("Mafa-A1*045:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*045:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*045:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_01():
    result = parse("Mafa-A1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_01_01_01():
    result = parse("Mafa-A1*018:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*018:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_036_02():
    result = parse("Mafa-A1*036:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*036:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*036:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_036_03():
    result = parse("Mafa-A1*036:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*036:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*036:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_036_04():
    result = parse("Mafa-A1*036:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*036:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*036:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_011_01():
    result = parse("Mafa-A1*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_040_03_01_01():
    result = parse("Mafa-A1*040:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*040:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*040:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_01():
    result = parse("Mafa-A1*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_011_02():
    result = parse("Mafa-A1*011:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*011:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*011:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_006_01_01():
    result = parse("Mafa-A1*006:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*006:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*006:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_063_03_01():
    result = parse("Mafa-A1*063:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*063:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*063:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_063_02_01_01():
    result = parse("Mafa-A1*063:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*063:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*063:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_02_01():
    result = parse("Mafa-A1*001:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*001:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_03():
    result = parse("Mafa-A1*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_027_01_01_01():
    result = parse("Mafa-A1*027:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*027:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*027:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_008_01():
    result = parse("Mafa-A1*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_01_01_01():
    result = parse("Mafa-A4*01:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*01:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_01_01_01():
    result = parse("Mafa-A5*30:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A5*30:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_031_01():
    result = parse("Mafa-A1*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_032_01():
    result = parse("Mafa-A1*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_033_01_01_01():
    result = parse("Mafa-A1*033:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*033:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*033:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_02_01_01():
    result = parse("Mafa-A3*13:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A3*13:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_036_01():
    result = parse("Mafa-A1*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_027_02():
    result = parse("Mafa-A1*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_038_01_01_01():
    result = parse("Mafa-A1*038:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*038:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*038:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_040_01_01_01():
    result = parse("Mafa-A1*040:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*040:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*040:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_041_01_01_01():
    result = parse("Mafa-A1*041:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*041:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*041:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_042_01():
    result = parse("Mafa-A1*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_01_01_01():
    result = parse("Mafa-A1*043:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*043:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_026_01():
    result = parse("Mafa-A1*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_02():
    result = parse("Mafa-A4*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_01_01():
    result = parse("Mafa-A1*001:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*001:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_01():
    result = parse("Mafa-A1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_02():
    result = parse("Mafa-A1*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_03():
    result = parse("Mafa-A1*010:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*010:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_02():
    result = parse("Mafa-A1*007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_019_01():
    result = parse("Mafa-A1*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_031_02():
    result = parse("Mafa-A1*031:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*031:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*031:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_040_02():
    result = parse("Mafa-A1*040:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*040:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*040:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_041_02():
    result = parse("Mafa-A1*041:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*041:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*041:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_02():
    result = parse("Mafa-A1*043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_052_01():
    result = parse("Mafa-A1*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_053_01_01():
    result = parse("Mafa-A1*053:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*053:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*053:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_055_01():
    result = parse("Mafa-A1*055:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*055:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*055:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_056_01():
    result = parse("Mafa-A1*056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_080_01():
    result = parse("Mafa-A1*080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_058_01():
    result = parse("Mafa-A1*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_059_01():
    result = parse("Mafa-A1*059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_01():
    result = parse("Mafa-A1*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_061_01():
    result = parse("Mafa-A1*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_062_01():
    result = parse("Mafa-A1*062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_063_01_01_01():
    result = parse("Mafa-A1*063:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*063:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*063:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_064_01():
    result = parse("Mafa-A1*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_064_02():
    result = parse("Mafa-A1*064:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*064:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*064:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_01_01():
    result = parse("Mafa-A1*065:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*065:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_01_02():
    result = parse("Mafa-A1*065:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*065:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_066_01():
    result = parse("Mafa-A1*066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_066_02():
    result = parse("Mafa-A1*066:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*066:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*066:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_067_01_01_01():
    result = parse("Mafa-A1*067:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*067:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*067:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_068_01():
    result = parse("Mafa-A1*068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_069_01():
    result = parse("Mafa-A1*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_071_01():
    result = parse("Mafa-A1*071:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*071:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*071:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_072_01():
    result = parse("Mafa-A1*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_01_01_01():
    result = parse("Mafa-A2*05:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_02():
    result = parse("Mafa-A2*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_03():
    result = parse("Mafa-A2*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_05():
    result = parse("Mafa-A2*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_06_01():
    result = parse("Mafa-A2*05:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_08():
    result = parse("Mafa-A2*05:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_09():
    result = parse("Mafa-A2*05:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_10():
    result = parse("Mafa-A2*05:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_11_01_01():
    result = parse("Mafa-A2*05:11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:11:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_12_01():
    result = parse("Mafa-A2*05:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_12_02_01():
    result = parse("Mafa-A2*05:12:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:12:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:12:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_13():
    result = parse("Mafa-A2*05:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_14():
    result = parse("Mafa-A2*05:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_15():
    result = parse("Mafa-A2*05:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_16():
    result = parse("Mafa-A2*05:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_17():
    result = parse("Mafa-A2*05:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_01():
    result = parse("Mafa-A3*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_01():
    result = parse("Mafa-A4*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_02():
    result = parse("Mafa-A4*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_02():
    result = parse("Mafa-A5*30:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A5*30:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_01():
    result = parse("Mafa-A6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_047_01():
    result = parse("Mafa-A1*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_001_02():
    result = parse("Mamu-A1*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_001_03():
    result = parse("Mamu-A1*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_14():
    result = parse("Mamu-A1*003:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_02():
    result = parse("Mamu-A1*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_03():
    result = parse("Mamu-A1*003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_04():
    result = parse("Mamu-A1*003:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_05():
    result = parse("Mamu-A1*003:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_01_02():
    result = parse("Mamu-A1*004:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*004:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_02_01():
    result = parse("Mamu-A1*004:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*004:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_010_02():
    result = parse("Mamu-A1*010:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*010:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*010:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_04():
    result = parse("Mamu-A1*007:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*007:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_01():
    result = parse("Mamu-A1*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_02():
    result = parse("Mamu-A1*018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_03():
    result = parse("Mamu-A1*018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_02():
    result = parse("Mamu-A1*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_03():
    result = parse("Mamu-A1*019:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_04_01():
    result = parse("Mamu-A1*019:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*019:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_025_02_01():
    result = parse("Mamu-A1*025:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*025:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*025:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_026_02_01():
    result = parse("Mamu-A1*026:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*026:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*026:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_02():
    result = parse("Mamu-A1*028:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*028:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_033_01():
    result = parse("Mamu-A1*033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_040_01():
    result = parse("Mamu-A1*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_041_01_01_01():
    result = parse("Mamu-A1*041:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*041:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*041:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_081_01():
    result = parse("Mamu-A1*081:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*081:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*081:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_048_01_01():
    result = parse("Mamu-A1*048:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*048:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*048:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_049_01():
    result = parse("Mamu-A1*049:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*049:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*049:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_050_01():
    result = parse("Mamu-A1*050:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*050:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*050:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_051_01():
    result = parse("Mamu-A1*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_053_01():
    result = parse("Mamu-A1*053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_054_01():
    result = parse("Mamu-A1*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_054_02():
    result = parse("Mamu-A1*054:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*054:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*054:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_056_01():
    result = parse("Mamu-A1*056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_056_02_01():
    result = parse("Mamu-A1*056:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*056:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*056:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_057_01():
    result = parse("Mamu-A1*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_061_01():
    result = parse("Mamu-A1*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_03_01_01():
    result = parse("Mamu-A2*05:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_55_01_01():
    result = parse("Mamu-A2*05:55:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:55:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:55:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_04_02():
    result = parse("Mamu-A2*05:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_13_01():
    result = parse("Mamu-A2*05:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_14():
    result = parse("Mamu-A2*05:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_15_01():
    result = parse("Mamu-A2*05:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_17():
    result = parse("Mamu-A2*05:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_18():
    result = parse("Mamu-A2*05:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_19():
    result = parse("Mamu-A2*05:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_20_01():
    result = parse("Mamu-A2*05:20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:20:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_21():
    result = parse("Mamu-A2*05:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_22():
    result = parse("Mamu-A2*05:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_23():
    result = parse("Mamu-A2*05:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_24():
    result = parse("Mamu-A2*05:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_25():
    result = parse("Mamu-A2*05:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_26():
    result = parse("Mamu-A2*05:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_24_02():
    result = parse("Mamu-A2*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_24_03():
    result = parse("Mamu-A2*24:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*24:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*24:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_07():
    result = parse("Mamu-A3*13:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_04_01():
    result = parse("Mamu-A4*14:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A4*14:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_01_01():
    result = parse("Mamu-A5*30:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A5*30:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_01_02():
    result = parse("Mamu-A5*30:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A5*30:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_02_01():
    result = parse("Mamu-A5*30:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A5*30:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_03():
    result = parse("Mamu-A5*30:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A5*30:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_01():
    result = parse("Mamu-A6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A7_01_01():
    result = parse("Mamu-A7*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A7*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A7_01_02():
    result = parse("Mamu-A7*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A7*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_001_01_01_01():
    result = parse("Mamu-A1*001:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*001:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*001:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_002_01_01_01():
    result = parse("Mamu-A1*002:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*002:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*002:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_05():
    result = parse("Mamu-A2*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_06():
    result = parse("Mamu-A2*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_07_01_01():
    result = parse("Mamu-A2*05:07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:07:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_09():
    result = parse("Mamu-A2*05:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_10():
    result = parse("Mamu-A2*05:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_11():
    result = parse("Mamu-A2*05:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_006_02():
    result = parse("Mamu-A1*006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_03():
    result = parse("Mamu-A1*007:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*007:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_02_01_01():
    result = parse("Mamu-A3*13:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A3*13:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_03_01_01():
    result = parse("Mamu-A3*13:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A3*13:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_05():
    result = parse("Mamu-A3*13:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_06():
    result = parse("Mamu-A3*13:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_016_01_01_01():
    result = parse("Mamu-A1*016:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*016:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*016:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_01_01_01():
    result = parse("Mamu-A1*019:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*019:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_06_01_01():
    result = parse("Mamu-A1*003:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*003:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_023_01_01_01():
    result = parse("Mamu-A1*023:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*023:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*023:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_025_01_01_01():
    result = parse("Mamu-A1*025:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*025:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*025:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_011_01_01_01():
    result = parse("Mamu-A1*011:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*011:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*011:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_01():
    result = parse("Mamu-A1*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_24_01_01_01():
    result = parse("Mamu-A2*24:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*24:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*24:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_04():
    result = parse("Mamu-A3*13:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_01_01():
    result = parse("Mamu-A1*003:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*003:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_01_01():
    result = parse("Mamu-A1*004:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*004:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_01():
    result = parse("Mamu-A2*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_02_01_01():
    result = parse("Mamu-A2*05:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_04_01_01():
    result = parse("Mamu-A2*05:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_006_01():
    result = parse("Mamu-A1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_01_01_01():
    result = parse("Mamu-A1*007:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*007:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_02():
    result = parse("Mamu-A1*007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_012_01_01_01():
    result = parse("Mamu-A1*012:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*012:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*012:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_01():
    result = parse("Mamu-A3*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_01_01():
    result = parse("Mamu-A2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_10():
    result = parse("Mamu-A3*13:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_02():
    result = parse("Mamu-A6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_03():
    result = parse("Mamu-A6*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A6*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_01_01_01():
    result = parse("Mamu-A1*008:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*008:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_11():
    result = parse("Mamu-A3*13:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_03_01_01():
    result = parse("Mamu-A1*028:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*028:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_073_01():
    result = parse("Mamu-A1*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_074_01():
    result = parse("Mamu-A1*074:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*074:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*074:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_02_03():
    result = parse("Mamu-A2*05:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_02_04():
    result = parse("Mamu-A2*05:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_03_03():
    result = parse("Mamu-A2*05:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_04_03():
    result = parse("Mamu-A2*05:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_04_04_01():
    result = parse("Mamu-A2*05:04:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:04:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:04:04:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_12():
    result = parse("Mamu-A2*05:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_15_02():
    result = parse("Mamu-A2*05:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_16_01():
    result = parse("Mamu-A2*05:16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:16:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_27():
    result = parse("Mamu-A2*05:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_28():
    result = parse("Mamu-A2*05:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_29():
    result = parse("Mamu-A2*05:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_30():
    result = parse("Mamu-A2*05:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_31():
    result = parse("Mamu-A2*05:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_32_01():
    result = parse("Mamu-A2*05:32:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:32:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:32:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_33():
    result = parse("Mamu-A2*05:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_34():
    result = parse("Mamu-A2*05:34")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:34") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:34") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_35():
    result = parse("Mamu-A2*05:35")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:35") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:35") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_36():
    result = parse("Mamu-A2*05:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_37_01_01():
    result = parse("Mamu-A2*05:37:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:37:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:37:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_08_01_01():
    result = parse("Mamu-A3*13:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A3*13:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_09():
    result = parse("Mamu-A3*13:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_01_01():
    result = parse("Mamu-A4*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_01_02_01():
    result = parse("Mamu-A4*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A4*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_02_01():
    result = parse("Mamu-A4*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_02_02_01_01():
    result = parse("Mamu-A4*02:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*02:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*02:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_02_03():
    result = parse("Mamu-A4*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_02_04_01_01():
    result = parse("Mamu-A4*02:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*02:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*02:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_05_01_01():
    result = parse("Mamu-A4*14:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_06():
    result = parse("Mamu-A4*14:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_01_01():
    result = parse("Mane-A2*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_01_02():
    result = parse("Mane-A2*05:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_01_03():
    result = parse("Mane-A2*05:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_01_04():
    result = parse("Mane-A2*05:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_02():
    result = parse("Mane-A2*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_03_01():
    result = parse("Mane-A2*05:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_04():
    result = parse("Mane-A2*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_05_01():
    result = parse("Mane-A2*05:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_05_02():
    result = parse("Mane-A2*05:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_06():
    result = parse("Mane-A2*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_07():
    result = parse("Mane-A2*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_08_01():
    result = parse("Mane-A2*05:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_09():
    result = parse("Mane-A2*05:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_10():
    result = parse("Mane-A2*05:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_11():
    result = parse("Mane-A2*05:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_12():
    result = parse("Mane-A2*05:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_13():
    result = parse("Mane-A2*05:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_14():
    result = parse("Mane-A2*05:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_15():
    result = parse("Mane-A2*05:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_084_01_01():
    result = parse("Mane-A1*084:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*084:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*084:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_082_02():
    result = parse("Mane-A1*082:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*082:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*082:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_082_03():
    result = parse("Mane-A1*082:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*082:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*082:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_17():
    result = parse("Mane-A2*05:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_19_01():
    result = parse("Mane-A2*05:19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:19:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_006_02():
    result = parse("Mane-A1*006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_052_01():
    result = parse("Mane-A1*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_010_01():
    result = parse("Mane-A1*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_01_01():
    result = parse("Mane-A3*13:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A3*13:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_01():
    result = parse("Mane-A1*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_02():
    result = parse("Mane-A1*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_072_01():
    result = parse("Mane-A1*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_031_01():
    result = parse("Mane-A1*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_085_01():
    result = parse("Mane-A1*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_084_02():
    result = parse("Mane-A1*084:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*084:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*084:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_01_01():
    result = parse("Mane-A4*14:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A4*14:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_02_01_01():
    result = parse("Mamu-B*082:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*082:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_01_02_01():
    result = parse("Mamu-B*079:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*079:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_09_01_01():
    result = parse("Mamu-B*046:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_01_01():
    result = parse("Mamu-B*046:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*046:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_029_01_01_01():
    result = parse("Mamu-B*029:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*029:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*029:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_01_02():
    result = parse("Mamu-B*044:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*044:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_027_03_01_01():
    result = parse("Mamu-B*027:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*027:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*027:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_05():
    result = parse("Mamu-A1*018:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_19():
    result = parse("Mamu-I*01:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_20_01_01():
    result = parse("Mamu-I*01:20:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:20:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:20:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_03():
    result = parse("Mamu-I*01:06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_F_02_03_01():
    result = parse("Mamu-F*02:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-F*02:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-F*02:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_12_01():
    result = parse("Mamu-E*02:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-E*02:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_13():
    result = parse("Mamu-E*02:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_14():
    result = parse("Mamu-E*02:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_15():
    result = parse("Mamu-E*02:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_08_01():
    result = parse("Mamu-DPA1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_06_02():
    result = parse("Mamu-DPA1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_24_01():
    result = parse("Patr-A*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_02_06():
    result = parse("Patr-C*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_15_01():
    result = parse("Patr-C*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_16_01():
    result = parse("Patr-C*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_16():
    result = parse("Patr-DRB1*03:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_094_01():
    result = parse("Mamu-B*094:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*094:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*094:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_03_01_01():
    result = parse("Mamu-B*063:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*063:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_095_01_01_01():
    result = parse("Mamu-B*095:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*095:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*095:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_03_02():
    result = parse("Mamu-DPB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W036_02_02():
    result = parse("Mafa-DRB*W036:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W036:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W036:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W066_01():
    result = parse("Mafa-DRB*W066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W067_01():
    result = parse("Mafa-DRB*W067:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W067:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W067:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_05():
    result = parse("Mafa-DRB*W006:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W068_01():
    result = parse("Mafa-DRB*W068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_05():
    result = parse("Mafa-DRB*W004:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_07_04():
    result = parse("Mafa-DRB1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_06():
    result = parse("Mafa-DRB6*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_07():
    result = parse("Mafa-DRB6*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_08():
    result = parse("Mafa-DRB6*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_09():
    result = parse("Mafa-DRB6*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_10():
    result = parse("Mafa-DRB6*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_11_01():
    result = parse("Mafa-DRB6*01:11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB6*01:11:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_12():
    result = parse("Mafa-DRB6*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_13_01():
    result = parse("Mafa-DRB6*01:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB6*01:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_14():
    result = parse("Mafa-DRB6*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_13_02():
    result = parse("Mafa-DRB6*01:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB6*01:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_15():
    result = parse("Mafa-DRB6*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_093_01():
    result = parse("Mafa-A1*093:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*093:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*093:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_02():
    result = parse("Mafa-A1*089:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_052_02():
    result = parse("Mafa-A1*052:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*052:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*052:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_04():
    result = parse("Mafa-A4*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_004_01():
    result = parse("Mafa-A1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_008_02():
    result = parse("Mafa-A1*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_008_03():
    result = parse("Mafa-A1*008:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*008:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*008:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_03():
    result = parse("Mafa-A1*089:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_094_01():
    result = parse("Mafa-A1*094:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*094:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*094:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_03_01():
    result = parse("Mafa-A3*13:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A3*13:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_095_01_01_01():
    result = parse("Mafa-B*095:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*095:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*095:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_08():
    result = parse("Mafa-B*013:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_01_02():
    result = parse("Mafa-B*101:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*101:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_033_02():
    result = parse("Mafa-B*033:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*033:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*033:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_017_01():
    result = parse("Mafa-B*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_02():
    result = parse("Mafa-B*007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_157_01():
    result = parse("Mafa-B*157:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*157:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*157:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_158_01():
    result = parse("Mafa-B*158:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*158:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*158:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_116_01():
    result = parse("Mafa-B*116:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*116:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*116:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_01():
    result = parse("Mafa-B*117:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*117:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_089_01_02_01():
    result = parse("Mafa-B*089:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*089:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*089:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_144_03N():
    result = parse("Mafa-B*144:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*144:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*144:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_056_02_01():
    result = parse("Mafa-B*056:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*056:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*056:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_02():
    result = parse("Mafa-B*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_151_02_01():
    result = parse("Mafa-B*151:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*151:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*151:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_089_01_01():
    result = parse("Mafa-B*089:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*089:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*089:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_03():
    result = parse("Mafa-B*048:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*048:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_099_01():
    result = parse("Mafa-B*099:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*099:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*099:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_108_01():
    result = parse("Mafa-B*108:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*108:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*108:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_01_02():
    result = parse("Mafa-B*007:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*007:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_159_01():
    result = parse("Mafa-B*159:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*159:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*159:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_01():
    result = parse("Mafa-B*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_12_01():
    result = parse("Mafa-I*01:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_13_01():
    result = parse("Mafa-I*01:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_11_01_01():
    result = parse("Mafa-I*01:11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-I*01:11:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_36_01():
    result = parse("Patr-B*36:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*36:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*36:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_04_02():
    result = parse("Patr-B*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_37_01():
    result = parse("Patr-B*37:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*37:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*37:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_04():
    result = parse("Mafa-DPA1*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_05():
    result = parse("Mafa-DPA1*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_25_03():
    result = parse("Mafa-DPB1*25:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*25:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*25:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W069_01():
    result = parse("Mafa-DRB*W069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_03():
    result = parse("Mafa-DQA1*26:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_07():
    result = parse("Mafa-DQB1*18:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_147_03():
    result = parse("Mafa-B*147:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*147:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*147:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_01_02():
    result = parse("Mafa-B*046:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*046:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Seen_DPB1_01():
    result = parse("Seen-DPB1*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Seen-DPB1*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Seen-DPB1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Thge_DQA1_01_01():
    result = parse("Thge-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Thge-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Thge-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Thge_DQA1_25_01():
    result = parse("Thge-DQA1*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Thge-DQA1*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Thge-DQA1*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_N1_01():
    result = parse("Saoe-N1*01")
    assert result.__class__ is Allele, f'Expected parse("Saoe-N1*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Saoe-N1*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_N3_01():
    result = parse("Saoe-N3*01")
    assert result.__class__ is Allele, f'Expected parse("Saoe-N3*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Saoe-N3*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Saoe_N4_01():
    result = parse("Saoe-N4*01")
    assert result.__class__ is Allele, f'Expected parse("Saoe-N4*01") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Saoe-N4*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_07():
    result = parse("Gogo-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB5_05_01():
    result = parse("Gogo-DRB5*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB5*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB5*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_10():
    result = parse("Mamu-DRB1*10:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_12():
    result = parse("Mamu-DRB3*04:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_15_01():
    result = parse("Mamu-DRB*W003:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W003:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_05():
    result = parse("Mamu-DRB*W004:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W004:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_06():
    result = parse("Mamu-A1*018:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B17_01_01():
    result = parse("Mafa-B17*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B17*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B17*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_38():
    result = parse("Mamu-A2*05:38")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:38") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:38") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_39():
    result = parse("Mamu-A2*05:39")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:39") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:39") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_03_01():
    result = parse("Mamu-A4*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_041_02_01():
    result = parse("Mamu-A1*041:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*041:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*041:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_001_05():
    result = parse("Mamu-A1*001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_001_04():
    result = parse("Mamu-A1*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_03_02():
    result = parse("Mamu-A4*14:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A4*14:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_07():
    result = parse("Mamu-A4*14:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_08():
    result = parse("Mamu-A4*14:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_03_02():
    result = parse("Mamu-A4*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_02_02():
    result = parse("Mamu-B*046:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*046:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_10():
    result = parse("Mamu-B*046:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_11():
    result = parse("Mamu-B*046:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_12():
    result = parse("Mamu-B*046:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_02_02():
    result = parse("Mamu-B17*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_04():
    result = parse("Mamu-B17*01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_06_01():
    result = parse("Mamu-B17*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_07_01():
    result = parse("Mamu-B17*01:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_08():
    result = parse("Mamu-B17*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B17*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_05():
    result = parse("Mamu-B17*01:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_06():
    result = parse("Mamu-B17*01:01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_09():
    result = parse("Mamu-B17*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B17*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_07_02():
    result = parse("Mamu-B17*01:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_10_01_01():
    result = parse("Mamu-B17*01:10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B17*01:10:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_11():
    result = parse("Mamu-B17*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B17*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_04_02():
    result = parse("Mamu-B17*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_07():
    result = parse("Mamu-B17*01:01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_12_01_01():
    result = parse("Mamu-B17*01:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B17*01:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_06_02():
    result = parse("Mamu-B17*01:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_08():
    result = parse("Mamu-B17*01:01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:08") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_03_02():
    result = parse("Mamu-B17*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_02():
    result = parse("Mamu-B11L*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B11L*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_01_01():
    result = parse("Mamu-B11L*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B11L*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_03():
    result = parse("Mamu-B11L*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B11L*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_01_02():
    result = parse("Mamu-B11L*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B11L*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_04_01_01():
    result = parse("Mamu-B11L*01:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B11L*01:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_05_01_01():
    result = parse("Mamu-B11L*01:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B11L*01:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_06_01_01():
    result = parse("Mamu-B11L*01:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B11L*01:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_07():
    result = parse("Mamu-B11L*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B11L*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_08():
    result = parse("Mamu-B11L*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B11L*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_02_01():
    result = parse("Mane-A3*13:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A3*13:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_04():
    result = parse("Mane-B17*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B17*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_02_01():
    result = parse("Mane-B17*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_01_01():
    result = parse("Mane-B17*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_05():
    result = parse("Mane-B17*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B17*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_03_01():
    result = parse("Mane-B17*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_01_02():
    result = parse("Mane-B17*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_06():
    result = parse("Mane-B17*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B17*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_07():
    result = parse("Mane-B17*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B17*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_01_04():
    result = parse("Mane-B17*01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_02_02():
    result = parse("Mane-B17*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_25():
    result = parse("Mane-I*01:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_02():
    result = parse("Mane-I*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_03():
    result = parse("Mane-I*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_04():
    result = parse("Mane-I*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_05():
    result = parse("Mane-I*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_06_01():
    result = parse("Mane-I*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_07():
    result = parse("Mane-I*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_24():
    result = parse("Mane-I*01:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_074_02():
    result = parse("Mamu-A1*074:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*074:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*074:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_04():
    result = parse("Mafa-A1*089:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_005_02_01_01():
    result = parse("Mamu-B*005:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*005:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*005:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_02_01_01():
    result = parse("Mamu-B*030:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*030:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_031_02_01_01():
    result = parse("Mamu-B*031:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*031:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*031:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_05_01_01():
    result = parse("Mamu-B*044:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*044:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_04():
    result = parse("Mamu-I*01:06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_05():
    result = parse("Mafa-A1*089:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_160_01():
    result = parse("Mafa-B*160:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*160:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*160:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_02_01():
    result = parse("Mafa-B*117:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*117:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_12_02():
    result = parse("Mafa-I*01:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_01():
    result = parse("Mafa-I*01:14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_053_02():
    result = parse("Mamu-A1*053:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*053:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*053:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_003_02():
    result = parse("Mamu-B*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_091_02():
    result = parse("Mamu-A1*091:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*091:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*091:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_049_02_01():
    result = parse("Mamu-A1*049:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*049:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*049:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_048_01_02():
    result = parse("Mamu-A1*048:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*048:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*048:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_05():
    result = parse("Mamu-A1*019:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_09():
    result = parse("Mamu-A1*003:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_075_02():
    result = parse("Mamu-B*075:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*075:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*075:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_04_01_01():
    result = parse("Mamu-B*045:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*045:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_036_01_02_01():
    result = parse("Mamu-B*036:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*036:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*036:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_04_01_01():
    result = parse("Mamu-B*069:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*069:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_03():
    result = parse("Mamu-B*017:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*017:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_003_01_01():
    result = parse("Mamu-B*003:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*003:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*003:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_002_02():
    result = parse("Mamu-B*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_02_01_01():
    result = parse("Mamu-B*007:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*007:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_01_02_01():
    result = parse("Mamu-B*001:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*001:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_010_01():
    result = parse("Mamu-B*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_066_01_01_01():
    result = parse("Mamu-B*066:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*066:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*066:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_036_02_01_01():
    result = parse("Mamu-B*036:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*036:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*036:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB5_03_05():
    result = parse("Masp-DRB5*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB5*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB5*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_06():
    result = parse("Masp-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W070_01():
    result = parse("Masp-DRB*W070:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W070:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W070:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W071_01():
    result = parse("Masp-DRB*W071:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W071:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W071:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_08():
    result = parse("Masp-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W025_05():
    result = parse("Masp-DRB*W025:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W025:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W025:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB5_03_06():
    result = parse("Masp-DRB5*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB5*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB5*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB1_03_07():
    result = parse("Masp-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Masp_DRB_W054_01():
    result = parse("Masp-DRB*W054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Masp-DRB*W054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Masp-DRB*W054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_13_01_01():
    result = parse("Mamu-B*046:13:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:13:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:13:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_09():
    result = parse("Mamu-A4*14:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_04_01_01():
    result = parse("Mamu-B*051:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_01_01_01():
    result = parse("Mamu-B*060:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_05_01_01():
    result = parse("Mamu-B*051:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_19_01_01():
    result = parse("Mamu-DPB1*19:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*19:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPB1*19:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_17():
    result = parse("Mamu-DQB1*18:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_15():
    result = parse("Mamu-DQB1*06:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_15_04():
    result = parse("Mamu-DQB1*15:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*15:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*15:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_11():
    result = parse("Mamu-DQB1*17:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_18():
    result = parse("Mamu-DQB1*18:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_13_02():
    result = parse("Mamu-DQB1*06:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_19_01():
    result = parse("Mamu-DQB1*18:19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*18:19:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_20():
    result = parse("Mamu-DQB1*18:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_21():
    result = parse("Mamu-DQB1*18:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_060_01():
    result = parse("Mamu-A1*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_04_01():
    result = parse("Mamu-A5*30:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A5*30:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_065_01():
    result = parse("Mamu-A1*065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_049_03():
    result = parse("Mamu-A1*049:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*049:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*049:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_040_02():
    result = parse("Mamu-A1*040:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*040:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*040:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_040_03():
    result = parse("Mamu-A1*040:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*040:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*040:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_01_01():
    result = parse("Paan-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_01_02():
    result = parse("Paan-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_25_01():
    result = parse("Paan-DQA1*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRA_01_01_01():
    result = parse("Patr-DRA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DRA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRA_01_01_02():
    result = parse("Patr-DRA*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRA*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DRA*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_01():
    result = parse("Mane-E*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_02():
    result = parse("Mane-E*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_03():
    result = parse("Mane-E*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_04():
    result = parse("Mane-E*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_05():
    result = parse("Mane-E*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_06():
    result = parse("Mane-E*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_07():
    result = parse("Mane-E*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_E_02_08():
    result = parse("Mane-E*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-E*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-E*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_01_01_01():
    result = parse("Mamu-B*001:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*001:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_002_01_01_01():
    result = parse("Mamu-B*002:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*002:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*002:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_004_01_01_01():
    result = parse("Mamu-B*004:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*004:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*004:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_005_01():
    result = parse("Mamu-B*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_006_01_01_01():
    result = parse("Mamu-B*006:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*006:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*006:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_01_01_01():
    result = parse("Mamu-B*007:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*007:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_008_01_01_01():
    result = parse("Mamu-B*008:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*008:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*008:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_011_01():
    result = parse("Mamu-B*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_01_01_01():
    result = parse("Mamu-B*017:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*017:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_01_01_01():
    result = parse("Mamu-B*030:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*030:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_03_01_01():
    result = parse("Mamu-B*030:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*030:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_031_01_01_01():
    result = parse("Mamu-B*031:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*031:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*031:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_01():
    result = parse("Mamu-B*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_02_01_01():
    result = parse("Mamu-B*051:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_03_01_01():
    result = parse("Mamu-B*051:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_052_01_01_01():
    result = parse("Mamu-B*052:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*052:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*052:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_057_02():
    result = parse("Mamu-B*057:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*057:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*057:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_02_01_01():
    result = parse("Mamu-B*068:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*068:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_081_01_01_01():
    result = parse("Mamu-B*081:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*081:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*081:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_077_01_01():
    result = parse("Mamu-B*077:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*077:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*077:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_078_01():
    result = parse("Mamu-B*078:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*078:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*078:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_01_01():
    result = parse("Mamu-B*079:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*079:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_080_01_01_01():
    result = parse("Mamu-B*080:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*080:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*080:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_006_02_01_01():
    result = parse("Mamu-B*006:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*006:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*006:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_04_01_01():
    result = parse("Mamu-B*007:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*007:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_013_01_01_01():
    result = parse("Mamu-B*013:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*013:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*013:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_014_01_01_01():
    result = parse("Mamu-B*014:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*014:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*014:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_015_01():
    result = parse("Mamu-B*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_015_02_01_01():
    result = parse("Mamu-B*015:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*015:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*015:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_016_01_01_01():
    result = parse("Mamu-B*016:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*016:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*016:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_018_01():
    result = parse("Mamu-B*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_01_01_01():
    result = parse("Mamu-B*019:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*019:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_03():
    result = parse("Mamu-B*019:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*019:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_06():
    result = parse("Mamu-B*068:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*068:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_021_01():
    result = parse("Mamu-B*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_021_02():
    result = parse("Mamu-B*021:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*021:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*021:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_021_03():
    result = parse("Mamu-B*021:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*021:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*021:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_022_01_01_01():
    result = parse("Mamu-B*022:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*022:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*022:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_023_01():
    result = parse("Mamu-B*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_025_01_01_01():
    result = parse("Mamu-B*025:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*025:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*025:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_057_05_01_01():
    result = parse("Mamu-B*057:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*057:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*057:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_027_01_01_01():
    result = parse("Mamu-B*027:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*027:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*027:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_027_02():
    result = parse("Mamu-B*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_028_01_01_01():
    result = parse("Mamu-B*028:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*028:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*028:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_028_02_01():
    result = parse("Mamu-B*028:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*028:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*028:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_028_02_02():
    result = parse("Mamu-B*028:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*028:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*028:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_028_03():
    result = parse("Mamu-B*028:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*028:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*028:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_029_01_02_01():
    result = parse("Mamu-B*029:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*029:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*029:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_03_02():
    result = parse("Mamu-B*030:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*030:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_03_03_01():
    result = parse("Mamu-B*030:03:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:03:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*030:03:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_04_01_01():
    result = parse("Mamu-B*030:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*030:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_032_01():
    result = parse("Mamu-B*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_033_01():
    result = parse("Mamu-B*033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_034_01_01_01():
    result = parse("Mamu-B*034:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*034:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*034:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_035_01_01_01():
    result = parse("Mamu-B*035:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*035:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*035:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_036_01_01_01():
    result = parse("Mamu-B*036:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*036:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*036:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_037_01_01_01():
    result = parse("Mamu-B*037:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*037:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*037:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_038_01_01_01():
    result = parse("Mamu-B*038:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*038:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*038:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_015_04_01_01():
    result = parse("Mamu-B*015:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*015:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*015:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_041_01_01_01():
    result = parse("Mamu-B*041:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*041:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*041:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_042_01_01_01():
    result = parse("Mamu-B*042:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*042:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*042:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_01_01_01():
    result = parse("Mamu-B*044:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*044:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_02():
    result = parse("Mamu-B*044:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*044:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_03_01_01():
    result = parse("Mamu-B*044:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*044:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_04_01_01():
    result = parse("Mamu-B*044:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*044:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_01():
    result = parse("Mamu-B*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_02_01_01():
    result = parse("Mamu-B*045:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*045:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_047_01_01_01():
    result = parse("Mamu-B*047:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*047:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*047:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_047_02():
    result = parse("Mamu-B*047:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*047:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*047:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_047_03():
    result = parse("Mamu-B*047:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*047:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*047:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_047_04():
    result = parse("Mamu-B*047:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*047:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*047:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_048_01_01_01():
    result = parse("Mamu-B*048:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*048:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*048:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_049_01_01_01():
    result = parse("Mamu-B*049:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*049:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*049:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_050_02_01_01():
    result = parse("Mamu-B*050:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*050:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*050:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_053_01():
    result = parse("Mamu-B*053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_053_02_01_01():
    result = parse("Mamu-B*053:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*053:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*053:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_054_01_01_01():
    result = parse("Mamu-B*054:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*054:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*054:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_055_01_01_01():
    result = parse("Mamu-B*055:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*055:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*055:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_01_01_01():
    result = parse("Mamu-B*056:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*056:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_057_01_01_01():
    result = parse("Mamu-B*057:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*057:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*057:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_058_02_01_01():
    result = parse("Mamu-B*058:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*058:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*058:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_013_03_01_01():
    result = parse("Mamu-B*013:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*013:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*013:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_02_01_01():
    result = parse("Mamu-B*060:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_061_01_01_01():
    result = parse("Mamu-B*061:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*061:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*061:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_061_02():
    result = parse("Mamu-B*061:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*061:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*061:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_062_01():
    result = parse("Mamu-B*062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_01_01():
    result = parse("Mamu-B*063:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*063:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_064_01_01_01():
    result = parse("Mamu-B*064:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*064:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*064:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_01_01_01():
    result = parse("Mamu-B*065:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*065:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_03_01_01():
    result = parse("Mamu-B*065:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*065:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_067_01_01_01():
    result = parse("Mamu-B*067:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*067:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*067:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_067_02():
    result = parse("Mamu-B*067:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*067:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*067:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_01_01_01():
    result = parse("Mamu-B*068:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*068:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_01_02_01():
    result = parse("Mamu-B*068:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*068:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_03_01_01():
    result = parse("Mamu-B*068:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*068:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_04():
    result = parse("Mamu-B*068:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*068:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_05():
    result = parse("Mamu-B*068:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*068:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_03_01_01():
    result = parse("Mamu-B*069:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*069:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_070_01_01_01():
    result = parse("Mamu-B*070:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*070:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*070:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_070_02_01_01():
    result = parse("Mamu-B*070:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*070:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*070:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_071_01_01_01():
    result = parse("Mamu-B*071:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*071:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*071:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_01_01():
    result = parse("Mamu-B*072:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_02_01_01():
    result = parse("Mamu-B*072:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_073_01_01_01():
    result = parse("Mamu-B*073:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*073:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*073:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_075_01_01_01():
    result = parse("Mamu-B*075:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*075:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*075:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_077_02():
    result = parse("Mamu-B*077:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*077:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*077:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_085_02():
    result = parse("Mamu-B*085:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*085:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*085:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_086_02_01_01():
    result = parse("Mamu-B*086:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*086:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*086:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_086_03():
    result = parse("Mamu-B*086:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*086:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*086:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_091_02():
    result = parse("Mamu-B*091:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*091:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*091:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_093_01_01_01():
    result = parse("Mamu-B*093:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*093:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*093:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_029_02():
    result = parse("Mamu-B*029:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*029:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*029:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_074_01_01_01():
    result = parse("Mamu-B*074:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*074:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*074:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_074_02_01_01():
    result = parse("Mamu-B*074:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*074:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*074:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_012_01_01_01():
    result = parse("Mamu-B*012:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*012:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*012:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_01_02_01():
    result = parse("Mamu-B*046:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_01_03():
    result = parse("Mamu-B*046:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*046:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_02_01():
    result = parse("Mamu-B*046:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*046:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_03_01_01():
    result = parse("Mamu-B*046:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_03_02():
    result = parse("Mamu-B*046:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*046:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_04_01_01():
    result = parse("Mamu-B*046:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_05_01_01():
    result = parse("Mamu-B*046:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_06_01_01():
    result = parse("Mamu-B*046:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_07():
    result = parse("Mamu-B*046:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_08():
    result = parse("Mamu-B*046:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_14():
    result = parse("Mamu-B*046:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_076_01_01():
    result = parse("Mamu-B*076:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*076:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*076:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_076_02():
    result = parse("Mamu-B*076:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*076:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*076:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_F_02_02():
    result = parse("Mamu-F*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-F*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-F*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_F_02_05():
    result = parse("Mamu-F*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-F*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-F*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_01_01_01():
    result = parse("Mafa-B*079:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*079:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_16():
    result = parse("Mamu-DQB1*06:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_03():
    result = parse("Mafa-A1*007:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_026_03():
    result = parse("Mamu-A1*026:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*026:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*026:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_06():
    result = parse("Mafa-A3*13:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_066_04():
    result = parse("Mafa-A1*066:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*066:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*066:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_095_01():
    result = parse("Mafa-A1*095:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*095:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*095:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_032_03():
    result = parse("Mafa-A1*032:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*032:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*032:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_019_02():
    result = parse("Mafa-A1*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_079_02():
    result = parse("Mafa-A1*079:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*079:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*079:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_096_01_01_01():
    result = parse("Mafa-A1*096:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*096:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*096:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_094_02():
    result = parse("Mafa-A1*094:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*094:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*094:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_076_02():
    result = parse("Mafa-A1*076:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*076:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*076:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_088_02():
    result = parse("Mafa-A1*088:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*088:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*088:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_05():
    result = parse("Mafa-A3*13:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_04():
    result = parse("Mafa-A1*060:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*060:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_004_04():
    result = parse("Mafa-A1*004:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*004:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*004:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_097_01():
    result = parse("Mafa-A1*097:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*097:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*097:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_085_01():
    result = parse("Mafa-A1*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_04():
    result = parse("Mafa-A1*007:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_03():
    result = parse("Mafa-A1*043:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_098_01():
    result = parse("Mafa-A1*098:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*098:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*098:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_019_04():
    result = parse("Mafa-A1*019:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*019:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*019:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_05():
    result = parse("Mafa-A4*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_02():
    result = parse("Mafa-A2*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_099_01():
    result = parse("Mafa-A1*099:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*099:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*099:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_062_03():
    result = parse("Mafa-A1*062:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*062:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*062:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_066_05():
    result = parse("Mafa-A1*066:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*066:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*066:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_03_01():
    result = parse("Mafa-A5*30:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A5*30:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_063_03_02():
    result = parse("Mafa-A1*063:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*063:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*063:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_087_02_01_01():
    result = parse("Mafa-A1*087:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*087:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*087:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_092_01_01():
    result = parse("Mafa-A1*092:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*092:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*092:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_011_03():
    result = parse("Mafa-A1*011:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*011:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*011:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_074_01_01():
    result = parse("Mafa-A1*074:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*074:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*074:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_04():
    result = parse("Mafa-A1*018:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*018:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_032_04_01_01():
    result = parse("Mafa-A1*032:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*032:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*032:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_02_02():
    result = parse("Mafa-A1*010:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*010:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_03():
    result = parse("Mafa-A1*022:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_04():
    result = parse("Mafa-A3*13:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_093_02():
    result = parse("Mafa-A1*093:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*093:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*093:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_074_02():
    result = parse("Mafa-A1*074:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*074:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*074:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_064_03():
    result = parse("Mafa-A1*064:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*064:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*064:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_002_02():
    result = parse("Mafa-A1*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_06_01_01():
    result = parse("Mafa-A1*089:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*089:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_02():
    result = parse("Mafa-A1*065:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*065:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_077_01_02():
    result = parse("Mafa-A1*077:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*077:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*077:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_03_01_01():
    result = parse("Mafa-A1*065:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*065:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_027_03():
    result = parse("Mafa-A1*027:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*027:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*027:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_019_05():
    result = parse("Mafa-A1*019:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*019:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*019:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_061_02():
    result = parse("Mafa-A1*061:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*061:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*061:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_04():
    result = parse("Mafa-A1*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_038_02():
    result = parse("Mafa-A1*038:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*038:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*038:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_059_02():
    result = parse("Mafa-A1*059:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*059:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*059:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_019_03():
    result = parse("Mafa-A1*019:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*019:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*019:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_04_01_01():
    result = parse("Mafa-A1*065:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*065:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_004_03():
    result = parse("Mafa-A1*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_004_02():
    result = parse("Mafa-A1*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_01():
    result = parse("Mafa-A2*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_055_01_01_01():
    result = parse("Mamu-A1*055:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*055:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*055:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_06():
    result = parse("Mamu-A1*019:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_07():
    result = parse("Mamu-A1*003:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_15_01_01():
    result = parse("Mamu-B*046:15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:15:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_04_01_01():
    result = parse("Mamu-A6*01:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A6*01:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_097_01_01_01():
    result = parse("Mamu-B*097:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*097:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*097:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_05():
    result = parse("Mamu-B*030:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*030:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_057_06_01_01():
    result = parse("Mamu-B*057:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*057:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*057:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_031_03_01_01():
    result = parse("Mamu-B*031:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*031:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*031:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_03_01_01():
    result = parse("Mamu-B*060:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_01_02():
    result = parse("Mamu-E*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-E*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_16():
    result = parse("Mamu-E*02:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_03_02():
    result = parse("Mamu-DRA*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_09():
    result = parse("Mamu-DRB5*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_23_03():
    result = parse("Mamu-DQA1*23:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*23:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*23:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_07_01():
    result = parse("Mamu-DPA1*02:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*02:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_02():
    result = parse("Mamu-B*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_02():
    result = parse("Ovar-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_04_01():
    result = parse("Ovar-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_01_01():
    result = parse("Ovar-DRB1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_05_01():
    result = parse("Ovar-DRB1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_09_01():
    result = parse("Ovar-DRB1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_01():
    result = parse("Ovar-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_03():
    result = parse("Ovar-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_07_01():
    result = parse("Ovar-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_01_01():
    result = parse("Ovar-N*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_02_01():
    result = parse("Ovar-N*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_50_02():
    result = parse("Ovar-N*50:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*50:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*50:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_50_01():
    result = parse("Ovar-N*50:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*50:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*50:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_07_01():
    result = parse("Ovar-N*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_03_01():
    result = parse("Ovar-N*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_08_01():
    result = parse("Ovar-N*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_05_01():
    result = parse("Ovar-N*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_04_01():
    result = parse("Ovar-N*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_50_00():
    result = parse("Ovar-N*50:00")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*50:00") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*50:00") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_50_03():
    result = parse("Ovar-N*50:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*50:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*50:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_06_01():
    result = parse("Ovar-N*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_08_01():
    result = parse("Ovar-DRB1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_02_01():
    result = parse("Ovar-DRB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_06_01():
    result = parse("Ovar-DRB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_01():
    result = parse("Ovar-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_01_02():
    result = parse("Ovar-DRB1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_11_01():
    result = parse("Ovar-DRB1*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_12_01():
    result = parse("Ovar-DRB1*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_08_02():
    result = parse("Ovar-DRB1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_13_01():
    result = parse("Ovar-DRB1*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_14_01():
    result = parse("Ovar-DRB1*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_04():
    result = parse("Ovar-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_15_01():
    result = parse("Ovar-DRB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_01():
    result = parse("Ovar-DRB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_07_02():
    result = parse("Ovar-DRB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_17_01():
    result = parse("Ovar-DRB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_04_02():
    result = parse("Ovar-DRB1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_15_02():
    result = parse("Ovar-DRB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_18_01():
    result = parse("Ovar-DRB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_05():
    result = parse("Ovar-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_19_01():
    result = parse("Ovar-DRB1*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_01_03():
    result = parse("Ovar-DRB1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_08_03():
    result = parse("Ovar-DRB1*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_17_02():
    result = parse("Ovar-DRB1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_20_01():
    result = parse("Ovar-DRB1*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_20_02():
    result = parse("Ovar-DRB1*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_13_02():
    result = parse("Ovar-DRB1*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_19_02():
    result = parse("Ovar-DRB1*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_02_01():
    result = parse("Ovar-DRB1*16:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DRB1*16:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_07():
    result = parse("Ovar-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_08_04():
    result = parse("Ovar-DRB1*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_21_01():
    result = parse("Ovar-DRB1*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_19_03():
    result = parse("Ovar-DRB1*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_06():
    result = parse("Ovar-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_09_02():
    result = parse("Ovar-DRB1*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_11_02():
    result = parse("Ovar-DRB1*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRA_01_01():
    result = parse("Ovar-DRA*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRA*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRA*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRA_01_02():
    result = parse("Ovar-DRA*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRA*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRA*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRA_02_01():
    result = parse("Ovar-DRA*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRA*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRA*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_22_01():
    result = parse("Ovar-DRB1*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_08():
    result = parse("Ovar-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_05_01_01():
    result = parse("Mamu-I*01:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_16():
    result = parse("Mamu-I*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_02():
    result = parse("Mafa-I*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_02_01():
    result = parse("Mafa-I*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_03():
    result = parse("Mafa-I*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_04():
    result = parse("Mafa-I*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_05():
    result = parse("Mafa-I*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_06_01():
    result = parse("Mafa-I*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_07():
    result = parse("Mafa-I*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_08():
    result = parse("Mafa-I*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_09():
    result = parse("Mafa-I*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_02_01_01():
    result = parse("Mamu-I*01:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_10_01_01():
    result = parse("Mafa-I*01:10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-I*01:10:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_03_01_01_01N():
    result = parse("Mafa-I*03:01:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*03:01:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-I*03:01:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_100_01():
    result = parse("Mafa-A1*100:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*100:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*100:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_101_01():
    result = parse("Mafa-A1*101:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*101:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*101:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_102_01():
    result = parse("Mafa-A1*102:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*102:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*102:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_101_02_01_01():
    result = parse("Mafa-A1*101:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*101:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*101:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_103_01():
    result = parse("Mafa-A1*103:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*103:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*103:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_05_01():
    result = parse("Mamu-I*01:06:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:06:05:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_06():
    result = parse("Mamu-I*01:06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_02_03():
    result = parse("Mamu-I*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_19_02():
    result = parse("Mamu-DPB1*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_19_01_02():
    result = parse("Mamu-DPB1*19:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*19:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPB1*19:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_08_01_01():
    result = parse("Mamu-I*01:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_01_02_01():
    result = parse("Mamu-I*01:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_01_03():
    result = parse("Mamu-A4*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_13():
    result = parse("Mamu-B17*01:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B17*01:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_09():
    result = parse("Mamu-B11L*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B11L*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_01():
    result = parse("Mamu-B16*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_02():
    result = parse("Mamu-B16*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_03():
    result = parse("Mamu-B16*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_04():
    result = parse("Mamu-B16*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_05():
    result = parse("Mamu-B16*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_06():
    result = parse("Mamu-B16*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_07():
    result = parse("Mamu-B16*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_08():
    result = parse("Mamu-B16*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_09():
    result = parse("Mamu-B16*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B16_01_10():
    result = parse("Mamu-B16*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B16*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B16*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_10_01_01():
    result = parse("Mamu-B11L*01:10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B11L*01:10:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_14():
    result = parse("Mamu-B17*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B17*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_15():
    result = parse("Mamu-B17*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B17*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_12():
    result = parse("Mamu-A3*13:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_02():
    result = parse("Mamu-A4*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_04_01():
    result = parse("Mamu-A1*028:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*028:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_02_05():
    result = parse("Mamu-A4*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_40():
    result = parse("Mamu-A2*05:40")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:40") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:40") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_15_03():
    result = parse("Mamu-A2*05:15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:15:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_41():
    result = parse("Mamu-A2*05:41")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:41") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:41") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_42():
    result = parse("Mamu-A2*05:42")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:42") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:42") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_43():
    result = parse("Mamu-A2*05:43")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:43") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:43") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_16():
    result = parse("Mamu-B*046:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_17():
    result = parse("Mamu-B*046:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_03_03():
    result = parse("Mamu-B*046:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*046:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_03():
    result = parse("Mane-A3*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_02_07():
    result = parse("Mane-B17*01:02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:02:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_08_01():
    result = parse("Mane-I*01:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_09():
    result = parse("Mane-I*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_03_02():
    result = parse("Mane-A2*05:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_05_03():
    result = parse("Mane-A2*05:05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:05:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_08_02():
    result = parse("Mane-A2*05:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_20():
    result = parse("Mane-A2*05:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_21():
    result = parse("Mane-A2*05:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_22():
    result = parse("Mane-A2*05:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_17_02():
    result = parse("Mane-I*01:17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:17:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_10():
    result = parse("Mane-I*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_03_01():
    result = parse("Mafa-A4*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_092_02():
    result = parse("Mafa-A1*092:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*092:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*092:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_03():
    result = parse("Mafa-A6*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B17_01_02():
    result = parse("Mafa-B17*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B17*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B17*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B17_01_03():
    result = parse("Mafa-B17*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B17*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B17*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B17_01_04():
    result = parse("Mafa-B17*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B17*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B17*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_02():
    result = parse("Mafa-B*046:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_03():
    result = parse("Mafa-B*046:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_04_01_01():
    result = parse("Mafa-B*046:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*046:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B11L_01_01():
    result = parse("Mafa-B11L*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B11L*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B11L*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B11L_01_02():
    result = parse("Mafa-B11L*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B11L*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B11L*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B11L_01_03():
    result = parse("Mafa-B11L*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B11L*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B11L*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_01():
    result = parse("Mafa-B16*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_02():
    result = parse("Mafa-B16*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_03():
    result = parse("Mafa-B16*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_04():
    result = parse("Mafa-B16*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_05():
    result = parse("Mafa-B16*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_06():
    result = parse("Mafa-B16*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_07():
    result = parse("Mafa-B16*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_08():
    result = parse("Mafa-B16*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_047_02():
    result = parse("Mafa-B*047:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*047:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*047:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_018_02_01_01():
    result = parse("Mafa-B*018:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*018:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*018:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_09():
    result = parse("Mafa-B*013:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_03():
    result = parse("Mafa-B*013:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_057_02_01_01():
    result = parse("Mafa-B*057:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*057:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*057:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_09():
    result = parse("Mafa-B16*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_33():
    result = parse("Mafa-A2*05:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_10():
    result = parse("Mafa-B16*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B16_01_11():
    result = parse("Mafa-B16*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B16*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B16*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_161_01():
    result = parse("Mafa-B*161:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*161:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*161:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_02_01():
    result = parse("Mamu-B17*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_03_01():
    result = parse("Mamu-B17*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_01():
    result = parse("Mamu-B17*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_04_01():
    result = parse("Mamu-B17*01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_02():
    result = parse("Mamu-B17*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_01_03():
    result = parse("Mamu-B17*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B17*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B17_01_05():
    result = parse("Mamu-B17*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B17*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B17*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_34_01_01():
    result = parse("Mamu-I*01:34:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:34:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:34:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_01_01():
    result = parse("Mamu-I*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_02_02():
    result = parse("Mamu-I*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_34():
    result = parse("Mafa-A2*05:34")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:34") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:34") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_05_01():
    result = parse("Mafa-A1*018:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*018:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_071_03():
    result = parse("Mafa-A1*071:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*071:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*071:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_092_03():
    result = parse("Mafa-A1*092:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*092:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*092:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_06():
    result = parse("Mafa-A1*018:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*018:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_062_04():
    result = parse("Mafa-A1*062:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*062:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*062:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_103_02():
    result = parse("Mafa-A1*103:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*103:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*103:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_114_01_01():
    result = parse("Mafa-B*114:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*114:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*114:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_15_01():
    result = parse("Mafa-I*01:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_16():
    result = parse("Mafa-I*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_13_02():
    result = parse("Mafa-I*01:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_17():
    result = parse("Mafa-I*01:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_18_01():
    result = parse("Mafa-I*01:18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:18:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_19_01_01():
    result = parse("Mafa-I*01:19:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:19:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-I*01:19:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_20N():
    result = parse("Mafa-I*01:20N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:20N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:20N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_03_01_02():
    result = parse("Mafa-I*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_35():
    result = parse("Mafa-A2*05:35")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:35") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:35") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_36():
    result = parse("Mafa-A2*05:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_05():
    result = parse("Mafa-A1*060:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*060:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_09():
    result = parse("Mafa-DRA*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_03_02():
    result = parse("Mamu-B*051:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*051:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_05():
    result = parse("Mamu-A1*028:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*028:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_02_01_01():
    result = parse("Mamu-B*056:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*056:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_03_01_01():
    result = parse("Mamu-B*079:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*079:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_01():
    result = parse("Mafa-DRA*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_05():
    result = parse("Mafa-DRA*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_04():
    result = parse("Mafa-DRA*01:03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_10_02():
    result = parse("Mafa-DRA*01:10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:10:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_06():
    result = parse("Mafa-DRA*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_05():
    result = parse("Mafa-DRA*01:03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_18():
    result = parse("Mafa-DRA*01:02:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:18") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_05():
    result = parse("Mafa-DRA*01:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_16():
    result = parse("Mafa-DRA*01:02:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:16") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_02_01():
    result = parse("Mamu-I*01:06:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:06:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_11_01_01():
    result = parse("Mamu-B11L*01:11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B11L*01:11:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_17():
    result = parse("Mamu-E*02:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_05():
    result = parse("Mamu-DQA1*24:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_04_02():
    result = parse("Mamu-DPA1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_08():
    result = parse("Mamu-DPA1*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_04_01():
    result = parse("Mamu-DPA1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_07():
    result = parse("Mafa-A1*089:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_07():
    result = parse("Mamu-E*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_09():
    result = parse("Mamu-E*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_074_03():
    result = parse("Mamu-A1*074:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*074:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*074:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_16_02():
    result = parse("Mamu-A2*05:16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:16:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_047_05():
    result = parse("Mamu-B*047:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*047:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*047:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_05():
    result = parse("Mamu-A5*30:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A5*30:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_F_02_06():
    result = parse("Mamu-F*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-F*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-F*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_01():
    result = parse("Mafa-B*134:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*134:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_135_01():
    result = parse("Mafa-B*135:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*135:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*135:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_136_01():
    result = parse("Mafa-B*136:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*136:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*136:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_012_01():
    result = parse("Mafa-B*012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_018_01_02():
    result = parse("Mafa-B*018:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*018:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*018:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_090_01():
    result = parse("Mafa-B*090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_01():
    result = parse("Mafa-B*048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_118_01():
    result = parse("Mafa-B*118:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*118:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*118:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_037_03():
    result = parse("Mafa-B*037:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*037:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*037:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_028_01():
    result = parse("Mafa-B*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_01_01():
    result = parse("Mafa-B*036:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*036:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_01_01():
    result = parse("Mafa-B*137:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*137:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_138_01():
    result = parse("Mafa-B*138:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*138:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*138:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_061_01():
    result = parse("Mafa-B*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_01():
    result = parse("Mafa-B*068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_019_01():
    result = parse("Mafa-B*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_139_01():
    result = parse("Mafa-B*139:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*139:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*139:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_058_01():
    result = parse("Mafa-B*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_141_01():
    result = parse("Mafa-B*141:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*141:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*141:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_010_01():
    result = parse("Mafa-B*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_142_01():
    result = parse("Mafa-B*142:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*142:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*142:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_143_01():
    result = parse("Mafa-B*143:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*143:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*143:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_121_01():
    result = parse("Mafa-B*121:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*121:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*121:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_04_01():
    result = parse("Mafa-I*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_144_01_01_01():
    result = parse("Mafa-B*144:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*144:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*144:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_145_01_01_01():
    result = parse("Mafa-B*145:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*145:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*145:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_085_01_01_01():
    result = parse("Mafa-B*085:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*085:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*085:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_17_01_01():
    result = parse("Mafa-B*030:17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*030:17:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_031_01():
    result = parse("Mafa-B*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_034_01():
    result = parse("Mafa-B*034:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*034:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*034:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_028_03_01_01():
    result = parse("Mafa-B*028:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*028:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*028:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_12_01_01():
    result = parse("Mafa-B*030:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*030:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_028_02_01_01():
    result = parse("Mafa-B*028:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*028:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*028:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_056_01_01_01():
    result = parse("Mafa-B*056:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*056:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*056:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_01():
    result = parse("Mafa-B*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_01_01_01():
    result = parse("Mafa-B*030:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*030:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_02_01():
    result = parse("Mafa-B*030:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_03_01_01():
    result = parse("Mafa-B*030:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*030:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_074_01N():
    result = parse("Mafa-B*074:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*074:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*074:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_01_01_01():
    result = parse("Mafa-B*007:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*007:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_018_01_01_01():
    result = parse("Mafa-B*018:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*018:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*018:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_073_01():
    result = parse("Mafa-B*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_02_01_01():
    result = parse("Mafa-B*134:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*134:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_104_01_01_01():
    result = parse("Mafa-B*104:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*104:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*104:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_104_02():
    result = parse("Mafa-B*104:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*104:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*104:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_064_01_01_01():
    result = parse("Mafa-B*064:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*064:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*064:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_147_01_01_01():
    result = parse("Mafa-B*147:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*147:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*147:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_147_02():
    result = parse("Mafa-B*147:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*147:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*147:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_148_01_01():
    result = parse("Mafa-B*148:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*148:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*148:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_037_01_01_01():
    result = parse("Mafa-B*037:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*037:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*037:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_04():
    result = parse("Mafa-B*007:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*007:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_144_02_01_01():
    result = parse("Mafa-B*144:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*144:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*144:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_01_01_01():
    result = parse("Mafa-B*046:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*046:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_019_03_01_01():
    result = parse("Mafa-B*019:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*019:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*019:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_01_01_01():
    result = parse("Mafa-B*088:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*088:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_150_01_01_01():
    result = parse("Mafa-B*150:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*150:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*150:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_151_01():
    result = parse("Mafa-B*151:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*151:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*151:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_033_01_01_01():
    result = parse("Mafa-B*033:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*033:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*033:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_01_01_01():
    result = parse("Mafa-B*098:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*098:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_057_01():
    result = parse("Mafa-B*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_149_01_01_01():
    result = parse("Mafa-B*149:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*149:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*149:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_136_03():
    result = parse("Mafa-B*136:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*136:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*136:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_11_01_01():
    result = parse("Mafa-B*013:11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*013:11:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_04():
    result = parse("Mafa-B*048:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*048:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_02():
    result = parse("Mafa-B*137:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*137:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_03():
    result = parse("Mafa-B*068:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_142_02():
    result = parse("Mafa-B*142:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*142:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*142:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_034_02():
    result = parse("Mafa-B*034:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*034:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*034:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_02():
    result = parse("Mafa-B*015:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_03():
    result = parse("Mafa-B*007:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*007:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_104_04():
    result = parse("Mafa-B*104:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*104:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*104:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_064_02():
    result = parse("Mafa-B*064:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*064:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*064:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_148_01_02():
    result = parse("Mafa-B*148:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*148:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*148:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_095_02():
    result = parse("Mafa-B*095:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*095:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*095:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_075_02_01():
    result = parse("Mafa-B*075:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*075:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*075:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_05():
    result = parse("Mafa-B*046:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_06():
    result = parse("Mafa-B*046:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_07N():
    result = parse("Mafa-B*046:07N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:07N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:07N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_04():
    result = parse("Mafa-B*013:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_150_01_02():
    result = parse("Mafa-B*150:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*150:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*150:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_150_02():
    result = parse("Mafa-B*150:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*150:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*150:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_027_01():
    result = parse("Mafa-B*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_027_02_01_01():
    result = parse("Mafa-B*027:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*027:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*027:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_01_01_01():
    result = parse("Mafa-B*044:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*044:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_01_02():
    result = parse("Mafa-B*044:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*044:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_02():
    result = parse("Mafa-B*044:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_03_01():
    result = parse("Mafa-B*044:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*044:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_01():
    result = parse("Mafa-B*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_02():
    result = parse("Mafa-B*051:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_025_01():
    result = parse("Mafa-B*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_162_01_01_01():
    result = parse("Mafa-B*162:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*162:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*162:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_002_01():
    result = parse("Mafa-B*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_002_02():
    result = parse("Mafa-B*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_065_01():
    result = parse("Mafa-B*065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_102_01():
    result = parse("Mafa-B*102:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*102:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*102:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_103_01_01():
    result = parse("Mafa-B*103:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*103:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*103:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_054_01():
    result = parse("Mafa-B*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_014_01_01_01():
    result = parse("Mafa-B*014:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*014:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*014:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_106_01():
    result = parse("Mafa-B*106:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*106:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*106:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_01():
    result = parse("Mafa-B*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_01():
    result = parse("Mafa-B*109:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_021_01():
    result = parse("Mafa-B*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_062_01N():
    result = parse("Mafa-B*062:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*062:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*062:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_02_01_01():
    result = parse("Mafa-B*068:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*068:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_139_02_01_01():
    result = parse("Mafa-B*139:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*139:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*139:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_104_01_02_01():
    result = parse("Mafa-B*104:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*104:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*104:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_02():
    result = parse("Mafa-B*069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_077_01():
    result = parse("Mafa-B*077:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*077:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*077:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_123_01_01():
    result = parse("Mane-B*123:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*123:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*123:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_01():
    result = parse("Mane-B*068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_02_01():
    result = parse("Mane-B*068:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*068:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_015_01():
    result = parse("Mane-B*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_045_01():
    result = parse("Mane-B*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_030_05():
    result = parse("Mane-B*030:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*030:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*030:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_016_01():
    result = parse("Mane-B*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_083_01():
    result = parse("Mane-B*083:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*083:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*083:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_105_01():
    result = parse("Mane-B*105:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*105:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*105:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_041_02():
    result = parse("Mane-B*041:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*041:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*041:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_125_01():
    result = parse("Mane-B*125:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*125:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*125:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_008_01():
    result = parse("Mane-B*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_03():
    result = parse("Mane-B*068:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafu_DRB_W031_01():
    result = parse("Mafu-DRB*W031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafu-DRB*W031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafu-DRB*W031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_08_01():
    result = parse("Mane-B17*01:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_02_05():
    result = parse("Mane-B17*01:02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:02:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_10():
    result = parse("Mane-B17*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B17*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_02_03():
    result = parse("Mane-B17*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_08_02():
    result = parse("Mane-B17*01:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_09():
    result = parse("Mane-B17*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B17*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_02_06():
    result = parse("Mane-B17*01:02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:02:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_02_04():
    result = parse("Mane-B17*01:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_09_01():
    result = parse("Ovar-N*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_10_01():
    result = parse("Ovar-N*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_11_01():
    result = parse("Ovar-N*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_14_02():
    result = parse("Ovar-DRB1*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_01_04():
    result = parse("Ovar-DRB1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_05_02():
    result = parse("Ovar-DRB1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_03():
    result = parse("Ovar-DRB1*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_02():
    result = parse("Ovar-DRB1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_12_02():
    result = parse("Ovar-DRB1*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_12_03():
    result = parse("Ovar-DRB1*12:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*12:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*12:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_09():
    result = parse("Ovar-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W072_01():
    result = parse("Mafa-DRB*W072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_04():
    result = parse("Mafa-B*030:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_05_01():
    result = parse("Mafa-B*030:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_061_03():
    result = parse("Mamu-B*061:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*061:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*061:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_01_02():
    result = parse("Mamu-A1*008:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*008:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_10():
    result = parse("Mamu-A1*003:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_042_01():
    result = parse("Mamu-A1*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_066_01():
    result = parse("Mamu-A1*066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_08_01_01():
    result = parse("Mamu-A1*018:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*018:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_07():
    result = parse("Mamu-A1*018:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_07():
    result = parse("Mamu-A1*019:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_022_03():
    result = parse("Mamu-A1*022:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*022:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*022:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_105_01():
    result = parse("Mamu-A1*105:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*105:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*105:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_023_02():
    result = parse("Mamu-A1*023:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*023:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*023:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_106_01():
    result = parse("Mamu-A1*106:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*106:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*106:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_06():
    result = parse("Mamu-A1*028:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*028:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_02():
    result = parse("Mamu-A1*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_03():
    result = parse("Mamu-A1*032:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*032:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_049_04():
    result = parse("Mamu-A1*049:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*049:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*049:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_107_01():
    result = parse("Mamu-A1*107:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*107:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*107:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_056_02_02():
    result = parse("Mamu-A1*056:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*056:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*056:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_108_01():
    result = parse("Mamu-A1*108:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*108:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*108:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_109_01():
    result = parse("Mamu-A1*109:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*109:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*109:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_105_02():
    result = parse("Mamu-A1*105:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*105:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*105:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_110_01():
    result = parse("Mamu-A1*110:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*110:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*110:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_111_01():
    result = parse("Mamu-A1*111:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*111:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*111:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_01_03():
    result = parse("Mamu-A2*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_15_04():
    result = parse("Mamu-A2*05:15:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:15:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:15:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_44():
    result = parse("Mamu-A2*05:44")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:44") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:44") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_45():
    result = parse("Mamu-A2*05:45")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:45") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:45") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_01_02_02():
    result = parse("Mamu-A4*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A4*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_32_02():
    result = parse("Mamu-A2*05:32:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:32:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:32:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_46():
    result = parse("Mamu-A2*05:46")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:46") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:46") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_06():
    result = parse("Mamu-A5*30:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A5*30:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_05_01_01():
    result = parse("Mamu-A6*01:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A6*01:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_01_03():
    result = parse("Mamu-A1*003:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*003:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_112_01():
    result = parse("Mamu-A1*112:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*112:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*112:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_105_04():
    result = parse("Mamu-A1*105:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*105:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*105:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_01_03():
    result = parse("Mamu-A1*008:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*008:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_105_03():
    result = parse("Mamu-A1*105:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*105:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*105:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_13():
    result = parse("Mamu-A3*13:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_030_02():
    result = parse("Mane-B*030:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*030:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*030:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_115_01():
    result = parse("Mane-B*115:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*115:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*115:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_015_02():
    result = parse("Mane-B*015:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*015:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*015:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_089_01():
    result = parse("Mane-B*089:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*089:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*089:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_104_01():
    result = parse("Mane-B*104:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*104:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*104:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_01():
    result = parse("Mane-B*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_039_01():
    result = parse("Mane-B*039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_035_01():
    result = parse("Mane-B*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_088_01():
    result = parse("Mane-B*088:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*088:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*088:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_121_01():
    result = parse("Mane-B*121:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*121:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*121:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_007_01():
    result = parse("Mane-B*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_103_01_01():
    result = parse("Mane-B*103:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*103:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*103:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_118_01_01():
    result = parse("Mane-B*118:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*118:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*118:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_104_02():
    result = parse("Mane-B*104:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*104:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*104:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_107_01():
    result = parse("Mane-B*107:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*107:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*107:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_120_01():
    result = parse("Mane-B*120:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*120:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*120:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_024_01():
    result = parse("Mane-B*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_061_01():
    result = parse("Mane-B*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_103_02():
    result = parse("Mane-B*103:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*103:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*103:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_056_01():
    result = parse("Mane-B*056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_116_01_01():
    result = parse("Mane-B*116:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*116:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*116:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_011_01():
    result = parse("Mane-B*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_030_03():
    result = parse("Mane-B*030:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*030:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*030:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_082_01():
    result = parse("Mane-B*082:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*082:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*082:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_118_02():
    result = parse("Mane-B*118:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*118:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*118:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_047_01_01():
    result = parse("Mane-B*047:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*047:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*047:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_112_01():
    result = parse("Mane-B*112:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*112:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*112:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_078_01():
    result = parse("Mane-B*078:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*078:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*078:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_043_01():
    result = parse("Mane-B*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_104_03():
    result = parse("Mane-B*104:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*104:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*104:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_052_01():
    result = parse("Mane-B*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_078_02():
    result = parse("Mane-B*078:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*078:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*078:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_119_01():
    result = parse("Mane-B*119:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*119:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*119:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_118_03():
    result = parse("Mane-B*118:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*118:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*118:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_082_02():
    result = parse("Mane-B*082:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*082:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*082:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_015_03():
    result = parse("Mane-B*015:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*015:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*015:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_110_01():
    result = parse("Mane-B*110:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*110:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*110:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_107_02():
    result = parse("Mane-B*107:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*107:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*107:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_017_03():
    result = parse("Mane-B*017:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*017:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*017:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_069_01():
    result = parse("Mane-B*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_030_04():
    result = parse("Mane-B*030:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*030:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*030:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_083_02():
    result = parse("Mane-B*083:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*083:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*083:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_091_01():
    result = parse("Mane-B*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_02_01():
    result = parse("Mane-B*072:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_061_02():
    result = parse("Mane-B*061:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*061:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*061:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_028_01_01():
    result = parse("Mane-B*028:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*028:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*028:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_113_01():
    result = parse("Mane-B*113:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*113:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*113:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_117_01():
    result = parse("Mane-B*117:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*117:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*117:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_01():
    result = parse("Mane-B*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_111_01():
    result = parse("Mane-B*111:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*111:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*111:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_082_03():
    result = parse("Mane-B*082:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*082:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*082:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_08_02():
    result = parse("Mane-I*01:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_11():
    result = parse("Mane-I*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_098_01():
    result = parse("Mane-B*098:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*098:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*098:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_08_03():
    result = parse("Mane-I*01:08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:08:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_03_02_01():
    result = parse("Mafa-B*030:03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*030:03:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_05():
    result = parse("Mamu-A1*007:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*007:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_059_01():
    result = parse("Mamu-A1*059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A7_02_01():
    result = parse("Mamu-A7*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A7*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_038_02_01_01():
    result = parse("Mamu-B*038:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*038:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*038:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_02_01():
    result = parse("Mamu-B*063:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*063:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_064_02():
    result = parse("Mamu-B*064:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*064:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*064:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_01_01_01():
    result = parse("Mamu-B*098:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_099_01():
    result = parse("Mamu-B*099:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*099:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*099:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_100_01():
    result = parse("Mamu-B*100:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*100:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*100:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_101_01():
    result = parse("Mamu-B*101:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*101:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*101:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_07():
    result = parse("Mamu-I*01:06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_02():
    result = parse("Mamu-I*01:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_05():
    result = parse("Mamu-DQA1*26:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_17():
    result = parse("Mamu-DQB1*06:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_22():
    result = parse("Mamu-DQB1*18:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_057_01():
    result = parse("Mane-B*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_02_01():
    result = parse("Mane-B*051:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*051:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_01_01():
    result = parse("Mane-B*079:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_108_01():
    result = parse("Mane-B*108:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*108:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*108:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_064_01():
    result = parse("Mane-B*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_058_01():
    result = parse("Mane-B*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_120_02_01():
    result = parse("Mane-B*120:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*120:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*120:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_089_02():
    result = parse("Mane-B*089:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*089:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*089:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_03():
    result = parse("Mane-B*072:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*072:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_041_01():
    result = parse("Mane-B*041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_114_01():
    result = parse("Mane-A1*114:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*114:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*114:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_04():
    result = parse("Mafa-A5*30:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A5*30:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_07_01():
    result = parse("Patr-DRB3*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_07_02():
    result = parse("Patr-DRB3*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_07_02():
    result = parse("Patr-DRB1*03:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DRB1*03:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB3_01_03():
    result = parse("Patr-DRB3*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB3*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB3*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_01_03():
    result = parse("Patr-DRB5*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_13():
    result = parse("Patr-DRB5*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_15():
    result = parse("Patr-DRB1*02:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_019_02():
    result = parse("Mafa-B*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_093_01():
    result = parse("Mafa-B*093:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*093:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*093:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_092_01():
    result = parse("Mafa-B*092:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*092:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*092:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_01_01_01():
    result = parse("Mafa-B*070:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*070:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_131_01():
    result = parse("Mafa-B*131:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*131:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*131:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W020_02():
    result = parse("Mafa-DRB*W020:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W020:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W020:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W063_03():
    result = parse("Mafa-DRB*W063:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W063:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W063:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_05():
    result = parse("Mafa-DRB*W007:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W026_03():
    result = parse("Mafa-DRB*W026:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W026:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W026:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W026_02_02():
    result = parse("Mafa-DRB*W026:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W026:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W026:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_05():
    result = parse("Mafa-DRB*W001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_03_02():
    result = parse("Mafa-DRB*W003:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W003:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W040_02():
    result = parse("Mafa-DRB*W040:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W040:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W040:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_06():
    result = parse("Mafa-DRB*W001:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W059_01_01():
    result = parse("Mafa-DRB*W059:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W059:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W059:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W059_01_02():
    result = parse("Mafa-DRB*W059:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W059:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W059:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W020_03():
    result = parse("Mafa-DRB*W020:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W020:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W020:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_12():
    result = parse("Mafa-DRB1*04:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_25():
    result = parse("Mafa-DRB1*03:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_11():
    result = parse("Mafa-DRB1*10:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_26():
    result = parse("Mafa-DRB1*03:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_10():
    result = parse("Mafa-DRB1*10:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_10():
    result = parse("Mafa-DRB4*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_16():
    result = parse("Mafa-DRB6*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_17():
    result = parse("Mafa-DRB6*01:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_18():
    result = parse("Mafa-DRB6*01:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_24():
    result = parse("Mafa-DRB6*01:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_23():
    result = parse("Mafa-DRB6*01:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_21():
    result = parse("Mafa-DRB6*01:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_22():
    result = parse("Mafa-DRB6*01:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_19():
    result = parse("Mafa-DRB6*01:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_28():
    result = parse("Mafa-DRB6*01:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_26():
    result = parse("Mafa-DRB6*01:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_29():
    result = parse("Mafa-DRB6*01:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_25():
    result = parse("Mafa-DRB6*01:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_11_02():
    result = parse("Mafa-DRB6*01:11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB6*01:11:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_27():
    result = parse("Mafa-DRB6*01:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_20():
    result = parse("Mafa-DRB6*01:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_08N():
    result = parse("Mafa-B*030:08N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:08N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:08N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_01_03():
    result = parse("Mafa-B*030:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_05_02():
    result = parse("Mafa-B*030:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_06():
    result = parse("Mafa-B*030:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_07_01():
    result = parse("Mafa-B*030:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_127_01N():
    result = parse("Mafa-B*127:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*127:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*127:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_01_02N():
    result = parse("Patr-B*01:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*01:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*01:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_13():
    result = parse("Patr-DRB1*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_14():
    result = parse("Patr-DRB1*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_02_01():
    result = parse("Mafa-DRA*01:03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-DRA*01:03:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_03():
    result = parse("Mafa-DRA*01:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_03():
    result = parse("Ovar-DRB1*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_10():
    result = parse("Ovar-DRB1*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_23_01():
    result = parse("Ovar-DRB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_04():
    result = parse("Ovar-DRB1*16:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_012_01():
    result = parse("BoLA-2*012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_015_02():
    result = parse("BoLA-6*015:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*015:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*015:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_006_02():
    result = parse("BoLA-2*006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_001_01():
    result = parse("BoLA-NC1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_008_02():
    result = parse("BoLA-2*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_001_01():
    result = parse("BoLA-DQA*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_001_02():
    result = parse("BoLA-DQA*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_001_03():
    result = parse("BoLA-DQA*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_01():
    result = parse("BoLA-DQA*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_02():
    result = parse("BoLA-DQA*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_03():
    result = parse("BoLA-DQA*002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_04():
    result = parse("BoLA-DQA*002:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_003_01():
    result = parse("BoLA-DQA*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_003_02():
    result = parse("BoLA-DQA*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_004_01():
    result = parse("BoLA-DQA*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_008_01():
    result = parse("BoLA-DQA*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_008_02():
    result = parse("BoLA-DQA*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_010_01_01():
    result = parse("BoLA-DQA*010:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*010:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*010:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_010_01_02():
    result = parse("BoLA-DQA*010:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*010:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*010:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_011_01():
    result = parse("BoLA-DQA*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_012_01_01():
    result = parse("BoLA-DQA*012:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*012:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*012:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_012_01_02():
    result = parse("BoLA-DQA*012:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*012:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*012:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_012_02_01():
    result = parse("BoLA-DQA*012:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*012:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*012:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_012_02_02():
    result = parse("BoLA-DQA*012:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*012:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*012:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_012_03():
    result = parse("BoLA-DQA*012:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*012:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*012:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_013_01():
    result = parse("BoLA-DQA*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_013_02():
    result = parse("BoLA-DQA*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_014_01():
    result = parse("BoLA-DQA*014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_020_01():
    result = parse("BoLA-DQA*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_020_02():
    result = parse("BoLA-DQA*020:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*020:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*020:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_021_01():
    result = parse("BoLA-DQA*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_01():
    result = parse("BoLA-DQA*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_02_01():
    result = parse("BoLA-DQA*022:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*022:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_02_02():
    result = parse("BoLA-DQA*022:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*022:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_02_03():
    result = parse("BoLA-DQA*022:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*022:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_03_01():
    result = parse("BoLA-DQA*022:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*022:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_03_02():
    result = parse("BoLA-DQA*022:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*022:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_04():
    result = parse("BoLA-DQA*022:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*022:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_05():
    result = parse("BoLA-DQA*022:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*022:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_023_01():
    result = parse("BoLA-DQA*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_024_01():
    result = parse("BoLA-DQA*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_025_01_01():
    result = parse("BoLA-DQA*025:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*025:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*025:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_025_01_02():
    result = parse("BoLA-DQA*025:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*025:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*025:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_025_02():
    result = parse("BoLA-DQA*025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_026_01():
    result = parse("BoLA-DQA*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_026_02():
    result = parse("BoLA-DQA*026:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*026:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*026:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_026_03():
    result = parse("BoLA-DQA*026:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*026:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*026:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_027_01_01():
    result = parse("BoLA-DQA*027:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*027:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*027:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_027_01_02():
    result = parse("BoLA-DQA*027:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*027:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQA*027:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_027_02():
    result = parse("BoLA-DQA*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_028_01():
    result = parse("BoLA-DQA*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_001_01():
    result = parse("BoLA-DQB*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_001_02():
    result = parse("BoLA-DQB*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_001_03():
    result = parse("BoLA-DQB*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_002_01():
    result = parse("BoLA-DQB*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_003_01():
    result = parse("BoLA-DQB*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_003_02():
    result = parse("BoLA-DQB*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_004_01():
    result = parse("BoLA-DQB*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_004_02():
    result = parse("BoLA-DQB*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_004_03():
    result = parse("BoLA-DQB*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_004_04():
    result = parse("BoLA-DQB*004:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*004:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*004:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_005_01():
    result = parse("BoLA-DQB*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_005_02():
    result = parse("BoLA-DQB*005:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*005:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*005:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_006_01():
    result = parse("BoLA-DQB*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_007_01():
    result = parse("BoLA-DQB*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_008_01():
    result = parse("BoLA-DQB*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_009_01():
    result = parse("BoLA-DQB*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_009_02():
    result = parse("BoLA-DQB*009:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*009:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*009:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_009_03():
    result = parse("BoLA-DQB*009:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*009:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*009:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_010_01():
    result = parse("BoLA-DQB*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_010_02_01():
    result = parse("BoLA-DQB*010:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*010:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQB*010:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_010_03():
    result = parse("BoLA-DQB*010:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*010:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*010:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_010_05():
    result = parse("BoLA-DQB*010:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*010:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*010:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_010_06():
    result = parse("BoLA-DQB*010:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*010:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*010:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_010_08():
    result = parse("BoLA-DQB*010:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*010:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*010:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_010_09():
    result = parse("BoLA-DQB*010:09")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*010:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*010:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_011_01():
    result = parse("BoLA-DQB*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_012_01():
    result = parse("BoLA-DQB*012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_013_01():
    result = parse("BoLA-DQB*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_014_01():
    result = parse("BoLA-DQB*014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_014_02():
    result = parse("BoLA-DQB*014:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*014:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*014:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_014_03():
    result = parse("BoLA-DQB*014:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*014:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*014:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_014_04():
    result = parse("BoLA-DQB*014:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*014:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*014:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_015_01():
    result = parse("BoLA-DQB*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_016_01():
    result = parse("BoLA-DQB*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_016_02():
    result = parse("BoLA-DQB*016:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*016:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*016:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_017_01():
    result = parse("BoLA-DQB*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_017_02():
    result = parse("BoLA-DQB*017:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*017:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*017:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_01():
    result = parse("BoLA-DQB*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_02():
    result = parse("BoLA-DQB*018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_03():
    result = parse("BoLA-DQB*018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_04():
    result = parse("BoLA-DQB*018:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_05():
    result = parse("BoLA-DQB*018:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_06():
    result = parse("BoLA-DQB*018:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_019_01():
    result = parse("BoLA-DQB*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_019_02():
    result = parse("BoLA-DQB*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_020_01():
    result = parse("BoLA-DQB*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_021_01():
    result = parse("BoLA-DQB*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_022_01():
    result = parse("BoLA-DQB*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_023_01():
    result = parse("BoLA-DQB*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_024_01():
    result = parse("BoLA-DQB*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_025_01():
    result = parse("BoLA-DQB*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_025_02():
    result = parse("BoLA-DQB*025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_026_01():
    result = parse("BoLA-DQB*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_027_01():
    result = parse("BoLA-DQB*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_027_02():
    result = parse("BoLA-DQB*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_028_01():
    result = parse("BoLA-DQB*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_029_01():
    result = parse("BoLA-DQB*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_030_01():
    result = parse("BoLA-DQB*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_031_01():
    result = parse("BoLA-DQB*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_032_01():
    result = parse("BoLA-DQB*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_033_01():
    result = parse("BoLA-DQB*033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_033_02():
    result = parse("BoLA-DQB*033:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*033:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*033:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_034_01():
    result = parse("BoLA-DQB*034:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*034:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*034:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_035_01():
    result = parse("BoLA-DQB*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_036_01():
    result = parse("BoLA-DQB*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_037_01():
    result = parse("BoLA-DQB*037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_038_01():
    result = parse("BoLA-DQB*038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_001_01():
    result = parse("BoLA-DRB3*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_002_01():
    result = parse("BoLA-DRB3*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_003_01():
    result = parse("BoLA-DRB3*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_003_02_01():
    result = parse("BoLA-DRB3*003:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*003:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*003:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_004_01():
    result = parse("BoLA-DRB3*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_005_01():
    result = parse("BoLA-DRB3*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_005_02():
    result = parse("BoLA-DRB3*005:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*005:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*005:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_005_03():
    result = parse("BoLA-DRB3*005:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*005:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*005:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_006_01():
    result = parse("BoLA-DRB3*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_007_01():
    result = parse("BoLA-DRB3*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_008_01():
    result = parse("BoLA-DRB3*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_009_01():
    result = parse("BoLA-DRB3*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_009_02():
    result = parse("BoLA-DRB3*009:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*009:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*009:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_010_01():
    result = parse("BoLA-DRB3*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_010_02():
    result = parse("BoLA-DRB3*010:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*010:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*010:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_011_01():
    result = parse("BoLA-DRB3*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_012_01():
    result = parse("BoLA-DRB3*012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_012_02():
    result = parse("BoLA-DRB3*012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_013_01():
    result = parse("BoLA-DRB3*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_014_01_01():
    result = parse("BoLA-DRB3*014:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*014:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*014:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_014_01_02():
    result = parse("BoLA-DRB3*014:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*014:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*014:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_015_01():
    result = parse("BoLA-DRB3*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_015_02():
    result = parse("BoLA-DRB3*015:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*015:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*015:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_016_01():
    result = parse("BoLA-DRB3*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_017_01():
    result = parse("BoLA-DRB3*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_017_02():
    result = parse("BoLA-DRB3*017:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*017:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*017:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_017_03():
    result = parse("BoLA-DRB3*017:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*017:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*017:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_018_01():
    result = parse("BoLA-DRB3*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_018_02():
    result = parse("BoLA-DRB3*018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_019_01():
    result = parse("BoLA-DRB3*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_01_01():
    result = parse("BoLA-DRB3*020:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*020:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_01_02():
    result = parse("BoLA-DRB3*020:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*020:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_02():
    result = parse("BoLA-DRB3*020:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_03():
    result = parse("BoLA-DRB3*020:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_021_01():
    result = parse("BoLA-DRB3*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_01():
    result = parse("BoLA-DRB3*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_02():
    result = parse("BoLA-DRB3*022:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_023_01():
    result = parse("BoLA-DRB3*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_01():
    result = parse("BoLA-DRB3*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_025_01_01():
    result = parse("BoLA-DRB3*025:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*025:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*025:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_025_01_02():
    result = parse("BoLA-DRB3*025:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*025:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*025:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_026_01():
    result = parse("BoLA-DRB3*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_01():
    result = parse("BoLA-DRB3*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_02():
    result = parse("BoLA-DRB3*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_03():
    result = parse("BoLA-DRB3*027:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_04():
    result = parse("BoLA-DRB3*027:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_05():
    result = parse("BoLA-DRB3*027:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_06():
    result = parse("BoLA-DRB3*027:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_028_01():
    result = parse("BoLA-DRB3*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_028_02():
    result = parse("BoLA-DRB3*028:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*028:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*028:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_029_01():
    result = parse("BoLA-DRB3*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_030_01():
    result = parse("BoLA-DRB3*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_030_02():
    result = parse("BoLA-DRB3*030:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*030:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*030:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_031_01():
    result = parse("BoLA-DRB3*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_032_01():
    result = parse("BoLA-DRB3*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_032_02():
    result = parse("BoLA-DRB3*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_033_01():
    result = parse("BoLA-DRB3*033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_034_01():
    result = parse("BoLA-DRB3*034:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*034:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*034:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_034_02():
    result = parse("BoLA-DRB3*034:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*034:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*034:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_035_01():
    result = parse("BoLA-DRB3*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_036_01():
    result = parse("BoLA-DRB3*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_037_01():
    result = parse("BoLA-DRB3*037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_038_01():
    result = parse("BoLA-DRB3*038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_039_01():
    result = parse("BoLA-DRB3*039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_040_01():
    result = parse("BoLA-DRB3*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_001_01():
    result = parse("BoLA-3*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_001_02():
    result = parse("BoLA-3*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_001_03():
    result = parse("BoLA-3*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_002_01():
    result = parse("BoLA-3*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_5_003_01():
    result = parse("BoLA-5*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-5*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_004_01():
    result = parse("BoLA-3*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_004_02():
    result = parse("BoLA-3*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_005_01():
    result = parse("BoLA-2*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_006_01():
    result = parse("BoLA-2*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_007_01():
    result = parse("BoLA-1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_008_01():
    result = parse("BoLA-2*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_009_01():
    result = parse("BoLA-1*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_010_01():
    result = parse("BoLA-3*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_011_01():
    result = parse("BoLA-3*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_013_01():
    result = parse("BoLA-6*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_013_02():
    result = parse("BoLA-6*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_014_01():
    result = parse("BoLA-6*014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_014_02():
    result = parse("BoLA-6*014:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*014:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*014:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_015_01():
    result = parse("BoLA-6*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_016_01():
    result = parse("BoLA-2*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_017_01():
    result = parse("BoLA-3*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_018_01():
    result = parse("BoLA-2*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_018_02():
    result = parse("BoLA-2*018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_019_01():
    result = parse("BoLA-1*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_020_01():
    result = parse("BoLA-1*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_021_01():
    result = parse("BoLA-1*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_022_01():
    result = parse("BoLA-2*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_023_01():
    result = parse("BoLA-1*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_4_024_01():
    result = parse("BoLA-4*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-4*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-4*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_4_024_02():
    result = parse("BoLA-4*024:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-4*024:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-4*024:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_025_01():
    result = parse("BoLA-2*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_026_01():
    result = parse("BoLA-2*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_026_02():
    result = parse("BoLA-2*026:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*026:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*026:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_027_01():
    result = parse("BoLA-3*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_027_02():
    result = parse("BoLA-3*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_028_01():
    result = parse("BoLA-1*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_029_01():
    result = parse("BoLA-1*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_030_01():
    result = parse("BoLA-2*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_031_01():
    result = parse("BoLA-1*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_032_01N():
    result = parse("BoLA-2*032:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*032:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*032:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_033_01N():
    result = parse("BoLA-3*033:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*033:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*033:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_034_01():
    result = parse("BoLA-6*034:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*034:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*034:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_035_01():
    result = parse("BoLA-3*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_036_01():
    result = parse("BoLA-3*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_037_01():
    result = parse("BoLA-3*037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_038_01():
    result = parse("BoLA-3*038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_002_01():
    result = parse("BoLA-NC1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_003_01():
    result = parse("BoLA-NC1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC2_001_01():
    result = parse("BoLA-NC2*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC2*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC2*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC2_001_02():
    result = parse("BoLA-NC2*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC2*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC2*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC3_001_01():
    result = parse("BoLA-NC3*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC3*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC3*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC4_001_01():
    result = parse("BoLA-NC4*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC4*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC4*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC4_002_01():
    result = parse("BoLA-NC4*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC4*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC4*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_004_01():
    result = parse("BoLA-NC1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_040_01():
    result = parse("BoLA-6*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_5_039_01():
    result = parse("BoLA-5*039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5*039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-5*039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_041_01():
    result = parse("BoLA-6*041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_01():
    result = parse("Bubu-DQA*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_020_01():
    result = parse("Bubu-DQA*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_051_01():
    result = parse("BoLA-DRB3*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_010_02():
    result = parse("BoLA-DQA*010:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*010:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*010:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_012_04():
    result = parse("BoLA-DQA*012:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*012:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*012:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_043_02():
    result = parse("BoLA-DRB3*043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_004_05():
    result = parse("BoLA-DQB*004:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*004:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*004:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_003_03():
    result = parse("BoLA-DQB*003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_040_01():
    result = parse("BoLA-DQB*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_039_01():
    result = parse("BoLA-DQB*039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_057_01():
    result = parse("BoLA-DRB3*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_058_01():
    result = parse("BoLA-DRB3*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_059_01():
    result = parse("BoLA-DRB3*059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_060_01():
    result = parse("BoLA-DRB3*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_061_01():
    result = parse("BoLA-DRB3*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_057_02():
    result = parse("BoLA-DRB3*057:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*057:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*057:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_062_01():
    result = parse("BoLA-DRB3*062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_063_01():
    result = parse("BoLA-DRB3*063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_064_01():
    result = parse("BoLA-DRB3*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_031_03():
    result = parse("BoLA-DRB3*031:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*031:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*031:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_065_01():
    result = parse("BoLA-DRB3*065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_066_01():
    result = parse("BoLA-DRB3*066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_043_03():
    result = parse("BoLA-DRB3*043:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*043:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*043:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_064_02():
    result = parse("BoLA-DRB3*064:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*064:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*064:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_022_07():
    result = parse("BoLA-DQA*022:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*022:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*022:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_042_01():
    result = parse("BoLA-1*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_009_02():
    result = parse("BoLA-1*009:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*009:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*009:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_043_01():
    result = parse("BoLA-2*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_044_01():
    result = parse("BoLA-2*044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_045_01():
    result = parse("BoLA-2*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_046_01():
    result = parse("BoLA-2*046:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*046:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*046:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_047_01():
    result = parse("BoLA-2*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_048_01():
    result = parse("BoLA-2*048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_016_02():
    result = parse("BoLA-2*016:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*016:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*016:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_049_01():
    result = parse("BoLA-1*049:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*049:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*049:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_050_01():
    result = parse("BoLA-3*050:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*050:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*050:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_051_01():
    result = parse("BoLA-3*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_052_01():
    result = parse("BoLA-3*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_053_01():
    result = parse("BoLA-3*053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_017_02():
    result = parse("BoLA-3*017:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*017:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*017:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_054_01():
    result = parse("BoLA-2*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_055_01():
    result = parse("BoLA-2*055:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*055:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*055:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_056_01():
    result = parse("BoLA-2*056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_008_02():
    result = parse("BoLA-DQB*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_030_22():
    result = parse("BoLA-DRB3*030:22")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*030:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*030:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_003_03():
    result = parse("BoLA-DRB3*003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_005_04():
    result = parse("BoLA-DRB3*005:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*005:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*005:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_013_02():
    result = parse("BoLA-DRB3*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_019_02():
    result = parse("BoLA-DRB3*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_043_01():
    result = parse("BoLA-DRB3*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_044_01():
    result = parse("BoLA-DRB3*044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_048_02():
    result = parse("BoLA-DRB3*048:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*048:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*048:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_25_02():
    result = parse("Mafa-A2*05:25:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:25:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:25:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_40_01_01():
    result = parse("Mafa-A2*05:40:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:40:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:40:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_41():
    result = parse("Mafa-A2*05:41")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:41") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:41") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_42():
    result = parse("Mafa-A2*05:42")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:42") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:42") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_43():
    result = parse("Mafa-A2*05:43")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:43") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:43") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_44():
    result = parse("Mafa-A2*05:44")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:44") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:44") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_38_02():
    result = parse("Mafa-A2*05:38:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:38:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:38:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_45():
    result = parse("Mafa-A2*05:45")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:45") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:45") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_46():
    result = parse("Mafa-A2*05:46")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:46") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:46") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_47():
    result = parse("Mafa-A2*05:47")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:47") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:47") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_03_02():
    result = parse("Mafa-A4*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_06():
    result = parse("Mafa-A4*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_03_01_01():
    result = parse("Mafa-A4*14:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_07():
    result = parse("Mafa-A4*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_07_01_01():
    result = parse("Mafa-A3*13:07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A3*13:07:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_04_01():
    result = parse("Mafa-A4*14:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A4*14:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_08():
    result = parse("Mafa-A4*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_041_04():
    result = parse("Mafa-A1*041:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*041:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*041:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_08_01():
    result = parse("Mafa-B*046:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*046:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_09_01_01():
    result = parse("Mafa-B*046:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*046:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_076_02():
    result = parse("Mafa-B*076:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*076:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*076:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_10_01_01():
    result = parse("Mafa-B*046:10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*046:10:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_11_01_01():
    result = parse("Mafa-B*046:11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*046:11:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_12_01_01():
    result = parse("Mafa-B*046:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*046:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_132_01():
    result = parse("Mafa-B*132:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*132:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*132:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_083_01():
    result = parse("Mafa-B*083:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*083:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*083:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_05():
    result = parse("Mafa-B*013:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_06_01_01():
    result = parse("Mafa-B*013:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*013:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_144_04():
    result = parse("Mafa-B*144:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*144:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*144:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_033_03():
    result = parse("Mafa-B*033:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*033:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*033:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_012_02():
    result = parse("Mamu-B*012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_04():
    result = parse("Mamu-B*017:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*017:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_057_04():
    result = parse("Mamu-B*057:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*057:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*057:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_028_04():
    result = parse("Mamu-B*028:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*028:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*028:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_05():
    result = parse("Mamu-B*045:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*045:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_134_01():
    result = parse("Mamu-B*134:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*134:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*134:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_21():
    result = parse("Mamu-I*01:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_009_01():
    result = parse("Mamu-A1*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_057_03():
    result = parse("Mamu-A1*057:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*057:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*057:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_50():
    result = parse("Mafa-A2*05:50")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:50") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:50") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_48():
    result = parse("Mafa-A2*05:48")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:48") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:48") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_03():
    result = parse("Mafa-A2*24:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_052_03():
    result = parse("Mafa-A1*052:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*052:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*052:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_098_02():
    result = parse("Mafa-A1*098:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*098:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*098:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_08():
    result = parse("Mafa-A3*13:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_086_02():
    result = parse("Mafa-A1*086:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*086:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*086:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_49():
    result = parse("Mafa-A2*05:49")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:49") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:49") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_164_01():
    result = parse("Mafa-B*164:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*164:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*164:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_03_01_01():
    result = parse("Mafa-B*051:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*051:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_064_03():
    result = parse("Mafa-B*064:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*064:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*064:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_131_02_01_01():
    result = parse("Mafa-B*131:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*131:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*131:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_152_01_01_01N():
    result = parse("Mafa-B*152:01:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*152:01:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*152:01:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_02_01_01():
    result = parse("Mafa-B*109:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*109:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_152_02N():
    result = parse("Mafa-B*152:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*152:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*152:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_126_01():
    result = parse("Mafa-B*126:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*126:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*126:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_160_02():
    result = parse("Mafa-B*160:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*160:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*160:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_02_01():
    result = parse("Mafa-B*050:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*050:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_03():
    result = parse("Mafa-B*050:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*050:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_131_03():
    result = parse("Mafa-B*131:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*131:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*131:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_03():
    result = parse("Mafa-B*109:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_02_02():
    result = parse("Mafa-B*117:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*117:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_157_02():
    result = parse("Mafa-B*157:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*157:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*157:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_03_01():
    result = parse("Mafa-B*060:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*060:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_21():
    result = parse("Mafa-I*01:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_10():
    result = parse("Mamu-A4*14:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B12_01_01():
    result = parse("Mane-B12*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B12*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B12*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_007_01():
    result = parse("Mane-A1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B16_01_01():
    result = parse("Mane-B16*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B16*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B16*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B16_01_02():
    result = parse("Mane-B16*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B16*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B16*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_51():
    result = parse("Mafa-A2*05:51")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:51") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:51") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_03_02():
    result = parse("Mafa-A4*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A4*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_09():
    result = parse("Mafa-A3*13:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_10():
    result = parse("Mafa-A3*13:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_067_02():
    result = parse("Mafa-A1*067:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*067:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*067:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_09():
    result = parse("Mafa-A4*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_05():
    result = parse("Mafa-A4*14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_06():
    result = parse("Mafa-A4*14:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_10():
    result = parse("Mafa-A4*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_11():
    result = parse("Mafa-A4*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_067_03():
    result = parse("Mafa-A1*067:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*067:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*067:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_086_03():
    result = parse("Mafa-A1*086:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*086:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*086:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_07():
    result = parse("Mafa-A4*14:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_08_01():
    result = parse("Mafa-A4*14:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A4*14:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_11():
    result = parse("Mafa-A3*13:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_045_02():
    result = parse("Mafa-A1*045:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*045:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*045:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_153_02():
    result = parse("Mafa-B*153:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*153:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*153:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_16_01():
    result = parse("Mafa-B*046:16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*046:16:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_153_03():
    result = parse("Mafa-B*153:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*153:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*153:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_13():
    result = parse("Mafa-B*046:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_14():
    result = parse("Mafa-B*046:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_52():
    result = parse("Mafa-A2*05:52")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:52") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:52") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_08_02():
    result = parse("Mafa-A4*14:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A4*14:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_12():
    result = parse("Mafa-A3*13:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_13():
    result = parse("Mafa-A3*13:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_01():
    result = parse("Mane-DQB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_02():
    result = parse("Mane-DQB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_01_01():
    result = parse("Mane-DQB1*17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*17:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_03():
    result = parse("Mane-DQB1*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_02():
    result = parse("Mane-DQB1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_04():
    result = parse("Mane-DQB1*18:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_01():
    result = parse("Mane-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_02():
    result = parse("Mane-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_09_01():
    result = parse("Mane-DQB1*06:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*06:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_03():
    result = parse("Mane-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_10():
    result = parse("Mane-DQB1*06:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_04():
    result = parse("Mane-DQB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_05():
    result = parse("Mane-DQB1*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_11():
    result = parse("Mane-DQB1*06:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_08_02():
    result = parse("Mane-DQB1*06:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*06:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_06():
    result = parse("Mane-DQB1*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_07():
    result = parse("Mane-DQB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_08_01():
    result = parse("Mane-DQB1*06:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*06:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_13():
    result = parse("Mane-DQB1*06:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_07_01():
    result = parse("Mafa-DQB1*17:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_09():
    result = parse("Mafa-DQB1*18:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_10():
    result = parse("Mafa-DQB1*18:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_11():
    result = parse("Mafa-DQB1*18:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_12():
    result = parse("Mafa-DQB1*18:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_13():
    result = parse("Mafa-DQB1*18:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_01_02():
    result = parse("Mafa-DQB1*18:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*18:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_06_01():
    result = parse("Mafa-DQB1*17:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_06_02():
    result = parse("Mafa-DQB1*17:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_16_02():
    result = parse("Mafa-DQB1*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W20_01():
    result = parse("Mane-DRB*W20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W27_01():
    result = parse("Mane-DRB*W27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W69_01():
    result = parse("Mane-DRB*W69:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W69:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W69:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W006_01():
    result = parse("Mane-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB5_03_01():
    result = parse("Mane-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB5_01_01():
    result = parse("Mane-DRB5*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB5*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB5*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_01():
    result = parse("Mane-DRB6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_02():
    result = parse("Mane-DRB6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_03():
    result = parse("Mane-DRB6*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_02():
    result = parse("Mane-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_03():
    result = parse("Mane-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W031_01():
    result = parse("Mane-DRB*W031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_04():
    result = parse("Mane-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_05():
    result = parse("Mane-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_06():
    result = parse("Mane-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_08():
    result = parse("Mane-DRB1*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_092_02():
    result = parse("Mamu-A1*092:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*092:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*092:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_03():
    result = parse("Mamu-B*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_003_01_02():
    result = parse("Mamu-B*003:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*003:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*003:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_090_03_01_01():
    result = parse("Mamu-B*090:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*090:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*090:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_090_02_02_01():
    result = parse("Mamu-B*090:02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*090:02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*090:02:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_06():
    result = parse("Mamu-B*045:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*045:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_037_02():
    result = parse("Mamu-B*037:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*037:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*037:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_05():
    result = parse("Mamu-B*069:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*069:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_04_01_01():
    result = parse("Mamu-B*060:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_116_01_01_01():
    result = parse("Mamu-B*116:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*116:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*116:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_048_03_01_01():
    result = parse("Mamu-B*048:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*048:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*048:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W031_02():
    result = parse("Mamu-DRB*W031:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W031:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W031:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_06():
    result = parse("Mamu-DRB*W004:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W004:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_03():
    result = parse("Mamu-DRB*W001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_04():
    result = parse("Mamu-DRB*W001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_05():
    result = parse("Mamu-DRB*W001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_21():
    result = parse("Mamu-DRB6*01:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_11():
    result = parse("Mamu-DRB1*10:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_12():
    result = parse("Mamu-DRB1*10:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W007_03():
    result = parse("Mamu-DRB*W007:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W007:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W007:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W007_04():
    result = parse("Mamu-DRB*W007:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W007:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W007:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W007_02_02():
    result = parse("Mamu-DRB*W007:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W007:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W007:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W040_03():
    result = parse("Mamu-DRB*W040:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W040:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W040:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W020_02_02():
    result = parse("Mamu-DRB*W020:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W020:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W020:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_27():
    result = parse("Mamu-DRB1*03:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W062_01():
    result = parse("Mamu-DRB*W062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_28():
    result = parse("Mamu-DRB1*03:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_29():
    result = parse("Mamu-DRB1*03:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_02_02():
    result = parse("Mamu-DQB1*06:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_18():
    result = parse("Mamu-DQB1*06:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_19():
    result = parse("Mamu-DQB1*06:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_23():
    result = parse("Mamu-DQB1*18:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_24():
    result = parse("Mamu-DQB1*18:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_12():
    result = parse("Mamu-DQB1*17:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_06_02():
    result = parse("Mamu-DQB1*17:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*17:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W69_02():
    result = parse("Mane-DRB*W69:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W69:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W69:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W003_01():
    result = parse("Mane-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_04():
    result = parse("Mane-DRB6*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB5_03_02():
    result = parse("Mane-DRB5*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB5*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB5*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB5_01_02():
    result = parse("Mane-DRB5*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB5*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB5*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB3_04_02():
    result = parse("Mane-DRB3*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB3*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB3*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB3_04_03():
    result = parse("Mane-DRB3*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB3*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB3*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W6_02():
    result = parse("Mane-DRB*W6:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W6:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W6:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W5_01():
    result = parse("Mane-DRB*W5:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W5:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W5:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB5_01_03():
    result = parse("Mane-DRB5*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB5*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB5*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB3_04_01():
    result = parse("Mane-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_14():
    result = parse("Mafa-DQB1*18:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_07_02():
    result = parse("Mafa-DQB1*17:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_15():
    result = parse("Mafa-DQB1*18:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_16():
    result = parse("Mafa-DQB1*18:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_08_01():
    result = parse("Mafa-DQB1*17:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_17():
    result = parse("Mafa-DQB1*18:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_16():
    result = parse("Mafa-DQB1*06:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_17_01():
    result = parse("Mafa-DQB1*06:17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:17:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_18():
    result = parse("Mafa-DQB1*06:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_19():
    result = parse("Mafa-DQB1*06:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_20():
    result = parse("Mafa-DQB1*06:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_21():
    result = parse("Mafa-DQB1*06:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_22():
    result = parse("Mafa-DQB1*06:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_23():
    result = parse("Mafa-DQB1*06:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_25():
    result = parse("Mafa-DQB1*06:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_26():
    result = parse("Mafa-DQB1*06:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_06():
    result = parse("Mamu-A1*007:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*007:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_117_01():
    result = parse("Mamu-A1*117:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*117:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*117:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_074_04():
    result = parse("Mamu-A1*074:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*074:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*074:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_07():
    result = parse("Mamu-A1*028:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*028:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_118_01():
    result = parse("Mamu-A1*118:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*118:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*118:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_04_02():
    result = parse("Mamu-B*007:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*007:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_105_01():
    result = parse("Mamu-B*105:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*105:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*105:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_013_02():
    result = parse("Mamu-B*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_015_03():
    result = parse("Mamu-B*015:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*015:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*015:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_016_02():
    result = parse("Mamu-B*016:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*016:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*016:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_07():
    result = parse("Mamu-B*045:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*045:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_22():
    result = parse("Mamu-I*01:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_029_03():
    result = parse("Mamu-B*029:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*029:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*029:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_102_01():
    result = parse("Mamu-B*102:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*102:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*102:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_156_01():
    result = parse("Mamu-B*156:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*156:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*156:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_036_03():
    result = parse("Mamu-B*036:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*036:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*036:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_162_01():
    result = parse("Mamu-B*162:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*162:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*162:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_06():
    result = parse("Mamu-B*044:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*044:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_18_01_01():
    result = parse("Mamu-B*046:18:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:18:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:18:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_19():
    result = parse("Mamu-B*046:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_06():
    result = parse("Mamu-B*051:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_07():
    result = parse("Mamu-B*051:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_054_02():
    result = parse("Mamu-B*054:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*054:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*054:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_03_01_01():
    result = parse("Mamu-B*056:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*056:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_04_01_01():
    result = parse("Mamu-B*056:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*056:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_142_01():
    result = parse("Mamu-B*142:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*142:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*142:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_061_04():
    result = parse("Mamu-B*061:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*061:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*061:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_02_02():
    result = parse("Mamu-B*063:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*063:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_066_02():
    result = parse("Mamu-B*066:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*066:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*066:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_07():
    result = parse("Mamu-B*068:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*068:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_06():
    result = parse("Mamu-B*069:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*069:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_071_02():
    result = parse("Mamu-B*071:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*071:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*071:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_163_01():
    result = parse("Mamu-B*163:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*163:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*163:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_073_02():
    result = parse("Mamu-B*073:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*073:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*073:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_05():
    result = parse("Mamu-B*082:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_083_02():
    result = parse("Mamu-B*083:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*083:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*083:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_085_03():
    result = parse("Mamu-B*085:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*085:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*085:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_124_01_01_01():
    result = parse("Mamu-B*124:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*124:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*124:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_092_02_01_01():
    result = parse("Mamu-B*092:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*092:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*092:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_092_03():
    result = parse("Mamu-B*092:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*092:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*092:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_04():
    result = parse("Mamu-B*063:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*063:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_05():
    result = parse("Mamu-B*063:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*063:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_23():
    result = parse("Mamu-I*01:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_09():
    result = parse("Mamu-I*01:06:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:09") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_10():
    result = parse("Mamu-I*01:06:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:10") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_24():
    result = parse("Mamu-I*01:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_04_03_01():
    result = parse("Mamu-DPA1*04:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*04:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*04:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_25():
    result = parse("Mamu-DQB1*18:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_04_03():
    result = parse("Mamu-DRA*01:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_03_02():
    result = parse("Mamu-DRB1*03:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*03:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_03_02():
    result = parse("Mamu-DRB*W003:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W003:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_18():
    result = parse("Mamu-E*02:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_12_02():
    result = parse("Mamu-E*02:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-E*02:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_147_01():
    result = parse("Mamu-B*147:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*147:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*147:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_03_03():
    result = parse("Mamu-DPB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_16_01():
    result = parse("Mamu-DPB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_17_01():
    result = parse("Mamu-DPB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_015_01():
    result = parse("Mamu-A1*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_091_03():
    result = parse("Mamu-B*091:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*091:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*091:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_09():
    result = parse("Mamu-DPA1*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_25():
    result = parse("Mamu-I*01:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_11():
    result = parse("Mamu-A4*14:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_12():
    result = parse("Mamu-A4*14:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_085_03():
    result = parse("Mamu-A1*085:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*085:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*085:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_085_04():
    result = parse("Mamu-A1*085:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*085:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*085:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_085_05():
    result = parse("Mamu-A1*085:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*085:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*085:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_022_05():
    result = parse("Mamu-A1*022:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*022:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*022:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_006_03():
    result = parse("Mamu-A1*006:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*006:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*006:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_022_04():
    result = parse("Mamu-A1*022:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*022:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*022:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_08():
    result = parse("Mamu-A1*019:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_090_01():
    result = parse("Mamu-A1*090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_11():
    result = parse("Mamu-A1*003:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_12():
    result = parse("Mamu-A1*003:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_13():
    result = parse("Mamu-A1*003:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_119_01():
    result = parse("Mamu-A1*119:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*119:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*119:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_074_06():
    result = parse("Mamu-A1*074:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*074:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*074:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_074_05():
    result = parse("Mamu-A1*074:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*074:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*074:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_057_04():
    result = parse("Mamu-A1*057:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*057:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*057:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_057_05():
    result = parse("Mamu-A1*057:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*057:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*057:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_091_03():
    result = parse("Mamu-A1*091:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*091:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*091:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_049_05():
    result = parse("Mamu-A1*049:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*049:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*049:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_05():
    result = parse("Mamu-A1*004:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*004:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_04():
    result = parse("Mamu-A1*004:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*004:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_120_01():
    result = parse("Mamu-A1*120:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*120:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*120:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_03():
    result = parse("Mamu-A1*043:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*043:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_04():
    result = parse("Mamu-A1*043:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*043:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_121_01():
    result = parse("Mamu-A1*121:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*121:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*121:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_06():
    result = parse("Mamu-A1*043:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*043:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_02_01():
    result = parse("Mamu-A1*043:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*043:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_05():
    result = parse("Mamu-A1*043:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*043:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_06():
    result = parse("Mamu-A1*032:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*032:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_05():
    result = parse("Mamu-A1*032:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*032:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_065_02():
    result = parse("Mamu-A1*065:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*065:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*065:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_24_04():
    result = parse("Mamu-A2*24:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*24:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*24:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_48():
    result = parse("Mamu-A2*05:48")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:48") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:48") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_49():
    result = parse("Mamu-A2*05:49")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:49") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:49") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_01_01():
    result = parse("Mafa-DOB*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_01_02():
    result = parse("Mafa-DOB*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_05_01_01():
    result = parse("Mafa-B*050:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*050:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_039_01_01_01():
    result = parse("Mafa-B*039:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*039:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*039:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_01_03():
    result = parse("Mafa-DOB*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_01_04():
    result = parse("Mafa-DOB*01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_04_01():
    result = parse("Mafa-DOB*01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_02_02():
    result = parse("Mafa-DOB*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_03_01():
    result = parse("Mafa-DOB*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_03_02():
    result = parse("Mafa-DOB*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_04_02():
    result = parse("Mafa-DOB*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_01_05():
    result = parse("Mafa-DOB*01:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_02_01():
    result = parse("Mafa-DOB*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_02_03():
    result = parse("Mafa-DOB*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_03_03():
    result = parse("Mafa-DOB*01:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_05():
    result = parse("Mafa-DOB*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DOB*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_03_04():
    result = parse("Mafa-DOB*01:03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:03:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOB_01_02_04():
    result = parse("Mafa-DOB*01:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOB*01:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOB*01:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_065_02_01_01():
    result = parse("Mafa-B*065:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*065:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*065:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_05_02():
    result = parse("Mafa-DPB1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_002_04_01_01():
    result = parse("Mafa-B*002:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*002:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*002:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_081_01():
    result = parse("Mafa-B*081:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*081:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*081:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_01_04():
    result = parse("Mafa-B*007:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*007:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_168_01():
    result = parse("Mafa-B*168:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*168:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*168:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_093_02_01_01():
    result = parse("Mafa-B*093:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*093:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*093:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_169_01():
    result = parse("Mafa-B*169:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*169:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*169:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_04_01():
    result = parse("Mafa-B*079:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*079:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_01_05_01():
    result = parse("Mafa-B*007:01:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:01:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*007:01:05:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_170_01():
    result = parse("Mafa-B*170:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*170:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*170:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_05():
    result = parse("Mafa-B*048:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*048:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_08_02():
    result = parse("Mafa-DQB1*17:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_082_01_01_01():
    result = parse("Mafa-B*082:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*082:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*082:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_13():
    result = parse("Mafa-DRB1*10:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_01_02():
    result = parse("Mamu-B*019:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*019:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_07():
    result = parse("Mafa-DPB1*15:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_145_01():
    result = parse("Mamu-B*145:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*145:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*145:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_149_01():
    result = parse("Mamu-B*149:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*149:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*149:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_07():
    result = parse("Mamu-B*044:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*044:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_07():
    result = parse("Mafa-DPB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_086_04():
    result = parse("Mamu-B*086:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*086:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*086:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_09_03():
    result = parse("Mafa-DPB1*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_086_05_01_01():
    result = parse("Mamu-B*086:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*086:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*086:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_034_02():
    result = parse("Mamu-B*034:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*034:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*034:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_034_03():
    result = parse("Mamu-B*034:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*034:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*034:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_027_04_01_01():
    result = parse("Mamu-B*027:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*027:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*027:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_025_01_02():
    result = parse("Mamu-B*025:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*025:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*025:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_137_01_01():
    result = parse("Mamu-B*137:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*137:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*137:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_106_01():
    result = parse("Mamu-B*106:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*106:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*106:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_035_02():
    result = parse("Mamu-B*035:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*035:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*035:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_07_01_01():
    result = parse("Mamu-B*069:07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*069:07:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_075_03():
    result = parse("Mamu-B*075:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*075:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*075:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_06():
    result = parse("Mamu-A1*004:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*004:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_07():
    result = parse("Mamu-A1*032:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*032:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_040_04():
    result = parse("Mamu-A1*040:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*040:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*040:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_122_01():
    result = parse("Mamu-A1*122:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*122:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*122:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_123_01():
    result = parse("Mamu-A1*123:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*123:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*123:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_026_02_02():
    result = parse("Mamu-A1*026:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*026:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*026:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_02_02():
    result = parse("Mamu-A1*043:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*043:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_16_03():
    result = parse("Mafa-DQB1*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_27():
    result = parse("Mafa-DQB1*06:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_28():
    result = parse("Mafa-DQB1*06:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_29():
    result = parse("Mafa-DQB1*06:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_30():
    result = parse("Mafa-DQB1*06:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_18():
    result = parse("Mafa-DQB1*18:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_19():
    result = parse("Mafa-DQB1*18:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_002_01_02():
    result = parse("Mafa-A1*002:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*002:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*002:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_09_01_01():
    result = parse("Mafa-DPA1*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_06_01():
    result = parse("Mafa-DPA1*02:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_07():
    result = parse("Mafa-DPA1*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_06_01_01():
    result = parse("Mafa-DPA1*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_03():
    result = parse("Mafa-DPA1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_08_01():
    result = parse("Mafa-DPA1*02:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_09_01():
    result = parse("Mafa-DPA1*02:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_10():
    result = parse("Mafa-DPA1*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_08_01():
    result = parse("Mafa-DPA1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_11():
    result = parse("Mafa-DPA1*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_12():
    result = parse("Mafa-DPA1*02:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_13_01():
    result = parse("Mafa-DPA1*02:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_14_01():
    result = parse("Mafa-DPA1*02:14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:14:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_04_02():
    result = parse("Mafa-DPA1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_08_02():
    result = parse("Mafa-DPA1*02:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_15_01():
    result = parse("Mafa-DPA1*02:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_16():
    result = parse("Mafa-DPA1*02:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_17():
    result = parse("Mafa-DPA1*02:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_18():
    result = parse("Mafa-DPA1*02:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_01_02():
    result = parse("Mafa-DPB1*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_04_01():
    result = parse("Mafa-DPB1*03:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*03:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_08():
    result = parse("Mafa-DPB1*06:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*06:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_032_05():
    result = parse("Mafa-A1*032:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*032:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*032:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_023_01():
    result = parse("Mafa-A1*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_068_02():
    result = parse("Mafa-A1*068:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*068:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*068:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_056_02():
    result = parse("Mafa-A1*056:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*056:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*056:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_062_05():
    result = parse("Mafa-A1*062:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*062:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*062:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_03():
    result = parse("Mafa-AG*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_01_03():
    result = parse("Mafa-B*007:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*007:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_002_03():
    result = parse("Mafa-B*002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_04_01_01():
    result = parse("Mafa-B*060:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_074_02_01_01():
    result = parse("Mafa-B*074:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*074:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*074:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_02_02():
    result = parse("Mafa-DPB1*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_05_01():
    result = parse("Mamu-AG*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-AG*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_115_01_01_01():
    result = parse("Mamu-B*115:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*115:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*115:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B20_01_01():
    result = parse("Mamu-B20*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B20*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B20*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B20_01_02():
    result = parse("Mamu-B20*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B20*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B20*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_01_02():
    result = parse("Mane-B*079:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_10():
    result = parse("Mafa-B*030:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_090_02():
    result = parse("Mafa-B*090:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*090:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*090:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_153_04():
    result = parse("Mafa-B*153:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*153:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*153:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_53():
    result = parse("Mafa-A2*05:53")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:53") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:53") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_54():
    result = parse("Mafa-A2*05:54")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:54") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:54") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_083_02():
    result = parse("Mafa-B*083:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*083:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*083:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_03_02():
    result = parse("Mafa-A3*13:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A3*13:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_11():
    result = parse("Mafa-B*030:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_099_02():
    result = parse("Mafa-B*099:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*099:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*099:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_01_01_01():
    result = parse("Mafa-AG*04:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-AG*04:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_02_01():
    result = parse("Mafa-AG*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_02():
    result = parse("Mafa-B*088:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*088:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_02_01_01():
    result = parse("Mafa-B*079:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*079:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B21_01_01_01_01N():
    result = parse("Mafa-B21*01:01:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B21*01:01:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B21*01:01:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B21_01_02():
    result = parse("Mafa-B21*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B21*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B21*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_04():
    result = parse("Mafa-A1*022:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_02_01_01():
    result = parse("Mafa-AG*04:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-AG*04:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_05_01():
    result = parse("Mafa-DQA1*05:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_04_01():
    result = parse("Mafa-DQA1*26:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*26:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_06_01():
    result = parse("Mafa-DQA1*05:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_03_02():
    result = parse("Mafa-DQA1*05:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_04():
    result = parse("Mafa-DQA1*24:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_05():
    result = parse("Mafa-DQA1*24:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_10():
    result = parse("Mafa-DQA1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_05():
    result = parse("Mafa-DQA1*26:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_11():
    result = parse("Mafa-DQA1*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_20():
    result = parse("Mafa-DQB1*18:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_31():
    result = parse("Mafa-DQB1*06:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_15_04():
    result = parse("Mafa-DQB1*15:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*15:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*15:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_32():
    result = parse("Mafa-DQB1*06:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_02():
    result = parse("Mafa-B*011:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*011:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_043_01_01_01():
    result = parse("Mafa-B*043:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*043:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*043:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_057_03_01_01():
    result = parse("Mafa-B*057:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*057:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*057:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_076_04():
    result = parse("Mafa-B*076:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*076:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*076:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_061_02_01_01():
    result = parse("Mafa-B*061:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*061:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*061:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_138_02_01_01():
    result = parse("Mafa-B*138:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*138:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*138:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_124_01_02():
    result = parse("Mafa-B*124:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*124:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*124:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_04_01_01():
    result = parse("Mafa-B*068:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*068:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_032_01():
    result = parse("Mafa-B*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_155_02_01_01():
    result = parse("Mafa-B*155:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*155:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*155:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_04():
    result = parse("Mafa-B*137:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*137:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_091_01():
    result = parse("Mafa-B*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_09():
    result = parse("Mamu-A1*018:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_02_01_04():
    result = parse("Mafa-DRA*02:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*02:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*02:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_06():
    result = parse("Mafa-DRA*01:03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_29():
    result = parse("Mafa-DRB1*03:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_06_02():
    result = parse("Mafa-DRB1*03:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W073_01():
    result = parse("Mafa-DRB*W073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_06():
    result = parse("Mamu-B*030:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*030:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_02_02():
    result = parse("Mamu-DPA1*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_04_04():
    result = parse("Mamu-DPA1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W033_01_02():
    result = parse("Mafa-DRB*W033:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W033:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W033:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_12():
    result = parse("Mafa-DRB1*10:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_07_03_01():
    result = parse("Mamu-DPB1*07:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*07:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPB1*07:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_02_02():
    result = parse("Mamu-DPB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_06():
    result = parse("Mamu-DQA1*26:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_02_02():
    result = parse("Mafa-DRB1*04:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_08():
    result = parse("Mafa-DRB3*04:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_07():
    result = parse("Mafa-DRB*W007:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_14_01():
    result = parse("Mafa-DRB1*04:14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:14:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W005_02():
    result = parse("Mafa-DRB*W005:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W005:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W005:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_30():
    result = parse("Mafa-DRB1*03:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_11():
    result = parse("Mafa-DRB*W003:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_06():
    result = parse("Mafa-DQA1*26:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_22():
    result = parse("Mafa-I*01:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_04_01_01():
    result = parse("Mafa-B*098:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*098:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_02():
    result = parse("Mafa-B*072:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_03_01_01():
    result = parse("Mafa-B*079:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*079:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_04():
    result = parse("Mafa-B*044:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_05_01_01():
    result = parse("Mafa-B*060:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_06():
    result = parse("Mafa-B*060:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_02_01_01():
    result = parse("Mafa-B*070:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*070:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_04_01_01():
    result = parse("Mafa-B*051:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*051:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_31_01_01():
    result = parse("Mafa-B*060:31:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:31:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:31:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_04_01_01():
    result = parse("Mafa-B*109:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*109:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_165_01():
    result = parse("Mafa-B*165:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*165:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*165:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_05_01_01():
    result = parse("Mafa-B*098:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*098:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_03_01_01():
    result = parse("Mafa-B*045:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*045:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B11L_01_04():
    result = parse("Mafa-B11L*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B11L*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B11L*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_164_02():
    result = parse("Mafa-B*164:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*164:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*164:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B11L_01_05():
    result = parse("Mafa-B11L*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B11L*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B11L*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_166_01():
    result = parse("Mafa-B*166:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*166:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*166:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_04_01_01():
    result = parse("Mafa-B*050:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*050:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_167_01_01_01N():
    result = parse("Mafa-B*167:01:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*167:01:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*167:01:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_12():
    result = parse("Mafa-DQA1*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_06_01():
    result = parse("Mafa-AG*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_03_01_01():
    result = parse("Mafa-AG*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_028_02():
    result = parse("Mafa-A1*028:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*028:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*028:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_06():
    result = parse("Mafa-A1*022:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_04_02():
    result = parse("Mafa-A1*065:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*065:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_12_02():
    result = parse("Mafa-DRB1*03:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_03():
    result = parse("Mafa-A1*003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_09():
    result = parse("Mafa-A4*14:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_01_01():
    result = parse("Mafa-A2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_002_03():
    result = parse("Mafa-A1*002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_04_01_01():
    result = parse("Mafa-A1*003:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*003:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_057_01():
    result = parse("Mafa-A1*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_102_02():
    result = parse("Mafa-A1*102:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*102:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*102:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_05():
    result = parse("Mafa-A2*24:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_05():
    result = parse("Mafa-A1*003:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*003:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_066_06_01_01():
    result = parse("Mafa-A1*066:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*066:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*066:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_028_01():
    result = parse("Mafa-A1*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_05():
    result = parse("Mafa-A1*022:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_13():
    result = parse("Mafa-B*030:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W066_02():
    result = parse("Mafa-DRB*W066:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W066:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W066:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W005_03():
    result = parse("Mafa-DRB*W005:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W005:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W005:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_22():
    result = parse("Mafa-DQB1*18:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_23():
    result = parse("Mafa-DQB1*18:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_24():
    result = parse("Mafa-DQB1*18:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_09():
    result = parse("Mafa-DQB1*17:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_12():
    result = parse("Mafa-DRB*W003:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_13():
    result = parse("Mafa-DRB*W003:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_13_01():
    result = parse("Mafa-DRB1*04:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_14():
    result = parse("Mafa-DRB*W003:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W037_03():
    result = parse("Mafa-DRB*W037:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W037:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W037:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_15():
    result = parse("Mafa-DRB*W003:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W026_04():
    result = parse("Mafa-DRB*W026:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W026:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W026:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_03_01_01():
    result = parse("Mafa-B*069:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*069:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_10_01_01():
    result = parse("Mafa-B*013:10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*013:10:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_04():
    result = parse("Mafa-B*069:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_05_01_01():
    result = parse("Mafa-B*007:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*007:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_034_03():
    result = parse("Mafa-B*034:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*034:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*034:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_05():
    result = parse("Mane-DQB1*18:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_06():
    result = parse("Mane-DQB1*18:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_07():
    result = parse("Mane-DQB1*18:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_04():
    result = parse("Mane-DQB1*17:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*17:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_05():
    result = parse("Mane-DQB1*17:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*17:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_16_01():
    result = parse("Mane-DQB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_08():
    result = parse("Mane-DQB1*18:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_33():
    result = parse("Mafa-DQB1*06:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_34():
    result = parse("Mafa-DQB1*06:34")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:34") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:34") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_05():
    result = parse("Mane-DRB6*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_09():
    result = parse("Mane-DRB1*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_13():
    result = parse("Mane-DRB1*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_14():
    result = parse("Mane-DRB1*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_10():
    result = parse("Mane-DRB1*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_12():
    result = parse("Mane-DRB1*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W005_02():
    result = parse("Mane-DRB*W005:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W005:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W005:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W25_01():
    result = parse("Mane-DRB*W25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_06():
    result = parse("Mane-DRB6*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_07():
    result = parse("Mane-DRB6*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_11():
    result = parse("Mane-DRB1*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_15():
    result = parse("Mane-DRB1*03:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_08():
    result = parse("Mane-DRB6*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_09():
    result = parse("Mane-DRB6*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W63_01():
    result = parse("Mane-DRB*W63:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W63:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W63:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W063_02():
    result = parse("Mane-DRB*W063:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W063:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W063:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_07_01():
    result = parse("Mane-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_10():
    result = parse("Mane-DRB6*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_10_01():
    result = parse("Mane-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_10_03():
    result = parse("Mane-DRB1*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB3_04_04():
    result = parse("Mane-DRB3*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB3*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB3*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB5_03_03():
    result = parse("Mane-DRB5*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB5*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB5*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W2_03():
    result = parse("Mane-DRB*W2:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W2:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W2:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W2_04():
    result = parse("Mane-DRB*W2:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W2:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W2:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_05():
    result = parse("Mane-DRB*W002:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W2_06():
    result = parse("Mane-DRB*W2:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W2:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W2:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W2_07():
    result = parse("Mane-DRB*W2:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W2:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W2:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_08():
    result = parse("Mane-DRB*W002:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W065_01():
    result = parse("Mane-DRB*W065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W026_01():
    result = parse("Mane-DRB*W026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W25_02():
    result = parse("Mane-DRB*W25:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W25:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W25:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W64_01():
    result = parse("Mane-DRB*W64:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W64:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W64:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_11():
    result = parse("Mane-DRB6*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB6_01_12():
    result = parse("Mane-DRB6*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB6*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB6*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W1_03():
    result = parse("Mane-DRB*W1:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W1:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W1:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_04():
    result = parse("Mane-DRB*W001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W1_05():
    result = parse("Mane-DRB*W1:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W1:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W1:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W1_06():
    result = parse("Mane-DRB*W1:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W1:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W1:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W1_07():
    result = parse("Mane-DRB*W1:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W1:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W1:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W21_01():
    result = parse("Mane-DRB*W21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_10_04():
    result = parse("Mane-DRB1*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_30():
    result = parse("Mafa-DRB6*01:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_09():
    result = parse("Mafa-DRB3*04:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_31():
    result = parse("Mafa-DRB6*01:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_32():
    result = parse("Mafa-DRB6*01:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_06():
    result = parse("Mafa-DRB*W004:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_14():
    result = parse("Mafa-DRB5*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_31():
    result = parse("Mafa-DRB1*03:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_32():
    result = parse("Mafa-DRB1*03:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_33():
    result = parse("Mafa-DRB1*03:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_16():
    result = parse("Mafa-DRB*W003:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB6_01_33():
    result = parse("Mafa-DRB6*01:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB6*01:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB6*01:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_13():
    result = parse("Mafa-DRB5*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_01_03():
    result = parse("Mafa-DRB5*03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB5*03:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B20_01_03():
    result = parse("Mamu-B20*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B20*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B20*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B20_01_04_01_01():
    result = parse("Mamu-B20*01:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B20*01:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B20*01:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_070_03_01_01():
    result = parse("Mamu-B*070:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*070:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*070:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_02_01():
    result = parse("Mane-B*079:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_01():
    result = parse("Mane-B*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_054_01():
    result = parse("Mane-B*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_01_03():
    result = parse("Mane-B*079:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_03():
    result = parse("Mane-B*079:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*079:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_01_04():
    result = parse("Mane-B*079:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_04():
    result = parse("Mane-B*079:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*079:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_05():
    result = parse("Mane-B*079:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*079:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_06():
    result = parse("Mane-B*079:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*079:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_01_05():
    result = parse("Mane-B*079:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_03_03():
    result = parse("Mane-A2*05:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_027_01():
    result = parse("Mane-B*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_027_02():
    result = parse("Mane-B*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B20_01_01():
    result = parse("Mane-B20*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B20*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B20*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_04():
    result = parse("Mane-B*068:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_115_02():
    result = parse("Mane-B*115:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*115:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*115:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_01_06():
    result = parse("Mane-B*079:01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:01:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_01_07():
    result = parse("Mane-B*079:01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:01:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_01():
    result = parse("Mafa-B*115:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*115:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B20_01_01_01_01N():
    result = parse("Mafa-B20*01:01:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B20*01:01:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B20*01:01:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_149_02_01_01():
    result = parse("Mafa-B*149:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*149:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*149:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B20_01_02():
    result = parse("Mafa-B20*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B20*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B20*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_037_04():
    result = parse("Mafa-B*037:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*037:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*037:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_04_01_01():
    result = parse("Mafa-A6*01:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A6*01:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_075_02_02():
    result = parse("Mafa-B*075:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*075:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*075:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_05():
    result = parse("Mafa-B*051:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W002_04():
    result = parse("Mamu-DRB*W002:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W002:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W002:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_12():
    result = parse("Mamu-DRB*W006:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_09_02():
    result = parse("Mamu-DRB*W006:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W006:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_10():
    result = parse("Mamu-DRB5*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB5*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_13():
    result = parse("Mamu-DRB1*10:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_26():
    result = parse("Mamu-DRB6*01:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W069_01():
    result = parse("Mamu-DRB*W069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_27():
    result = parse("Mamu-DRB6*01:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_30():
    result = parse("Mamu-DRB1*03:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_31N():
    result = parse("Mamu-DRB1*03:31N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:31N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:31N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W067_01():
    result = parse("Mane-DRB*W067:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W067:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W067:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W4_01():
    result = parse("Mane-DRB*W4:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W4:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W4:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_08():
    result = parse("Mane-DRB*W001:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_09():
    result = parse("Mane-DRB*W001:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W1_10():
    result = parse("Mane-DRB*W1:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W1:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W1:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_11():
    result = parse("Mane-DRB*W001:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_01():
    result = parse("Mane-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W27_02():
    result = parse("Mane-DRB*W27:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W27:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W27:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W74_01_01():
    result = parse("Mane-DRB*W74:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W74:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB*W74:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W021_02():
    result = parse("Mane-DRB*W021:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W021:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W021:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W075_01():
    result = parse("Mafa-DRB*W075:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W075:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W075:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_02_02():
    result = parse("Mafa-DRB3*04:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB3*04:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_02_01_01():
    result = parse("Mafa-B*101:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*101:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_017_02_01_01():
    result = parse("Mafa-B*017:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*017:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*017:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_008_01():
    result = parse("Mafa-B*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_02():
    result = parse("Mafa-B*036:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*036:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_037_05():
    result = parse("Mafa-B*037:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*037:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*037:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_082_06():
    result = parse("Mafa-B*082:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*082:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*082:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_03():
    result = parse("Mafa-B*015:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_105_01_01_01():
    result = parse("Mafa-B*105:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*105:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*105:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_104_01_03():
    result = parse("Mafa-B*104:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*104:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*104:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_04():
    result = parse("Mafa-A2*24:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_02_02():
    result = parse("Mafa-A1*001:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*001:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_085_02():
    result = parse("Mafa-A1*085:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*085:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*085:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_124_01():
    result = parse("Mafa-A1*124:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*124:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*124:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_15():
    result = parse("Mafa-A3*13:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_05():
    result = parse("Mafa-A6*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_16():
    result = parse("Mafa-A3*13:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_20():
    result = parse("Mamu-DQB1*06:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_21():
    result = parse("Mamu-DQB1*06:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_22():
    result = parse("Mamu-DQB1*06:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_074_01_02():
    result = parse("Mafa-A1*074:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*074:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*074:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_058_02_01_01():
    result = parse("Mafa-A1*058:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*058:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*058:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_032_03_02():
    result = parse("Mafa-A1*032:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*032:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*032:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_015_03():
    result = parse("Mafa-A1*015:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*015:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*015:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_10():
    result = parse("Mafa-A4*14:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_01_02_01():
    result = parse("Mafa-A1*001:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*001:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_040_01_02():
    result = parse("Mafa-A1*040:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*040:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*040:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_042_02_01_01():
    result = parse("Mafa-A1*042:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*042:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*042:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_02_03():
    result = parse("Mamu-DQB1*06:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_15_05():
    result = parse("Mamu-DQB1*15:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*15:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*15:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_13():
    result = parse("Mamu-DQB1*17:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_26N():
    result = parse("Mamu-DQB1*18:26N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:26N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:26N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_02():
    result = parse("Mafa-B*060:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_03_02():
    result = parse("Mane-B17*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B17_01_01_03():
    result = parse("Mane-B17*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B17*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B17*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_061_05():
    result = parse("Mamu-B*061:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*061:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*061:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B11L_01_12():
    result = parse("Mamu-B11L*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B11L*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B11L*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_03_01_01():
    result = parse("Mamu-B*082:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*082:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_04():
    result = parse("Mamu-B*082:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_05():
    result = parse("Mamu-B*007:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*007:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_095_02_01_01():
    result = parse("Mamu-B*095:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*095:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*095:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_090_02_01():
    result = parse("Mamu-B*090:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*090:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*090:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_018_02_01_01():
    result = parse("Mamu-B*018:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*018:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*018:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_089_02():
    result = parse("Mamu-B*089:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*089:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*089:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_057_03():
    result = parse("Mamu-B*057:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*057:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*057:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_089_01_02():
    result = parse("Mamu-B*089:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*089:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*089:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_02():
    result = parse("Mane-A4*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_115_01():
    result = parse("Mane-A1*115:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*115:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*115:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_01_02():
    result = parse("Mane-A3*13:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A3*13:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_14_01():
    result = parse("Mane-A3*13:14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A3*13:14:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_116_01():
    result = parse("Mane-A1*116:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*116:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*116:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_04():
    result = parse("Mane-A3*13:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_05():
    result = parse("Mane-A3*13:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_02_01():
    result = parse("Mane-A4*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_06():
    result = parse("Mane-A3*13:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_07():
    result = parse("Mane-A3*13:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_08():
    result = parse("Mane-A3*13:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_01_01_01():
    result = parse("Mane-A4*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A4*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_04():
    result = parse("Mane-A4*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_132_01():
    result = parse("Mane-B*132:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*132:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*132:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_133_01():
    result = parse("Mane-B*133:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*133:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*133:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_105_02():
    result = parse("Mane-B*105:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*105:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*105:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_37():
    result = parse("Mafa-A2*05:37")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:37") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:37") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_38_01():
    result = parse("Mafa-A2*05:38:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:38:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:38:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_39():
    result = parse("Mafa-A2*05:39")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:39") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:39") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_01_02_01():
    result = parse("Mafa-B*036:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*036:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_151_02_02():
    result = parse("Mafa-B*151:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*151:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*151:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_013_04_01_01():
    result = parse("Mamu-B*013:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*013:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*013:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_15_03():
    result = parse("Ovar-DRB1*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_08_05():
    result = parse("Ovar-DRB1*08:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*08:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*08:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_05():
    result = parse("Ovar-DRB1*16:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_22_02():
    result = parse("Ovar-DRB1*22:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*22:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*22:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_24_01():
    result = parse("Ovar-DRB1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_05_03():
    result = parse("Ovar-DRB1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_G_01_01():
    result = parse("Patr-G*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-G*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-G*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W033_02():
    result = parse("Mafa-DRB*W033:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W033:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W033:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W013_03():
    result = parse("Aovo-DRB*W013:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W013:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W013:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_01_01():
    result = parse("Chae-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_02_01():
    result = parse("Chae-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_02_01():
    result = parse("Chae-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_01_01():
    result = parse("Chae-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_174_02N():
    result = parse("Mafa-B*174:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*174:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*174:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_01():
    result = parse("Patr-AL*01:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_02():
    result = parse("Patr-AL*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-AL*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_02():
    result = parse("Patr-AL*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-AL*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_03():
    result = parse("Patr-AL*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-AL*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_03():
    result = parse("Patr-AL*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-AL*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_05_01():
    result = parse("Popy-A*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_03_02():
    result = parse("Popy-A*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_04_01():
    result = parse("Popy-A*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_04_02():
    result = parse("Popy-A*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_06():
    result = parse("Mafa-DRB*W006:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_07():
    result = parse("Mafa-DRB*W001:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_08_02():
    result = parse("Mafa-DRB1*03:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W036_04():
    result = parse("Mafa-DRB*W036:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W036:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W036:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_15():
    result = parse("Mafa-DRB5*03:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W025_05():
    result = parse("Mafa-DRB*W025:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W025:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W025:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_08():
    result = parse("Mafa-DRB*W001:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_07_01():
    result = parse("Mafa-DRB*W006:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W006:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W025_06():
    result = parse("Mafa-DRB*W025:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W025:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W025:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W049_01_02():
    result = parse("Mafa-DRB*W049:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W049:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W049:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB5_03_16():
    result = parse("Mafa-DRB5*03:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB5*03:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB5*03:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_07():
    result = parse("Mafa-DRA*01:03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_09():
    result = parse("Mafa-DRA*01:01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:01:09") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_20():
    result = parse("Mafa-DRA*01:02:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:20") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_10_01():
    result = parse("Mafa-DRA*01:10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:10:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_10_01():
    result = parse("Mafa-DPA1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_19():
    result = parse("Mafa-DPA1*02:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_20():
    result = parse("Mafa-DPA1*02:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_21_01():
    result = parse("Mafa-DPA1*02:21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:21:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_13_02():
    result = parse("Mafa-DPA1*02:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_06_02():
    result = parse("Mafa-DPA1*02:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_22():
    result = parse("Mafa-DPA1*02:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_07():
    result = parse("Mafa-DPB1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_05():
    result = parse("Mafa-DPB1*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_07_01():
    result = parse("Mafa-DQA1*05:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_01_02():
    result = parse("Mafa-DQA1*26:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*26:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_13():
    result = parse("Mafa-DQA1*01:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_14_01():
    result = parse("Mafa-DQA1*01:14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:14:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_08():
    result = parse("Mafa-DQA1*05:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_09():
    result = parse("Mafa-DQA1*05:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_08_02():
    result = parse("Mafa-DQA1*01:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_02_02():
    result = parse("Mafa-DQB1*17:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*17:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_17_02():
    result = parse("Mafa-DRB1*03:17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:17:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_084_01_02():
    result = parse("Mane-A1*084:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*084:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*084:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_006_03():
    result = parse("Mane-A1*006:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*006:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*006:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_084_03():
    result = parse("Mane-A1*084:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*084:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*084:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A6_01_02():
    result = parse("Mane-A6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_004_01():
    result = parse("Mane-B*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_017_04():
    result = parse("Mane-B*017:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*017:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*017:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_045_02():
    result = parse("Mane-B*045:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*045:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*045:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_046_01_01():
    result = parse("Mane-B*046:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*046:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*046:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_046_01_02():
    result = parse("Mane-B*046:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*046:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*046:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_03():
    result = parse("Mane-B*051:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*051:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_04_01():
    result = parse("Mane-B*051:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*051:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_04_02():
    result = parse("Mane-B*051:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*051:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_02_02():
    result = parse("Mane-B*051:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*051:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_04_03():
    result = parse("Mane-B*051:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*051:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_054_02():
    result = parse("Mane-B*054:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*054:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*054:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_02():
    result = parse("Mane-B*060:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*060:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_02_02():
    result = parse("Mane-B*068:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*068:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_05():
    result = parse("Mane-B*068:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_06():
    result = parse("Mane-B*068:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_02_03():
    result = parse("Mane-B*068:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*068:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_04_01():
    result = parse("Mane-B*072:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_02_02():
    result = parse("Mane-B*072:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_05():
    result = parse("Mane-B*072:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*072:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_171_01():
    result = parse("Mane-B*171:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*171:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*171:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_078_03():
    result = parse("Mane-B*078:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*078:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*078:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_07():
    result = parse("Mane-B*079:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*079:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_02_02():
    result = parse("Mane-B*079:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*079:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_088_03():
    result = parse("Mane-B*088:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*088:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*088:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_088_04():
    result = parse("Mane-B*088:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*088:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*088:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_089_03():
    result = parse("Mane-B*089:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*089:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*089:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_089_04():
    result = parse("Mane-B*089:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*089:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*089:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_091_02():
    result = parse("Mane-B*091:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*091:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*091:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_112_02():
    result = parse("Mane-B*112:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*112:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*112:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_162_01():
    result = parse("Mane-B*162:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*162:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*162:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_084_04():
    result = parse("Mane-A1*084:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*084:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*084:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_027_03_01():
    result = parse("Mane-B*027:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*027:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*027:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_027_03_02():
    result = parse("Mane-B*027:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*027:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*027:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_039_02():
    result = parse("Mane-B*039:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*039:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*039:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_04_02():
    result = parse("Mane-B*072:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_108_02():
    result = parse("Mane-B*108:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*108:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*108:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_123_01_02():
    result = parse("Mane-B*123:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*123:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*123:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A5_30_01():
    result = parse("Mane-A5*30:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A5*30:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A5*30:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_144_01():
    result = parse("Mane-B*144:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*144:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*144:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_04_01():
    result = parse("Chsa-A*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_02_01():
    result = parse("Chsa-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_03_01():
    result = parse("Chsa-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_03_02():
    result = parse("Chsa-A*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_01_01():
    result = parse("Chsa-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_05_01():
    result = parse("Chsa-A*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_06_01():
    result = parse("Chsa-A*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_08_01():
    result = parse("Chsa-B*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_08_02():
    result = parse("Chsa-B*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_11_01():
    result = parse("Chsa-B*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_11_02():
    result = parse("Chsa-B*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_14_01():
    result = parse("Chsa-B*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_02_01():
    result = parse("Chsa-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_04_01():
    result = parse("Chsa-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_01():
    result = parse("Chsa-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_02_01():
    result = parse("Chsa-B*15:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*15:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_13_01():
    result = parse("Chsa-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_07_01():
    result = parse("Chsa-B*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_01_01():
    result = parse("Chsa-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_10_01():
    result = parse("Chsa-B*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_09_01():
    result = parse("Chsa-B*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_05_01():
    result = parse("Chsa-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_12_01():
    result = parse("Chsa-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_06_01():
    result = parse("Chsa-B*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_041_03_02():
    result = parse("Mafa-A1*041:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*041:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*041:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_03_03():
    result = parse("Patr-A*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_03_04():
    result = parse("Patr-A*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_21_01():
    result = parse("Patr-A*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_06_02():
    result = parse("Patr-B*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_19_02():
    result = parse("Patr-B*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_20_02():
    result = parse("Patr-B*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_22_03():
    result = parse("Patr-B*22:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*22:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*22:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_04():
    result = parse("Patr-B*23:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*23:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_33_01_01_01():
    result = parse("Patr-B*33:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*33:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-B*33:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_34_01():
    result = parse("Patr-B*34:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*34:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*34:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_13_03():
    result = parse("Patr-C*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DPB1_28_01():
    result = parse("Aona-DPB1*28:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DPB1*28:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DPB1*28:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DPB1_28_02():
    result = parse("Aona-DPB1*28:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DPB1*28:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DPB1*28:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DPB1_28_03():
    result = parse("Aona-DPB1*28:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DPB1*28:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DPB1*28:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_16():
    result = parse("Caja-DRB1*03:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_20():
    result = parse("Mamu-DRB6*01:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_22():
    result = parse("Mamu-DRB6*01:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_23():
    result = parse("Mamu-DRB6*01:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_24():
    result = parse("Mamu-DRB6*01:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_11():
    result = parse("Mamu-DRB*W006:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_06_07():
    result = parse("Patr-DQB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_DRB_W076_01():
    result = parse("Sala-DRB*W076:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-DRB*W076:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-DRB*W076:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_DRB1_03_01():
    result = parse("Sala-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_DRB_W076_02():
    result = parse("Sala-DRB*W076:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-DRB*W076:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-DRB*W076:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_DRB_W077_01():
    result = parse("Sala-DRB*W077:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-DRB*W077:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-DRB*W077:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_DRB_W076_03():
    result = parse("Sala-DRB*W076:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-DRB*W076:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-DRB*W076:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_DRB_W078_01():
    result = parse("Sala-DRB*W078:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-DRB*W078:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-DRB*W078:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_171_01():
    result = parse("Mafa-B*171:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*171:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*171:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_125_01():
    result = parse("Mafa-A1*125:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*125:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*125:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_03_01():
    result = parse("Mafa-E*02:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-E*02:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_04_01_01():
    result = parse("Mafa-E*02:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-E*02:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_05N():
    result = parse("Mafa-E*02:05N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:05N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:05N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_25():
    result = parse("Mafa-DQB1*18:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_01():
    result = parse("Sala-G*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_02():
    result = parse("Sala-G*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_03():
    result = parse("Sala-G*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_04():
    result = parse("Sala-G*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_09N():
    result = parse("Sala-G*03:09N")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:09N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:09N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_10N():
    result = parse("Sala-G*03:10N")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:10N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:10N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_11N():
    result = parse("Sala-G*03:11N")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:11N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:11N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_05():
    result = parse("Sala-G*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_06():
    result = parse("Sala-G*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_07():
    result = parse("Sala-G*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_12N():
    result = parse("Sala-G*03:12N")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:12N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:12N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_08():
    result = parse("Sala-G*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sala_G_03_13N():
    result = parse("Sala-G*03:13N")
    assert result.__class__ is Allele, (
        f'Expected parse("Sala-G*03:13N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sala-G*03:13N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_054_02():
    result = parse("Mafa-B*054:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*054:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*054:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_07():
    result = parse("Mafa-B*060:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_172_01():
    result = parse("Mafa-B*172:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*172:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*172:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_176_01():
    result = parse("Mafa-B*176:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*176:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*176:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_06_01():
    result = parse("Mafa-B*051:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*051:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_131_02_01_02():
    result = parse("Mafa-B*131:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*131:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*131:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_04():
    result = parse("Mafa-B*045:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*045:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_177_01():
    result = parse("Mafa-B*177:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*177:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*177:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_089_02():
    result = parse("Mafa-B*089:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*089:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*089:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_173_01():
    result = parse("Mafa-B*173:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*173:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*173:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_08():
    result = parse("Mafa-B*060:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_078_04():
    result = parse("Mafa-B*078:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*078:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*078:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_057_04():
    result = parse("Mafa-B*057:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*057:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*057:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_06_02():
    result = parse("Mafa-AG*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_175_01_01_01():
    result = parse("Mafa-B*175:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*175:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*175:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_01_01():
    result = parse("Mafa-AG*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_14():
    result = parse("Mafa-B*030:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_04_01():
    result = parse("Mafa-AG*04:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*04:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_03_01_02():
    result = parse("Mafa-AG*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_065_03():
    result = parse("Mafa-B*065:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*065:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*065:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_02_01():
    result = parse("Mafa-AG*05:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*05:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_03_02():
    result = parse("Mafa-AG*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_05_01_01():
    result = parse("Mafa-G*02:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-G*02:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_05_01_02():
    result = parse("Mafa-G*02:05:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:05:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-G*02:05:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_06():
    result = parse("Mafa-G*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_07():
    result = parse("Mafa-G*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_08():
    result = parse("Mafa-G*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_G_02_09():
    result = parse("Mafa-G*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-G*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-G*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_01_02_01():
    result = parse("Mamu-B*098:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_178_01_01_01N():
    result = parse("Mamu-B*178:01:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*178:01:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*178:01:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_178_02N():
    result = parse("Mamu-B*178:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*178:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*178:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_076_03_01_01():
    result = parse("Mamu-B*076:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*076:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*076:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_179_01_01():
    result = parse("Mane-B*179:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*179:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*179:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_098_02():
    result = parse("Mane-B*098:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*098:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*098:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_178_01_01():
    result = parse("Mane-B*178:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*178:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*178:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_178_01_02():
    result = parse("Mane-B*178:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*178:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*178:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_02_04():
    result = parse("Mane-B*068:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*068:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_131_01_01():
    result = parse("Mane-B*131:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*131:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*131:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_142_01():
    result = parse("Mane-B*142:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*142:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*142:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_131_01_02():
    result = parse("Mane-B*131:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*131:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*131:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_03():
    result = parse("Mane-B*060:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*060:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_008_04():
    result = parse("Mafa-A1*008:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*008:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*008:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_04():
    result = parse("Mafa-A1*043:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_01_02():
    result = parse("Mafa-AG*05:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*05:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_05():
    result = parse("Mafa-AG*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_02_02():
    result = parse("Mafa-AG*05:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*05:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_03():
    result = parse("Mafa-AG*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_04():
    result = parse("Mafa-AG*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_07_01_01():
    result = parse("Mafa-AG*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_01_01():
    result = parse("Mafa-AG*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_06_03():
    result = parse("Mafa-AG*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_07_01_02():
    result = parse("Mafa-AG*07:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*07:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*07:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_174_01_01_01N():
    result = parse("Mafa-B*174:01:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*174:01:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*174:01:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_162_02_01():
    result = parse("Mafa-B*162:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*162:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*162:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_175_02():
    result = parse("Mafa-B*175:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*175:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*175:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_089_01_03():
    result = parse("Mafa-B*089:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*089:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*089:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_017_03():
    result = parse("Mafa-B*017:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*017:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*017:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_05():
    result = parse("Mafa-B*044:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_06_02_01():
    result = parse("Mafa-B*051:06:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:06:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*051:06:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_012_02():
    result = parse("Mafa-B*012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_013_01():
    result = parse("Mane-B*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_047_01_02():
    result = parse("Mane-B*047:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*047:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*047:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_180_01():
    result = parse("Mane-B*180:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*180:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*180:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_115_03():
    result = parse("Mane-B*115:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*115:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*115:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_181_01():
    result = parse("Mafa-B*181:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*181:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*181:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_182_01():
    result = parse("Mafa-B*182:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*182:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*182:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_07():
    result = parse("Mafa-B*051:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_06_01_01():
    result = parse("Mafa-B*044:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*044:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_181_02():
    result = parse("Mafa-B*181:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*181:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*181:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_080_01():
    result = parse("Mafa-B*080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_06_03():
    result = parse("Mafa-B*051:06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*051:06:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_03():
    result = parse("Mafa-B*070:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*070:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_02_02():
    result = parse("Mafa-B*070:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*070:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_01_02():
    result = parse("Mafa-B*070:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*070:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_05():
    result = parse("Mafa-B*109:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_07():
    result = parse("Mafa-B*044:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_03():
    result = parse("Mafa-B*072:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_02_02():
    result = parse("Mafa-B*048:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*048:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_183_01_01_01():
    result = parse("Mafa-B*183:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*183:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*183:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_02_01_01():
    result = parse("Mafa-B*115:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*115:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_03():
    result = parse("Mafa-B*115:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*115:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_057_05_01():
    result = parse("Mafa-B*057:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*057:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*057:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_083_03():
    result = parse("Mafa-B*083:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*083:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*083:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_161_03():
    result = parse("Mafa-B*161:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*161:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*161:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_154_02():
    result = parse("Mafa-B*154:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*154:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*154:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_183_02():
    result = parse("Mafa-B*183:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*183:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*183:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_059_03():
    result = parse("Mafa-A1*059:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*059:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*059:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_04():
    result = parse("Mamu-B*019:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*019:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_02_01():
    result = parse("Mamu-B*072:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_015_04_02():
    result = parse("Mamu-B*015:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*015:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*015:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_034_04():
    result = parse("Mamu-B*034:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*034:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*034:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_186_01():
    result = parse("Mafa-B*186:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*186:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*186:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_057_05_02():
    result = parse("Mafa-B*057:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*057:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*057:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_178_01():
    result = parse("Mafa-B*178:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*178:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*178:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_08():
    result = parse("Mafa-B*044:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_06():
    result = parse("Mafa-B*109:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_185_01():
    result = parse("Mane-B*185:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*185:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*185:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_AG_03_01():
    result = parse("Mane-AG*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-AG*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-AG*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_099_01():
    result = parse("Mane-B*099:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*099:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*099:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_047_02():
    result = parse("Mane-B*047:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*047:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*047:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_178_02():
    result = parse("Mane-B*178:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*178:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*178:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_089_05():
    result = parse("Mane-B*089:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*089:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*089:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_179_01_02():
    result = parse("Mane-B*179:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*179:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*179:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_08():
    result = parse("Mafa-DRA*01:01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:01:08") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_23():
    result = parse("Mafa-DPA1*02:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_08():
    result = parse("Mafa-DPB1*15:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_06():
    result = parse("Mafa-DQA1*24:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_02():
    result = parse("Mamu-DPB1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_03():
    result = parse("Mamu-DPB1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_04():
    result = parse("Mamu-DPB1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_05():
    result = parse("Mamu-DPB1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_07_04():
    result = parse("Mamu-DPB1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_06():
    result = parse("Mamu-DPB1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B1_01_01():
    result = parse("Caja-B1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B3_01_01():
    result = parse("Caja-B3*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B3*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B3*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B4_01_01():
    result = parse("Caja-B4*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B4*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B4*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B6_01_01():
    result = parse("Caja-B6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B7_01_01():
    result = parse("Caja-B7*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B7*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B7*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B2_01_01Ps():
    result = parse("Caja-B2*01:01Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B2*01:01Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B2*01:01Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B5_01_01Ps():
    result = parse("Caja-B5*01:01Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B5*01:01Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B5*01:01Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B8_01_01Ps():
    result = parse("Caja-B8*01:01Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B8*01:01Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B8*01:01Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B9_01_01Ps():
    result = parse("Caja-B9*01:01Ps")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B9*01:01Ps") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B9*01:01Ps") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_07():
    result = parse("Mamu-DPB1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W033_04():
    result = parse("Mafa-DRB*W033:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W033:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W033:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB_W008_01():
    result = parse("Gogo-DRB*W008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB*W008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB*W008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_02_01():
    result = parse("Gogo-DRB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_DRB1_03_02():
    result = parse("Gogo-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DRB1_02_01():
    result = parse("Papa-DRB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DRB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DRB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DRB1_02_02():
    result = parse("Papa-DRB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DRB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DRB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DRB1_03_01():
    result = parse("Papa-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_DRB1_03_02():
    result = parse("Papa-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_07():
    result = parse("Patr-DRB1*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_08():
    result = parse("Patr-DRB1*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_09():
    result = parse("Patr-DRB1*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_10():
    result = parse("Patr-DRB1*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_01():
    result = parse("Patr-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_04():
    result = parse("Patr-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_03_03():
    result = parse("Patr-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Male_DRB1_03_01():
    result = parse("Male-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Male-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Male-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Male_DRB1_04_01():
    result = parse("Male-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Male-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Male-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_01():
    result = parse("Mamu-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_02():
    result = parse("Mamu-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_06_02():
    result = parse("Mamu-DRB1*04:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*04:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_01():
    result = parse("Mamu-DRA*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRA*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_063_01():
    result = parse("Mafa-B*063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_064_04():
    result = parse("Mafa-B*064:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*064:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*064:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_15():
    result = parse("Mafa-B*046:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_063_02():
    result = parse("Mafa-B*063:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*063:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*063:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_07():
    result = parse("Mafa-B*098:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_054_03():
    result = parse("Mafa-B*054:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*054:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*054:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_05():
    result = parse("Mafa-B*045:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*045:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_04_02():
    result = parse("Mafa-AG*04:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*04:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_03_01_03():
    result = parse("Mafa-AG*03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*03:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_11_01_01():
    result = parse("Mafa-A4*14:11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:11:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_05():
    result = parse("Mafa-A5*30:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A5*30:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_18():
    result = parse("Mafa-A3*13:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_02_03():
    result = parse("Mafa-AG*05:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*05:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_02_04():
    result = parse("Mafa-AG*05:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-AG*05:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_01_02():
    result = parse("Mafa-AG*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_06():
    result = parse("Mafa-AG*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_05_05():
    result = parse("Mafa-AG*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_01_03_01_01():
    result = parse("Mafa-AG*01:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*01:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-AG*01:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_07_02():
    result = parse("Mafa-AG*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_07():
    result = parse("Mafa-AG*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_01_04():
    result = parse("Mafa-AG*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_21_01():
    result = parse("Mafa-DPB1*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_02_03():
    result = parse("Mafa-DQA1*24:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*24:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_15_01():
    result = parse("Math-DPB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_22_01():
    result = parse("Math-DPB1*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_09_01_01():
    result = parse("Math-DPB1*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_01_01():
    result = parse("Math-DPB1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_09_02():
    result = parse("Math-DPB1*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_03_01():
    result = parse("Math-DPB1*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_09_01_02():
    result = parse("Math-DPB1*09:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*09:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*09:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_03():
    result = parse("Math-DPB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_01_01():
    result = parse("Math-DPB1*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_02():
    result = parse("Math-DPB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_02():
    result = parse("Math-DPB1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_03_02():
    result = parse("Math-DPB1*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_01_02():
    result = parse("Math-DPB1*06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*06:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_05():
    result = parse("Math-DPB1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_15_02():
    result = parse("Math-DPB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_04():
    result = parse("Math-DPB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_04():
    result = parse("Math-DPB1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_17():
    result = parse("Mafa-DRB*W003:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_01():
    result = parse("Math-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_01():
    result = parse("Math-DQB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_02():
    result = parse("Math-DQB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_15_02_02():
    result = parse("Math-DQB1*15:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*15:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DQB1*15:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_15_02_01():
    result = parse("Math-DQB1*15:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*15:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DQB1*15:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_17_01():
    result = parse("Math-DQB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_17_02():
    result = parse("Math-DQB1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_02():
    result = parse("Math-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_15_01():
    result = parse("Math-DQB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_03():
    result = parse("Math-DQB1*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_03():
    result = parse("Math-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_04():
    result = parse("Math-DQB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_05():
    result = parse("Math-DQB1*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_06():
    result = parse("Math-DQB1*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_07():
    result = parse("Math-DQB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_17_03():
    result = parse("Math-DQB1*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_04():
    result = parse("Math-DQB1*18:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_05():
    result = parse("Math-DQB1*18:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_06():
    result = parse("Math-DQB1*18:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_06_08():
    result = parse("Math-DQB1*06:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*06:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*06:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_06N():
    result = parse("Math-DPB1*01:06N")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:06N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:06N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W021_01():
    result = parse("Paan-DRB*W021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W081_01():
    result = parse("Paan-DRB*W081:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W081:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W081:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W006_01():
    result = parse("Paan-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W006_02():
    result = parse("Paan-DRB*W006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W007_01():
    result = parse("Paan-DRB*W007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W081_02():
    result = parse("Paan-DRB*W081:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W081:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W081:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W081_03():
    result = parse("Paan-DRB*W081:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W081:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W081:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W001_01():
    result = parse("Paan-DRB*W001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W001_02():
    result = parse("Paan-DRB*W001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W001_03():
    result = parse("Paan-DRB*W001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W003_01():
    result = parse("Paan-DRB*W003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W004_01():
    result = parse("Paan-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W027_01():
    result = parse("Paan-DRB*W027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W028_01():
    result = parse("Paan-DRB*W028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W048_01():
    result = parse("Paan-DRB*W048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W053_01():
    result = parse("Paan-DRB*W053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W082_01():
    result = parse("Paan-DRB*W082:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W082:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W082:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W056_01():
    result = parse("Paan-DRB*W056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB3_04_01():
    result = parse("Paan-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB3_04_02():
    result = parse("Paan-DRB3*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB3*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB3*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W056_02():
    result = parse("Paan-DRB*W056:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W056:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W056:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB3_04_03():
    result = parse("Paan-DRB3*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB3*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB3*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W026_01():
    result = parse("Paan-DRB*W026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W057_01():
    result = parse("Paan-DRB*W057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W083_01():
    result = parse("Paan-DRB*W083:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W083:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W083:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W058_01():
    result = parse("Paan-DRB*W058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB1_03_01():
    result = parse("Paan-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB1_07_01():
    result = parse("Paan-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB1_10_01():
    result = parse("Paan-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W084_01():
    result = parse("Paan-DRB*W084:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W084:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W084:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB1_03_02():
    result = parse("Paan-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB1_03_03():
    result = parse("Paan-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB1_03_04():
    result = parse("Paan-DRB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB1_03_05():
    result = parse("Paan-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB5_03_01():
    result = parse("Paan-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB5_03_02():
    result = parse("Paan-DRB5*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB5*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB5*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB5_03_03():
    result = parse("Paan-DRB5*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB5*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB5*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB5_03_04():
    result = parse("Paan-DRB5*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB5*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB5*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_01():
    result = parse("Paan-DRB6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_02():
    result = parse("Paan-DRB6*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_03():
    result = parse("Paan-DRB6*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_04():
    result = parse("Paan-DRB6*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_05():
    result = parse("Paan-DRB6*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_06():
    result = parse("Paan-DRB6*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_07():
    result = parse("Paan-DRB6*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_08():
    result = parse("Paan-DRB6*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_09():
    result = parse("Paan-DRB6*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_09():
    result = parse("Mafa-DRB*W001:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_10_01():
    result = parse("Mafa-DRB*W001:10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W001:10:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_11_01():
    result = parse("Mafa-DRB*W001:11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W001:11:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_07():
    result = parse("Mafa-DRB*W027:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W027:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_07():
    result = parse("Mafa-DRB*W002:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W002:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W036_05():
    result = parse("Mafa-DRB*W036:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W036:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W036:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_18():
    result = parse("Mafa-DRB*W003:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_19():
    result = parse("Mafa-DRB*W003:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_20():
    result = parse("Mafa-DRB*W003:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_07():
    result = parse("Mafa-DRB*W004:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_08():
    result = parse("Mafa-DRB*W004:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_09():
    result = parse("Mafa-DRB*W004:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W080_01():
    result = parse("Mafa-DRB*W080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W069_02():
    result = parse("Mafa-DRB*W069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_08():
    result = parse("Mafa-DRB*W006:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_09():
    result = parse("Mafa-DRB*W006:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_10():
    result = parse("Mafa-DRB*W006:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_11():
    result = parse("Mafa-DRB*W006:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_08():
    result = parse("Mafa-DRB*W007:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_36():
    result = parse("Mafa-DRB1*03:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_34():
    result = parse("Mafa-DRB1*03:34")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:34") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:34") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_35():
    result = parse("Mafa-DRB1*03:35")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:35") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:35") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_15_01():
    result = parse("Mafa-DRB1*04:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_14_02():
    result = parse("Mafa-DRB1*04:14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:14:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_14():
    result = parse("Mafa-DRB1*10:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_10():
    result = parse("Mafa-DRB3*04:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_05_01():
    result = parse("Mamu-DRB*W001:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W001:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_05_02():
    result = parse("Mamu-DRB*W001:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W001:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_04():
    result = parse("Mamu-DRB*W027:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W021_05():
    result = parse("Mamu-DRB*W021:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W021:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W021:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_10():
    result = parse("Mamu-DRB*W025:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_11():
    result = parse("Mamu-DRB*W025:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_12():
    result = parse("Mamu-DRB*W025:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W025:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W069_02():
    result = parse("Mamu-DRB*W069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_05():
    result = parse("Mamu-DRB*W027:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_06():
    result = parse("Mamu-DRB*W027:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W002_05():
    result = parse("Mamu-DRB*W002:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W002:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W002:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W079_01():
    result = parse("Mamu-DRB*W079:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W079:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W079:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W003_15_02():
    result = parse("Mamu-DRB*W003:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W003:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W003:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_07():
    result = parse("Mamu-DRB*W004:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W004:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W059_01():
    result = parse("Mamu-DRB*W059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_08():
    result = parse("Mamu-DRB*W004:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W004:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W005_03():
    result = parse("Mamu-DRB*W005:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W005:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W005:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_07():
    result = parse("Mamu-DRB*W027:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_08():
    result = parse("Mamu-DRB*W027:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_03_13_02():
    result = parse("Mamu-DRB1*03:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*03:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*03:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_18():
    result = parse("Mamu-DRB1*04:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_07_05():
    result = parse("Mamu-DRB1*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_02_02():
    result = parse("Mamu-DRB1*10:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB1*10:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_14():
    result = parse("Mamu-DRB1*10:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_15():
    result = parse("Mamu-DRB1*10:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_13():
    result = parse("Mamu-DRB3*04:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_14():
    result = parse("Mamu-DRB3*04:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_15_01():
    result = parse("Mamu-DRB3*04:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB3*04:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_15_02():
    result = parse("Mamu-DRB3*04:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB3*04:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_16_01():
    result = parse("Mamu-DRB3*04:16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB3*04:16:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_16_02():
    result = parse("Mamu-DRB3*04:16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB3*04:16:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W25_03():
    result = parse("Mane-DRB*W25:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W25:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W25:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_09():
    result = parse("Mane-DRB*W002:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W2_10():
    result = parse("Mane-DRB*W2:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W2:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W2:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_11():
    result = parse("Mane-DRB*W002:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W3_02():
    result = parse("Mane-DRB*W3:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W3:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W3:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W074_01_02():
    result = parse("Mane-DRB*W074:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W074:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB*W074:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W25_04():
    result = parse("Mane-DRB*W25:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W25:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W25:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W003_03():
    result = parse("Mane-DRB*W003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W3_04():
    result = parse("Mane-DRB*W3:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W3:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W3:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_03():
    result = parse("Mane-DRB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_10_05():
    result = parse("Mane-DRB1*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W028_02():
    result = parse("Mamu-DRB*W028:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W028:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W028:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_16():
    result = parse("Mamu-DRB1*10:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB5_03_07_02():
    result = parse("Mamu-DRB5*03:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB5*03:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB5*03:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_21_02():
    result = parse("Mafa-DPA1*02:21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:21:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_24():
    result = parse("Mafa-DPA1*02:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_10_02():
    result = parse("Mafa-DPA1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_25():
    result = parse("Mafa-DPA1*02:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_05():
    result = parse("Mafa-DPA1*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_06():
    result = parse("Mafa-DPA1*07:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_07():
    result = parse("Mafa-DPA1*07:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_08():
    result = parse("Mafa-DPA1*07:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_06_01_02():
    result = parse("Mafa-DPA1*06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*06:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_09():
    result = parse("Mafa-DPA1*07:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_10_04():
    result = parse("Mafa-DPA1*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_10():
    result = parse("Mafa-DPA1*07:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_11_01():
    result = parse("Mafa-DPA1*07:11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*07:11:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_12():
    result = parse("Mafa-DPA1*07:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_22():
    result = parse("Mafa-DRA*01:02:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:22") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_10():
    result = parse("Mafa-DRA*01:01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:01:10") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_033_02():
    result = parse("Mamu-A1*033:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*033:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*033:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_040_05():
    result = parse("Mamu-A1*040:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*040:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*040:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_054_03():
    result = parse("Mamu-A1*054:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*054:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*054:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_043_07():
    result = parse("Mamu-A1*043:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*043:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*043:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_01_01():
    result = parse("Mafa-DOA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_04():
    result = parse("Mafa-DOA*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DOA*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_01_02():
    result = parse("Mafa-DOA*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_03_01():
    result = parse("Mafa-DOA*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_01():
    result = parse("Mafa-DOA*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_03_02():
    result = parse("Mafa-DOA*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_02():
    result = parse("Mafa-DOA*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_03():
    result = parse("Mafa-DOA*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_05():
    result = parse("Mafa-DOA*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DOA*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_04():
    result = parse("Mafa-DOA*01:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_05():
    result = parse("Mafa-DOA*01:02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_01_03():
    result = parse("Mafa-DOA*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_06():
    result = parse("Mafa-DOA*01:02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_07():
    result = parse("Mafa-DOA*01:02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DOA_01_02_08():
    result = parse("Mafa-DOA*01:02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DOA*01:02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DOA*01:02:08") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_04_08():
    result = parse("Mafa-AG*04:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*04:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*04:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_06_04_01_01():
    result = parse("Mafa-AG*06:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*06:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-AG*06:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_06_05():
    result = parse("Mafa-AG*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-AG*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_11_01_02():
    result = parse("Mafa-A4*14:11:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:11:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:11:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_03_02():
    result = parse("Mafa-B*060:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*060:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_131_02_01_03():
    result = parse("Mafa-B*131:02:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*131:02:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*131:02:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_07():
    result = parse("Mafa-B*109:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_161_04():
    result = parse("Mafa-B*161:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*161:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*161:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_08():
    result = parse("Mafa-B*109:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_07():
    result = parse("Math-DQB1*18:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DQB1_18_08N():
    result = parse("Math-DQB1*18:08N")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DQB1*18:08N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DQB1*18:08N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_04_01_01_01():
    result = parse("Poab-B*04:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*04:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*04:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_A_04_01():
    result = parse("Poab-A*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-A*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Poab-A*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_C_02_01():
    result = parse("Poab-C*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-C*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Poab-C*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_A_03_01():
    result = parse("Poab-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Poab-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_A_06_01():
    result = parse("Poab-A*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-A*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Poab-A*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_08_01_01_01():
    result = parse("Poab-B*08:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*08:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*08:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_02_01_01_01():
    result = parse("Poab-B*02:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*02:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*02:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_08_02():
    result = parse("Poab-B*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Poab-B*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_03_02_01_01():
    result = parse("Poab-B*03:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*03:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*03:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_01_01_01_01():
    result = parse("Poab-B*01:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*01:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*01:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_04_01_02():
    result = parse("Poab-B*04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Poab-B*04:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_03_01_01_01():
    result = parse("Poab-B*03:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*03:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*03:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_10_01_01_01():
    result = parse("Poab-B*10:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*10:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*10:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_05_01_01_01():
    result = parse("Poab-B*05:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*05:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*05:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_08_01():
    result = parse("Popy-A*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_07_01():
    result = parse("Popy-A*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_01_02():
    result = parse("Popy-A*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_03():
    result = parse("Popy-B*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_04_02_01_01():
    result = parse("Popy-B*04:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*04:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*04:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_12_01():
    result = parse("Popy-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_11_01():
    result = parse("Popy-B*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_13_01():
    result = parse("Popy-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_02_01_01():
    result = parse("Popy-B*08:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*08:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_03_03_01_01():
    result = parse("Popy-B*03:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*03:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*03:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_05_02():
    result = parse("Popy-B*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_11_01_02():
    result = parse("Caja-G*11:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*11:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*11:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_14_01():
    result = parse("Caja-G*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_19_01():
    result = parse("Caja-G*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_15_02():
    result = parse("Caja-G*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_13_01():
    result = parse("Caja-G*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_01():
    result = parse("Caja-G*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_01():
    result = parse("Caja-G*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_03_01():
    result = parse("Caja-G*10:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*10:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_01_01():
    result = parse("Caja-G*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_02():
    result = parse("Caja-G*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_03():
    result = parse("Caja-G*12:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*12:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_11_02():
    result = parse("Caja-G*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_03_01():
    result = parse("Caja-G*08:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*08:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_16_01():
    result = parse("Caja-G*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_02():
    result = parse("Caja-G*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_03():
    result = parse("Caja-G*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_04():
    result = parse("Caja-G*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_02():
    result = parse("Caja-G*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_14_02():
    result = parse("Caja-G*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_02():
    result = parse("Caja-G*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_11_03():
    result = parse("Caja-G*11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*11:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_20_01():
    result = parse("Caja-G*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_13_02():
    result = parse("Caja-G*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_22_02():
    result = parse("Math-DPB1*22:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*22:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*22:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_05N():
    result = parse("Math-DPB1*06:05N")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:05N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:05N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_01_03():
    result = parse("Mafa-DQA1*26:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*26:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_03_03():
    result = parse("Mafa-DQA1*01:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_16():
    result = parse("Mafa-DQA1*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_08_03():
    result = parse("Mafa-DQA1*01:08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:08:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_01_02():
    result = parse("Mafa-DQA1*05:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_05_02():
    result = parse("Mafa-DQA1*05:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_07():
    result = parse("Mafa-DQA1*26:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_07_02():
    result = parse("Mafa-DQA1*05:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_07_02():
    result = parse("Mafa-DQA1*01:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_07():
    result = parse("Mafa-DQA1*24:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_17():
    result = parse("Mafa-DQA1*01:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_08():
    result = parse("Mafa-DQA1*24:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_04_02():
    result = parse("Mafa-DQA1*26:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*26:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_09():
    result = parse("Mafa-DQA1*24:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_14_02():
    result = parse("Mafa-DQA1*01:14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:14:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_35():
    result = parse("Mafa-DQB1*06:35")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:35") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:35") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_01():
    result = parse("Mafa-DMA*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DMA*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_02_01():
    result = parse("Mafa-DMA*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMA*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_03():
    result = parse("Mafa-DMA*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DMA*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_02_02():
    result = parse("Mafa-DMA*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMA*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_04_01():
    result = parse("Mafa-DMA*02:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMA*02:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_05():
    result = parse("Mafa-DMA*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DMA*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_04_02():
    result = parse("Mafa-DMA*02:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMA*02:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_04_03():
    result = parse("Mafa-DMA*02:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMA*02:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_06():
    result = parse("Mafa-DMA*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DMA*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_04_04():
    result = parse("Mafa-DMA*02:04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMA*02:04:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMA_02_07():
    result = parse("Mafa-DMA*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMA*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DMA*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMB_03_01_01():
    result = parse("Mafa-DMB*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMB*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMB*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMB_03_02_01():
    result = parse("Mafa-DMB*03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMB*03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMB*03:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMB_03_01_02():
    result = parse("Mafa-DMB*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMB*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMB*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMB_03_03():
    result = parse("Mafa-DMB*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMB*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DMB*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMB_03_04():
    result = parse("Mafa-DMB*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMB*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DMB*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMB_03_02_02():
    result = parse("Mafa-DMB*03:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMB*03:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMB*03:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DMB_03_01_03():
    result = parse("Mafa-DMB*03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DMB*03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DMB*03:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_17_02():
    result = parse("Mafa-DQB1*06:17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:17:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_26():
    result = parse("Mafa-DQB1*18:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_15_02():
    result = parse("Mafa-DPA1*02:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_26():
    result = parse("Mafa-DPA1*02:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_06():
    result = parse("Math-DPB1*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_052_02():
    result = parse("Mamu-A1*052:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*052:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*052:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_057_06():
    result = parse("Mamu-A1*057:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*057:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*057:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_187_01():
    result = parse("Mamu-B*187:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*187:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*187:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_052_03():
    result = parse("Mamu-A1*052:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*052:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*052:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_091_04():
    result = parse("Mamu-B*091:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*091:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*091:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_03():
    result = parse("Mamu-B*098:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*098:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_101_02():
    result = parse("Mamu-B*101:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*101:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*101:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_05():
    result = parse("Mamu-B*019:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*019:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_03():
    result = parse("Mamu-B*072:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*072:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_08():
    result = parse("Mamu-B*051:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_187_02():
    result = parse("Mamu-B*187:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*187:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*187:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_150_01():
    result = parse("Mamu-B*150:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*150:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*150:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_124_02():
    result = parse("Mamu-B*124:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*124:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*124:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_06():
    result = parse("Mamu-B*019:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*019:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_188_01():
    result = parse("Mamu-B*188:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*188:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*188:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_189_01():
    result = parse("Mamu-B*189:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*189:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*189:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_075_04():
    result = parse("Mamu-B*075:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*075:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*075:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_04():
    result = parse("Mamu-B*098:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*098:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_039_02():
    result = parse("Mamu-B*039:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*039:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*039:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRA_01_01_01():
    result = parse("Chsa-DRA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-DRA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DQA1_05_01():
    result = parse("Chsa-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DQA1_01_01():
    result = parse("Chsa-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DPA1_08_01():
    result = parse("Chsa-DPA1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DPA1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DPA1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DPA1_02_01():
    result = parse("Chsa-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DPA1_02_02():
    result = parse("Chsa-DPA1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DPA1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DPA1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DPA1_02_03():
    result = parse("Chsa-DPA1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DPA1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DPA1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DPA1_02_04():
    result = parse("Chsa-DPA1*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DPA1*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DPA1*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB5_01_01():
    result = parse("Chsa-DRB5*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB5*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB5*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB_W085_01():
    result = parse("Chsa-DRB*W085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB*W085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB*W085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB_W086_01():
    result = parse("Chsa-DRB*W086:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB*W086:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB*W086:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB1_07_01():
    result = parse("Chsa-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB_W061_01():
    result = parse("Chsa-DRB*W061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB*W061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB*W061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB1_03_01():
    result = parse("Chsa-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB_W007_01():
    result = parse("Chsa-DRB*W007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB*W007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB*W007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DQB1_17_01():
    result = parse("Chsa-DQB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DQB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DQB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DQB1_06_01():
    result = parse("Chsa-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DQB1_15_01():
    result = parse("Chsa-DQB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DQB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DQB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DPB1_09_01():
    result = parse("Chsa-DPB1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DPB1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DPB1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DPB1_23_01():
    result = parse("Chsa-DPB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DPB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DPB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_07_01():
    result = parse("Chsa-A*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_03_03_01():
    result = parse("Chsa-A*03:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*03:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-A*03:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_03_04():
    result = parse("Chsa-A*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_08_01():
    result = parse("Chsa-A*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_16_01():
    result = parse("Chsa-B*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_13_02():
    result = parse("Chsa-B*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_09_02():
    result = parse("Chsa-B*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_03():
    result = parse("Chsa-B*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_17_01():
    result = parse("Chsa-B*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_18_01():
    result = parse("Chsa-B*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_04():
    result = parse("Chsa-B*15:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_E_02_01_01():
    result = parse("Chsa-E*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-E*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-E*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_E_02_02():
    result = parse("Chsa-E*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-E*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-E*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_E_02_01_02():
    result = parse("Chsa-E*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-E*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-E*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB_W006_01():
    result = parse("Chsa-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB_W021_01():
    result = parse("Chsa-DRB*W021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB*W021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB*W021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB_W087_01():
    result = parse("Chsa-DRB*W087:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB*W087:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB*W087:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB1_07_02():
    result = parse("Chsa-DRB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRB5_03_01():
    result = parse("Chsa-DRB5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRB5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-DRB5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_18():
    result = parse("Mafa-DQA1*01:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_19():
    result = parse("Mafa-DQA1*01:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_20():
    result = parse("Mafa-DQA1*01:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_10():
    result = parse("Mafa-DQA1*24:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_11():
    result = parse("Mafa-DQA1*24:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_08_04():
    result = parse("Mafa-DQA1*01:08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:08:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_01():
    result = parse("Mafa-F*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-F*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_02_01():
    result = parse("Mafa-F*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-F*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_03_01():
    result = parse("Mafa-F*02:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-F*02:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_04_01():
    result = parse("Mafa-F*02:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-F*02:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_05():
    result = parse("Mafa-F*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-F*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_06_01():
    result = parse("Mafa-F*02:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-F*02:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_07():
    result = parse("Mafa-F*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-F*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_02_02():
    result = parse("Mafa-F*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-F*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_012_01_2():
    result = parse("DLA-DQA1*012:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*012:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*012:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_014_01_2():
    result = parse("DLA-DQA1*014:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*014:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*014:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_016_01_2():
    result = parse("DLA-DQA1*016:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*016:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*016:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_017_01():
    result = parse("DLA-DQA1*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_018_01():
    result = parse("DLA-DQA1*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_020_01():
    result = parse("DLA-DQA1*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_021_01():
    result = parse("DLA-DQA1*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_023_01():
    result = parse("DLA-DQA1*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_025_01():
    result = parse("DLA-DQA1*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_026_01():
    result = parse("DLA-DQA1*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_008_03():
    result = parse("DLA-DQB1*008:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*008:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*008:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_013_04():
    result = parse("DLA-DQB1*013:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*013:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*013:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_013_05():
    result = parse("DLA-DQB1*013:05")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*013:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*013:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_023_02():
    result = parse("DLA-DQB1*023:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*023:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*023:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_023_03():
    result = parse("DLA-DQB1*023:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*023:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*023:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_029_01():
    result = parse("DLA-DQB1*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_036_01():
    result = parse("DLA-DQB1*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_037_01():
    result = parse("DLA-DQB1*037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_038_01():
    result = parse("DLA-DQB1*038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_038_02():
    result = parse("DLA-DQB1*038:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*038:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*038:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_039_01():
    result = parse("DLA-DQB1*039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_040_01():
    result = parse("DLA-DQB1*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_041_01():
    result = parse("DLA-DQB1*041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_042_01():
    result = parse("DLA-DQB1*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_044_01():
    result = parse("DLA-DQB1*044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_047_01():
    result = parse("DLA-DQB1*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_048_01():
    result = parse("DLA-DQB1*048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_049_01():
    result = parse("DLA-DQB1*049:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*049:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*049:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_051_01():
    result = parse("DLA-DQB1*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_053_01():
    result = parse("DLA-DQB1*053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_054_01():
    result = parse("DLA-DQB1*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_055_01():
    result = parse("DLA-DQB1*055:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*055:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*055:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_056_01():
    result = parse("DLA-DQB1*056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_059_01():
    result = parse("DLA-DQB1*059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_03():
    result = parse("DLA-DRB1*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_002_03():
    result = parse("DLA-DRB1*002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_003_02():
    result = parse("DLA-DRB1*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_011_02():
    result = parse("DLA-DRB1*011:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*011:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*011:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_011_03():
    result = parse("DLA-DRB1*011:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*011:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*011:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_013_02():
    result = parse("DLA-DRB1*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_015_04():
    result = parse("DLA-DRB1*015:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*015:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*015:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_017_02():
    result = parse("DLA-DRB1*017:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*017:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*017:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_017_03():
    result = parse("DLA-DRB1*017:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*017:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*017:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_020_02():
    result = parse("DLA-DRB1*020:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*020:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*020:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_023_02():
    result = parse("DLA-DRB1*023:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*023:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*023:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_033_01_2():
    result = parse("DLA-DRB1*033:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*033:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*033:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_036_02():
    result = parse("DLA-DRB1*036:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*036:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*036:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_045_02():
    result = parse("DLA-DRB1*045:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*045:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*045:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_048_01():
    result = parse("DLA-DRB1*048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_048_02():
    result = parse("DLA-DRB1*048:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*048:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*048:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_049_01():
    result = parse("DLA-DRB1*049:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*049:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*049:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_049_02():
    result = parse("DLA-DRB1*049:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*049:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*049:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_050_01():
    result = parse("DLA-DRB1*050:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*050:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*050:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_027_02():
    result = parse("DLA-DRB1*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_052_01():
    result = parse("DLA-DRB1*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_053_01():
    result = parse("DLA-DRB1*053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_054_01():
    result = parse("DLA-DRB1*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_055_01():
    result = parse("DLA-DRB1*055:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*055:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*055:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_056_01():
    result = parse("DLA-DRB1*056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_057_01():
    result = parse("DLA-DRB1*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_058_01():
    result = parse("DLA-DRB1*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_059_01():
    result = parse("DLA-DRB1*059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_061_01():
    result = parse("DLA-DRB1*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_062_01():
    result = parse("DLA-DRB1*062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_062_02():
    result = parse("DLA-DRB1*062:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*062:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*062:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_068_01():
    result = parse("DLA-DRB1*068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_069_01():
    result = parse("DLA-DRB1*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_070_01():
    result = parse("DLA-DRB1*070:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*070:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*070:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_071_01():
    result = parse("DLA-DRB1*071:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*071:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*071:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_072_01():
    result = parse("DLA-DRB1*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_073_01():
    result = parse("DLA-DRB1*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_074_01():
    result = parse("DLA-DRB1*074:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*074:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*074:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_075_01():
    result = parse("DLA-DRB1*075:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*075:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*075:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_076_01():
    result = parse("DLA-DRB1*076:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*076:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*076:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_077_01():
    result = parse("DLA-DRB1*077:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*077:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*077:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_078_01():
    result = parse("DLA-DRB1*078:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*078:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*078:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_079_01():
    result = parse("DLA-DRB1*079:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*079:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*079:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_080_01():
    result = parse("DLA-DRB1*080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_081_01():
    result = parse("DLA-DRB1*081:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*081:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*081:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_082_01():
    result = parse("DLA-DRB1*082:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*082:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*082:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_073_02():
    result = parse("DLA-DRB1*073:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*073:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*073:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_084_01():
    result = parse("DLA-DRB1*084:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*084:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*084:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_085_01():
    result = parse("DLA-DRB1*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_086_01():
    result = parse("DLA-DRB1*086:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*086:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*086:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_087_01():
    result = parse("DLA-DRB1*087:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*087:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*087:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_088_01():
    result = parse("DLA-DRB1*088:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*088:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*088:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_089_01():
    result = parse("DLA-DRB1*089:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*089:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*089:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_090_01_1():
    result = parse("DLA-DRB1*090:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*090:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*090:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_090_01_2():
    result = parse("DLA-DRB1*090:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*090:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*090:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_091_01():
    result = parse("DLA-DRB1*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_092_01_1():
    result = parse("DLA-DRB1*092:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*092:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*092:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_093_01():
    result = parse("DLA-DRB1*093:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*093:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*093:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_094_01():
    result = parse("DLA-DRB1*094:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*094:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*094:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_002_04():
    result = parse("DLA-DRB1*002:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*002:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*002:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_096_01():
    result = parse("DLA-DRB1*096:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*096:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*096:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_09701():
    result = parse("DLA-DRB1*09701")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*09701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*09701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_048_04():
    result = parse("DLA-DRB1*048:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*048:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*048:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_099_01():
    result = parse("DLA-DRB1*099:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*099:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*099:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_100_01():
    result = parse("DLA-DRB1*100:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*100:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*100:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_101_01():
    result = parse("DLA-DRB1*101:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*101:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*101:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_109_01():
    result = parse("DLA-DRB1*109:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*109:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*109:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_109_02():
    result = parse("DLA-DRB1*109:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*109:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*109:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_001_01():
    result = parse("Eqca-DQA1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_002_01():
    result = parse("Eqca-DQA1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_002_02():
    result = parse("Eqca-DQA1*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_003_01():
    result = parse("Eqca-DQA1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_004_01():
    result = parse("Eqca-DQA1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA2_001_01():
    result = parse("Eqca-DQA2*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA2*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA2*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA2_002_01():
    result = parse("Eqca-DQA2*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA2*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA2*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA2_002_02():
    result = parse("Eqca-DQA2*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA2*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA2*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA3_001_01():
    result = parse("Eqca-DQA3*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA3*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA3*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRA_001_01():
    result = parse("Eqca-DRA*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRA*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRA*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRA_001_02():
    result = parse("Eqca-DRA*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRA*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRA*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRA_001_03():
    result = parse("Eqca-DRA*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRA*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRA*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRA_001_04():
    result = parse("Eqca-DRA*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRA*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRA*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_001_01():
    result = parse("Eqca-DRB1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_002_01():
    result = parse("Eqca-DRB1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_003_01():
    result = parse("Eqca-DRB1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_004_01():
    result = parse("Eqca-DRB1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_005_01():
    result = parse("Eqca-DRB1*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB2_001_01():
    result = parse("Eqca-DRB2*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB2*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB2*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB2_002_01():
    result = parse("Eqca-DRB2*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB2*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB2*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB2_003_01():
    result = parse("Eqca-DRB2*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB2*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB2*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB2_004_01():
    result = parse("Eqca-DRB2*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB2*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB2*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB2_001_02():
    result = parse("Eqca-DRB2*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB2*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB2*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB3_001_01_02():
    result = parse("Eqca-DRB3*001:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB3*001:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Eqca-DRB3*001:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB3_001_01_01():
    result = parse("Eqca-DRB3*001:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB3*001:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Eqca-DRB3*001:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB3_002_01():
    result = parse("Eqca-DRB3*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB3*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB3*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB1_001_01():
    result = parse("Eqca-DQB1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB1_002_01():
    result = parse("Eqca-DQB1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB1_003_01():
    result = parse("Eqca-DQB1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB1_004_01():
    result = parse("Eqca-DQB1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB1_005_01():
    result = parse("Eqca-DQB1*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB1*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB1*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB2_001_01():
    result = parse("Eqca-DQB2*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB2*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB2*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB2_002_01():
    result = parse("Eqca-DQB2*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB2*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB2*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB2_003_01():
    result = parse("Eqca-DQB2*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB2*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB2*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB2_004_01():
    result = parse("Eqca-DQB2*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB2*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB2*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB2_005_01():
    result = parse("Eqca-DQB2*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB2*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB2*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB3_001_01():
    result = parse("Eqca-DQB3*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB3*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB3*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB3_002_01():
    result = parse("Eqca-DQB3*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB3*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB3*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB3_003_01():
    result = parse("Eqca-DQB3*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB3*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB3*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_1_001_01():
    result = parse("Eqca-1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_2_001_01():
    result = parse("Eqca-2*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-2*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-2*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_3_001_01():
    result = parse("Eqca-3*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-3*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-3*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_6_001_01():
    result = parse("Eqca-6*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-6*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-6*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_4_001_01():
    result = parse("Eqca-4*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-4*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-4*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_7_002_01():
    result = parse("Eqca-7*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-7*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-7*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_N_001_01():
    result = parse("Eqca-N*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-N*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-N*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_16_001_01():
    result = parse("Eqca-16*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-16*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-16*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_7_001_01():
    result = parse("Eqca-7*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-7*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-7*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_16_003_01():
    result = parse("Eqca-16*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-16*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-16*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_2_003_02():
    result = parse("Eqca-2*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-2*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-2*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_1_002_01():
    result = parse("Eqca-1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_1_002_02():
    result = parse("Eqca-1*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-1*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-1*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_2_001_02():
    result = parse("Eqca-2*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-2*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-2*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_2_001_03():
    result = parse("Eqca-2*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-2*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-2*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_N_005_01():
    result = parse("Eqca-N*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-N*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-N*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_1_003_01():
    result = parse("Eqca-1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_N_006_01():
    result = parse("Eqca-N*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-N*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-N*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_Ap_06_01():
    result = parse("Popy-Ap*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-Ap*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-Ap*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_Ap_03_01():
    result = parse("Popy-Ap*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-Ap*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-Ap*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_Ap_05_01():
    result = parse("Popy-Ap*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-Ap*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-Ap*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_Ap_02_02():
    result = parse("Popy-Ap*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-Ap*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-Ap*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_Ap_01_01():
    result = parse("Popy-Ap*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-Ap*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-Ap*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_Ap_04_01():
    result = parse("Popy-Ap*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-Ap*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-Ap*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_Ap_02_01():
    result = parse("Popy-Ap*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-Ap*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-Ap*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_05():
    result = parse("Caja-G*08:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_04():
    result = parse("Caja-G*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_05():
    result = parse("Caja-G*09:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_03_02():
    result = parse("Caja-G*10:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*10:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_01():
    result = parse("Caja-G*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_04():
    result = parse("Caja-G*12:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*12:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_03():
    result = parse("Caja-G*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_17_01():
    result = parse("Caja-G*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_01_02():
    result = parse("Caja-G*07:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*07:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_02():
    result = parse("Caja-G*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_27():
    result = parse("Mafa-DQB1*18:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_15_05():
    result = parse("Mafa-DQB1*15:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*15:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*15:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_17_02():
    result = parse("Mafa-DPB1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_03():
    result = parse("Mamu-DPA1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_06_03():
    result = parse("Mamu-DPA1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_06_01_02():
    result = parse("Mamu-DPA1*06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*06:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_02():
    result = parse("Mamu-DPA1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_10():
    result = parse("Mamu-DPA1*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_10_01():
    result = parse("Mamu-DPA1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_11_01():
    result = parse("Mamu-DPA1*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_11():
    result = parse("Mamu-DPA1*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_08_02():
    result = parse("Mamu-DPA1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_01_02():
    result = parse("Mamu-DQA1*24:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*24:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_22_01():
    result = parse("Atfu-G*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_04_01():
    result = parse("Caja-G*18:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*18:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_06_01():
    result = parse("Atfu-G*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_03_02():
    result = parse("Caja-G*12:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*12:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_23_01():
    result = parse("Atfu-G*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_24_01():
    result = parse("Atfu-G*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_06_03():
    result = parse("Atfu-G*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_03_01():
    result = parse("Caja-G*18:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*18:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_06_02():
    result = parse("Atfu-G*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_25_01():
    result = parse("Atfu-G*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_01_01():
    result = parse("Atfu-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_01_02():
    result = parse("Atfu-B*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_02_01():
    result = parse("Atfu-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_03_01():
    result = parse("Atfu-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_02_04():
    result = parse("Atfu-B*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_04_01():
    result = parse("Atfu-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_04_02():
    result = parse("Atfu-B*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B_03_01_01():
    result = parse("Caja-B*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-B*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B_01_01():
    result = parse("Caja-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B3_01_02():
    result = parse("Caja-B3*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B3*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B3*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B_01_02():
    result = parse("Caja-B*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_13_03():
    result = parse("Caja-G*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_21_01():
    result = parse("Caja-G*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_05():
    result = parse("Caja-G*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_04():
    result = parse("Caja-G*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_03_01():
    result = parse("Caja-G*12:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*12:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_17_02():
    result = parse("Caja-G*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_03_03():
    result = parse("Caja-G*10:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*10:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_06():
    result = parse("Caja-G*09:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_28():
    result = parse("Mamu-DRB6*01:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_06():
    result = parse("Mamu-DQA1*24:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_05_03():
    result = parse("Mamu-DQA1*01:05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*01:05:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_07():
    result = parse("Mamu-DQA1*24:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_29():
    result = parse("Mamu-DRB6*01:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB6*01:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB6_01_12_02():
    result = parse("Mamu-DRB6*01:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB6*01:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB6*01:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_05_05():
    result = parse("Mamu-DQA1*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_08():
    result = parse("Mamu-DQA1*24:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_07():
    result = parse("Mamu-DQA1*26:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_09():
    result = parse("Mamu-DQA1*24:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_24_10():
    result = parse("Mamu-DQA1*24:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*24:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*24:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_04():
    result = parse("Mamu-DPA1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_07():
    result = parse("Mamu-DRA*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRA*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_08():
    result = parse("Mamu-DRA*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRA*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_001_02():
    result = parse("Math-A1*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_126_01():
    result = parse("Math-A1*126:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*126:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*126:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_057_01():
    result = parse("Math-A1*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_06_01():
    result = parse("Mamu-DRA*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_054_01():
    result = parse("Math-A1*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_09():
    result = parse("Mamu-DRA*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRA*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A2_24_01():
    result = parse("Math-A2*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A2*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A2*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_004_01():
    result = parse("Math-A1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_040_01():
    result = parse("Math-A1*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_A1_060_01():
    result = parse("Maar-A1*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-A1*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-A1*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_A1_129_01():
    result = parse("Maar-A1*129:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-A1*129:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-A1*129:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_23_02():
    result = parse("Atfu-G*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_G_23_03():
    result = parse("Atfu-G*23:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-G*23:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-G*23:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_05_01():
    result = parse("Atfu-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_03_02():
    result = parse("Atfu-B*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_01_03():
    result = parse("Atfu-B*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_02_02():
    result = parse("Atfu-B*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Atfu_B_02_03():
    result = parse("Atfu-B*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Atfu-B*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Atfu-B*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_039_01():
    result = parse("Maas-B*039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_030_01():
    result = parse("Maas-B*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_043_01():
    result = parse("Maas-B*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B_02_01():
    result = parse("Caja-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_017_01():
    result = parse("Maas-B*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_13_04():
    result = parse("Caja-G*13:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*13:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*13:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_069_01():
    result = parse("Maas-B*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_22_01():
    result = parse("Caja-G*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_045_01():
    result = parse("Maas-B*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_19_02():
    result = parse("Caja-G*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_017_02():
    result = parse("Maas-B*017:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*017:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*017:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_037_01():
    result = parse("Maas-B*037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_069_02():
    result = parse("Maas-B*069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_04_04():
    result = parse("Mamu-DRA*01:04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:04:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_075_01():
    result = parse("Maas-B*075:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*075:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*075:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_043_02():
    result = parse("Maas-B*043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_04_05():
    result = parse("Mamu-DRA*01:04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:04:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_007_02():
    result = parse("Mane-B*007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_04_06():
    result = parse("Mamu-DRA*01:04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:04:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_111_02():
    result = parse("Mane-B*111:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*111:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*111:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_02_01():
    result = parse("Mamu-DRA*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRA*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_047_03():
    result = parse("Mane-B*047:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*047:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*047:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_019_01():
    result = parse("Mane-B*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_120_03():
    result = parse("Mane-B*120:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*120:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*120:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_04_03():
    result = parse("Mane-B*072:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_115_04():
    result = parse("Mane-B*115:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*115:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*115:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_04():
    result = parse("Mane-B*060:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*060:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_I_01_01():
    result = parse("Maas-I*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-I*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-I*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_I_01_02():
    result = parse("Maas-I*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-I*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-I*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_28():
    result = parse("Mafa-DQB1*18:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_I_01_03():
    result = parse("Maas-I*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-I*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-I*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_36():
    result = parse("Mafa-DQB1*06:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_02_03():
    result = parse("Mamu-DPB1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_139_01():
    result = parse("Maar-B*139:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*139:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*139:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_05_02():
    result = parse("Mamu-DPB1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_I_01_04():
    result = parse("Maar-I*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-I*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-I*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_06_02():
    result = parse("Mafa-DPB1*06:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*06:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_I_01_05():
    result = parse("Maar-I*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-I*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-I*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_16_01():
    result = parse("Mafa-DPB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_12():
    result = parse("Mane-I*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_04_02():
    result = parse("Mamu-DQA1*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_05_06():
    result = parse("Mamu-DQA1*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_13():
    result = parse("Mane-I*01:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_I_01_01():
    result = parse("Math-I*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-I*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-I*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_02_04():
    result = parse("Mafa-DQA1*24:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*24:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_I_01_02():
    result = parse("Math-I*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-I*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-I*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_114_01():
    result = parse("Math-B*114:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*114:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*114:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_21():
    result = parse("Mafa-DQA1*01:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_I_01_03():
    result = parse("Math-I*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-I*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-I*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_15_02():
    result = parse("Mafa-DQA1*01:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_PS2_01_02():
    result = parse("Caja-PS2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-PS2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-PS2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_PS2_01_03():
    result = parse("Caja-PS2*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-PS2*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-PS2*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_PS2_01_04():
    result = parse("Caja-PS2*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-PS2*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-PS2*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_23():
    result = parse("Mafa-DRA*01:02:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:23") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_04_02():
    result = parse("Mafa-DRA*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_11():
    result = parse("Mafa-DRA*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_12():
    result = parse("Mafa-DRA*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_09():
    result = parse("Mafa-DRA*01:03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:09") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_06_02():
    result = parse("Mamu-DRA*01:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_04_07():
    result = parse("Mamu-DRA*01:04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:04:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_08():
    result = parse("Mamu-DRA*01:02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:08") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_02_09():
    result = parse("Mamu-DRA*01:02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:02:09") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_06_03():
    result = parse("Mamu-DRA*01:06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRA*01:06:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_02_01():
    result = parse("Mane-DRA*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_02_02():
    result = parse("Mane-DRA*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_05():
    result = parse("Mane-DRA*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRA*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_06_01():
    result = parse("Mane-DRA*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_06_02():
    result = parse("Mane-DRA*01:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_03_01():
    result = parse("Mane-DRA*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_03_02():
    result = parse("Mane-DRA*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_04_01():
    result = parse("Mane-DRA*01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_01():
    result = parse("Mane-DRA*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRA*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_02_01_01():
    result = parse("Mane-DRA*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_02_01_02():
    result = parse("Mane-DRA*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_085_01():
    result = parse("Math-A1*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_001_01():
    result = parse("Math-A1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_A1_085_02():
    result = parse("Math-A1*085:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-A1*085:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-A1*085:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_A1_127_01():
    result = parse("Maar-A1*127:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-A1*127:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-A1*127:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_A1_011_01():
    result = parse("Maar-A1*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-A1*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-A1*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_A1_128_01():
    result = parse("Maas-A1*128:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-A1*128:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-A1*128:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_A1_054_01():
    result = parse("Maas-A1*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-A1*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-A1*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_A1_064_01():
    result = parse("Maas-A1*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-A1*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-A1*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_A1_023_01():
    result = parse("Maas-A1*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-A1*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-A1*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_A1_075_01():
    result = parse("Maas-A1*075:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-A1*075:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-A1*075:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_A1_062_01():
    result = parse("Maas-A1*062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-A1*062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-A1*062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_064_01():
    result = parse("Mane-A1*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_003_02():
    result = parse("Mane-A1*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_072_02():
    result = parse("Mane-A1*072:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*072:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*072:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_073_01():
    result = parse("Mane-A1*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_085_01():
    result = parse("Math-B*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_036_01():
    result = parse("Math-B*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_030_01():
    result = parse("Math-B*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_147_01():
    result = parse("Math-B*147:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*147:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*147:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_074_01():
    result = parse("Math-B*074:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*074:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*074:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_007_01():
    result = parse("Math-B*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_190_01():
    result = parse("Math-B*190:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*190:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*190:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_112_01():
    result = parse("Math-B*112:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*112:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*112:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_121_01():
    result = parse("Math-B*121:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*121:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*121:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_100_01():
    result = parse("Math-B*100:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*100:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*100:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_047_01():
    result = parse("Math-B*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_155_01():
    result = parse("Math-B*155:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*155:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*155:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_191_01():
    result = parse("Math-B*191:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*191:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*191:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_091_01():
    result = parse("Math-B*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_072_01():
    result = parse("Math-B*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_082_01():
    result = parse("Maar-B*082:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*082:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*082:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_072_01():
    result = parse("Maar-B*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_069_01():
    result = parse("Maar-B*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_069_02():
    result = parse("Maar-B*069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_139_02():
    result = parse("Maar-B*139:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*139:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*139:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_017_01():
    result = parse("Maar-B*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_B_007_01():
    result = parse("Maar-B*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-B*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-B*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_007_01():
    result = parse("Maas-B*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maas_B_074_01():
    result = parse("Maas-B*074:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maas-B*074:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maas-B*074:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_09_02():
    result = parse("Mafa-DPA1*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_12():
    result = parse("Mafa-DQA1*24:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_38_01():
    result = parse("Patr-B*38:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*38:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*38:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_22_04():
    result = parse("Patr-B*22:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*22:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*22:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_22_05():
    result = parse("Patr-B*22:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*22:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*22:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_05():
    result = parse("Patr-B*23:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*23:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_39_01():
    result = parse("Patr-B*39:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*39:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*39:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_28_02():
    result = parse("Patr-B*28:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*28:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*28:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_06_03():
    result = parse("Patr-B*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_09_02():
    result = parse("Mafa-DPA1*02:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_13():
    result = parse("Mafa-DPA1*07:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_12():
    result = parse("Mamu-DPA1*02:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_192_01():
    result = parse("Math-B*192:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*192:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*192:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_B_017_01():
    result = parse("Math-B*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-B*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-B*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_A1_129_02():
    result = parse("Maar-A1*129:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-A1*129:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-A1*129:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Maar_A1_018_01():
    result = parse("Maar-A1*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Maar-A1*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Maar-A1*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMA_02_01_01():
    result = parse("Mamu-DMA*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMA*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMA*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMA_02_01_02():
    result = parse("Mamu-DMA*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMA*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMA*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMA_02_02_01():
    result = parse("Mamu-DMA*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMA*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMA*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMA_02_02_02():
    result = parse("Mamu-DMA*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMA*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMA*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMA_02_01_03():
    result = parse("Mamu-DMA*02:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMA*02:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMA*02:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMA_02_01_04():
    result = parse("Mamu-DMA*02:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMA*02:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMA*02:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMB_03_01():
    result = parse("Mamu-DMB*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMB*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DMB*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMB_03_03():
    result = parse("Mamu-DMB*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMB*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DMB*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMB_03_02_01():
    result = parse("Mamu-DMB*03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMB*03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMB*03:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DMB_03_02_02():
    result = parse("Mamu-DMB*03:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DMB*03:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DMB*03:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_092_03():
    result = parse("Mamu-A1*092:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*092:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*092:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_01():
    result = parse("Mane-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_02():
    result = parse("Mane-DPA1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_03():
    result = parse("Mane-DPA1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_04():
    result = parse("Mane-DPA1*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_05():
    result = parse("Mane-DPA1*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_04_01():
    result = parse("Mane-DPA1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_04_02():
    result = parse("Mane-DPA1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_07_01():
    result = parse("Mane-DPA1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_07_02():
    result = parse("Mane-DPA1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_07_03():
    result = parse("Mane-DPA1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_06():
    result = parse("Mane-DPA1*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_07():
    result = parse("Mane-DPA1*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_07_04():
    result = parse("Mane-DPA1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_11_01():
    result = parse("Mafa-DPA1*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G2Ps_01():
    result = parse("Caja-G2Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G2Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G2Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_17_03():
    result = parse("Caja-G*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G4Ps_01():
    result = parse("Caja-G4Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G4Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G4Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_F1Ps_01():
    result = parse("Caja-F1Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-F1Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-F1Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G5Ps_01():
    result = parse("Caja-G5Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G5Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G5Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G6Ps_01():
    result = parse("Caja-G6Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G6Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G6Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_F2Ps_01():
    result = parse("Caja-F2Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-F2Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-F2Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_06():
    result = parse("Caja-G*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G8Ps_01():
    result = parse("Caja-G8Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G8Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G8Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G9Ps_01():
    result = parse("Caja-G9Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G9Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G9Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G10Ps_01():
    result = parse("Caja-G10Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G10Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G10Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G11Ps_01():
    result = parse("Caja-G11Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G11Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G11Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_F3Ps_01():
    result = parse("Caja-F3Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-F3Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-F3Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_F_04_01():
    result = parse("Caja-F*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-F*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-F*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G13Ps_01():
    result = parse("Caja-G13Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G13Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G13Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G14Ps_01():
    result = parse("Caja-G14Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G14Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-G14Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_F5Ps_01():
    result = parse("Caja-F5Ps*01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-F5Ps*01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Caja-F5Ps*01") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_010_03():
    result = parse("Mamu-A1*010:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*010:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*010:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_09():
    result = parse("Mamu-A1*019:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_090_02():
    result = parse("Mamu-A1*090:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*090:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*090:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_090_03():
    result = parse("Mamu-A1*090:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*090:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*090:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_13():
    result = parse("Mamu-A4*14:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_06():
    result = parse("Mamu-A6*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A6*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_07():
    result = parse("Mamu-B*019:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*019:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_09_01_01():
    result = parse("Mamu-B*045:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*045:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_050_03():
    result = parse("Mamu-B*050:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*050:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*050:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_09_01_01():
    result = parse("Mamu-B*051:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_05():
    result = parse("Mamu-B*056:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*056:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_05():
    result = parse("Mamu-B*060:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_01_02_01():
    result = parse("Mamu-B*060:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_067_01_02():
    result = parse("Mamu-B*067:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*067:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*067:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_074_03Sp():
    result = parse("Mamu-B*074:03Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*074:03Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*074:03Sp") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_07():
    result = parse("Mamu-B*082:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_088_01_02_01():
    result = parse("Mamu-B*088:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*088:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*088:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_05():
    result = parse("Mamu-B*098:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*098:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_193_01_01_01():
    result = parse("Mamu-B*193:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*193:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*193:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_002_03():
    result = parse("Mamu-B*002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_142_02_01_01():
    result = parse("Mamu-B*142:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*142:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*142:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_150_02():
    result = parse("Mamu-B*150:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*150:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*150:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_151_01():
    result = parse("Mamu-B*151:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*151:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*151:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_167_01_01_01():
    result = parse("Mamu-B*167:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*167:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*167:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_27():
    result = parse("Mamu-I*01:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_28():
    result = parse("Mamu-I*01:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_01_03():
    result = parse("Mamu-I*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB1_03_05():
    result = parse("Aovo-DRB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB1_03_06():
    result = parse("Aovo-DRB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB1_03_07():
    result = parse("Aovo-DRB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_10_01():
    result = parse("Chae-A*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_07():
    result = parse("Caja-G*08:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_11_01():
    result = parse("Chae-A*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_03_01():
    result = parse("Chae-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_12_01():
    result = parse("Chae-A*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_09_01():
    result = parse("Chae-A*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_13_01():
    result = parse("Chae-A*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_18_01():
    result = parse("Chae-A*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_17_02():
    result = parse("Chae-A*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_14_01():
    result = parse("Chae-A*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_15_01():
    result = parse("Chae-A*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_16_01():
    result = parse("Chae-A*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_A_01_01():
    result = parse("Cemi-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_17_01():
    result = parse("Chae-A*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_A_02_01():
    result = parse("Cemi-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_A_01_02():
    result = parse("Chae-A*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-A*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-A*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_A_03_01():
    result = parse("Cemi-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_19_01():
    result = parse("Chae-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_B_01_01():
    result = parse("Cemi-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_13_01():
    result = parse("Chae-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_B_02_01():
    result = parse("Cemi-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_20_01():
    result = parse("Chae-B*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_B_03_01():
    result = parse("Cemi-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_21_01():
    result = parse("Chae-B*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cemi_B_04_01():
    result = parse("Cemi-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cemi-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Cemi-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_22_01():
    result = parse("Chae-B*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_13_02():
    result = parse("Chae-B*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_23_01():
    result = parse("Chae-B*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_24_01():
    result = parse("Chae-B*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_21_02():
    result = parse("Chae-B*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_25_01():
    result = parse("Chae-B*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_03_01():
    result = parse("Chae-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_16_01():
    result = parse("Chae-B*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_26_01():
    result = parse("Chae-B*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_27_01():
    result = parse("Chae-B*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_15_01():
    result = parse("Chae-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_15_02():
    result = parse("Chae-B*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_15_03():
    result = parse("Chae-B*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_03_02():
    result = parse("Chae-B*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_15_02():
    result = parse("Mane-DQB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chae_B_05_01():
    result = parse("Chae-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chae-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chae-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W091_01():
    result = parse("Aovo-DRB*W091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W092_01():
    result = parse("Aovo-DRB*W092:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W092:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W092:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_07():
    result = parse("Mane-DQB1*17:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*17:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W092_02():
    result = parse("Aovo-DRB*W092:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W092:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W092:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W091_02():
    result = parse("Aovo-DRB*W091:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W091:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W091:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W093_01():
    result = parse("Aovo-DRB*W093:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W093:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W093:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W088_01():
    result = parse("Aovo-DRB*W088:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W088:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W088:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_19N():
    result = parse("Mamu-DRB1*04:19N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:19N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:19N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W018_01():
    result = parse("Aovo-DRB*W018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W018_02():
    result = parse("Aovo-DRB*W018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W018_03():
    result = parse("Aovo-DRB*W018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W090_01():
    result = parse("Aovo-DRB*W090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB3_06_01():
    result = parse("Aovo-DRB3*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB3*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB3*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W091_01():
    result = parse("Aona-DRB*W091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W029_10():
    result = parse("Aona-DRB*W029:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W029:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W029:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W030_02():
    result = parse("Aona-DRB*W030:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W030:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W030:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_28():
    result = parse("Aona-DRB1*03:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W089_01():
    result = parse("Aona-DRB*W089:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W089:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W089:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_25_01():
    result = parse("Aona-DRB3*06:25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB3*06:25:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_26():
    result = parse("Aona-DRB3*06:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_27():
    result = parse("Aona-DRB3*06:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_25_02():
    result = parse("Aona-DRB3*06:25:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:25:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Aona-DRB3*06:25:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB3_06_28():
    result = parse("Aona-DRB3*06:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB3*06:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB3*06:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W027_09():
    result = parse("Mamu-DRB*W027:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W027:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W027:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB1_03_29():
    result = parse("Aona-DRB1*03:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB1*03:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB1*03:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_18():
    result = parse("Mamu-DRB1*10:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aona_DRB_W018_08():
    result = parse("Aona-DRB*W018:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Aona-DRB*W018:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aona-DRB*W018:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W030_01():
    result = parse("Aovo-DRB*W030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Aovo_DRB_W045_01():
    result = parse("Aovo-DRB*W045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Aovo-DRB*W045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Aovo-DRB*W045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_02_04_01():
    result = parse("Mamu-DPB1*02:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*02:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPB1*02:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_19_04():
    result = parse("Mamu-DPB1*19:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*19:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*19:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_10():
    result = parse("Mamu-DPB1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_19_05():
    result = parse("Mamu-DPB1*19:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*19:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*19:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_02_04_02():
    result = parse("Mamu-DPB1*02:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*02:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPB1*02:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_08():
    result = parse("Mamu-DPA1*07:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_07_02_02():
    result = parse("Mamu-DPB1*07:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*07:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPB1*07:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_07_02():
    result = parse("Mamu-DPA1*02:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*02:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_09_01():
    result = parse("Mamu-DPA1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_04_05():
    result = parse("Mamu-DPA1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_04_06():
    result = parse("Mamu-DPA1*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_16():
    result = parse("Mamu-DPB1*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_09():
    result = parse("Mamu-DPA1*07:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_09():
    result = parse("Mamu-DPB1*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_11():
    result = parse("Mamu-DPB1*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_12():
    result = parse("Mamu-DPB1*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_13():
    result = parse("Mamu-DPB1*01:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_14():
    result = parse("Mamu-DPB1*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_15_02():
    result = parse("Mamu-DPB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_15_03():
    result = parse("Mamu-DPB1*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_19_03():
    result = parse("Mamu-DPB1*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_15():
    result = parse("Mamu-DPB1*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_01_02():
    result = parse("Mane-DPB1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_02_01():
    result = parse("Mane-DPB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_03_01():
    result = parse("Mane-DPB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_04_01():
    result = parse("Mane-DPB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_05_01():
    result = parse("Mane-DPB1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_10_01():
    result = parse("Mane-DPB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_15_01():
    result = parse("Mane-DPB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_21():
    result = parse("Mamu-DPA1*02:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_13():
    result = parse("Mamu-DPA1*02:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_14():
    result = parse("Mamu-DPA1*02:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_15():
    result = parse("Mamu-DPA1*02:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_16():
    result = parse("Mamu-DPA1*02:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_17():
    result = parse("Mamu-DPA1*02:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_18():
    result = parse("Mamu-DPA1*02:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_19():
    result = parse("Mamu-DPA1*02:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_02_20():
    result = parse("Mamu-DPA1*02:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*02:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*02:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_04_03_02():
    result = parse("Mamu-DPA1*04:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*04:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DPA1*04:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_05():
    result = parse("Mamu-DPA1*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_07():
    result = parse("Mamu-DPA1*07:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_06():
    result = parse("Mamu-DPA1*07:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_08_03():
    result = parse("Mamu-DPA1*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_01_01():
    result = parse("Mane-DQA1*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_01_02():
    result = parse("Mane-DQA1*05:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*05:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_02():
    result = parse("Mane-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_03():
    result = parse("Mane-DQA1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_05():
    result = parse("Mane-DQA1*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_06():
    result = parse("Mane-DQA1*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_24_01():
    result = parse("Mane-DQA1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_24_02():
    result = parse("Mane-DQA1*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_26_01():
    result = parse("Mane-DQA1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_10():
    result = parse("Mamu-B*051:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_06():
    result = parse("Mamu-B*060:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_115_02_01_01():
    result = parse("Mamu-B*115:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*115:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*115:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_DRA_01_01_02():
    result = parse("Chsa-DRA*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-DRA*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-DRA*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_10_17():
    result = parse("Mamu-DRB1*10:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*10:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*10:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_20():
    result = parse("Mamu-DRB1*04:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_15():
    result = parse("Mamu-DQB1*17:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_19_02():
    result = parse("Mamu-DQB1*18:19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*18:19:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_15_01_02():
    result = parse("Mamu-DQB1*15:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*15:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*15:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_18_01():
    result = parse("Mamu-DPB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_15_01_01_01():
    result = parse("Poab-B*15:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*15:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*15:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_02_02_01_01():
    result = parse("Poab-B*02:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*02:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*02:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_03_04_01_01():
    result = parse("Popy-B*03:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*03:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*03:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_03_05_01_01():
    result = parse("Popy-B*03:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*03:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*03:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_03_06_01_01():
    result = parse("Popy-B*03:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*03:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*03:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_03_03_01_01():
    result = parse("Poab-B*03:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*03:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*03:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_04_02_01_01():
    result = parse("Poab-B*04:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*04:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*04:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_10_01_01_01():
    result = parse("Popy-B*10:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*10:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*10:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_06_02():
    result = parse("Mamu-DQB1*06:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQB1*06:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_23():
    result = parse("Mamu-DQB1*06:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_24():
    result = parse("Mamu-DQB1*06:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_25():
    result = parse("Mamu-DQB1*06:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_15_06():
    result = parse("Mamu-DQB1*15:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*15:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*15:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_17_14():
    result = parse("Mamu-DQB1*17:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*17:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*17:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_012_02():
    result = parse("Mamu-A1*012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_04_02():
    result = parse("Mamu-A1*019:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*019:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_04_02():
    result = parse("Mamu-A1*028:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*028:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_049_02_02():
    result = parse("Mamu-A1*049:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*049:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*049:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_130_01():
    result = parse("Mamu-A1*130:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*130:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*130:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_01_02_02():
    result = parse("Mamu-A2*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_14():
    result = parse("Mamu-A4*14:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_011_02():
    result = parse("Mamu-B*011:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*011:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*011:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_050_04():
    result = parse("Mamu-B*050:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*050:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*050:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_04():
    result = parse("Mamu-B*072:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*072:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_076_01_02():
    result = parse("Mamu-B*076:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*076:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*076:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_01_01():
    result = parse("Mane-DQA1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_01_02():
    result = parse("Mane-DQA1*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_04():
    result = parse("Mane-DQA1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_05():
    result = parse("Mane-DQA1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_07():
    result = parse("Mane-DQA1*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_24_03():
    result = parse("Mane-DQA1*24:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*24:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*24:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_26_02():
    result = parse("Mane-DQA1*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_26_04():
    result = parse("Mane-DQA1*26:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*26:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*26:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_09():
    result = parse("Mamu-DQA1*26:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_24_04():
    result = parse("Mane-DQA1*24:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*24:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*24:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_19_06():
    result = parse("Mamu-DPB1*19:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*19:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*19:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_A_08_01():
    result = parse("Poab-A*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-A*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Poab-A*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_05_02():
    result = parse("Popy-A*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_A_06_01():
    result = parse("Popy-A*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-A*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-A*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_07():
    result = parse("Popy-B*08:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*08:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_08_04_01_01():
    result = parse("Poab-B*08:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*08:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*08:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_A_09_01():
    result = parse("Poab-A*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-A*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Poab-A*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_08_03_01_01():
    result = parse("Poab-B*08:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*08:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*08:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_06_01_01_01():
    result = parse("Poab-B*06:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*06:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*06:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_06_01():
    result = parse("Gogo-B*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_07_01():
    result = parse("Gogo-B*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_06_01():
    result = parse("Papa-A*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_05_02():
    result = parse("Papa-A*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_08_01():
    result = parse("Papa-A*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_01_01():
    result = parse("Papa-C*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_04_01():
    result = parse("Papa-C*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_02():
    result = parse("Papa-C*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_05_03_01_01():
    result = parse("Popy-B*05:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*05:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*05:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_14_01_01_01():
    result = parse("Popy-B*14:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*14:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*14:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_05_04_01_01():
    result = parse("Popy-B*05:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*05:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*05:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_05_02_01_01():
    result = parse("Poab-B*05:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*05:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*05:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_05_03_01_01():
    result = parse("Poab-B*05:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*05:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*05:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_08_01_01_02():
    result = parse("Poab-B*08:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*08:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*08:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_02_01_02():
    result = parse("Popy-B*08:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Popy-B*08:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_04():
    result = parse("Popy-B*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_05():
    result = parse("Popy-B*08:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*08:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_08_06():
    result = parse("Popy-B*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_03_03_01_02():
    result = parse("Poab-B*03:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*03:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*03:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_05_03_02_01():
    result = parse("Poab-B*05:03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*05:03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*05:03:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_07_01_01_01():
    result = parse("Poab-B*07:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*07:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*07:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Poab_B_08_05_01_01():
    result = parse("Poab-B*08:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Poab-B*08:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Poab-B*08:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPA1_07_10():
    result = parse("Mamu-DPA1*07:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPA1*07:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPA1*07:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRA_01_10():
    result = parse("Mamu-DRA*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRA*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRA*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_05():
    result = parse("Mafa-A1*007:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_07():
    result = parse("Mafa-A1*018:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*018:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_05_02():
    result = parse("Mafa-A1*018:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*018:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_07():
    result = parse("Mafa-A1*022:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_05():
    result = parse("Mafa-A1*043:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_06():
    result = parse("Mafa-A1*043:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W094_01():
    result = parse("Hymo-DRB*W094:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W094:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W094:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W094_02():
    result = parse("Hymo-DRB*W094:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W094:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W094:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W095_01():
    result = parse("Hymo-DRB*W095:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W095:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W095:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W096_01():
    result = parse("Hymo-DRB*W096:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W096:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W096:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W096_02():
    result = parse("Hymo-DRB*W096:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W096:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W096:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB1_04_01():
    result = parse("Hymo-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB1_04_02():
    result = parse("Hymo-DRB1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB1_04_03():
    result = parse("Hymo-DRB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB1_04_04():
    result = parse("Hymo-DRB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W097_01():
    result = parse("Hymo-DRB*W097:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W097:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W097:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W100_01():
    result = parse("Hymo-DRB*W100:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W100:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W100:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W098_01():
    result = parse("Hymo-DRB*W098:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W098:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W098:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W098_02():
    result = parse("Hymo-DRB*W098:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W098:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W098:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W099_01():
    result = parse("Hymo-DRB*W099:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W099:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W099:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_07():
    result = parse("Mafa-A1*043:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_08():
    result = parse("Mafa-A1*043:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_054_02():
    result = parse("Mafa-A1*054:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*054:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*054:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_068_03():
    result = parse("Mafa-A1*068:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*068:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*068:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_076_03():
    result = parse("Mafa-A1*076:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*076:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*076:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_086_04():
    result = parse("Mafa-A1*086:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*086:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*086:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_091_01():
    result = parse("Mafa-A1*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_12():
    result = parse("Mafa-A4*14:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_002_05():
    result = parse("Mafa-B*002:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*002:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*002:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_004_03_02():
    result = parse("Mafa-B*004:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*004:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*004:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_008_02():
    result = parse("Mafa-B*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_008_03():
    result = parse("Mafa-B*008:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*008:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*008:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_04():
    result = parse("Mafa-B*015:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_194_01():
    result = parse("Mafa-B*194:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*194:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*194:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_023_02():
    result = parse("Mafa-B*023:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*023:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*023:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_023_01():
    result = parse("Mafa-B*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_024_01():
    result = parse("Mafa-B*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_028_05():
    result = parse("Mafa-B*028:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*028:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*028:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_01_04():
    result = parse("Mafa-B*030:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_03():
    result = parse("Mafa-B*036:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*036:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_195_01():
    result = parse("Mafa-B*195:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*195:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*195:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_195_02():
    result = parse("Mafa-B*195:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*195:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*195:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_06():
    result = parse("Mafa-B*045:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*045:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_052_01():
    result = parse("Mafa-B*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_055_01():
    result = parse("Mafa-B*055:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*055:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*055:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_10():
    result = parse("Mafa-B*060:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_11():
    result = parse("Mafa-B*060:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_05():
    result = parse("Mafa-B*068:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_02_02():
    result = parse("Mafa-B*079:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*079:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_01_02():
    result = parse("Mafa-B*079:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*079:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_081_02():
    result = parse("Mafa-B*081:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*081:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*081:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_03():
    result = parse("Mafa-B*088:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*088:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_089_01_04():
    result = parse("Mafa-B*089:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*089:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*089:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_097_01():
    result = parse("Mafa-B*097:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*097:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*097:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_100_01_01():
    result = parse("Mafa-B*100:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*100:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*100:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_03():
    result = parse("Mafa-B*101:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*101:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_09():
    result = parse("Mafa-B*109:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_196_01():
    result = parse("Mafa-B*196:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*196:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*196:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_124_02():
    result = parse("Mafa-B*124:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*124:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*124:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_147_04():
    result = parse("Mafa-B*147:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*147:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*147:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_154_01_02():
    result = parse("Mafa-B*154:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*154:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*154:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_07_01_02():
    result = parse("Papa-B*07:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*07:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Papa-B*07:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_27():
    result = parse("Mafa-DPA1*02:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMA_01_01_01():
    result = parse("SLA-DMA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DMA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMA_01_01_02():
    result = parse("SLA-DMA*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMA*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DMA*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMA_02_01():
    result = parse("SLA-DMA*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMA*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DMA*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMA_03_01():
    result = parse("SLA-DMA*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMA*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DMA*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMA_04_01():
    result = parse("SLA-DMA*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMA*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DMA*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_01():
    result = parse("SLA-DQA*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_02():
    result = parse("SLA-DQA*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_03():
    result = parse("SLA-DQA*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_07():
    result = parse("SLA-DQA*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_06():
    result = parse("SLA-DQA*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_01():
    result = parse("SLA-DQA*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_02_01():
    result = parse("SLA-DQA*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQA*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_02_02():
    result = parse("SLA-DQA*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQA*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_03():
    result = parse("SLA-DQA*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_04_01():
    result = parse("SLA-DQA*02:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQA*02:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_06():
    result = parse("SLA-DQA*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_05():
    result = parse("SLA-DQA*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_07():
    result = parse("SLA-DQA*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_03_01():
    result = parse("SLA-DQA*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_03_03():
    result = parse("SLA-DQA*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_03_02():
    result = parse("SLA-DQA*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_04_01():
    result = parse("SLA-DQA*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_04_02():
    result = parse("SLA-DQA*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_06_01():
    result = parse("SLA-DQA*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_05_01():
    result = parse("SLA-DQA*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_01_01():
    result = parse("SLA-DQB1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_01_03():
    result = parse("SLA-DQB1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_01_04():
    result = parse("SLA-DQB1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_01_05():
    result = parse("SLA-DQB1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_01_06():
    result = parse("SLA-DQB1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_01_02():
    result = parse("SLA-DQB1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_01():
    result = parse("SLA-DQB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_02():
    result = parse("SLA-DQB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_03():
    result = parse("SLA-DQB1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_04():
    result = parse("SLA-DQB1*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_07():
    result = parse("SLA-DQB1*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_05():
    result = parse("SLA-DQB1*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_08():
    result = parse("SLA-DQB1*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_09():
    result = parse("SLA-DQB1*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_06():
    result = parse("SLA-DQB1*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_03_01():
    result = parse("SLA-DQB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_03_02():
    result = parse("SLA-DQB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_03_03():
    result = parse("SLA-DQB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_04_01_01():
    result = parse("SLA-DQB1*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_04_01_02():
    result = parse("SLA-DQB1*04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*04:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_04_02_01():
    result = parse("SLA-DQB1*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_04_02_02():
    result = parse("SLA-DQB1*04:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*04:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*04:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_04_04():
    result = parse("SLA-DQB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_04_03():
    result = parse("SLA-DQB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_04_05():
    result = parse("SLA-DQB1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_05_01():
    result = parse("SLA-DQB1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_05_02():
    result = parse("SLA-DQB1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_05_03():
    result = parse("SLA-DQB1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_05_04():
    result = parse("SLA-DQB1*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_06_01():
    result = parse("SLA-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_06_02_01():
    result = parse("SLA-DQB1*06:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*06:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*06:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_07_01_01():
    result = parse("SLA-DQB1*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_08_01():
    result = parse("SLA-DQB1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_08_02():
    result = parse("SLA-DQB1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_08_03():
    result = parse("SLA-DQB1*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_09_01():
    result = parse("SLA-DQB1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_09_02():
    result = parse("SLA-DQB1*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_10_01():
    result = parse("SLA-DQB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_03_04():
    result = parse("SLA-DQB1*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_11_01():
    result = parse("SLA-DQB1*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_06_03():
    result = parse("SLA-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_08_06():
    result = parse("SLA-DQB1*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_10_02():
    result = parse("SLA-DQB1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_01_01_01():
    result = parse("SLA-DRA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_01_01_02():
    result = parse("SLA-DRA*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_01_01_03():
    result = parse("SLA-DRA*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_01_01_04():
    result = parse("SLA-DRA*01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*01:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_01_01():
    result = parse("SLA-DRA*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_01_02():
    result = parse("SLA-DRA*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_02_01():
    result = parse("SLA-DRA*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_02_02():
    result = parse("SLA-DRA*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_02_03():
    result = parse("SLA-DRA*02:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_02_04():
    result = parse("SLA-DRA*02:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_03_01():
    result = parse("SLA-DRA*02:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_02_03_02():
    result = parse("SLA-DRA*02:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*02:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRA*02:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_03_01():
    result = parse("SLA-DRA*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRA*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_01_01():
    result = parse("SLA-DRB1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_01_02():
    result = parse("SLA-DRB1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_01_01():
    result = parse("SLA-DRB1*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_01_02():
    result = parse("SLA-DRB1*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_01_03():
    result = parse("SLA-DRB1*02:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*02:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_02():
    result = parse("SLA-DRB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_03():
    result = parse("SLA-DRB1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_04():
    result = parse("SLA-DRB1*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_05():
    result = parse("SLA-DRB1*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_06():
    result = parse("SLA-DRB1*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_07():
    result = parse("SLA-DRB1*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_08():
    result = parse("SLA-DRB1*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_09():
    result = parse("SLA-DRB1*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_03_01():
    result = parse("SLA-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_01():
    result = parse("SLA-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_02():
    result = parse("SLA-DRB1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_03():
    result = parse("SLA-DRB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_04():
    result = parse("SLA-DRB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_05():
    result = parse("SLA-DRB1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_06():
    result = parse("SLA-DRB1*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_05_01():
    result = parse("SLA-DRB1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_05_02():
    result = parse("SLA-DRB1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_05_03():
    result = parse("SLA-DRB1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_05_04():
    result = parse("SLA-DRB1*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_05_05():
    result = parse("SLA-DRB1*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_05_06():
    result = parse("SLA-DRB1*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_05_07():
    result = parse("SLA-DRB1*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_06_01():
    result = parse("SLA-DRB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_06_02():
    result = parse("SLA-DRB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_06_03():
    result = parse("SLA-DRB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_06_04():
    result = parse("SLA-DRB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_06_05():
    result = parse("SLA-DRB1*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_07_01():
    result = parse("SLA-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_07_02():
    result = parse("SLA-DRB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_07_03():
    result = parse("SLA-DRB1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_08_01():
    result = parse("SLA-DRB1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_08_02():
    result = parse("SLA-DRB1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_08_03():
    result = parse("SLA-DRB1*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_08_04():
    result = parse("SLA-DRB1*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_08_05():
    result = parse("SLA-DRB1*08:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*08:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*08:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_08_06():
    result = parse("SLA-DRB1*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_01_01():
    result = parse("SLA-DRB1*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_01_02():
    result = parse("SLA-DRB1*09:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*09:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_03():
    result = parse("SLA-DRB1*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_02():
    result = parse("SLA-DRB1*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_01_01():
    result = parse("SLA-DRB1*10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*10:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_04():
    result = parse("SLA-DRB1*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_03():
    result = parse("SLA-DRB1*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_05():
    result = parse("SLA-DRB1*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_02():
    result = parse("SLA-DRB1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_01():
    result = parse("SLA-DRB1*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_02():
    result = parse("SLA-DRB1*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_03():
    result = parse("SLA-DRB1*11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_04():
    result = parse("SLA-DRB1*11:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_05():
    result = parse("SLA-DRB1*11:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_06():
    result = parse("SLA-DRB1*11:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_13_01():
    result = parse("SLA-DRB1*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_14_01():
    result = parse("SLA-DRB1*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_13_02():
    result = parse("SLA-DRB1*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_05():
    result = parse("SLA-DRB1*09:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*09:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_06():
    result = parse("SLA-DRB1*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_01_03():
    result = parse("SLA-DRB1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_01_04():
    result = parse("SLA-DRB1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_03_02():
    result = parse("SLA-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_16_01():
    result = parse("SLA-DRB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_16_02():
    result = parse("SLA-DRB1*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_14_02():
    result = parse("SLA-DRB1*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_15_01():
    result = parse("SLA-DRB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_17_01():
    result = parse("SLA-DRB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_15_02N():
    result = parse("SLA-DRB1*15:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*15:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*15:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_15_03N():
    result = parse("SLA-DRB1*15:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*15:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*15:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_06():
    result = parse("SLA-DRB1*09:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*09:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_07():
    result = parse("SLA-DRB1*10:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*10:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_07():
    result = parse("SLA-DRB1*09:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*09:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_08():
    result = parse("SLA-DRB1*10:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*10:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_07_04():
    result = parse("SLA-DRB1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_01_05():
    result = parse("SLA-DRB1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_16_03():
    result = parse("SLA-DRB1*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_09_04():
    result = parse("SLA-DRB1*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_12_02():
    result = parse("SLA-DRB1*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_12_03():
    result = parse("SLA-DRB1*12:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*12:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*12:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_12_01_02():
    result = parse("SLA-DRB1*12:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*12:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*12:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_01_01():
    result = parse("SLA-DRB2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_02_01():
    result = parse("SLA-DRB2*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_03_01():
    result = parse("SLA-DRB2*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_05_01():
    result = parse("SLA-DRB2*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_06_01():
    result = parse("SLA-DRB2*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_07_01():
    result = parse("SLA-DRB2*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_04_01():
    result = parse("SLA-DRB2*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_08_01():
    result = parse("SLA-DRB2*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_09_01():
    result = parse("SLA-DRB2*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB3_01_01():
    result = parse("SLA-DRB3*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB3*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB3*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB3_02_01():
    result = parse("SLA-DRB3*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB3*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB3*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB3_03_01():
    result = parse("SLA-DRB3*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB3*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB3*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB3_04_01():
    result = parse("SLA-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB3_05_01():
    result = parse("SLA-DRB3*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB3*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB3*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB4_01_01():
    result = parse("SLA-DRB4*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB4*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB4*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB5_01_01():
    result = parse("SLA-DRB5*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB5*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB5*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_01_01():
    result = parse("SLA-1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_01_02():
    result = parse("SLA-1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_01_03():
    result = parse("SLA-1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_02_01():
    result = parse("SLA-1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_02_02():
    result = parse("SLA-1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_02_03():
    result = parse("SLA-1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_04_01_01():
    result = parse("SLA-1*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-1*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_04_03():
    result = parse("SLA-1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_04_04():
    result = parse("SLA-1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_04_05():
    result = parse("SLA-1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_05_01():
    result = parse("SLA-1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_06_01():
    result = parse("SLA-1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_06_02():
    result = parse("SLA-1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_01():
    result = parse("SLA-1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_02():
    result = parse("SLA-1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_01():
    result = parse("SLA-1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_02():
    result = parse("SLA-1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_12():
    result = parse("SLA-1*08:12")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_13():
    result = parse("SLA-1*08:13")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_07():
    result = parse("SLA-1*08:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_08():
    result = parse("SLA-1*08:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_03():
    result = parse("SLA-1*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_05():
    result = parse("SLA-1*08:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_01_01():
    result = parse("SLA-1*11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-1*11:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_02():
    result = parse("SLA-1*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_03():
    result = parse("SLA-1*11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_04():
    result = parse("SLA-1*11:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_06():
    result = parse("SLA-1*11:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_12_01():
    result = parse("SLA-1*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_12_03():
    result = parse("SLA-1*12:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*12:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*12:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_12_02():
    result = parse("SLA-1*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_13_01():
    result = parse("SLA-1*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_13_02():
    result = parse("SLA-1*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_16_01():
    result = parse("SLA-1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_16_02():
    result = parse("SLA-1*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_17_01():
    result = parse("SLA-1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_14_02():
    result = parse("SLA-1*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_14_01():
    result = parse("SLA-1*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_03():
    result = parse("SLA-1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_18_01():
    result = parse("SLA-1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_05():
    result = parse("SLA-1*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_09_02():
    result = parse("SLA-1*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_10_02():
    result = parse("SLA-1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_10_01():
    result = parse("SLA-1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_02_02():
    result = parse("SLA-2*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_03_01():
    result = parse("SLA-2*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_03_02():
    result = parse("SLA-2*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_03_03():
    result = parse("SLA-2*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_01():
    result = parse("SLA-2*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_02_01():
    result = parse("SLA-2*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-2*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_02_02():
    result = parse("SLA-2*04:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-2*04:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_01():
    result = parse("SLA-2*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_02():
    result = parse("SLA-2*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_05():
    result = parse("SLA-2*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_03():
    result = parse("SLA-2*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_06():
    result = parse("SLA-2*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_04():
    result = parse("SLA-2*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_01():
    result = parse("SLA-2*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_03():
    result = parse("SLA-2*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_06():
    result = parse("SLA-2*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_02_01():
    result = parse("SLA-2*06:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-2*06:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_08():
    result = parse("SLA-2*06:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_07_01():
    result = parse("SLA-2*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_07_02():
    result = parse("SLA-2*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_07_03():
    result = parse("SLA-2*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_01():
    result = parse("SLA-2*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_02():
    result = parse("SLA-2*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_04():
    result = parse("SLA-2*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_05():
    result = parse("SLA-2*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_03():
    result = parse("SLA-2*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_11_01_01():
    result = parse("SLA-2*11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-2*11:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_11_01_02():
    result = parse("SLA-2*11:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*11:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-2*11:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_11_03():
    result = parse("SLA-2*11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*11:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_12_01():
    result = parse("SLA-2*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_12_02():
    result = parse("SLA-2*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_17_01():
    result = parse("SLA-2*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_11_04():
    result = parse("SLA-2*11:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*11:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*11:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_15_01():
    result = parse("SLA-2*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_16_02():
    result = parse("SLA-2*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_08_05():
    result = parse("SLA-2*08:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*08:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*08:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_08_02():
    result = parse("SLA-2*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_09_01():
    result = parse("SLA-2*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_09_03():
    result = parse("SLA-2*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_09_02():
    result = parse("SLA-2*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_13_01():
    result = parse("SLA-2*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_14_01():
    result = parse("SLA-2*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_01_01():
    result = parse("SLA-3*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_01_04():
    result = parse("SLA-3*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_01_03():
    result = parse("SLA-3*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_01_02():
    result = parse("SLA-3*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_01():
    result = parse("SLA-3*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_02():
    result = parse("SLA-3*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_03():
    result = parse("SLA-3*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_04():
    result = parse("SLA-3*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_05():
    result = parse("SLA-3*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_08():
    result = parse("SLA-3*03:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_09():
    result = parse("SLA-3*03:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_06():
    result = parse("SLA-3*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_01():
    result = parse("SLA-3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_02():
    result = parse("SLA-3*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_03():
    result = parse("SLA-3*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_01():
    result = parse("SLA-3*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_02():
    result = parse("SLA-3*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_03_01():
    result = parse("SLA-3*05:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-3*05:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_05():
    result = parse("SLA-3*05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*05:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_06_01():
    result = parse("SLA-3*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_06_02():
    result = parse("SLA-3*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_07_01_01():
    result = parse("SLA-3*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-3*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_07_01_02():
    result = parse("SLA-3*07:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*07:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-3*07:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_07_02():
    result = parse("SLA-3*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_07_03():
    result = parse("SLA-3*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_08_01():
    result = parse("SLA-3*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_01_01():
    result = parse("SLA-6*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_02_01():
    result = parse("SLA-6*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_03_01():
    result = parse("SLA-6*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_04_01():
    result = parse("SLA-6*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_05_01():
    result = parse("SLA-6*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_06_01():
    result = parse("SLA-6*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_07_01():
    result = parse("SLA-6*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_08_01():
    result = parse("SLA-6*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_09_01():
    result = parse("SLA-6*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_16_01():
    result = parse("SLA-2*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_08_03():
    result = parse("SLA-2*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_24_01():
    result = parse("Mamu-DPB1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_082_01_01():
    result = parse("Mane-A1*082:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*082:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*082:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_01_01():
    result = parse("Mafa-B*041:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*041:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_38_02():
    result = parse("Mamu-I*01:38:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:38:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:38:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_38_01():
    result = parse("Mamu-I*01:38:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:38:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:38:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_01_04():
    result = parse("Mamu-I*01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_37():
    result = parse("Mamu-I*01:37")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:37") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:37") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_32():
    result = parse("Mamu-I*01:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_03_01():
    result = parse("Mafa-B*098:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_01_01():
    result = parse("Mamu-I*01:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_140_01_01():
    result = parse("Mafa-B*140:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*140:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*140:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_146_01_01():
    result = parse("Mafa-B*146:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*146:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*146:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_001_01_01_01():
    result = parse("Mafa-B*001:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*001:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*001:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_01_01():
    result = parse("Mafa-B*050:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*050:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_03_01():
    result = parse("Mane-A4*14:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A4*14:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_14N():
    result = parse("Mafa-A3*13:14N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:14N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:14N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_06_02():
    result = parse("Mane-DQB1*17:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*17:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_03_01():
    result = parse("Mane-DQB1*17:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*17:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_14():
    result = parse("Mane-DQB1*06:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_12_01():
    result = parse("Mane-DQB1*06:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*06:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_07_01():
    result = parse("Mane-DRB1*03:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB1*03:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_40():
    result = parse("Mafa-DQB1*06:40")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:40") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:40") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_24_01():
    result = parse("Mafa-DQB1*06:24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:24:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_06_01():
    result = parse("Mamu-B*082:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*082:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_110_01_01_01():
    result = parse("Mafa-B*110:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*110:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*110:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_091_03():
    result = parse("Mafa-B*091:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*091:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*091:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_07():
    result = parse("Mafa-B*069:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_073_02():
    result = parse("Mafa-B*073:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*073:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*073:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_056_05_01_01():
    result = parse("Mafa-B*056:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*056:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*056:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_17_01_01():
    result = parse("Mafa-DPB1*17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*17:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_04_01():
    result = parse("Mafa-DPA1*07:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*07:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_13_02():
    result = parse("Mafa-DPA1*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_04_01():
    result = parse("Mafa-DPB1*02:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*02:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_09_01():
    result = parse("Mafa-DQA1*01:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_06_01():
    result = parse("Mafa-B*098:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_02_01():
    result = parse("Mane-DRB1*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB1*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_06():
    result = parse("Mafa-A1*007:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_010_03():
    result = parse("Mane-A1*010:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*010:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*010:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_038_01():
    result = parse("Mane-A1*038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_047_01():
    result = parse("Mane-A1*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_053_01():
    result = parse("Mane-A1*053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_061_01():
    result = parse("Mane-A1*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_063_01():
    result = parse("Mane-A1*063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_063_03():
    result = parse("Mane-A1*063:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*063:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*063:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_063_02_02():
    result = parse("Mane-A1*063:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*063:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*063:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_069_01():
    result = parse("Mane-A1*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_072_04():
    result = parse("Mane-A1*072:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*072:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*072:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_086_01N():
    result = parse("Mane-A1*086:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*086:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*086:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_01_01():
    result = parse("Mane-A2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_09():
    result = parse("Mane-A3*13:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_13():
    result = parse("Mane-A3*13:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_01_02():
    result = parse("Mane-A4*14:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A4*14:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A6_01_01_02():
    result = parse("Mane-A6*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A6*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A6*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_004_02():
    result = parse("Mane-B*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_008_02():
    result = parse("Mane-B*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_017_05():
    result = parse("Mane-B*017:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*017:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*017:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_031_01():
    result = parse("Mane-B*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_041_05():
    result = parse("Mane-B*041:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*041:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*041:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_056_02():
    result = parse("Mane-B*056:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*056:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*056:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_056_03():
    result = parse("Mane-B*056:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*056:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*056:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_100_01_01():
    result = parse("Mane-B*100:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*100:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*100:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_03_01():
    result = parse("Mane-B*060:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*060:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_05():
    result = parse("Mane-B*060:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*060:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_08():
    result = parse("Mane-B*068:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_069_02():
    result = parse("Mane-B*069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_078_04():
    result = parse("Mane-B*078:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*078:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*078:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_080_01():
    result = parse("Mane-B*080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_081_01_01():
    result = parse("Mane-B*081:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*081:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*081:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_200_01():
    result = parse("Mane-B*200:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*200:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*200:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_125_02():
    result = parse("Mane-B*125:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*125:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*125:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_150_01():
    result = parse("Mane-B*150:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*150:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*150:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_14_01():
    result = parse("Mane-I*01:14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:14:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_05():
    result = parse("Mane-A4*14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_03_02():
    result = parse("Mane-A4*14:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A4*14:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_07_01():
    result = parse("Mane-B*068:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*068:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_088_02_01():
    result = parse("Mane-B*088:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*088:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*088:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_003_03():
    result = parse("Mane-A1*003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_03_01_01():
    result = parse("Chsa-B*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_178_03():
    result = parse("Mane-B*178:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*178:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*178:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_57_01():
    result = parse("Mafa-A2*05:57:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:57:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:57:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_022_01_01():
    result = parse("Mafa-B*022:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*022:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*022:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_161_02_01():
    result = parse("Mafa-B*161:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*161:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*161:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_184_01_01_01():
    result = parse("Mafa-B*184:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*184:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*184:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_17_01():
    result = parse("Mafa-A3*13:17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A3*13:17:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W033_01_01():
    result = parse("Mane-DRB*W033:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W033:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB*W033:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_42():
    result = parse("Mafa-DPA1*02:42")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:42") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:42") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_43():
    result = parse("Mafa-DPA1*02:43")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:43") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:43") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_10_03_01():
    result = parse("Mafa-DPA1*10:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*10:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*10:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_16_01():
    result = parse("Mafa-DQA1*01:16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:16:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_02_01_01():
    result = parse("Mamu-B*098:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_117_01_01():
    result = parse("Mamu-B*117:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*117:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*117:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_08_01():
    result = parse("Mamu-DQA1*26:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*26:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_07():
    result = parse("Mane-DRA*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRA*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_074_03_01_01():
    result = parse("Mamu-B*074:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*074:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*074:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_06_01_01():
    result = parse("Mamu-B*098:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_06():
    result = parse("Caja-G*07:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*07:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_06_01():
    result = parse("Mane-DQB1*17:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*17:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_01_01_01():
    result = parse("Mane-DPB1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DPB1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_05_01():
    result = parse("Mamu-B*072:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*072:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_06():
    result = parse("Mane-DQA1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_26_03_01():
    result = parse("Mane-DQA1*26:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*26:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*26:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_08_01():
    result = parse("Mane-DPB1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_19_01():
    result = parse("Mane-DPB1*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_19_02():
    result = parse("Mane-DPB1*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_21_01():
    result = parse("Mane-DPB1*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_21_01():
    result = parse("Mamu-DPB1*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_21_02():
    result = parse("Mamu-DPB1*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_23_01():
    result = parse("Mamu-DPB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_23_02():
    result = parse("Mamu-DPB1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_21_03():
    result = parse("Mamu-DPB1*21:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*21:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*21:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_01_17():
    result = parse("Mamu-DPB1*01:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*01:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*01:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_02_02():
    result = parse("Mamu-DQA1*26:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*26:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_08():
    result = parse("Mane-DPA1*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_09_01():
    result = parse("Mane-DPA1*02:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DPA1*02:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_09_02():
    result = parse("Mane-DPA1*02:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DPA1*02:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_09_01():
    result = parse("Mane-DPA1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_02_02():
    result = parse("Mane-DPB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_04_02():
    result = parse("Mane-DPB1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_02_01():
    result = parse("Mane-DQA1*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_07():
    result = parse("Mane-DQA1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_02_02():
    result = parse("Mane-DQA1*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_08():
    result = parse("Mane-DQA1*05:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_04_01():
    result = parse("Mane-DQA1*05:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*05:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_09():
    result = parse("Mane-DQA1*05:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_04_02():
    result = parse("Mane-DQA1*05:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*05:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_23_01():
    result = parse("Mane-DQA1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_24_05():
    result = parse("Mane-DQA1*24:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*24:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*24:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_15_01_01():
    result = parse("Mane-DQB1*15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*15:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_15_01_02():
    result = parse("Mane-DQB1*15:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*15:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*15:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_08():
    result = parse("Mane-DQB1*17:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*17:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_09():
    result = parse("Mane-DQB1*17:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*17:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_09():
    result = parse("Mane-DQB1*18:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_04_02():
    result = parse("Mane-DRA*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_02_03():
    result = parse("Mane-DRA*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_02_04():
    result = parse("Mane-DRA*01:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_02_01_03():
    result = parse("Mane-DRA*02:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*02:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*02:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_02_01_04():
    result = parse("Mane-DRA*02:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*02:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*02:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_03():
    result = parse("Mane-DRB*W001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_05():
    result = parse("Mane-DRB*W001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_06():
    result = parse("Mane-DRB*W001:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_10():
    result = parse("Mane-DRB*W001:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_12_01():
    result = parse("Mane-DRB*W001:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB*W001:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_12_02():
    result = parse("Mane-DRB*W001:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB*W001:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W020_01():
    result = parse("Mane-DRB*W020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W021_01():
    result = parse("Mane-DRB*W021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W021_03():
    result = parse("Mane-DRB*W021:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W021:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W021:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W021_04():
    result = parse("Mane-DRB*W021:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W021:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W021:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W025_03():
    result = parse("Mane-DRB*W025:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W025:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W025:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W025_04():
    result = parse("Mane-DRB*W025:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W025:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W025:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W025_05():
    result = parse("Mane-DRB*W025:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W025:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W025:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W027_01():
    result = parse("Mane-DRB*W027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W027_02():
    result = parse("Mane-DRB*W027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_06_01():
    result = parse("Mafa-A1*060:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*060:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W027_03():
    result = parse("Mane-DRB*W027:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W027:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W027:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_06():
    result = parse("Mane-DRB*W002:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_07():
    result = parse("Mane-DRB*W002:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_10():
    result = parse("Mane-DRB*W002:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_12():
    result = parse("Mane-DRB*W002:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_13():
    result = parse("Mane-DRB*W002:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W033_01_02():
    result = parse("Mane-DRB*W033:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W033:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB*W033:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W003_02():
    result = parse("Mane-DRB*W003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W003_05():
    result = parse("Mane-DRB*W003:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W003:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W003:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W053_01():
    result = parse("Mane-DRB*W053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W005_01():
    result = parse("Mane-DRB*W005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W005_03():
    result = parse("Mane-DRB*W005:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W005:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W005:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W069_02():
    result = parse("Mane-DRB*W069:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W069:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W069:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W006_02():
    result = parse("Mane-DRB*W006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W074_01_01():
    result = parse("Mane-DRB*W074:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W074:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB*W074:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W007_01():
    result = parse("Mane-DRB*W007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_027_03_01_01():
    result = parse("Mafa-B*027:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*027:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*027:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_16():
    result = parse("Mane-DRB1*03:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_04():
    result = parse("Mane-DRB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W074_02():
    result = parse("Mane-DRB*W074:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W074:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W074:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_10_02_02():
    result = parse("Mane-DRB1*10:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*10:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB1*10:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_10_02_01():
    result = parse("Mane-DRB1*10:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*10:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB1*10:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_09_01():
    result = parse("Mafa-B*060:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*060:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_25_04():
    result = parse("Mafa-DPB1*25:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*25:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*25:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_09():
    result = parse("Mafa-DPB1*15:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_06_01_01():
    result = parse("Mafa-B*068:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*068:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_17_01_02():
    result = parse("Mafa-DPB1*17:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*17:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*17:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_21_02():
    result = parse("Mafa-DPB1*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_04_01():
    result = parse("Mafa-B*115:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*115:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_35():
    result = parse("Mafa-DPA1*02:35")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:35") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:35") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_08():
    result = parse("Mafa-DPB1*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_09():
    result = parse("Mafa-DPB1*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_10():
    result = parse("Mafa-DPB1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_06():
    result = parse("Mafa-DPB1*02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*02:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_23_01():
    result = parse("Mafa-I*01:23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:23:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_07_02():
    result = parse("Mafa-DPB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_09_04():
    result = parse("Mafa-DPB1*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_24():
    result = parse("Mafa-I*01:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_23_01():
    result = parse("Mafa-DPB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_25():
    result = parse("Mafa-I*01:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_26_01():
    result = parse("Mafa-I*01:26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:26:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_18_03_01():
    result = parse("Mafa-DPB1*18:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*18:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*18:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_10():
    result = parse("Mafa-DQA1*05:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_03_02():
    result = parse("Mafa-I*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_17_01():
    result = parse("Mane-DPB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_23_01():
    result = parse("Mane-DPB1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_189_01():
    result = parse("Mafa-B*189:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*189:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*189:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_123_02():
    result = parse("Mamu-A1*123:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*123:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*123:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_12_01():
    result = parse("Mane-DPA1*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_06_02():
    result = parse("Mafa-DPB1*19:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*19:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_01_01():
    result = parse("Ceat-A1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_02_01():
    result = parse("Ceat-A1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_01():
    result = parse("Ceat-Aw1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_02():
    result = parse("Ceat-Aw1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_03():
    result = parse("Ceat-Aw1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_04():
    result = parse("Ceat-Aw1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_05():
    result = parse("Ceat-Aw1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_06():
    result = parse("Ceat-Aw1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_07():
    result = parse("Ceat-Aw1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw1_01_08():
    result = parse("Ceat-Aw1*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw1*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw1*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_03_01():
    result = parse("Ceat-A1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw2_01_01():
    result = parse("Ceat-Aw2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw2_01_02():
    result = parse("Ceat-Aw2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_04_01():
    result = parse("Ceat-A1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_28():
    result = parse("Mafa-DPA1*02:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_05_01():
    result = parse("Ceat-A1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_29():
    result = parse("Mafa-DPA1*02:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_06_01():
    result = parse("Ceat-A1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_06_04():
    result = parse("Mafa-DPA1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_07_01():
    result = parse("Ceat-A1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_30_01():
    result = parse("Mafa-DPA1*02:30:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:30:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:30:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw2_03_01():
    result = parse("Ceat-Aw2*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw2*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw2*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_10_05():
    result = parse("Mafa-DPA1*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw2_02_01():
    result = parse("Ceat-Aw2*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw2*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw2*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_31():
    result = parse("Mafa-DPA1*02:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_08_01():
    result = parse("Ceat-A1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_32_01():
    result = parse("Mafa-DPA1*02:32:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:32:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:32:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_09_01():
    result = parse("Ceat-A1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_33():
    result = parse("Mafa-DPA1*02:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_10_01():
    result = parse("Ceat-A1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_34():
    result = parse("Mafa-DPA1*02:34")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:34") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:34") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw2_02_02():
    result = parse("Ceat-Aw2*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw2*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw2*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_14():
    result = parse("Mafa-DPA1*07:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_Aw2_02_03():
    result = parse("Ceat-Aw2*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-Aw2*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-Aw2*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_11_01():
    result = parse("Ceat-A1*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_36():
    result = parse("Mafa-DPA1*02:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_12_01():
    result = parse("Ceat-A1*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_13_01():
    result = parse("Ceat-A1*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_37():
    result = parse("Mafa-DPA1*02:37")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:37") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:37") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_14_01():
    result = parse("Ceat-A1*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_11_02():
    result = parse("Ceat-A1*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_15_01():
    result = parse("Ceat-A1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_04_02():
    result = parse("Mafa-DPA1*07:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*07:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_16_01():
    result = parse("Ceat-A1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_A1_17_01():
    result = parse("Ceat-A1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-A1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-A1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_01_01Sp():
    result = parse("Ceat-B*01:01Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*01:01Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*01:01Sp") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_02_01():
    result = parse("Ceat-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_10_03_02():
    result = parse("Mafa-DPA1*10:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*10:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*10:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_03_01():
    result = parse("Ceat-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_38():
    result = parse("Mafa-DPA1*02:38")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:38") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:38") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_03_02():
    result = parse("Ceat-B*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_39():
    result = parse("Mafa-DPA1*02:39")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:39") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:39") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_04_01():
    result = parse("Ceat-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_40():
    result = parse("Mafa-DPA1*02:40")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:40") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:40") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_05_01():
    result = parse("Ceat-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_08_02():
    result = parse("Mafa-DPA1*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_04_02():
    result = parse("Ceat-B*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_06_01():
    result = parse("Ceat-B*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_06_02():
    result = parse("Ceat-B*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_07_01():
    result = parse("Ceat-B*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_08_01():
    result = parse("Ceat-B*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_09_01():
    result = parse("Ceat-B*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_09_02():
    result = parse("Ceat-B*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_10_01():
    result = parse("Ceat-B*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_10_02():
    result = parse("Ceat-B*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_10_03():
    result = parse("Ceat-B*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_11_01():
    result = parse("Ceat-B*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_11_02():
    result = parse("Ceat-B*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_12_01():
    result = parse("Ceat-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_13_01N():
    result = parse("Ceat-B*13:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*13:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*13:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_14_01N():
    result = parse("Ceat-B*14:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*14:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*14:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_13_02N():
    result = parse("Ceat-B*13:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*13:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*13:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_15_01():
    result = parse("Ceat-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_16_01():
    result = parse("Ceat-B*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_16_02():
    result = parse("Ceat-B*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_17_02():
    result = parse("Ceat-B*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_17_01_01():
    result = parse("Ceat-B*17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ceat-B*17:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_17_03():
    result = parse("Ceat-B*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_17_01_02():
    result = parse("Ceat-B*17:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*17:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ceat-B*17:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_18_01():
    result = parse("Ceat-B*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_18_02():
    result = parse("Ceat-B*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_19_01():
    result = parse("Ceat-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_20_01N():
    result = parse("Ceat-B*20:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*20:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*20:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_21_01():
    result = parse("Ceat-B*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_22_01():
    result = parse("Ceat-B*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_23_01():
    result = parse("Ceat-B*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_23_02():
    result = parse("Ceat-B*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_24_01():
    result = parse("Ceat-B*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_25_01():
    result = parse("Ceat-B*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_21_03():
    result = parse("Mafa-DPB1*21:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*21:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*21:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_26_01():
    result = parse("Ceat-B*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_03_04():
    result = parse("Mafa-DQA1*01:03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:03:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_27_01():
    result = parse("Ceat-B*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_06_02():
    result = parse("Mafa-DQA1*01:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_27_02():
    result = parse("Ceat-B*27:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*27:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*27:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_09_02():
    result = parse("Mafa-DQA1*01:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_27_03():
    result = parse("Ceat-B*27:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*27:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*27:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_27_04():
    result = parse("Ceat-B*27:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*27:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*27:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_27_05():
    result = parse("Ceat-B*27:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*27:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*27:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_22():
    result = parse("Mafa-DQA1*01:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_28_01N():
    result = parse("Ceat-B*28:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*28:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*28:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_23():
    result = parse("Mafa-DQA1*01:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_29_01():
    result = parse("Ceat-B*29:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*29:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*29:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_05_03():
    result = parse("Mafa-DQA1*05:05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:05:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_05_02():
    result = parse("Ceat-B*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_11():
    result = parse("Mafa-DQA1*05:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_30_01():
    result = parse("Ceat-B*30:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*30:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*30:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_12():
    result = parse("Mafa-DQA1*05:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_31_01():
    result = parse("Ceat-B*31:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*31:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*31:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_13():
    result = parse("Mafa-DQA1*05:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_31_02():
    result = parse("Ceat-B*31:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*31:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*31:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_23_01_02():
    result = parse("Mafa-DQA1*23:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*23:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*23:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_31_03():
    result = parse("Ceat-B*31:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*31:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*31:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_23_02():
    result = parse("Mafa-DQA1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_31_04():
    result = parse("Ceat-B*31:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*31:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*31:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_32_01():
    result = parse("Ceat-B*32:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*32:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*32:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_08():
    result = parse("Mafa-DQA1*26:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_32_02():
    result = parse("Ceat-B*32:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*32:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*32:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_09():
    result = parse("Mafa-DQA1*26:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_33_01():
    result = parse("Ceat-B*33:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*33:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*33:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_10():
    result = parse("Mafa-DQA1*26:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_34_01():
    result = parse("Ceat-B*34:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*34:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*34:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_35_01():
    result = parse("Ceat-B*35:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*35:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*35:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_35_02():
    result = parse("Ceat-B*35:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*35:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*35:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_11():
    result = parse("Mafa-DQA1*26:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_35_03():
    result = parse("Ceat-B*35:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*35:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*35:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_36_01N():
    result = parse("Ceat-B*36:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*36:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*36:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_37_01():
    result = parse("Ceat-B*37:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*37:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*37:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_38_01():
    result = parse("Ceat-B*38:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*38:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*38:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_10():
    result = parse("Mamu-A1*018:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_38_02():
    result = parse("Ceat-B*38:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*38:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*38:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_39_01():
    result = parse("Ceat-B*39:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*39:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*39:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_40_01():
    result = parse("Ceat-B*40:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*40:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*40:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_40_02():
    result = parse("Ceat-B*40:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*40:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*40:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_41_01():
    result = parse("Ceat-B*41:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*41:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*41:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_37():
    result = parse("Mafa-DQB1*06:37")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:37") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:37") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_42_01():
    result = parse("Ceat-B*42:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*42:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*42:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_38():
    result = parse("Mafa-DQB1*06:38")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:38") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:38") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_43_01Sp():
    result = parse("Ceat-B*43:01Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*43:01Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*43:01Sp") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_39():
    result = parse("Mafa-DQB1*06:39")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:39") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:39") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_15_02():
    result = parse("Ceat-B*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_44_01N():
    result = parse("Ceat-B*44:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*44:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*44:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_45_01N():
    result = parse("Ceat-B*45:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*45:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*45:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_45_02():
    result = parse("Ceat-B*45:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*45:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*45:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_10():
    result = parse("Mafa-DQB1*17:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_46_01():
    result = parse("Ceat-B*46:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*46:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*46:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_47_01():
    result = parse("Ceat-B*47:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*47:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*47:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_48_01():
    result = parse("Ceat-B*48:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*48:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*48:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_49_01():
    result = parse("Ceat-B*49:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*49:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*49:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_29():
    result = parse("Mafa-DQB1*18:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_50_01():
    result = parse("Ceat-B*50:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*50:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*50:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_51_01():
    result = parse("Ceat-B*51:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*51:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*51:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_52_01():
    result = parse("Ceat-B*52:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*52:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*52:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_15_03():
    result = parse("Ceat-B*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_06_03():
    result = parse("Ceat-B*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_53_01():
    result = parse("Ceat-B*53:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*53:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*53:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_54_01():
    result = parse("Ceat-B*54:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*54:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*54:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_55_01():
    result = parse("Ceat-B*55:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*55:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*55:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_49_02():
    result = parse("Ceat-B*49:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*49:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*49:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_56_01():
    result = parse("Ceat-B*56:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*56:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*56:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_57_01():
    result = parse("Ceat-B*57:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*57:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*57:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_06_04():
    result = parse("Ceat-B*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_58_01():
    result = parse("Ceat-B*58:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*58:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*58:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_11():
    result = parse("Mamu-A1*018:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_51_02():
    result = parse("Ceat-B*51:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*51:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*51:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_40_03():
    result = parse("Ceat-B*40:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*40:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*40:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_59_01():
    result = parse("Ceat-B*59:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*59:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*59:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_60_01():
    result = parse("Ceat-B*60:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*60:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*60:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_022_06():
    result = parse("Mamu-A1*022:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*022:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*022:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_F_01_01():
    result = parse("Ceat-F*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-F*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-F*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_F_01_02():
    result = parse("Ceat-F*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-F*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-F*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_028_08():
    result = parse("Mamu-A1*028:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*028:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*028:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_F_01_03():
    result = parse("Ceat-F*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-F*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-F*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_F_01_04():
    result = parse("Ceat-F*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-F*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-F*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_01_01():
    result = parse("Ceat-I*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_01_02():
    result = parse("Ceat-I*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_01_03():
    result = parse("Ceat-I*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_02_01():
    result = parse("Ceat-I*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_02_02():
    result = parse("Ceat-I*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_02_03():
    result = parse("Ceat-I*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_02_04():
    result = parse("Ceat-I*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_02_05():
    result = parse("Ceat-I*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_147_02():
    result = parse("Mamu-B*147:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*147:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*147:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_I_03_01():
    result = parse("Ceat-I*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-I*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-I*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_02_02():
    result = parse("Mamu-B*007:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*007:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_B_61_01():
    result = parse("Ceat-B*61:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-B*61:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-B*61:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_05():
    result = parse("Mamu-B*017:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*017:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_11():
    result = parse("Mafa-DPB1*15:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_034_05():
    result = parse("Mamu-B*034:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*034:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*034:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_08():
    result = parse("Mafa-A1*022:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_130_01_01_01():
    result = parse("Mafa-A1*130:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*130:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*130:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_09_01():
    result = parse("Mafa-A1*022:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*022:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_11():
    result = parse("Mafa-DPB1*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_09():
    result = parse("Mafa-A1*043:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_04_02():
    result = parse("Mafa-DPB1*02:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*02:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_10():
    result = parse("Mafa-A1*043:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_10():
    result = parse("Mafa-DPB1*15:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_056_03_01():
    result = parse("Mafa-A1*056:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*056:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*056:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_01_11():
    result = parse("Mafa-DRA*01:01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:01:11") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_057_02():
    result = parse("Mafa-A1*057:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*057:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*057:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_25():
    result = parse("Mafa-DRA*01:02:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:25") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_26():
    result = parse("Mafa-DRA*01:02:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:26") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_10_03():
    result = parse("Mafa-DRA*01:10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:10:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_02_01():
    result = parse("Mafa-A1*090:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*090:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_13_01():
    result = parse("Mafa-DRA*01:13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:13:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_092_04():
    result = parse("Mafa-A1*092:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*092:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*092:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_24():
    result = parse("Mafa-DRA*01:02:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:24") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_55():
    result = parse("Mafa-A2*05:55")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:55") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:55") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_19_01_01():
    result = parse("Mafa-A3*13:19:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:19:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A3*13:19:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_13():
    result = parse("Mafa-A4*14:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_044_08():
    result = parse("Mamu-B*044:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*044:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*044:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_06_01_01():
    result = parse("Mafa-A5*30:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A5*30:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_06_01():
    result = parse("Mafa-B*007:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*007:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_12():
    result = parse("Mafa-B*013:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_06():
    result = parse("Mamu-B*056:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*056:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_021_02():
    result = parse("Mafa-B*021:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*021:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*021:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_023_03():
    result = parse("Mafa-B*023:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*023:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*023:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_07_01_01():
    result = parse("Mamu-B*060:07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:07:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_15_01():
    result = parse("Mafa-B*030:15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:15:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_01_02():
    result = parse("Mamu-B*063:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*063:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_034_04_01_01():
    result = parse("Mafa-B*034:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*034:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*034:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_04():
    result = parse("Mafa-B*036:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*036:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_038_01_01():
    result = parse("Mafa-B*038:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*038:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*038:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_02_01_01():
    result = parse("Mafa-B*041:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*041:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_03():
    result = parse("Mafa-B*041:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*041:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_06():
    result = parse("Mafa-B*050:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*050:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_02_02():
    result = parse("Mamu-B*068:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*068:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_056_04():
    result = parse("Mafa-B*056:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*056:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*056:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_061_03_01_01():
    result = parse("Mafa-B*061:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*061:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*061:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_061_04_01_01():
    result = parse("Mafa-B*061:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*061:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*061:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_08():
    result = parse("Mamu-B*082:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_066_01():
    result = parse("Mafa-B*066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_085_04():
    result = parse("Mamu-B*085:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*085:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*085:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_07_01():
    result = parse("Mafa-B*068:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*068:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_04():
    result = parse("Mafa-B*070:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*070:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_075_03():
    result = parse("Mafa-B*075:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*075:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*075:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_155_01():
    result = parse("Mamu-B*155:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*155:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*155:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_101_03():
    result = parse("Mamu-B*101:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*101:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*101:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_101_04():
    result = parse("Mamu-B*101:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*101:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*101:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_124_01_03():
    result = parse("Mafa-B*124:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*124:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*124:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_144_05():
    result = parse("Mafa-B*144:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*144:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*144:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_147_03():
    result = parse("Mamu-B*147:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*147:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*147:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_184_02():
    result = parse("Mafa-B*184:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*184:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*184:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_23_02():
    result = parse("Mafa-DPB1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_09_01_02():
    result = parse("Mafa-DPB1*09:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*09:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*09:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A8_01_01():
    result = parse("Mafa-A8*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A8*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A8*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_03_01():
    result = parse("Paan-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_04_01():
    result = parse("Paan-A*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_06_01():
    result = parse("Paan-B*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_07_01():
    result = parse("Paan-B*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_05_01():
    result = parse("Paan-A*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_08_01():
    result = parse("Paan-B*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_09_01():
    result = parse("Paan-B*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_06_01():
    result = parse("Paan-A*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_06_02():
    result = parse("Paan-B*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_07_01():
    result = parse("Paan-A*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_B_10_01():
    result = parse("Paha-B*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-B*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-B*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_11_01():
    result = parse("Paan-B*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_07_02():
    result = parse("Paan-B*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_02_01():
    result = parse("Paan-B*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_12_01():
    result = parse("Paan-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_08_01():
    result = parse("Paan-A*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_01_02():
    result = parse("Paan-B*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_13_01():
    result = parse("Paan-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_14_01():
    result = parse("Paan-B*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_02_02():
    result = parse("Paan-B*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_15_02():
    result = parse("Paan-B*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_16_01():
    result = parse("Paan-B*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_17_01():
    result = parse("Paan-B*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_18_01():
    result = parse("Paan-B*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_19_01():
    result = parse("Paan-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_20_01():
    result = parse("Paan-B*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_09_01():
    result = parse("Paan-A*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_10_01():
    result = parse("Paan-A*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_11_01():
    result = parse("Paan-A*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_21_01():
    result = parse("Paan-B*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_15_01():
    result = parse("Paan-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_22_01():
    result = parse("Paan-B*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_23_01():
    result = parse("Paan-B*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_24_01():
    result = parse("Paan-B*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_25_01():
    result = parse("Paan-B*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_26_01():
    result = parse("Paan-B*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_10_02():
    result = parse("Paan-B*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_27_01():
    result = parse("Paan-B*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_20_02():
    result = parse("Paan-B*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_02_03():
    result = parse("Paan-B*02:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_24_02():
    result = parse("Paan-B*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_09_02():
    result = parse("Paan-B*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_12_01():
    result = parse("Paan-A*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_28_01():
    result = parse("Paan-B*28:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*28:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*28:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_29_01():
    result = parse("Paan-B*29:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*29:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*29:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_15_03():
    result = parse("Paan-B*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_23_03():
    result = parse("Mafa-DQA1*23:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*23:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*23:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_01_02():
    result = parse("Mafa-DQA1*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_12():
    result = parse("Mafa-DQA1*26:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_41():
    result = parse("Mafa-DQB1*06:41")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:41") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:41") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_10_04():
    result = parse("Mafa-DRA*01:10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:10:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_30():
    result = parse("Mafa-DQB1*18:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_23_04():
    result = parse("Mafa-DQA1*23:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*23:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*23:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_E_02_01():
    result = parse("Ceat-E*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-E*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-E*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_E_02_02Sp():
    result = parse("Ceat-E*02:02Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-E*02:02Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-E*02:02Sp") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ceat_E_02_03():
    result = parse("Ceat-E*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ceat-E*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ceat-E*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_10():
    result = parse("Mamu-DQA1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_05_07():
    result = parse("Mamu-DQA1*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_08_02():
    result = parse("Mamu-DQA1*26:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DQA1*26:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_10():
    result = parse("Mamu-DQA1*26:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_26_11():
    result = parse("Mamu-DQA1*26:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*26:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*26:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_008_05():
    result = parse("Mafa-A1*008:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*008:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*008:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_074_03():
    result = parse("Mafa-A1*074:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*074:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*074:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_098_03():
    result = parse("Mafa-A1*098:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*098:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*098:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_01_02():
    result = parse("Mafa-B*041:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*041:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_09():
    result = parse("Mafa-B*044:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_08_01_01():
    result = parse("Mafa-B*098:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*098:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_09():
    result = parse("Mafa-B*098:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_091_02():
    result = parse("Mafa-B*091:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*091:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*091:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_10_02():
    result = parse("Mafa-I*01:10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:10:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_15_02():
    result = parse("Mafa-I*01:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_052_04():
    result = parse("Mafa-A1*052:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*052:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*052:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_021_03():
    result = parse("Mafa-B*021:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*021:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*021:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_20_01():
    result = parse("Mafa-B*030:20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:20:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_08():
    result = parse("Caja-G*08:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_09():
    result = parse("Caja-G*08:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_10():
    result = parse("Caja-G*08:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_03_03():
    result = parse("Mafa-B*060:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*060:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_04():
    result = parse("Caja-G*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_08():
    result = parse("Mafa-B*068:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_11():
    result = parse("Caja-G*08:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_09():
    result = parse("Mafa-B*068:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_12():
    result = parse("Caja-G*08:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_05():
    result = parse("Mafa-B*070:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*070:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_23_01():
    result = parse("Caja-G*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_24_01():
    result = parse("Caja-G*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_06():
    result = parse("Caja-G*18:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_13():
    result = parse("Caja-G*08:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_10():
    result = parse("Mafa-B*098:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_14():
    result = parse("Caja-G*08:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_114_02():
    result = parse("Mafa-B*114:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*114:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*114:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_04_02():
    result = parse("Mafa-B*115:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*115:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_07():
    result = parse("Caja-G*18:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_157_03():
    result = parse("Mafa-B*157:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*157:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*157:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_15():
    result = parse("Caja-G*08:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_16():
    result = parse("Caja-G*08:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_27():
    result = parse("Mafa-I*01:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_18():
    result = parse("Caja-G*08:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_19():
    result = parse("Caja-G*08:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_08():
    result = parse("Caja-G*18:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_21():
    result = parse("Caja-G*08:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_08():
    result = parse("Caja-G*09:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_17():
    result = parse("Caja-G*08:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_22():
    result = parse("Caja-G*08:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_09():
    result = parse("Caja-G*18:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_25_01():
    result = parse("Caja-G*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_20():
    result = parse("Caja-G*08:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_41():
    result = parse("Mafa-DPA1*02:41")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:41") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:41") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_07_04():
    result = parse("Mafa-DPB1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_18_03_02():
    result = parse("Mafa-DPB1*18:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*18:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*18:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_01_04():
    result = parse("Mafa-DQA1*26:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*26:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_07():
    result = parse("Caja-G*09:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_16_02_01():
    result = parse("Caja-G*16:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*16:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*16:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_05():
    result = parse("Caja-G*18:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_B_02_01():
    result = parse("Paha-B*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-B*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-B*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_56():
    result = parse("Mafa-A2*05:56")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:56") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:56") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_06():
    result = parse("Mafa-A2*24:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_14():
    result = parse("Mafa-A4*14:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_B_12_01():
    result = parse("Paha-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_B_18_01():
    result = parse("Paha-B*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-B*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-B*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_08():
    result = parse("Mafa-B*050:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*050:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_B_19_01():
    result = parse("Paha-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_11():
    result = parse("Mafa-B*098:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_B_15_01():
    result = parse("Paha-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_28():
    result = parse("Mafa-I*01:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_B_30_01_01():
    result = parse("Paha-B*30:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-B*30:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paha-B*30:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_01_02():
    result = parse("Paan-A*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_002_02():
    result = parse("Mamu-A1*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_041_02_02():
    result = parse("Mamu-A1*041:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*041:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*041:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_056_04():
    result = parse("Mamu-A1*056:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*056:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*056:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_108_02_01_01():
    result = parse("Mamu-A1*108:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*108:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*108:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_123_03_01_01():
    result = parse("Mamu-A1*123:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*123:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*123:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_01_04():
    result = parse("Mamu-A2*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_05():
    result = parse("Mafa-A1*001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_06_01_01():
    result = parse("Mafa-A1*003:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*003:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_04_02():
    result = parse("Mamu-A4*14:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A4*14:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_15():
    result = parse("Mamu-A4*14:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_07():
    result = parse("Mafa-A1*010:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*010:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_16():
    result = parse("Mamu-A4*14:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_04_02():
    result = parse("Mamu-A5*30:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A5*30:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_02_02():
    result = parse("Mamu-A5*30:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A5*30:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_03_03():
    result = parse("Mamu-AG*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-AG*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_AG_03_04():
    result = parse("Mamu-AG*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-AG*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_04():
    result = parse("Mamu-B*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_011_03():
    result = parse("Mamu-B*011:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*011:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*011:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_01_02():
    result = parse("Mamu-B*017:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*017:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_09_02():
    result = parse("Mafa-A1*022:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*022:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_036_04_01_01():
    result = parse("Mamu-B*036:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*036:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*036:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_033_02():
    result = parse("Mafa-A1*033:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*033:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*033:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_10():
    result = parse("Mamu-B*045:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*045:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_188_02_01_01():
    result = parse("Mamu-B*188:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*188:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*188:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_11():
    result = parse("Mamu-B*051:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_12():
    result = parse("Mamu-B*051:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_054_03_01_01():
    result = parse("Mamu-B*054:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*054:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*054:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_054_04():
    result = parse("Mamu-B*054:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*054:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*054:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_08():
    result = parse("Mamu-B*060:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_09():
    result = parse("Mamu-B*060:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_086_01_02_01N():
    result = parse("Mafa-A1*086:01:02:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*086:01:02:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*086:01:02:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_01_03():
    result = parse("Mamu-B*068:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*068:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_03():
    result = parse("Mafa-A1*090:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*090:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_08():
    result = parse("Mamu-B*069:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*069:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_101_02_02():
    result = parse("Mafa-A1*101:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*101:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*101:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_102_03_01():
    result = parse("Mafa-A1*102:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*102:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*102:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_070_04():
    result = parse("Mamu-B*070:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*070:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*070:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_06_01_01():
    result = parse("Mamu-B*072:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_180_01_01_01():
    result = parse("Mamu-B*180:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*180:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*180:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_07():
    result = parse("Mamu-B*072:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*072:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_05_02():
    result = parse("Mamu-B*072:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*072:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_074_04():
    result = parse("Mamu-B*074:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*074:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*074:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_07():
    result = parse("Mafa-A2*24:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_10():
    result = parse("Mamu-B*060:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_04():
    result = parse("Mamu-B*079:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*079:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_01_03():
    result = parse("Mamu-B*079:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*079:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_17_02():
    result = parse("Mafa-A3*13:17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A3*13:17:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_20():
    result = parse("Mafa-A3*13:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_09():
    result = parse("Mamu-B*082:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_06_02():
    result = parse("Mamu-B*082:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*082:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_10():
    result = parse("Mamu-B*082:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_16_01_01():
    result = parse("Mafa-A4*14:16:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:16:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:16:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_086_06_01_01():
    result = parse("Mamu-B*086:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*086:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*086:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_095_03():
    result = parse("Mamu-B*095:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*095:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*095:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_15():
    result = parse("Mafa-A4*14:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_06_02_01():
    result = parse("Mamu-B*098:06:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:06:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:06:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_07_01_01():
    result = parse("Mafa-B*007:07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*007:07:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_02_02_01():
    result = parse("Mamu-B*098:02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:02:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_08_01_01():
    result = parse("Mafa-B*007:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*007:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_07():
    result = parse("Mamu-B*098:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*098:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_05():
    result = parse("Mafa-DPB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_101_05():
    result = parse("Mamu-B*101:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*101:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*101:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_13_01_01():
    result = parse("Mafa-B*013:13:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:13:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*013:13:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_14():
    result = parse("Mafa-B*013:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_187_03_01_01():
    result = parse("Mamu-B*187:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*187:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*187:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_109_01_01_01():
    result = parse("Mamu-B*109:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*109:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*109:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_109_02():
    result = parse("Mamu-B*109:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*109:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*109:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_109_03_01_01():
    result = parse("Mamu-B*109:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*109:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*109:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_139_01():
    result = parse("Mamu-B*139:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*139:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*139:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_142_03_01_01():
    result = parse("Mamu-B*142:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*142:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*142:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_146_01():
    result = parse("Mamu-B*146:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*146:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*146:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_151_02():
    result = parse("Mamu-B*151:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*151:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*151:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_151_03():
    result = parse("Mamu-B*151:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*151:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*151:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_162_02_01_01():
    result = parse("Mamu-B*162:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*162:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*162:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_03_02():
    result = parse("Mafa-DRB3*04:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB3*04:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W059_02():
    result = parse("Mafa-DRB*W059:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W059:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W059:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_06():
    result = parse("Mamu-DRB*W001:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W001:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W001_07():
    result = parse("Mamu-DRB*W001:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W001:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W001:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W053_01():
    result = parse("Mamu-DRB*W053:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W053:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W053:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_08_06():
    result = parse("Mafa-DPB1*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W020_04():
    result = parse("Mamu-DRB*W020:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W020:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W020:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W025_05_02():
    result = parse("Mamu-DRB*W025:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W025:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W025:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W068_01():
    result = parse("Mamu-DRB*W068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W004_02_02():
    result = parse("Mamu-DRB*W004:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W004:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W004:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W059_02():
    result = parse("Mamu-DRB*W059:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W059:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W059:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_09_03():
    result = parse("Mamu-DRB*W006:09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-DRB*W006:09:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W006_13():
    result = parse("Mamu-DRB*W006:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W006:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W006:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W007_05():
    result = parse("Mamu-DRB*W007:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W007:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W007:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W101_01():
    result = parse("Mamu-DRB*W101:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W101:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W101:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_014_02():
    result = parse("Mafa-B*014:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*014:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*014:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_025_02_01_01():
    result = parse("Mafa-B*025:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*025:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*025:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB1_04_21():
    result = parse("Mamu-DRB1*04:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB1*04:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB1*04:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_027_03_02():
    result = parse("Mafa-B*027:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*027:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*027:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB_W102_01():
    result = parse("Mamu-DRB*W102:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB*W102:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB*W102:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DRB3_04_17():
    result = parse("Mamu-DRB3*04:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DRB3*04:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DRB3*04:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_16():
    result = parse("Mafa-B*030:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_12():
    result = parse("Mafa-DPB1*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_29():
    result = parse("Mamu-I*01:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_06_01_01():
    result = parse("Mafa-B*048:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*048:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_30():
    result = parse("Mamu-I*01:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_12Sp():
    result = parse("Mamu-I*01:06:12Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:12Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:12Sp") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_12():
    result = parse("Mafa-B*060:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_31():
    result = parse("Mamu-I*01:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_13_01_01():
    result = parse("Mafa-B*060:13:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:13:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:13:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_13():
    result = parse("Mamu-I*01:06:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:13") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_061_03_02():
    result = parse("Mafa-B*061:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*061:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*061:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_03():
    result = parse("Mamu-I*01:09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:09:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_061_04_02_01():
    result = parse("Mafa-B*061:04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*061:04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*061:04:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_065_04_01_01():
    result = parse("Mafa-B*065:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*065:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*065:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_04():
    result = parse("Mamu-I*01:09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:09:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_12():
    result = parse("Mafa-DPB1*15:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_10():
    result = parse("Mafa-B*068:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_05():
    result = parse("Mafa-B*069:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_04():
    result = parse("Mafa-B*072:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_077_02():
    result = parse("Mafa-B*077:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*077:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*077:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_04_01_01():
    result = parse("Mafa-B*088:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*088:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_03():
    result = parse("Mafa-B*117:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*117:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_124_03_01_01():
    result = parse("Mafa-B*124:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*124:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*124:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_05():
    result = parse("Mafa-B*137:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*137:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_161_05():
    result = parse("Mafa-B*161:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*161:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*161:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_29():
    result = parse("Mafa-I*01:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_30():
    result = parse("Mafa-I*01:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_11_02():
    result = parse("Mafa-I*01:11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:11:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_02():
    result = parse("Mafa-I*01:14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_004_02():
    result = parse("Mane-A1*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_054_01():
    result = parse("Mane-A1*054:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*054:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*054:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_095_01():
    result = parse("Mane-A1*095:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*095:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*095:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_018_01():
    result = parse("Mane-A1*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_03():
    result = parse("Mane-A1*019:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*019:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_040_01():
    result = parse("Mane-A1*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_063_02_01():
    result = parse("Mane-A1*063:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*063:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*063:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_074_01():
    result = parse("Mane-A1*074:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*074:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*074:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_082_01_02():
    result = parse("Mane-A1*082:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*082:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*082:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_084_05():
    result = parse("Mane-A1*084:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*084:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*084:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_086_02N():
    result = parse("Mane-A1*086:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*086:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*086:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_086_03N():
    result = parse("Mane-A1*086:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*086:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*086:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_23():
    result = parse("Mane-A2*05:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_10():
    result = parse("Mane-A3*13:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_06():
    result = parse("Mane-A4*14:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_15():
    result = parse("Mane-A4*14:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_07():
    result = parse("Mane-A4*14:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_08():
    result = parse("Mane-A4*14:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_09():
    result = parse("Mane-A4*14:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A6_01_03():
    result = parse("Mane-A6*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A6*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A6*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A7_01_01():
    result = parse("Mane-A7*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A7*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A7*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_007_03():
    result = parse("Mane-B*007:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*007:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*007:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_013_02():
    result = parse("Mane-B*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_014_01N():
    result = parse("Mane-B*014:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*014:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*014:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_014_02_01N():
    result = parse("Mane-B*014:02:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*014:02:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*014:02:01N") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_025_01():
    result = parse("Mane-B*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_027_04():
    result = parse("Mane-B*027:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*027:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*027:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_028_02():
    result = parse("Mane-B*028:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*028:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*028:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_041_03():
    result = parse("Mane-B*041:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*041:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*041:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_045_03():
    result = parse("Mane-B*045:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*045:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*045:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_13():
    result = parse("Mamu-B*051:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_09():
    result = parse("Mane-B*068:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_070_01():
    result = parse("Mane-B*070:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*070:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*070:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_109_04_01_01():
    result = parse("Mamu-B*109:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*109:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*109:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_AG_02_01():
    result = parse("Paan-AG*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-AG*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-AG*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_AG_01_03():
    result = parse("Paan-AG*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-AG*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-AG*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_082_04():
    result = parse("Mane-B*082:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*082:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*082:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_083_03():
    result = parse("Mane-B*083:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*083:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*083:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_13_01():
    result = parse("Paan-A*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_14_03():
    result = parse("Paan-A*14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*14:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_088_05():
    result = parse("Mane-B*088:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*088:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*088:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_089_06():
    result = parse("Mane-B*089:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*089:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*089:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_14_02():
    result = parse("Paan-A*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_098_03():
    result = parse("Mane-B*098:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*098:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*098:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_14_01_01():
    result = parse("Paan-A*14:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*14:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-A*14:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_099_02():
    result = parse("Mane-B*099:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*099:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*099:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_14_01_02():
    result = parse("Paan-A*14:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*14:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-A*14:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_103_03():
    result = parse("Mane-B*103:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*103:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*103:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_02_02():
    result = parse("Paan-A*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_15_01():
    result = parse("Paan-A*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_104_04():
    result = parse("Mane-B*104:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*104:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*104:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_16_01():
    result = parse("Paan-A*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_105_03():
    result = parse("Mane-B*105:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*105:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*105:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_17_01():
    result = parse("Paan-A*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_107_04():
    result = parse("Mane-B*107:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*107:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*107:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_111_03():
    result = parse("Mane-B*111:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*111:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*111:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_03():
    result = parse("Paan-B*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_04():
    result = parse("Paan-B*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_AG_01_01_02():
    result = parse("Paan-AG*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-AG*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-AG*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_115_05():
    result = parse("Mane-B*115:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*115:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*115:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_06_01():
    result = parse("Paan-B*02:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_116_02():
    result = parse("Mane-B*116:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*116:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*116:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_07():
    result = parse("Paan-B*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_117_02():
    result = parse("Mane-B*117:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*117:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*117:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_118_01_02():
    result = parse("Mane-B*118:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*118:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*118:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_197_01():
    result = parse("Mane-B*197:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*197:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*197:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_06_02():
    result = parse("Mane-I*01:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_15():
    result = parse("Mane-I*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_16():
    result = parse("Mane-I*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_17_01():
    result = parse("Mane-I*01:17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:17:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_18():
    result = parse("Mane-I*01:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_19():
    result = parse("Mane-I*01:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_20_01():
    result = parse("Mane-I*01:20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:20:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_21():
    result = parse("Mane-I*01:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_003_01():
    result = parse("Malo-A1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_073_01():
    result = parse("Malo-A1*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A2_05_01():
    result = parse("Malo-A2*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A2*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A2*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_114_01():
    result = parse("Malo-A1*114:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*114:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*114:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_040_01():
    result = parse("Malo-A1*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_049_01():
    result = parse("Malo-A1*049:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*049:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*049:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_090_01():
    result = parse("Malo-A1*090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A3_13_01():
    result = parse("Malo-A3*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A3*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A3*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_127_01():
    result = parse("Malo-A1*127:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*127:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*127:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A2_05_02():
    result = parse("Malo-A2*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A2*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A2*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_006_01():
    result = parse("Malo-A1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_A_01_01():
    result = parse("Paha-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_098_01():
    result = parse("Malo-A1*098:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*098:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*098:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_003_02():
    result = parse("Malo-A1*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_063_01():
    result = parse("Malo-A1*063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_080_01():
    result = parse("Malo-A1*080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A1_127_02():
    result = parse("Malo-A1*127:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A1*127:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A1*127:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_A2_05_03():
    result = parse("Malo-A2*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-A2*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-A2*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_42():
    result = parse("Mafa-DQB1*06:42")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:42") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:42") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_13_02():
    result = parse("Mafa-DQB1*06:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_056_01():
    result = parse("Malo-B*056:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*056:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*056:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_072_01():
    result = parse("Malo-B*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_047_01():
    result = parse("Malo-B*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_043_01():
    result = parse("Malo-B*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W003_21():
    result = parse("Mafa-DRB*W003:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W003:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W003:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_019_01():
    result = parse("Malo-B*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_058_01():
    result = parse("Malo-B*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_025_01():
    result = parse("Malo-B*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_120_01():
    result = parse("Malo-B*120:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*120:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*120:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_20_02():
    result = parse("Mafa-DPB1*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_041_01():
    result = parse("Malo-B*041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_024_01():
    result = parse("Malo-B*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_073_01():
    result = parse("Malo-B*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_047_02():
    result = parse("Malo-B*047:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*047:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*047:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_094_01():
    result = parse("Malo-B*094:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*094:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*094:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_065_01():
    result = parse("Malo-B*065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_018_01():
    result = parse("Malo-B*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_017_01():
    result = parse("Malo-B*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_061_01():
    result = parse("Malo-B*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_013_01():
    result = parse("Malo-B*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_078_01():
    result = parse("Malo-B*078:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*078:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*078:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_032_01():
    result = parse("Malo-B*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_051_01():
    result = parse("Malo-B*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_B_091_01():
    result = parse("Malo-B*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-B*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-B*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_I_01_01():
    result = parse("Malo-I*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-I*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-I*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_004_01_02():
    result = parse("Mane-A1*004:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*004:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*004:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_006_04():
    result = parse("Mane-A1*006:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*006:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*006:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_006_05():
    result = parse("Mane-A1*006:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*006:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*006:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_010_02():
    result = parse("Mane-A1*010:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*010:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*010:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_05():
    result = parse("Mane-A1*019:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*019:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Hymo_DRB_W103_01():
    result = parse("Hymo-DRB*W103:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Hymo-DRB*W103:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Hymo-DRB*W103:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_032_02():
    result = parse("Mane-A1*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_054_03():
    result = parse("Mane-A1*054:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*054:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*054:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_131_01_01():
    result = parse("Mane-A1*131:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*131:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*131:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_131_01_02():
    result = parse("Mane-A1*131:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*131:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*131:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_131_02():
    result = parse("Mane-A1*131:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*131:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*131:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_132_01():
    result = parse("Mane-A1*132:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*132:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*132:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_114_02():
    result = parse("Mane-A1*114:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*114:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*114:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_054_01Sp():
    result = parse("Mane-B*054:01Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*054:01Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*054:01Sp") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_130_01():
    result = parse("Mane-A1*130:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*130:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*130:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_22_01():
    result = parse("Mane-I*01:22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:22:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_26():
    result = parse("Mane-A2*05:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_20_02():
    result = parse("Mane-I*01:20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:20:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_054_02():
    result = parse("Mane-A1*054:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*054:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*054:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_018_02():
    result = parse("Mane-A1*018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_12():
    result = parse("Mane-A3*13:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_04_01():
    result = parse("Mane-A1*019:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*019:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_01_03():
    result = parse("Mane-A4*14:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A4*14:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_038_02():
    result = parse("Mane-A1*038:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*038:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*038:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_01_04():
    result = parse("Mane-A4*14:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A4*14:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_072_03():
    result = parse("Mane-A1*072:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*072:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*072:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_13():
    result = parse("Mane-A4*14:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_093_01():
    result = parse("Mane-A1*093:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*093:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*093:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_24():
    result = parse("Mane-A2*05:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_14():
    result = parse("Mane-A4*14:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_25():
    result = parse("Mane-A2*05:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_19_02():
    result = parse("Mane-A2*05:19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A2*05:19:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_11():
    result = parse("Mane-A3*13:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A3*13:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_014_04N():
    result = parse("Mane-B*014:04N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*014:04N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*014:04N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_10():
    result = parse("Mane-A4*14:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_014_02_02N():
    result = parse("Mane-B*014:02:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*014:02:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*014:02:02N") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_11():
    result = parse("Mane-A4*14:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_015_04():
    result = parse("Mane-B*015:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*015:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*015:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_12():
    result = parse("Mane-A4*14:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A7_01_02():
    result = parse("Mane-A7*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A7*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A7*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_028_01_02():
    result = parse("Mane-B*028:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*028:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*028:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_136_01():
    result = parse("Mane-B*136:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*136:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*136:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_014_03N():
    result = parse("Mane-B*014:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*014:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*014:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_032_01():
    result = parse("Mane-B*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_024_02():
    result = parse("Mane-B*024:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*024:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*024:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_035_02():
    result = parse("Mane-B*035:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*035:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*035:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_041_04():
    result = parse("Mane-B*041:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*041:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*041:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_046_02():
    result = parse("Mane-B*046:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*046:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*046:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_05():
    result = parse("Mane-B*051:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*051:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_06():
    result = parse("Mane-B*051:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*051:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_04_05():
    result = parse("Mane-B*051:04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*051:04:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_07():
    result = parse("Mane-B*051:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*051:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_063_01Sp():
    result = parse("Mane-B*063:01Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*063:01Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*063:01Sp") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_055_01():
    result = parse("Mane-B*055:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*055:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*055:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_10():
    result = parse("Mane-B*068:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_06():
    result = parse("Mane-B*072:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*072:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_08():
    result = parse("Mane-B*079:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*079:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_100_01_02():
    result = parse("Mane-B*100:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*100:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*100:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_03_02():
    result = parse("Mane-B*060:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*060:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_082_05():
    result = parse("Mane-B*082:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*082:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*082:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_064_02():
    result = parse("Mane-B*064:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*064:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*064:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_088_02_02():
    result = parse("Mane-B*088:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*088:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*088:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_065_01_01():
    result = parse("Mane-B*065:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*065:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*065:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_065_01_02():
    result = parse("Mane-B*065:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*065:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*065:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_103_01_02():
    result = parse("Mane-B*103:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*103:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*103:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_07_02():
    result = parse("Mane-B*068:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*068:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_112_03():
    result = parse("Mane-B*112:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*112:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*112:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_11():
    result = parse("Mane-B*068:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_189_01():
    result = parse("Mane-B*189:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*189:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*189:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_069_03():
    result = parse("Mane-B*069:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*069:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*069:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_23_01():
    result = parse("Mane-I*01:23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:23:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_02_03():
    result = parse("Mane-B*072:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_07():
    result = parse("Mane-B*072:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*072:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_08():
    result = parse("Mane-B*072:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*072:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_02_04():
    result = parse("Mane-B*072:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_04_04():
    result = parse("Mane-B*072:04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*072:04:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_078_05():
    result = parse("Mane-B*078:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*078:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*078:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_081_01_02():
    result = parse("Mane-B*081:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*081:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*081:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_103_04():
    result = parse("Mane-B*103:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*103:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*103:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_107_05():
    result = parse("Mane-B*107:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*107:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*107:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_108_03():
    result = parse("Mane-B*108:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*108:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*108:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_116_01_02():
    result = parse("Mane-B*116:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*116:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*116:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_118_04N():
    result = parse("Mane-B*118:04N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*118:04N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*118:04N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_199_01():
    result = parse("Mane-B*199:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*199:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*199:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_26():
    result = parse("Mane-I*01:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_27():
    result = parse("Mane-I*01:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_28():
    result = parse("Mane-I*01:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_06_06():
    result = parse("Mamu-DPB1*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_29():
    result = parse("Mane-I*01:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_14_02():
    result = parse("Mane-I*01:14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:14:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_04_02():
    result = parse("Mane-I*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_30():
    result = parse("Mane-I*01:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_06_03():
    result = parse("Mane-I*01:06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:06:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_31():
    result = parse("Mane-I*01:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_32():
    result = parse("Mane-I*01:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_33():
    result = parse("Mane-I*01:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Popy_B_03_07():
    result = parse("Popy-B*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Popy-B*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Popy-B*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_34():
    result = parse("Mane-I*01:34")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:34") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:34") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_14_03():
    result = parse("Mane-I*01:14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:14:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_06_02():
    result = parse("Mafa-I*01:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_23():
    result = parse("Caja-G*08:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_039_02_01_01():
    result = parse("Mafa-B*039:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*039:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*039:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_11():
    result = parse("Mamu-DQA1*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_24_02():
    result = parse("Mafa-DQA1*01:24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:24:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQA1_01_12():
    result = parse("Mamu-DQA1*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQA1*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQA1*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_06_26():
    result = parse("Mamu-DQB1*06:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*06:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*06:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DQB1_18_27():
    result = parse("Mamu-DQB1*18:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DQB1*18:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DQB1*18:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_DPB1_06_07():
    result = parse("Mamu-DPB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-DPB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-DPB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_43():
    result = parse("Mafa-DQB1*06:43")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:43") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:43") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_24_01():
    result = parse("Mafa-DQA1*01:24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:24:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_24_02():
    result = parse("Mafa-DQB1*06:24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:24:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_15_01_02():
    result = parse("Mafa-DQB1*15:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*15:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*15:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_31():
    result = parse("Mafa-DQB1*18:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_039_03():
    result = parse("Mafa-B*039:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*039:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*039:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_038_01_02_01():
    result = parse("Mafa-B*038:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*038:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*038:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_06_01_01():
    result = parse("Mafa-B*137:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*137:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_202_01():
    result = parse("Mafa-B*202:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*202:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*202:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_08_01_01():
    result = parse("Mafa-B*051:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*051:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_09():
    result = parse("Mafa-B*007:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*007:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_11():
    result = parse("Mafa-B*068:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_180_01_01_01():
    result = parse("Mafa-B*180:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*180:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*180:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_112_01_01_01():
    result = parse("Mafa-B*112:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*112:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*112:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_12_01_01():
    result = parse("Mafa-B*068:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*068:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_161_02_02_01():
    result = parse("Mafa-B*161:02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*161:02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*161:02:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_092_02_01_01():
    result = parse("Mafa-B*092:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*092:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*092:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_081_03_01_01():
    result = parse("Mafa-B*081:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*081:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*081:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_081_04():
    result = parse("Mafa-B*081:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*081:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*081:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_04_02():
    result = parse("Mane-A1*019:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*019:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_06():
    result = parse("Mane-A1*019:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*019:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_04():
    result = parse("Mafa-B*101:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*101:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_054_04():
    result = parse("Mane-A1*054:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*054:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*054:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_060_01():
    result = parse("Mane-A1*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_006_01_01_01():
    result = parse("Mafa-B*006:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*006:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*006:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_074_02():
    result = parse("Mane-A1*074:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*074:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*074:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_082_02():
    result = parse("Mafa-B*082:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*082:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*082:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_082_01_03():
    result = parse("Mane-A1*082:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*082:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A1*082:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_10():
    result = parse("Mafa-A1*022:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A4_14_16():
    result = parse("Mane-A4*14:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A4*14:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A4*14:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_11():
    result = parse("Mafa-A1*022:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_014_05():
    result = parse("Mane-B*014:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*014:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*014:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_041_06():
    result = parse("Mane-B*041:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*041:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*041:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_145_01():
    result = parse("Mane-B*145:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*145:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*145:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_07():
    result = parse("Mafa-A1*007:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_12():
    result = parse("Mane-B*068:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_072_09():
    result = parse("Mane-B*072:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*072:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*072:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_081_02():
    result = parse("Mane-B*081:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*081:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*081:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_091_02():
    result = parse("Mafa-A1*091:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*091:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*091:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_088_06():
    result = parse("Mane-B*088:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*088:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*088:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_091_03():
    result = parse("Mafa-A1*091:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*091:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*091:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_101_01():
    result = parse("Mane-B*101:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*101:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*101:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_040_04():
    result = parse("Mafa-A1*040:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*040:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*040:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_150_02():
    result = parse("Mane-B*150:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*150:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*150:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_099_02():
    result = parse("Mafa-A1*099:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*099:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*099:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_22_02():
    result = parse("Mane-I*01:22:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:22:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:22:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_35():
    result = parse("Mane-I*01:35")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:35") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:35") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_018_03():
    result = parse("Mane-A1*018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_06_02():
    result = parse("Mafa-A1*003:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*003:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_08_01_01():
    result = parse("Mafa-A1*018:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*018:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_056_03_02():
    result = parse("Mafa-A1*056:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*056:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*056:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_17():
    result = parse("Caja-DRB1*03:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_18():
    result = parse("Caja-DRB1*03:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_11_04():
    result = parse("Caja-G*11:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*11:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*11:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_11_05():
    result = parse("Caja-G*11:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*11:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*11:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_078_03():
    result = parse("Mafa-A1*078:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*078:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*078:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_05():
    result = parse("Caja-G*12:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*12:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_12_06():
    result = parse("Caja-G*12:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*12:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*12:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_15_03():
    result = parse("Caja-G*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_19_03():
    result = parse("Caja-G*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_17():
    result = parse("Mafa-A4*14:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_048_01():
    result = parse("Mafa-A1*048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_10():
    result = parse("Mane-DPA1*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_11():
    result = parse("Mane-DPA1*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_04_03():
    result = parse("Mafa-DPA1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_01_01_02():
    result = parse("Mane-DPB1*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DPB1*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_01_03():
    result = parse("Mane-DPB1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_03_02():
    result = parse("Mane-DPB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_04_03():
    result = parse("Mane-DPB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_07_01():
    result = parse("Mane-DPB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_15_02():
    result = parse("Mane-DPB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_02_03():
    result = parse("Mane-DQA1*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_08():
    result = parse("Mane-DQA1*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_09():
    result = parse("Mane-DQA1*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_10():
    result = parse("Mane-DQA1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_05_10():
    result = parse("Mane-DQA1*05:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*05:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*05:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_23_02():
    result = parse("Mane-DQA1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_24_06():
    result = parse("Mane-DQA1*24:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*24:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*24:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_26_03_02():
    result = parse("Mane-DQA1*26:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*26:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQA1*26:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_15():
    result = parse("Mane-DQB1*06:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*06:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_03_02():
    result = parse("Mane-DQB1*17:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*17:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_18_10():
    result = parse("Mane-DQB1*18:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*18:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*18:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_02_05():
    result = parse("Mane-DRA*01:02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:02:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_03_03():
    result = parse("Mane-DRA*01:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRA*01:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_13():
    result = parse("Mane-DRB*W001:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_14():
    result = parse("Mane-DRB*W001:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_03():
    result = parse("Mane-DRB*W002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_14():
    result = parse("Mane-DRB*W002:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W006_03():
    result = parse("Mane-DRB*W006:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W006:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W006:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W025_06():
    result = parse("Mane-DRB*W025:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W025:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W025:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W069_03():
    result = parse("Mane-DRB*W069:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W069:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W069:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W059_01():
    result = parse("Mane-DRB*W059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_05():
    result = parse("Mane-DRB1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_07_02():
    result = parse("Mane-DRB1*03:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB1*03:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_06():
    result = parse("Mane-DRB1*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_15():
    result = parse("Mane-DRB*W002:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_10_06():
    result = parse("Mane-DRB1*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_11():
    result = parse("Mane-DQA1*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_06_12_02():
    result = parse("Mane-DQB1*06:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*06:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DQB1*06:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQB1_17_10():
    result = parse("Mane-DQB1*17:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQB1*17:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQB1*17:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W025_01():
    result = parse("Mane-DRB*W025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W025_02():
    result = parse("Mane-DRB*W025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W063_01():
    result = parse("Mane-DRB*W063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W064_01():
    result = parse("Mane-DRB*W064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W069_01():
    result = parse("Mane-DRB*W069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W001_07():
    result = parse("Mane-DRB*W001:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W001:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W001:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_04():
    result = parse("Mane-DRB*W002:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W002_16():
    result = parse("Mane-DRB*W002:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W002:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W002:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W003_04():
    result = parse("Mane-DRB*W003:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W003:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W003:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB_W004_01():
    result = parse("Mane-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_07():
    result = parse("Mamu-A1*007:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*007:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_02_02():
    result = parse("Mane-DRB1*04:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-DRB1*04:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_04_07():
    result = parse("Mane-DRB1*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_51():
    result = parse("Mamu-A2*05:51")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:51") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:51") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_20_02():
    result = parse("Mamu-A2*05:20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:20:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_02_05_01():
    result = parse("Mamu-A2*05:02:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:02:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:02:05:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_07():
    result = parse("Mamu-A6*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A6*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_117_01_02():
    result = parse("Mamu-B*117:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*117:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*117:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_14_01_01():
    result = parse("Mamu-B*051:14:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:14:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:14:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_11():
    result = parse("Mamu-B*060:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_12():
    result = parse("Mamu-B*060:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_061_06():
    result = parse("Mamu-B*061:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*061:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*061:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_070_05_01_01():
    result = parse("Mamu-B*070:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*070:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*070:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_109_05():
    result = parse("Mamu-B*109:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*109:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*109:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_109_06_01_01():
    result = parse("Mamu-B*109:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*109:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*109:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_201_01_01_01():
    result = parse("Mamu-B*201:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*201:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*201:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_114_01_01_01():
    result = parse("Mamu-B*114:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*114:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*114:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_134_03_01_01():
    result = parse("Mamu-B*134:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*134:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*134:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_21():
    result = parse("Mamu-E*02:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_34_02():
    result = parse("Mamu-I*01:34:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:34:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:34:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_35():
    result = parse("Mamu-I*01:35")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:35") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:35") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_36():
    result = parse("Mamu-I*01:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_03_01():
    result = parse("Mamu-I*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_14():
    result = parse("Mafa-DQA1*05:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_15():
    result = parse("Mafa-DQA1*05:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_25():
    result = parse("Mafa-DQA1*01:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_23_05():
    result = parse("Mafa-DQA1*23:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*23:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*23:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_001_01_03():
    result = parse("Mafa-A1*001:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*001:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*001:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_07():
    result = parse("Mafa-A1*003:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*003:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_028_03():
    result = parse("Mafa-A1*028:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*028:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*028:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_028_04():
    result = parse("Mafa-A1*028:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*028:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*028:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_028_05():
    result = parse("Mafa-A1*028:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*028:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*028:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_11():
    result = parse("Mafa-A1*043:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_045_03():
    result = parse("Mafa-A1*045:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*045:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*045:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_055_02():
    result = parse("Mafa-A1*055:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*055:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*055:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_05():
    result = parse("Mafa-A1*065:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*065:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_077_02_01():
    result = parse("Mafa-A1*077:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*077:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*077:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_077_02_02():
    result = parse("Mafa-A1*077:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*077:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*077:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_086_05():
    result = parse("Mafa-A1*086:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*086:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*086:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_02_02():
    result = parse("Mafa-A1*090:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*090:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_095_02():
    result = parse("Mafa-A1*095:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*095:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*095:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_096_02():
    result = parse("Mafa-A1*096:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*096:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*096:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_097_02():
    result = parse("Mafa-A1*097:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*097:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*097:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_102_03_02():
    result = parse("Mafa-A1*102:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*102:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*102:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_58():
    result = parse("Mafa-A2*05:58")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:58") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:58") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_06_04():
    result = parse("Mafa-A2*05:06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:06:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_59():
    result = parse("Mafa-A2*05:59")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:59") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:59") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_08():
    result = parse("Mafa-A2*24:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_21_01_01():
    result = parse("Mafa-A3*13:21:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:21:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A3*13:21:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_22_01_01():
    result = parse("Mafa-A3*13:22:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:22:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A3*13:22:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_12():
    result = parse("Mafa-A4*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_18():
    result = parse("Mafa-A4*14:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_19_01_01():
    result = parse("Mafa-A4*14:19:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:19:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:19:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_001_02():
    result = parse("Mafa-B*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_001_01_02():
    result = parse("Mafa-B*001:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*001:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*001:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_006_01_02():
    result = parse("Mafa-B*006:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*006:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*006:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_10():
    result = parse("Mafa-B*007:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*007:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_06_02():
    result = parse("Mafa-B*007:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*007:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_15():
    result = parse("Mafa-B*013:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_16():
    result = parse("Mafa-B*013:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_17():
    result = parse("Mafa-B*013:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_18_01_01():
    result = parse("Mafa-B*013:18:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:18:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*013:18:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_05():
    result = parse("Mafa-B*015:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_06():
    result = parse("Mafa-B*015:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_016_02():
    result = parse("Mafa-B*016:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*016:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*016:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_018_03():
    result = parse("Mafa-B*018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_019_04():
    result = parse("Mafa-B*019:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*019:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*019:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_021_05():
    result = parse("Mafa-B*021:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*021:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*021:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_01_05():
    result = parse("Mafa-B*030:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_18():
    result = parse("Mafa-B*030:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_19():
    result = parse("Mafa-B*030:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_031_03():
    result = parse("Mafa-B*031:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*031:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*031:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_043_02():
    result = parse("Mafa-B*043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_01_03():
    result = parse("Mafa-B*044:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*044:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_07_01_01():
    result = parse("Mafa-B*045:07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*045:07:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_047_03():
    result = parse("Mafa-B*047:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*047:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*047:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_02_02():
    result = parse("Mafa-B*050:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*050:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_056_05_02():
    result = parse("Mafa-B*056:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*056:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*056:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_16():
    result = parse("Mafa-B*060:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_14():
    result = parse("Mafa-B*060:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_061_05():
    result = parse("Mafa-B*061:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*061:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*061:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_064_05():
    result = parse("Mafa-B*064:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*064:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*064:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_065_05():
    result = parse("Mafa-B*065:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*065:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*065:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_067_01_01_01():
    result = parse("Mafa-B*067:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*067:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*067:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_067_02():
    result = parse("Mafa-B*067:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*067:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*067:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_06_02():
    result = parse("Mafa-B*068:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*068:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_07_02_01():
    result = parse("Mafa-B*068:07:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:07:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*068:07:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_06():
    result = parse("Mafa-B*069:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_073_03():
    result = parse("Mafa-B*073:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*073:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*073:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_203_01():
    result = parse("Mafa-B*203:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*203:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*203:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_082_03():
    result = parse("Mafa-B*082:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*082:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*082:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_083_04():
    result = parse("Mafa-B*083:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*083:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*083:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_05_01_01():
    result = parse("Mafa-B*088:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*088:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_095_03():
    result = parse("Mafa-B*095:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*095:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*095:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_095_04():
    result = parse("Mafa-B*095:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*095:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*095:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_12_01_01():
    result = parse("Mafa-B*098:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*098:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_17_01_01():
    result = parse("Mafa-B*098:17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*098:17:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_103_01_02():
    result = parse("Mafa-B*103:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*103:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*103:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_110_01_02_01():
    result = parse("Mafa-B*110:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*110:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*110:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_04():
    result = parse("Mafa-B*117:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*117:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_124_04_01_01():
    result = parse("Mafa-B*124:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*124:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*124:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_07():
    result = parse("Mafa-B*137:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*137:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_08():
    result = parse("Mafa-B*137:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*137:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_140_01_02():
    result = parse("Mafa-B*140:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*140:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*140:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_181_03():
    result = parse("Mafa-B*181:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*181:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*181:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_184_01_02():
    result = parse("Mafa-B*184:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*184:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*184:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_31():
    result = parse("Mafa-I*01:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_32_01_01():
    result = parse("Mafa-I*01:32:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:32:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-I*01:32:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_33():
    result = parse("Mafa-I*01:33")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:33") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:33") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_34():
    result = parse("Mafa-I*01:34")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:34") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:34") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_35_01():
    result = parse("Mafa-I*01:35:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:35:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:35:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_01_02():
    result = parse("Mafa-I*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_07():
    result = parse("Caja-DRB*W012:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W012:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_04_01():
    result = parse("Caja-DRB*W012:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_03_01():
    result = parse("Caja-DRB*W012:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_03_02():
    result = parse("Caja-DRB*W012:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPA1_02_01():
    result = parse("Malo-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPA1_04_01():
    result = parse("Malo-DPA1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPA1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPA1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPA1_02_02():
    result = parse("Malo-DPA1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPA1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPA1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPA1_08_01():
    result = parse("Malo-DPA1*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPA1*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPA1*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPA1_07_01():
    result = parse("Malo-DPA1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPA1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPA1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPA1_07_02():
    result = parse("Malo-DPA1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPA1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPA1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_57_02():
    result = parse("Mafa-A2*05:57:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:57:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:57:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPB1_10_01():
    result = parse("Malo-DPB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_60():
    result = parse("Mafa-A2*05:60")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:60") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:60") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPB1_15_01():
    result = parse("Malo-DPB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPB1_02_01():
    result = parse("Malo-DPB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_01_13():
    result = parse("Mafa-A4*01:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*01:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*01:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPB1_06_01():
    result = parse("Malo-DPB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DPB1_02_02():
    result = parse("Malo-DPB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DPB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DPB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_26_01():
    result = parse("Malo-DQA1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_24_03():
    result = parse("Malo-DQA1*24:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*24:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*24:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_05_01():
    result = parse("Malo-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_05_02():
    result = parse("Malo-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_01_01():
    result = parse("Malo-DQA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_01_02():
    result = parse("Malo-DQA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_24_01():
    result = parse("Malo-DQA1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_07():
    result = parse("Mafa-B*015:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_24_02():
    result = parse("Malo-DQA1*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_194_02():
    result = parse("Mafa-B*194:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*194:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*194:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_05_03():
    result = parse("Malo-DQA1*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_021_06():
    result = parse("Mafa-B*021:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*021:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*021:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQA1_26_02():
    result = parse("Malo-DQA1*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQA1*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQA1*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_022_01_02():
    result = parse("Mafa-B*022:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*022:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*022:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_16_01():
    result = parse("Malo-DQB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_06_01():
    result = parse("Malo-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_17_01():
    result = parse("Malo-DQB1*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_15_01():
    result = parse("Malo-DQB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_06_02():
    result = parse("Malo-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_20_02():
    result = parse("Mafa-B*030:20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:20:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_18_04():
    result = parse("Malo-DQB1*18:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*18:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*18:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_17_02():
    result = parse("Malo-DQB1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_04():
    result = parse("Mafa-B*041:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*041:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_18_01():
    result = parse("Malo-DQB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_10():
    result = parse("Mafa-B*044:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_18_02():
    result = parse("Malo-DQB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_11():
    result = parse("Mafa-B*044:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_18_03():
    result = parse("Malo-DQB1*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_15_02():
    result = parse("Malo-DQB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DQB1_06_03():
    result = parse("Malo-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_01_02():
    result = parse("Mafa-B*050:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*050:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRA_01_01_01():
    result = parse("Malo-DRA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DRA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRA_01_02():
    result = parse("Malo-DRA*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRA*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRA*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_02_03():
    result = parse("Mafa-B*050:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*050:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRA_01_04():
    result = parse("Malo-DRA*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRA*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRA*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_09():
    result = parse("Mafa-B*050:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*050:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRA_01_03():
    result = parse("Malo-DRA*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRA*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRA*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_10_01():
    result = parse("Mafa-B*050:10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*050:10:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRA_01_01_02():
    result = parse("Malo-DRA*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRA*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DRA*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_09():
    result = parse("Mafa-B*051:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRA_01_05():
    result = parse("Malo-DRA*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRA*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRA*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W020_01():
    result = parse("Malo-DRB*W020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W025_01():
    result = parse("Malo-DRB*W025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_05():
    result = parse("Mafa-B*117:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*117:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_10_01():
    result = parse("Malo-DRB1*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_058_02():
    result = parse("Mafa-B*058:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*058:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*058:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W067_01():
    result = parse("Malo-DRB*W067:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W067:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W067:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_03_01():
    result = parse("Malo-DRB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_15():
    result = parse("Mafa-B*060:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W072_01():
    result = parse("Malo-DRB*W072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_067_03():
    result = parse("Mafa-B*067:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*067:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*067:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W006_01():
    result = parse("Malo-DRB*W006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB3_04_01():
    result = parse("Malo-DRB3*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB3*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB3*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_08():
    result = parse("Mafa-B*069:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W002_01():
    result = parse("Malo-DRB*W002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_05_01_01():
    result = parse("Mafa-B*072:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*072:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_03_02():
    result = parse("Malo-DRB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_05_02():
    result = parse("Mafa-B*072:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*072:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_07_01():
    result = parse("Malo-DRB1*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_06():
    result = parse("Mafa-B*072:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_04_01():
    result = parse("Malo-DRB1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_074_03():
    result = parse("Mafa-B*074:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*074:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*074:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_04_02():
    result = parse("Malo-DRB1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W031_01():
    result = parse("Malo-DRB*W031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_081_05_01():
    result = parse("Mafa-B*081:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*081:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*081:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W004_01():
    result = parse("Malo-DRB*W004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_10_02():
    result = parse("Malo-DRB1*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_06():
    result = parse("Mafa-B*088:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*088:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W025_02():
    result = parse("Malo-DRB*W025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W033_01():
    result = parse("Malo-DRB*W033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_13():
    result = parse("Mafa-B*098:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_14():
    result = parse("Mafa-B*098:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_06_02():
    result = parse("Mafa-B*098:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_15():
    result = parse("Mafa-B*098:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_114_04():
    result = parse("Mafa-B*114:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*114:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*114:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_05_01_01():
    result = parse("Mafa-B*115:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*115:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRA_01_01_01():
    result = parse("Paan-DRA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DRA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_06():
    result = parse("Mafa-B*117:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*117:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRA_01_01_02():
    result = parse("Paan-DRA*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRA*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DRA*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_117_07():
    result = parse("Mafa-B*117:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*117:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*117:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRA_01_01_03():
    result = parse("Paan-DRA*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRA*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DRA*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_121_02():
    result = parse("Mafa-B*121:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*121:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*121:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRA_01_02_01():
    result = parse("Paan-DRA*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRA*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DRA*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_03_01_01():
    result = parse("Mafa-B*134:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*134:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRA_01_02_02():
    result = parse("Paan-DRA*01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRA*01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DRA*01:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_04():
    result = parse("Mafa-B*134:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*134:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRA_01_03():
    result = parse("Paan-DRA*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRA*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRA*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_146_01_02():
    result = parse("Mafa-B*146:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*146:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*146:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_06_01():
    result = parse("Paan-DQB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_18_01():
    result = parse("Paan-DQB1*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_151_03():
    result = parse("Mafa-B*151:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*151:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*151:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_02_01():
    result = parse("Paan-DPA1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_04_01():
    result = parse("Paan-DPA1*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_04_02():
    result = parse("Paan-DPA1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_04_03():
    result = parse("Paan-DPA1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_09_01():
    result = parse("Paan-DPA1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_06_01_01():
    result = parse("Mafa-E*02:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-E*02:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_07():
    result = parse("Mafa-E*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_14_01_01():
    result = parse("Paan-DPA1*14:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*14:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DPA1*14:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_08():
    result = parse("Mafa-E*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_14_01_02():
    result = parse("Paan-DPA1*14:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*14:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DPA1*14:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_09():
    result = parse("Mafa-E*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_15_01():
    result = parse("Paan-DPA1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_10():
    result = parse("Mafa-E*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_11():
    result = parse("Mafa-E*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_16_01():
    result = parse("Paan-DPA1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_12():
    result = parse("Mafa-E*02:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_03_01_01():
    result = parse("Paan-DPB1*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DPB1*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_36():
    result = parse("Mafa-I*01:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_37_01():
    result = parse("Mafa-I*01:37:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:37:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:37:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_03():
    result = parse("Mafa-I*01:14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_03_01_02():
    result = parse("Paan-DPB1*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DPB1*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_04():
    result = parse("Mafa-I*01:14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_03_02():
    result = parse("Paan-DPB1*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPB1*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_38_01():
    result = parse("Mafa-I*01:38:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:38:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:38:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_03_03():
    result = parse("Paan-DPB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_26_02():
    result = parse("Mafa-I*01:26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:26:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_06_01():
    result = parse("Paan-DPB1*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPB1*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_39():
    result = parse("Mafa-I*01:39")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:39") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:39") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_26_01():
    result = parse("Paan-DPB1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPB1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_40():
    result = parse("Mafa-I*01:40")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:40") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:40") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_41():
    result = parse("Mafa-I*01:41")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:41") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:41") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_01_02():
    result = parse("Chsa-A*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_42():
    result = parse("Mafa-I*01:42")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:42") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:42") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_06_02():
    result = parse("Chsa-A*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_43_01():
    result = parse("Mafa-I*01:43:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:43:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:43:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_06_03():
    result = parse("Chsa-A*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_05():
    result = parse("Mafa-I*01:14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_06_04():
    result = parse("Chsa-A*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_06():
    result = parse("Mafa-I*01:14:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_06_05():
    result = parse("Chsa-A*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_03_03():
    result = parse("Mafa-I*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_006_02():
    result = parse("Mafa-A1*006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_15_01():
    result = parse("Chsa-A*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_10_01():
    result = parse("Chsa-A*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_11_01():
    result = parse("Chsa-A*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_12_01():
    result = parse("Chsa-A*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_13_01():
    result = parse("Chsa-A*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_14_01():
    result = parse("Chsa-A*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_02_02():
    result = parse("Chsa-A*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_06_02():
    result = parse("Mafa-A1*060:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*060:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_17_01():
    result = parse("Chsa-A*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_062_06():
    result = parse("Mafa-A1*062:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*062:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*062:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_02_03():
    result = parse("Chsa-A*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_02_04():
    result = parse("Chsa-A*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_067_04():
    result = parse("Mafa-A1*067:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*067:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*067:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_03_06():
    result = parse("Chsa-A*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_06_02():
    result = parse("Paan-DQB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_06_03():
    result = parse("Paan-DQB1*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_06_04():
    result = parse("Paan-DQB1*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_01_02():
    result = parse("Chsa-B*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_06_05():
    result = parse("Paan-DQB1*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_11_03():
    result = parse("Chsa-B*11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*11:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_15_01():
    result = parse("Paan-DQB1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_11_04():
    result = parse("Chsa-B*11:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*11:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*11:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_15_02():
    result = parse("Paan-DQB1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_16_01():
    result = parse("Paan-DQB1*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_06():
    result = parse("Chsa-B*15:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_18_02():
    result = parse("Paan-DQB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_13_03():
    result = parse("Chsa-B*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_18_03():
    result = parse("Paan-DQB1*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_13_04():
    result = parse("Chsa-B*13:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*13:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*13:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQB1_18_04():
    result = parse("Paan-DQB1*18:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQB1*18:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQB1*18:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_13_05():
    result = parse("Chsa-B*13:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*13:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*13:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_19_01():
    result = parse("Chsa-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_02_12():
    result = parse("Mane-DPA1*02:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*02:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*02:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_19_02():
    result = parse("Chsa-B*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPA1_13_01():
    result = parse("Mane-DPA1*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPA1*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPA1*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_19_03():
    result = parse("Chsa-B*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_04_04():
    result = parse("Mane-DPB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_03_01_02():
    result = parse("Chsa-B*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DPB1_09_01():
    result = parse("Mane-DPB1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DPB1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DPB1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_03_01_03():
    result = parse("Chsa-B*03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*03:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DQA1_01_12():
    result = parse("Mane-DQA1*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DQA1*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DQA1*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_20_01():
    result = parse("Chsa-B*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_08():
    result = parse("Mane-DRA*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRA*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_14_02():
    result = parse("Chsa-B*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRA_01_09():
    result = parse("Mane-DRA*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRA*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRA*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_14_03():
    result = parse("Chsa-B*14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*14:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_DRB1_03_17():
    result = parse("Mane-DRB1*03:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-DRB1*03:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-DRB1*03:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_09_03_01():
    result = parse("Chsa-B*09:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*09:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*09:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_09_03_02():
    result = parse("Chsa-B*09:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*09:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*09:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_21_01():
    result = parse("Chsa-B*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_21_02():
    result = parse("Chsa-B*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_25_01():
    result = parse("Chsa-B*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_13_06():
    result = parse("Chsa-B*13:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*13:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*13:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_14_04():
    result = parse("Chsa-B*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_14_05():
    result = parse("Chsa-B*14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*14:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_28_01():
    result = parse("Chsa-B*28:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*28:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*28:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_14_06():
    result = parse("Chsa-B*14:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*14:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*14:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_07():
    result = parse("Chsa-B*15:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_31_01():
    result = parse("Chsa-B*31:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*31:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*31:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_32_01():
    result = parse("Chsa-B*32:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*32:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*32:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_33_01():
    result = parse("Chsa-B*33:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*33:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*33:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_34_01():
    result = parse("Chsa-B*34:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*34:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*34:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB_W036_01():
    result = parse("Paan-DRB*W036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB*W036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB*W036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DRB6_01_10():
    result = parse("Paan-DRB6*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DRB6*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DRB6*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_020_03():
    result = parse("BoLA-DQA*020:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*020:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*020:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_009_03():
    result = parse("BoLA-DRB3*009:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*009:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*009:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_015_04():
    result = parse("BoLA-DRB3*015:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*015:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*015:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_015_05():
    result = parse("BoLA-DRB3*015:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*015:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*015:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_015_06():
    result = parse("BoLA-DRB3*015:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*015:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*015:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_067_01():
    result = parse("BoLA-DRB3*067:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*067:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*067:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_068_01():
    result = parse("BoLA-DRB3*068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_069_01():
    result = parse("BoLA-DRB3*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_031_04():
    result = parse("BoLA-DRB3*031:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*031:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*031:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_070_01():
    result = parse("BoLA-DRB3*070:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*070:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*070:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_004_02():
    result = parse("BoLA-DRB3*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQB_001_01():
    result = parse("Bubu-DQB*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQB*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQB*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_002_01():
    result = parse("Bubu-DQA*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_021_01():
    result = parse("Bubu-DQA*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_022_01():
    result = parse("Bubu-DQA*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_02():
    result = parse("Bubu-DQA*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_023_01():
    result = parse("Bubu-DQA*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_024_01():
    result = parse("Bubu-DQA*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_029_01():
    result = parse("BoLA-DQA*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_027_03():
    result = parse("BoLA-DQA*027:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*027:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*027:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_013_02():
    result = parse("BoLA-DQB*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_029_03():
    result = parse("BoLA-DQB*029:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*029:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*029:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_07():
    result = parse("BoLA-DQB*018:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_044_02():
    result = parse("BoLA-2*044:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*044:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*044:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_001_03():
    result = parse("BoLA-DRB3*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_071_01():
    result = parse("BoLA-DRB3*071:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*071:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*071:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_029_02():
    result = parse("BoLA-DQB*029:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*029:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*029:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_036_02():
    result = parse("BoLA-DQB*036:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*036:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*036:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_030_01():
    result = parse("BoLA-DQA*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_08():
    result = parse("BoLA-DQB*018:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_004_03():
    result = parse("BoLA-3*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_017_03():
    result = parse("BoLA-3*017:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*017:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*017:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_026_03():
    result = parse("BoLA-2*026:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*026:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*026:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_031_02():
    result = parse("BoLA-1*031:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*031:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*031:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_032_02():
    result = parse("BoLA-2*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_050_02():
    result = parse("BoLA-3*050:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*050:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*050:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_057_01():
    result = parse("BoLA-2*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_058_01():
    result = parse("BoLA-3*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_059_01():
    result = parse("BoLA-3*059:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*059:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*059:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_060_01():
    result = parse("BoLA-2*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_061_01():
    result = parse("BoLA-1*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_062_01():
    result = parse("BoLA-2*062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_4_063_01():
    result = parse("BoLA-4*063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-4*063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-4*063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_5_064_01():
    result = parse("BoLA-5*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-5*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_065_01():
    result = parse("BoLA-3*065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_066_01():
    result = parse("BoLA-3*066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_066_02():
    result = parse("BoLA-3*066:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*066:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*066:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_067_01():
    result = parse("BoLA-1*067:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*067:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*067:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_068_01():
    result = parse("BoLA-3*068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_069_01():
    result = parse("BoLA-2*069:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*069:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*069:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_070_01():
    result = parse("BoLA-2*070:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*070:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*070:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_071_01():
    result = parse("BoLA-2*071:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*071:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*071:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_5_072_01():
    result = parse("BoLA-5*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-5*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_073_01():
    result = parse("BoLA-3*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_074_01():
    result = parse("BoLA-1*074:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*074:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*074:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_002_02SV():
    result = parse("BoLA-NC1*002:02SV")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*002:02SV") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*002:02SV") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_005_01():
    result = parse("BoLA-NC1*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_006_01():
    result = parse("BoLA-NC1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_007_01():
    result = parse("BoLA-NC1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC2_001_03():
    result = parse("BoLA-NC2*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC2*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC2*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC4_002_02():
    result = parse("BoLA-NC4*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC4*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC4*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC4_003_01():
    result = parse("BoLA-NC4*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC4*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC4*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC5_001_01():
    result = parse("BoLA-NC5*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC5*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC5*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_001_04():
    result = parse("BoLA-DQA*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_001_05():
    result = parse("BoLA-DQA*001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_05():
    result = parse("BoLA-DQA*002:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_008_03():
    result = parse("BoLA-DQA*008:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*008:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*008:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_06():
    result = parse("BoLA-DQA*002:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_027_04():
    result = parse("BoLA-DQA*027:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*027:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*027:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_031_01():
    result = parse("BoLA-DQA*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_09():
    result = parse("BoLA-DQB*018:09")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_041_01():
    result = parse("BoLA-DQB*041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_014_05():
    result = parse("BoLA-DQB*014:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*014:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*014:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_001_04():
    result = parse("BoLA-DQB*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_097_01():
    result = parse("DLA-DRB1*097:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*097:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*097:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_001_01():
    result = parse("DLA-88*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_003_01():
    result = parse("DLA-DQA1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_004_01():
    result = parse("DLA-DQA1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_005_01_1():
    result = parse("DLA-DQA1*005:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*005:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*005:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_005_01_2():
    result = parse("DLA-DQA1*005:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*005:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*005:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_006_01():
    result = parse("DLA-DQA1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_007_01():
    result = parse("DLA-DQA1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_008_01():
    result = parse("DLA-DQA1*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_005_01():
    result = parse("DLA-88*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_009_01():
    result = parse("DLA-DQA1*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_010_01():
    result = parse("DLA-DQA1*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_011_01():
    result = parse("DLA-DQA1*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_012_01_1():
    result = parse("DLA-DQA1*012:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*012:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*012:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_013_01():
    result = parse("DLA-DQA1*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_014_01_1():
    result = parse("DLA-DQA1*014:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*014:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*014:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_015_01():
    result = parse("DLA-DQA1*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_001_01():
    result = parse("DLA-DQB1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_002_01():
    result = parse("DLA-DQB1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_003_01():
    result = parse("DLA-DQB1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_004_01():
    result = parse("DLA-DQB1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_005_01():
    result = parse("DLA-DQB1*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_006_01():
    result = parse("DLA-88*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_005_02():
    result = parse("DLA-DQB1*005:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*005:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*005:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_007_01():
    result = parse("DLA-DQB1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_008_01_1():
    result = parse("DLA-DQB1*008:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*008:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQB1*008:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_008_01_2():
    result = parse("DLA-DQB1*008:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*008:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQB1*008:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_008_02():
    result = parse("DLA-DQB1*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_011_01():
    result = parse("DLA-DQB1*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_012_01():
    result = parse("DLA-DQB1*012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_013_01():
    result = parse("DLA-DQB1*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_013_02():
    result = parse("DLA-DQB1*013:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*013:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*013:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_013_03():
    result = parse("DLA-DQB1*013:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*013:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*013:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_014_01():
    result = parse("DLA-DQB1*014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_015_01():
    result = parse("DLA-DQB1*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_016_01():
    result = parse("DLA-DQB1*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_017_01():
    result = parse("DLA-DQB1*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_018_01():
    result = parse("DLA-DQB1*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_019_01():
    result = parse("DLA-DQB1*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_007_01():
    result = parse("DLA-88*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_020_01():
    result = parse("DLA-DQB1*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_020_02():
    result = parse("DLA-DQB1*020:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*020:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*020:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_021_01():
    result = parse("DLA-DQB1*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_022_01():
    result = parse("DLA-DQB1*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_023_01():
    result = parse("DLA-DQB1*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_024_01():
    result = parse("DLA-DQB1*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_026_01():
    result = parse("DLA-DQB1*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_027_01():
    result = parse("DLA-DQB1*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_028_01():
    result = parse("DLA-DQB1*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_030_01():
    result = parse("DLA-DQB1*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_031_01():
    result = parse("DLA-DQB1*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_032_01():
    result = parse("DLA-DQB1*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_033_01():
    result = parse("DLA-DQB1*033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_034_01():
    result = parse("DLA-DQB1*034:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*034:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*034:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_035_01():
    result = parse("DLA-DQB1*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_01():
    result = parse("DLA-DRB1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_02():
    result = parse("DLA-DRB1*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_008_01():
    result = parse("DLA-88*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_002_01():
    result = parse("DLA-DRB1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_002_02():
    result = parse("DLA-DRB1*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_003_01():
    result = parse("DLA-DRB1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_004_01():
    result = parse("DLA-DRB1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_005_01():
    result = parse("DLA-DRB1*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_006_01():
    result = parse("DLA-DRB1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_007_01():
    result = parse("DLA-DRB1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_008_01():
    result = parse("DLA-DRB1*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_008_02():
    result = parse("DLA-DRB1*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_009_01():
    result = parse("DLA-DRB1*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_010_01_1():
    result = parse("DLA-DRB1*010:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*010:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*010:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_010_01_2():
    result = parse("DLA-DRB1*010:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*010:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*010:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_011_01():
    result = parse("DLA-DRB1*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_012_01():
    result = parse("DLA-DRB1*012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_013_01():
    result = parse("DLA-DRB1*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_014_01():
    result = parse("DLA-DRB1*014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_015_01():
    result = parse("DLA-DRB1*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_015_02():
    result = parse("DLA-DRB1*015:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*015:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*015:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_009_01():
    result = parse("DLA-88*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_015_03():
    result = parse("DLA-DRB1*015:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*015:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*015:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_016_01():
    result = parse("DLA-DRB1*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_017_01():
    result = parse("DLA-DRB1*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_018_01():
    result = parse("DLA-DRB1*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_012_03():
    result = parse("DLA-DRB1*012:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*012:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*012:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_020_01():
    result = parse("DLA-DRB1*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_021_01():
    result = parse("DLA-DRB1*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_022_01():
    result = parse("DLA-DRB1*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_023_01():
    result = parse("DLA-DRB1*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_024_01():
    result = parse("DLA-DRB1*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_025_01():
    result = parse("DLA-DRB1*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_026_01():
    result = parse("DLA-DRB1*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_027_01():
    result = parse("DLA-DRB1*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_028_01():
    result = parse("DLA-DRB1*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_029_01():
    result = parse("DLA-DRB1*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_030_01():
    result = parse("DLA-DRB1*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_031_01():
    result = parse("DLA-DRB1*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_032_01():
    result = parse("DLA-DRB1*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_033_01_1():
    result = parse("DLA-DRB1*033:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*033:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*033:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_035_01():
    result = parse("DLA-DRB1*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_010_01():
    result = parse("DLA-88*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_036_01():
    result = parse("DLA-DRB1*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_037_01():
    result = parse("DLA-DRB1*037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_038_01():
    result = parse("DLA-DRB1*038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_032_02():
    result = parse("DLA-DRB1*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_040_01():
    result = parse("DLA-DRB1*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_041_01():
    result = parse("DLA-DRB1*041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_042_01():
    result = parse("DLA-DRB1*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_043_01():
    result = parse("DLA-DRB1*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_044_01():
    result = parse("DLA-DRB1*044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_045_01():
    result = parse("DLA-DRB1*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_046_01():
    result = parse("DLA-DRB1*046:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*046:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*046:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_047_01():
    result = parse("DLA-DRB1*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_013_01():
    result = parse("DLA-88*013:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*013:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*013:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_014_01():
    result = parse("DLA-88*014:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*014:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*014:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_015_01():
    result = parse("DLA-88*015:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*015:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*015:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_016_01():
    result = parse("DLA-88*016:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*016:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*016:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_002_01():
    result = parse("DLA-88*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_016_02():
    result = parse("DLA-88*016:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*016:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*016:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_017_01():
    result = parse("DLA-88*017:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*017:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*017:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_018_01():
    result = parse("DLA-88*018:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*018:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*018:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_019_01():
    result = parse("DLA-88*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_020_01():
    result = parse("DLA-88*020:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*020:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*020:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_022_01():
    result = parse("DLA-88*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_023_01():
    result = parse("DLA-88*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_024_01():
    result = parse("DLA-88*024:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*024:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*024:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_025_01():
    result = parse("DLA-88*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_026_01():
    result = parse("DLA-88*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_003_01():
    result = parse("DLA-88*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_027_01():
    result = parse("DLA-88*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_028_01():
    result = parse("DLA-88*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_028_02():
    result = parse("DLA-88*028:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*028:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*028:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_030_01():
    result = parse("DLA-88*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_031_01():
    result = parse("DLA-88*031:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*031:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*031:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_034_01():
    result = parse("DLA-88*034:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*034:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*034:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_035_01():
    result = parse("DLA-88*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_037_01():
    result = parse("DLA-88*037:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*037:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*037:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_038_01():
    result = parse("DLA-88*038:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*038:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*038:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_039_01():
    result = parse("DLA-88*039:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*039:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*039:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_004_01():
    result = parse("DLA-88*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_041_01():
    result = parse("DLA-88*041:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*041:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*041:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_042_01():
    result = parse("DLA-88*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_043_01():
    result = parse("DLA-88*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_044_01():
    result = parse("DLA-88*044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_048_01():
    result = parse("DLA-88*048:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*048:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*048:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_501_01():
    result = parse("DLA-88*501:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*501:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*501:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_502_01():
    result = parse("DLA-88*502:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*502:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*502:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_503_01():
    result = parse("DLA-88*503:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*503:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*503:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_504_01():
    result = parse("DLA-88*504:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*504:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*504:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_505_01():
    result = parse("DLA-88*505:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*505:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*505:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_004_02():
    result = parse("DLA-88*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_506_01():
    result = parse("DLA-88*506:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*506:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*506:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_507_01():
    result = parse("DLA-88*507:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*507:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*507:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_508_01():
    result = parse("DLA-88*508:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*508:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*508:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_001_01():
    result = parse("DLA-DQA1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_002_01():
    result = parse("DLA-DQA1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAA_0101():
    result = parse("Onmy-DAA*0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAA*0101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAA*0101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAA_0201():
    result = parse("Onmy-DAA*0201")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAA*0201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAA*0201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAA_0301():
    result = parse("Onmy-DAA*0301")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAA*0301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAA*0301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_0101():
    result = parse("Onmy-DAB*0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*0101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*0101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_0201():
    result = parse("Onmy-DAB*0201")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*0201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*0201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_030101():
    result = parse("Onmy-DAB*030101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*030101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-DAB*030101") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_030102():
    result = parse("Onmy-DAB*030102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*030102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-DAB*030102") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_0401():
    result = parse("Onmy-DAB*0401")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*0401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*0401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_050101():
    result = parse("Onmy-DAB*050101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*050101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-DAB*050101") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_050102():
    result = parse("Onmy-DAB*050102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*050102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-DAB*050102") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_0502():
    result = parse("Onmy-DAB*0502")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*0502") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*0502") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_060101():
    result = parse("Onmy-DAB*060101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*060101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-DAB*060101") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_060102():
    result = parse("Onmy-DAB*060102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*060102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-DAB*060102") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_0701():
    result = parse("Onmy-DAB*0701")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*0701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*0701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_0801():
    result = parse("Onmy-DAB*0801")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*0801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*0801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_0901():
    result = parse("Onmy-DAB*0901")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*0901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*0901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1001():
    result = parse("Onmy-DAB*1001")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1101():
    result = parse("Onmy-DAB*1101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1201():
    result = parse("Onmy-DAB*1201")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1301():
    result = parse("Onmy-DAB*1301")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1401():
    result = parse("Onmy-DAB*1401")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1501():
    result = parse("Onmy-DAB*1501")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1601():
    result = parse("Onmy-DAB*1601")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1602():
    result = parse("Onmy-DAB*1602")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1602") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1602") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_DAB_1701():
    result = parse("Onmy-DAB*1701")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-DAB*1701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-DAB*1701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_010101():
    result = parse("Onmy-UBA*010101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*010101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*010101") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_010102():
    result = parse("Onmy-UBA*010102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*010102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*010102") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_010103():
    result = parse("Onmy-UBA*010103")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*010103") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*010103") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0102():
    result = parse("Onmy-UBA*0102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0102") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0103():
    result = parse("Onmy-UBA*0103")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0103") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0103") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0104():
    result = parse("Onmy-UBA*0104")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0104") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0104") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0201():
    result = parse("Onmy-UBA*0201")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0202():
    result = parse("Onmy-UBA*0202")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0202") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0202") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0301():
    result = parse("Onmy-UBA*0301")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_040101():
    result = parse("Onmy-UBA*040101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*040101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*040101") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_040102():
    result = parse("Onmy-UBA*040102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*040102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*040102") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0501():
    result = parse("Onmy-UBA*0501")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0502():
    result = parse("Onmy-UBA*0502")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0502") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0502") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0503():
    result = parse("Onmy-UBA*0503")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0503") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0503") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0601():
    result = parse("Onmy-UBA*0601")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0701():
    result = parse("Onmy-UBA*0701")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0801():
    result = parse("Onmy-UBA*0801")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0901():
    result = parse("Onmy-UBA*0901")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_0902():
    result = parse("Onmy-UBA*0902")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*0902") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*0902") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1001():
    result = parse("Onmy-UBA*1001")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1101():
    result = parse("Onmy-UBA*1101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1201():
    result = parse("Onmy-UBA*1201")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1301():
    result = parse("Onmy-UBA*1301")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1401():
    result = parse("Onmy-UBA*1401")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1501():
    result = parse("Onmy-UBA*1501")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_150101():
    result = parse("Onmy-UBA*150101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*150101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*150101") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_150102():
    result = parse("Onmy-UBA*150102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*150102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*150102") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1502():
    result = parse("Onmy-UBA*1502")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1502") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1502") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1601():
    result = parse("Onmy-UBA*1601")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1701():
    result = parse("Onmy-UBA*1701")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1801():
    result = parse("Onmy-UBA*1801")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1802():
    result = parse("Onmy-UBA*1802")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1802") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1802") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_1901():
    result = parse("Onmy-UBA*1901")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*1901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*1901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2001():
    result = parse("Onmy-UBA*2001")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2101():
    result = parse("Onmy-UBA*2101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2201():
    result = parse("Onmy-UBA*2201")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2301():
    result = parse("Onmy-UBA*2301")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2401():
    result = parse("Onmy-UBA*2401")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2501():
    result = parse("Onmy-UBA*2501")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2601():
    result = parse("Onmy-UBA*2601")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_270101():
    result = parse("Onmy-UBA*270101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*270101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*270101") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_270102():
    result = parse("Onmy-UBA*270102")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*270102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*270102") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_270103():
    result = parse("Onmy-UBA*270103")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*270103") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Onmy-UBA*270103") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2801():
    result = parse("Onmy-UBA*2801")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_2901():
    result = parse("Onmy-UBA*2901")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*2901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*2901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_3001():
    result = parse("Onmy-UBA*3001")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*3001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*3001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_3101():
    result = parse("Onmy-UBA*3101")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*3101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*3101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Onmy_UBA_3201():
    result = parse("Onmy-UBA*3201")
    assert result.__class__ is Allele, (
        f'Expected parse("Onmy-UBA*3201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Onmy-UBA*3201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0101():
    result = parse("Sasa-DAA*0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0102():
    result = parse("Sasa-DAA*0102")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0102") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0201():
    result = parse("Sasa-DAA*0201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0301():
    result = parse("Sasa-DAA*0301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0302():
    result = parse("Sasa-DAA*0302")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0302") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0302") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_030301():
    result = parse("Sasa-DAA*030301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*030301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Sasa-DAA*030301") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_030302():
    result = parse("Sasa-DAA*030302")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*030302") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Sasa-DAA*030302") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0304():
    result = parse("Sasa-DAA*0304")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0304") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0304") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0305():
    result = parse("Sasa-DAA*0305")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0305") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0305") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0401():
    result = parse("Sasa-DAA*0401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0501():
    result = parse("Sasa-DAA*0501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0502():
    result = parse("Sasa-DAA*0502")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0502") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0502") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0601():
    result = parse("Sasa-DAA*0601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0701():
    result = parse("Sasa-DAA*0701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0801():
    result = parse("Sasa-DAA*0801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_0901():
    result = parse("Sasa-DAA*0901")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*0901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*0901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_1001():
    result = parse("Sasa-DAA*1001")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*1001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*1001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_1101():
    result = parse("Sasa-DAA*1101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*1101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*1101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_1201():
    result = parse("Sasa-DAA*1201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*1201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*1201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_1202():
    result = parse("Sasa-DAA*1202")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*1202") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*1202") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_1301():
    result = parse("Sasa-DAA*1301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*1301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*1301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAA_1401():
    result = parse("Sasa-DAA*1401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAA*1401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAA*1401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0101():
    result = parse("Sasa-DAB*0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0102():
    result = parse("Sasa-DAB*0102")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0102") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0201():
    result = parse("Sasa-DAB*0201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0202():
    result = parse("Sasa-DAB*0202")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0202") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0202") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0301():
    result = parse("Sasa-DAB*0301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0302():
    result = parse("Sasa-DAB*0302")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0302") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0302") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0303():
    result = parse("Sasa-DAB*0303")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0303") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0303") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0401():
    result = parse("Sasa-DAB*0401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0501():
    result = parse("Sasa-DAB*0501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0601():
    result = parse("Sasa-DAB*0601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0701():
    result = parse("Sasa-DAB*0701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0801():
    result = parse("Sasa-DAB*0801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0802():
    result = parse("Sasa-DAB*0802")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0802") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0802") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0901():
    result = parse("Sasa-DAB*0901")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_0902():
    result = parse("Sasa-DAB*0902")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*0902") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*0902") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1001():
    result = parse("Sasa-DAB*1001")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1101():
    result = parse("Sasa-DAB*1101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1102():
    result = parse("Sasa-DAB*1102")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1102") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1102") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1201():
    result = parse("Sasa-DAB*1201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1202():
    result = parse("Sasa-DAB*1202")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1202") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1202") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1301():
    result = parse("Sasa-DAB*1301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1302():
    result = parse("Sasa-DAB*1302")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1302") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1302") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1303():
    result = parse("Sasa-DAB*1303")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1303") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1303") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1401():
    result = parse("Sasa-DAB*1401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1501():
    result = parse("Sasa-DAB*1501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1502():
    result = parse("Sasa-DAB*1502")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1502") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1502") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1601():
    result = parse("Sasa-DAB*1601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1701():
    result = parse("Sasa-DAB*1701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1702():
    result = parse("Sasa-DAB*1702")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1702") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1702") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1703():
    result = parse("Sasa-DAB*1703")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1703") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1703") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1801():
    result = parse("Sasa-DAB*1801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_1901():
    result = parse("Sasa-DAB*1901")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*1901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*1901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2001():
    result = parse("Sasa-DAB*2001")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2101():
    result = parse("Sasa-DAB*2101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2201():
    result = parse("Sasa-DAB*2201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2301():
    result = parse("Sasa-DAB*2301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2302():
    result = parse("Sasa-DAB*2302")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2302") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2302") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2401():
    result = parse("Sasa-DAB*2401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2501():
    result = parse("Sasa-DAB*2501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2601():
    result = parse("Sasa-DAB*2601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2701():
    result = parse("Sasa-DAB*2701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_DAB_2801():
    result = parse("Sasa-DAB*2801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-DAB*2801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-DAB*2801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0101():
    result = parse("Sasa-UBA*0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0201():
    result = parse("Sasa-UBA*0201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0301():
    result = parse("Sasa-UBA*0301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0401():
    result = parse("Sasa-UBA*0401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0402():
    result = parse("Sasa-UBA*0402")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0402") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0402") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0501():
    result = parse("Sasa-UBA*0501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0601():
    result = parse("Sasa-UBA*0601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0602():
    result = parse("Sasa-UBA*0602")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0602") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0602") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0603():
    result = parse("Sasa-UBA*0603")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0603") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0603") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0701():
    result = parse("Sasa-UBA*0701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0801():
    result = parse("Sasa-UBA*0801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0901():
    result = parse("Sasa-UBA*0901")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1001():
    result = parse("Sasa-UBA*1001")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1101():
    result = parse("Sasa-UBA*1101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1201():
    result = parse("Sasa-UBA*1201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1301():
    result = parse("Sasa-UBA*1301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1401():
    result = parse("Sasa-UBA*1401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1501():
    result = parse("Sasa-UBA*1501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1601():
    result = parse("Sasa-UBA*1601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1701():
    result = parse("Sasa-UBA*1701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1801():
    result = parse("Sasa-UBA*1801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1802():
    result = parse("Sasa-UBA*1802")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1802") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1802") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_1901():
    result = parse("Sasa-UBA*1901")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*1901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*1901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2001():
    result = parse("Sasa-UBA*2001")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2002():
    result = parse("Sasa-UBA*2002")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2002") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2002") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2101():
    result = parse("Sasa-UBA*2101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2201():
    result = parse("Sasa-UBA*2201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2301():
    result = parse("Sasa-UBA*2301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2401():
    result = parse("Sasa-UBA*2401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2402():
    result = parse("Sasa-UBA*2402")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2402") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2402") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2403():
    result = parse("Sasa-UBA*2403")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2403") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2403") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2501():
    result = parse("Sasa-UBA*2501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2601():
    result = parse("Sasa-UBA*2601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2602():
    result = parse("Sasa-UBA*2602")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2602") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2602") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2701():
    result = parse("Sasa-UBA*2701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2801():
    result = parse("Sasa-UBA*2801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_2901():
    result = parse("Sasa-UBA*2901")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*2901") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*2901") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3001():
    result = parse("Sasa-UBA*3001")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3001") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3001") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3101():
    result = parse("Sasa-UBA*3101")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3101") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3101") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3201():
    result = parse("Sasa-UBA*3201")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3201") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3201") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3301():
    result = parse("Sasa-UBA*3301")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3301") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3301") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3401():
    result = parse("Sasa-UBA*3401")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3401") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3401") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A_av1():
    result = parse("Rano-A*av1")
    assert result.__class__ is Allele, f'Expected parse("Rano-A*av1") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A*av1") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A_f():
    result = parse("Rano-A*f")
    assert result.__class__ is Allele, f'Expected parse("Rano-A*f") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A*f") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A_g():
    result = parse("Rano-A*g")
    assert result.__class__ is Allele, f'Expected parse("Rano-A*g") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A*g") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A_k():
    result = parse("Rano-A*k")
    assert result.__class__ is Allele, f'Expected parse("Rano-A*k") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A*k") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A_l():
    result = parse("Rano-A*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-A*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A_u():
    result = parse("Rano-A*u")
    assert result.__class__ is Allele, f'Expected parse("Rano-A*u") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A*u") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_b():
    result = parse("Rano-A1*b")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*b") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*b") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_c():
    result = parse("Rano-A1*c")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*c") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*c") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_f():
    result = parse("Rano-A1*f")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*f") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*f") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_h():
    result = parse("Rano-A1*h")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*h") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*h") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_k():
    result = parse("Rano-A1*k")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*k") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*k") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_n():
    result = parse("Rano-A1*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_o():
    result = parse("Rano-A1*o")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*o") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*o") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A1_q():
    result = parse("Rano-A1*q")
    assert result.__class__ is Allele, f'Expected parse("Rano-A1*q") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A1*q") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A2_b():
    result = parse("Rano-A2*b")
    assert result.__class__ is Allele, f'Expected parse("Rano-A2*b") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A2*b") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A2_c():
    result = parse("Rano-A2*c")
    assert result.__class__ is Allele, f'Expected parse("Rano-A2*c") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A2*c") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A2_f():
    result = parse("Rano-A2*f")
    assert result.__class__ is Allele, f'Expected parse("Rano-A2*f") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A2*f") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A2_h():
    result = parse("Rano-A2*h")
    assert result.__class__ is Allele, f'Expected parse("Rano-A2*h") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A2*h") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A2_n():
    result = parse("Rano-A2*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-A2*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A2*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A2_o():
    result = parse("Rano-A2*o")
    assert result.__class__ is Allele, f'Expected parse("Rano-A2*o") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A2*o") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A2_q():
    result = parse("Rano-A2*q")
    assert result.__class__ is Allele, f'Expected parse("Rano-A2*q") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A2*q") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A3_n():
    result = parse("Rano-A3*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-A3*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A3*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_A3_o():
    result = parse("Rano-A3*o")
    assert result.__class__ is Allele, f'Expected parse("Rano-A3*o") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-A3*o") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_C113_l():
    result = parse("Rano-C113*l")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-C113*l") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-C113*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Cc9_c():
    result = parse("Rano-Cc9*c")
    assert result.__class__ is Allele, f'Expected parse("Rano-Cc9*c") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Cc9*c") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Cc22_c():
    result = parse("Rano-Cc22*c")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-Cc22*c") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Cc22*c") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Cc23_c():
    result = parse("Rano-Cc23*c")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-Cc23*c") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Cc23*c") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE1_c():
    result = parse("Rano-CE1*c")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE1*c") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE1*c") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE1_n():
    result = parse("Rano-CE1*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE1*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE1*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE2_n():
    result = parse("Rano-CE2*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE2*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE2*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE3_n():
    result = parse("Rano-CE3*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE3*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE3*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE4_n():
    result = parse("Rano-CE4*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE4*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE4*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE5_g():
    result = parse("Rano-CE5*g")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE5*g") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE5*g") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE5_l():
    result = parse("Rano-CE5*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE5*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE5*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE5_n():
    result = parse("Rano-CE5*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE5*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE5*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE7_n():
    result = parse("Rano-CE7*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-CE7*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE7*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE10_n():
    result = parse("Rano-CE10*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE10*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE10*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE11_n():
    result = parse("Rano-CE11*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE11*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE11*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE12_n():
    result = parse("Rano-CE12*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE12*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE12*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE13_n():
    result = parse("Rano-CE13*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE13*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE13*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE14_n():
    result = parse("Rano-CE14*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE14*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE14*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE15_n():
    result = parse("Rano-CE15*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE15*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE15*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE16_f():
    result = parse("Rano-CE16*f")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE16*f") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE16*f") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE16_g():
    result = parse("Rano-CE16*g")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE16*g") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE16*g") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_CE16_n():
    result = parse("Rano-CE16*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-CE16*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-CE16*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Clw2_l():
    result = parse("Rano-Clw2*l")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-Clw2*l") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Clw2*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_E_u():
    result = parse("Rano-E*u")
    assert result.__class__ is Allele, f'Expected parse("Rano-E*u") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-E*u") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_EC2_r21():
    result = parse("Rano-EC2*r21")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-EC2*r21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-EC2*r21") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_EC3_r21():
    result = parse("Rano-EC3*r21")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-EC3*r21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-EC3*r21") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_K_u():
    result = parse("Rano-K*u")
    assert result.__class__ is Allele, f'Expected parse("Rano-K*u") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-K*u") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_L3_l():
    result = parse("Rano-L3*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-L3*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-L3*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M1_2_n():
    result = parse("Rano-M1-2*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M1-2*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M1-2*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M1_4_n():
    result = parse("Rano-M1-4*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M1-4*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M1-4*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M1_5_n():
    result = parse("Rano-M1-5*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M1-5*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M1-5*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M2_lv1():
    result = parse("Rano-M2*lv1")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M2*lv1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M2*lv1") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M3_1_av1():
    result = parse("Rano-M3-1*av1")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M3-1*av1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M3-1*av1") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M3_1_n():
    result = parse("Rano-M3-1*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M3-1*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M3-1*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M5_n():
    result = parse("Rano-M5*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-M5*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M5*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M6_1_n():
    result = parse("Rano-M6-1*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M6-1*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M6-1*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M10_1_n():
    result = parse("Rano-M10-1*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M10-1*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M10-1*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_N2_n():
    result = parse("Rano-N2*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-N2*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-N2*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_N2_r21():
    result = parse("Rano-N2*r21")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-N2*r21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-N2*r21") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_N3_n():
    result = parse("Rano-N3*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-N3*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-N3*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_N3_r21():
    result = parse("Rano-N3*r21")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-N3*r21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-N3*r21") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_O1_n():
    result = parse("Rano-O1*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-O1*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-O1*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_S2_n():
    result = parse("Rano-S2*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-S2*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-S2*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_T18_n():
    result = parse("Rano-T18*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-T18*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-T18*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_U2_c():
    result = parse("Rano-U2*c")
    assert result.__class__ is Allele, f'Expected parse("Rano-U2*c") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-U2*c") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_9_5_f():
    result = parse("Rano-9.5*f")
    assert result.__class__ is Allele, f'Expected parse("Rano-9.5*f") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-9.5*f") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_9_6_f():
    result = parse("Rano-9.6*f")
    assert result.__class__ is Allele, f'Expected parse("Rano-9.6*f") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-9.6*f") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_119_l():
    result = parse("Rano-119*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-119*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-119*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Bb_l():
    result = parse("Rano-Bb*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-Bb*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Bb*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Bb_n():
    result = parse("Rano-Bb*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-Bb*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Bb*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Bb_u():
    result = parse("Rano-Bb*u")
    assert result.__class__ is Allele, f'Expected parse("Rano-Bb*u") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Bb*u") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Da_l():
    result = parse("Rano-Da*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-Da*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Da*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Da_n():
    result = parse("Rano-Da*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-Da*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Da*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Da_u():
    result = parse("Rano-Da*u")
    assert result.__class__ is Allele, f'Expected parse("Rano-Da*u") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Da*u") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Db1_a():
    result = parse("Rano-Db1*a")
    assert result.__class__ is Allele, f'Expected parse("Rano-Db1*a") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Db1*a") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Db1_l():
    result = parse("Rano-Db1*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-Db1*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Db1*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Db1_n():
    result = parse("Rano-Db1*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-Db1*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Db1*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_Db1_u():
    result = parse("Rano-Db1*u")
    assert result.__class__ is Allele, f'Expected parse("Rano-Db1*u") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-Db1*u") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_DMa_n():
    result = parse("Rano-DMa*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-DMa*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-DMa*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_DOa_n():
    result = parse("Rano-DOa*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-DOa*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-DOa*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_DOa_u():
    result = parse("Rano-DOa*u")
    assert result.__class__ is Allele, f'Expected parse("Rano-DOa*u") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-DOa*u") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_04_01_02():
    result = parse("SLA-1*04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-1*04:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_04_02():
    result = parse("SLA-1*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_04():
    result = parse("SLA-1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_06():
    result = parse("SLA-1*07:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_04():
    result = parse("SLA-1*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_06():
    result = parse("SLA-1*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_09():
    result = parse("SLA-1*08:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_10():
    result = parse("SLA-1*08:10")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_11():
    result = parse("SLA-1*08:11")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_14():
    result = parse("SLA-1*08:14")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_09_01():
    result = parse("SLA-1*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_01_02():
    result = parse("SLA-1*11:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-1*11:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_05():
    result = parse("SLA-1*11:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_07():
    result = parse("SLA-1*11:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_08():
    result = parse("SLA-1*11:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_09():
    result = parse("SLA-1*11:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_10():
    result = parse("SLA-1*11:10")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_14_03():
    result = parse("SLA-1*14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*14:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_15_01():
    result = parse("SLA-1*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_15_02():
    result = parse("SLA-1*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_16_03():
    result = parse("SLA-1*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_17_02():
    result = parse("SLA-1*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_19_01():
    result = parse("SLA-1*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_20_01():
    result = parse("SLA-1*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_22_01():
    result = parse("SLA-1*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_01_01():
    result = parse("SLA-2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_01_02():
    result = parse("SLA-2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_02_01():
    result = parse("SLA-2*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_02_03():
    result = parse("SLA-2*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_02_04():
    result = parse("SLA-2*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_02_05():
    result = parse("SLA-2*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_03():
    result = parse("SLA-2*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_04():
    result = parse("SLA-2*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_05():
    result = parse("SLA-2*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_06():
    result = parse("SLA-2*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_07():
    result = parse("SLA-2*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_08():
    result = parse("SLA-2*04:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_04_09():
    result = parse("SLA-2*04:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*04:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*04:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_07():
    result = parse("SLA-2*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_02_02():
    result = parse("SLA-2*06:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-2*06:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_04():
    result = parse("SLA-2*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_05():
    result = parse("SLA-2*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_07():
    result = parse("SLA-2*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_09():
    result = parse("SLA-2*06:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_10():
    result = parse("SLA-2*06:10")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_11():
    result = parse("SLA-2*06:11")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_12():
    result = parse("SLA-2*06:12")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_13():
    result = parse("SLA-2*06:13")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_14():
    result = parse("SLA-2*06:14")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_15():
    result = parse("SLA-2*06:15")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_07_04():
    result = parse("SLA-2*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_07_05():
    result = parse("SLA-2*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_08_01():
    result = parse("SLA-2*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_08_04():
    result = parse("SLA-2*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_08_06():
    result = parse("SLA-2*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_09_04():
    result = parse("SLA-2*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_09_06():
    result = parse("SLA-2*09:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*09:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*09:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_06():
    result = parse("SLA-2*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_07():
    result = parse("SLA-2*10:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_08():
    result = parse("SLA-2*10:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_11_02():
    result = parse("SLA-2*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_11_05():
    result = parse("SLA-2*11:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*11:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*11:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_13_02():
    result = parse("SLA-2*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_16_03():
    result = parse("SLA-2*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_18_01():
    result = parse("SLA-2*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_18_02():
    result = parse("SLA-2*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_18_03():
    result = parse("SLA-2*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_07():
    result = parse("SLA-3*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_10():
    result = parse("SLA-3*03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_04():
    result = parse("SLA-3*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_05():
    result = parse("SLA-3*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_06():
    result = parse("SLA-3*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_07():
    result = parse("SLA-3*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_04():
    result = parse("SLA-3*05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*05:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_06():
    result = parse("SLA-3*05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*05:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_5_01_01_01():
    result = parse("SLA-5*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-5*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-5*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_5_02_01():
    result = parse("SLA-5*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-5*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-5*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_7_01_01():
    result = parse("SLA-7*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-7*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-7*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_7_02_01():
    result = parse("SLA-7*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-7*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-7*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_7_03_01():
    result = parse("SLA-7*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-7*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-7*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_8_01_01():
    result = parse("SLA-8*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-8*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-8*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_8_02_01():
    result = parse("SLA-8*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-8*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-8*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_8_04_01():
    result = parse("SLA-8*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-8*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-8*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_8_05_01():
    result = parse("SLA-8*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-8*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-8*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_12_02_01():
    result = parse("SLA-12*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-12*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-12*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_12_04_01():
    result = parse("SLA-12*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-12*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-12*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_12_05_01():
    result = parse("SLA-12*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-12*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-12*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_12_06_01():
    result = parse("SLA-12*06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-12*06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-12*06:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRA_04_01():
    result = parse("SLA-DRA*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRA*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRA*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_07():
    result = parse("SLA-DRB1*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_08():
    result = parse("SLA-DRB1*04:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_06_06():
    result = parse("SLA-DRB1*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_07():
    result = parse("SLA-DRB1*11:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_08():
    result = parse("SLA-DRB1*11:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_12_01_01():
    result = parse("SLA-DRB1*12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*12:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_04():
    result = parse("SLA-DQA*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_05():
    result = parse("SLA-DQA*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_10():
    result = parse("SLA-DQB1*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_11():
    result = parse("SLA-DQB1*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_03_05():
    result = parse("SLA-DQB1*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_06_02_02():
    result = parse("SLA-DQB1*06:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*06:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*06:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_07_01_02():
    result = parse("SLA-DQB1*07:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*07:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*07:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_07_01_03():
    result = parse("SLA-DQB1*07:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*07:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQB1*07:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_08_04():
    result = parse("SLA-DQB1*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_08_05():
    result = parse("SLA-DQB1*08:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*08:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*08:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMA_05_01():
    result = parse("SLA-DMA*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMA*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DMA*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMB_01_01():
    result = parse("SLA-DMB*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMB*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DMB*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DOA_01_01():
    result = parse("SLA-DOA*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DOA*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DOA*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DOA_02_01():
    result = parse("SLA-DOA*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DOA*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DOA*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DOB1_01_01():
    result = parse("SLA-DOB1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DOB1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DOB1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DOB1_02_01():
    result = parse("SLA-DOB1*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DOB1*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DOB1*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_10_01():
    result = parse("SLA-DRB2*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_11_01():
    result = parse("SLA-DRB2*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB2_12_01():
    result = parse("SLA-DRB2*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB2*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB2*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_002_01_01():
    result = parse("Gaga-BF1*002:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*002:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*002:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_004_01_01():
    result = parse("Gaga-BF1*004:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*004:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*004:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_005_01_01():
    result = parse("Gaga-BF1*005:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*005:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*005:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_006_01_01():
    result = parse("Gaga-BF1*006:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*006:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*006:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_005_02_01():
    result = parse("Gaga-BF1*005:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*005:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*005:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_009_01_01():
    result = parse("Gaga-BF1*009:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*009:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*009:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_012_01_01():
    result = parse("Gaga-BF1*012:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*012:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*012:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_004_02_01():
    result = parse("Gaga-BF1*004:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*004:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*004:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_017_01_01():
    result = parse("Gaga-BF1*017:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*017:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*017:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_012_02_01():
    result = parse("Gaga-BF1*012:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*012:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*012:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_023_01_01():
    result = parse("Gaga-BF1*023:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*023:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*023:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF1_004_03_01():
    result = parse("Gaga-BF1*004:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF1*004:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF1*004:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_002_01_01():
    result = parse("Gaga-BF2*002:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*002:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*002:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_004_01_01():
    result = parse("Gaga-BF2*004:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*004:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*004:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_005_01_01():
    result = parse("Gaga-BF2*005:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*005:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*005:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_006_01_01():
    result = parse("Gaga-BF2*006:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*006:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*006:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_005_02_01():
    result = parse("Gaga-BF2*005:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*005:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*005:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_009_01_01():
    result = parse("Gaga-BF2*009:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*009:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*009:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_012_01_01():
    result = parse("Gaga-BF2*012:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*012:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*012:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_014_01_01():
    result = parse("Gaga-BF2*014:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*014:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*014:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_015_01_01():
    result = parse("Gaga-BF2*015:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*015:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*015:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_017_01_01():
    result = parse("Gaga-BF2*017:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*017:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*017:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_018_01_01():
    result = parse("Gaga-BF2*018:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*018:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*018:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_015_02_01():
    result = parse("Gaga-BF2*015:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*015:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*015:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_021_01_01():
    result = parse("Gaga-BF2*021:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*021:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*021:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_002_02_01():
    result = parse("Gaga-BF2*002:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*002:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*002:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gaga_BF2_024_01_01():
    result = parse("Gaga-BF2*024:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gaga-BF2*024:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gaga-BF2*024:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_04():
    result = parse("Ovar-DRB1*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_25_01():
    result = parse("Ovar-DRB1*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_04_03():
    result = parse("Ovar-DRB1*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_13_03():
    result = parse("Ovar-DRB1*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_01_05():
    result = parse("Ovar-DRB1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_11():
    result = parse("Ovar-DRB1*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_05():
    result = parse("Ovar-DRB1*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_12():
    result = parse("Ovar-DRB1*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_06():
    result = parse("Ovar-DRB1*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_01_06():
    result = parse("Ovar-DRB1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_07():
    result = parse("Ovar-DRB1*10:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_12_04():
    result = parse("Ovar-DRB1*12:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*12:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*12:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_17_04():
    result = parse("Ovar-DRB1*17:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*17:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*17:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_02_02():
    result = parse("Ovar-DRB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_17_03():
    result = parse("Ovar-DRB1*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_04_04():
    result = parse("Ovar-DRB1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_04_05():
    result = parse("Ovar-DRB1*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_06():
    result = parse("Ovar-DRB1*16:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_07():
    result = parse("Ovar-DRB1*16:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_08():
    result = parse("Ovar-DRB1*16:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_25_02():
    result = parse("Ovar-DRB1*25:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*25:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*25:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_08_06():
    result = parse("Ovar-DRB1*08:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*08:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*08:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_14_03():
    result = parse("Ovar-DRB1*14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*14:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_20_03():
    result = parse("Ovar-DRB1*20:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*20:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*20:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_08():
    result = parse("Ovar-DRB1*10:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_02_02():
    result = parse("Ovar-DRB1*16:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DRB1*16:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_12_01():
    result = parse("Ovar-N*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_18_02():
    result = parse("Ovar-DRB1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_13_01():
    result = parse("Ovar-N*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_14_01():
    result = parse("Ovar-N*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_15_01():
    result = parse("Ovar-N*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_16_01():
    result = parse("Ovar-N*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_17_01():
    result = parse("Ovar-N*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_18_01():
    result = parse("Ovar-N*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_19_01():
    result = parse("Ovar-N*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_20_01():
    result = parse("Ovar-N*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_21_01():
    result = parse("Ovar-N*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_13_02():
    result = parse("Ovar-N*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_06_02():
    result = parse("Ovar-DRB1*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_07_03():
    result = parse("Ovar-DRB1*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_26_01():
    result = parse("Ovar-DRB1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_09():
    result = parse("Ovar-DRB1*10:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_04_06():
    result = parse("Ovar-DRB1*04:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*04:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*04:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_12_01_02():
    result = parse("Ovar-DRB1*12:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*12:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DRB1*12:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_02_03():
    result = parse("Ovar-DRB1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_18_03():
    result = parse("Ovar-DRB1*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_03_13():
    result = parse("Ovar-DRB1*03:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*03:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*03:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_20_04():
    result = parse("Ovar-DRB1*20:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*20:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*20:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_09():
    result = parse("Ovar-DRB1*16:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_10():
    result = parse("Ovar-DRB1*10:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_14_04():
    result = parse("Ovar-DRB1*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_14_05():
    result = parse("Ovar-DRB1*14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*14:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_10():
    result = parse("Ovar-DRB1*16:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_08_03_02():
    result = parse("Ovar-DRB1*08:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*08:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DRB1*08:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_C_l():
    result = parse("Rano-C*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-C*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-C*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_L1_l():
    result = parse("Rano-L1*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-L1*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-L1*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_L2_l():
    result = parse("Rano-L2*l")
    assert result.__class__ is Allele, f'Expected parse("Rano-L2*l") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-L2*l") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M2_n():
    result = parse("Rano-M2*n")
    assert result.__class__ is Allele, f'Expected parse("Rano-M2*n") to be Allele but got {result}'
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M2*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Rano_M10_3_n():
    result = parse("Rano-M10-3*n")
    assert result.__class__ is Allele, (
        f'Expected parse("Rano-M10-3*n") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 1, (
        'Expected parse("Rano-M10-3*n") to have 1 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_01():
    result = parse("Patr-DPA1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_16():
    result = parse("Mamu-I*01:06:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:16") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_07():
    result = parse("Mamu-I*01:09:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:09:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_18():
    result = parse("Mamu-I*01:06:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:18") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_40():
    result = parse("Mamu-I*01:40")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:40") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:40") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_26_02():
    result = parse("Mamu-I*01:26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:26:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_17_01_01():
    result = parse("Mamu-A4*14:17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:17:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_04_01():
    result = parse("Mamu-A1*032:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*032:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_26_01():
    result = parse("Mamu-I*01:26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:26:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_04():
    result = parse("Mamu-B*065:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*065:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_33_01():
    result = parse("Mamu-I*01:33:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:33:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:33:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_149_02_01_01():
    result = parse("Mamu-B*149:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*149:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*149:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_03_05():
    result = parse("Chsa-A*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_23_01():
    result = parse("Chsa-A*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_20_01():
    result = parse("Chsa-A*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_21_01():
    result = parse("Chsa-A*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_22_01():
    result = parse("Chsa-A*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_22_02():
    result = parse("Chsa-A*22:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*22:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*22:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_24_01():
    result = parse("Chsa-A*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_22_04():
    result = parse("Chsa-A*22:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*22:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*22:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_35_01():
    result = parse("Chsa-B*35:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*35:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*35:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_35_02():
    result = parse("Chsa-B*35:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*35:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*35:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_35_03():
    result = parse("Chsa-B*35:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*35:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*35:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_37_01():
    result = parse("Chsa-B*37:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*37:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*37:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_38_01():
    result = parse("Chsa-B*38:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*38:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*38:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_39_01():
    result = parse("Chsa-B*39:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*39:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*39:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_37_02():
    result = parse("Chsa-B*37:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*37:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*37:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_40_01():
    result = parse("Chsa-B*40:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*40:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*40:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_40_02():
    result = parse("Chsa-B*40:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*40:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*40:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_37_03():
    result = parse("Chsa-B*37:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*37:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*37:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_36_02():
    result = parse("Chsa-B*36:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*36:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*36:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_41_01():
    result = parse("Chsa-B*41:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*41:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*41:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_134_02_01():
    result = parse("Mamu-B*134:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*134:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*134:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_18():
    result = parse("Mamu-A4*14:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_08_01():
    result = parse("Mamu-B*045:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*045:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_10_01_01():
    result = parse("Paan-B*10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*10:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_11_02_01():
    result = parse("Paan-B*11:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*11:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*11:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_12_02_01():
    result = parse("Paan-B*12:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*12:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*12:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_31_01():
    result = parse("Paan-B*31:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*31:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*31:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_50_01():
    result = parse("Mamu-A2*05:50:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:50:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:50:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_10_01():
    result = parse("Mamu-B*060:10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*060:10:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_07():
    result = parse("Math-DPB1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_19_02():
    result = parse("Paan-DPB1*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPB1*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_21_01():
    result = parse("Paan-DPB1*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPB1*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_02_02():
    result = parse("Paan-DPA1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_02_03():
    result = parse("Paan-DPA1*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_07_01_01():
    result = parse("Math-DPB1*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_08():
    result = parse("Math-DPB1*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_09():
    result = parse("Math-DPB1*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPB1_19_01():
    result = parse("Paan-DPB1*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPB1*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPB1*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_01_03():
    result = parse("Paan-DQA1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_01_04():
    result = parse("Paan-DQA1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_01_05():
    result = parse("Paan-DQA1*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_05_01():
    result = parse("Paan-DQA1*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_05_02():
    result = parse("Paan-DQA1*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_24_01():
    result = parse("Paan-DQA1*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_25_02_01():
    result = parse("Paan-DQA1*25:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*25:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DQA1*25:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_25_02_02():
    result = parse("Paan-DQA1*25:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*25:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-DQA1*25:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DQA1_26_01():
    result = parse("Paan-DQA1*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DQA1*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DQA1*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_DPA1_14_02():
    result = parse("Paan-DPA1*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-DPA1*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-DPA1*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_06():
    result = parse("Patr-DPA1*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_04():
    result = parse("Patr-DPA1*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_02():
    result = parse("Patr-DPA1*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_08():
    result = parse("Patr-DPA1*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_07():
    result = parse("Patr-DPA1*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_03_02_02():
    result = parse("Patr-DQB1*03:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*03:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQB1*03:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_03_03_02():
    result = parse("Patr-DQB1*03:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*03:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQB1*03:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_17_01():
    result = parse("Papa-B*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_11_01():
    result = parse("Papa-B*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_19_01():
    result = parse("Papa-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_09_01():
    result = parse("Papa-B*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_12_01():
    result = parse("Papa-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_13_01():
    result = parse("Papa-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_10_01():
    result = parse("Papa-B*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_17_02():
    result = parse("Papa-B*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_19_02():
    result = parse("Papa-B*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_02_02():
    result = parse("Papa-B*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_08_01():
    result = parse("Papa-B*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_18_01():
    result = parse("Papa-B*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_20_01():
    result = parse("Papa-B*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_15_01():
    result = parse("Papa-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_12_02():
    result = parse("Papa-B*12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*12:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_14_01():
    result = parse("Papa-B*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_08_02():
    result = parse("Papa-B*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_16_01():
    result = parse("Papa-B*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_09():
    result = parse("Patr-DPA1*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_03():
    result = parse("Patr-DPA1*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_05_02_02():
    result = parse("Patr-DQA1*05:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*05:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQA1*05:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_20_04_01():
    result = parse("Patr-DQA1*20:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*20:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQA1*20:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQA1_20_04_02():
    result = parse("Patr-DQA1*20:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQA1*20:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DQA1*20:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_16():
    result = parse("Patr-DPB1*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_17():
    result = parse("Patr-DPB1*01:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_18():
    result = parse("Patr-DPB1*01:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_01_03():
    result = parse("Paan-A*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_02_03():
    result = parse("Paan-A*02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*02:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_14_04():
    result = parse("Paan-A*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_18_01():
    result = parse("Paan-A*18:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*18:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*18:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_19_01():
    result = parse("Paan-A*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_21_01():
    result = parse("Paan-A*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_22_01():
    result = parse("Paan-A*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_08():
    result = parse("Paan-B*02:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_09():
    result = parse("Paan-B*02:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_10():
    result = parse("Paan-B*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_04_01_01():
    result = parse("Paan-B*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_04_02():
    result = parse("Paan-B*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_04_03():
    result = parse("Paan-B*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_05_01():
    result = parse("Paan-B*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_09_03():
    result = parse("Paan-B*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_09_04():
    result = parse("Paan-B*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_10_01_02():
    result = parse("Paan-B*10:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*10:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*10:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_10_03():
    result = parse("Paan-B*10:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*10:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*10:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_10_04():
    result = parse("Paan-B*10:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*10:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*10:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_11_02_02():
    result = parse("Paan-B*11:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*11:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*11:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_12_02_02():
    result = parse("Paan-B*12:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*12:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*12:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_12_03():
    result = parse("Paan-B*12:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*12:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*12:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_15_04():
    result = parse("Paan-B*15:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*15:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*15:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_20_03():
    result = parse("Paan-B*20:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*20:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*20:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_23_02():
    result = parse("Paan-B*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_26_03():
    result = parse("Paan-B*26:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*26:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*26:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_32_01():
    result = parse("Paan-B*32:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*32:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*32:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_33_01():
    result = parse("Paan-B*33:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*33:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*33:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_33_02():
    result = parse("Paan-B*33:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*33:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*33:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_34_01():
    result = parse("Paan-B*34:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*34:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*34:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_35_01():
    result = parse("Paan-B*35:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*35:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*35:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_36_01():
    result = parse("Paan-B*36:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*36:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*36:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_37_01():
    result = parse("Paan-B*37:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*37:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*37:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_38_01():
    result = parse("Paan-B*38:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*38:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*38:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_39_01():
    result = parse("Paan-B*39:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*39:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*39:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_39_02():
    result = parse("Paan-B*39:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*39:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*39:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_39_03():
    result = parse("Paan-B*39:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*39:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*39:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_40_01():
    result = parse("Paan-B*40:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*40:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*40:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_41_01():
    result = parse("Paan-B*41:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*41:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*41:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_41_02():
    result = parse("Paan-B*41:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*41:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*41:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_41_03():
    result = parse("Paan-B*41:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*41:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*41:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_41_04():
    result = parse("Paan-B*41:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*41:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*41:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_42_01():
    result = parse("Paan-B*42:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*42:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*42:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_43_01():
    result = parse("Paan-B*43:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*43:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*43:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_44_01():
    result = parse("Paan-B*44:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*44:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*44:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_001_06_01_01():
    result = parse("Mamu-A1*001:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*001:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*001:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_095_01():
    result = parse("Mamu-A1*095:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*095:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*095:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_15():
    result = parse("Mamu-A1*003:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_16():
    result = parse("Mamu-A1*003:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_17():
    result = parse("Mamu-A1*003:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_02():
    result = parse("Mamu-A1*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_12():
    result = parse("Mamu-A1*018:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_027_01():
    result = parse("Mamu-A1*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_032_04_02_01():
    result = parse("Mamu-A1*032:04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*032:04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*032:04:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_056_05():
    result = parse("Mamu-A1*056:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*056:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*056:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_065_03():
    result = parse("Mamu-A1*065:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*065:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*065:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_065_04():
    result = parse("Mamu-A1*065:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*065:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*065:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_065_05():
    result = parse("Mamu-A1*065:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*065:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*065:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_52():
    result = parse("Mamu-A2*05:52")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:52") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:52") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_53():
    result = parse("Mamu-A2*05:53")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:53") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:53") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_54():
    result = parse("Mamu-A2*05:54")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:54") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A2*05:54") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_50_02():
    result = parse("Mamu-A2*05:50:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:50:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:50:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_19():
    result = parse("Mamu-A4*14:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A4*14:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_03_04_01():
    result = parse("Mamu-A4*14:03:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:03:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:03:04:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_20_01_01():
    result = parse("Mamu-A4*14:20:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:20:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:20:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_17_02():
    result = parse("Mamu-A4*14:17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A4*14:17:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_05_02():
    result = parse("Mamu-A4*14:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A4*14:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_21_01_01():
    result = parse("Mamu-A4*14:21:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:21:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:21:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_22_01_01():
    result = parse("Mamu-A4*14:22:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:22:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:22:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A5_30_04_03():
    result = parse("Mamu-A5*30:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5*30:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A5*30:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A6_01_08():
    result = parse("Mamu-A6*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A6*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_05():
    result = parse("Mamu-B*001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_06():
    result = parse("Mamu-B*007:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*007:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_011_04():
    result = parse("Mamu-B*011:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*011:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*011:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_036_05():
    result = parse("Mamu-B*036:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*036:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*036:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_041_02():
    result = parse("Mamu-B*041:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*041:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*041:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_08_02():
    result = parse("Mamu-B*045:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*045:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_01_04():
    result = parse("Mamu-B*046:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*046:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_20_01_01():
    result = parse("Mamu-B*046:20:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:20:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:20:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_21():
    result = parse("Mamu-B*046:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_15_01_01():
    result = parse("Mamu-B*051:15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:15:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_16_01_01():
    result = parse("Mamu-B*051:16:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:16:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:16:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_17_01_01():
    result = parse("Mamu-B*051:17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:17:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_18():
    result = parse("Mamu-B*051:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_19():
    result = parse("Mamu-B*051:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_20_01_01():
    result = parse("Mamu-B*051:20:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:20:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:20:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_04_02_01():
    result = parse("Mamu-B*051:04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:04:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_21_01_01():
    result = parse("Mamu-B*051:21:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:21:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:21:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_22():
    result = parse("Mamu-B*051:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_23():
    result = parse("Mamu-B*051:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_07():
    result = parse("Mamu-B*056:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*056:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_13():
    result = parse("Mamu-B*060:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_10_02():
    result = parse("Mamu-B*060:10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*060:10:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_14():
    result = parse("Mamu-B*060:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_05_01_01():
    result = parse("Mamu-B*065:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*065:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_06_01_01():
    result = parse("Mamu-B*065:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*065:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_03_03():
    result = parse("Mamu-B*065:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*065:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_08():
    result = parse("Mamu-B*068:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*068:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_070_06():
    result = parse("Mamu-B*070:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*070:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*070:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_070_07_01_01():
    result = parse("Mamu-B*070:07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*070:07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*070:07:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_03():
    result = parse("Mamu-B*072:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*072:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_08():
    result = parse("Mamu-B*072:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*072:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_04():
    result = parse("Mamu-B*072:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*072:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_05():
    result = parse("Mamu-B*072:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*072:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_088_02():
    result = parse("Mamu-B*088:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*088:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*088:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_05():
    result = parse("Mamu-B*079:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*079:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_087_01_02():
    result = parse("Mamu-B*087:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*087:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*087:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_088_01_03():
    result = parse("Mamu-B*088:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*088:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*088:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_088_03():
    result = parse("Mamu-B*088:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*088:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*088:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_088_04():
    result = parse("Mamu-B*088:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*088:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*088:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_093_02():
    result = parse("Mamu-B*093:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*093:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*093:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_097_01_02_01():
    result = parse("Mamu-B*097:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*097:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*097:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_01_03():
    result = parse("Mamu-B*098:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*098:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_08_01_01():
    result = parse("Mamu-B*098:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_06_03_01():
    result = parse("Mamu-B*098:06:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:06:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:06:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_204_01():
    result = parse("Mamu-B*204:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*204:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*204:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_204_02():
    result = parse("Mamu-B*204:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*204:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*204:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_110_01():
    result = parse("Mamu-B*110:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*110:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*110:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_116_02():
    result = parse("Mamu-B*116:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*116:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*116:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_134_02_02():
    result = parse("Mamu-B*134:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*134:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*134:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_149_02_02():
    result = parse("Mamu-B*149:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*149:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*149:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_180_02():
    result = parse("Mamu-B*180:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*180:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*180:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_22():
    result = parse("Mamu-E*02:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_23():
    result = parse("Mamu-E*02:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_39():
    result = parse("Mamu-I*01:39")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:39") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:39") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_17():
    result = parse("Mamu-I*01:06:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:17") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_05():
    result = parse("Mamu-I*01:09:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:09:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_19():
    result = parse("Mamu-I*01:06:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:19") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_20():
    result = parse("Mamu-I*01:06:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:20") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_06_01():
    result = parse("Mamu-I*01:09:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:09:06:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_20_03_01():
    result = parse("Mamu-I*01:20:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:20:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:20:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_41():
    result = parse("Mamu-I*01:41")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:41") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:41") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_33_02():
    result = parse("Mamu-I*01:33:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:33:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:33:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_42():
    result = parse("Mamu-I*01:42")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:42") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:42") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_43():
    result = parse("Mamu-I*01:43")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:43") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:43") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_09_08():
    result = parse("Mamu-I*01:09:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:09:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:09:08") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_44():
    result = parse("Mamu-I*01:44")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:44") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:44") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_20_01():
    result = parse("Paan-A*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_26_02():
    result = parse("Paan-B*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_8_03_01():
    result = parse("SLA-8*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-8*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-8*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_39_04():
    result = parse("Paan-B*39:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*39:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*39:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_06_02():
    result = parse("Paan-B*02:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_02_04():
    result = parse("Paan-B*02:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_45_01():
    result = parse("Paan-B*45:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*45:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*45:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_46_01():
    result = parse("Paan-B*46:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*46:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*46:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_41_05():
    result = parse("Paan-B*41:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*41:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*41:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_47_01():
    result = parse("Paan-B*47:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*47:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*47:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_26_04():
    result = parse("Paan-B*26:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*26:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*26:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_48_01N():
    result = parse("Paan-B*48:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*48:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*48:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_04_04():
    result = parse("Paan-B*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_24_03():
    result = parse("Paan-B*24:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*24:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*24:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_22_03():
    result = parse("Chsa-A*22:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*22:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*22:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_36_01():
    result = parse("Chsa-B*36:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*36:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*36:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_03_03_02():
    result = parse("Chsa-A*03:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*03:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-A*03:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_25_01():
    result = parse("Chsa-A*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_06_06():
    result = parse("Chsa-A*06:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*06:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*06:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_06_07():
    result = parse("Chsa-A*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_26_01():
    result = parse("Chsa-A*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_17_02():
    result = parse("Chsa-A*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_03_01_04():
    result = parse("Chsa-B*03:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*03:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*03:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_09_04():
    result = parse("Chsa-B*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_42_01():
    result = parse("Chsa-B*42:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*42:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*42:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_13_07():
    result = parse("Chsa-B*13:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*13:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*13:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_14_07():
    result = parse("Chsa-B*14:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*14:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*14:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_43_01():
    result = parse("Chsa-B*43:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*43:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*43:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_44_01():
    result = parse("Chsa-B*44:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*44:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*44:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_45_01():
    result = parse("Chsa-B*45:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*45:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*45:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_08():
    result = parse("Chsa-B*15:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_15_02():
    result = parse("Chsa-A*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_A_23_02():
    result = parse("Chsa-A*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-A*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-A*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_02_02():
    result = parse("Chsa-B*15:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*15:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_09():
    result = parse("Chsa-B*15:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_10():
    result = parse("Chsa-B*15:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*15:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_32_02():
    result = parse("Chsa-B*32:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*32:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*32:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_32_03():
    result = parse("Chsa-B*32:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*32:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-B*32:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_01_01_02():
    result = parse("Gogo-A*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-A*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_04_01_01_02():
    result = parse("Gogo-A*04:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*04:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*04:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_04_01_01_03():
    result = parse("Gogo-A*04:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*04:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*04:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_04_01_01_04():
    result = parse("Gogo-A*04:01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*04:01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*04:01:01:04") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_04_01_01_05():
    result = parse("Gogo-A*04:01:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*04:01:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*04:01:01:05") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_07_01_01_01():
    result = parse("Gogo-A*07:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*07:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*07:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_07_01_01_02():
    result = parse("Gogo-A*07:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*07:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*07:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_OKO_01_02():
    result = parse("Gogo-OKO*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-OKO*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-OKO*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_OKO_02_01():
    result = parse("Gogo-OKO*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-OKO*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-OKO*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_OKO_02_02():
    result = parse("Gogo-OKO*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-OKO*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-OKO*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_05_02_01_01N():
    result = parse("Gogo-A*05:02:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*05:02:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*05:02:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_05_02_01_02N():
    result = parse("Gogo-A*05:02:01:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*05:02:01:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*05:02:01:02N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_05_03_01_01N():
    result = parse("Gogo-A*05:03:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*05:03:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*05:03:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_05_03_01_02N():
    result = parse("Gogo-A*05:03:01:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*05:03:01:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*05:03:01:02N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_05_03_01_03N():
    result = parse("Gogo-A*05:03:01:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*05:03:01:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-A*05:03:01:03N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_A_05_04N():
    result = parse("Gogo-A*05:04N")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-A*05:04N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-A*05:04N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_01_01_01_02():
    result = parse("Gogo-B*01:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*01:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-B*01:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_01_04_01_01():
    result = parse("Gogo-B*01:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*01:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-B*01:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_01_04_01_02():
    result = parse("Gogo-B*01:04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*01:04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-B*01:04:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_03_02():
    result = parse("Gogo-B*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_03_03():
    result = parse("Gogo-B*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_04_01_01_02():
    result = parse("Gogo-B*04:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*04:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-B*04:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_07_02():
    result = parse("Gogo-B*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_07_03():
    result = parse("Gogo-B*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_07_04():
    result = parse("Gogo-B*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_B_12_01():
    result = parse("Gogo-B*12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-B*12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-B*12:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_01_01_03():
    result = parse("Gogo-C*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Gogo-C*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_01_04():
    result = parse("Gogo-C*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-C*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_02_04_01_02():
    result = parse("Gogo-C*02:04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*02:04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-C*02:04:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_02_05():
    result = parse("Gogo-C*02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-C*02:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gobe_OKO_01_01():
    result = parse("Gobe-OKO*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gobe-OKO*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gobe-OKO*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gobe_B_13_01():
    result = parse("Gobe-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gobe-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gobe-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gobe_C_02_01():
    result = parse("Gobe-C*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gobe-C*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gobe-C*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_B_15_02_03():
    result = parse("Chsa-B*15:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-B*15:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chsa-B*15:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chsa_AG_01_01():
    result = parse("Chsa-AG*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chsa-AG*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chsa-AG*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_01_04():
    result = parse("Mamu-A1*008:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*008:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_02_06_01():
    result = parse("Mamu-A2*05:02:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:02:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A2*05:02:06:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_06_01_01():
    result = parse("Mamu-B*001:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*001:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_01_03_01():
    result = parse("Mamu-B*019:01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*019:01:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_022_02():
    result = parse("Mamu-B*022:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*022:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*022:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_09_01_01():
    result = parse("Mamu-B*098:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_10_01_01():
    result = parse("Mamu-B*098:10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:10:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_11():
    result = parse("Mamu-B*098:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*098:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_134_04_01_01():
    result = parse("Mamu-B*134:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*134:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*134:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_45_01_01():
    result = parse("Mamu-I*01:45:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:45:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:45:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_46():
    result = parse("Mamu-I*01:46")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:46") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:46") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_06_03():
    result = parse("Mafa-DRB1*03:06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:06:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_37():
    result = parse("Mafa-DRB1*03:37")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:37") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:37") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_16():
    result = parse("Mafa-DRB1*04:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_17():
    result = parse("Mafa-DRB1*04:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_087_01():
    result = parse("Mane-A1*087:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*087:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*087:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_27():
    result = parse("Mane-A2*05:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A3_13_14_02():
    result = parse("Mane-A3*13:14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A3*13:14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-A3*13:14:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_019_02():
    result = parse("Mane-B*019:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*019:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*019:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_032_02():
    result = parse("Mane-B*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_08():
    result = parse("Mane-B*051:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*051:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_051_09():
    result = parse("Mane-B*051:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*051:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*051:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_068_13():
    result = parse("Mane-B*068:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*068:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*068:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_079_09():
    result = parse("Mane-B*079:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*079:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*079:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_112_04():
    result = parse("Mane-B*112:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*112:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*112:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_112_05():
    result = parse("Mane-B*112:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*112:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*112:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_150_03():
    result = parse("Mane-B*150:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*150:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*150:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_199_02():
    result = parse("Mane-B*199:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*199:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*199:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_36():
    result = parse("Mane-I*01:36")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:36") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-I*01:36") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_004_03():
    result = parse("Mane-A1*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_019_07():
    result = parse("Mane-A1*019:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*019:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*019:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_061_02():
    result = parse("Mane-A1*061:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*061:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*061:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_084_06():
    result = parse("Mane-A1*084:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*084:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*084:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_090_01():
    result = parse("Mane-A1*090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A1_114_03():
    result = parse("Mane-A1*114:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A1*114:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A1*114:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_A2_05_28():
    result = parse("Mane-A2*05:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-A2*05:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-A2*05:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_030_06():
    result = parse("Mane-B*030:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*030:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*030:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_043_02():
    result = parse("Mane-B*043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_060_06():
    result = parse("Mane-B*060:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*060:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*060:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_078_06():
    result = parse("Mane-B*078:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*078:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*078:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_103_05():
    result = parse("Mane-B*103:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*103:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mane-B*103:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_B_120_02_02():
    result = parse("Mane-B*120:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-B*120:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-B*120:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mane_I_01_23_02():
    result = parse("Mane-I*01:23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mane-I*01:23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mane-I*01:23:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_08_02():
    result = parse("Paan-A*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_01_04():
    result = parse("Paan-A*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_23_01():
    result = parse("Paan-A*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_24_01():
    result = parse("Paan-A*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_25_01():
    result = parse("Paan-A*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_26_01():
    result = parse("Paan-A*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_27_01():
    result = parse("Paan-A*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_11_03():
    result = parse("Paan-B*11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*11:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_37_02():
    result = parse("Paan-B*37:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*37:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*37:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_18_02():
    result = parse("Paan-B*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_02_05():
    result = parse("Paan-B*02:02:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:02:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:02:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_09_05():
    result = parse("Paan-B*09:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*09:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*09:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_29_02():
    result = parse("Paan-B*29:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*29:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*29:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_04_05():
    result = parse("Paan-B*04:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*04:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*04:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_21_02():
    result = parse("Paan-B*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_32_02():
    result = parse("Paan-B*32:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*32:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*32:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_27_02():
    result = parse("Paan-B*27:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*27:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*27:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_27_03():
    result = parse("Paan-B*27:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*27:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*27:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_12():
    result = parse("Mafa-DRB*W001:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_08():
    result = parse("Mafa-DRB*W002:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W002:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_11():
    result = parse("Paan-B*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_18_03():
    result = parse("Paan-B*18:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*18:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*18:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_10():
    result = parse("Mafa-DRB*W004:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_08_02():
    result = parse("Paan-B*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_20_04():
    result = parse("Paan-B*20:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*20:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*20:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_49_01():
    result = parse("Paan-B*49:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*49:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*49:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_50_01():
    result = parse("Paan-B*50:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*50:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*50:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W005_04():
    result = parse("Mafa-DRB*W005:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W005:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W005:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_51_01():
    result = parse("Paan-B*51:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*51:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*51:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_12_01():
    result = parse("Mafa-DRB*W006:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W006:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_52_01():
    result = parse("Paan-B*52:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*52:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*52:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_12_02():
    result = parse("Mafa-DRB*W006:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W006:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_53_01():
    result = parse("Paan-B*53:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*53:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*53:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_54_01():
    result = parse("Paan-B*54:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*54:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*54:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_55_01():
    result = parse("Paan-B*55:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*55:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*55:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_56_01():
    result = parse("Paan-B*56:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*56:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*56:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_55_02():
    result = parse("Paan-B*55:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*55:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*55:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_07_03():
    result = parse("Paan-B*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_12():
    result = parse("Paan-B*02:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_31_02():
    result = parse("Paan-B*31:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*31:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*31:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_57_01():
    result = parse("Paan-B*57:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*57:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*57:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_58_01():
    result = parse("Paan-B*58:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*58:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*58:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_59_01():
    result = parse("Paan-B*59:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*59:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*59:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_31_03():
    result = parse("Paan-B*31:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*31:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*31:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_13():
    result = parse("Mafa-DRB*W006:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W020_04():
    result = parse("Mafa-DRB*W020:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W020:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W020:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W025_07():
    result = parse("Mafa-DRB*W025:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W025:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W025:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W026_05():
    result = parse("Mafa-DRB*W026:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W026:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W026:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W066_03():
    result = parse("Mafa-DRB*W066:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W066:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W066:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W067_02():
    result = parse("Mafa-DRB*W067:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W067:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W067:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB3_04_11():
    result = parse("Mafa-DRB3*04:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB3*04:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB3*04:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_15_02():
    result = parse("Mafa-DRB1*04:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W104_01():
    result = parse("Mafa-DRB*W104:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W104:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W104:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_02_01_05():
    result = parse("Mafa-DRA*02:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*02:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*02:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_18():
    result = parse("Mafa-DRB1*04:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_11_02():
    result = parse("Mafa-DRB1*04:11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:11:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_13():
    result = parse("Mafa-DRB*W001:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_14():
    result = parse("Mafa-DRB*W001:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_16():
    result = parse("Mafa-DRB*W001:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W020_01_02():
    result = parse("Mafa-DRB*W020:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W020:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W020:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_13():
    result = parse("Mafa-DQA1*26:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_06_02():
    result = parse("Mafa-DQB1*06:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_44():
    result = parse("Mafa-DQB1*06:44")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:44") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:44") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_15_03_02():
    result = parse("Mafa-DQB1*15:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*15:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*15:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_18_32():
    result = parse("Mafa-DQB1*18:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*18:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*18:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_14_02():
    result = parse("Mafa-DPA1*02:14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:14:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_44():
    result = parse("Mafa-DPA1*02:44")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:44") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:44") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_45():
    result = parse("Mafa-DPA1*02:45")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:45") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:45") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_06_05():
    result = parse("Mafa-DPA1*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_11_02():
    result = parse("Mafa-DPA1*07:11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*07:11:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_02_03():
    result = parse("Mafa-DPB1*01:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*01:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_04_02():
    result = parse("Mafa-DPB1*03:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*03:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_09():
    result = parse("Mafa-DPB1*06:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*06:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_07_03_02():
    result = parse("Mafa-DPB1*07:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*07:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*07:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_01_02():
    result = parse("Mafa-DPB1*15:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*15:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_15_13():
    result = parse("Mafa-DPB1*15:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*15:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*15:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_13():
    result = parse("Mafa-DPB1*01:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_042_01():
    result = parse("BoLA-DQB*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_10():
    result = parse("BoLA-DQB*018:10")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_11():
    result = parse("BoLA-DQB*018:11")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_072_01():
    result = parse("BoLA-DRB3*072:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*072:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*072:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_073_01():
    result = parse("BoLA-DRB3*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_08():
    result = parse("BoLA-DRB3*020:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_050_01_01():
    result = parse("BoLA-DRB3*050:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*050:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DRB3*050:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_034_03():
    result = parse("BoLA-DRB3*034:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*034:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*034:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_001_01():
    result = parse("Bogr-DQA1*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA2_001_01():
    result = parse("Bogr-DQA2*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA2*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA2*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_03_05():
    result = parse("Patr-A*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_03_06():
    result = parse("Patr-A*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_03_07():
    result = parse("Patr-A*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_17_04():
    result = parse("Patr-A*17:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*17:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*17:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_08_01_01_02():
    result = parse("Patr-A*08:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*08:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*08:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_13_01_01_02():
    result = parse("Patr-A*13:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*13:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*13:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_15_01_01_02():
    result = parse("Patr-A*15:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*15:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*15:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_15_03():
    result = parse("Patr-A*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_17_01_01_02():
    result = parse("Patr-A*17:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*17:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*17:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_07_03():
    result = parse("Patr-B*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_07_04():
    result = parse("Patr-B*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_07_05():
    result = parse("Patr-B*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_11_03():
    result = parse("Patr-B*11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*11:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_12_03():
    result = parse("Patr-B*12:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*12:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*12:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_19_03():
    result = parse("Patr-B*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_19_04():
    result = parse("Patr-B*19:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*19:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*19:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_20_01_02():
    result = parse("Patr-B*20:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*20:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-B*20:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_21_02():
    result = parse("Patr-B*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_40_01_01_01():
    result = parse("Patr-B*40:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*40:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-B*40:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_40_01_01_02():
    result = parse("Patr-B*40:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*40:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-B*40:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_40_02():
    result = parse("Patr-B*40:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*40:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*40:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_22_06():
    result = parse("Patr-B*22:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*22:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*22:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_22_07():
    result = parse("Patr-B*22:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*22:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*22:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_01_02():
    result = parse("Patr-B*23:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-B*23:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_06():
    result = parse("Patr-B*23:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*23:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_23_07():
    result = parse("Patr-B*23:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*23:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*23:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_33_01_01_02():
    result = parse("Patr-B*33:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*33:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-B*33:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_35_02():
    result = parse("Patr-B*35:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*35:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*35:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_38_02():
    result = parse("Patr-B*38:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*38:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*38:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_03_06():
    result = parse("Papa-A*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_04_02():
    result = parse("Papa-A*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_06_03_01_01():
    result = parse("Papa-A*06:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*06:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-A*06:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_06_03_01_02():
    result = parse("Papa-A*06:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*06:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-A*06:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_06_03_01_03():
    result = parse("Papa-A*06:03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*06:03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-A*06:03:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_05_03():
    result = parse("Papa-A*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_08_03():
    result = parse("Papa-A*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_08_02():
    result = parse("Papa-A*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_03_02():
    result = parse("Papa-A*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_03_03_01_02():
    result = parse("Papa-A*03:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*03:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-A*03:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_09_02():
    result = parse("Papa-A*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_03_05():
    result = parse("Papa-A*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_06_02():
    result = parse("Papa-A*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_02_02_02():
    result = parse("Patr-C*03:02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*03:02:02:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_05():
    result = parse("Patr-C*03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*03:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_17_01():
    result = parse("Patr-C*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_17_02():
    result = parse("Patr-C*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_04_02():
    result = parse("Patr-C*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_07_02():
    result = parse("Patr-C*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_15_02():
    result = parse("Patr-C*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_07_03():
    result = parse("Patr-C*07:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*07:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*07:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_01_01_03():
    result = parse("Patr-C*09:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*09:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_01_01_04():
    result = parse("Patr-C*09:01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*09:01:01:04") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_09_02_01_02():
    result = parse("Patr-C*09:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*09:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-C*09:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_10_02():
    result = parse("Patr-C*10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*10:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_11_01_02():
    result = parse("Patr-C*11:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*11:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-C*11:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_13_04():
    result = parse("Patr-C*13:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*13:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*13:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_06():
    result = parse("Patr-C*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_C_03_07():
    result = parse("Patr-C*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-C*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-C*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_02():
    result = parse("Patr-AL*01:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_03():
    result = parse("Patr-AL*01:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_04():
    result = parse("Patr-AL*01:01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:04") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_05():
    result = parse("Patr-AL*01:01:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:05") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_06():
    result = parse("Patr-AL*01:01:01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:06") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_07():
    result = parse("Patr-AL*01:01:01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:07") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_08():
    result = parse("Patr-AL*01:01:01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:08") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_01_01_09():
    result = parse("Patr-AL*01:01:01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:01:01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-AL*01:01:01:09") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_AL_01_04():
    result = parse("Patr-AL*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-AL*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-AL*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_01_01_03():
    result = parse("Papa-C*03:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*03:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_03_01_01():
    result = parse("Papa-C*03:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*03:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_03_01_02():
    result = parse("Papa-C*03:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*03:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_04_03():
    result = parse("Papa-C*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_04_02():
    result = parse("Papa-C*04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*04:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_05_03():
    result = parse("Papa-C*05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*05:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_05_01():
    result = parse("Papa-C*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_05_02():
    result = parse("Papa-C*05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-C*05:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_06_01_01_02():
    result = parse("Papa-C*06:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*06:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*06:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_25_01_01_01():
    result = parse("Patr-A*25:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*25:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*25:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_25_01_01_02():
    result = parse("Patr-A*25:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*25:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Patr-A*25:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_26_01():
    result = parse("Patr-A*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_A_27_01():
    result = parse("Patr-A*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-A*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-A*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_30_02():
    result = parse("Patr-B*30:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*30:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*30:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_B_11_04():
    result = parse("Patr-B*11:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-B*11:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-B*11:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_01_03():
    result = parse("Papa-B*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_15_02():
    result = parse("Papa-B*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_01_02():
    result = parse("Papa-B*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_21_01():
    result = parse("Papa-B*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_16_02():
    result = parse("Papa-B*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_22_01():
    result = parse("Papa-B*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_03_03_01_01():
    result = parse("Papa-A*03:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*03:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-A*03:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_09_01():
    result = parse("Papa-A*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_A_03_04():
    result = parse("Papa-A*03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-A*03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-A*03:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_04_01_01():
    result = parse("Papa-C*03:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*03:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_03_04_01_02():
    result = parse("Papa-C*03:04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*03:04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*03:04:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_C_06_01_01_01():
    result = parse("Papa-C*06:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-C*06:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Papa-C*06:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_01_01():
    result = parse("Chpy-A*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_10_01_01():
    result = parse("Chpy-A*10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-A*10:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_10_01_02():
    result = parse("Chpy-A*10:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*10:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-A*10:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_02_01():
    result = parse("Chpy-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_03_01():
    result = parse("Chpy-A*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_03_02():
    result = parse("Chpy-A*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_04_01():
    result = parse("Chpy-A*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_06_01_01():
    result = parse("Chpy-A*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-A*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_06_02():
    result = parse("Chpy-A*06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*06:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_06_03():
    result = parse("Chpy-A*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_06_01_02():
    result = parse("Chpy-A*06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-A*06:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_06_04():
    result = parse("Chpy-A*06:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*06:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*06:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_06_05():
    result = parse("Chpy-A*06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*06:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_20_01():
    result = parse("Chpy-A*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_27_01():
    result = parse("Chpy-A*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_08_01():
    result = parse("Chpy-A*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_28_01():
    result = parse("Chpy-A*28:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*28:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*28:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_29_01():
    result = parse("Chpy-A*29:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*29:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*29:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_30_01_01():
    result = parse("Chpy-A*30:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*30:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-A*30:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_30_01_02():
    result = parse("Chpy-A*30:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*30:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-A*30:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_30_02():
    result = parse("Chpy-A*30:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*30:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*30:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_22_01():
    result = parse("Chpy-A*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_16_01():
    result = parse("Chpy-A*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_14_01():
    result = parse("Chpy-A*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_14_02():
    result = parse("Chpy-A*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_14_03():
    result = parse("Chpy-A*14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*14:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_14_04():
    result = parse("Chpy-A*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_14_05():
    result = parse("Chpy-A*14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*14:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_14_06():
    result = parse("Chpy-A*14:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*14:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*14:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_14_07():
    result = parse("Chpy-A*14:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*14:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*14:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_17_01():
    result = parse("Chpy-A*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_17_02():
    result = parse("Chpy-A*17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*17:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_17_03():
    result = parse("Chpy-A*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_17_04():
    result = parse("Chpy-A*17:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*17:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*17:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_17_05():
    result = parse("Chpy-A*17:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*17:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*17:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_17_06():
    result = parse("Chpy-A*17:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*17:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*17:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_31_01():
    result = parse("Chpy-A*31:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*31:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*31:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_13_01():
    result = parse("Chpy-A*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_32_01():
    result = parse("Chpy-A*32:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*32:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*32:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_32_02():
    result = parse("Chpy-A*32:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*32:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*32:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_A_21_01():
    result = parse("Chpy-A*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-A*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-A*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_01_01():
    result = parse("Chpy-B*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_03_01():
    result = parse("Chpy-B*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_46_01():
    result = parse("Chpy-B*46:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*46:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*46:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_46_02():
    result = parse("Chpy-B*46:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*46:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*46:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_46_03():
    result = parse("Chpy-B*46:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*46:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*46:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_46_04():
    result = parse("Chpy-B*46:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*46:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*46:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_07_01():
    result = parse("Chpy-B*07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*07:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_08_01():
    result = parse("Chpy-B*08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*08:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_08_02():
    result = parse("Chpy-B*08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*08:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_09_01():
    result = parse("Chpy-B*09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*09:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_09_02():
    result = parse("Chpy-B*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_09_03():
    result = parse("Chpy-B*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_09_04():
    result = parse("Chpy-B*09:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*09:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*09:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_09_05():
    result = parse("Chpy-B*09:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*09:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*09:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_11_01():
    result = parse("Chpy-B*11:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*11:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*11:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_11_02():
    result = parse("Chpy-B*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_47_01():
    result = parse("Chpy-B*47:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*47:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*47:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_01():
    result = parse("Chpy-B*13:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_48_01():
    result = parse("Chpy-B*48:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*48:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*48:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_02():
    result = parse("Chpy-B*13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_03():
    result = parse("Chpy-B*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_04():
    result = parse("Chpy-B*13:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_05_01():
    result = parse("Chpy-B*13:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-B*13:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_05_02():
    result = parse("Chpy-B*13:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-B*13:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_06():
    result = parse("Chpy-B*13:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_48_02():
    result = parse("Chpy-B*48:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*48:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*48:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_48_03():
    result = parse("Chpy-B*48:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*48:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*48:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_07():
    result = parse("Chpy-B*13:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_08():
    result = parse("Chpy-B*13:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_13_09():
    result = parse("Chpy-B*13:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*13:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*13:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_14_01():
    result = parse("Chpy-B*14:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*14:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*14:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_14_02():
    result = parse("Chpy-B*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_14_03():
    result = parse("Chpy-B*14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*14:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_14_04():
    result = parse("Chpy-B*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_14_05():
    result = parse("Chpy-B*14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*14:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_14_06():
    result = parse("Chpy-B*14:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*14:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*14:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_31_01():
    result = parse("Chpy-B*31:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*31:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*31:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_01():
    result = parse("Chpy-B*15:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_02():
    result = parse("Chpy-B*15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_03():
    result = parse("Chpy-B*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_04_01():
    result = parse("Chpy-B*15:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-B*15:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_04_02():
    result = parse("Chpy-B*15:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-B*15:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_05():
    result = parse("Chpy-B*15:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_06():
    result = parse("Chpy-B*15:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_07():
    result = parse("Chpy-B*15:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_08():
    result = parse("Chpy-B*15:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_09():
    result = parse("Chpy-B*15:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_10():
    result = parse("Chpy-B*15:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_11():
    result = parse("Chpy-B*15:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_12():
    result = parse("Chpy-B*15:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_13():
    result = parse("Chpy-B*15:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_14():
    result = parse("Chpy-B*15:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_15_15():
    result = parse("Chpy-B*15:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*15:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*15:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_16_01():
    result = parse("Chpy-B*16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*16:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_16_02():
    result = parse("Chpy-B*16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*16:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_16_03():
    result = parse("Chpy-B*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_17_01():
    result = parse("Chpy-B*17:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*17:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*17:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_19_01():
    result = parse("Chpy-B*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_19_02():
    result = parse("Chpy-B*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_49_01():
    result = parse("Chpy-B*49:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*49:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*49:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_20_01():
    result = parse("Chpy-B*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_20_02():
    result = parse("Chpy-B*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_21_01():
    result = parse("Chpy-B*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_21_02():
    result = parse("Chpy-B*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_21_03():
    result = parse("Chpy-B*21:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*21:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*21:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_50_01():
    result = parse("Chpy-B*50:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*50:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*50:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_41_01():
    result = parse("Chpy-B*41:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*41:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*41:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_51_01():
    result = parse("Chpy-B*51:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*51:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*51:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_52_01():
    result = parse("Chpy-B*52:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*52:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*52:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_32_01():
    result = parse("Chpy-B*32:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*32:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*32:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_32_02():
    result = parse("Chpy-B*32:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*32:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*32:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_32_03():
    result = parse("Chpy-B*32:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*32:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*32:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_34_01():
    result = parse("Chpy-B*34:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*34:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*34:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_34_02():
    result = parse("Chpy-B*34:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*34:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*34:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_53_01():
    result = parse("Chpy-B*53:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*53:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*53:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_26_01():
    result = parse("Chpy-B*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_26_02():
    result = parse("Chpy-B*26:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*26:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*26:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_26_03():
    result = parse("Chpy-B*26:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*26:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*26:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_26_04():
    result = parse("Chpy-B*26:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*26:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*26:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_54_01():
    result = parse("Chpy-B*54:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*54:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*54:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_35_01():
    result = parse("Chpy-B*35:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*35:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*35:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_36_01():
    result = parse("Chpy-B*36:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*36:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*36:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_36_02():
    result = parse("Chpy-B*36:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*36:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*36:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_36_03():
    result = parse("Chpy-B*36:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*36:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*36:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_36_04():
    result = parse("Chpy-B*36:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*36:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*36:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_37_01():
    result = parse("Chpy-B*37:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*37:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*37:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_37_02():
    result = parse("Chpy-B*37:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*37:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*37:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_44_01_01():
    result = parse("Chpy-B*44:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*44:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-B*44:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_44_02():
    result = parse("Chpy-B*44:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*44:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*44:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_44_01_02():
    result = parse("Chpy-B*44:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*44:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Chpy-B*44:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_44_03():
    result = parse("Chpy-B*44:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*44:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*44:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_45_01():
    result = parse("Chpy-B*45:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*45:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*45:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_55_01():
    result = parse("Chpy-B*55:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*55:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*55:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_39_01():
    result = parse("Chpy-B*39:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*39:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*39:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W020_02():
    result = parse("Malo-DRB*W020:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W020:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W020:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB_W020_03():
    result = parse("Malo-DRB*W020:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB*W020:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB*W020:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DRB1_03_03():
    result = parse("Malo-DRB1*03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DRB1*03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DRB1*03:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_13_02():
    result = parse("Mafa-DRB1*04:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_15():
    result = parse("Mafa-DRB1*10:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*10:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_15():
    result = parse("Mafa-DRB*W001:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_11():
    result = parse("Mafa-DRB*W004:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_12():
    result = parse("Mafa-DRB*W004:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_08():
    result = parse("Mafa-DRB*W027:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W027:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_45():
    result = parse("Mafa-DQB1*06:45")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:45") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:45") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_46():
    result = parse("Mafa-DQB1*06:46")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:46") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*06:46") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_30_02():
    result = parse("Mafa-DPA1*02:30:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:30:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:30:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_46():
    result = parse("Mafa-DPA1*02:46")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:46") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:46") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_15():
    result = parse("Mafa-DPA1*07:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_09_01_02():
    result = parse("Mafa-DPA1*09:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*09:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*09:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_14():
    result = parse("Mafa-DPB1*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_15():
    result = parse("Mafa-DPB1*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_14_03():
    result = parse("Mafa-DQA1*01:14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:14:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_26():
    result = parse("Mafa-DQA1*01:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*01:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_14():
    result = parse("Mafa-DQA1*26:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*26:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_002_04():
    result = parse("Mafa-A1*002:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*002:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*002:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_08():
    result = parse("Mafa-A1*007:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_008_06():
    result = parse("Mafa-A1*008:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*008:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*008:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_028_06():
    result = parse("Mafa-A1*028:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*028:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*028:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_042_03():
    result = parse("Mafa-A1*042:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*042:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*042:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_073_01():
    result = parse("Mafa-A1*073:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*073:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*073:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB4_01_11():
    result = parse("Mafa-DRB4*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB4*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB4*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_045_04():
    result = parse("Mafa-A1*045:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*045:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*045:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_23():
    result = parse("Mafa-A3*13:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_20():
    result = parse("Mafa-A4*14:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A4*14:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_06():
    result = parse("Mafa-A6*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_07():
    result = parse("Mafa-A6*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_08_01():
    result = parse("Mafa-A6*01:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A6*01:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_001_03():
    result = parse("Mafa-B*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_11():
    result = parse("Mafa-B*007:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*007:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_19():
    result = parse("Mafa-B*013:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_08():
    result = parse("Mafa-B*015:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_08():
    result = parse("Mafa-A1*003:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*003:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_018_09():
    result = parse("Mafa-A1*018:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*018:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*018:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_019_06():
    result = parse("Mafa-A1*019:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*019:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*019:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_028_07():
    result = parse("Mafa-A1*028:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*028:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*028:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_032_06():
    result = parse("Mafa-A1*032:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*032:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*032:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_033_03():
    result = parse("Mafa-A1*033:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*033:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*033:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_055_03():
    result = parse("Mafa-A1*055:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*055:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*055:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_074_01_03():
    result = parse("Mafa-A1*074:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*074:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*074:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_08():
    result = parse("Mafa-A1*089:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_01_02():
    result = parse("Mafa-A1*090:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*090:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_100_02():
    result = parse("Mafa-A1*100:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*100:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*100:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_06_05():
    result = parse("Mafa-A2*05:06:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:06:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:06:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_61():
    result = parse("Mafa-A2*05:61")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:61") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:61") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_24():
    result = parse("Mafa-A3*13:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_09_01():
    result = parse("Mafa-A6*01:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A6*01:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_08_02():
    result = parse("Mafa-A6*01:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A6*01:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_10():
    result = parse("Mafa-A6*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_11():
    result = parse("Mafa-A6*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Papa_B_09_02():
    result = parse("Papa-B*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Papa-B*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Papa-B*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_35_02():
    result = parse("Mafa-I*01:35:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:35:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:35:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_07():
    result = parse("Mafa-I*01:14:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:07") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_019_03_02_01():
    result = parse("Mafa-B*019:03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*019:03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*019:03:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_019_05():
    result = parse("Mafa-B*019:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*019:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*019:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_21():
    result = parse("Mafa-B*030:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_032_02_01_01():
    result = parse("Mafa-B*032:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*032:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*032:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_039_01_02():
    result = parse("Mafa-B*039:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*039:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*039:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_16():
    result = parse("Mafa-B*051:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_17():
    result = parse("Mafa-B*060:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_18():
    result = parse("Mafa-B*060:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_19():
    result = parse("Mafa-B*060:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_20_01():
    result = parse("Mafa-B*060:20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*060:20:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_064_06():
    result = parse("Mafa-B*064:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*064:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*064:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_075_04():
    result = parse("Mafa-B*075:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*075:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*075:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_05():
    result = parse("Mafa-B*079:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*079:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_082_04():
    result = parse("Mafa-B*082:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*082:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*082:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_082_05():
    result = parse("Mafa-B*082:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*082:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*082:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_090_03():
    result = parse("Mafa-B*090:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*090:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*090:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_091_04():
    result = parse("Mafa-B*091:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*091:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*091:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_16_01():
    result = parse("Mafa-B*098:16:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:16:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:16:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_08_02_01():
    result = parse("Mafa-B*098:08:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:08:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*098:08:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_112_02_01_01():
    result = parse("Mafa-B*112:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*112:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*112:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_108_02():
    result = parse("Mafa-B*108:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*108:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*108:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_01_02():
    result = parse("Mafa-B*137:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*137:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_09():
    result = parse("Mafa-B*137:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*137:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_146_02():
    result = parse("Mafa-B*146:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*146:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*146:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_148_02():
    result = parse("Mafa-B*148:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*148:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*148:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_150_03():
    result = parse("Mafa-B*150:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*150:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*150:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_180_02_01_01():
    result = parse("Mafa-B*180:02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*180:02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*180:02:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_17_02():
    result = parse("Mafa-B*098:17:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:17:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:17:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_17_03():
    result = parse("Mafa-B*098:17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:17:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_17_04():
    result = parse("Mafa-B*098:17:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:17:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:17:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_003_02():
    result = parse("Mafa-B*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_004_04_01_01():
    result = parse("Mafa-B*004:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*004:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*004:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_005_02N():
    result = parse("Mafa-B*005:02N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*005:02N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*005:02N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_03_01_01():
    result = parse("Mafa-B*011:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*011:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_04():
    result = parse("Mafa-B*011:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*011:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_09():
    result = parse("Mafa-B*015:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_021_07():
    result = parse("Mafa-B*021:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*021:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*021:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_031_04():
    result = parse("Mafa-B*031:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*031:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*031:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_01_03_01():
    result = parse("Mafa-B*036:01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*036:01:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_05N():
    result = parse("Mafa-B*041:05N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:05N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*041:05N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_043_03N():
    result = parse("Mafa-B*043:03N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*043:03N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*043:03N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_01_02():
    result = parse("Mafa-B*045:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*045:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_16_02():
    result = parse("Mafa-B*046:16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*046:16:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_10():
    result = parse("Mafa-B*051:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_11():
    result = parse("Mafa-B*051:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_12_01_01():
    result = parse("Mafa-B*051:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*051:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_13():
    result = parse("Mafa-B*051:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_21():
    result = parse("Mafa-B*060:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_22():
    result = parse("Mafa-B*060:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_069_09():
    result = parse("Mafa-B*069:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*069:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*069:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_07():
    result = parse("Mafa-B*072:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_08():
    result = parse("Mafa-B*072:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_09_01_01():
    result = parse("Mafa-B*072:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*072:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_10():
    result = parse("Mafa-B*072:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_077_03():
    result = parse("Mafa-B*077:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*077:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*077:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_02_03_01():
    result = parse("Mafa-B*079:02:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:02:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*079:02:03:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_06_01_01():
    result = parse("Mafa-B*079:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*079:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_081_05_02_01():
    result = parse("Mafa-B*081:05:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*081:05:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*081:05:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_081_06():
    result = parse("Mafa-B*081:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*081:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*081:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_082_07():
    result = parse("Mafa-B*082:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*082:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*082:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_091_05():
    result = parse("Mafa-B*091:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*091:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*091:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_099_03():
    result = parse("Mafa-B*099:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*099:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*099:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_100_01_02():
    result = parse("Mafa-B*100:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*100:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*100:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_10_02():
    result = parse("Mafa-B*046:10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*046:10:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_06_01_01():
    result = parse("Mafa-B*115:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*115:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_05_02():
    result = parse("Mafa-B*115:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*115:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_124_05_01_01N():
    result = parse("Mafa-B*124:05:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*124:05:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*124:05:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_05_01_01():
    result = parse("Mafa-B*134:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*134:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_035_01():
    result = parse("Mafa-B*035:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*035:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*035:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_145_02():
    result = parse("Mafa-B*145:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*145:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*145:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_161_06():
    result = parse("Mafa-B*161:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*161:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*161:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_205_01_01_01():
    result = parse("Mafa-B*205:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*205:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*205:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_162_03_01_01():
    result = parse("Mafa-B*162:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*162:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*162:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_24():
    result = parse("Mamu-E*02:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_25_01():
    result = parse("Mamu-E*02:25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-E*02:25:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_01_02_01():
    result = parse("Mafa-E*02:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-E*02:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_03_02_01():
    result = parse("Mafa-E*02:03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-E*02:03:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_13():
    result = parse("Mafa-E*02:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_26():
    result = parse("Mamu-E*02:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_27():
    result = parse("Mamu-E*02:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_25_02():
    result = parse("Mamu-E*02:25:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:25:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-E*02:25:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_28():
    result = parse("Mamu-E*02:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_29():
    result = parse("Mamu-E*02:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_E_02_30():
    result = parse("Mamu-E*02:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-E*02:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-E*02:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_E_02_14():
    result = parse("Mafa-E*02:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-E*02:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-E*02:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_11():
    result = parse("SLA-DRB1*02:11")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W105_01():
    result = parse("Alpi-DRB*W105:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W105:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W105:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W105_02():
    result = parse("Alpi-DRB*W105:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W105:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W105:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W105_03():
    result = parse("Alpi-DRB*W105:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W105:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W105:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W105_04():
    result = parse("Alpi-DRB*W105:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W105:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W105:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W105_05():
    result = parse("Alpi-DRB*W105:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W105:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W105:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_01():
    result = parse("Alpi-DRB*W106:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_02():
    result = parse("Alpi-DRB*W106:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_03():
    result = parse("Alpi-DRB*W106:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_04():
    result = parse("Alpi-DRB*W106:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_05():
    result = parse("Alpi-DRB*W106:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_06():
    result = parse("Alpi-DRB*W106:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_07_01():
    result = parse("Alpi-DRB*W106:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Alpi-DRB*W106:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_08():
    result = parse("Alpi-DRB*W106:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_07_02():
    result = parse("Alpi-DRB*W106:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Alpi-DRB*W106:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W106_09():
    result = parse("Alpi-DRB*W106:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W106:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W106:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W107_01():
    result = parse("Alpi-DRB*W107:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W107:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W107:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W107_02():
    result = parse("Alpi-DRB*W107:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W107:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W107:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W107_03():
    result = parse("Alpi-DRB*W107:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W107:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W107:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W108_01():
    result = parse("Alpi-DRB*W108:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W108:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W108:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W105_06():
    result = parse("Alpi-DRB*W105:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W105:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W105:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_39_05():
    result = parse("Paan-B*39:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*39:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*39:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMA_02_01_01():
    result = parse("Malo-DMA*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMA*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMA*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMA_02_01_02():
    result = parse("Malo-DMA*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMA*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMA*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMA_02_02_01():
    result = parse("Malo-DMA*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMA*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMA*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMA_02_02_02():
    result = parse("Malo-DMA*02:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMA*02:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMA*02:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMB_03_01_01():
    result = parse("Malo-DMB*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMB*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMB*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMB_03_02():
    result = parse("Malo-DMB*03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMB*03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DMB*03:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMB_03_01_02():
    result = parse("Malo-DMB*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMB*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMB*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMB_03_01_03():
    result = parse("Malo-DMB*03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMB*03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMB*03:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DMB_03_01_04():
    result = parse("Malo-DMB*03:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DMB*03:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DMB*03:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOA_01_01():
    result = parse("Malo-DOA*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOA*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOA*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOA_01_02():
    result = parse("Malo-DOA*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOA*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOA*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOA_01_03():
    result = parse("Malo-DOA*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOA*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOA*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOA_01_04():
    result = parse("Malo-DOA*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOA*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOA*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOA_01_05():
    result = parse("Malo-DOA*01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOA*01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOA*01:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOA_01_06():
    result = parse("Malo-DOA*01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOA*01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOA*01:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOA_01_07():
    result = parse("Malo-DOA*01:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOA*01:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOA*01:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOB_01_01_01():
    result = parse("Malo-DOB*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOB*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DOB*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOB_01_02():
    result = parse("Malo-DOB*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOB*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOB*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOB_01_01_02():
    result = parse("Malo-DOB*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOB*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Malo-DOB*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Malo_DOB_01_03():
    result = parse("Malo-DOB*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Malo-DOB*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Malo-DOB*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B_03_01_03():
    result = parse("Caja-B*03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B*03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-B*03:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B_03_01_02():
    result = parse("Caja-B*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-B*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_24():
    result = parse("Caja-G*08:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_03_02():
    result = parse("Caja-G*18:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*18:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_04_02():
    result = parse("Caja-G*18:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*18:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_10():
    result = parse("Caja-G*18:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_11():
    result = parse("Caja-G*18:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_25():
    result = parse("Caja-G*08:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_03():
    result = parse("Bubu-DQA*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_F_02_18():
    result = parse("Mafa-F*02:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-F*02:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-F*02:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRA_01_03_02():
    result = parse("Patr-DRA*01:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRA*01:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DRA*01:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRA_01_03_01():
    result = parse("Patr-DRA*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRA*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DRA*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_F_01_02():
    result = parse("Gogo-F*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-F*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-F*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_F_01_01():
    result = parse("Gogo-F*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-F*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Gogo-F*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_11_09():
    result = parse("SLA-DRB1*11:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*11:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*11:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_10_01_02():
    result = parse("SLA-DRB1*10:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*10:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DRB1*10:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_10():
    result = parse("SLA-DRB1*04:10")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_14_03():
    result = parse("SLA-DRB1*14:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*14:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*14:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_06_07():
    result = parse("SLA-DRB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_09():
    result = parse("SLA-DRB1*04:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_02_10():
    result = parse("SLA-DRB1*02:10")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*02:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*02:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_12_05():
    result = parse("SLA-DRB1*12:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*12:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*12:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPA1_01_10():
    result = parse("Patr-DPA1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPA1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPA1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_02_02():
    result = parse("Patr-DPB1*02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*02:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_002_02():
    result = parse("Bubu-DQA*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_04():
    result = parse("Bubu-DQA*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_002_03():
    result = parse("Bubu-DQA*002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_021_02():
    result = parse("Bubu-DQA*021:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*021:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*021:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_002_04():
    result = parse("Bubu-DQA*002:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*002:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*002:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_05():
    result = parse("Bubu-DQA*001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_06():
    result = parse("Bubu-DQA*001:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_025_01():
    result = parse("Bubu-DQA*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_026_01():
    result = parse("Bubu-DQA*026:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*026:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*026:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_027_01():
    result = parse("Bubu-DQA*027:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*027:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*027:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_07():
    result = parse("Bubu-DQA*001:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_024_02():
    result = parse("Bubu-DQA*024:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*024:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*024:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_024_03():
    result = parse("Bubu-DQA*024:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*024:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*024:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_028_01():
    result = parse("Bubu-DQA*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_08():
    result = parse("Bubu-DQA*001:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_021_03():
    result = parse("Bubu-DQA*021:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*021:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*021:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_022_02():
    result = parse("Bubu-DQA*022:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*022:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*022:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_09():
    result = parse("Bubu-DQA*001:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_021_04():
    result = parse("Bubu-DQA*021:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*021:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*021:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_021_05():
    result = parse("Bubu-DQA*021:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*021:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*021:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_029_01():
    result = parse("Bubu-DQA*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_10():
    result = parse("Bubu-DQA*001:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*001:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_002_05():
    result = parse("Bubu-DQA*002:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*002:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*002:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_022_03():
    result = parse("Bubu-DQA*022:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*022:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bubu-DQA*022:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_022_07():
    result = parse("Mamu-A1*022:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*022:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*022:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A2_05_13_02():
    result = parse("Mamu-A2*05:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2*05:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A2*05:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_07():
    result = parse("Mamu-B*007:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*007:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_037_03():
    result = parse("Mamu-B*037:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*037:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*037:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_054_05_01_01():
    result = parse("Mamu-B*054:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*054:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*054:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_08_01_01():
    result = parse("Mamu-B*056:08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*056:08:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_15_01_01():
    result = parse("Mamu-B*060:15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:15:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_16():
    result = parse("Mamu-B*060:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_164_01():
    result = parse("Mamu-B*164:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*164:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*164:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_082_11():
    result = parse("Mamu-B*082:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*082:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*082:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_12_01_01():
    result = parse("Mamu-B*098:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_13():
    result = parse("Mamu-B*098:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*098:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_14():
    result = parse("Mamu-B*098:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*098:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_134_05():
    result = parse("Mamu-B*134:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*134:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*134:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_142_04():
    result = parse("Mamu-B*142:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*142:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*142:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_47():
    result = parse("Mamu-I*01:47")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:47") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:47") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_48_01():
    result = parse("Mamu-I*01:48:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:48:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:48:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_49_01():
    result = parse("Mamu-I*01:49:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:49:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:49:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_18_02():
    result = parse("Mamu-I*01:18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:18:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_48_02():
    result = parse("Mamu-I*01:48:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:48:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:48:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_106_02():
    result = parse("Mamu-B*106:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*106:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*106:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_008_07():
    result = parse("Mafa-A1*008:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*008:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*008:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_08():
    result = parse("Mafa-A1*010:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*010:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_12():
    result = parse("Mafa-A1*022:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_053_02():
    result = parse("Mafa-A1*053:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*053:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*053:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_04_01():
    result = parse("Mafa-A1*090:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*090:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_099_03():
    result = parse("Mafa-A1*099:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*099:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*099:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_25_03():
    result = parse("Mafa-A2*05:25:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:25:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:25:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_62_01():
    result = parse("Mafa-A2*05:62:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:62:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:62:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_63():
    result = parse("Mafa-A2*05:63")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:63") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:63") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_64_01_01():
    result = parse("Mafa-A2*05:64:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:64:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:64:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_05_01():
    result = parse("Mafa-B*011:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*011:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_032_03_01_01():
    result = parse("Mafa-B*032:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*032:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*032:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_11_01_01():
    result = parse("Mafa-B*050:11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*050:11:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_23():
    result = parse("Mafa-B*060:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_24_01_01():
    result = parse("Mafa-B*060:24:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:24:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:24:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_074_04_01_01():
    result = parse("Mafa-B*074:04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*074:04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*074:04:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_09_01():
    result = parse("Mafa-B*079:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*079:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_201_01_01_01():
    result = parse("Mafa-B*201:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*201:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*201:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_08():
    result = parse("Mafa-I*01:14:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:08") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_44():
    result = parse("Mafa-I*01:44")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:44") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:44") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_09():
    result = parse("Mafa-I*01:14:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:09") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_43_02():
    result = parse("Mafa-I*01:43:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:43:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:43:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_03_01_01_01():
    result = parse("Gogo-C*03:01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*03:01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-C*03:01:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_03_01_01_02():
    result = parse("Gogo-C*03:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*03:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-C*03:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Gogo_C_03_01_01_03():
    result = parse("Gogo-C*03:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Gogo-C*03:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Gogo-C*03:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_03():
    result = parse("Mamu-A1*008:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*008:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_14():
    result = parse("Mamu-A3*13:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A3*13:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_021_04():
    result = parse("Mamu-B*021:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*021:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*021:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_22():
    result = parse("Mamu-B*046:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*046:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_047_01_02():
    result = parse("Mamu-B*047:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*047:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*047:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_048_04():
    result = parse("Mamu-B*048:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*048:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*048:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_053_03_01_01():
    result = parse("Mamu-B*053:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*053:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*053:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_03_02_01():
    result = parse("Mamu-B*068:03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*068:03:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_09_01_01():
    result = parse("Mamu-B*072:09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:09:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_10_01_01():
    result = parse("Mamu-B*072:10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:10:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_077_01_02_01():
    result = parse("Mamu-B*077:01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*077:01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*077:01:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_091_05_01_01():
    result = parse("Mamu-B*091:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*091:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*091:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_124_03_01_01():
    result = parse("Mamu-B*124:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*124:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*124:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_134_06_01_01():
    result = parse("Mamu-B*134:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*134:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*134:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_137_01_02():
    result = parse("Mamu-B*137:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*137:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*137:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_180_03_01_01():
    result = parse("Mamu-B*180:03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*180:03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*180:03:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_21():
    result = parse("Mamu-I*01:06:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:21") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_50():
    result = parse("Mamu-I*01:50")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:50") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-I*01:50") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_22():
    result = parse("Mamu-I*01:06:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:22") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_05_02():
    result = parse("Mamu-I*01:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_51_01_01():
    result = parse("Mamu-I*01:51:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:51:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:51:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_34_03():
    result = parse("Mamu-I*01:34:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:34:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:34:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_13_02():
    result = parse("Patr-DPB1*01:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Patr-DPB1*01:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DPB1_01_19():
    result = parse("Patr-DPB1*01:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DPB1*01:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DPB1*01:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRA_01_04():
    result = parse("Patr-DRA*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRA*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRA*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DQB1_03_06():
    result = parse("Patr-DQB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DQB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DQB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_04_02():
    result = parse("Caja-DRB*W012:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_03_03():
    result = parse("Caja-DRB*W012:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_02_03():
    result = parse("Caja-DRB*W012:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_02_04():
    result = parse("Caja-DRB*W012:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-DRB*W012:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_05():
    result = parse("Caja-DRB*W012:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W012:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W012_06():
    result = parse("Caja-DRB*W012:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W012:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W012:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_25():
    result = parse("Caja-DRB*W016:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_26():
    result = parse("Caja-DRB*W016:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_19():
    result = parse("Caja-DRB1*03:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_20N():
    result = parse("Caja-DRB1*03:20N")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:20N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:20N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_05():
    result = parse("Mafa-A1*090:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*090:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_060_07():
    result = parse("Mafa-A1*060:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*060:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*060:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_06():
    result = parse("Mafa-A1*090:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*090:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_091_04():
    result = parse("Mafa-A1*091:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*091:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*091:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_11_03():
    result = parse("Mafa-I*01:11:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:11:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:11:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_45():
    result = parse("Mafa-I*01:45")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:45") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:45") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_10():
    result = parse("Mafa-I*01:14:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:10") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_46():
    result = parse("Mafa-I*01:46")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:46") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:46") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_38_02():
    result = parse("Mafa-I*01:38:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:38:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:38:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_47():
    result = parse("Mafa-I*01:47")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:47") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:47") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_65():
    result = parse("Mafa-A2*05:65")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:65") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:65") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_66_01_01():
    result = parse("Mafa-A2*05:66:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:66:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A2*05:66:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_25():
    result = parse("Mafa-A3*13:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_06():
    result = parse("Mafa-B*011:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*011:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_05_02():
    result = parse("Mafa-B*011:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*011:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_015_10():
    result = parse("Mafa-B*015:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*015:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*015:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_028_06():
    result = parse("Mafa-B*028:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*028:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*028:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_02_02():
    result = parse("Mafa-B*030:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_032_04():
    result = parse("Mafa-B*032:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*032:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*032:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_033_04():
    result = parse("Mafa-B*033:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*033:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*033:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_17():
    result = parse("Mafa-B*046:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_18():
    result = parse("Mafa-B*046:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_14():
    result = parse("Mafa-B*051:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_15():
    result = parse("Mafa-B*051:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_26():
    result = parse("Mafa-B*060:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_25():
    result = parse("Mafa-B*060:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_20_02():
    result = parse("Mafa-B*060:20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*060:20:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_13():
    result = parse("Mafa-B*068:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_02_04():
    result = parse("Mafa-B*079:02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*079:02:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_01_03():
    result = parse("Mafa-B*079:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*079:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_05():
    result = parse("Mafa-B*101:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*101:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_06():
    result = parse("Mafa-B*101:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*101:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_04_03():
    result = parse("Mafa-B*115:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*115:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_01_03():
    result = parse("Mafa-B*137:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*137:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_26():
    result = parse("Mafa-A3*13:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_12():
    result = parse("Mafa-A6*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_206_01():
    result = parse("Mafa-B*206:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*206:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*206:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_029_04():
    result = parse("Mamu-B*029:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*029:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*029:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_52_01_01():
    result = parse("Mamu-I*01:52:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:52:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-I*01:52:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_06_23():
    result = parse("Mamu-I*01:06:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:06:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:06:23") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_12():
    result = parse("Mafa-A1*043:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_002_06():
    result = parse("Mafa-B*002:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*002:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*002:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_038_02():
    result = parse("Mafa-B*038:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*038:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*038:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_03_02():
    result = parse("Mafa-B*044:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*044:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_052_02():
    result = parse("Mafa-B*052:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*052:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*052:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_052_03():
    result = parse("Mafa-B*052:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*052:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*052:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_01_02():
    result = parse("Mafa-B*060:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*060:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_27():
    result = parse("Mafa-B*060:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_07():
    result = parse("Mafa-B*079:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*079:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_089_03():
    result = parse("Mafa-B*089:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*089:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*089:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_089_01_05():
    result = parse("Mafa-B*089:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*089:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*089:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_101_07():
    result = parse("Mafa-B*101:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*101:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*101:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_06():
    result = parse("Mafa-B*134:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*134:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_203_02():
    result = parse("Mafa-B*203:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*203:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*203:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_038_01_02():
    result = parse("Mamu-B*038:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*038:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*038:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Chpy_B_36_05():
    result = parse("Chpy-B*36:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Chpy-B*36:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Chpy-B*36:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Alpi_DRB_W107_04():
    result = parse("Alpi-DRB*W107:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Alpi-DRB*W107:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Alpi-DRB*W107:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_07_07():
    result = parse("Caja-G*07:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*07:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*07:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_03_02():
    result = parse("Caja-G*08:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*08:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_01_03_01_02():
    result = parse("Mafa-AG*01:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*01:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-AG*01:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_21():
    result = parse("Caja-DRB1*03:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_22():
    result = parse("Caja-DRB1*03:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_23():
    result = parse("Caja-DRB1*03:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_24():
    result = parse("Caja-DRB1*03:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_25():
    result = parse("Caja-DRB1*03:25")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB1_03_26():
    result = parse("Caja-DRB1*03:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB1*03:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB1*03:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_05():
    result = parse("Caja-G*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_06():
    result = parse("Caja-G*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_10_07():
    result = parse("Caja-G*10:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*10:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*10:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_16_03():
    result = parse("Caja-G*16:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*16:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*16:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_12():
    result = parse("Caja-G*18:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_18_13():
    result = parse("Caja-G*18:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*18:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*18:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_E_03_01_02():
    result = parse("Caja-E*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-E*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-E*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_02_04():
    result = parse("Paan-A*02:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*02:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*02:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_28_01():
    result = parse("Paan-A*28:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*28:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*28:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_09_02():
    result = parse("Paan-A*09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*09:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_19_02():
    result = parse("Paan-A*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_A_29_01():
    result = parse("Paan-A*29:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-A*29:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-A*29:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_02_06():
    result = parse("Paan-B*02:02:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:02:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*02:02:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_02_13():
    result = parse("Paan-B*02:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*02:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*02:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_04_01_02():
    result = parse("Paan-B*04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*04:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_09_06():
    result = parse("Paan-B*09:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*09:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*09:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_10_05():
    result = parse("Paan-B*10:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*10:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*10:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_10_06():
    result = parse("Paan-B*10:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*10:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*10:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_11_04():
    result = parse("Paan-B*11:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*11:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*11:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_14_02():
    result = parse("Paan-B*14:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*14:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*14:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_15_05():
    result = parse("Paan-B*15:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*15:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*15:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_20_05():
    result = parse("Paan-B*20:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*20:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*20:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_20_06():
    result = parse("Paan-B*20:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*20:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*20:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_30_01_02():
    result = parse("Paan-B*30:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*30:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*30:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_28_02():
    result = parse("Paan-B*28:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*28:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*28:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_32_03():
    result = parse("Paan-B*32:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*32:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*32:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_32_04():
    result = parse("Paan-B*32:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*32:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*32:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_60_01():
    result = parse("Paan-B*60:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*60:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paan-B*60:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_07_02():
    result = parse("Mafa-B*030:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_133_01():
    result = parse("Mafa-A1*133:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*133:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*133:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_134_01():
    result = parse("Mafa-A1*134:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*134:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*134:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_089_09():
    result = parse("Mafa-A1*089:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*089:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*089:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_27():
    result = parse("Mafa-A3*13:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_07():
    result = parse("Mafa-A5*30:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A5*30:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_09_02():
    result = parse("Mafa-A6*01:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A6*01:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_035_02():
    result = parse("Mafa-B*035:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*035:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*035:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_08():
    result = parse("Mafa-B*045:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*045:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_28():
    result = parse("Mafa-B*060:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_14():
    result = parse("Mafa-B*068:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_07_01():
    result = parse("Mafa-B*115:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*115:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_37_02():
    result = parse("Mafa-I*01:37:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:37:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:37:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_010_09():
    result = parse("Mafa-A1*010:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*010:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*010:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_023_02():
    result = parse("Mafa-A1*023:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*023:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*023:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_135_01():
    result = parse("Mafa-A1*135:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*135:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*135:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_13():
    result = parse("Mafa-A1*043:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_056_04():
    result = parse("Mafa-A1*056:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*056:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*056:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_080_02():
    result = parse("Mafa-A1*080:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*080:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*080:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_28():
    result = parse("Mafa-A3*13:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_03_02():
    result = parse("Mafa-A5*30:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A5*30:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_097_02():
    result = parse("Mafa-B*097:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*097:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*097:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_48():
    result = parse("Mafa-I*01:48")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:48") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:48") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_B_04_01():
    result = parse("Caja-B*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-B*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-B*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_24():
    result = parse("Mamu-B*051:24")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*051:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_17():
    result = parse("Mamu-B*060:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB_043_01():
    result = parse("Bofr-DQB*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_01():
    result = parse("Bofr-DQB1*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_02():
    result = parse("Bofr-DQB1*043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_03():
    result = parse("Bofr-DQB1*043:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_04():
    result = parse("Bofr-DQB1*043:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_05():
    result = parse("Bofr-DQB1*043:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_026_04():
    result = parse("BoLA-2*026:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*026:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*026:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_023_01():
    result = parse("Bofr-DQB1*023:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*023:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*023:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_06():
    result = parse("Bofr-DQB1*043:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_07():
    result = parse("Bofr-DQB1*043:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_08():
    result = parse("Bofr-DQB1*043:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_09():
    result = parse("Bofr-DQB1*043:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_10():
    result = parse("Bofr-DQB1*043:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_10_02():
    result = parse("Bofr-DQB1*043:10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bofr-DQB1*043:10:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_11():
    result = parse("Bofr-DQB1*043:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_12():
    result = parse("Bofr-DQB1*043:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_13():
    result = parse("Bofr-DQB1*043:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_14():
    result = parse("Bofr-DQB1*043:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_03_02():
    result = parse("Bofr-DQB1*043:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bofr-DQB1*043:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_15():
    result = parse("Bofr-DQB1*043:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_05_02():
    result = parse("Bofr-DQB1*043:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bofr-DQB1*043:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_010_10():
    result = parse("Bofr-DQB1*010:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*010:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*010:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_16():
    result = parse("Bofr-DQB1*043:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_044_01():
    result = parse("Bofr-DQB1*044:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*044:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*044:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_010_05():
    result = parse("Bofr-DQB1*010:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*010:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*010:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_04_02():
    result = parse("Bofr-DQB1*043:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bofr-DQB1*043:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_17():
    result = parse("Bofr-DQB1*043:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_001_05():
    result = parse("Bofr-DQB1*001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_18():
    result = parse("Bofr-DQB1*043:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_045_01():
    result = parse("Bofr-DQB1*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_010_11():
    result = parse("Bofr-DQB1*010:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*010:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*010:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_19():
    result = parse("Bofr-DQB1*043:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_20():
    result = parse("Bofr-DQB1*043:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_18_02():
    result = parse("Bofr-DQB1*043:18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bofr-DQB1*043:18:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_21():
    result = parse("Bofr-DQB1*043:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_22():
    result = parse("Bofr-DQB1*043:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_13_02():
    result = parse("Bofr-DQB1*043:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bofr-DQB1*043:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bofr_DQB1_043_23():
    result = parse("Bofr-DQB1*043:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Bofr-DQB1*043:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bofr-DQB1*043:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_032_01():
    result = parse("BoLA-DQA*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_001_06():
    result = parse("BoLA-DQA*001:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*001:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*001:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_033_01():
    result = parse("BoLA-DQA*033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_07():
    result = parse("BoLA-DQA*002:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQA_002_08():
    result = parse("BoLA-DQA*002:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQA*002:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQA*002:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_04_02():
    result = parse("BoLA-DQB*018:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("BoLA-DQB*018:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DQB_018_12():
    result = parse("BoLA-DQB*018:12")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DQB*018:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DQB*018:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_007_02():
    result = parse("BoLA-DRB3*007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_011_02():
    result = parse("BoLA-DRB3*011:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*011:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*011:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_029_02():
    result = parse("BoLA-DRB3*029:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*029:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*029:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_064_03():
    result = parse("BoLA-DRB3*064:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*064:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*064:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_07():
    result = parse("BoLA-DRB3*027:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_02():
    result = parse("BoLA-DRB3*024:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_03():
    result = parse("BoLA-DRB3*024:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_08():
    result = parse("BoLA-DRB3*027:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_04():
    result = parse("BoLA-DRB3*024:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_05():
    result = parse("BoLA-DRB3*024:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_09():
    result = parse("BoLA-DRB3*027:09")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_06():
    result = parse("BoLA-DRB3*024:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_004_03():
    result = parse("BoLA-DRB3*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_03():
    result = parse("BoLA-DRB3*022:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_074_01():
    result = parse("BoLA-DRB3*074:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*074:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*074:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_028_03():
    result = parse("BoLA-DRB3*028:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*028:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*028:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_04():
    result = parse("BoLA-DRB3*022:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_075_01():
    result = parse("BoLA-DRB3*075:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*075:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*075:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_05():
    result = parse("BoLA-DRB3*022:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_075_02():
    result = parse("BoLA-DRB3*075:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*075:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*075:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_075_03():
    result = parse("BoLA-DRB3*075:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*075:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*075:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_076_01():
    result = parse("BoLA-DRB3*076:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*076:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*076:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_077_01():
    result = parse("BoLA-DRB3*077:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*077:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*077:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_011_03():
    result = parse("BoLA-DRB3*011:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*011:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*011:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_078_01():
    result = parse("BoLA-DRB3*078:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*078:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*078:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_018_03():
    result = parse("BoLA-DRB3*018:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*018:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*018:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_09():
    result = parse("BoLA-DRB3*020:09")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_10():
    result = parse("BoLA-DRB3*020:10")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_077_02():
    result = parse("BoLA-DRB3*077:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*077:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*077:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_07():
    result = parse("BoLA-DRB3*024:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paha_A_02_01():
    result = parse("Paha-A*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paha-A*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Paha-A*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_043_14():
    result = parse("Mafa-A1*043:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*043:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*043:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_047_02():
    result = parse("Mafa-A1*047:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*047:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*047:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_067_05():
    result = parse("Mafa-A1*067:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*067:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*067:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_027_04():
    result = parse("Mafa-B*027:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*027:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*027:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_12_01_01():
    result = parse("Mafa-B*050:12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*050:12:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_180_03():
    result = parse("Mafa-B*180:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*180:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*180:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_074_05N():
    result = parse("Mafa-B*074:05N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*074:05N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*074:05N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_074_06_01_01N():
    result = parse("Mafa-B*074:06:01:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*074:06:01:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*074:06:01:01N") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_03_03():
    result = parse("Math-DPB1*01:03:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:03:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*01:03:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_07_01_02():
    result = parse("Math-DPB1*07:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*07:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*07:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_01_02():
    result = parse("Math-DPB1*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_10():
    result = parse("Math-DPB1*01:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_07():
    result = parse("Math-DPB1*06:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_01_03():
    result = parse("Math-DPB1*06:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Math-DPB1*06:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_11():
    result = parse("Math-DPB1*01:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_01_12():
    result = parse("Math-DPB1*01:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*01:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*01:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_08():
    result = parse("Math-DPB1*06:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Math_DPB1_06_09():
    result = parse("Math-DPB1*06:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Math-DPB1*06:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Math-DPB1*06:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_073_02():
    result = parse("Mafa-A1*073:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*073:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*073:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_17():
    result = parse("SLA-1*08:17")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_21_02():
    result = parse("SLA-1*21:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*21:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*21:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_07():
    result = parse("SLA-1*07:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_23_02():
    result = parse("SLA-1*23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*23:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_07_08():
    result = parse("SLA-1*07:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*07:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*07:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_20_02():
    result = parse("SLA-1*20:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*20:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*20:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_16():
    result = parse("SLA-1*08:16")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_23_03():
    result = parse("SLA-1*23:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*23:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*23:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_19_02():
    result = parse("SLA-1*19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*19:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_21_01():
    result = parse("SLA-1*21:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*21:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*21:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_15():
    result = parse("SLA-1*08:15")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_18_02():
    result = parse("SLA-1*18:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*18:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*18:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_21_03():
    result = parse("SLA-1*21:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*21:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*21:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_23_01():
    result = parse("SLA-1*23:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*23:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*23:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_23_04():
    result = parse("SLA-1*23:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*23:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*23:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_15_03():
    result = parse("SLA-1*15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*15:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_12_03_01():
    result = parse("SLA-12*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-12*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-12*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_12_01_01():
    result = parse("SLA-12*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-12*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-12*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_03_02():
    result = parse("SLA-3*05:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-3*05:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_075_01():
    result = parse("BoLA-2*075:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*075:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*075:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_4_076_01():
    result = parse("BoLA-4*076:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-4*076:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-4*076:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Paan_B_30_01_01():
    result = parse("Paan-B*30:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Paan-B*30:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Paan-B*30:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DMA_01_01_03():
    result = parse("SLA-DMA*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DMA*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DMA*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_5_01_01_02():
    result = parse("SLA-5*01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-5*01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-5*01:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_08():
    result = parse("SLA-DQA*01:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_02_12():
    result = parse("SLA-DQB1*02:12")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*02:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*02:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_04_03():
    result = parse("SLA-DQA*04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*04:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_5_03_01():
    result = parse("SLA-5*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-5*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-5*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DOB1_03_01():
    result = parse("SLA-DOB1*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DOB1*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DOB1*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_MIC2_01_01():
    result = parse("SLA-MIC2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-MIC2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-MIC2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3801():
    result = parse("Sasa-UBA*3801")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3801") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3801") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3601():
    result = parse("Sasa-UBA*3601")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3601") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3601") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_0302():
    result = parse("Sasa-UBA*0302")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*0302") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*0302") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3701():
    result = parse("Sasa-UBA*3701")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3701") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3701") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3402():
    result = parse("Sasa-UBA*3402")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3402") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3402") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Sasa_UBA_3501():
    result = parse("Sasa-UBA*3501")
    assert result.__class__ is Allele, (
        f'Expected parse("Sasa-UBA*3501") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Sasa-UBA*3501") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_22_01():
    result = parse("Ovar-N*22:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*22:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*22:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQB1_07_02():
    result = parse("SLA-DQB1*07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQB1*07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQB1*07:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_02_04_02():
    result = parse("SLA-DQA*02:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*02:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("SLA-DQA*02:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DQA_01_09():
    result = parse("SLA-DQA*01:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DQA*01:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DQA*01:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_14_04():
    result = parse("SLA-DRB1*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_DRB1_04_11():
    result = parse("SLA-DRB1*04:11")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-DRB1*04:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-DRB1*04:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_6_10_01():
    result = parse("SLA-6*10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-6*10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-6*10:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_11_11():
    result = parse("SLA-1*11:11")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*11:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*11:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_04_08():
    result = parse("SLA-3*04:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*04:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*04:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_08():
    result = parse("SLA-3*05:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*05:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_05_07():
    result = parse("SLA-3*05:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*05:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*05:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_08_18():
    result = parse("SLA-1*08:18")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*08:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*08:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_14_04():
    result = parse("SLA-1*14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*14:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_14_05():
    result = parse("SLA-1*14:05")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*14:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*14:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_07_06():
    result = parse("SLA-2*07:06")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*07:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*07:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_01_03():
    result = parse("SLA-2*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_08_07():
    result = parse("SLA-2*08:07")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*08:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*08:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_13_03():
    result = parse("SLA-2*13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*13:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_08():
    result = parse("SLA-2*05:08")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_06_16():
    result = parse("SLA-2*06:16")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*06:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*06:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_06_03():
    result = parse("SLA-3*06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*06:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_09_09():
    result = parse("Caja-G*09:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*09:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*09:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_16_02_02():
    result = parse("Caja-G*16:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*16:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Caja-G*16:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_19_04():
    result = parse("Caja-G*19:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*19:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*19:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_DRB_W016_27():
    result = parse("Caja-DRB*W016:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-DRB*W016:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-DRB*W016:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_24_01():
    result = parse("Ovar-N*24:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*24:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*24:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_02_01_01():
    result = parse("Cahi-DRB1*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_01_01_01():
    result = parse("Cahi-DRB1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_25_01():
    result = parse("Ovar-N*25:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*25:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*25:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_26_01():
    result = parse("Ovar-N*26:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*26:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*26:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_N_27_01():
    result = parse("Ovar-N*27:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-N*27:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-N*27:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Caja_G_08_26():
    result = parse("Caja-G*08:26")
    assert result.__class__ is Allele, (
        f'Expected parse("Caja-G*08:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Caja-G*08:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_14_04():
    result = parse("Mafa-DQA1*01:14:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:14:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:14:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_03_01_01():
    result = parse("Cahi-DRB1*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_04_01_01():
    result = parse("Cahi-DRB1*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_05_01_01():
    result = parse("Cahi-DRB1*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_06_01_01():
    result = parse("Cahi-DRB1*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_07_01_01():
    result = parse("Cahi-DRB1*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_08_01_01():
    result = parse("Cahi-DRB1*08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*08:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_09_01_01():
    result = parse("Cahi-DRB1*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_10_01_01():
    result = parse("Cahi-DRB1*10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*10:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_10_02_01():
    result = parse("Cahi-DRB1*10:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*10:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*10:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_11_01_01():
    result = parse("Cahi-DRB1*11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*11:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_03_02_01():
    result = parse("Cahi-DRB1*03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*03:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_12_01_01():
    result = parse("Cahi-DRB1*12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*12:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_13_01_01():
    result = parse("Cahi-DRB1*13:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*13:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*13:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_14_01_01():
    result = parse("Cahi-DRB1*14:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*14:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*14:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_15_01_01():
    result = parse("Cahi-DRB1*15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*15:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_08_02_01():
    result = parse("Cahi-DRB1*08:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*08:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*08:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_16_01_01():
    result = parse("Cahi-DRB1*16:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*16:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*16:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_06_01_02():
    result = parse("Cahi-DRB1*06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*06:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_10_03_01():
    result = parse("Cahi-DRB1*10:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*10:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*10:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_04_02_01():
    result = parse("Cahi-DRB1*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_09_02_01():
    result = parse("Cahi-DRB1*09:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*09:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*09:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_17_01_01():
    result = parse("Cahi-DRB1*17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*17:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_08_03_01():
    result = parse("Cahi-DRB1*08:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*08:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*08:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_16_02_01():
    result = parse("Cahi-DRB1*16:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*16:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*16:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_11_02_01():
    result = parse("Cahi-DRB1*11:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*11:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*11:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Cahi_DRB1_18_01_01():
    result = parse("Cahi-DRB1*18:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Cahi-DRB1*18:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Cahi-DRB1*18:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_07_04():
    result = parse("Ovar-DRB1*07:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*07:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*07:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DMB_01_01():
    result = parse("Eqca-DMB*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DMB*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DMB*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DMB_02_01():
    result = parse("Eqca-DMB*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DMB*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DMB*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DMB_03_01():
    result = parse("Eqca-DMB*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DMB*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DMB*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DMB_04_01():
    result = parse("Eqca-DMB*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DMB*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DMB*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DMB_04_01_01():
    result = parse("Eqca-DMB*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DMB*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Eqca-DMB*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DMB_05_01():
    result = parse("Eqca-DMB*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DMB*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DMB*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DOB_01_01():
    result = parse("Eqca-DOB*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DOB*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DOB*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DOB_01_02():
    result = parse("Eqca-DOB*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DOB*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DOB*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DOB_01_01_01():
    result = parse("Eqca-DOB*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DOB*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Eqca-DOB*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DOB_01_03():
    result = parse("Eqca-DOB*01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DOB*01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DOB*01:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_002_03():
    result = parse("Eqca-DQA1*002:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*002:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*002:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB1_006_01():
    result = parse("Eqca-DQB1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB2_005_01():
    result = parse("Eqca-DRB2*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB2*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB2*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB3_002_02():
    result = parse("Eqca-DRB3*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB3*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB3*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRA_002_01():
    result = parse("Eqca-DRA*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRA*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRA*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_006_01():
    result = parse("Eqca-DRB1*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_002_04():
    result = parse("Eqca-DQA1*002:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*002:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*002:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQB1_007_01():
    result = parse("Eqca-DQB1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQB1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQB1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_005_01():
    result = parse("Eqca-DQA1*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_007_01():
    result = parse("Eqca-DRB1*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB2_006_01():
    result = parse("Eqca-DRB2*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB2*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB2*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DQA1_002_05():
    result = parse("Eqca-DQA1*002:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DQA1*002:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DQA1*002:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_008_01():
    result = parse("Eqca-DRB1*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Eqca_DRB1_007_02():
    result = parse("Eqca-DRB1*007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Eqca-DRB1*007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Eqca-DRB1*007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_10_09():
    result = parse("SLA-2*10:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*10:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*10:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_20_01():
    result = parse("SLA-2*20:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*20:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*20:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_19_01():
    result = parse("SLA-2*19:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*19:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*19:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_1_19_03():
    result = parse("SLA-1*19:03")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-1*19:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-1*19:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_12():
    result = parse("SLA-3*03:12")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_3_03_11():
    result = parse("SLA-3*03:11")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-3*03:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-3*03:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_11_01_01():
    result = parse("SLA-11*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-11*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-11*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_2_05_09():
    result = parse("SLA-2*05:09")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2*05:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-2*05:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA2_002_01():
    result = parse("Bogr-DQA2*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA2*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA2*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA2_002_02():
    result = parse("Bogr-DQA2*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA2*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA2*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA2_003_01():
    result = parse("Bogr-DQA2*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA2*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA2*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_002_01():
    result = parse("Bogr-DQA1*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_003_01():
    result = parse("Bogr-DQA1*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_004_01():
    result = parse("Bogr-DQA1*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_001_01():
    result = parse("Bogr-DQB*001:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*001:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*001:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_002_01():
    result = parse("Bogr-DQB*002:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*002:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*002:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_003_01():
    result = parse("Bogr-DQB*003:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*003:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*003:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_004_02():
    result = parse("Bogr-DQA1*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_004_01_02():
    result = parse("Bogr-DQA1*004:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*004:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bogr-DQA1*004:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_03_01_01():
    result = parse("Ovar-DQA1*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_03_01_01():
    result = parse("Ovar-DQA2*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_04_01_01():
    result = parse("Ovar-DQA2*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_03_01_02():
    result = parse("Ovar-DQA1*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_05_01_01():
    result = parse("Ovar-DQA2*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_04_01_01():
    result = parse("Ovar-DQA1*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_03_02_01():
    result = parse("Ovar-DQA1*03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*03:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_04_02_01():
    result = parse("Ovar-DQA2*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_01_03_01():
    result = parse("Ovar-DQA2*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_05_02_01():
    result = parse("Ovar-DQA1*05:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*05:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*05:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_06_01_01():
    result = parse("Ovar-DQA1*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_02_01_01():
    result = parse("Ovar-DQA2*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_05_01_01():
    result = parse("Ovar-DQA1*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_01_02_01():
    result = parse("Ovar-DQA2*01:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*01:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*01:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_02_01_02():
    result = parse("Ovar-DQA1*02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*02:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_02_01_01():
    result = parse("Ovar-DQA1*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_01_01_01():
    result = parse("Ovar-DQA2*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_04_02_01():
    result = parse("Ovar-DQA1*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_10_01_01():
    result = parse("Ovar-DQA2*10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*10:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_05_03_01():
    result = parse("Ovar-DQB1*05:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*05:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*05:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_07_01_01():
    result = parse("Ovar-DQB2*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_15_01_01():
    result = parse("Ovar-DQB2*15:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*15:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*15:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_16_01_01():
    result = parse("Ovar-DQB2*16:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*16:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*16:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_07_02_01():
    result = parse("Ovar-DQB2*07:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*07:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*07:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_05_02_01():
    result = parse("Ovar-DQB1*05:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*05:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*05:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_09_01_02():
    result = parse("Ovar-DQA2*09:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*09:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*09:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2like_03_01_01():
    result = parse("Ovar-DQB2like*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2like*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2like*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_10_01_01():
    result = parse("Ovar-DQB1*10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*10:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_05_01_01():
    result = parse("Ovar-DQB1*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_08_01_01():
    result = parse("Ovar-DQA2*08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*08:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_18_01_01():
    result = parse("Ovar-DQB2*18:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*18:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*18:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_08_02_01():
    result = parse("Ovar-DQB2*08:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*08:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*08:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_08_01_01():
    result = parse("Ovar-DQB2*08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*08:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_17_01_01():
    result = parse("Ovar-DQB2*17:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*17:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*17:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_09_01_01():
    result = parse("Ovar-DQA2*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_05_01_02():
    result = parse("Ovar-DQB1*05:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*05:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*05:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_11_01_01():
    result = parse("Ovar-DQB1*11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*11:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_03_02_01():
    result = parse("Ovar-DQB1*03:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*03:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*03:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2like_02_01_01():
    result = parse("Ovar-DQA2like*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2like*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2like*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_06_01_01():
    result = parse("Ovar-DQB2*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_14_01_01():
    result = parse("Ovar-DQB2*14:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*14:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*14:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_09_01_01():
    result = parse("Ovar-DQB2*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_10_01_01():
    result = parse("Ovar-DQB2*10:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*10:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*10:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_14_02_01():
    result = parse("Ovar-DQB2*14:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*14:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*14:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_05_01_01():
    result = parse("Ovar-DQB2*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2like_01_01_01():
    result = parse("Ovar-DQA2like*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2like*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2like*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_03_01_01():
    result = parse("Ovar-DQB1*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2like_03_01_01():
    result = parse("Ovar-DQA2like*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2like*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2like*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_04_01_01():
    result = parse("Ovar-DQB1*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_13_01_01():
    result = parse("Ovar-DQB2*13:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*13:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*13:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_12_01_01():
    result = parse("Ovar-DQB2*12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*12:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_08_03_01():
    result = parse("Ovar-DQB2*08:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*08:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*08:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_01_01_01():
    result = parse("Ovar-DQB1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_23_01_01():
    result = parse("Ovar-DQB2*23:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*23:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*23:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_07_02_01():
    result = parse("Ovar-DQB1*07:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*07:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*07:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_03_01_01():
    result = parse("Ovar-DQB2*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_11_01_01():
    result = parse("Ovar-DQB2*11:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*11:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*11:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_04_01_01():
    result = parse("Ovar-DQB2*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_07_03_01():
    result = parse("Ovar-DQB1*07:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*07:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*07:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2like_01_01_01():
    result = parse("Ovar-DQB2like*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2like*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2like*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_22_01_01():
    result = parse("Ovar-DQB2*22:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*22:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*22:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_02_01_01():
    result = parse("Ovar-DQB1*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_02_01_01():
    result = parse("Ovar-DQB2*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_07_01_01():
    result = parse("Ovar-DQB1*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_21_01_01():
    result = parse("Ovar-DQB2*21:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*21:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*21:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_12_01_01():
    result = parse("Ovar-DQB1*12:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*12:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*12:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_08_01_01():
    result = parse("Ovar-DQB1*08:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*08:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*08:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_05_04_01():
    result = parse("Ovar-DQB1*05:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*05:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*05:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_06_01_01():
    result = parse("Ovar-DQB1*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_08_02_01():
    result = parse("Ovar-DQB1*08:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*08:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*08:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_13_01_01():
    result = parse("Ovar-DQB1*13:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*13:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*13:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2like_02_01_01():
    result = parse("Ovar-DQB2like*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2like*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2like*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_09_01_01():
    result = parse("Ovar-DQB1*09:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*09:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*09:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_01_01_01():
    result = parse("Ovar-DQB2*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_19_01_01():
    result = parse("Ovar-DQB2*19:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*19:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*19:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2_20_01_01():
    result = parse("Ovar-DQB2*20:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2*20:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2*20:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB1_14_01_01():
    result = parse("Ovar-DQB1*14:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB1*14:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB1*14:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQB2like_02_02_01():
    result = parse("Ovar-DQB2like*02:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQB2like*02:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQB2like*02:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_07_01_01():
    result = parse("Ovar-DQA1*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA1_01_01_01():
    result = parse("Ovar-DQA1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_06_01_01():
    result = parse("Ovar-DQA2*06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*06:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DQA2_07_01_01():
    result = parse("Ovar-DQA2*07:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DQA2*07:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovar-DQA2*07:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_002_02():
    result = parse("Bogr-DQB*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_02_02():
    result = parse("Bubu-DQA*001:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bubu-DQA*001:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_001_09_02():
    result = parse("Bubu-DQA*001:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*001:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bubu-DQA*001:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_22_02_02():
    result = parse("Bubu-DQA*22:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*22:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bubu-DQA*22:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_11_02():
    result = parse("Bogr-DQA1*11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*11:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_024_01_02():
    result = parse("Bubu-DQA*024:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*024:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bubu-DQA*024:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_028_01_02():
    result = parse("Bubu-DQA*028:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*028:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bubu-DQA*028:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bubu_DQA_028_01_03():
    result = parse("Bubu-DQA*028:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bubu-DQA*028:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Bubu-DQA*028:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_003_02():
    result = parse("Bogr-DQA1*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_005_01():
    result = parse("Bogr-DQA1*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA1_004_03():
    result = parse("Bogr-DQA1*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA1*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA1*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQA2_004_01():
    result = parse("Bogr-DQA2*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQA2*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQA2*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_004_01():
    result = parse("Bogr-DQB*004:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*004:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*004:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_004_02():
    result = parse("Bogr-DQB*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_005_01():
    result = parse("Bogr-DQB*005:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*005:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*005:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_001_02():
    result = parse("Bogr-DQB*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_006_01():
    result = parse("Bogr-DQB*006:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*006:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*006:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_004_03():
    result = parse("Bogr-DQB*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_001_03():
    result = parse("Bogr-DQB*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_007_01():
    result = parse("Bogr-DQB*007:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*007:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*007:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_001_04():
    result = parse("Bogr-DQB*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_008_01():
    result = parse("Bogr-DQB*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_009_01():
    result = parse("Bogr-DQB*009:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*009:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*009:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_009_02():
    result = parse("Bogr-DQB*009:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*009:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*009:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_010_01():
    result = parse("Bogr-DQB*010:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*010:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*010:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Bogr_DQB_011_01():
    result = parse("Bogr-DQB*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Bogr-DQB*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Bogr-DQB*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_29():
    result = parse("Mafa-A3*13:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_08():
    result = parse("Mafa-A5*30:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A5*30:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_09():
    result = parse("Mafa-B*045:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*045:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_144_06():
    result = parse("Mafa-B*144:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*144:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*144:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_08_02():
    result = parse("Mafa-B*046:08:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:08:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*046:08:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_29():
    result = parse("Mafa-B*060:29")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_063_03():
    result = parse("Mafa-B*063:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*063:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*063:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_11():
    result = parse("Mafa-B*072:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_092_03():
    result = parse("Mafa-B*092:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*092:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*092:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_18():
    result = parse("Mafa-B*098:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_08():
    result = parse("Mafa-B*115:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*115:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_115_07_02():
    result = parse("Mafa-B*115:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*115:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*115:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_11():
    result = parse("Mafa-I*01:14:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:11") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_19():
    result = parse("Mafa-DRB1*04:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*04:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_04_15_03():
    result = parse("Mafa-DRB1*04:15:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*04:15:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*04:15:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_07_05():
    result = parse("Mafa-DRB1*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_10_02():
    result = parse("Mafa-DRB*W001:10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W001:10:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_04_02():
    result = parse("Mafa-DRB*W002:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W002:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_13():
    result = parse("Mafa-DRB*W004:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_14():
    result = parse("Mafa-DRB*W006:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W021_04_02():
    result = parse("Mafa-DRB*W021:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W021:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W021:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W111_01():
    result = parse("Mafa-DRB*W111:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W111:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W111:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W027_01_02():
    result = parse("Mafa-DRB*W027:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W027:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W027:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W068_02():
    result = parse("Mafa-DRB*W068:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W068:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W068:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_01_03_05():
    result = parse("Mafa-DQA1*01:03:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*01:03:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*01:03:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_16():
    result = parse("Mafa-DQA1*05:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*05:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_02_02():
    result = parse("Mafa-DQA1*05:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_24_13():
    result = parse("Mafa-DQA1*24:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*24:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQA1*24:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_11_02():
    result = parse("Mafa-DQB1*06:11:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:11:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:11:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_06_12_02():
    result = parse("Mafa-DQB1*06:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*06:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQB1*06:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_11():
    result = parse("Mafa-DQB1*17:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_06():
    result = parse("Mafa-DPB1*03:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*03:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_04_03():
    result = parse("Mafa-DPB1*03:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*03:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_23_01_03():
    result = parse("Mafa-DQA1*23:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*23:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*23:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_19_07():
    result = parse("Mafa-DPB1*19:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*19:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*19:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_9_02_01():
    result = parse("SLA-9*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-9*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-9*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_TAP1_01_01():
    result = parse("SLA-TAP1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-TAP1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-TAP1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_TAP2_01_01():
    result = parse("SLA-TAP2*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-TAP2*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-TAP2*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_9_05_01():
    result = parse("SLA-9*05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-9*05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-9*05:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_9_04_01():
    result = parse("SLA-9*04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-9*04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-9*04:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_9_03_01():
    result = parse("SLA-9*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-9*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-9*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_4_02_01():
    result = parse("SLA-4*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-4*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-4*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_4_03_01():
    result = parse("SLA-4*03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-4*03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-4*03:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_MIC1_01_01():
    result = parse("SLA-MIC1*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-MIC1*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-MIC1*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_11_02_01():
    result = parse("SLA-11*02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-11*02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-11*02:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_4_01_01():
    result = parse("SLA-4*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-4*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-4*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_SLA_9_01_01():
    result = parse("SLA-9*01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-9*01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("SLA-9*01:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_27():
    result = parse("Mafa-DRA*01:02:27")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:27") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_14():
    result = parse("Mafa-DRA*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_13_02():
    result = parse("Mafa-DRA*01:13:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:13:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:13:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_02_28():
    result = parse("Mafa-DRA*01:02:28")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:02:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:02:28") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_15():
    result = parse("Mafa-DRA*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRA*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRA_01_03_10():
    result = parse("Mafa-DRA*01:03:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRA*01:03:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRA*01:03:10") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_01_02():
    result = parse("Mafa-DRB1*03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*03:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_38():
    result = parse("Mafa-DRB1*03:38")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:38") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:38") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_03_39():
    result = parse("Mafa-DRB1*03:39")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*03:39") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*03:39") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_07_06():
    result = parse("Mafa-DRB1*07:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*07:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB1*07:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB1_10_05_02():
    result = parse("Mafa-DRB1*10:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB1*10:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB1*10:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_17():
    result = parse("Mafa-DRB*W001:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W001_18():
    result = parse("Mafa-DRB*W001:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W001:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W001:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W002_09():
    result = parse("Mafa-DRB*W002:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W002:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W002:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_14():
    result = parse("Mafa-DRB*W004:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W004_15():
    result = parse("Mafa-DRB*W004:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W004:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W004:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_07_02():
    result = parse("Mafa-DRB*W006:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W006:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_15():
    result = parse("Mafa-DRB*W006:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W006_16():
    result = parse("Mafa-DRB*W006:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W006:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W006:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W007_09():
    result = parse("Mafa-DRB*W007:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W007:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W007:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W021_05():
    result = parse("Mafa-DRB*W021:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W021:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W021:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W040_01_02():
    result = parse("Mafa-DRB*W040:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W040:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DRB*W040:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DRB_W067_03():
    result = parse("Mafa-DRB*W067:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DRB*W067:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DRB*W067:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_06_02():
    result = parse("Mafa-DQA1*05:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_05_04_02():
    result = parse("Mafa-DQA1*05:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*05:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*05:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQA1_26_01_05():
    result = parse("Mafa-DQA1*26:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQA1*26:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DQA1*26:01:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_17_12():
    result = parse("Mafa-DQB1*17:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*17:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*17:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DQB1_24_02():
    result = parse("Mafa-DQB1*24:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DQB1*24:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DQB1*24:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_06_03():
    result = parse("Mafa-DPA1*02:06:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:06:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:06:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_08_03():
    result = parse("Mafa-DPA1*02:08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:08:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_32_02():
    result = parse("Mafa-DPA1*02:32:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:32:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPA1*02:32:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_47():
    result = parse("Mafa-DPA1*02:47")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:47") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:47") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_48():
    result = parse("Mafa-DPA1*02:48")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:48") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:48") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_02_49():
    result = parse("Mafa-DPA1*02:49")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*02:49") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*02:49") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_04_04():
    result = parse("Mafa-DPA1*04:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*04:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*04:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_07_16():
    result = parse("Mafa-DPA1*07:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*07:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*07:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_08_03():
    result = parse("Mafa-DPA1*08:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*08:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*08:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_08_04():
    result = parse("Mafa-DPA1*08:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*08:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*08:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPA1_09_03():
    result = parse("Mafa-DPA1*09:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPA1*09:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPA1*09:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_04_02():
    result = parse("Mafa-DPB1*01:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-DPB1*01:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_01_16():
    result = parse("Mafa-DPB1*01:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*01:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*01:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_02_07():
    result = parse("Mafa-DPB1*02:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*02:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*02:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_03_07():
    result = parse("Mafa-DPB1*03:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*03:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*03:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_06_10():
    result = parse("Mafa-DPB1*06:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*06:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*06:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_07_05():
    result = parse("Mafa-DPB1*07:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*07:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*07:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_07_06():
    result = parse("Mafa-DPB1*07:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*07:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*07:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_08_08():
    result = parse("Mafa-DPB1*08:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*08:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*08:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_09_05():
    result = parse("Mafa-DPB1*09:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*09:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*09:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_09_06():
    result = parse("Mafa-DPB1*09:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*09:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*09:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_DPB1_17_03():
    result = parse("Mafa-DPB1*17:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-DPB1*17:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-DPB1*17:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_004_05():
    result = parse("Mafa-A1*004:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*004:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*004:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_007_09():
    result = parse("Mafa-A1*007:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*007:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*007:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_015_04():
    result = parse("Mafa-A1*015:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*015:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*015:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_015_01_02():
    result = parse("Mafa-A1*015:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*015:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*015:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_019_07():
    result = parse("Mafa-A1*019:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*019:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*019:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_022_13():
    result = parse("Mafa-A1*022:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*022:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*022:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_041_05():
    result = parse("Mafa-A1*041:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*041:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*041:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_006_03():
    result = parse("Mafa-A1*006:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*006:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*006:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_053_01_02():
    result = parse("Mafa-A1*053:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*053:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*053:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_063_04_01():
    result = parse("Mafa-A1*063:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*063:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*063:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_063_05():
    result = parse("Mafa-A1*063:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*063:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*063:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_063_04_02():
    result = parse("Mafa-A1*063:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*063:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*063:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_065_06():
    result = parse("Mafa-A1*065:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*065:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*065:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_066_07():
    result = parse("Mafa-A1*066:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*066:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*066:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_067_06():
    result = parse("Mafa-A1*067:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*067:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*067:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_025_01():
    result = parse("Mafa-A1*025:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*025:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*025:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_025_02():
    result = parse("Mafa-A1*025:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*025:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*025:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_074_01_04():
    result = parse("Mafa-A1*074:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*074:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*074:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_07():
    result = parse("Mafa-A1*090:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*090:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_090_04_02():
    result = parse("Mafa-A1*090:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*090:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*090:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_092_01_02():
    result = parse("Mafa-A1*092:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*092:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*092:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_003_09():
    result = parse("Mafa-A1*003:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*003:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*003:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_01_02():
    result = parse("Mafa-A2*01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*01:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_67():
    result = parse("Mafa-A2*05:67")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:67") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:67") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_68():
    result = parse("Mafa-A2*05:68")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:68") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:68") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_62_02():
    result = parse("Mafa-A2*05:62:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:62:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A2*05:62:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_05_69():
    result = parse("Mafa-A2*05:69")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*05:69") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*05:69") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A2_24_09():
    result = parse("Mafa-A2*24:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A2*24:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A2*24:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_30():
    result = parse("Mafa-A3*13:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_07_02():
    result = parse("Mafa-A3*13:07:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:07:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A3*13:07:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_31():
    result = parse("Mafa-A3*13:31")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:31") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:31") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A3_13_32():
    result = parse("Mafa-A3*13:32")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A3*13:32") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A3*13:32") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_14():
    result = parse("Mafa-A6*01:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A6_01_15():
    result = parse("Mafa-A6*01:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A6*01:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A6*01:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_003_03():
    result = parse("Mafa-B*003:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*003:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*003:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_008_04():
    result = parse("Mafa-B*008:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*008:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*008:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_011_07():
    result = parse("Mafa-B*011:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*011:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*011:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_013_20():
    result = parse("Mafa-B*013:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*013:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*013:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_02_03():
    result = parse("Mafa-B*030:02:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:02:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:02:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_22():
    result = parse("Mafa-B*030:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*030:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_15_02():
    result = parse("Mafa-B*030:15:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:15:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*030:15:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_033_05():
    result = parse("Mafa-B*033:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*033:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*033:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_034_05():
    result = parse("Mafa-B*034:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*034:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*034:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_035_03():
    result = parse("Mafa-B*035:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*035:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*035:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_035_04():
    result = parse("Mafa-B*035:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*035:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*035:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_01_04():
    result = parse("Mafa-B*036:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*036:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_036_05():
    result = parse("Mafa-B*036:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*036:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*036:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_12():
    result = parse("Mafa-B*044:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_13():
    result = parse("Mafa-B*044:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_14():
    result = parse("Mafa-B*044:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_15():
    result = parse("Mafa-B*044:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*044:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_03_02():
    result = parse("Mafa-B*045:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*045:03:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_045_10():
    result = parse("Mafa-B*045:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*045:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*045:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_19():
    result = parse("Mafa-B*046:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_20():
    result = parse("Mafa-B*046:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_046_21():
    result = parse("Mafa-B*046:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*046:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*046:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_048_07():
    result = parse("Mafa-B*048:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*048:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*048:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_10_02():
    result = parse("Mafa-B*050:10:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:10:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*050:10:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_17():
    result = parse("Mafa-B*051:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_18():
    result = parse("Mafa-B*051:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_19():
    result = parse("Mafa-B*051:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_20():
    result = parse("Mafa-B*051:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_21():
    result = parse("Mafa-B*051:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_22():
    result = parse("Mafa-B*051:22")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_23():
    result = parse("Mafa-B*051:23")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*051:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_056_06():
    result = parse("Mafa-B*056:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*056:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*056:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_30():
    result = parse("Mafa-B*060:30")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*060:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_15():
    result = parse("Mafa-B*068:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_16():
    result = parse("Mafa-B*068:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*068:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_06_01_01():
    result = parse("Mafa-B*070:06:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:06:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*070:06:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_07():
    result = parse("Mafa-B*070:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*070:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_08():
    result = parse("Mafa-B*070:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*070:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_12_01():
    result = parse("Mafa-B*072:12:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:12:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*072:12:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_12_02():
    result = parse("Mafa-B*072:12:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:12:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*072:12:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_13():
    result = parse("Mafa-B*072:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_05_03():
    result = parse("Mafa-B*072:05:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:05:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*072:05:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_05_04():
    result = parse("Mafa-B*072:05:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:05:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*072:05:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_05_05():
    result = parse("Mafa-B*072:05:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:05:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*072:05:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_05_06():
    result = parse("Mafa-B*072:05:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:05:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*072:05:06") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_14():
    result = parse("Mafa-B*072:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_15():
    result = parse("Mafa-B*072:15")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_16():
    result = parse("Mafa-B*072:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_17():
    result = parse("Mafa-B*072:17")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_073_04():
    result = parse("Mafa-B*073:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*073:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*073:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_08():
    result = parse("Mafa-B*079:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*079:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_09_02():
    result = parse("Mafa-B*079:09:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:09:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*079:09:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_072_18():
    result = parse("Mafa-B*072:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*072:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*072:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_04_02():
    result = parse("Mafa-B*088:04:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:04:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*088:04:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_088_04_03():
    result = parse("Mafa-B*088:04:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*088:04:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*088:04:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_092_04():
    result = parse("Mafa-B*092:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*092:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*092:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_093_03():
    result = parse("Mafa-B*093:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*093:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*093:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_19():
    result = parse("Mafa-B*098:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_16_02():
    result = parse("Mafa-B*098:16:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:16:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:16:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_17_05():
    result = parse("Mafa-B*098:17:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:17:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*098:17:05") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_20():
    result = parse("Mafa-B*098:20")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_098_21():
    result = parse("Mafa-B*098:21")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*098:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*098:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_100_02():
    result = parse("Mafa-B*100:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*100:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*100:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_102_02():
    result = parse("Mafa-B*102:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*102:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*102:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_49():
    result = parse("Mafa-I*01:49")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:49") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:49") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_50():
    result = parse("Mafa-I*01:50")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:50") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:50") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_23_02():
    result = parse("Mafa-I*01:23:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:23:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:23:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_01_03():
    result = parse("Mafa-I*01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_51():
    result = parse("Mafa-I*01:51")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:51") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:51") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_13_03():
    result = parse("Mafa-I*01:13:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:13:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:13:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_10():
    result = parse("Mafa-B*109:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_109_11():
    result = parse("Mafa-B*109:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*109:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*109:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_110_02():
    result = parse("Mafa-B*110:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*110:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*110:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_114_05():
    result = parse("Mafa-B*114:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*114:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*114:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_131_04():
    result = parse("Mafa-B*131:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*131:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*131:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_07():
    result = parse("Mafa-B*134:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*134:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_134_05_02():
    result = parse("Mafa-B*134:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*134:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*134:05:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_137_10():
    result = parse("Mafa-B*137:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*137:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*137:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_144_07():
    result = parse("Mafa-B*144:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*144:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*144:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_147_05():
    result = parse("Mafa-B*147:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*147:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*147:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_162_01_02():
    result = parse("Mafa-B*162:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*162:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*162:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_162_02_02():
    result = parse("Mafa-B*162:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*162:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-B*162:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_52():
    result = parse("Mafa-I*01:52")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:52") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:52") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_53():
    result = parse("Mafa-I*01:53")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:53") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:53") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_19_02():
    result = parse("Mafa-I*01:19:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:19:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:19:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_12():
    result = parse("Mafa-I*01:14:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:12") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_54():
    result = parse("Mafa-I*01:54")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:54") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:54") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_55():
    result = parse("Mafa-I*01:55")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:55") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:55") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_56():
    result = parse("Mafa-I*01:56")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:56") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:56") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_57():
    result = parse("Mafa-I*01:57")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:57") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:57") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_58():
    result = parse("Mafa-I*01:58")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:58") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:58") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_59():
    result = parse("Mafa-I*01:59")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:59") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:59") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_60():
    result = parse("Mafa-I*01:60")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:60") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:60") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_61():
    result = parse("Mafa-I*01:61")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:61") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:61") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_13():
    result = parse("Mafa-I*01:14:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:13") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_62():
    result = parse("Mafa-I*01:62")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:62") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:62") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_14_14():
    result = parse("Mafa-I*01:14:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:14:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-I*01:14:14") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_63():
    result = parse("Mafa-I*01:63")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:63") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:63") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_64():
    result = parse("Mafa-I*01:64")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:64") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:64") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_65():
    result = parse("Mafa-I*01:65")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:65") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:65") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_66():
    result = parse("Mafa-I*01:66")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:66") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:66") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_67():
    result = parse("Mafa-I*01:67")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:67") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:67") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_68():
    result = parse("Mafa-I*01:68")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:68") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:68") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_69():
    result = parse("Mafa-I*01:69")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:69") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:69") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_70():
    result = parse("Mafa-I*01:70")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:70") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-I*01:70") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_054_03():
    result = parse("Mafa-A1*054:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*054:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-A1*054:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_092_01_01Sp():
    result = parse("Mafa-A1*092:01:01Sp")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*092:01:01Sp") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mafa-A1*092:01:01Sp") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_042_01():
    result = parse("Mafa-B*042:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*042:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mafa-B*042:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB1_02_16():
    result = parse("Patr-DRB1*02:16")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB1*02:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB1*02:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Patr_DRB5_03_14():
    result = parse("Patr-DRB5*03:14")
    assert result.__class__ is Allele, (
        f'Expected parse("Patr-DRB5*03:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Patr-DRB5*03:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_19_04():
    result = parse("Ovar-DRB1*19:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*19:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*19:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_16_11():
    result = parse("Ovar-DRB1*16:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*16:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*16:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_12_05():
    result = parse("Ovar-DRB1*12:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*12:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*12:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_28_01():
    result = parse("Ovar-DRB1*28:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*28:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*28:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_04_07():
    result = parse("Ovar-DRB1*04:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*04:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*04:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_11():
    result = parse("Ovar-DRB1*10:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_10_12():
    result = parse("Ovar-DRB1*10:12")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*10:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*10:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovar_DRB1_25_03():
    result = parse("Ovar-DRB1*25:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovar-DRB1*25:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Ovar-DRB1*25:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_079_01():
    result = parse("BoLA-DRB3*079:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*079:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*079:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_002_02():
    result = parse("BoLA-DRB3*002:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*002:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*002:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_080_01():
    result = parse("BoLA-DRB3*080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_081_01():
    result = parse("BoLA-DRB3*081:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*081:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*081:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_06():
    result = parse("BoLA-DRB3*022:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_082_01():
    result = parse("BoLA-DRB3*082:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*082:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*082:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_08():
    result = parse("BoLA-DRB3*024:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_09():
    result = parse("BoLA-DRB3*024:09")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_019_03():
    result = parse("BoLA-DRB3*019:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*019:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*019:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_083_01():
    result = parse("BoLA-DRB3*083:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*083:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*083:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_084_01():
    result = parse("BoLA-DRB3*084:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*084:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*084:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_085_01():
    result = parse("BoLA-DRB3*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_10():
    result = parse("BoLA-DRB3*024:10")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_086_01():
    result = parse("BoLA-DRB3*086:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*086:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*086:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_087_01():
    result = parse("BoLA-DRB3*087:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*087:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*087:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_088_01():
    result = parse("BoLA-DRB3*088:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*088:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*088:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_087_02():
    result = parse("BoLA-DRB3*087:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*087:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*087:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_028_04():
    result = parse("BoLA-DRB3*028:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*028:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*028:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_085_02():
    result = parse("BoLA-DRB3*085:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*085:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*085:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_021_02():
    result = parse("BoLA-DRB3*021:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*021:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*021:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_11():
    result = parse("BoLA-DRB3*024:11")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_089_01():
    result = parse("BoLA-DRB3*089:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*089:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*089:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_090_01():
    result = parse("BoLA-DRB3*090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_086_02():
    result = parse("BoLA-DRB3*086:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*086:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*086:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_090_02():
    result = parse("BoLA-DRB3*090:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*090:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*090:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_091_01():
    result = parse("BoLA-DRB3*091:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*091:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*091:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_092_01():
    result = parse("BoLA-DRB3*092:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*092:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*092:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_10():
    result = parse("BoLA-DRB3*027:10")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_093_01():
    result = parse("BoLA-DRB3*093:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*093:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*093:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_094_01():
    result = parse("BoLA-DRB3*094:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*094:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*094:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_095_01():
    result = parse("BoLA-DRB3*095:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*095:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*095:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_096_01():
    result = parse("BoLA-DRB3*096:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*096:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*096:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_086_03():
    result = parse("BoLA-DRB3*086:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*086:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*086:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_097_01():
    result = parse("BoLA-DRB3*097:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*097:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*097:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_11():
    result = parse("BoLA-DRB3*027:11")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_098_01():
    result = parse("BoLA-DRB3*098:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*098:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*098:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_12():
    result = parse("BoLA-DRB3*024:12")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_099_01():
    result = parse("BoLA-DRB3*099:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*099:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*099:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_081_02():
    result = parse("BoLA-DRB3*081:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*081:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*081:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_090_03():
    result = parse("BoLA-DRB3*090:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*090:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*090:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_01():
    result = parse("BoLA-DRB3*100:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_099_02():
    result = parse("BoLA-DRB3*099:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*099:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*099:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_101_01():
    result = parse("BoLA-DRB3*101:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*101:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*101:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_102_01():
    result = parse("BoLA-DRB3*102:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*102:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*102:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_103_01():
    result = parse("BoLA-DRB3*103:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*103:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*103:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_07():
    result = parse("BoLA-DRB3*022:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_104_01():
    result = parse("BoLA-DRB3*104:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*104:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*104:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_105_01():
    result = parse("BoLA-DRB3*105:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*105:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*105:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_106_01():
    result = parse("BoLA-DRB3*106:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*106:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*106:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_107_01():
    result = parse("BoLA-DRB3*107:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*107:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*107:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_081_03():
    result = parse("BoLA-DRB3*081:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*081:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*081:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_107_02():
    result = parse("BoLA-DRB3*107:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*107:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*107:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_010_03():
    result = parse("BoLA-DRB3*010:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*010:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*010:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_094_02():
    result = parse("BoLA-DRB3*094:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*094:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*094:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_018_04():
    result = parse("BoLA-DRB3*018:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*018:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*018:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_005_05():
    result = parse("BoLA-DRB3*005:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*005:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*005:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_108_01():
    result = parse("BoLA-DRB3*108:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*108:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*108:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_13():
    result = parse("BoLA-DRB3*024:13")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_08():
    result = parse("BoLA-DRB3*022:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_109_01():
    result = parse("BoLA-DRB3*109:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*109:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*109:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_14():
    result = parse("BoLA-DRB3*024:14")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_028_05():
    result = parse("BoLA-DRB3*028:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*028:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*028:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_15():
    result = parse("BoLA-DRB3*024:15")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_16():
    result = parse("BoLA-DRB3*024:16")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_009_04():
    result = parse("BoLA-DRB3*009:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*009:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*009:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_110_01():
    result = parse("BoLA-DRB3*110:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*110:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*110:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_097_02():
    result = parse("BoLA-DRB3*097:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*097:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*097:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_010_04():
    result = parse("BoLA-DRB3*010:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*010:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*010:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_17():
    result = parse("BoLA-DRB3*024:17")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_18():
    result = parse("BoLA-DRB3*024:18")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_111_01():
    result = parse("BoLA-DRB3*111:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*111:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*111:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_19():
    result = parse("BoLA-DRB3*024:19")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_20():
    result = parse("BoLA-DRB3*024:20")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:20") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:20") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_032_03():
    result = parse("BoLA-DRB3*032:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*032:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*032:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_12():
    result = parse("BoLA-DRB3*027:12")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_13():
    result = parse("BoLA-DRB3*027:13")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_087_03():
    result = parse("BoLA-DRB3*087:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*087:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*087:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_21():
    result = parse("BoLA-DRB3*024:21")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:21") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:21") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_112_01():
    result = parse("BoLA-DRB3*112:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*112:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*112:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_02():
    result = parse("BoLA-DRB3*100:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_11():
    result = parse("BoLA-DRB3*020:11")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_094_03():
    result = parse("BoLA-DRB3*094:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*094:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*094:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_107_03():
    result = parse("BoLA-DRB3*107:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*107:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*107:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_22():
    result = parse("BoLA-DRB3*024:22")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:22") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:22") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_113_01():
    result = parse("BoLA-DRB3*113:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*113:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*113:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_114_01():
    result = parse("BoLA-DRB3*114:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*114:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*114:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_09():
    result = parse("BoLA-DRB3*022:09")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_115_01():
    result = parse("BoLA-DRB3*115:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*115:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*115:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_116_01():
    result = parse("BoLA-DRB3*116:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*116:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*116:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_117_01():
    result = parse("BoLA-DRB3*117:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*117:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*117:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_005_06():
    result = parse("BoLA-DRB3*005:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*005:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*005:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_118_01():
    result = parse("BoLA-DRB3*118:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*118:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*118:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_114_02():
    result = parse("BoLA-DRB3*114:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*114:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*114:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_03():
    result = parse("BoLA-DRB3*100:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_116_02():
    result = parse("BoLA-DRB3*116:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*116:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*116:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_105_02():
    result = parse("BoLA-DRB3*105:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*105:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*105:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_14():
    result = parse("BoLA-DRB3*027:14")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_119_01():
    result = parse("BoLA-DRB3*119:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*119:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*119:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_008_02():
    result = parse("BoLA-DRB3*008:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*008:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*008:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_120_01():
    result = parse("BoLA-DRB3*120:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*120:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*120:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_097_03():
    result = parse("BoLA-DRB3*097:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*097:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*097:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_011_04():
    result = parse("BoLA-DRB3*011:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*011:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*011:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_121_01():
    result = parse("BoLA-DRB3*121:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*121:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*121:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_04():
    result = parse("BoLA-DRB3*100:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_122_01():
    result = parse("BoLA-DRB3*122:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*122:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*122:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_086_04():
    result = parse("BoLA-DRB3*086:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*086:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*086:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_097_04():
    result = parse("BoLA-DRB3*097:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*097:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*097:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_097_05():
    result = parse("BoLA-DRB3*097:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*097:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*097:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_123_01():
    result = parse("BoLA-DRB3*123:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*123:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*123:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_05():
    result = parse("BoLA-DRB3*100:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_06():
    result = parse("BoLA-DRB3*100:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_124_01():
    result = parse("BoLA-DRB3*124:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*124:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*124:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_12():
    result = parse("BoLA-DRB3*020:12")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_125_01():
    result = parse("BoLA-DRB3*125:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*125:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*125:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_116_03():
    result = parse("BoLA-DRB3*116:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*116:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*116:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_126_01():
    result = parse("BoLA-DRB3*126:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*126:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*126:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_23():
    result = parse("BoLA-DRB3*024:23")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:23") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:23") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_013_03():
    result = parse("BoLA-DRB3*013:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*013:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*013:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_112_02():
    result = parse("BoLA-DRB3*112:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*112:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*112:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_07():
    result = parse("BoLA-DRB3*100:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_13():
    result = parse("BoLA-DRB3*020:13")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_127_01():
    result = parse("BoLA-DRB3*127:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*127:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*127:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_010_05():
    result = parse("BoLA-DRB3*010:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*010:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*010:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_128_01():
    result = parse("BoLA-DRB3*128:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*128:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*128:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_10():
    result = parse("BoLA-DRB3*022:10")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_087_04():
    result = parse("BoLA-DRB3*087:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*087:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*087:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_005_07():
    result = parse("BoLA-DRB3*005:07")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*005:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*005:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_100_08():
    result = parse("BoLA-DRB3*100:08")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*100:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*100:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_129_01():
    result = parse("BoLA-DRB3*129:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*129:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*129:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_130_01():
    result = parse("BoLA-DRB3*130:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*130:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*130:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_011_05():
    result = parse("BoLA-DRB3*011:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*011:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*011:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_24():
    result = parse("BoLA-DRB3*024:24")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:24") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:24") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_15():
    result = parse("BoLA-DRB3*027:15")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:15") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:15") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_007_03():
    result = parse("BoLA-DRB3*007:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*007:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*007:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_131_01():
    result = parse("BoLA-DRB3*131:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*131:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*131:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_105_03():
    result = parse("BoLA-DRB3*105:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*105:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*105:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_012_03():
    result = parse("BoLA-DRB3*012:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*012:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*012:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_013_04():
    result = parse("BoLA-DRB3*013:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*013:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*013:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_097_06():
    result = parse("BoLA-DRB3*097:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*097:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*097:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_25():
    result = parse("BoLA-DRB3*024:25")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:25") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:25") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_020_14():
    result = parse("BoLA-DRB3*020:14")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*020:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*020:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_132_01():
    result = parse("BoLA-DRB3*132:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*132:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*132:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_133_01():
    result = parse("BoLA-DRB3*133:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*133:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*133:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_134_01():
    result = parse("BoLA-DRB3*134:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*134:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*134:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_135_01():
    result = parse("BoLA-DRB3*135:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*135:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*135:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_136_01():
    result = parse("BoLA-DRB3*136:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*136:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*136:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_137_01():
    result = parse("BoLA-DRB3*137:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*137:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*137:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_138_01():
    result = parse("BoLA-DRB3*138:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*138:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*138:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_16():
    result = parse("BoLA-DRB3*027:16")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:16") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:16") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_028_06():
    result = parse("BoLA-DRB3*028:06")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*028:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*028:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_26():
    result = parse("BoLA-DRB3*024:26")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:26") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:26") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_11():
    result = parse("BoLA-DRB3*022:11")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_12():
    result = parse("BoLA-DRB3*022:12")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:12") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:12") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_139_01():
    result = parse("BoLA-DRB3*139:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*139:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*139:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_112_03():
    result = parse("BoLA-DRB3*112:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*112:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*112:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_140_01():
    result = parse("BoLA-DRB3*140:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*140:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*140:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_014_02():
    result = parse("BoLA-DRB3*014:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*014:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*014:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_141_01():
    result = parse("BoLA-DRB3*141:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*141:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*141:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_007_04():
    result = parse("BoLA-DRB3*007:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*007:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*007:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_027_17():
    result = parse("BoLA-DRB3*027:17")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*027:17") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*027:17") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_142_01():
    result = parse("BoLA-DRB3*142:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*142:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*142:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_13():
    result = parse("BoLA-DRB3*022:13")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_022_14():
    result = parse("BoLA-DRB3*022:14")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*022:14") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*022:14") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_27():
    result = parse("BoLA-DRB3*024:27")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:27") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:27") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_121_02():
    result = parse("BoLA-DRB3*121:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*121:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*121:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_143_01():
    result = parse("BoLA-DRB3*143:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*143:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*143:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_144_01():
    result = parse("BoLA-DRB3*144:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*144:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*144:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_02_02():
    result = parse("Mamu-B*017:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*017:02:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_016_02():
    result = parse("Mamu-A1*016:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*016:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*016:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_048_02():
    result = parse("DLA-DQB1*048:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*048:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*048:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_127_01():
    result = parse("DLA-DRB1*127:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*127:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*127:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_004_02():
    result = parse("DLA-DQA1*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_129_01():
    result = parse("DLA-DRB1*129:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*129:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*129:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_509_01():
    result = parse("DLA-88*509:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*509:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*509:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_104_01():
    result = parse("DLA-DRB1*104:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*104:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*104:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_009_03():
    result = parse("DLA-DRB1*009:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*009:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*009:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_042_03():
    result = parse("DLA-DRB1*042:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*042:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*042:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_064_01():
    result = parse("DLA-DRB1*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_119_01():
    result = parse("DLA-DRB1*119:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*119:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*119:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_065_01():
    result = parse("DLA-DQB1*065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_054_02():
    result = parse("DLA-DQB1*054:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*054:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*054:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_018_02():
    result = parse("DLA-DRB1*018:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*018:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*018:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_015_05():
    result = parse("DLA-DRB1*015:05")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*015:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*015:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_042_02():
    result = parse("DLA-DRB1*042:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*042:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*042:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_040_01():
    result = parse("DLA-88*040:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*040:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*040:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_903_01():
    result = parse("DLA-DQB1*903:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*903:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*903:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_124_01():
    result = parse("DLA-DRB1*124:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*124:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*124:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_027_01_2():
    result = parse("DLA-DQA1*027:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*027:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*027:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_064_01():
    result = parse("DLA-DQB1*064:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*064:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*064:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_052_01():
    result = parse("DLA-DQB1*052:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*052:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*052:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_027_01_1():
    result = parse("DLA-DQA1*027:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*027:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*027:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_105_02():
    result = parse("DLA-DRB1*105:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*105:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*105:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_131_01():
    result = parse("DLA-DRB1*131:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*131:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*131:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_005_05():
    result = parse("DLA-DQB1*005:05")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*005:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*005:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_027_02():
    result = parse("DLA-DQA1*027:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*027:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*027:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_066_01():
    result = parse("DLA-DRB1*066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_020_03():
    result = parse("DLA-DQB1*020:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*020:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*020:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_022_01():
    result = parse("DLA-DQA1*022:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*022:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*022:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_029_01():
    result = parse("DLA-DQA1*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_106_01():
    result = parse("DLA-DRB1*106:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*106:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*106:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_008_04():
    result = parse("DLA-DQB1*008:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*008:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*008:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_066_01():
    result = parse("DLA-DQB1*066:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*066:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*066:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_067_01():
    result = parse("DLA-DQB1*067:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*067:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*067:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_029_01():
    result = parse("DLA-88*029:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*029:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*029:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_024_01_2():
    result = parse("DLA-DQA1*024:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*024:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*024:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_045_02():
    result = parse("DLA-88*045:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*045:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*045:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_013_06():
    result = parse("DLA-DQB1*013:06")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*013:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*013:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_089_02():
    result = parse("DLA-DRB1*089:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*089:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*089:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_110_01():
    result = parse("DLA-DRB1*110:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*110:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*110:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_005_03():
    result = parse("DLA-DQB1*005:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*005:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*005:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_012_01():
    result = parse("DLA-88*012:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*012:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*012:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_023_04():
    result = parse("DLA-DQB1*023:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*023:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*023:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_122_01():
    result = parse("DLA-DRB1*122:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*122:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*122:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_049_01():
    result = parse("DLA-88*049:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*049:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*049:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_046_01():
    result = parse("DLA-88*046:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*046:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*046:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_097_02():
    result = parse("DLA-DRB1*097:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*097:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*097:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_028_01():
    result = parse("DLA-DQA1*028:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*028:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*028:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_123_01():
    result = parse("DLA-DRB1*123:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*123:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*123:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_048_03():
    result = parse("DLA-DRB1*048:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*048:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*048:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_05():
    result = parse("DLA-DRB1*001:05")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_089_03():
    result = parse("DLA-DRB1*089:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*089:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*089:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_105_01():
    result = parse("DLA-DRB1*105:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*105:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*105:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_012_02():
    result = parse("DLA-88*012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_017_02():
    result = parse("DLA-DQB1*017:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*017:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*017:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_103_01():
    result = parse("DLA-DRB1*103:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*103:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*103:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_067_02():
    result = parse("DLA-DRB1*067:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*067:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*067:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_64_001_02():
    result = parse("DLA-64*001:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-64*001:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-64*001:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_118_01():
    result = parse("DLA-DRB1*118:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*118:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*118:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_057_01():
    result = parse("DLA-DQB1*057:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*057:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*057:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_065_01():
    result = parse("DLA-DRB1*065:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*065:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*065:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_047_01():
    result = parse("DLA-88*047:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*047:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*047:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_067_01():
    result = parse("DLA-DRB1*067:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*067:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*067:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_008_03():
    result = parse("DLA-DRB1*008:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*008:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*008:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_050_02():
    result = parse("DLA-DQB1*050:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*050:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*050:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_126_01():
    result = parse("DLA-DRB1*126:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*126:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*126:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_037_02():
    result = parse("DLA-DRB1*037:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*037:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*037:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_058_01():
    result = parse("DLA-DQB1*058:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*058:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*058:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_004_02():
    result = parse("DLA-DRB1*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_032_01():
    result = parse("DLA-88*032:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*032:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*032:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_044_02():
    result = parse("DLA-DQB1*044:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*044:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*044:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_033_01():
    result = parse("DLA-88*033:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*033:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*033:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_64_001_01_01():
    result = parse("DLA-64*001:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-64*001:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-64*001:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_036_01():
    result = parse("DLA-88*036:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*036:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*036:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_064_02():
    result = parse("DLA-DRB1*064:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*064:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*064:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_092_01_2():
    result = parse("DLA-DRB1*092:01:2")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*092:01:2") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*092:01:2") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_028_04():
    result = parse("DLA-88*028:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*028:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*028:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_130_01():
    result = parse("DLA-DRB1*130:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*130:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*130:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_032_02():
    result = parse("DLA-88*032:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*032:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*032:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_028_03():
    result = parse("DLA-88*028:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*028:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*028:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_051_01():
    result = parse("DLA-88*051:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*051:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*051:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_128_01():
    result = parse("DLA-DRB1*128:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*128:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*128:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_008_05():
    result = parse("DLA-DQB1*008:05")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*008:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*008:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_050_01():
    result = parse("DLA-DQB1*050:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*050:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*050:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_113_01():
    result = parse("DLA-DRB1*113:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*113:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*113:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_06():
    result = parse("DLA-DRB1*001:06")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_060_01():
    result = parse("DLA-DQB1*060:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*060:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*060:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_110_02():
    result = parse("DLA-DRB1*110:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*110:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*110:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_08():
    result = parse("DLA-DRB1*001:08")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_063_01():
    result = parse("DLA-DRB1*063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_080_02():
    result = parse("DLA-DRB1*080:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*080:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*080:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_062_01():
    result = parse("DLA-DQB1*062:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*062:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*062:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_011_04():
    result = parse("DLA-DRB1*011:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*011:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*011:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_034_02():
    result = parse("DLA-DQB1*034:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*034:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*034:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_015_07():
    result = parse("DLA-DRB1*015:07")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*015:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*015:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_063_01():
    result = parse("DLA-DQB1*063:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*063:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*063:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_030_01():
    result = parse("DLA-DQA1*030:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*030:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*030:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_016_04():
    result = parse("DLA-88*016:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*016:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*016:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_008_06():
    result = parse("DLA-DQB1*008:06")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*008:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*008:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_004_02():
    result = parse("DLA-DQB1*004:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*004:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*004:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_902_01():
    result = parse("DLA-DQB1*902:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*902:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*902:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_019_01():
    result = parse("DLA-DQA1*019:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*019:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQA1*019:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_114_01():
    result = parse("DLA-DRB1*114:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*114:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*114:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_068_01():
    result = parse("DLA-DQB1*068:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*068:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*068:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_056_02():
    result = parse("DLA-DRB1*056:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*056:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*056:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_64_001_01_04():
    result = parse("DLA-64*001:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-64*001:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-64*001:01:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_007_02():
    result = parse("DLA-DQB1*007:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*007:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*007:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_043_01():
    result = parse("DLA-DQB1*043:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*043:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*043:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_117_01():
    result = parse("DLA-DRB1*117:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*117:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*117:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_045_01():
    result = parse("DLA-88*045:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*045:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*045:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_011_01():
    result = parse("DLA-88*011:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*011:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*011:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_009_02():
    result = parse("DLA-DRB1*009:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*009:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*009:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_012_02():
    result = parse("DLA-DRB1*012:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*012:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*012:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_120_01():
    result = parse("DLA-DRB1*120:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*120:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*120:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_006_03():
    result = parse("DLA-DRB1*006:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*006:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*006:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_049_02():
    result = parse("DLA-DQB1*049:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*049:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*049:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_508_02():
    result = parse("DLA-88*508:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*508:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*508:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_022_02():
    result = parse("DLA-DQB1*022:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*022:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*022:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_121_01():
    result = parse("DLA-DRB1*121:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*121:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*121:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_077_02():
    result = parse("DLA-DRB1*077:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*077:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*077:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_07():
    result = parse("DLA-DRB1*001:07")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_901_01():
    result = parse("DLA-DQB1*901:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*901:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*901:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_115_01():
    result = parse("DLA-DRB1*115:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*115:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*115:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_006_02():
    result = parse("DLA-DRB1*006:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*006:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*006:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_038_02():
    result = parse("DLA-DRB1*038:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*038:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*038:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_016_03():
    result = parse("DLA-88*016:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*016:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*016:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_015_06():
    result = parse("DLA-DRB1*015:06")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*015:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*015:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_029_02():
    result = parse("DLA-DQB1*029:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*029:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*029:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_045_03():
    result = parse("DLA-DRB1*045:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*045:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*045:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_112_01():
    result = parse("DLA-DRB1*112:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*112:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*112:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_017_04():
    result = parse("DLA-DRB1*017:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*017:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*017:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_001_03():
    result = parse("DLA-88*001:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*001:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*001:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_043_02():
    result = parse("DLA-DRB1*043:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*043:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*043:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_004_03():
    result = parse("DLA-DRB1*004:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*004:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*004:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_035_02():
    result = parse("DLA-DQB1*035:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*035:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*035:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_023_05():
    result = parse("DLA-DQB1*023:05")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*023:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*023:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_038_03():
    result = parse("DLA-DQB1*038:03")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*038:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*038:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_061_01():
    result = parse("DLA-DQB1*061:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*061:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*061:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_003_02():
    result = parse("DLA-88*003:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*003:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*003:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_125_01():
    result = parse("DLA-DRB1*125:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*125:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*125:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_005_04():
    result = parse("DLA-DQB1*005:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*005:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*005:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQB1_061_02():
    result = parse("DLA-DQB1*061:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQB1*061:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DQB1*061:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_024_01_1():
    result = parse("DLA-DQA1*024:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*024:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*024:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_117_02():
    result = parse("DLA-DRB1*117:02")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*117:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*117:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_116_01():
    result = parse("DLA-DRB1*116:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*116:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*116:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_108_01():
    result = parse("DLA-DRB1*108:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*108:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*108:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_092_01_3():
    result = parse("DLA-DRB1*092:01:3")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*092:01:3") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DRB1*092:01:3") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_021_01():
    result = parse("DLA-88*021:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*021:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*021:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_111_01():
    result = parse("DLA-DRB1*111:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*111:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*111:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DQA1_016_01_1():
    result = parse("DLA-DQA1*016:01:1")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DQA1*016:01:1") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("DLA-DQA1*016:01:1") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_DRB1_001_04():
    result = parse("DLA-DRB1*001:04")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-DRB1*001:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-DRB1*001:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_DLA_88_050_01():
    result = parse("DLA-88*050:01")
    assert result.__class__ is Allele, (
        f'Expected parse("DLA-88*050:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("DLA-88*050:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_08():
    result = parse("Mamu-A1*004:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*004:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_04():
    result = parse("Mamu-A1*008:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*008:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_010_04():
    result = parse("Mamu-A1*010:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*010:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*010:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_053_03():
    result = parse("Mamu-A1*053:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*053:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*053:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_09():
    result = parse("Mamu-B*069:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*069:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_076_04():
    result = parse("Mamu-B*076:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*076:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*076:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_077_03():
    result = parse("Mamu-B*077:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*077:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*077:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A4_14_03_01_02():
    result = parse("Mamu-A4*14:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4*14:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A4*14:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_043_01_01_02():
    result = parse("Mamu-B*043:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*043:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*043:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_043_01_01_03():
    result = parse("Mamu-B*043:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*043:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*043:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_048_01_01_02():
    result = parse("Mamu-B*048:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*048:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*048:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_03_01_02():
    result = parse("Mamu-B*060:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_02_01_02():
    result = parse("Mamu-B*072:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B20_01_04_01_02():
    result = parse("Mamu-B20*01:04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B20*01:04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B20*01:04:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_001_01_01_02():
    result = parse("Mamu-A1*001:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*001:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*001:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_002_01_01_02():
    result = parse("Mamu-A1*002:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*002:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A1*002:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_18():
    result = parse("Mamu-A1*003:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_003_19():
    result = parse("Mamu-A1*003:19")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*003:19") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*003:19") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_004_01_03():
    result = parse("Mamu-A1*004:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*004:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*004:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_006_04():
    result = parse("Mamu-A1*006:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*006:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*006:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_007_08():
    result = parse("Mamu-A1*007:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*007:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*007:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_008_05():
    result = parse("Mamu-A1*008:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*008:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*008:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_011_04():
    result = parse("Mamu-A1*011:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*011:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*011:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_018_13():
    result = parse("Mamu-A1*018:13")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*018:13") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*018:13") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_019_10():
    result = parse("Mamu-A1*019:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*019:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*019:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_063_01_02():
    result = parse("Mamu-A1*063:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*063:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*063:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_063_01_03():
    result = parse("Mamu-A1*063:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*063:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*063:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_123_04():
    result = parse("Mamu-A1*123:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*123:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*123:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A7_01_04():
    result = parse("Mamu-A7*01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7*01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A7*01:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_001_09():
    result = parse("Mamu-B*001:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*001:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*001:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_09():
    result = parse("Mamu-B*007:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*007:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_10():
    result = parse("Mamu-B*007:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*007:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_017_07():
    result = parse("Mamu-B*017:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*017:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*017:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_07():
    result = parse("Mamu-B*030:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*030:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_036_01_03():
    result = parse("Mamu-B*036:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*036:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*036:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_037_01_02():
    result = parse("Mamu-B*037:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*037:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*037:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_039_03():
    result = parse("Mamu-B*039:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*039:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*039:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_039_04():
    result = parse("Mamu-B*039:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*039:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*039:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_039_05():
    result = parse("Mamu-B*039:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*039:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*039:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_041_03():
    result = parse("Mamu-B*041:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*041:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*041:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_045_11():
    result = parse("Mamu-B*045:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*045:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*045:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_048_05():
    result = parse("Mamu-B*048:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*048:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*048:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_056_09():
    result = parse("Mamu-B*056:09")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*056:09") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*056:09") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_18():
    result = parse("Mamu-B*060:18")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:18") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*060:18") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_07():
    result = parse("Mamu-B*065:07")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:07") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*065:07") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_08():
    result = parse("Mamu-B*065:08")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:08") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*065:08") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_03_04():
    result = parse("Mamu-B*065:03:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:03:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*065:03:04") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_066_03():
    result = parse("Mamu-B*066:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*066:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*066:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_069_10():
    result = parse("Mamu-B*069:10")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*069:10") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*069:10") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_11():
    result = parse("Mamu-B*072:11")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:11") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*072:11") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_075_05():
    result = parse("Mamu-B*075:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*075:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*075:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_083_01_02():
    result = parse("Mamu-B*083:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*083:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*083:01:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_090_04():
    result = parse("Mamu-B*090:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*090:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*090:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_091_06():
    result = parse("Mamu-B*091:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*091:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*091:06") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_101_06_01():
    result = parse("Mamu-B*101:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*101:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*101:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_101_06_02():
    result = parse("Mamu-B*101:06:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*101:06:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-B*101:06:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_117_02():
    result = parse("Mamu-B*117:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*117:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-B*117:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_I_01_49_02():
    result = parse("Mamu-I*01:49:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-I*01:49:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-I*01:49:02") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_026_04():
    result = parse("Mamu-A1*026:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*026:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("Mamu-A1*026:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A1_048_01_03():
    result = parse("Mamu-A1*048:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1*048:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Mamu-A1*048:01:03") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_A3_13_02_01_02():
    result = parse("Mamu-A3*13:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3*13:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-A3*13:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_006_01_01_02():
    result = parse("Mamu-B*006:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*006:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*006:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_02_01_02():
    result = parse("Mamu-B*007:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*007:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_03_01_02():
    result = parse("Mamu-B*030:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*030:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_01_02():
    result = parse("Mamu-B*072:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_01_03():
    result = parse("Mamu-B*072:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_01_04():
    result = parse("Mamu-B*072:01:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:01:04") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_01_05():
    result = parse("Mamu-B*072:01:01:05")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:01:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:01:05") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_014_03():
    result = parse("BoLA-DRB3*014:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*014:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*014:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_28():
    result = parse("BoLA-DRB3*024:28")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:28") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:28") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_29():
    result = parse("BoLA-DRB3*024:29")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:29") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:29") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_088_02():
    result = parse("BoLA-DRB3*088:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*088:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*088:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_090_04():
    result = parse("BoLA-DRB3*090:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*090:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*090:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_090_05():
    result = parse("BoLA-DRB3*090:05")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*090:05") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*090:05") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_145_01():
    result = parse("BoLA-DRB3*145:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*145:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*145:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_146_01():
    result = parse("BoLA-DRB3*146:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*146:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*146:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_147_01():
    result = parse("BoLA-DRB3*147:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*147:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*147:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_148_01():
    result = parse("BoLA-DRB3*148:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*148:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*148:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_149_01():
    result = parse("BoLA-DRB3*149:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*149:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*149:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_150_01():
    result = parse("BoLA-DRB3*150:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*150:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*150:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_151_01():
    result = parse("BoLA-DRB3*151:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*151:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*151:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_152_01():
    result = parse("BoLA-DRB3*152:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*152:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*152:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_153_01():
    result = parse("BoLA-DRB3*153:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*153:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*153:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_154_01():
    result = parse("BoLA-DRB3*154:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*154:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*154:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_154_02():
    result = parse("BoLA-DRB3*154:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*154:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*154:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_155_01():
    result = parse("BoLA-DRB3*155:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*155:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*155:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_156_01():
    result = parse("BoLA-DRB3*156:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*156:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*156:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_157_01():
    result = parse("BoLA-DRB3*157:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*157:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*157:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_158_01():
    result = parse("BoLA-DRB3*158:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*158:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*158:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_159_01():
    result = parse("BoLA-DRB3*159:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*159:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*159:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_160_01():
    result = parse("BoLA-DRB3*160:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*160:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*160:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_024_30():
    result = parse("BoLA-DRB3*024:30")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*024:30") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*024:30") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_161_01():
    result = parse("BoLA-DRB3*161:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*161:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*161:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_002_01_01_02():
    result = parse("Mamu-B*002:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*002:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*002:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_007_02_01_03():
    result = parse("Mamu-B*007:02:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*007:02:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*007:02:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_019_01_01_02():
    result = parse("Mamu-B*019:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*019:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*019:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_022_01_01_02():
    result = parse("Mamu-B*022:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*022:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*022:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_030_03_03_02():
    result = parse("Mamu-B*030:03:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*030:03:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*030:03:03:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_041_01_01_02():
    result = parse("Mamu-B*041:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*041:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*041:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_046_15_01_02():
    result = parse("Mamu-B*046:15:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*046:15:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*046:15:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_02_01_02():
    result = parse("Mamu-B*051:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_051_04_02_02():
    result = parse("Mamu-B*051:04:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*051:04:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*051:04:02:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_053_02_01_02():
    result = parse("Mamu-B*053:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*053:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*053:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_01_01_02():
    result = parse("Mamu-B*060:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_060_03_01_03():
    result = parse("Mamu-B*060:03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*060:03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*060:03:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_063_03_01_02():
    result = parse("Mamu-B*063:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*063:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*063:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_01_01_02():
    result = parse("Mamu-B*068:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*068:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_068_02_01_02():
    result = parse("Mamu-B*068:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*068:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*068:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_01_06():
    result = parse("Mamu-B*072:01:01:06")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:01:06") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:01:06") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_01_02_02():
    result = parse("Mamu-B*072:01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:01:02:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_072_02_01_03():
    result = parse("Mamu-B*072:02:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*072:02:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*072:02:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_079_02_01_02():
    result = parse("Mamu-B*079:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*079:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*079:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_098_12_01_02():
    result = parse("Mamu-B*098:12:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*098:12:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*098:12:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_162_02_01_02():
    result = parse("Mamu-B*162:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*162:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*162:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B20_01_04_01_03():
    result = parse("Mamu-B20*01:04:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B20*01:04:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B20*01:04:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_012_01_01_02():
    result = parse("Mamu-B*012:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*012:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*012:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_065_02_01_02():
    result = parse("Mamu-B*065:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*065:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*065:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_097_01_02_02():
    result = parse("Mamu-B*097:01:02:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*097:01:02:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*097:01:02:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_03_01_02():
    result = parse("Mafa-A4*14:03:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:03:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:03:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_06_01_02():
    result = parse("Mafa-A5*30:06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A5*30:06:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_01_01_02():
    result = parse("Mafa-B*007:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*007:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_007_01_05_02():
    result = parse("Mafa-B*007:01:05:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*007:01:05:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*007:01:05:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_037_01_01_02():
    result = parse("Mafa-B*037:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*037:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*037:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_01_01_02():
    result = parse("Mafa-B*044:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*044:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_044_01_01_03():
    result = parse("Mafa-B*044:01:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*044:01:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*044:01:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_08_01_02():
    result = parse("Mafa-B*051:08:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:08:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*051:08:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_04_01_02():
    result = parse("Mafa-B*060:04:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:04:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:04:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_060_13_01_02():
    result = parse("Mafa-B*060:13:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*060:13:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*060:13:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_02_01_02():
    result = parse("Mafa-B*068:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*068:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_01_01_02():
    result = parse("Mafa-B*070:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*070:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_093_02_01_02():
    result = parse("Mafa-B*093:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*093:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*093:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_I_01_10_01_02():
    result = parse("Mafa-I*01:10:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-I*01:10:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-I*01:10:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_AG_01_05_01_01():
    result = parse("Mafa-AG*01:05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-AG*01:05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-AG*01:05:01:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mamu_B_188_02_01_02():
    result = parse("Mamu-B*188:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B*188:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mamu-B*188:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A1_015_01_01_02():
    result = parse("Mafa-A1*015:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A1*015:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A1*015:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_03_01_03():
    result = parse("Mafa-A4*14:03:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:03:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:03:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A4_14_04_02_01():
    result = parse("Mafa-A4*14:04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A4*14:04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A4*14:04:02:01") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_A5_30_06_01_03():
    result = parse("Mafa-A5*30:06:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-A5*30:06:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-A5*30:06:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_030_01_01_02():
    result = parse("Mafa-B*030:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*030:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*030:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_02_01_02():
    result = parse("Mafa-B*041:02:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:02:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*041:02:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_041_02_01_03():
    result = parse("Mafa-B*041:02:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*041:02:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*041:02:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_050_11_01_02():
    result = parse("Mafa-B*050:11:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*050:11:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*050:11:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_051_08_01_03():
    result = parse("Mafa-B*051:08:01:03")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*051:08:01:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*051:08:01:03") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_149_01_01_02():
    result = parse("Mafa-B*149:01:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*149:01:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*149:01:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_131_02_01_04():
    result = parse("Mafa-B*131:02:01:04")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*131:02:01:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*131:02:01:04") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_079_02_03_02():
    result = parse("Mafa-B*079:02:03:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*079:02:03:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*079:02:03:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_070_06_01_02():
    result = parse("Mafa-B*070:06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*070:06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*070:06:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_162_01():
    result = parse("BoLA-DRB3*162:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*162:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*162:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_DRB3_163_01():
    result = parse("BoLA-DRB3*163:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-DRB3*163:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-DRB3*163:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_Mafa_B_068_06_01_02():
    result = parse("Mafa-B*068:06:01:02")
    assert result.__class__ is Allele, (
        f'Expected parse("Mafa-B*068:06:01:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 4, (
        'Expected parse("Mafa-B*068:06:01:02") to have 4 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_01_01():
    result = parse("Ovca-DRA*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_01_01_01():
    result = parse("Ovca-DRB1*01:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*01:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*01:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_02_01_01():
    result = parse("Ovca-DRB1*02:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*02:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*02:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_03_01_01():
    result = parse("Ovca-DRB1*03:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*03:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*03:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_04_01_01():
    result = parse("Ovca-DRB1*04:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*04:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*04:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_05_01_01():
    result = parse("Ovca-DRB1*05:01:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*05:01:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*05:01:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_04_02_01():
    result = parse("Ovca-DRB1*04:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*04:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*04:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_05_02_01():
    result = parse("Ovca-DRB1*05:02:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*05:02:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*05:02:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRB1_05_03_01():
    result = parse("Ovca-DRB1*05:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRB1*05:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRB1*05:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_03_01():
    result = parse("Ovca-DRA*01:03:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:03:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:03:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_04_01():
    result = parse("Ovca-DRA*01:04:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:04:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:04:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_05_01():
    result = parse("Ovca-DRA*01:05:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:05:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:05:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_06_01():
    result = parse("Ovca-DRA*01:06:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:06:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:06:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_07_01():
    result = parse("Ovca-DRA*01:07:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:07:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:07:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_08_01():
    result = parse("Ovca-DRA*01:08:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:08:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:08:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_09_01():
    result = parse("Ovca-DRA*01:09:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:09:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:09:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_Ovca_DRA_01_10_01():
    result = parse("Ovca-DRA*01:10:01")
    assert result.__class__ is Allele, (
        f'Expected parse("Ovca-DRA*01:10:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 3, (
        'Expected parse("Ovca-DRA*01:10:01") to have 3 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_1_097_01():
    result = parse("BoLA-1*097:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1*097:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-1*097:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_085_01():
    result = parse("BoLA-2*085:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*085:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*085:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_087_01():
    result = parse("BoLA-2*087:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*087:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*087:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_045_02():
    result = parse("BoLA-2*045:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*045:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*045:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_089_01():
    result = parse("BoLA-2*089:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*089:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*089:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_079_01():
    result = parse("BoLA-2*079:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*079:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*079:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_071_02():
    result = parse("BoLA-2*071:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*071:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*071:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_060_02():
    result = parse("BoLA-2*060:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*060:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*060:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_2_016_03():
    result = parse("BoLA-2*016:03")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2*016:03") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-2*016:03") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_083_01():
    result = parse("BoLA-3*083:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*083:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*083:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_084_01N():
    result = parse("BoLA-3*084:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*084:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*084:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_086_01N():
    result = parse("BoLA-3*086:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*086:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*086:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_088_01():
    result = parse("BoLA-3*088:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*088:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*088:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_078_01():
    result = parse("BoLA-3*078:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*078:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*078:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_004_04():
    result = parse("BoLA-3*004:04")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*004:04") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*004:04") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_098_01N():
    result = parse("BoLA-3*098:01N")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*098:01N") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*098:01N") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_005_02():
    result = parse("BoLA-NC1*005:02")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*005:02") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*005:02") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_NC1_008_01():
    result = parse("BoLA-NC1*008:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1*008:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-NC1*008:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_080_01():
    result = parse("BoLA-3*080:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*080:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*080:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_3_081_01():
    result = parse("BoLA-3*081:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3*081:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-3*081:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields


def test_BoLA_6_090_01():
    result = parse("BoLA-6*090:01")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6*090:01") to be Allele but got {result}'
    )
    assert result.num_allele_fields == 2, (
        'Expected parse("BoLA-6*090:01") to have 2 field(s) but got %d'
    ) % result.num_allele_fields
