from mhcgnomes import Allele, AlleleWithoutGene, Gene, parse


def test_HLA_A01_01():
    result = parse("HLA-A01:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:01") to be Allele but got {result}'


def test_HLA_A01_02():
    result = parse("HLA-A01:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:02") to be Allele but got {result}'


def test_HLA_A01_03():
    result = parse("HLA-A01:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:03") to be Allele but got {result}'


def test_HLA_A01_06():
    result = parse("HLA-A01:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:06") to be Allele but got {result}'


def test_HLA_A01_07():
    result = parse("HLA-A01:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:07") to be Allele but got {result}'


def test_HLA_A01_08():
    result = parse("HLA-A01:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:08") to be Allele but got {result}'


def test_HLA_A01_09():
    result = parse("HLA-A01:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:09") to be Allele but got {result}'


def test_HLA_A01_10():
    result = parse("HLA-A01:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:10") to be Allele but got {result}'


def test_HLA_A01_12():
    result = parse("HLA-A01:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:12") to be Allele but got {result}'


def test_HLA_A01_13():
    result = parse("HLA-A01:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:13") to be Allele but got {result}'


def test_HLA_A01_14():
    result = parse("HLA-A01:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:14") to be Allele but got {result}'


def test_HLA_A01_17():
    result = parse("HLA-A01:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:17") to be Allele but got {result}'


def test_HLA_A01_19():
    result = parse("HLA-A01:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:19") to be Allele but got {result}'


def test_HLA_A01_20():
    result = parse("HLA-A01:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:20") to be Allele but got {result}'


def test_HLA_A01_21():
    result = parse("HLA-A01:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:21") to be Allele but got {result}'


def test_HLA_A01_23():
    result = parse("HLA-A01:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:23") to be Allele but got {result}'


def test_HLA_A01_24():
    result = parse("HLA-A01:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:24") to be Allele but got {result}'


def test_HLA_A01_25():
    result = parse("HLA-A01:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:25") to be Allele but got {result}'


def test_HLA_A01_26():
    result = parse("HLA-A01:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:26") to be Allele but got {result}'


def test_HLA_A01_28():
    result = parse("HLA-A01:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:28") to be Allele but got {result}'


def test_HLA_A01_29():
    result = parse("HLA-A01:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:29") to be Allele but got {result}'


def test_HLA_A01_30():
    result = parse("HLA-A01:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:30") to be Allele but got {result}'


def test_HLA_A01_32():
    result = parse("HLA-A01:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:32") to be Allele but got {result}'


def test_HLA_A01_33():
    result = parse("HLA-A01:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:33") to be Allele but got {result}'


def test_HLA_A01_35():
    result = parse("HLA-A01:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:35") to be Allele but got {result}'


def test_HLA_A01_36():
    result = parse("HLA-A01:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:36") to be Allele but got {result}'


def test_HLA_A01_37():
    result = parse("HLA-A01:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:37") to be Allele but got {result}'


def test_HLA_A01_38():
    result = parse("HLA-A01:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:38") to be Allele but got {result}'


def test_HLA_A01_39():
    result = parse("HLA-A01:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:39") to be Allele but got {result}'


def test_HLA_A01_40():
    result = parse("HLA-A01:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:40") to be Allele but got {result}'


def test_HLA_A01_41():
    result = parse("HLA-A01:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:41") to be Allele but got {result}'


def test_HLA_A01_42():
    result = parse("HLA-A01:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:42") to be Allele but got {result}'


def test_HLA_A01_43():
    result = parse("HLA-A01:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:43") to be Allele but got {result}'


def test_HLA_A01_44():
    result = parse("HLA-A01:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:44") to be Allele but got {result}'


def test_HLA_A01_45():
    result = parse("HLA-A01:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:45") to be Allele but got {result}'


def test_HLA_A01_46():
    result = parse("HLA-A01:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:46") to be Allele but got {result}'


def test_HLA_A01_47():
    result = parse("HLA-A01:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:47") to be Allele but got {result}'


def test_HLA_A01_48():
    result = parse("HLA-A01:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:48") to be Allele but got {result}'


def test_HLA_A01_49():
    result = parse("HLA-A01:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:49") to be Allele but got {result}'


def test_HLA_A01_50():
    result = parse("HLA-A01:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:50") to be Allele but got {result}'


def test_HLA_A01_51():
    result = parse("HLA-A01:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:51") to be Allele but got {result}'


def test_HLA_A01_54():
    result = parse("HLA-A01:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:54") to be Allele but got {result}'


def test_HLA_A01_55():
    result = parse("HLA-A01:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:55") to be Allele but got {result}'


def test_HLA_A01_58():
    result = parse("HLA-A01:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:58") to be Allele but got {result}'


def test_HLA_A01_59():
    result = parse("HLA-A01:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:59") to be Allele but got {result}'


def test_HLA_A01_60():
    result = parse("HLA-A01:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:60") to be Allele but got {result}'


def test_HLA_A01_61():
    result = parse("HLA-A01:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:61") to be Allele but got {result}'


def test_HLA_A01_62():
    result = parse("HLA-A01:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:62") to be Allele but got {result}'


def test_HLA_A01_63():
    result = parse("HLA-A01:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:63") to be Allele but got {result}'


def test_HLA_A01_64():
    result = parse("HLA-A01:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:64") to be Allele but got {result}'


def test_HLA_A01_65():
    result = parse("HLA-A01:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:65") to be Allele but got {result}'


def test_HLA_A01_66():
    result = parse("HLA-A01:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-A01:66") to be Allele but got {result}'


def test_HLA_A02_01():
    result = parse("HLA-A02:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:01") to be Allele but got {result}'


def test_HLA_A02_02():
    result = parse("HLA-A02:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:02") to be Allele but got {result}'


def test_HLA_A02_03():
    result = parse("HLA-A02:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:03") to be Allele but got {result}'


def test_HLA_A02_04():
    result = parse("HLA-A02:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:04") to be Allele but got {result}'


def test_HLA_A02_05():
    result = parse("HLA-A02:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:05") to be Allele but got {result}'


def test_HLA_A02_06():
    result = parse("HLA-A02:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:06") to be Allele but got {result}'


def test_HLA_A02_07():
    result = parse("HLA-A02:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:07") to be Allele but got {result}'


def test_HLA_A02_08():
    result = parse("HLA-A02:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:08") to be Allele but got {result}'


def test_HLA_A02_09():
    result = parse("HLA-A02:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:09") to be Allele but got {result}'


def test_HLA_A02_10():
    result = parse("HLA-A02:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:10") to be Allele but got {result}'


def test_HLA_A02_11():
    result = parse("HLA-A02:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:11") to be Allele but got {result}'


def test_HLA_A02_12():
    result = parse("HLA-A02:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:12") to be Allele but got {result}'


def test_HLA_A02_13():
    result = parse("HLA-A02:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:13") to be Allele but got {result}'


def test_HLA_A02_14():
    result = parse("HLA-A02:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:14") to be Allele but got {result}'


def test_HLA_A02_16():
    result = parse("HLA-A02:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:16") to be Allele but got {result}'


def test_HLA_A02_17():
    result = parse("HLA-A02:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:17") to be Allele but got {result}'


def test_HLA_A02_18():
    result = parse("HLA-A02:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:18") to be Allele but got {result}'


def test_HLA_A02_19():
    result = parse("HLA-A02:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:19") to be Allele but got {result}'


def test_HLA_A02_20():
    result = parse("HLA-A02:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:20") to be Allele but got {result}'


def test_HLA_A02_21():
    result = parse("HLA-A02:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:21") to be Allele but got {result}'


def test_HLA_A02_22():
    result = parse("HLA-A02:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:22") to be Allele but got {result}'


def test_HLA_A02_24():
    result = parse("HLA-A02:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:24") to be Allele but got {result}'


def test_HLA_A02_25():
    result = parse("HLA-A02:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:25") to be Allele but got {result}'


def test_HLA_A02_26():
    result = parse("HLA-A02:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:26") to be Allele but got {result}'


def test_HLA_A02_27():
    result = parse("HLA-A02:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:27") to be Allele but got {result}'


def test_HLA_A02_28():
    result = parse("HLA-A02:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:28") to be Allele but got {result}'


def test_HLA_A02_29():
    result = parse("HLA-A02:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:29") to be Allele but got {result}'


def test_HLA_A02_30():
    result = parse("HLA-A02:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:30") to be Allele but got {result}'


def test_HLA_A02_31():
    result = parse("HLA-A02:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:31") to be Allele but got {result}'


def test_HLA_A02_33():
    result = parse("HLA-A02:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:33") to be Allele but got {result}'


def test_HLA_A02_34():
    result = parse("HLA-A02:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:34") to be Allele but got {result}'


def test_HLA_A02_35():
    result = parse("HLA-A02:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:35") to be Allele but got {result}'


def test_HLA_A02_36():
    result = parse("HLA-A02:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:36") to be Allele but got {result}'


def test_HLA_A02_37():
    result = parse("HLA-A02:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:37") to be Allele but got {result}'


def test_HLA_A02_38():
    result = parse("HLA-A02:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:38") to be Allele but got {result}'


def test_HLA_A02_39():
    result = parse("HLA-A02:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:39") to be Allele but got {result}'


def test_HLA_A02_40():
    result = parse("HLA-A02:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:40") to be Allele but got {result}'


def test_HLA_A02_41():
    result = parse("HLA-A02:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:41") to be Allele but got {result}'


def test_HLA_A02_42():
    result = parse("HLA-A02:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:42") to be Allele but got {result}'


def test_HLA_A02_44():
    result = parse("HLA-A02:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:44") to be Allele but got {result}'


def test_HLA_A02_45():
    result = parse("HLA-A02:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:45") to be Allele but got {result}'


def test_HLA_A02_46():
    result = parse("HLA-A02:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:46") to be Allele but got {result}'


def test_HLA_A02_47():
    result = parse("HLA-A02:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:47") to be Allele but got {result}'


def test_HLA_A02_48():
    result = parse("HLA-A02:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:48") to be Allele but got {result}'


def test_HLA_A02_49():
    result = parse("HLA-A02:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:49") to be Allele but got {result}'


def test_HLA_A02_50():
    result = parse("HLA-A02:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:50") to be Allele but got {result}'


def test_HLA_A02_51():
    result = parse("HLA-A02:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:51") to be Allele but got {result}'


def test_HLA_A02_52():
    result = parse("HLA-A02:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:52") to be Allele but got {result}'


def test_HLA_A02_54():
    result = parse("HLA-A02:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:54") to be Allele but got {result}'


def test_HLA_A02_55():
    result = parse("HLA-A02:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:55") to be Allele but got {result}'


def test_HLA_A02_56():
    result = parse("HLA-A02:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:56") to be Allele but got {result}'


def test_HLA_A02_57():
    result = parse("HLA-A02:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:57") to be Allele but got {result}'


def test_HLA_A02_58():
    result = parse("HLA-A02:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:58") to be Allele but got {result}'


def test_HLA_A02_59():
    result = parse("HLA-A02:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:59") to be Allele but got {result}'


def test_HLA_A02_60():
    result = parse("HLA-A02:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:60") to be Allele but got {result}'


def test_HLA_A02_61():
    result = parse("HLA-A02:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:61") to be Allele but got {result}'


def test_HLA_A02_62():
    result = parse("HLA-A02:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:62") to be Allele but got {result}'


def test_HLA_A02_63():
    result = parse("HLA-A02:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:63") to be Allele but got {result}'


def test_HLA_A02_64():
    result = parse("HLA-A02:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:64") to be Allele but got {result}'


def test_HLA_A02_65():
    result = parse("HLA-A02:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:65") to be Allele but got {result}'


def test_HLA_A02_66():
    result = parse("HLA-A02:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:66") to be Allele but got {result}'


def test_HLA_A02_67():
    result = parse("HLA-A02:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:67") to be Allele but got {result}'


def test_HLA_A02_68():
    result = parse("HLA-A02:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:68") to be Allele but got {result}'


def test_HLA_A02_69():
    result = parse("HLA-A02:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:69") to be Allele but got {result}'


def test_HLA_A02_70():
    result = parse("HLA-A02:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:70") to be Allele but got {result}'


def test_HLA_A02_71():
    result = parse("HLA-A02:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:71") to be Allele but got {result}'


def test_HLA_A02_72():
    result = parse("HLA-A02:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:72") to be Allele but got {result}'


def test_HLA_A02_73():
    result = parse("HLA-A02:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:73") to be Allele but got {result}'


def test_HLA_A02_74():
    result = parse("HLA-A02:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:74") to be Allele but got {result}'


def test_HLA_A02_75():
    result = parse("HLA-A02:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:75") to be Allele but got {result}'


def test_HLA_A02_76():
    result = parse("HLA-A02:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:76") to be Allele but got {result}'


def test_HLA_A02_77():
    result = parse("HLA-A02:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:77") to be Allele but got {result}'


def test_HLA_A02_78():
    result = parse("HLA-A02:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:78") to be Allele but got {result}'


def test_HLA_A02_79():
    result = parse("HLA-A02:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:79") to be Allele but got {result}'


def test_HLA_A02_80():
    result = parse("HLA-A02:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:80") to be Allele but got {result}'


def test_HLA_A02_81():
    result = parse("HLA-A02:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:81") to be Allele but got {result}'


def test_HLA_A02_84():
    result = parse("HLA-A02:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:84") to be Allele but got {result}'


def test_HLA_A02_85():
    result = parse("HLA-A02:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:85") to be Allele but got {result}'


def test_HLA_A02_86():
    result = parse("HLA-A02:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:86") to be Allele but got {result}'


def test_HLA_A02_87():
    result = parse("HLA-A02:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:87") to be Allele but got {result}'


def test_HLA_A02_89():
    result = parse("HLA-A02:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:89") to be Allele but got {result}'


def test_HLA_A02_90():
    result = parse("HLA-A02:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:90") to be Allele but got {result}'


def test_HLA_A02_91():
    result = parse("HLA-A02:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:91") to be Allele but got {result}'


def test_HLA_A02_92():
    result = parse("HLA-A02:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:92") to be Allele but got {result}'


def test_HLA_A02_93():
    result = parse("HLA-A02:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:93") to be Allele but got {result}'


def test_HLA_A02_95():
    result = parse("HLA-A02:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:95") to be Allele but got {result}'


def test_HLA_A02_96():
    result = parse("HLA-A02:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:96") to be Allele but got {result}'


def test_HLA_A02_97():
    result = parse("HLA-A02:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:97") to be Allele but got {result}'


def test_HLA_A02_99():
    result = parse("HLA-A02:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-A02:99") to be Allele but got {result}'


def test_HLA_A02_101():
    result = parse("HLA-A02:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:101") to be Allele but got {result}'
    )


def test_HLA_A02_102():
    result = parse("HLA-A02:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:102") to be Allele but got {result}'
    )


def test_HLA_A02_103():
    result = parse("HLA-A02:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:103") to be Allele but got {result}'
    )


def test_HLA_A02_104():
    result = parse("HLA-A02:104")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:104") to be Allele but got {result}'
    )


def test_HLA_A02_105():
    result = parse("HLA-A02:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:105") to be Allele but got {result}'
    )


def test_HLA_A02_106():
    result = parse("HLA-A02:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:106") to be Allele but got {result}'
    )


def test_HLA_A02_107():
    result = parse("HLA-A02:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:107") to be Allele but got {result}'
    )


def test_HLA_A02_108():
    result = parse("HLA-A02:108")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:108") to be Allele but got {result}'
    )


def test_HLA_A02_109():
    result = parse("HLA-A02:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:109") to be Allele but got {result}'
    )


def test_HLA_A02_110():
    result = parse("HLA-A02:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:110") to be Allele but got {result}'
    )


def test_HLA_A02_111():
    result = parse("HLA-A02:111")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:111") to be Allele but got {result}'
    )


def test_HLA_A02_112():
    result = parse("HLA-A02:112")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:112") to be Allele but got {result}'
    )


def test_HLA_A02_114():
    result = parse("HLA-A02:114")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:114") to be Allele but got {result}'
    )


def test_HLA_A02_115():
    result = parse("HLA-A02:115")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:115") to be Allele but got {result}'
    )


def test_HLA_A02_116():
    result = parse("HLA-A02:116")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:116") to be Allele but got {result}'
    )


def test_HLA_A02_117():
    result = parse("HLA-A02:117")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:117") to be Allele but got {result}'
    )


def test_HLA_A02_118():
    result = parse("HLA-A02:118")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:118") to be Allele but got {result}'
    )


def test_HLA_A02_119():
    result = parse("HLA-A02:119")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:119") to be Allele but got {result}'
    )


def test_HLA_A02_120():
    result = parse("HLA-A02:120")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:120") to be Allele but got {result}'
    )


def test_HLA_A02_121():
    result = parse("HLA-A02:121")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:121") to be Allele but got {result}'
    )


def test_HLA_A02_122():
    result = parse("HLA-A02:122")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:122") to be Allele but got {result}'
    )


def test_HLA_A02_123():
    result = parse("HLA-A02:123")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:123") to be Allele but got {result}'
    )


def test_HLA_A02_124():
    result = parse("HLA-A02:124")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:124") to be Allele but got {result}'
    )


def test_HLA_A02_126():
    result = parse("HLA-A02:126")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:126") to be Allele but got {result}'
    )


def test_HLA_A02_127():
    result = parse("HLA-A02:127")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:127") to be Allele but got {result}'
    )


def test_HLA_A02_128():
    result = parse("HLA-A02:128")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:128") to be Allele but got {result}'
    )


def test_HLA_A02_129():
    result = parse("HLA-A02:129")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:129") to be Allele but got {result}'
    )


def test_HLA_A02_130():
    result = parse("HLA-A02:130")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:130") to be Allele but got {result}'
    )


def test_HLA_A02_131():
    result = parse("HLA-A02:131")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:131") to be Allele but got {result}'
    )


def test_HLA_A02_132():
    result = parse("HLA-A02:132")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:132") to be Allele but got {result}'
    )


def test_HLA_A02_133():
    result = parse("HLA-A02:133")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:133") to be Allele but got {result}'
    )


def test_HLA_A02_134():
    result = parse("HLA-A02:134")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:134") to be Allele but got {result}'
    )


def test_HLA_A02_135():
    result = parse("HLA-A02:135")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:135") to be Allele but got {result}'
    )


def test_HLA_A02_136():
    result = parse("HLA-A02:136")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:136") to be Allele but got {result}'
    )


def test_HLA_A02_137():
    result = parse("HLA-A02:137")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:137") to be Allele but got {result}'
    )


def test_HLA_A02_138():
    result = parse("HLA-A02:138")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:138") to be Allele but got {result}'
    )


def test_HLA_A02_139():
    result = parse("HLA-A02:139")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:139") to be Allele but got {result}'
    )


def test_HLA_A02_140():
    result = parse("HLA-A02:140")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:140") to be Allele but got {result}'
    )


def test_HLA_A02_141():
    result = parse("HLA-A02:141")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:141") to be Allele but got {result}'
    )


def test_HLA_A02_142():
    result = parse("HLA-A02:142")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:142") to be Allele but got {result}'
    )


def test_HLA_A02_143():
    result = parse("HLA-A02:143")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:143") to be Allele but got {result}'
    )


def test_HLA_A02_144():
    result = parse("HLA-A02:144")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:144") to be Allele but got {result}'
    )


def test_HLA_A02_145():
    result = parse("HLA-A02:145")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:145") to be Allele but got {result}'
    )


def test_HLA_A02_146():
    result = parse("HLA-A02:146")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:146") to be Allele but got {result}'
    )


def test_HLA_A02_147():
    result = parse("HLA-A02:147")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:147") to be Allele but got {result}'
    )


def test_HLA_A02_148():
    result = parse("HLA-A02:148")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:148") to be Allele but got {result}'
    )


def test_HLA_A02_149():
    result = parse("HLA-A02:149")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:149") to be Allele but got {result}'
    )


def test_HLA_A02_150():
    result = parse("HLA-A02:150")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:150") to be Allele but got {result}'
    )


def test_HLA_A02_151():
    result = parse("HLA-A02:151")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:151") to be Allele but got {result}'
    )


def test_HLA_A02_152():
    result = parse("HLA-A02:152")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:152") to be Allele but got {result}'
    )


def test_HLA_A02_153():
    result = parse("HLA-A02:153")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:153") to be Allele but got {result}'
    )


def test_HLA_A02_154():
    result = parse("HLA-A02:154")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:154") to be Allele but got {result}'
    )


def test_HLA_A02_155():
    result = parse("HLA-A02:155")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:155") to be Allele but got {result}'
    )


def test_HLA_A02_156():
    result = parse("HLA-A02:156")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:156") to be Allele but got {result}'
    )


def test_HLA_A02_157():
    result = parse("HLA-A02:157")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:157") to be Allele but got {result}'
    )


def test_HLA_A02_158():
    result = parse("HLA-A02:158")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:158") to be Allele but got {result}'
    )


def test_HLA_A02_159():
    result = parse("HLA-A02:159")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:159") to be Allele but got {result}'
    )


def test_HLA_A02_160():
    result = parse("HLA-A02:160")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:160") to be Allele but got {result}'
    )


def test_HLA_A02_161():
    result = parse("HLA-A02:161")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:161") to be Allele but got {result}'
    )


def test_HLA_A02_162():
    result = parse("HLA-A02:162")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:162") to be Allele but got {result}'
    )


def test_HLA_A02_163():
    result = parse("HLA-A02:163")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:163") to be Allele but got {result}'
    )


def test_HLA_A02_164():
    result = parse("HLA-A02:164")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:164") to be Allele but got {result}'
    )


def test_HLA_A02_165():
    result = parse("HLA-A02:165")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:165") to be Allele but got {result}'
    )


def test_HLA_A02_166():
    result = parse("HLA-A02:166")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:166") to be Allele but got {result}'
    )


def test_HLA_A02_167():
    result = parse("HLA-A02:167")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:167") to be Allele but got {result}'
    )


def test_HLA_A02_168():
    result = parse("HLA-A02:168")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:168") to be Allele but got {result}'
    )


def test_HLA_A02_169():
    result = parse("HLA-A02:169")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:169") to be Allele but got {result}'
    )


def test_HLA_A02_170():
    result = parse("HLA-A02:170")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:170") to be Allele but got {result}'
    )


def test_HLA_A02_171():
    result = parse("HLA-A02:171")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:171") to be Allele but got {result}'
    )


def test_HLA_A02_172():
    result = parse("HLA-A02:172")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:172") to be Allele but got {result}'
    )


def test_HLA_A02_173():
    result = parse("HLA-A02:173")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:173") to be Allele but got {result}'
    )


def test_HLA_A02_174():
    result = parse("HLA-A02:174")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:174") to be Allele but got {result}'
    )


def test_HLA_A02_175():
    result = parse("HLA-A02:175")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:175") to be Allele but got {result}'
    )


def test_HLA_A02_176():
    result = parse("HLA-A02:176")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:176") to be Allele but got {result}'
    )


def test_HLA_A02_177():
    result = parse("HLA-A02:177")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:177") to be Allele but got {result}'
    )


def test_HLA_A02_178():
    result = parse("HLA-A02:178")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:178") to be Allele but got {result}'
    )


def test_HLA_A02_179():
    result = parse("HLA-A02:179")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:179") to be Allele but got {result}'
    )


def test_HLA_A02_180():
    result = parse("HLA-A02:180")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:180") to be Allele but got {result}'
    )


def test_HLA_A02_181():
    result = parse("HLA-A02:181")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:181") to be Allele but got {result}'
    )


def test_HLA_A02_182():
    result = parse("HLA-A02:182")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:182") to be Allele but got {result}'
    )


def test_HLA_A02_183():
    result = parse("HLA-A02:183")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:183") to be Allele but got {result}'
    )


def test_HLA_A02_184():
    result = parse("HLA-A02:184")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:184") to be Allele but got {result}'
    )


def test_HLA_A02_185():
    result = parse("HLA-A02:185")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:185") to be Allele but got {result}'
    )


def test_HLA_A02_186():
    result = parse("HLA-A02:186")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:186") to be Allele but got {result}'
    )


def test_HLA_A02_187():
    result = parse("HLA-A02:187")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:187") to be Allele but got {result}'
    )


def test_HLA_A02_188():
    result = parse("HLA-A02:188")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:188") to be Allele but got {result}'
    )


def test_HLA_A02_189():
    result = parse("HLA-A02:189")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:189") to be Allele but got {result}'
    )


def test_HLA_A02_190():
    result = parse("HLA-A02:190")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:190") to be Allele but got {result}'
    )


def test_HLA_A02_191():
    result = parse("HLA-A02:191")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:191") to be Allele but got {result}'
    )


def test_HLA_A02_192():
    result = parse("HLA-A02:192")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:192") to be Allele but got {result}'
    )


def test_HLA_A02_193():
    result = parse("HLA-A02:193")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:193") to be Allele but got {result}'
    )


def test_HLA_A02_194():
    result = parse("HLA-A02:194")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:194") to be Allele but got {result}'
    )


def test_HLA_A02_195():
    result = parse("HLA-A02:195")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:195") to be Allele but got {result}'
    )


def test_HLA_A02_196():
    result = parse("HLA-A02:196")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:196") to be Allele but got {result}'
    )


def test_HLA_A02_197():
    result = parse("HLA-A02:197")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:197") to be Allele but got {result}'
    )


def test_HLA_A02_198():
    result = parse("HLA-A02:198")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:198") to be Allele but got {result}'
    )


def test_HLA_A02_199():
    result = parse("HLA-A02:199")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:199") to be Allele but got {result}'
    )


def test_HLA_A02_200():
    result = parse("HLA-A02:200")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:200") to be Allele but got {result}'
    )


def test_HLA_A02_201():
    result = parse("HLA-A02:201")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:201") to be Allele but got {result}'
    )


def test_HLA_A02_202():
    result = parse("HLA-A02:202")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:202") to be Allele but got {result}'
    )


def test_HLA_A02_203():
    result = parse("HLA-A02:203")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:203") to be Allele but got {result}'
    )


def test_HLA_A02_204():
    result = parse("HLA-A02:204")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:204") to be Allele but got {result}'
    )


def test_HLA_A02_205():
    result = parse("HLA-A02:205")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:205") to be Allele but got {result}'
    )


def test_HLA_A02_206():
    result = parse("HLA-A02:206")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:206") to be Allele but got {result}'
    )


def test_HLA_A02_207():
    result = parse("HLA-A02:207")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:207") to be Allele but got {result}'
    )


def test_HLA_A02_208():
    result = parse("HLA-A02:208")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:208") to be Allele but got {result}'
    )


def test_HLA_A02_209():
    result = parse("HLA-A02:209")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:209") to be Allele but got {result}'
    )


def test_HLA_A02_210():
    result = parse("HLA-A02:210")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:210") to be Allele but got {result}'
    )


def test_HLA_A02_211():
    result = parse("HLA-A02:211")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:211") to be Allele but got {result}'
    )


def test_HLA_A02_212():
    result = parse("HLA-A02:212")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:212") to be Allele but got {result}'
    )


def test_HLA_A02_213():
    result = parse("HLA-A02:213")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:213") to be Allele but got {result}'
    )


def test_HLA_A02_214():
    result = parse("HLA-A02:214")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:214") to be Allele but got {result}'
    )


def test_HLA_A02_215():
    result = parse("HLA-A02:215")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:215") to be Allele but got {result}'
    )


def test_HLA_A02_216():
    result = parse("HLA-A02:216")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:216") to be Allele but got {result}'
    )


def test_HLA_A02_217():
    result = parse("HLA-A02:217")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:217") to be Allele but got {result}'
    )


def test_HLA_A02_218():
    result = parse("HLA-A02:218")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:218") to be Allele but got {result}'
    )


def test_HLA_A02_219():
    result = parse("HLA-A02:219")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:219") to be Allele but got {result}'
    )


def test_HLA_A02_220():
    result = parse("HLA-A02:220")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:220") to be Allele but got {result}'
    )


def test_HLA_A02_221():
    result = parse("HLA-A02:221")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:221") to be Allele but got {result}'
    )


def test_HLA_A02_224():
    result = parse("HLA-A02:224")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:224") to be Allele but got {result}'
    )


def test_HLA_A02_228():
    result = parse("HLA-A02:228")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:228") to be Allele but got {result}'
    )


def test_HLA_A02_229():
    result = parse("HLA-A02:229")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:229") to be Allele but got {result}'
    )


def test_HLA_A02_230():
    result = parse("HLA-A02:230")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:230") to be Allele but got {result}'
    )


def test_HLA_A02_231():
    result = parse("HLA-A02:231")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:231") to be Allele but got {result}'
    )


def test_HLA_A02_232():
    result = parse("HLA-A02:232")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:232") to be Allele but got {result}'
    )


def test_HLA_A02_233():
    result = parse("HLA-A02:233")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:233") to be Allele but got {result}'
    )


def test_HLA_A02_234():
    result = parse("HLA-A02:234")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:234") to be Allele but got {result}'
    )


def test_HLA_A02_235():
    result = parse("HLA-A02:235")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:235") to be Allele but got {result}'
    )


def test_HLA_A02_236():
    result = parse("HLA-A02:236")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:236") to be Allele but got {result}'
    )


def test_HLA_A02_237():
    result = parse("HLA-A02:237")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:237") to be Allele but got {result}'
    )


def test_HLA_A02_238():
    result = parse("HLA-A02:238")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:238") to be Allele but got {result}'
    )


def test_HLA_A02_239():
    result = parse("HLA-A02:239")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:239") to be Allele but got {result}'
    )


def test_HLA_A02_240():
    result = parse("HLA-A02:240")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:240") to be Allele but got {result}'
    )


def test_HLA_A02_241():
    result = parse("HLA-A02:241")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:241") to be Allele but got {result}'
    )


def test_HLA_A02_242():
    result = parse("HLA-A02:242")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:242") to be Allele but got {result}'
    )


def test_HLA_A02_243():
    result = parse("HLA-A02:243")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:243") to be Allele but got {result}'
    )


def test_HLA_A02_244():
    result = parse("HLA-A02:244")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:244") to be Allele but got {result}'
    )


def test_HLA_A02_245():
    result = parse("HLA-A02:245")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:245") to be Allele but got {result}'
    )


def test_HLA_A02_246():
    result = parse("HLA-A02:246")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:246") to be Allele but got {result}'
    )


def test_HLA_A02_247():
    result = parse("HLA-A02:247")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:247") to be Allele but got {result}'
    )


def test_HLA_A02_248():
    result = parse("HLA-A02:248")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:248") to be Allele but got {result}'
    )


def test_HLA_A02_249():
    result = parse("HLA-A02:249")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:249") to be Allele but got {result}'
    )


def test_HLA_A02_251():
    result = parse("HLA-A02:251")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:251") to be Allele but got {result}'
    )


def test_HLA_A02_252():
    result = parse("HLA-A02:252")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:252") to be Allele but got {result}'
    )


def test_HLA_A02_253():
    result = parse("HLA-A02:253")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:253") to be Allele but got {result}'
    )


def test_HLA_A02_254():
    result = parse("HLA-A02:254")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:254") to be Allele but got {result}'
    )


def test_HLA_A02_255():
    result = parse("HLA-A02:255")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:255") to be Allele but got {result}'
    )


def test_HLA_A02_256():
    result = parse("HLA-A02:256")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:256") to be Allele but got {result}'
    )


def test_HLA_A02_257():
    result = parse("HLA-A02:257")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:257") to be Allele but got {result}'
    )


def test_HLA_A02_258():
    result = parse("HLA-A02:258")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:258") to be Allele but got {result}'
    )


def test_HLA_A02_259():
    result = parse("HLA-A02:259")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:259") to be Allele but got {result}'
    )


def test_HLA_A02_260():
    result = parse("HLA-A02:260")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:260") to be Allele but got {result}'
    )


def test_HLA_A02_261():
    result = parse("HLA-A02:261")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:261") to be Allele but got {result}'
    )


def test_HLA_A02_262():
    result = parse("HLA-A02:262")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:262") to be Allele but got {result}'
    )


def test_HLA_A02_263():
    result = parse("HLA-A02:263")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:263") to be Allele but got {result}'
    )


def test_HLA_A02_264():
    result = parse("HLA-A02:264")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:264") to be Allele but got {result}'
    )


def test_HLA_A02_265():
    result = parse("HLA-A02:265")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:265") to be Allele but got {result}'
    )


def test_HLA_A02_266():
    result = parse("HLA-A02:266")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A02:266") to be Allele but got {result}'
    )


def test_HLA_A03_01():
    result = parse("HLA-A03:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:01") to be Allele but got {result}'


def test_HLA_A03_02():
    result = parse("HLA-A03:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:02") to be Allele but got {result}'


def test_HLA_A03_04():
    result = parse("HLA-A03:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:04") to be Allele but got {result}'


def test_HLA_A03_05():
    result = parse("HLA-A03:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:05") to be Allele but got {result}'


def test_HLA_A03_06():
    result = parse("HLA-A03:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:06") to be Allele but got {result}'


def test_HLA_A03_07():
    result = parse("HLA-A03:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:07") to be Allele but got {result}'


def test_HLA_A03_08():
    result = parse("HLA-A03:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:08") to be Allele but got {result}'


def test_HLA_A03_09():
    result = parse("HLA-A03:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:09") to be Allele but got {result}'


def test_HLA_A03_10():
    result = parse("HLA-A03:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:10") to be Allele but got {result}'


def test_HLA_A03_12():
    result = parse("HLA-A03:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:12") to be Allele but got {result}'


def test_HLA_A03_13():
    result = parse("HLA-A03:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:13") to be Allele but got {result}'


def test_HLA_A03_14():
    result = parse("HLA-A03:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:14") to be Allele but got {result}'


def test_HLA_A03_15():
    result = parse("HLA-A03:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:15") to be Allele but got {result}'


def test_HLA_A03_16():
    result = parse("HLA-A03:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:16") to be Allele but got {result}'


def test_HLA_A03_17():
    result = parse("HLA-A03:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:17") to be Allele but got {result}'


def test_HLA_A03_18():
    result = parse("HLA-A03:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:18") to be Allele but got {result}'


def test_HLA_A03_19():
    result = parse("HLA-A03:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:19") to be Allele but got {result}'


def test_HLA_A03_20():
    result = parse("HLA-A03:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:20") to be Allele but got {result}'


def test_HLA_A03_22():
    result = parse("HLA-A03:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:22") to be Allele but got {result}'


def test_HLA_A03_23():
    result = parse("HLA-A03:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:23") to be Allele but got {result}'


def test_HLA_A03_24():
    result = parse("HLA-A03:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:24") to be Allele but got {result}'


def test_HLA_A03_25():
    result = parse("HLA-A03:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:25") to be Allele but got {result}'


def test_HLA_A03_26():
    result = parse("HLA-A03:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:26") to be Allele but got {result}'


def test_HLA_A03_27():
    result = parse("HLA-A03:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:27") to be Allele but got {result}'


def test_HLA_A03_28():
    result = parse("HLA-A03:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:28") to be Allele but got {result}'


def test_HLA_A03_29():
    result = parse("HLA-A03:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:29") to be Allele but got {result}'


def test_HLA_A03_30():
    result = parse("HLA-A03:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:30") to be Allele but got {result}'


def test_HLA_A03_31():
    result = parse("HLA-A03:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:31") to be Allele but got {result}'


def test_HLA_A03_32():
    result = parse("HLA-A03:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:32") to be Allele but got {result}'


def test_HLA_A03_33():
    result = parse("HLA-A03:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:33") to be Allele but got {result}'


def test_HLA_A03_34():
    result = parse("HLA-A03:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:34") to be Allele but got {result}'


def test_HLA_A03_35():
    result = parse("HLA-A03:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:35") to be Allele but got {result}'


def test_HLA_A03_37():
    result = parse("HLA-A03:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:37") to be Allele but got {result}'


def test_HLA_A03_38():
    result = parse("HLA-A03:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:38") to be Allele but got {result}'


def test_HLA_A03_39():
    result = parse("HLA-A03:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:39") to be Allele but got {result}'


def test_HLA_A03_40():
    result = parse("HLA-A03:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:40") to be Allele but got {result}'


def test_HLA_A03_41():
    result = parse("HLA-A03:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:41") to be Allele but got {result}'


def test_HLA_A03_42():
    result = parse("HLA-A03:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:42") to be Allele but got {result}'


def test_HLA_A03_43():
    result = parse("HLA-A03:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:43") to be Allele but got {result}'


def test_HLA_A03_44():
    result = parse("HLA-A03:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:44") to be Allele but got {result}'


def test_HLA_A03_45():
    result = parse("HLA-A03:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:45") to be Allele but got {result}'


def test_HLA_A03_46():
    result = parse("HLA-A03:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:46") to be Allele but got {result}'


def test_HLA_A03_47():
    result = parse("HLA-A03:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:47") to be Allele but got {result}'


def test_HLA_A03_48():
    result = parse("HLA-A03:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:48") to be Allele but got {result}'


def test_HLA_A03_49():
    result = parse("HLA-A03:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:49") to be Allele but got {result}'


def test_HLA_A03_50():
    result = parse("HLA-A03:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:50") to be Allele but got {result}'


def test_HLA_A03_51():
    result = parse("HLA-A03:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:51") to be Allele but got {result}'


def test_HLA_A03_52():
    result = parse("HLA-A03:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:52") to be Allele but got {result}'


def test_HLA_A03_53():
    result = parse("HLA-A03:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:53") to be Allele but got {result}'


def test_HLA_A03_54():
    result = parse("HLA-A03:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:54") to be Allele but got {result}'


def test_HLA_A03_55():
    result = parse("HLA-A03:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:55") to be Allele but got {result}'


def test_HLA_A03_56():
    result = parse("HLA-A03:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:56") to be Allele but got {result}'


def test_HLA_A03_57():
    result = parse("HLA-A03:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:57") to be Allele but got {result}'


def test_HLA_A03_58():
    result = parse("HLA-A03:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:58") to be Allele but got {result}'


def test_HLA_A03_59():
    result = parse("HLA-A03:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:59") to be Allele but got {result}'


def test_HLA_A03_60():
    result = parse("HLA-A03:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:60") to be Allele but got {result}'


def test_HLA_A03_61():
    result = parse("HLA-A03:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:61") to be Allele but got {result}'


def test_HLA_A03_62():
    result = parse("HLA-A03:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:62") to be Allele but got {result}'


def test_HLA_A03_63():
    result = parse("HLA-A03:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:63") to be Allele but got {result}'


def test_HLA_A03_64():
    result = parse("HLA-A03:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:64") to be Allele but got {result}'


def test_HLA_A03_65():
    result = parse("HLA-A03:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:65") to be Allele but got {result}'


def test_HLA_A03_66():
    result = parse("HLA-A03:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:66") to be Allele but got {result}'


def test_HLA_A03_67():
    result = parse("HLA-A03:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:67") to be Allele but got {result}'


def test_HLA_A03_70():
    result = parse("HLA-A03:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:70") to be Allele but got {result}'


def test_HLA_A03_71():
    result = parse("HLA-A03:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:71") to be Allele but got {result}'


def test_HLA_A03_72():
    result = parse("HLA-A03:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:72") to be Allele but got {result}'


def test_HLA_A03_73():
    result = parse("HLA-A03:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:73") to be Allele but got {result}'


def test_HLA_A03_74():
    result = parse("HLA-A03:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:74") to be Allele but got {result}'


def test_HLA_A03_75():
    result = parse("HLA-A03:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:75") to be Allele but got {result}'


def test_HLA_A03_76():
    result = parse("HLA-A03:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:76") to be Allele but got {result}'


def test_HLA_A03_77():
    result = parse("HLA-A03:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:77") to be Allele but got {result}'


def test_HLA_A03_78():
    result = parse("HLA-A03:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:78") to be Allele but got {result}'


def test_HLA_A03_79():
    result = parse("HLA-A03:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:79") to be Allele but got {result}'


def test_HLA_A03_80():
    result = parse("HLA-A03:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:80") to be Allele but got {result}'


def test_HLA_A03_81():
    result = parse("HLA-A03:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:81") to be Allele but got {result}'


def test_HLA_A03_82():
    result = parse("HLA-A03:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-A03:82") to be Allele but got {result}'


def test_HLA_A11_01():
    result = parse("HLA-A11:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:01") to be Allele but got {result}'


def test_HLA_A11_02():
    result = parse("HLA-A11:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:02") to be Allele but got {result}'


def test_HLA_A11_03():
    result = parse("HLA-A11:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:03") to be Allele but got {result}'


def test_HLA_A11_04():
    result = parse("HLA-A11:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:04") to be Allele but got {result}'


def test_HLA_A11_05():
    result = parse("HLA-A11:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:05") to be Allele but got {result}'


def test_HLA_A11_06():
    result = parse("HLA-A11:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:06") to be Allele but got {result}'


def test_HLA_A11_07():
    result = parse("HLA-A11:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:07") to be Allele but got {result}'


def test_HLA_A11_08():
    result = parse("HLA-A11:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:08") to be Allele but got {result}'


def test_HLA_A11_09():
    result = parse("HLA-A11:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:09") to be Allele but got {result}'


def test_HLA_A11_10():
    result = parse("HLA-A11:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:10") to be Allele but got {result}'


def test_HLA_A11_11():
    result = parse("HLA-A11:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:11") to be Allele but got {result}'


def test_HLA_A11_12():
    result = parse("HLA-A11:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:12") to be Allele but got {result}'


def test_HLA_A11_13():
    result = parse("HLA-A11:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:13") to be Allele but got {result}'


def test_HLA_A11_14():
    result = parse("HLA-A11:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:14") to be Allele but got {result}'


def test_HLA_A11_15():
    result = parse("HLA-A11:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:15") to be Allele but got {result}'


def test_HLA_A11_16():
    result = parse("HLA-A11:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:16") to be Allele but got {result}'


def test_HLA_A11_17():
    result = parse("HLA-A11:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:17") to be Allele but got {result}'


def test_HLA_A11_18():
    result = parse("HLA-A11:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:18") to be Allele but got {result}'


def test_HLA_A11_19():
    result = parse("HLA-A11:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:19") to be Allele but got {result}'


def test_HLA_A11_20():
    result = parse("HLA-A11:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:20") to be Allele but got {result}'


def test_HLA_A11_22():
    result = parse("HLA-A11:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:22") to be Allele but got {result}'


def test_HLA_A11_23():
    result = parse("HLA-A11:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:23") to be Allele but got {result}'


def test_HLA_A11_24():
    result = parse("HLA-A11:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:24") to be Allele but got {result}'


def test_HLA_A11_25():
    result = parse("HLA-A11:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:25") to be Allele but got {result}'


def test_HLA_A11_26():
    result = parse("HLA-A11:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:26") to be Allele but got {result}'


def test_HLA_A11_27():
    result = parse("HLA-A11:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:27") to be Allele but got {result}'


def test_HLA_A11_29():
    result = parse("HLA-A11:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:29") to be Allele but got {result}'


def test_HLA_A11_30():
    result = parse("HLA-A11:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:30") to be Allele but got {result}'


def test_HLA_A11_31():
    result = parse("HLA-A11:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:31") to be Allele but got {result}'


def test_HLA_A11_32():
    result = parse("HLA-A11:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:32") to be Allele but got {result}'


def test_HLA_A11_33():
    result = parse("HLA-A11:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:33") to be Allele but got {result}'


def test_HLA_A11_34():
    result = parse("HLA-A11:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:34") to be Allele but got {result}'


def test_HLA_A11_35():
    result = parse("HLA-A11:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:35") to be Allele but got {result}'


def test_HLA_A11_36():
    result = parse("HLA-A11:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:36") to be Allele but got {result}'


def test_HLA_A11_37():
    result = parse("HLA-A11:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:37") to be Allele but got {result}'


def test_HLA_A11_38():
    result = parse("HLA-A11:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:38") to be Allele but got {result}'


def test_HLA_A11_39():
    result = parse("HLA-A11:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:39") to be Allele but got {result}'


def test_HLA_A11_40():
    result = parse("HLA-A11:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:40") to be Allele but got {result}'


def test_HLA_A11_41():
    result = parse("HLA-A11:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:41") to be Allele but got {result}'


def test_HLA_A11_42():
    result = parse("HLA-A11:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:42") to be Allele but got {result}'


def test_HLA_A11_43():
    result = parse("HLA-A11:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:43") to be Allele but got {result}'


def test_HLA_A11_44():
    result = parse("HLA-A11:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:44") to be Allele but got {result}'


def test_HLA_A11_45():
    result = parse("HLA-A11:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:45") to be Allele but got {result}'


def test_HLA_A11_46():
    result = parse("HLA-A11:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:46") to be Allele but got {result}'


def test_HLA_A11_47():
    result = parse("HLA-A11:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:47") to be Allele but got {result}'


def test_HLA_A11_48():
    result = parse("HLA-A11:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:48") to be Allele but got {result}'


def test_HLA_A11_49():
    result = parse("HLA-A11:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:49") to be Allele but got {result}'


def test_HLA_A11_51():
    result = parse("HLA-A11:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:51") to be Allele but got {result}'


def test_HLA_A11_53():
    result = parse("HLA-A11:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:53") to be Allele but got {result}'


def test_HLA_A11_54():
    result = parse("HLA-A11:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:54") to be Allele but got {result}'


def test_HLA_A11_55():
    result = parse("HLA-A11:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:55") to be Allele but got {result}'


def test_HLA_A11_56():
    result = parse("HLA-A11:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:56") to be Allele but got {result}'


def test_HLA_A11_57():
    result = parse("HLA-A11:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:57") to be Allele but got {result}'


def test_HLA_A11_58():
    result = parse("HLA-A11:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:58") to be Allele but got {result}'


def test_HLA_A11_59():
    result = parse("HLA-A11:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:59") to be Allele but got {result}'


def test_HLA_A11_60():
    result = parse("HLA-A11:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:60") to be Allele but got {result}'


def test_HLA_A11_61():
    result = parse("HLA-A11:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:61") to be Allele but got {result}'


def test_HLA_A11_62():
    result = parse("HLA-A11:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:62") to be Allele but got {result}'


def test_HLA_A11_63():
    result = parse("HLA-A11:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:63") to be Allele but got {result}'


def test_HLA_A11_64():
    result = parse("HLA-A11:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-A11:64") to be Allele but got {result}'


def test_HLA_A23_01():
    result = parse("HLA-A23:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:01") to be Allele but got {result}'


def test_HLA_A23_02():
    result = parse("HLA-A23:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:02") to be Allele but got {result}'


def test_HLA_A23_03():
    result = parse("HLA-A23:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:03") to be Allele but got {result}'


def test_HLA_A23_04():
    result = parse("HLA-A23:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:04") to be Allele but got {result}'


def test_HLA_A23_05():
    result = parse("HLA-A23:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:05") to be Allele but got {result}'


def test_HLA_A23_06():
    result = parse("HLA-A23:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:06") to be Allele but got {result}'


def test_HLA_A23_09():
    result = parse("HLA-A23:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:09") to be Allele but got {result}'


def test_HLA_A23_10():
    result = parse("HLA-A23:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:10") to be Allele but got {result}'


def test_HLA_A23_12():
    result = parse("HLA-A23:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:12") to be Allele but got {result}'


def test_HLA_A23_13():
    result = parse("HLA-A23:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:13") to be Allele but got {result}'


def test_HLA_A23_14():
    result = parse("HLA-A23:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:14") to be Allele but got {result}'


def test_HLA_A23_15():
    result = parse("HLA-A23:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:15") to be Allele but got {result}'


def test_HLA_A23_16():
    result = parse("HLA-A23:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:16") to be Allele but got {result}'


def test_HLA_A23_17():
    result = parse("HLA-A23:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:17") to be Allele but got {result}'


def test_HLA_A23_18():
    result = parse("HLA-A23:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:18") to be Allele but got {result}'


def test_HLA_A23_20():
    result = parse("HLA-A23:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:20") to be Allele but got {result}'


def test_HLA_A23_21():
    result = parse("HLA-A23:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:21") to be Allele but got {result}'


def test_HLA_A23_22():
    result = parse("HLA-A23:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:22") to be Allele but got {result}'


def test_HLA_A23_23():
    result = parse("HLA-A23:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:23") to be Allele but got {result}'


def test_HLA_A23_24():
    result = parse("HLA-A23:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:24") to be Allele but got {result}'


def test_HLA_A23_25():
    result = parse("HLA-A23:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:25") to be Allele but got {result}'


def test_HLA_A23_26():
    result = parse("HLA-A23:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A23:26") to be Allele but got {result}'


def test_HLA_A24_02():
    result = parse("HLA-A24:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:02") to be Allele but got {result}'


def test_HLA_A24_03():
    result = parse("HLA-A24:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:03") to be Allele but got {result}'


def test_HLA_A24_04():
    result = parse("HLA-A24:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:04") to be Allele but got {result}'


def test_HLA_A24_05():
    result = parse("HLA-A24:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:05") to be Allele but got {result}'


def test_HLA_A24_06():
    result = parse("HLA-A24:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:06") to be Allele but got {result}'


def test_HLA_A24_07():
    result = parse("HLA-A24:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:07") to be Allele but got {result}'


def test_HLA_A24_08():
    result = parse("HLA-A24:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:08") to be Allele but got {result}'


def test_HLA_A24_10():
    result = parse("HLA-A24:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:10") to be Allele but got {result}'


def test_HLA_A24_13():
    result = parse("HLA-A24:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:13") to be Allele but got {result}'


def test_HLA_A24_14():
    result = parse("HLA-A24:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:14") to be Allele but got {result}'


def test_HLA_A24_15():
    result = parse("HLA-A24:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:15") to be Allele but got {result}'


def test_HLA_A24_17():
    result = parse("HLA-A24:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:17") to be Allele but got {result}'


def test_HLA_A24_18():
    result = parse("HLA-A24:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:18") to be Allele but got {result}'


def test_HLA_A24_19():
    result = parse("HLA-A24:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:19") to be Allele but got {result}'


def test_HLA_A24_20():
    result = parse("HLA-A24:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:20") to be Allele but got {result}'


def test_HLA_A24_21():
    result = parse("HLA-A24:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:21") to be Allele but got {result}'


def test_HLA_A24_22():
    result = parse("HLA-A24:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:22") to be Allele but got {result}'


def test_HLA_A24_23():
    result = parse("HLA-A24:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:23") to be Allele but got {result}'


def test_HLA_A24_24():
    result = parse("HLA-A24:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:24") to be Allele but got {result}'


def test_HLA_A24_25():
    result = parse("HLA-A24:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:25") to be Allele but got {result}'


def test_HLA_A24_26():
    result = parse("HLA-A24:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:26") to be Allele but got {result}'


def test_HLA_A24_27():
    result = parse("HLA-A24:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:27") to be Allele but got {result}'


def test_HLA_A24_28():
    result = parse("HLA-A24:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:28") to be Allele but got {result}'


def test_HLA_A24_29():
    result = parse("HLA-A24:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:29") to be Allele but got {result}'


def test_HLA_A24_30():
    result = parse("HLA-A24:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:30") to be Allele but got {result}'


def test_HLA_A24_31():
    result = parse("HLA-A24:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:31") to be Allele but got {result}'


def test_HLA_A24_32():
    result = parse("HLA-A24:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:32") to be Allele but got {result}'


def test_HLA_A24_33():
    result = parse("HLA-A24:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:33") to be Allele but got {result}'


def test_HLA_A24_34():
    result = parse("HLA-A24:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:34") to be Allele but got {result}'


def test_HLA_A24_35():
    result = parse("HLA-A24:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:35") to be Allele but got {result}'


def test_HLA_A24_37():
    result = parse("HLA-A24:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:37") to be Allele but got {result}'


def test_HLA_A24_38():
    result = parse("HLA-A24:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:38") to be Allele but got {result}'


def test_HLA_A24_39():
    result = parse("HLA-A24:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:39") to be Allele but got {result}'


def test_HLA_A24_41():
    result = parse("HLA-A24:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:41") to be Allele but got {result}'


def test_HLA_A24_42():
    result = parse("HLA-A24:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:42") to be Allele but got {result}'


def test_HLA_A24_43():
    result = parse("HLA-A24:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:43") to be Allele but got {result}'


def test_HLA_A24_44():
    result = parse("HLA-A24:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:44") to be Allele but got {result}'


def test_HLA_A24_46():
    result = parse("HLA-A24:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:46") to be Allele but got {result}'


def test_HLA_A24_47():
    result = parse("HLA-A24:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:47") to be Allele but got {result}'


def test_HLA_A24_49():
    result = parse("HLA-A24:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:49") to be Allele but got {result}'


def test_HLA_A24_50():
    result = parse("HLA-A24:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:50") to be Allele but got {result}'


def test_HLA_A24_51():
    result = parse("HLA-A24:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:51") to be Allele but got {result}'


def test_HLA_A24_52():
    result = parse("HLA-A24:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:52") to be Allele but got {result}'


def test_HLA_A24_53():
    result = parse("HLA-A24:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:53") to be Allele but got {result}'


def test_HLA_A24_54():
    result = parse("HLA-A24:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:54") to be Allele but got {result}'


def test_HLA_A24_55():
    result = parse("HLA-A24:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:55") to be Allele but got {result}'


def test_HLA_A24_56():
    result = parse("HLA-A24:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:56") to be Allele but got {result}'


def test_HLA_A24_57():
    result = parse("HLA-A24:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:57") to be Allele but got {result}'


def test_HLA_A24_58():
    result = parse("HLA-A24:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:58") to be Allele but got {result}'


def test_HLA_A24_59():
    result = parse("HLA-A24:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:59") to be Allele but got {result}'


def test_HLA_A24_61():
    result = parse("HLA-A24:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:61") to be Allele but got {result}'


def test_HLA_A24_62():
    result = parse("HLA-A24:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:62") to be Allele but got {result}'


def test_HLA_A24_63():
    result = parse("HLA-A24:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:63") to be Allele but got {result}'


def test_HLA_A24_64():
    result = parse("HLA-A24:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:64") to be Allele but got {result}'


def test_HLA_A24_66():
    result = parse("HLA-A24:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:66") to be Allele but got {result}'


def test_HLA_A24_67():
    result = parse("HLA-A24:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:67") to be Allele but got {result}'


def test_HLA_A24_68():
    result = parse("HLA-A24:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:68") to be Allele but got {result}'


def test_HLA_A24_69():
    result = parse("HLA-A24:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:69") to be Allele but got {result}'


def test_HLA_A24_70():
    result = parse("HLA-A24:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:70") to be Allele but got {result}'


def test_HLA_A24_71():
    result = parse("HLA-A24:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:71") to be Allele but got {result}'


def test_HLA_A24_72():
    result = parse("HLA-A24:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:72") to be Allele but got {result}'


def test_HLA_A24_73():
    result = parse("HLA-A24:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:73") to be Allele but got {result}'


def test_HLA_A24_74():
    result = parse("HLA-A24:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:74") to be Allele but got {result}'


def test_HLA_A24_75():
    result = parse("HLA-A24:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:75") to be Allele but got {result}'


def test_HLA_A24_76():
    result = parse("HLA-A24:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:76") to be Allele but got {result}'


def test_HLA_A24_77():
    result = parse("HLA-A24:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:77") to be Allele but got {result}'


def test_HLA_A24_78():
    result = parse("HLA-A24:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:78") to be Allele but got {result}'


def test_HLA_A24_79():
    result = parse("HLA-A24:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:79") to be Allele but got {result}'


def test_HLA_A24_80():
    result = parse("HLA-A24:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:80") to be Allele but got {result}'


def test_HLA_A24_81():
    result = parse("HLA-A24:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:81") to be Allele but got {result}'


def test_HLA_A24_82():
    result = parse("HLA-A24:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:82") to be Allele but got {result}'


def test_HLA_A24_85():
    result = parse("HLA-A24:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:85") to be Allele but got {result}'


def test_HLA_A24_87():
    result = parse("HLA-A24:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:87") to be Allele but got {result}'


def test_HLA_A24_88():
    result = parse("HLA-A24:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:88") to be Allele but got {result}'


def test_HLA_A24_89():
    result = parse("HLA-A24:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:89") to be Allele but got {result}'


def test_HLA_A24_91():
    result = parse("HLA-A24:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:91") to be Allele but got {result}'


def test_HLA_A24_92():
    result = parse("HLA-A24:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:92") to be Allele but got {result}'


def test_HLA_A24_93():
    result = parse("HLA-A24:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:93") to be Allele but got {result}'


def test_HLA_A24_94():
    result = parse("HLA-A24:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:94") to be Allele but got {result}'


def test_HLA_A24_95():
    result = parse("HLA-A24:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:95") to be Allele but got {result}'


def test_HLA_A24_96():
    result = parse("HLA-A24:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:96") to be Allele but got {result}'


def test_HLA_A24_97():
    result = parse("HLA-A24:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:97") to be Allele but got {result}'


def test_HLA_A24_98():
    result = parse("HLA-A24:98")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:98") to be Allele but got {result}'


def test_HLA_A24_99():
    result = parse("HLA-A24:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-A24:99") to be Allele but got {result}'


def test_HLA_A24_100():
    result = parse("HLA-A24:100")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:100") to be Allele but got {result}'
    )


def test_HLA_A24_101():
    result = parse("HLA-A24:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:101") to be Allele but got {result}'
    )


def test_HLA_A24_102():
    result = parse("HLA-A24:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:102") to be Allele but got {result}'
    )


def test_HLA_A24_103():
    result = parse("HLA-A24:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:103") to be Allele but got {result}'
    )


def test_HLA_A24_104():
    result = parse("HLA-A24:104")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:104") to be Allele but got {result}'
    )


def test_HLA_A24_105():
    result = parse("HLA-A24:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:105") to be Allele but got {result}'
    )


def test_HLA_A24_106():
    result = parse("HLA-A24:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:106") to be Allele but got {result}'
    )


def test_HLA_A24_107():
    result = parse("HLA-A24:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:107") to be Allele but got {result}'
    )


def test_HLA_A24_108():
    result = parse("HLA-A24:108")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:108") to be Allele but got {result}'
    )


def test_HLA_A24_109():
    result = parse("HLA-A24:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:109") to be Allele but got {result}'
    )


def test_HLA_A24_110():
    result = parse("HLA-A24:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:110") to be Allele but got {result}'
    )


def test_HLA_A24_111():
    result = parse("HLA-A24:111")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:111") to be Allele but got {result}'
    )


def test_HLA_A24_112():
    result = parse("HLA-A24:112")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:112") to be Allele but got {result}'
    )


def test_HLA_A24_113():
    result = parse("HLA-A24:113")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:113") to be Allele but got {result}'
    )


def test_HLA_A24_114():
    result = parse("HLA-A24:114")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:114") to be Allele but got {result}'
    )


def test_HLA_A24_115():
    result = parse("HLA-A24:115")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:115") to be Allele but got {result}'
    )


def test_HLA_A24_116():
    result = parse("HLA-A24:116")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:116") to be Allele but got {result}'
    )


def test_HLA_A24_117():
    result = parse("HLA-A24:117")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:117") to be Allele but got {result}'
    )


def test_HLA_A24_118():
    result = parse("HLA-A24:118")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:118") to be Allele but got {result}'
    )


def test_HLA_A24_119():
    result = parse("HLA-A24:119")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:119") to be Allele but got {result}'
    )


def test_HLA_A24_120():
    result = parse("HLA-A24:120")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:120") to be Allele but got {result}'
    )


def test_HLA_A24_121():
    result = parse("HLA-A24:121")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:121") to be Allele but got {result}'
    )


def test_HLA_A24_122():
    result = parse("HLA-A24:122")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:122") to be Allele but got {result}'
    )


def test_HLA_A24_123():
    result = parse("HLA-A24:123")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:123") to be Allele but got {result}'
    )


def test_HLA_A24_124():
    result = parse("HLA-A24:124")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:124") to be Allele but got {result}'
    )


def test_HLA_A24_125():
    result = parse("HLA-A24:125")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:125") to be Allele but got {result}'
    )


def test_HLA_A24_126():
    result = parse("HLA-A24:126")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:126") to be Allele but got {result}'
    )


def test_HLA_A24_127():
    result = parse("HLA-A24:127")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:127") to be Allele but got {result}'
    )


def test_HLA_A24_128():
    result = parse("HLA-A24:128")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:128") to be Allele but got {result}'
    )


def test_HLA_A24_129():
    result = parse("HLA-A24:129")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:129") to be Allele but got {result}'
    )


def test_HLA_A24_130():
    result = parse("HLA-A24:130")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:130") to be Allele but got {result}'
    )


def test_HLA_A24_131():
    result = parse("HLA-A24:131")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:131") to be Allele but got {result}'
    )


def test_HLA_A24_133():
    result = parse("HLA-A24:133")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:133") to be Allele but got {result}'
    )


def test_HLA_A24_134():
    result = parse("HLA-A24:134")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:134") to be Allele but got {result}'
    )


def test_HLA_A24_135():
    result = parse("HLA-A24:135")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:135") to be Allele but got {result}'
    )


def test_HLA_A24_136():
    result = parse("HLA-A24:136")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:136") to be Allele but got {result}'
    )


def test_HLA_A24_137():
    result = parse("HLA-A24:137")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:137") to be Allele but got {result}'
    )


def test_HLA_A24_138():
    result = parse("HLA-A24:138")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:138") to be Allele but got {result}'
    )


def test_HLA_A24_139():
    result = parse("HLA-A24:139")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:139") to be Allele but got {result}'
    )


def test_HLA_A24_140():
    result = parse("HLA-A24:140")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:140") to be Allele but got {result}'
    )


def test_HLA_A24_141():
    result = parse("HLA-A24:141")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:141") to be Allele but got {result}'
    )


def test_HLA_A24_142():
    result = parse("HLA-A24:142")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:142") to be Allele but got {result}'
    )


def test_HLA_A24_143():
    result = parse("HLA-A24:143")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:143") to be Allele but got {result}'
    )


def test_HLA_A24_144():
    result = parse("HLA-A24:144")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-A24:144") to be Allele but got {result}'
    )


def test_HLA_A25_01():
    result = parse("HLA-A25:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:01") to be Allele but got {result}'


def test_HLA_A25_02():
    result = parse("HLA-A25:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:02") to be Allele but got {result}'


def test_HLA_A25_03():
    result = parse("HLA-A25:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:03") to be Allele but got {result}'


def test_HLA_A25_04():
    result = parse("HLA-A25:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:04") to be Allele but got {result}'


def test_HLA_A25_05():
    result = parse("HLA-A25:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:05") to be Allele but got {result}'


def test_HLA_A25_06():
    result = parse("HLA-A25:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:06") to be Allele but got {result}'


def test_HLA_A25_07():
    result = parse("HLA-A25:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:07") to be Allele but got {result}'


def test_HLA_A25_08():
    result = parse("HLA-A25:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:08") to be Allele but got {result}'


def test_HLA_A25_09():
    result = parse("HLA-A25:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:09") to be Allele but got {result}'


def test_HLA_A25_10():
    result = parse("HLA-A25:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:10") to be Allele but got {result}'


def test_HLA_A25_11():
    result = parse("HLA-A25:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:11") to be Allele but got {result}'


def test_HLA_A25_13():
    result = parse("HLA-A25:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A25:13") to be Allele but got {result}'


def test_HLA_A26_01():
    result = parse("HLA-A26:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:01") to be Allele but got {result}'


def test_HLA_A26_02():
    result = parse("HLA-A26:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:02") to be Allele but got {result}'


def test_HLA_A26_03():
    result = parse("HLA-A26:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:03") to be Allele but got {result}'


def test_HLA_A26_04():
    result = parse("HLA-A26:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:04") to be Allele but got {result}'


def test_HLA_A26_05():
    result = parse("HLA-A26:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:05") to be Allele but got {result}'


def test_HLA_A26_06():
    result = parse("HLA-A26:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:06") to be Allele but got {result}'


def test_HLA_A26_07():
    result = parse("HLA-A26:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:07") to be Allele but got {result}'


def test_HLA_A26_08():
    result = parse("HLA-A26:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:08") to be Allele but got {result}'


def test_HLA_A26_09():
    result = parse("HLA-A26:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:09") to be Allele but got {result}'


def test_HLA_A26_10():
    result = parse("HLA-A26:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:10") to be Allele but got {result}'


def test_HLA_A26_12():
    result = parse("HLA-A26:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:12") to be Allele but got {result}'


def test_HLA_A26_13():
    result = parse("HLA-A26:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:13") to be Allele but got {result}'


def test_HLA_A26_14():
    result = parse("HLA-A26:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:14") to be Allele but got {result}'


def test_HLA_A26_15():
    result = parse("HLA-A26:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:15") to be Allele but got {result}'


def test_HLA_A26_16():
    result = parse("HLA-A26:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:16") to be Allele but got {result}'


def test_HLA_A26_17():
    result = parse("HLA-A26:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:17") to be Allele but got {result}'


def test_HLA_A26_18():
    result = parse("HLA-A26:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:18") to be Allele but got {result}'


def test_HLA_A26_19():
    result = parse("HLA-A26:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:19") to be Allele but got {result}'


def test_HLA_A26_20():
    result = parse("HLA-A26:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:20") to be Allele but got {result}'


def test_HLA_A26_21():
    result = parse("HLA-A26:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:21") to be Allele but got {result}'


def test_HLA_A26_22():
    result = parse("HLA-A26:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:22") to be Allele but got {result}'


def test_HLA_A26_23():
    result = parse("HLA-A26:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:23") to be Allele but got {result}'


def test_HLA_A26_24():
    result = parse("HLA-A26:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:24") to be Allele but got {result}'


def test_HLA_A26_26():
    result = parse("HLA-A26:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:26") to be Allele but got {result}'


def test_HLA_A26_27():
    result = parse("HLA-A26:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:27") to be Allele but got {result}'


def test_HLA_A26_28():
    result = parse("HLA-A26:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:28") to be Allele but got {result}'


def test_HLA_A26_29():
    result = parse("HLA-A26:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:29") to be Allele but got {result}'


def test_HLA_A26_30():
    result = parse("HLA-A26:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:30") to be Allele but got {result}'


def test_HLA_A26_31():
    result = parse("HLA-A26:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:31") to be Allele but got {result}'


def test_HLA_A26_32():
    result = parse("HLA-A26:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:32") to be Allele but got {result}'


def test_HLA_A26_33():
    result = parse("HLA-A26:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:33") to be Allele but got {result}'


def test_HLA_A26_34():
    result = parse("HLA-A26:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:34") to be Allele but got {result}'


def test_HLA_A26_35():
    result = parse("HLA-A26:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:35") to be Allele but got {result}'


def test_HLA_A26_36():
    result = parse("HLA-A26:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:36") to be Allele but got {result}'


def test_HLA_A26_37():
    result = parse("HLA-A26:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:37") to be Allele but got {result}'


def test_HLA_A26_38():
    result = parse("HLA-A26:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:38") to be Allele but got {result}'


def test_HLA_A26_39():
    result = parse("HLA-A26:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:39") to be Allele but got {result}'


def test_HLA_A26_40():
    result = parse("HLA-A26:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:40") to be Allele but got {result}'


def test_HLA_A26_41():
    result = parse("HLA-A26:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:41") to be Allele but got {result}'


def test_HLA_A26_42():
    result = parse("HLA-A26:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:42") to be Allele but got {result}'


def test_HLA_A26_43():
    result = parse("HLA-A26:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:43") to be Allele but got {result}'


def test_HLA_A26_45():
    result = parse("HLA-A26:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:45") to be Allele but got {result}'


def test_HLA_A26_46():
    result = parse("HLA-A26:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:46") to be Allele but got {result}'


def test_HLA_A26_47():
    result = parse("HLA-A26:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:47") to be Allele but got {result}'


def test_HLA_A26_48():
    result = parse("HLA-A26:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:48") to be Allele but got {result}'


def test_HLA_A26_49():
    result = parse("HLA-A26:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:49") to be Allele but got {result}'


def test_HLA_A26_50():
    result = parse("HLA-A26:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-A26:50") to be Allele but got {result}'


def test_HLA_A29_01():
    result = parse("HLA-A29:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:01") to be Allele but got {result}'


def test_HLA_A29_02():
    result = parse("HLA-A29:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:02") to be Allele but got {result}'


def test_HLA_A29_03():
    result = parse("HLA-A29:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:03") to be Allele but got {result}'


def test_HLA_A29_04():
    result = parse("HLA-A29:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:04") to be Allele but got {result}'


def test_HLA_A29_05():
    result = parse("HLA-A29:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:05") to be Allele but got {result}'


def test_HLA_A29_06():
    result = parse("HLA-A29:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:06") to be Allele but got {result}'


def test_HLA_A29_07():
    result = parse("HLA-A29:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:07") to be Allele but got {result}'


def test_HLA_A29_09():
    result = parse("HLA-A29:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:09") to be Allele but got {result}'


def test_HLA_A29_10():
    result = parse("HLA-A29:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:10") to be Allele but got {result}'


def test_HLA_A29_11():
    result = parse("HLA-A29:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:11") to be Allele but got {result}'


def test_HLA_A29_12():
    result = parse("HLA-A29:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:12") to be Allele but got {result}'


def test_HLA_A29_13():
    result = parse("HLA-A29:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:13") to be Allele but got {result}'


def test_HLA_A29_14():
    result = parse("HLA-A29:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:14") to be Allele but got {result}'


def test_HLA_A29_15():
    result = parse("HLA-A29:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:15") to be Allele but got {result}'


def test_HLA_A29_16():
    result = parse("HLA-A29:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:16") to be Allele but got {result}'


def test_HLA_A29_17():
    result = parse("HLA-A29:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:17") to be Allele but got {result}'


def test_HLA_A29_18():
    result = parse("HLA-A29:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:18") to be Allele but got {result}'


def test_HLA_A29_19():
    result = parse("HLA-A29:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:19") to be Allele but got {result}'


def test_HLA_A29_20():
    result = parse("HLA-A29:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:20") to be Allele but got {result}'


def test_HLA_A29_21():
    result = parse("HLA-A29:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:21") to be Allele but got {result}'


def test_HLA_A29_22():
    result = parse("HLA-A29:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A29:22") to be Allele but got {result}'


def test_HLA_A30_01():
    result = parse("HLA-A30:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:01") to be Allele but got {result}'


def test_HLA_A30_02():
    result = parse("HLA-A30:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:02") to be Allele but got {result}'


def test_HLA_A30_03():
    result = parse("HLA-A30:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:03") to be Allele but got {result}'


def test_HLA_A30_04():
    result = parse("HLA-A30:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:04") to be Allele but got {result}'


def test_HLA_A30_06():
    result = parse("HLA-A30:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:06") to be Allele but got {result}'


def test_HLA_A30_07():
    result = parse("HLA-A30:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:07") to be Allele but got {result}'


def test_HLA_A30_08():
    result = parse("HLA-A30:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:08") to be Allele but got {result}'


def test_HLA_A30_09():
    result = parse("HLA-A30:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:09") to be Allele but got {result}'


def test_HLA_A30_10():
    result = parse("HLA-A30:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:10") to be Allele but got {result}'


def test_HLA_A30_11():
    result = parse("HLA-A30:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:11") to be Allele but got {result}'


def test_HLA_A30_12():
    result = parse("HLA-A30:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:12") to be Allele but got {result}'


def test_HLA_A30_13():
    result = parse("HLA-A30:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:13") to be Allele but got {result}'


def test_HLA_A30_15():
    result = parse("HLA-A30:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:15") to be Allele but got {result}'


def test_HLA_A30_16():
    result = parse("HLA-A30:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:16") to be Allele but got {result}'


def test_HLA_A30_17():
    result = parse("HLA-A30:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:17") to be Allele but got {result}'


def test_HLA_A30_18():
    result = parse("HLA-A30:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:18") to be Allele but got {result}'


def test_HLA_A30_19():
    result = parse("HLA-A30:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:19") to be Allele but got {result}'


def test_HLA_A30_20():
    result = parse("HLA-A30:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:20") to be Allele but got {result}'


def test_HLA_A30_22():
    result = parse("HLA-A30:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:22") to be Allele but got {result}'


def test_HLA_A30_23():
    result = parse("HLA-A30:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:23") to be Allele but got {result}'


def test_HLA_A30_24():
    result = parse("HLA-A30:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:24") to be Allele but got {result}'


def test_HLA_A30_25():
    result = parse("HLA-A30:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:25") to be Allele but got {result}'


def test_HLA_A30_26():
    result = parse("HLA-A30:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:26") to be Allele but got {result}'


def test_HLA_A30_28():
    result = parse("HLA-A30:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:28") to be Allele but got {result}'


def test_HLA_A30_29():
    result = parse("HLA-A30:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:29") to be Allele but got {result}'


def test_HLA_A30_30():
    result = parse("HLA-A30:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:30") to be Allele but got {result}'


def test_HLA_A30_31():
    result = parse("HLA-A30:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:31") to be Allele but got {result}'


def test_HLA_A30_32():
    result = parse("HLA-A30:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:32") to be Allele but got {result}'


def test_HLA_A30_33():
    result = parse("HLA-A30:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:33") to be Allele but got {result}'


def test_HLA_A30_34():
    result = parse("HLA-A30:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:34") to be Allele but got {result}'


def test_HLA_A30_35():
    result = parse("HLA-A30:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:35") to be Allele but got {result}'


def test_HLA_A30_36():
    result = parse("HLA-A30:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:36") to be Allele but got {result}'


def test_HLA_A30_37():
    result = parse("HLA-A30:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:37") to be Allele but got {result}'


def test_HLA_A30_38():
    result = parse("HLA-A30:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:38") to be Allele but got {result}'


def test_HLA_A30_39():
    result = parse("HLA-A30:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:39") to be Allele but got {result}'


def test_HLA_A30_40():
    result = parse("HLA-A30:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:40") to be Allele but got {result}'


def test_HLA_A30_41():
    result = parse("HLA-A30:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A30:41") to be Allele but got {result}'


def test_HLA_A31_01():
    result = parse("HLA-A31:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:01") to be Allele but got {result}'


def test_HLA_A31_02():
    result = parse("HLA-A31:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:02") to be Allele but got {result}'


def test_HLA_A31_03():
    result = parse("HLA-A31:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:03") to be Allele but got {result}'


def test_HLA_A31_04():
    result = parse("HLA-A31:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:04") to be Allele but got {result}'


def test_HLA_A31_05():
    result = parse("HLA-A31:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:05") to be Allele but got {result}'


def test_HLA_A31_06():
    result = parse("HLA-A31:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:06") to be Allele but got {result}'


def test_HLA_A31_07():
    result = parse("HLA-A31:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:07") to be Allele but got {result}'


def test_HLA_A31_08():
    result = parse("HLA-A31:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:08") to be Allele but got {result}'


def test_HLA_A31_09():
    result = parse("HLA-A31:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:09") to be Allele but got {result}'


def test_HLA_A31_10():
    result = parse("HLA-A31:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:10") to be Allele but got {result}'


def test_HLA_A31_11():
    result = parse("HLA-A31:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:11") to be Allele but got {result}'


def test_HLA_A31_12():
    result = parse("HLA-A31:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:12") to be Allele but got {result}'


def test_HLA_A31_13():
    result = parse("HLA-A31:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:13") to be Allele but got {result}'


def test_HLA_A31_15():
    result = parse("HLA-A31:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:15") to be Allele but got {result}'


def test_HLA_A31_16():
    result = parse("HLA-A31:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:16") to be Allele but got {result}'


def test_HLA_A31_17():
    result = parse("HLA-A31:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:17") to be Allele but got {result}'


def test_HLA_A31_18():
    result = parse("HLA-A31:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:18") to be Allele but got {result}'


def test_HLA_A31_19():
    result = parse("HLA-A31:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:19") to be Allele but got {result}'


def test_HLA_A31_20():
    result = parse("HLA-A31:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:20") to be Allele but got {result}'


def test_HLA_A31_21():
    result = parse("HLA-A31:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:21") to be Allele but got {result}'


def test_HLA_A31_22():
    result = parse("HLA-A31:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:22") to be Allele but got {result}'


def test_HLA_A31_23():
    result = parse("HLA-A31:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:23") to be Allele but got {result}'


def test_HLA_A31_24():
    result = parse("HLA-A31:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:24") to be Allele but got {result}'


def test_HLA_A31_25():
    result = parse("HLA-A31:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:25") to be Allele but got {result}'


def test_HLA_A31_26():
    result = parse("HLA-A31:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:26") to be Allele but got {result}'


def test_HLA_A31_27():
    result = parse("HLA-A31:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:27") to be Allele but got {result}'


def test_HLA_A31_28():
    result = parse("HLA-A31:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:28") to be Allele but got {result}'


def test_HLA_A31_29():
    result = parse("HLA-A31:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:29") to be Allele but got {result}'


def test_HLA_A31_30():
    result = parse("HLA-A31:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:30") to be Allele but got {result}'


def test_HLA_A31_31():
    result = parse("HLA-A31:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:31") to be Allele but got {result}'


def test_HLA_A31_32():
    result = parse("HLA-A31:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:32") to be Allele but got {result}'


def test_HLA_A31_33():
    result = parse("HLA-A31:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:33") to be Allele but got {result}'


def test_HLA_A31_34():
    result = parse("HLA-A31:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:34") to be Allele but got {result}'


def test_HLA_A31_35():
    result = parse("HLA-A31:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:35") to be Allele but got {result}'


def test_HLA_A31_36():
    result = parse("HLA-A31:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:36") to be Allele but got {result}'


def test_HLA_A31_37():
    result = parse("HLA-A31:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A31:37") to be Allele but got {result}'


def test_HLA_A32_01():
    result = parse("HLA-A32:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:01") to be Allele but got {result}'


def test_HLA_A32_02():
    result = parse("HLA-A32:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:02") to be Allele but got {result}'


def test_HLA_A32_03():
    result = parse("HLA-A32:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:03") to be Allele but got {result}'


def test_HLA_A32_04():
    result = parse("HLA-A32:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:04") to be Allele but got {result}'


def test_HLA_A32_05():
    result = parse("HLA-A32:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:05") to be Allele but got {result}'


def test_HLA_A32_06():
    result = parse("HLA-A32:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:06") to be Allele but got {result}'


def test_HLA_A32_07():
    result = parse("HLA-A32:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:07") to be Allele but got {result}'


def test_HLA_A32_08():
    result = parse("HLA-A32:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:08") to be Allele but got {result}'


def test_HLA_A32_09():
    result = parse("HLA-A32:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:09") to be Allele but got {result}'


def test_HLA_A32_10():
    result = parse("HLA-A32:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:10") to be Allele but got {result}'


def test_HLA_A32_12():
    result = parse("HLA-A32:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:12") to be Allele but got {result}'


def test_HLA_A32_13():
    result = parse("HLA-A32:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:13") to be Allele but got {result}'


def test_HLA_A32_14():
    result = parse("HLA-A32:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:14") to be Allele but got {result}'


def test_HLA_A32_15():
    result = parse("HLA-A32:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:15") to be Allele but got {result}'


def test_HLA_A32_16():
    result = parse("HLA-A32:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:16") to be Allele but got {result}'


def test_HLA_A32_17():
    result = parse("HLA-A32:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:17") to be Allele but got {result}'


def test_HLA_A32_18():
    result = parse("HLA-A32:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:18") to be Allele but got {result}'


def test_HLA_A32_20():
    result = parse("HLA-A32:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:20") to be Allele but got {result}'


def test_HLA_A32_21():
    result = parse("HLA-A32:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:21") to be Allele but got {result}'


def test_HLA_A32_22():
    result = parse("HLA-A32:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:22") to be Allele but got {result}'


def test_HLA_A32_23():
    result = parse("HLA-A32:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:23") to be Allele but got {result}'


def test_HLA_A32_24():
    result = parse("HLA-A32:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:24") to be Allele but got {result}'


def test_HLA_A32_25():
    result = parse("HLA-A32:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A32:25") to be Allele but got {result}'


def test_HLA_A33_01():
    result = parse("HLA-A33:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:01") to be Allele but got {result}'


def test_HLA_A33_03():
    result = parse("HLA-A33:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:03") to be Allele but got {result}'


def test_HLA_A33_04():
    result = parse("HLA-A33:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:04") to be Allele but got {result}'


def test_HLA_A33_05():
    result = parse("HLA-A33:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:05") to be Allele but got {result}'


def test_HLA_A33_06():
    result = parse("HLA-A33:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:06") to be Allele but got {result}'


def test_HLA_A33_07():
    result = parse("HLA-A33:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:07") to be Allele but got {result}'


def test_HLA_A33_08():
    result = parse("HLA-A33:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:08") to be Allele but got {result}'


def test_HLA_A33_09():
    result = parse("HLA-A33:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:09") to be Allele but got {result}'


def test_HLA_A33_10():
    result = parse("HLA-A33:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:10") to be Allele but got {result}'


def test_HLA_A33_11():
    result = parse("HLA-A33:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:11") to be Allele but got {result}'


def test_HLA_A33_12():
    result = parse("HLA-A33:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:12") to be Allele but got {result}'


def test_HLA_A33_13():
    result = parse("HLA-A33:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:13") to be Allele but got {result}'


def test_HLA_A33_14():
    result = parse("HLA-A33:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:14") to be Allele but got {result}'


def test_HLA_A33_15():
    result = parse("HLA-A33:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:15") to be Allele but got {result}'


def test_HLA_A33_16():
    result = parse("HLA-A33:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:16") to be Allele but got {result}'


def test_HLA_A33_17():
    result = parse("HLA-A33:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:17") to be Allele but got {result}'


def test_HLA_A33_18():
    result = parse("HLA-A33:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:18") to be Allele but got {result}'


def test_HLA_A33_19():
    result = parse("HLA-A33:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:19") to be Allele but got {result}'


def test_HLA_A33_20():
    result = parse("HLA-A33:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:20") to be Allele but got {result}'


def test_HLA_A33_21():
    result = parse("HLA-A33:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:21") to be Allele but got {result}'


def test_HLA_A33_22():
    result = parse("HLA-A33:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:22") to be Allele but got {result}'


def test_HLA_A33_23():
    result = parse("HLA-A33:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:23") to be Allele but got {result}'


def test_HLA_A33_24():
    result = parse("HLA-A33:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:24") to be Allele but got {result}'


def test_HLA_A33_25():
    result = parse("HLA-A33:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:25") to be Allele but got {result}'


def test_HLA_A33_26():
    result = parse("HLA-A33:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:26") to be Allele but got {result}'


def test_HLA_A33_27():
    result = parse("HLA-A33:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:27") to be Allele but got {result}'


def test_HLA_A33_28():
    result = parse("HLA-A33:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:28") to be Allele but got {result}'


def test_HLA_A33_29():
    result = parse("HLA-A33:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:29") to be Allele but got {result}'


def test_HLA_A33_30():
    result = parse("HLA-A33:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:30") to be Allele but got {result}'


def test_HLA_A33_31():
    result = parse("HLA-A33:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A33:31") to be Allele but got {result}'


def test_HLA_A34_01():
    result = parse("HLA-A34:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:01") to be Allele but got {result}'


def test_HLA_A34_02():
    result = parse("HLA-A34:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:02") to be Allele but got {result}'


def test_HLA_A34_03():
    result = parse("HLA-A34:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:03") to be Allele but got {result}'


def test_HLA_A34_04():
    result = parse("HLA-A34:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:04") to be Allele but got {result}'


def test_HLA_A34_05():
    result = parse("HLA-A34:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:05") to be Allele but got {result}'


def test_HLA_A34_06():
    result = parse("HLA-A34:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:06") to be Allele but got {result}'


def test_HLA_A34_07():
    result = parse("HLA-A34:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:07") to be Allele but got {result}'


def test_HLA_A34_08():
    result = parse("HLA-A34:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A34:08") to be Allele but got {result}'


def test_HLA_A36_01():
    result = parse("HLA-A36:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A36:01") to be Allele but got {result}'


def test_HLA_A36_02():
    result = parse("HLA-A36:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A36:02") to be Allele but got {result}'


def test_HLA_A36_03():
    result = parse("HLA-A36:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A36:03") to be Allele but got {result}'


def test_HLA_A36_04():
    result = parse("HLA-A36:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A36:04") to be Allele but got {result}'


def test_HLA_A36_05():
    result = parse("HLA-A36:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A36:05") to be Allele but got {result}'


def test_HLA_A43_01():
    result = parse("HLA-A43:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A43:01") to be Allele but got {result}'


def test_HLA_A66_01():
    result = parse("HLA-A66:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:01") to be Allele but got {result}'


def test_HLA_A66_02():
    result = parse("HLA-A66:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:02") to be Allele but got {result}'


def test_HLA_A66_03():
    result = parse("HLA-A66:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:03") to be Allele but got {result}'


def test_HLA_A66_04():
    result = parse("HLA-A66:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:04") to be Allele but got {result}'


def test_HLA_A66_05():
    result = parse("HLA-A66:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:05") to be Allele but got {result}'


def test_HLA_A66_06():
    result = parse("HLA-A66:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:06") to be Allele but got {result}'


def test_HLA_A66_07():
    result = parse("HLA-A66:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:07") to be Allele but got {result}'


def test_HLA_A66_08():
    result = parse("HLA-A66:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:08") to be Allele but got {result}'


def test_HLA_A66_09():
    result = parse("HLA-A66:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:09") to be Allele but got {result}'


def test_HLA_A66_10():
    result = parse("HLA-A66:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:10") to be Allele but got {result}'


def test_HLA_A66_11():
    result = parse("HLA-A66:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:11") to be Allele but got {result}'


def test_HLA_A66_12():
    result = parse("HLA-A66:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:12") to be Allele but got {result}'


def test_HLA_A66_13():
    result = parse("HLA-A66:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:13") to be Allele but got {result}'


def test_HLA_A66_14():
    result = parse("HLA-A66:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:14") to be Allele but got {result}'


def test_HLA_A66_15():
    result = parse("HLA-A66:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A66:15") to be Allele but got {result}'


def test_HLA_A68_01():
    result = parse("HLA-A68:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:01") to be Allele but got {result}'


def test_HLA_A68_02():
    result = parse("HLA-A68:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:02") to be Allele but got {result}'


def test_HLA_A68_03():
    result = parse("HLA-A68:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:03") to be Allele but got {result}'


def test_HLA_A68_04():
    result = parse("HLA-A68:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:04") to be Allele but got {result}'


def test_HLA_A68_05():
    result = parse("HLA-A68:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:05") to be Allele but got {result}'


def test_HLA_A68_06():
    result = parse("HLA-A68:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:06") to be Allele but got {result}'


def test_HLA_A68_07():
    result = parse("HLA-A68:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:07") to be Allele but got {result}'


def test_HLA_A68_08():
    result = parse("HLA-A68:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:08") to be Allele but got {result}'


def test_HLA_A68_09():
    result = parse("HLA-A68:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:09") to be Allele but got {result}'


def test_HLA_A68_10():
    result = parse("HLA-A68:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:10") to be Allele but got {result}'


def test_HLA_A68_12():
    result = parse("HLA-A68:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:12") to be Allele but got {result}'


def test_HLA_A68_13():
    result = parse("HLA-A68:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:13") to be Allele but got {result}'


def test_HLA_A68_14():
    result = parse("HLA-A68:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:14") to be Allele but got {result}'


def test_HLA_A68_15():
    result = parse("HLA-A68:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:15") to be Allele but got {result}'


def test_HLA_A68_16():
    result = parse("HLA-A68:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:16") to be Allele but got {result}'


def test_HLA_A68_17():
    result = parse("HLA-A68:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:17") to be Allele but got {result}'


def test_HLA_A68_19():
    result = parse("HLA-A68:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:19") to be Allele but got {result}'


def test_HLA_A68_20():
    result = parse("HLA-A68:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:20") to be Allele but got {result}'


def test_HLA_A68_21():
    result = parse("HLA-A68:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:21") to be Allele but got {result}'


def test_HLA_A68_22():
    result = parse("HLA-A68:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:22") to be Allele but got {result}'


def test_HLA_A68_23():
    result = parse("HLA-A68:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:23") to be Allele but got {result}'


def test_HLA_A68_24():
    result = parse("HLA-A68:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:24") to be Allele but got {result}'


def test_HLA_A68_25():
    result = parse("HLA-A68:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:25") to be Allele but got {result}'


def test_HLA_A68_26():
    result = parse("HLA-A68:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:26") to be Allele but got {result}'


def test_HLA_A68_27():
    result = parse("HLA-A68:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:27") to be Allele but got {result}'


def test_HLA_A68_28():
    result = parse("HLA-A68:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:28") to be Allele but got {result}'


def test_HLA_A68_29():
    result = parse("HLA-A68:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:29") to be Allele but got {result}'


def test_HLA_A68_30():
    result = parse("HLA-A68:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:30") to be Allele but got {result}'


def test_HLA_A68_31():
    result = parse("HLA-A68:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:31") to be Allele but got {result}'


def test_HLA_A68_32():
    result = parse("HLA-A68:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:32") to be Allele but got {result}'


def test_HLA_A68_33():
    result = parse("HLA-A68:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:33") to be Allele but got {result}'


def test_HLA_A68_34():
    result = parse("HLA-A68:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:34") to be Allele but got {result}'


def test_HLA_A68_35():
    result = parse("HLA-A68:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:35") to be Allele but got {result}'


def test_HLA_A68_36():
    result = parse("HLA-A68:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:36") to be Allele but got {result}'


def test_HLA_A68_37():
    result = parse("HLA-A68:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:37") to be Allele but got {result}'


def test_HLA_A68_38():
    result = parse("HLA-A68:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:38") to be Allele but got {result}'


def test_HLA_A68_39():
    result = parse("HLA-A68:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:39") to be Allele but got {result}'


def test_HLA_A68_40():
    result = parse("HLA-A68:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:40") to be Allele but got {result}'


def test_HLA_A68_41():
    result = parse("HLA-A68:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:41") to be Allele but got {result}'


def test_HLA_A68_42():
    result = parse("HLA-A68:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:42") to be Allele but got {result}'


def test_HLA_A68_43():
    result = parse("HLA-A68:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:43") to be Allele but got {result}'


def test_HLA_A68_44():
    result = parse("HLA-A68:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:44") to be Allele but got {result}'


def test_HLA_A68_45():
    result = parse("HLA-A68:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:45") to be Allele but got {result}'


def test_HLA_A68_46():
    result = parse("HLA-A68:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:46") to be Allele but got {result}'


def test_HLA_A68_47():
    result = parse("HLA-A68:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:47") to be Allele but got {result}'


def test_HLA_A68_48():
    result = parse("HLA-A68:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:48") to be Allele but got {result}'


def test_HLA_A68_50():
    result = parse("HLA-A68:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:50") to be Allele but got {result}'


def test_HLA_A68_51():
    result = parse("HLA-A68:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:51") to be Allele but got {result}'


def test_HLA_A68_52():
    result = parse("HLA-A68:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:52") to be Allele but got {result}'


def test_HLA_A68_53():
    result = parse("HLA-A68:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:53") to be Allele but got {result}'


def test_HLA_A68_54():
    result = parse("HLA-A68:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-A68:54") to be Allele but got {result}'


def test_HLA_A69_01():
    result = parse("HLA-A69:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A69:01") to be Allele but got {result}'


def test_HLA_A74_01():
    result = parse("HLA-A74:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:01") to be Allele but got {result}'


def test_HLA_A74_02():
    result = parse("HLA-A74:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:02") to be Allele but got {result}'


def test_HLA_A74_03():
    result = parse("HLA-A74:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:03") to be Allele but got {result}'


def test_HLA_A74_04():
    result = parse("HLA-A74:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:04") to be Allele but got {result}'


def test_HLA_A74_05():
    result = parse("HLA-A74:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:05") to be Allele but got {result}'


def test_HLA_A74_06():
    result = parse("HLA-A74:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:06") to be Allele but got {result}'


def test_HLA_A74_07():
    result = parse("HLA-A74:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:07") to be Allele but got {result}'


def test_HLA_A74_08():
    result = parse("HLA-A74:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:08") to be Allele but got {result}'


def test_HLA_A74_09():
    result = parse("HLA-A74:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:09") to be Allele but got {result}'


def test_HLA_A74_10():
    result = parse("HLA-A74:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:10") to be Allele but got {result}'


def test_HLA_A74_11():
    result = parse("HLA-A74:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:11") to be Allele but got {result}'


def test_HLA_A74_13():
    result = parse("HLA-A74:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-A74:13") to be Allele but got {result}'


def test_HLA_A80_01():
    result = parse("HLA-A80:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-A80:01") to be Allele but got {result}'


def test_HLA_A80_02():
    result = parse("HLA-A80:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-A80:02") to be Allele but got {result}'


def test_HLA_B07_02():
    result = parse("HLA-B07:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:02") to be Allele but got {result}'


def test_HLA_B07_03():
    result = parse("HLA-B07:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:03") to be Allele but got {result}'


def test_HLA_B07_04():
    result = parse("HLA-B07:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:04") to be Allele but got {result}'


def test_HLA_B07_05():
    result = parse("HLA-B07:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:05") to be Allele but got {result}'


def test_HLA_B07_06():
    result = parse("HLA-B07:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:06") to be Allele but got {result}'


def test_HLA_B07_07():
    result = parse("HLA-B07:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:07") to be Allele but got {result}'


def test_HLA_B07_08():
    result = parse("HLA-B07:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:08") to be Allele but got {result}'


def test_HLA_B07_09():
    result = parse("HLA-B07:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:09") to be Allele but got {result}'


def test_HLA_B07_10():
    result = parse("HLA-B07:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:10") to be Allele but got {result}'


def test_HLA_B07_11():
    result = parse("HLA-B07:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:11") to be Allele but got {result}'


def test_HLA_B07_12():
    result = parse("HLA-B07:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:12") to be Allele but got {result}'


def test_HLA_B07_13():
    result = parse("HLA-B07:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:13") to be Allele but got {result}'


def test_HLA_B07_14():
    result = parse("HLA-B07:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:14") to be Allele but got {result}'


def test_HLA_B07_15():
    result = parse("HLA-B07:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:15") to be Allele but got {result}'


def test_HLA_B07_16():
    result = parse("HLA-B07:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:16") to be Allele but got {result}'


def test_HLA_B07_17():
    result = parse("HLA-B07:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:17") to be Allele but got {result}'


def test_HLA_B07_18():
    result = parse("HLA-B07:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:18") to be Allele but got {result}'


def test_HLA_B07_19():
    result = parse("HLA-B07:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:19") to be Allele but got {result}'


def test_HLA_B07_20():
    result = parse("HLA-B07:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:20") to be Allele but got {result}'


def test_HLA_B07_21():
    result = parse("HLA-B07:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:21") to be Allele but got {result}'


def test_HLA_B07_22():
    result = parse("HLA-B07:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:22") to be Allele but got {result}'


def test_HLA_B07_23():
    result = parse("HLA-B07:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:23") to be Allele but got {result}'


def test_HLA_B07_24():
    result = parse("HLA-B07:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:24") to be Allele but got {result}'


def test_HLA_B07_25():
    result = parse("HLA-B07:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:25") to be Allele but got {result}'


def test_HLA_B07_26():
    result = parse("HLA-B07:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:26") to be Allele but got {result}'


def test_HLA_B07_27():
    result = parse("HLA-B07:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:27") to be Allele but got {result}'


def test_HLA_B07_28():
    result = parse("HLA-B07:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:28") to be Allele but got {result}'


def test_HLA_B07_29():
    result = parse("HLA-B07:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:29") to be Allele but got {result}'


def test_HLA_B07_30():
    result = parse("HLA-B07:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:30") to be Allele but got {result}'


def test_HLA_B07_31():
    result = parse("HLA-B07:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:31") to be Allele but got {result}'


def test_HLA_B07_32():
    result = parse("HLA-B07:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:32") to be Allele but got {result}'


def test_HLA_B07_33():
    result = parse("HLA-B07:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:33") to be Allele but got {result}'


def test_HLA_B07_34():
    result = parse("HLA-B07:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:34") to be Allele but got {result}'


def test_HLA_B07_35():
    result = parse("HLA-B07:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:35") to be Allele but got {result}'


def test_HLA_B07_36():
    result = parse("HLA-B07:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:36") to be Allele but got {result}'


def test_HLA_B07_37():
    result = parse("HLA-B07:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:37") to be Allele but got {result}'


def test_HLA_B07_38():
    result = parse("HLA-B07:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:38") to be Allele but got {result}'


def test_HLA_B07_39():
    result = parse("HLA-B07:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:39") to be Allele but got {result}'


def test_HLA_B07_40():
    result = parse("HLA-B07:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:40") to be Allele but got {result}'


def test_HLA_B07_41():
    result = parse("HLA-B07:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:41") to be Allele but got {result}'


def test_HLA_B07_42():
    result = parse("HLA-B07:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:42") to be Allele but got {result}'


def test_HLA_B07_43():
    result = parse("HLA-B07:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:43") to be Allele but got {result}'


def test_HLA_B07_44():
    result = parse("HLA-B07:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:44") to be Allele but got {result}'


def test_HLA_B07_45():
    result = parse("HLA-B07:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:45") to be Allele but got {result}'


def test_HLA_B07_46():
    result = parse("HLA-B07:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:46") to be Allele but got {result}'


def test_HLA_B07_47():
    result = parse("HLA-B07:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:47") to be Allele but got {result}'


def test_HLA_B07_48():
    result = parse("HLA-B07:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:48") to be Allele but got {result}'


def test_HLA_B07_50():
    result = parse("HLA-B07:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:50") to be Allele but got {result}'


def test_HLA_B07_51():
    result = parse("HLA-B07:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:51") to be Allele but got {result}'


def test_HLA_B07_52():
    result = parse("HLA-B07:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:52") to be Allele but got {result}'


def test_HLA_B07_53():
    result = parse("HLA-B07:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:53") to be Allele but got {result}'


def test_HLA_B07_54():
    result = parse("HLA-B07:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:54") to be Allele but got {result}'


def test_HLA_B07_55():
    result = parse("HLA-B07:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:55") to be Allele but got {result}'


def test_HLA_B07_56():
    result = parse("HLA-B07:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:56") to be Allele but got {result}'


def test_HLA_B07_57():
    result = parse("HLA-B07:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:57") to be Allele but got {result}'


def test_HLA_B07_58():
    result = parse("HLA-B07:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:58") to be Allele but got {result}'


def test_HLA_B07_59():
    result = parse("HLA-B07:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:59") to be Allele but got {result}'


def test_HLA_B07_60():
    result = parse("HLA-B07:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:60") to be Allele but got {result}'


def test_HLA_B07_61():
    result = parse("HLA-B07:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:61") to be Allele but got {result}'


def test_HLA_B07_62():
    result = parse("HLA-B07:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:62") to be Allele but got {result}'


def test_HLA_B07_63():
    result = parse("HLA-B07:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:63") to be Allele but got {result}'


def test_HLA_B07_64():
    result = parse("HLA-B07:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:64") to be Allele but got {result}'


def test_HLA_B07_65():
    result = parse("HLA-B07:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:65") to be Allele but got {result}'


def test_HLA_B07_66():
    result = parse("HLA-B07:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:66") to be Allele but got {result}'


def test_HLA_B07_68():
    result = parse("HLA-B07:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:68") to be Allele but got {result}'


def test_HLA_B07_69():
    result = parse("HLA-B07:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:69") to be Allele but got {result}'


def test_HLA_B07_70():
    result = parse("HLA-B07:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:70") to be Allele but got {result}'


def test_HLA_B07_71():
    result = parse("HLA-B07:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:71") to be Allele but got {result}'


def test_HLA_B07_72():
    result = parse("HLA-B07:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:72") to be Allele but got {result}'


def test_HLA_B07_73():
    result = parse("HLA-B07:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:73") to be Allele but got {result}'


def test_HLA_B07_74():
    result = parse("HLA-B07:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:74") to be Allele but got {result}'


def test_HLA_B07_75():
    result = parse("HLA-B07:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:75") to be Allele but got {result}'


def test_HLA_B07_76():
    result = parse("HLA-B07:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:76") to be Allele but got {result}'


def test_HLA_B07_77():
    result = parse("HLA-B07:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:77") to be Allele but got {result}'


def test_HLA_B07_78():
    result = parse("HLA-B07:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:78") to be Allele but got {result}'


def test_HLA_B07_79():
    result = parse("HLA-B07:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:79") to be Allele but got {result}'


def test_HLA_B07_80():
    result = parse("HLA-B07:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:80") to be Allele but got {result}'


def test_HLA_B07_81():
    result = parse("HLA-B07:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:81") to be Allele but got {result}'


def test_HLA_B07_82():
    result = parse("HLA-B07:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:82") to be Allele but got {result}'


def test_HLA_B07_83():
    result = parse("HLA-B07:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:83") to be Allele but got {result}'


def test_HLA_B07_84():
    result = parse("HLA-B07:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:84") to be Allele but got {result}'


def test_HLA_B07_85():
    result = parse("HLA-B07:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:85") to be Allele but got {result}'


def test_HLA_B07_86():
    result = parse("HLA-B07:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:86") to be Allele but got {result}'


def test_HLA_B07_87():
    result = parse("HLA-B07:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:87") to be Allele but got {result}'


def test_HLA_B07_88():
    result = parse("HLA-B07:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:88") to be Allele but got {result}'


def test_HLA_B07_89():
    result = parse("HLA-B07:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:89") to be Allele but got {result}'


def test_HLA_B07_90():
    result = parse("HLA-B07:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:90") to be Allele but got {result}'


def test_HLA_B07_91():
    result = parse("HLA-B07:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:91") to be Allele but got {result}'


def test_HLA_B07_92():
    result = parse("HLA-B07:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:92") to be Allele but got {result}'


def test_HLA_B07_93():
    result = parse("HLA-B07:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:93") to be Allele but got {result}'


def test_HLA_B07_94():
    result = parse("HLA-B07:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:94") to be Allele but got {result}'


def test_HLA_B07_95():
    result = parse("HLA-B07:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:95") to be Allele but got {result}'


def test_HLA_B07_96():
    result = parse("HLA-B07:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:96") to be Allele but got {result}'


def test_HLA_B07_97():
    result = parse("HLA-B07:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:97") to be Allele but got {result}'


def test_HLA_B07_98():
    result = parse("HLA-B07:98")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:98") to be Allele but got {result}'


def test_HLA_B07_99():
    result = parse("HLA-B07:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-B07:99") to be Allele but got {result}'


def test_HLA_B07_100():
    result = parse("HLA-B07:100")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:100") to be Allele but got {result}'
    )


def test_HLA_B07_101():
    result = parse("HLA-B07:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:101") to be Allele but got {result}'
    )


def test_HLA_B07_102():
    result = parse("HLA-B07:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:102") to be Allele but got {result}'
    )


def test_HLA_B07_103():
    result = parse("HLA-B07:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:103") to be Allele but got {result}'
    )


def test_HLA_B07_104():
    result = parse("HLA-B07:104")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:104") to be Allele but got {result}'
    )


def test_HLA_B07_105():
    result = parse("HLA-B07:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:105") to be Allele but got {result}'
    )


def test_HLA_B07_106():
    result = parse("HLA-B07:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:106") to be Allele but got {result}'
    )


def test_HLA_B07_107():
    result = parse("HLA-B07:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:107") to be Allele but got {result}'
    )


def test_HLA_B07_108():
    result = parse("HLA-B07:108")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:108") to be Allele but got {result}'
    )


def test_HLA_B07_109():
    result = parse("HLA-B07:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:109") to be Allele but got {result}'
    )


def test_HLA_B07_110():
    result = parse("HLA-B07:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:110") to be Allele but got {result}'
    )


def test_HLA_B07_112():
    result = parse("HLA-B07:112")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:112") to be Allele but got {result}'
    )


def test_HLA_B07_113():
    result = parse("HLA-B07:113")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:113") to be Allele but got {result}'
    )


def test_HLA_B07_114():
    result = parse("HLA-B07:114")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:114") to be Allele but got {result}'
    )


def test_HLA_B07_115():
    result = parse("HLA-B07:115")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B07:115") to be Allele but got {result}'
    )


def test_HLA_B08_01():
    result = parse("HLA-B08:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:01") to be Allele but got {result}'


def test_HLA_B08_02():
    result = parse("HLA-B08:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:02") to be Allele but got {result}'


def test_HLA_B08_03():
    result = parse("HLA-B08:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:03") to be Allele but got {result}'


def test_HLA_B08_04():
    result = parse("HLA-B08:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:04") to be Allele but got {result}'


def test_HLA_B08_05():
    result = parse("HLA-B08:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:05") to be Allele but got {result}'


def test_HLA_B08_07():
    result = parse("HLA-B08:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:07") to be Allele but got {result}'


def test_HLA_B08_09():
    result = parse("HLA-B08:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:09") to be Allele but got {result}'


def test_HLA_B08_10():
    result = parse("HLA-B08:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:10") to be Allele but got {result}'


def test_HLA_B08_11():
    result = parse("HLA-B08:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:11") to be Allele but got {result}'


def test_HLA_B08_12():
    result = parse("HLA-B08:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:12") to be Allele but got {result}'


def test_HLA_B08_13():
    result = parse("HLA-B08:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:13") to be Allele but got {result}'


def test_HLA_B08_14():
    result = parse("HLA-B08:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:14") to be Allele but got {result}'


def test_HLA_B08_15():
    result = parse("HLA-B08:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:15") to be Allele but got {result}'


def test_HLA_B08_16():
    result = parse("HLA-B08:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:16") to be Allele but got {result}'


def test_HLA_B08_17():
    result = parse("HLA-B08:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:17") to be Allele but got {result}'


def test_HLA_B08_18():
    result = parse("HLA-B08:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:18") to be Allele but got {result}'


def test_HLA_B08_20():
    result = parse("HLA-B08:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:20") to be Allele but got {result}'


def test_HLA_B08_21():
    result = parse("HLA-B08:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:21") to be Allele but got {result}'


def test_HLA_B08_22():
    result = parse("HLA-B08:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:22") to be Allele but got {result}'


def test_HLA_B08_23():
    result = parse("HLA-B08:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:23") to be Allele but got {result}'


def test_HLA_B08_24():
    result = parse("HLA-B08:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:24") to be Allele but got {result}'


def test_HLA_B08_25():
    result = parse("HLA-B08:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:25") to be Allele but got {result}'


def test_HLA_B08_26():
    result = parse("HLA-B08:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:26") to be Allele but got {result}'


def test_HLA_B08_27():
    result = parse("HLA-B08:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:27") to be Allele but got {result}'


def test_HLA_B08_28():
    result = parse("HLA-B08:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:28") to be Allele but got {result}'


def test_HLA_B08_29():
    result = parse("HLA-B08:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:29") to be Allele but got {result}'


def test_HLA_B08_31():
    result = parse("HLA-B08:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:31") to be Allele but got {result}'


def test_HLA_B08_32():
    result = parse("HLA-B08:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:32") to be Allele but got {result}'


def test_HLA_B08_33():
    result = parse("HLA-B08:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:33") to be Allele but got {result}'


def test_HLA_B08_34():
    result = parse("HLA-B08:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:34") to be Allele but got {result}'


def test_HLA_B08_35():
    result = parse("HLA-B08:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:35") to be Allele but got {result}'


def test_HLA_B08_36():
    result = parse("HLA-B08:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:36") to be Allele but got {result}'


def test_HLA_B08_37():
    result = parse("HLA-B08:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:37") to be Allele but got {result}'


def test_HLA_B08_38():
    result = parse("HLA-B08:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:38") to be Allele but got {result}'


def test_HLA_B08_39():
    result = parse("HLA-B08:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:39") to be Allele but got {result}'


def test_HLA_B08_40():
    result = parse("HLA-B08:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:40") to be Allele but got {result}'


def test_HLA_B08_41():
    result = parse("HLA-B08:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:41") to be Allele but got {result}'


def test_HLA_B08_42():
    result = parse("HLA-B08:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:42") to be Allele but got {result}'


def test_HLA_B08_43():
    result = parse("HLA-B08:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:43") to be Allele but got {result}'


def test_HLA_B08_44():
    result = parse("HLA-B08:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:44") to be Allele but got {result}'


def test_HLA_B08_45():
    result = parse("HLA-B08:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:45") to be Allele but got {result}'


def test_HLA_B08_46():
    result = parse("HLA-B08:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:46") to be Allele but got {result}'


def test_HLA_B08_47():
    result = parse("HLA-B08:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:47") to be Allele but got {result}'


def test_HLA_B08_48():
    result = parse("HLA-B08:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:48") to be Allele but got {result}'


def test_HLA_B08_49():
    result = parse("HLA-B08:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:49") to be Allele but got {result}'


def test_HLA_B08_50():
    result = parse("HLA-B08:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:50") to be Allele but got {result}'


def test_HLA_B08_51():
    result = parse("HLA-B08:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:51") to be Allele but got {result}'


def test_HLA_B08_52():
    result = parse("HLA-B08:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:52") to be Allele but got {result}'


def test_HLA_B08_53():
    result = parse("HLA-B08:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:53") to be Allele but got {result}'


def test_HLA_B08_54():
    result = parse("HLA-B08:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:54") to be Allele but got {result}'


def test_HLA_B08_55():
    result = parse("HLA-B08:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:55") to be Allele but got {result}'


def test_HLA_B08_56():
    result = parse("HLA-B08:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:56") to be Allele but got {result}'


def test_HLA_B08_57():
    result = parse("HLA-B08:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:57") to be Allele but got {result}'


def test_HLA_B08_58():
    result = parse("HLA-B08:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:58") to be Allele but got {result}'


def test_HLA_B08_59():
    result = parse("HLA-B08:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:59") to be Allele but got {result}'


def test_HLA_B08_60():
    result = parse("HLA-B08:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:60") to be Allele but got {result}'


def test_HLA_B08_61():
    result = parse("HLA-B08:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:61") to be Allele but got {result}'


def test_HLA_B08_62():
    result = parse("HLA-B08:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B08:62") to be Allele but got {result}'


def test_HLA_B13_01():
    result = parse("HLA-B13:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:01") to be Allele but got {result}'


def test_HLA_B13_02():
    result = parse("HLA-B13:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:02") to be Allele but got {result}'


def test_HLA_B13_03():
    result = parse("HLA-B13:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:03") to be Allele but got {result}'


def test_HLA_B13_04():
    result = parse("HLA-B13:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:04") to be Allele but got {result}'


def test_HLA_B13_06():
    result = parse("HLA-B13:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:06") to be Allele but got {result}'


def test_HLA_B13_09():
    result = parse("HLA-B13:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:09") to be Allele but got {result}'


def test_HLA_B13_10():
    result = parse("HLA-B13:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:10") to be Allele but got {result}'


def test_HLA_B13_11():
    result = parse("HLA-B13:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:11") to be Allele but got {result}'


def test_HLA_B13_12():
    result = parse("HLA-B13:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:12") to be Allele but got {result}'


def test_HLA_B13_13():
    result = parse("HLA-B13:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:13") to be Allele but got {result}'


def test_HLA_B13_14():
    result = parse("HLA-B13:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:14") to be Allele but got {result}'


def test_HLA_B13_15():
    result = parse("HLA-B13:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:15") to be Allele but got {result}'


def test_HLA_B13_16():
    result = parse("HLA-B13:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:16") to be Allele but got {result}'


def test_HLA_B13_17():
    result = parse("HLA-B13:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:17") to be Allele but got {result}'


def test_HLA_B13_18():
    result = parse("HLA-B13:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:18") to be Allele but got {result}'


def test_HLA_B13_19():
    result = parse("HLA-B13:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:19") to be Allele but got {result}'


def test_HLA_B13_20():
    result = parse("HLA-B13:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:20") to be Allele but got {result}'


def test_HLA_B13_21():
    result = parse("HLA-B13:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:21") to be Allele but got {result}'


def test_HLA_B13_22():
    result = parse("HLA-B13:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:22") to be Allele but got {result}'


def test_HLA_B13_23():
    result = parse("HLA-B13:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:23") to be Allele but got {result}'


def test_HLA_B13_25():
    result = parse("HLA-B13:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:25") to be Allele but got {result}'


def test_HLA_B13_26():
    result = parse("HLA-B13:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:26") to be Allele but got {result}'


def test_HLA_B13_27():
    result = parse("HLA-B13:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:27") to be Allele but got {result}'


def test_HLA_B13_28():
    result = parse("HLA-B13:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:28") to be Allele but got {result}'


def test_HLA_B13_29():
    result = parse("HLA-B13:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:29") to be Allele but got {result}'


def test_HLA_B13_30():
    result = parse("HLA-B13:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:30") to be Allele but got {result}'


def test_HLA_B13_31():
    result = parse("HLA-B13:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:31") to be Allele but got {result}'


def test_HLA_B13_32():
    result = parse("HLA-B13:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:32") to be Allele but got {result}'


def test_HLA_B13_33():
    result = parse("HLA-B13:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:33") to be Allele but got {result}'


def test_HLA_B13_34():
    result = parse("HLA-B13:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:34") to be Allele but got {result}'


def test_HLA_B13_35():
    result = parse("HLA-B13:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:35") to be Allele but got {result}'


def test_HLA_B13_36():
    result = parse("HLA-B13:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:36") to be Allele but got {result}'


def test_HLA_B13_37():
    result = parse("HLA-B13:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:37") to be Allele but got {result}'


def test_HLA_B13_38():
    result = parse("HLA-B13:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:38") to be Allele but got {result}'


def test_HLA_B13_39():
    result = parse("HLA-B13:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B13:39") to be Allele but got {result}'


def test_HLA_B14_01():
    result = parse("HLA-B14:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:01") to be Allele but got {result}'


def test_HLA_B14_02():
    result = parse("HLA-B14:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:02") to be Allele but got {result}'


def test_HLA_B14_03():
    result = parse("HLA-B14:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:03") to be Allele but got {result}'


def test_HLA_B14_04():
    result = parse("HLA-B14:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:04") to be Allele but got {result}'


def test_HLA_B14_05():
    result = parse("HLA-B14:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:05") to be Allele but got {result}'


def test_HLA_B14_06():
    result = parse("HLA-B14:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:06") to be Allele but got {result}'


def test_HLA_B14_08():
    result = parse("HLA-B14:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:08") to be Allele but got {result}'


def test_HLA_B14_09():
    result = parse("HLA-B14:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:09") to be Allele but got {result}'


def test_HLA_B14_10():
    result = parse("HLA-B14:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:10") to be Allele but got {result}'


def test_HLA_B14_11():
    result = parse("HLA-B14:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:11") to be Allele but got {result}'


def test_HLA_B14_12():
    result = parse("HLA-B14:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:12") to be Allele but got {result}'


def test_HLA_B14_13():
    result = parse("HLA-B14:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:13") to be Allele but got {result}'


def test_HLA_B14_14():
    result = parse("HLA-B14:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:14") to be Allele but got {result}'


def test_HLA_B14_15():
    result = parse("HLA-B14:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:15") to be Allele but got {result}'


def test_HLA_B14_16():
    result = parse("HLA-B14:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:16") to be Allele but got {result}'


def test_HLA_B14_17():
    result = parse("HLA-B14:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:17") to be Allele but got {result}'


def test_HLA_B14_18():
    result = parse("HLA-B14:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B14:18") to be Allele but got {result}'


def test_HLA_B15_01():
    result = parse("HLA-B15:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:01") to be Allele but got {result}'


def test_HLA_B15_02():
    result = parse("HLA-B15:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:02") to be Allele but got {result}'


def test_HLA_B15_03():
    result = parse("HLA-B15:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:03") to be Allele but got {result}'


def test_HLA_B15_04():
    result = parse("HLA-B15:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:04") to be Allele but got {result}'


def test_HLA_B15_05():
    result = parse("HLA-B15:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:05") to be Allele but got {result}'


def test_HLA_B15_06():
    result = parse("HLA-B15:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:06") to be Allele but got {result}'


def test_HLA_B15_07():
    result = parse("HLA-B15:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:07") to be Allele but got {result}'


def test_HLA_B15_08():
    result = parse("HLA-B15:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:08") to be Allele but got {result}'


def test_HLA_B15_09():
    result = parse("HLA-B15:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:09") to be Allele but got {result}'


def test_HLA_B15_10():
    result = parse("HLA-B15:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:10") to be Allele but got {result}'


def test_HLA_B15_11():
    result = parse("HLA-B15:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:11") to be Allele but got {result}'


def test_HLA_B15_12():
    result = parse("HLA-B15:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:12") to be Allele but got {result}'


def test_HLA_B15_13():
    result = parse("HLA-B15:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:13") to be Allele but got {result}'


def test_HLA_B15_14():
    result = parse("HLA-B15:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:14") to be Allele but got {result}'


def test_HLA_B15_15():
    result = parse("HLA-B15:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:15") to be Allele but got {result}'


def test_HLA_B15_16():
    result = parse("HLA-B15:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:16") to be Allele but got {result}'


def test_HLA_B15_17():
    result = parse("HLA-B15:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:17") to be Allele but got {result}'


def test_HLA_B15_18():
    result = parse("HLA-B15:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:18") to be Allele but got {result}'


def test_HLA_B15_19():
    result = parse("HLA-B15:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:19") to be Allele but got {result}'


def test_HLA_B15_20():
    result = parse("HLA-B15:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:20") to be Allele but got {result}'


def test_HLA_B15_21():
    result = parse("HLA-B15:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:21") to be Allele but got {result}'


def test_HLA_B15_23():
    result = parse("HLA-B15:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:23") to be Allele but got {result}'


def test_HLA_B15_24():
    result = parse("HLA-B15:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:24") to be Allele but got {result}'


def test_HLA_B15_25():
    result = parse("HLA-B15:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:25") to be Allele but got {result}'


def test_HLA_B15_27():
    result = parse("HLA-B15:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:27") to be Allele but got {result}'


def test_HLA_B15_28():
    result = parse("HLA-B15:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:28") to be Allele but got {result}'


def test_HLA_B15_29():
    result = parse("HLA-B15:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:29") to be Allele but got {result}'


def test_HLA_B15_30():
    result = parse("HLA-B15:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:30") to be Allele but got {result}'


def test_HLA_B15_31():
    result = parse("HLA-B15:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:31") to be Allele but got {result}'


def test_HLA_B15_32():
    result = parse("HLA-B15:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:32") to be Allele but got {result}'


def test_HLA_B15_33():
    result = parse("HLA-B15:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:33") to be Allele but got {result}'


def test_HLA_B15_34():
    result = parse("HLA-B15:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:34") to be Allele but got {result}'


def test_HLA_B15_35():
    result = parse("HLA-B15:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:35") to be Allele but got {result}'


def test_HLA_B15_36():
    result = parse("HLA-B15:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:36") to be Allele but got {result}'


def test_HLA_B15_37():
    result = parse("HLA-B15:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:37") to be Allele but got {result}'


def test_HLA_B15_38():
    result = parse("HLA-B15:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:38") to be Allele but got {result}'


def test_HLA_B15_39():
    result = parse("HLA-B15:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:39") to be Allele but got {result}'


def test_HLA_B15_40():
    result = parse("HLA-B15:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:40") to be Allele but got {result}'


def test_HLA_B15_42():
    result = parse("HLA-B15:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:42") to be Allele but got {result}'


def test_HLA_B15_43():
    result = parse("HLA-B15:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:43") to be Allele but got {result}'


def test_HLA_B15_44():
    result = parse("HLA-B15:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:44") to be Allele but got {result}'


def test_HLA_B15_45():
    result = parse("HLA-B15:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:45") to be Allele but got {result}'


def test_HLA_B15_46():
    result = parse("HLA-B15:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:46") to be Allele but got {result}'


def test_HLA_B15_47():
    result = parse("HLA-B15:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:47") to be Allele but got {result}'


def test_HLA_B15_48():
    result = parse("HLA-B15:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:48") to be Allele but got {result}'


def test_HLA_B15_49():
    result = parse("HLA-B15:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:49") to be Allele but got {result}'


def test_HLA_B15_50():
    result = parse("HLA-B15:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:50") to be Allele but got {result}'


def test_HLA_B15_51():
    result = parse("HLA-B15:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:51") to be Allele but got {result}'


def test_HLA_B15_52():
    result = parse("HLA-B15:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:52") to be Allele but got {result}'


def test_HLA_B15_53():
    result = parse("HLA-B15:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:53") to be Allele but got {result}'


def test_HLA_B15_54():
    result = parse("HLA-B15:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:54") to be Allele but got {result}'


def test_HLA_B15_55():
    result = parse("HLA-B15:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:55") to be Allele but got {result}'


def test_HLA_B15_56():
    result = parse("HLA-B15:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:56") to be Allele but got {result}'


def test_HLA_B15_57():
    result = parse("HLA-B15:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:57") to be Allele but got {result}'


def test_HLA_B15_58():
    result = parse("HLA-B15:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:58") to be Allele but got {result}'


def test_HLA_B15_60():
    result = parse("HLA-B15:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:60") to be Allele but got {result}'


def test_HLA_B15_61():
    result = parse("HLA-B15:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:61") to be Allele but got {result}'


def test_HLA_B15_62():
    result = parse("HLA-B15:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:62") to be Allele but got {result}'


def test_HLA_B15_63():
    result = parse("HLA-B15:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:63") to be Allele but got {result}'


def test_HLA_B15_64():
    result = parse("HLA-B15:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:64") to be Allele but got {result}'


def test_HLA_B15_65():
    result = parse("HLA-B15:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:65") to be Allele but got {result}'


def test_HLA_B15_66():
    result = parse("HLA-B15:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:66") to be Allele but got {result}'


def test_HLA_B15_67():
    result = parse("HLA-B15:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:67") to be Allele but got {result}'


def test_HLA_B15_68():
    result = parse("HLA-B15:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:68") to be Allele but got {result}'


def test_HLA_B15_69():
    result = parse("HLA-B15:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:69") to be Allele but got {result}'


def test_HLA_B15_70():
    result = parse("HLA-B15:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:70") to be Allele but got {result}'


def test_HLA_B15_71():
    result = parse("HLA-B15:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:71") to be Allele but got {result}'


def test_HLA_B15_72():
    result = parse("HLA-B15:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:72") to be Allele but got {result}'


def test_HLA_B15_73():
    result = parse("HLA-B15:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:73") to be Allele but got {result}'


def test_HLA_B15_74():
    result = parse("HLA-B15:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:74") to be Allele but got {result}'


def test_HLA_B15_75():
    result = parse("HLA-B15:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:75") to be Allele but got {result}'


def test_HLA_B15_76():
    result = parse("HLA-B15:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:76") to be Allele but got {result}'


def test_HLA_B15_77():
    result = parse("HLA-B15:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:77") to be Allele but got {result}'


def test_HLA_B15_78():
    result = parse("HLA-B15:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:78") to be Allele but got {result}'


def test_HLA_B15_80():
    result = parse("HLA-B15:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:80") to be Allele but got {result}'


def test_HLA_B15_81():
    result = parse("HLA-B15:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:81") to be Allele but got {result}'


def test_HLA_B15_82():
    result = parse("HLA-B15:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:82") to be Allele but got {result}'


def test_HLA_B15_83():
    result = parse("HLA-B15:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:83") to be Allele but got {result}'


def test_HLA_B15_84():
    result = parse("HLA-B15:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:84") to be Allele but got {result}'


def test_HLA_B15_85():
    result = parse("HLA-B15:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:85") to be Allele but got {result}'


def test_HLA_B15_86():
    result = parse("HLA-B15:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:86") to be Allele but got {result}'


def test_HLA_B15_87():
    result = parse("HLA-B15:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:87") to be Allele but got {result}'


def test_HLA_B15_88():
    result = parse("HLA-B15:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:88") to be Allele but got {result}'


def test_HLA_B15_89():
    result = parse("HLA-B15:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:89") to be Allele but got {result}'


def test_HLA_B15_90():
    result = parse("HLA-B15:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:90") to be Allele but got {result}'


def test_HLA_B15_91():
    result = parse("HLA-B15:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:91") to be Allele but got {result}'


def test_HLA_B15_92():
    result = parse("HLA-B15:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:92") to be Allele but got {result}'


def test_HLA_B15_93():
    result = parse("HLA-B15:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:93") to be Allele but got {result}'


def test_HLA_B15_95():
    result = parse("HLA-B15:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:95") to be Allele but got {result}'


def test_HLA_B15_96():
    result = parse("HLA-B15:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:96") to be Allele but got {result}'


def test_HLA_B15_97():
    result = parse("HLA-B15:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:97") to be Allele but got {result}'


def test_HLA_B15_98():
    result = parse("HLA-B15:98")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:98") to be Allele but got {result}'


def test_HLA_B15_99():
    result = parse("HLA-B15:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-B15:99") to be Allele but got {result}'


def test_HLA_B15_101():
    result = parse("HLA-B15:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:101") to be Allele but got {result}'
    )


def test_HLA_B15_102():
    result = parse("HLA-B15:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:102") to be Allele but got {result}'
    )


def test_HLA_B15_103():
    result = parse("HLA-B15:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:103") to be Allele but got {result}'
    )


def test_HLA_B15_104():
    result = parse("HLA-B15:104")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:104") to be Allele but got {result}'
    )


def test_HLA_B15_105():
    result = parse("HLA-B15:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:105") to be Allele but got {result}'
    )


def test_HLA_B15_106():
    result = parse("HLA-B15:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:106") to be Allele but got {result}'
    )


def test_HLA_B15_107():
    result = parse("HLA-B15:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:107") to be Allele but got {result}'
    )


def test_HLA_B15_108():
    result = parse("HLA-B15:108")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:108") to be Allele but got {result}'
    )


def test_HLA_B15_109():
    result = parse("HLA-B15:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:109") to be Allele but got {result}'
    )


def test_HLA_B15_110():
    result = parse("HLA-B15:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:110") to be Allele but got {result}'
    )


def test_HLA_B15_112():
    result = parse("HLA-B15:112")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:112") to be Allele but got {result}'
    )


def test_HLA_B15_113():
    result = parse("HLA-B15:113")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:113") to be Allele but got {result}'
    )


def test_HLA_B15_114():
    result = parse("HLA-B15:114")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:114") to be Allele but got {result}'
    )


def test_HLA_B15_115():
    result = parse("HLA-B15:115")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:115") to be Allele but got {result}'
    )


def test_HLA_B15_116():
    result = parse("HLA-B15:116")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:116") to be Allele but got {result}'
    )


def test_HLA_B15_117():
    result = parse("HLA-B15:117")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:117") to be Allele but got {result}'
    )


def test_HLA_B15_118():
    result = parse("HLA-B15:118")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:118") to be Allele but got {result}'
    )


def test_HLA_B15_119():
    result = parse("HLA-B15:119")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:119") to be Allele but got {result}'
    )


def test_HLA_B15_120():
    result = parse("HLA-B15:120")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:120") to be Allele but got {result}'
    )


def test_HLA_B15_121():
    result = parse("HLA-B15:121")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:121") to be Allele but got {result}'
    )


def test_HLA_B15_122():
    result = parse("HLA-B15:122")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:122") to be Allele but got {result}'
    )


def test_HLA_B15_123():
    result = parse("HLA-B15:123")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:123") to be Allele but got {result}'
    )


def test_HLA_B15_124():
    result = parse("HLA-B15:124")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:124") to be Allele but got {result}'
    )


def test_HLA_B15_125():
    result = parse("HLA-B15:125")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:125") to be Allele but got {result}'
    )


def test_HLA_B15_126():
    result = parse("HLA-B15:126")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:126") to be Allele but got {result}'
    )


def test_HLA_B15_127():
    result = parse("HLA-B15:127")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:127") to be Allele but got {result}'
    )


def test_HLA_B15_128():
    result = parse("HLA-B15:128")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:128") to be Allele but got {result}'
    )


def test_HLA_B15_129():
    result = parse("HLA-B15:129")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:129") to be Allele but got {result}'
    )


def test_HLA_B15_131():
    result = parse("HLA-B15:131")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:131") to be Allele but got {result}'
    )


def test_HLA_B15_132():
    result = parse("HLA-B15:132")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:132") to be Allele but got {result}'
    )


def test_HLA_B15_133():
    result = parse("HLA-B15:133")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:133") to be Allele but got {result}'
    )


def test_HLA_B15_134():
    result = parse("HLA-B15:134")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:134") to be Allele but got {result}'
    )


def test_HLA_B15_135():
    result = parse("HLA-B15:135")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:135") to be Allele but got {result}'
    )


def test_HLA_B15_136():
    result = parse("HLA-B15:136")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:136") to be Allele but got {result}'
    )


def test_HLA_B15_137():
    result = parse("HLA-B15:137")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:137") to be Allele but got {result}'
    )


def test_HLA_B15_138():
    result = parse("HLA-B15:138")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:138") to be Allele but got {result}'
    )


def test_HLA_B15_139():
    result = parse("HLA-B15:139")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:139") to be Allele but got {result}'
    )


def test_HLA_B15_140():
    result = parse("HLA-B15:140")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:140") to be Allele but got {result}'
    )


def test_HLA_B15_141():
    result = parse("HLA-B15:141")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:141") to be Allele but got {result}'
    )


def test_HLA_B15_142():
    result = parse("HLA-B15:142")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:142") to be Allele but got {result}'
    )


def test_HLA_B15_143():
    result = parse("HLA-B15:143")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:143") to be Allele but got {result}'
    )


def test_HLA_B15_144():
    result = parse("HLA-B15:144")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:144") to be Allele but got {result}'
    )


def test_HLA_B15_145():
    result = parse("HLA-B15:145")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:145") to be Allele but got {result}'
    )


def test_HLA_B15_146():
    result = parse("HLA-B15:146")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:146") to be Allele but got {result}'
    )


def test_HLA_B15_147():
    result = parse("HLA-B15:147")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:147") to be Allele but got {result}'
    )


def test_HLA_B15_148():
    result = parse("HLA-B15:148")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:148") to be Allele but got {result}'
    )


def test_HLA_B15_150():
    result = parse("HLA-B15:150")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:150") to be Allele but got {result}'
    )


def test_HLA_B15_151():
    result = parse("HLA-B15:151")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:151") to be Allele but got {result}'
    )


def test_HLA_B15_152():
    result = parse("HLA-B15:152")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:152") to be Allele but got {result}'
    )


def test_HLA_B15_153():
    result = parse("HLA-B15:153")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:153") to be Allele but got {result}'
    )


def test_HLA_B15_154():
    result = parse("HLA-B15:154")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:154") to be Allele but got {result}'
    )


def test_HLA_B15_155():
    result = parse("HLA-B15:155")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:155") to be Allele but got {result}'
    )


def test_HLA_B15_156():
    result = parse("HLA-B15:156")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:156") to be Allele but got {result}'
    )


def test_HLA_B15_157():
    result = parse("HLA-B15:157")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:157") to be Allele but got {result}'
    )


def test_HLA_B15_158():
    result = parse("HLA-B15:158")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:158") to be Allele but got {result}'
    )


def test_HLA_B15_159():
    result = parse("HLA-B15:159")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:159") to be Allele but got {result}'
    )


def test_HLA_B15_160():
    result = parse("HLA-B15:160")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:160") to be Allele but got {result}'
    )


def test_HLA_B15_161():
    result = parse("HLA-B15:161")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:161") to be Allele but got {result}'
    )


def test_HLA_B15_162():
    result = parse("HLA-B15:162")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:162") to be Allele but got {result}'
    )


def test_HLA_B15_163():
    result = parse("HLA-B15:163")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:163") to be Allele but got {result}'
    )


def test_HLA_B15_164():
    result = parse("HLA-B15:164")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:164") to be Allele but got {result}'
    )


def test_HLA_B15_165():
    result = parse("HLA-B15:165")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:165") to be Allele but got {result}'
    )


def test_HLA_B15_166():
    result = parse("HLA-B15:166")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:166") to be Allele but got {result}'
    )


def test_HLA_B15_167():
    result = parse("HLA-B15:167")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:167") to be Allele but got {result}'
    )


def test_HLA_B15_168():
    result = parse("HLA-B15:168")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:168") to be Allele but got {result}'
    )


def test_HLA_B15_169():
    result = parse("HLA-B15:169")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:169") to be Allele but got {result}'
    )


def test_HLA_B15_170():
    result = parse("HLA-B15:170")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:170") to be Allele but got {result}'
    )


def test_HLA_B15_171():
    result = parse("HLA-B15:171")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:171") to be Allele but got {result}'
    )


def test_HLA_B15_172():
    result = parse("HLA-B15:172")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:172") to be Allele but got {result}'
    )


def test_HLA_B15_173():
    result = parse("HLA-B15:173")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:173") to be Allele but got {result}'
    )


def test_HLA_B15_174():
    result = parse("HLA-B15:174")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:174") to be Allele but got {result}'
    )


def test_HLA_B15_175():
    result = parse("HLA-B15:175")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:175") to be Allele but got {result}'
    )


def test_HLA_B15_176():
    result = parse("HLA-B15:176")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:176") to be Allele but got {result}'
    )


def test_HLA_B15_177():
    result = parse("HLA-B15:177")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:177") to be Allele but got {result}'
    )


def test_HLA_B15_178():
    result = parse("HLA-B15:178")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:178") to be Allele but got {result}'
    )


def test_HLA_B15_179():
    result = parse("HLA-B15:179")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:179") to be Allele but got {result}'
    )


def test_HLA_B15_180():
    result = parse("HLA-B15:180")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:180") to be Allele but got {result}'
    )


def test_HLA_B15_183():
    result = parse("HLA-B15:183")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:183") to be Allele but got {result}'
    )


def test_HLA_B15_184():
    result = parse("HLA-B15:184")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:184") to be Allele but got {result}'
    )


def test_HLA_B15_185():
    result = parse("HLA-B15:185")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:185") to be Allele but got {result}'
    )


def test_HLA_B15_186():
    result = parse("HLA-B15:186")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:186") to be Allele but got {result}'
    )


def test_HLA_B15_187():
    result = parse("HLA-B15:187")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:187") to be Allele but got {result}'
    )


def test_HLA_B15_188():
    result = parse("HLA-B15:188")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:188") to be Allele but got {result}'
    )


def test_HLA_B15_189():
    result = parse("HLA-B15:189")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:189") to be Allele but got {result}'
    )


def test_HLA_B15_191():
    result = parse("HLA-B15:191")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:191") to be Allele but got {result}'
    )


def test_HLA_B15_192():
    result = parse("HLA-B15:192")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:192") to be Allele but got {result}'
    )


def test_HLA_B15_193():
    result = parse("HLA-B15:193")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:193") to be Allele but got {result}'
    )


def test_HLA_B15_194():
    result = parse("HLA-B15:194")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:194") to be Allele but got {result}'
    )


def test_HLA_B15_195():
    result = parse("HLA-B15:195")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:195") to be Allele but got {result}'
    )


def test_HLA_B15_196():
    result = parse("HLA-B15:196")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:196") to be Allele but got {result}'
    )


def test_HLA_B15_197():
    result = parse("HLA-B15:197")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:197") to be Allele but got {result}'
    )


def test_HLA_B15_198():
    result = parse("HLA-B15:198")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:198") to be Allele but got {result}'
    )


def test_HLA_B15_199():
    result = parse("HLA-B15:199")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:199") to be Allele but got {result}'
    )


def test_HLA_B15_200():
    result = parse("HLA-B15:200")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:200") to be Allele but got {result}'
    )


def test_HLA_B15_201():
    result = parse("HLA-B15:201")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:201") to be Allele but got {result}'
    )


def test_HLA_B15_202():
    result = parse("HLA-B15:202")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B15:202") to be Allele but got {result}'
    )


def test_HLA_B18_01():
    result = parse("HLA-B18:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:01") to be Allele but got {result}'


def test_HLA_B18_02():
    result = parse("HLA-B18:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:02") to be Allele but got {result}'


def test_HLA_B18_03():
    result = parse("HLA-B18:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:03") to be Allele but got {result}'


def test_HLA_B18_04():
    result = parse("HLA-B18:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:04") to be Allele but got {result}'


def test_HLA_B18_05():
    result = parse("HLA-B18:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:05") to be Allele but got {result}'


def test_HLA_B18_06():
    result = parse("HLA-B18:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:06") to be Allele but got {result}'


def test_HLA_B18_07():
    result = parse("HLA-B18:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:07") to be Allele but got {result}'


def test_HLA_B18_08():
    result = parse("HLA-B18:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:08") to be Allele but got {result}'


def test_HLA_B18_09():
    result = parse("HLA-B18:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:09") to be Allele but got {result}'


def test_HLA_B18_10():
    result = parse("HLA-B18:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:10") to be Allele but got {result}'


def test_HLA_B18_11():
    result = parse("HLA-B18:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:11") to be Allele but got {result}'


def test_HLA_B18_12():
    result = parse("HLA-B18:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:12") to be Allele but got {result}'


def test_HLA_B18_13():
    result = parse("HLA-B18:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:13") to be Allele but got {result}'


def test_HLA_B18_14():
    result = parse("HLA-B18:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:14") to be Allele but got {result}'


def test_HLA_B18_15():
    result = parse("HLA-B18:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:15") to be Allele but got {result}'


def test_HLA_B18_18():
    result = parse("HLA-B18:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:18") to be Allele but got {result}'


def test_HLA_B18_19():
    result = parse("HLA-B18:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:19") to be Allele but got {result}'


def test_HLA_B18_20():
    result = parse("HLA-B18:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:20") to be Allele but got {result}'


def test_HLA_B18_21():
    result = parse("HLA-B18:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:21") to be Allele but got {result}'


def test_HLA_B18_22():
    result = parse("HLA-B18:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:22") to be Allele but got {result}'


def test_HLA_B18_24():
    result = parse("HLA-B18:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:24") to be Allele but got {result}'


def test_HLA_B18_25():
    result = parse("HLA-B18:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:25") to be Allele but got {result}'


def test_HLA_B18_26():
    result = parse("HLA-B18:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:26") to be Allele but got {result}'


def test_HLA_B18_27():
    result = parse("HLA-B18:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:27") to be Allele but got {result}'


def test_HLA_B18_28():
    result = parse("HLA-B18:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:28") to be Allele but got {result}'


def test_HLA_B18_29():
    result = parse("HLA-B18:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:29") to be Allele but got {result}'


def test_HLA_B18_30():
    result = parse("HLA-B18:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:30") to be Allele but got {result}'


def test_HLA_B18_31():
    result = parse("HLA-B18:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:31") to be Allele but got {result}'


def test_HLA_B18_32():
    result = parse("HLA-B18:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:32") to be Allele but got {result}'


def test_HLA_B18_33():
    result = parse("HLA-B18:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:33") to be Allele but got {result}'


def test_HLA_B18_34():
    result = parse("HLA-B18:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:34") to be Allele but got {result}'


def test_HLA_B18_35():
    result = parse("HLA-B18:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:35") to be Allele but got {result}'


def test_HLA_B18_36():
    result = parse("HLA-B18:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:36") to be Allele but got {result}'


def test_HLA_B18_37():
    result = parse("HLA-B18:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:37") to be Allele but got {result}'


def test_HLA_B18_38():
    result = parse("HLA-B18:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:38") to be Allele but got {result}'


def test_HLA_B18_39():
    result = parse("HLA-B18:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:39") to be Allele but got {result}'


def test_HLA_B18_40():
    result = parse("HLA-B18:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:40") to be Allele but got {result}'


def test_HLA_B18_41():
    result = parse("HLA-B18:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:41") to be Allele but got {result}'


def test_HLA_B18_42():
    result = parse("HLA-B18:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:42") to be Allele but got {result}'


def test_HLA_B18_43():
    result = parse("HLA-B18:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:43") to be Allele but got {result}'


def test_HLA_B18_44():
    result = parse("HLA-B18:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:44") to be Allele but got {result}'


def test_HLA_B18_45():
    result = parse("HLA-B18:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:45") to be Allele but got {result}'


def test_HLA_B18_46():
    result = parse("HLA-B18:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:46") to be Allele but got {result}'


def test_HLA_B18_47():
    result = parse("HLA-B18:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:47") to be Allele but got {result}'


def test_HLA_B18_48():
    result = parse("HLA-B18:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:48") to be Allele but got {result}'


def test_HLA_B18_49():
    result = parse("HLA-B18:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:49") to be Allele but got {result}'


def test_HLA_B18_50():
    result = parse("HLA-B18:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B18:50") to be Allele but got {result}'


def test_HLA_B27_01():
    result = parse("HLA-B27:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:01") to be Allele but got {result}'


def test_HLA_B27_02():
    result = parse("HLA-B27:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:02") to be Allele but got {result}'


def test_HLA_B27_03():
    result = parse("HLA-B27:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:03") to be Allele but got {result}'


def test_HLA_B27_04():
    result = parse("HLA-B27:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:04") to be Allele but got {result}'


def test_HLA_B27_05():
    result = parse("HLA-B27:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:05") to be Allele but got {result}'


def test_HLA_B27_06():
    result = parse("HLA-B27:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:06") to be Allele but got {result}'


def test_HLA_B27_07():
    result = parse("HLA-B27:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:07") to be Allele but got {result}'


def test_HLA_B27_08():
    result = parse("HLA-B27:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:08") to be Allele but got {result}'


def test_HLA_B27_09():
    result = parse("HLA-B27:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:09") to be Allele but got {result}'


def test_HLA_B27_10():
    result = parse("HLA-B27:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:10") to be Allele but got {result}'


def test_HLA_B27_11():
    result = parse("HLA-B27:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:11") to be Allele but got {result}'


def test_HLA_B27_12():
    result = parse("HLA-B27:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:12") to be Allele but got {result}'


def test_HLA_B27_13():
    result = parse("HLA-B27:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:13") to be Allele but got {result}'


def test_HLA_B27_14():
    result = parse("HLA-B27:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:14") to be Allele but got {result}'


def test_HLA_B27_15():
    result = parse("HLA-B27:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:15") to be Allele but got {result}'


def test_HLA_B27_16():
    result = parse("HLA-B27:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:16") to be Allele but got {result}'


def test_HLA_B27_17():
    result = parse("HLA-B27:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:17") to be Allele but got {result}'


def test_HLA_B27_18():
    result = parse("HLA-B27:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:18") to be Allele but got {result}'


def test_HLA_B27_19():
    result = parse("HLA-B27:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:19") to be Allele but got {result}'


def test_HLA_B27_20():
    result = parse("HLA-B27:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:20") to be Allele but got {result}'


def test_HLA_B27_21():
    result = parse("HLA-B27:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:21") to be Allele but got {result}'


def test_HLA_B27_23():
    result = parse("HLA-B27:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:23") to be Allele but got {result}'


def test_HLA_B27_24():
    result = parse("HLA-B27:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:24") to be Allele but got {result}'


def test_HLA_B27_25():
    result = parse("HLA-B27:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:25") to be Allele but got {result}'


def test_HLA_B27_26():
    result = parse("HLA-B27:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:26") to be Allele but got {result}'


def test_HLA_B27_27():
    result = parse("HLA-B27:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:27") to be Allele but got {result}'


def test_HLA_B27_28():
    result = parse("HLA-B27:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:28") to be Allele but got {result}'


def test_HLA_B27_29():
    result = parse("HLA-B27:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:29") to be Allele but got {result}'


def test_HLA_B27_30():
    result = parse("HLA-B27:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:30") to be Allele but got {result}'


def test_HLA_B27_31():
    result = parse("HLA-B27:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:31") to be Allele but got {result}'


def test_HLA_B27_32():
    result = parse("HLA-B27:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:32") to be Allele but got {result}'


def test_HLA_B27_33():
    result = parse("HLA-B27:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:33") to be Allele but got {result}'


def test_HLA_B27_34():
    result = parse("HLA-B27:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:34") to be Allele but got {result}'


def test_HLA_B27_35():
    result = parse("HLA-B27:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:35") to be Allele but got {result}'


def test_HLA_B27_36():
    result = parse("HLA-B27:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:36") to be Allele but got {result}'


def test_HLA_B27_37():
    result = parse("HLA-B27:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:37") to be Allele but got {result}'


def test_HLA_B27_38():
    result = parse("HLA-B27:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:38") to be Allele but got {result}'


def test_HLA_B27_39():
    result = parse("HLA-B27:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:39") to be Allele but got {result}'


def test_HLA_B27_40():
    result = parse("HLA-B27:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:40") to be Allele but got {result}'


def test_HLA_B27_41():
    result = parse("HLA-B27:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:41") to be Allele but got {result}'


def test_HLA_B27_42():
    result = parse("HLA-B27:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:42") to be Allele but got {result}'


def test_HLA_B27_43():
    result = parse("HLA-B27:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:43") to be Allele but got {result}'


def test_HLA_B27_44():
    result = parse("HLA-B27:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:44") to be Allele but got {result}'


def test_HLA_B27_45():
    result = parse("HLA-B27:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:45") to be Allele but got {result}'


def test_HLA_B27_46():
    result = parse("HLA-B27:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:46") to be Allele but got {result}'


def test_HLA_B27_47():
    result = parse("HLA-B27:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:47") to be Allele but got {result}'


def test_HLA_B27_48():
    result = parse("HLA-B27:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:48") to be Allele but got {result}'


def test_HLA_B27_49():
    result = parse("HLA-B27:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:49") to be Allele but got {result}'


def test_HLA_B27_50():
    result = parse("HLA-B27:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:50") to be Allele but got {result}'


def test_HLA_B27_51():
    result = parse("HLA-B27:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:51") to be Allele but got {result}'


def test_HLA_B27_52():
    result = parse("HLA-B27:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:52") to be Allele but got {result}'


def test_HLA_B27_53():
    result = parse("HLA-B27:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:53") to be Allele but got {result}'


def test_HLA_B27_54():
    result = parse("HLA-B27:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:54") to be Allele but got {result}'


def test_HLA_B27_55():
    result = parse("HLA-B27:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:55") to be Allele but got {result}'


def test_HLA_B27_56():
    result = parse("HLA-B27:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:56") to be Allele but got {result}'


def test_HLA_B27_57():
    result = parse("HLA-B27:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:57") to be Allele but got {result}'


def test_HLA_B27_58():
    result = parse("HLA-B27:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:58") to be Allele but got {result}'


def test_HLA_B27_60():
    result = parse("HLA-B27:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:60") to be Allele but got {result}'


def test_HLA_B27_61():
    result = parse("HLA-B27:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:61") to be Allele but got {result}'


def test_HLA_B27_62():
    result = parse("HLA-B27:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:62") to be Allele but got {result}'


def test_HLA_B27_63():
    result = parse("HLA-B27:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:63") to be Allele but got {result}'


def test_HLA_B27_67():
    result = parse("HLA-B27:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:67") to be Allele but got {result}'


def test_HLA_B27_68():
    result = parse("HLA-B27:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:68") to be Allele but got {result}'


def test_HLA_B27_69():
    result = parse("HLA-B27:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-B27:69") to be Allele but got {result}'


def test_HLA_B35_01():
    result = parse("HLA-B35:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:01") to be Allele but got {result}'


def test_HLA_B35_02():
    result = parse("HLA-B35:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:02") to be Allele but got {result}'


def test_HLA_B35_03():
    result = parse("HLA-B35:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:03") to be Allele but got {result}'


def test_HLA_B35_04():
    result = parse("HLA-B35:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:04") to be Allele but got {result}'


def test_HLA_B35_05():
    result = parse("HLA-B35:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:05") to be Allele but got {result}'


def test_HLA_B35_06():
    result = parse("HLA-B35:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:06") to be Allele but got {result}'


def test_HLA_B35_07():
    result = parse("HLA-B35:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:07") to be Allele but got {result}'


def test_HLA_B35_08():
    result = parse("HLA-B35:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:08") to be Allele but got {result}'


def test_HLA_B35_09():
    result = parse("HLA-B35:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:09") to be Allele but got {result}'


def test_HLA_B35_10():
    result = parse("HLA-B35:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:10") to be Allele but got {result}'


def test_HLA_B35_11():
    result = parse("HLA-B35:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:11") to be Allele but got {result}'


def test_HLA_B35_12():
    result = parse("HLA-B35:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:12") to be Allele but got {result}'


def test_HLA_B35_13():
    result = parse("HLA-B35:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:13") to be Allele but got {result}'


def test_HLA_B35_14():
    result = parse("HLA-B35:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:14") to be Allele but got {result}'


def test_HLA_B35_15():
    result = parse("HLA-B35:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:15") to be Allele but got {result}'


def test_HLA_B35_16():
    result = parse("HLA-B35:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:16") to be Allele but got {result}'


def test_HLA_B35_17():
    result = parse("HLA-B35:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:17") to be Allele but got {result}'


def test_HLA_B35_18():
    result = parse("HLA-B35:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:18") to be Allele but got {result}'


def test_HLA_B35_19():
    result = parse("HLA-B35:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:19") to be Allele but got {result}'


def test_HLA_B35_20():
    result = parse("HLA-B35:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:20") to be Allele but got {result}'


def test_HLA_B35_21():
    result = parse("HLA-B35:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:21") to be Allele but got {result}'


def test_HLA_B35_22():
    result = parse("HLA-B35:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:22") to be Allele but got {result}'


def test_HLA_B35_23():
    result = parse("HLA-B35:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:23") to be Allele but got {result}'


def test_HLA_B35_24():
    result = parse("HLA-B35:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:24") to be Allele but got {result}'


def test_HLA_B35_25():
    result = parse("HLA-B35:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:25") to be Allele but got {result}'


def test_HLA_B35_26():
    result = parse("HLA-B35:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:26") to be Allele but got {result}'


def test_HLA_B35_27():
    result = parse("HLA-B35:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:27") to be Allele but got {result}'


def test_HLA_B35_28():
    result = parse("HLA-B35:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:28") to be Allele but got {result}'


def test_HLA_B35_29():
    result = parse("HLA-B35:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:29") to be Allele but got {result}'


def test_HLA_B35_30():
    result = parse("HLA-B35:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:30") to be Allele but got {result}'


def test_HLA_B35_31():
    result = parse("HLA-B35:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:31") to be Allele but got {result}'


def test_HLA_B35_32():
    result = parse("HLA-B35:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:32") to be Allele but got {result}'


def test_HLA_B35_33():
    result = parse("HLA-B35:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:33") to be Allele but got {result}'


def test_HLA_B35_34():
    result = parse("HLA-B35:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:34") to be Allele but got {result}'


def test_HLA_B35_35():
    result = parse("HLA-B35:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:35") to be Allele but got {result}'


def test_HLA_B35_36():
    result = parse("HLA-B35:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:36") to be Allele but got {result}'


def test_HLA_B35_37():
    result = parse("HLA-B35:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:37") to be Allele but got {result}'


def test_HLA_B35_38():
    result = parse("HLA-B35:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:38") to be Allele but got {result}'


def test_HLA_B35_39():
    result = parse("HLA-B35:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:39") to be Allele but got {result}'


def test_HLA_B35_41():
    result = parse("HLA-B35:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:41") to be Allele but got {result}'


def test_HLA_B35_42():
    result = parse("HLA-B35:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:42") to be Allele but got {result}'


def test_HLA_B35_43():
    result = parse("HLA-B35:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:43") to be Allele but got {result}'


def test_HLA_B35_44():
    result = parse("HLA-B35:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:44") to be Allele but got {result}'


def test_HLA_B35_45():
    result = parse("HLA-B35:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:45") to be Allele but got {result}'


def test_HLA_B35_46():
    result = parse("HLA-B35:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:46") to be Allele but got {result}'


def test_HLA_B35_47():
    result = parse("HLA-B35:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:47") to be Allele but got {result}'


def test_HLA_B35_48():
    result = parse("HLA-B35:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:48") to be Allele but got {result}'


def test_HLA_B35_49():
    result = parse("HLA-B35:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:49") to be Allele but got {result}'


def test_HLA_B35_50():
    result = parse("HLA-B35:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:50") to be Allele but got {result}'


def test_HLA_B35_51():
    result = parse("HLA-B35:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:51") to be Allele but got {result}'


def test_HLA_B35_52():
    result = parse("HLA-B35:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:52") to be Allele but got {result}'


def test_HLA_B35_54():
    result = parse("HLA-B35:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:54") to be Allele but got {result}'


def test_HLA_B35_55():
    result = parse("HLA-B35:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:55") to be Allele but got {result}'


def test_HLA_B35_56():
    result = parse("HLA-B35:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:56") to be Allele but got {result}'


def test_HLA_B35_57():
    result = parse("HLA-B35:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:57") to be Allele but got {result}'


def test_HLA_B35_58():
    result = parse("HLA-B35:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:58") to be Allele but got {result}'


def test_HLA_B35_59():
    result = parse("HLA-B35:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:59") to be Allele but got {result}'


def test_HLA_B35_60():
    result = parse("HLA-B35:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:60") to be Allele but got {result}'


def test_HLA_B35_61():
    result = parse("HLA-B35:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:61") to be Allele but got {result}'


def test_HLA_B35_62():
    result = parse("HLA-B35:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:62") to be Allele but got {result}'


def test_HLA_B35_63():
    result = parse("HLA-B35:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:63") to be Allele but got {result}'


def test_HLA_B35_64():
    result = parse("HLA-B35:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:64") to be Allele but got {result}'


def test_HLA_B35_66():
    result = parse("HLA-B35:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:66") to be Allele but got {result}'


def test_HLA_B35_67():
    result = parse("HLA-B35:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:67") to be Allele but got {result}'


def test_HLA_B35_68():
    result = parse("HLA-B35:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:68") to be Allele but got {result}'


def test_HLA_B35_69():
    result = parse("HLA-B35:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:69") to be Allele but got {result}'


def test_HLA_B35_70():
    result = parse("HLA-B35:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:70") to be Allele but got {result}'


def test_HLA_B35_71():
    result = parse("HLA-B35:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:71") to be Allele but got {result}'


def test_HLA_B35_72():
    result = parse("HLA-B35:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:72") to be Allele but got {result}'


def test_HLA_B35_74():
    result = parse("HLA-B35:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:74") to be Allele but got {result}'


def test_HLA_B35_75():
    result = parse("HLA-B35:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:75") to be Allele but got {result}'


def test_HLA_B35_76():
    result = parse("HLA-B35:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:76") to be Allele but got {result}'


def test_HLA_B35_77():
    result = parse("HLA-B35:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:77") to be Allele but got {result}'


def test_HLA_B35_78():
    result = parse("HLA-B35:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:78") to be Allele but got {result}'


def test_HLA_B35_79():
    result = parse("HLA-B35:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:79") to be Allele but got {result}'


def test_HLA_B35_80():
    result = parse("HLA-B35:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:80") to be Allele but got {result}'


def test_HLA_B35_81():
    result = parse("HLA-B35:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:81") to be Allele but got {result}'


def test_HLA_B35_82():
    result = parse("HLA-B35:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:82") to be Allele but got {result}'


def test_HLA_B35_83():
    result = parse("HLA-B35:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:83") to be Allele but got {result}'


def test_HLA_B35_84():
    result = parse("HLA-B35:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:84") to be Allele but got {result}'


def test_HLA_B35_85():
    result = parse("HLA-B35:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:85") to be Allele but got {result}'


def test_HLA_B35_86():
    result = parse("HLA-B35:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:86") to be Allele but got {result}'


def test_HLA_B35_87():
    result = parse("HLA-B35:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:87") to be Allele but got {result}'


def test_HLA_B35_88():
    result = parse("HLA-B35:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:88") to be Allele but got {result}'


def test_HLA_B35_89():
    result = parse("HLA-B35:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:89") to be Allele but got {result}'


def test_HLA_B35_90():
    result = parse("HLA-B35:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:90") to be Allele but got {result}'


def test_HLA_B35_91():
    result = parse("HLA-B35:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:91") to be Allele but got {result}'


def test_HLA_B35_92():
    result = parse("HLA-B35:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:92") to be Allele but got {result}'


def test_HLA_B35_93():
    result = parse("HLA-B35:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:93") to be Allele but got {result}'


def test_HLA_B35_94():
    result = parse("HLA-B35:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:94") to be Allele but got {result}'


def test_HLA_B35_95():
    result = parse("HLA-B35:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:95") to be Allele but got {result}'


def test_HLA_B35_96():
    result = parse("HLA-B35:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:96") to be Allele but got {result}'


def test_HLA_B35_97():
    result = parse("HLA-B35:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:97") to be Allele but got {result}'


def test_HLA_B35_98():
    result = parse("HLA-B35:98")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:98") to be Allele but got {result}'


def test_HLA_B35_99():
    result = parse("HLA-B35:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-B35:99") to be Allele but got {result}'


def test_HLA_B35_100():
    result = parse("HLA-B35:100")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:100") to be Allele but got {result}'
    )


def test_HLA_B35_101():
    result = parse("HLA-B35:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:101") to be Allele but got {result}'
    )


def test_HLA_B35_102():
    result = parse("HLA-B35:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:102") to be Allele but got {result}'
    )


def test_HLA_B35_103():
    result = parse("HLA-B35:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:103") to be Allele but got {result}'
    )


def test_HLA_B35_104():
    result = parse("HLA-B35:104")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:104") to be Allele but got {result}'
    )


def test_HLA_B35_105():
    result = parse("HLA-B35:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:105") to be Allele but got {result}'
    )


def test_HLA_B35_106():
    result = parse("HLA-B35:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:106") to be Allele but got {result}'
    )


def test_HLA_B35_107():
    result = parse("HLA-B35:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:107") to be Allele but got {result}'
    )


def test_HLA_B35_108():
    result = parse("HLA-B35:108")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:108") to be Allele but got {result}'
    )


def test_HLA_B35_109():
    result = parse("HLA-B35:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:109") to be Allele but got {result}'
    )


def test_HLA_B35_110():
    result = parse("HLA-B35:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:110") to be Allele but got {result}'
    )


def test_HLA_B35_111():
    result = parse("HLA-B35:111")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:111") to be Allele but got {result}'
    )


def test_HLA_B35_112():
    result = parse("HLA-B35:112")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:112") to be Allele but got {result}'
    )


def test_HLA_B35_113():
    result = parse("HLA-B35:113")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:113") to be Allele but got {result}'
    )


def test_HLA_B35_114():
    result = parse("HLA-B35:114")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:114") to be Allele but got {result}'
    )


def test_HLA_B35_115():
    result = parse("HLA-B35:115")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:115") to be Allele but got {result}'
    )


def test_HLA_B35_116():
    result = parse("HLA-B35:116")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:116") to be Allele but got {result}'
    )


def test_HLA_B35_117():
    result = parse("HLA-B35:117")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:117") to be Allele but got {result}'
    )


def test_HLA_B35_118():
    result = parse("HLA-B35:118")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:118") to be Allele but got {result}'
    )


def test_HLA_B35_119():
    result = parse("HLA-B35:119")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:119") to be Allele but got {result}'
    )


def test_HLA_B35_120():
    result = parse("HLA-B35:120")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:120") to be Allele but got {result}'
    )


def test_HLA_B35_121():
    result = parse("HLA-B35:121")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:121") to be Allele but got {result}'
    )


def test_HLA_B35_122():
    result = parse("HLA-B35:122")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:122") to be Allele but got {result}'
    )


def test_HLA_B35_123():
    result = parse("HLA-B35:123")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:123") to be Allele but got {result}'
    )


def test_HLA_B35_124():
    result = parse("HLA-B35:124")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:124") to be Allele but got {result}'
    )


def test_HLA_B35_125():
    result = parse("HLA-B35:125")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:125") to be Allele but got {result}'
    )


def test_HLA_B35_126():
    result = parse("HLA-B35:126")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:126") to be Allele but got {result}'
    )


def test_HLA_B35_127():
    result = parse("HLA-B35:127")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:127") to be Allele but got {result}'
    )


def test_HLA_B35_128():
    result = parse("HLA-B35:128")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:128") to be Allele but got {result}'
    )


def test_HLA_B35_131():
    result = parse("HLA-B35:131")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:131") to be Allele but got {result}'
    )


def test_HLA_B35_132():
    result = parse("HLA-B35:132")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:132") to be Allele but got {result}'
    )


def test_HLA_B35_133():
    result = parse("HLA-B35:133")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:133") to be Allele but got {result}'
    )


def test_HLA_B35_135():
    result = parse("HLA-B35:135")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:135") to be Allele but got {result}'
    )


def test_HLA_B35_136():
    result = parse("HLA-B35:136")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:136") to be Allele but got {result}'
    )


def test_HLA_B35_137():
    result = parse("HLA-B35:137")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:137") to be Allele but got {result}'
    )


def test_HLA_B35_138():
    result = parse("HLA-B35:138")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:138") to be Allele but got {result}'
    )


def test_HLA_B35_139():
    result = parse("HLA-B35:139")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:139") to be Allele but got {result}'
    )


def test_HLA_B35_140():
    result = parse("HLA-B35:140")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:140") to be Allele but got {result}'
    )


def test_HLA_B35_141():
    result = parse("HLA-B35:141")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:141") to be Allele but got {result}'
    )


def test_HLA_B35_142():
    result = parse("HLA-B35:142")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:142") to be Allele but got {result}'
    )


def test_HLA_B35_143():
    result = parse("HLA-B35:143")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:143") to be Allele but got {result}'
    )


def test_HLA_B35_144():
    result = parse("HLA-B35:144")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B35:144") to be Allele but got {result}'
    )


def test_HLA_B37_01():
    result = parse("HLA-B37:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:01") to be Allele but got {result}'


def test_HLA_B37_02():
    result = parse("HLA-B37:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:02") to be Allele but got {result}'


def test_HLA_B37_04():
    result = parse("HLA-B37:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:04") to be Allele but got {result}'


def test_HLA_B37_05():
    result = parse("HLA-B37:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:05") to be Allele but got {result}'


def test_HLA_B37_06():
    result = parse("HLA-B37:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:06") to be Allele but got {result}'


def test_HLA_B37_07():
    result = parse("HLA-B37:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:07") to be Allele but got {result}'


def test_HLA_B37_08():
    result = parse("HLA-B37:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:08") to be Allele but got {result}'


def test_HLA_B37_09():
    result = parse("HLA-B37:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:09") to be Allele but got {result}'


def test_HLA_B37_10():
    result = parse("HLA-B37:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:10") to be Allele but got {result}'


def test_HLA_B37_11():
    result = parse("HLA-B37:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:11") to be Allele but got {result}'


def test_HLA_B37_12():
    result = parse("HLA-B37:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:12") to be Allele but got {result}'


def test_HLA_B37_13():
    result = parse("HLA-B37:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:13") to be Allele but got {result}'


def test_HLA_B37_14():
    result = parse("HLA-B37:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:14") to be Allele but got {result}'


def test_HLA_B37_15():
    result = parse("HLA-B37:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:15") to be Allele but got {result}'


def test_HLA_B37_17():
    result = parse("HLA-B37:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:17") to be Allele but got {result}'


def test_HLA_B37_18():
    result = parse("HLA-B37:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:18") to be Allele but got {result}'


def test_HLA_B37_19():
    result = parse("HLA-B37:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:19") to be Allele but got {result}'


def test_HLA_B37_20():
    result = parse("HLA-B37:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:20") to be Allele but got {result}'


def test_HLA_B37_21():
    result = parse("HLA-B37:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:21") to be Allele but got {result}'


def test_HLA_B37_22():
    result = parse("HLA-B37:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:22") to be Allele but got {result}'


def test_HLA_B37_23():
    result = parse("HLA-B37:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B37:23") to be Allele but got {result}'


def test_HLA_B38_01():
    result = parse("HLA-B38:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:01") to be Allele but got {result}'


def test_HLA_B38_02():
    result = parse("HLA-B38:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:02") to be Allele but got {result}'


def test_HLA_B38_03():
    result = parse("HLA-B38:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:03") to be Allele but got {result}'


def test_HLA_B38_04():
    result = parse("HLA-B38:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:04") to be Allele but got {result}'


def test_HLA_B38_05():
    result = parse("HLA-B38:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:05") to be Allele but got {result}'


def test_HLA_B38_06():
    result = parse("HLA-B38:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:06") to be Allele but got {result}'


def test_HLA_B38_07():
    result = parse("HLA-B38:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:07") to be Allele but got {result}'


def test_HLA_B38_08():
    result = parse("HLA-B38:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:08") to be Allele but got {result}'


def test_HLA_B38_09():
    result = parse("HLA-B38:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:09") to be Allele but got {result}'


def test_HLA_B38_10():
    result = parse("HLA-B38:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:10") to be Allele but got {result}'


def test_HLA_B38_11():
    result = parse("HLA-B38:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:11") to be Allele but got {result}'


def test_HLA_B38_12():
    result = parse("HLA-B38:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:12") to be Allele but got {result}'


def test_HLA_B38_13():
    result = parse("HLA-B38:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:13") to be Allele but got {result}'


def test_HLA_B38_14():
    result = parse("HLA-B38:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:14") to be Allele but got {result}'


def test_HLA_B38_15():
    result = parse("HLA-B38:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:15") to be Allele but got {result}'


def test_HLA_B38_16():
    result = parse("HLA-B38:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:16") to be Allele but got {result}'


def test_HLA_B38_17():
    result = parse("HLA-B38:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:17") to be Allele but got {result}'


def test_HLA_B38_18():
    result = parse("HLA-B38:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:18") to be Allele but got {result}'


def test_HLA_B38_19():
    result = parse("HLA-B38:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:19") to be Allele but got {result}'


def test_HLA_B38_20():
    result = parse("HLA-B38:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:20") to be Allele but got {result}'


def test_HLA_B38_21():
    result = parse("HLA-B38:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:21") to be Allele but got {result}'


def test_HLA_B38_22():
    result = parse("HLA-B38:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:22") to be Allele but got {result}'


def test_HLA_B38_23():
    result = parse("HLA-B38:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B38:23") to be Allele but got {result}'


def test_HLA_B39_01():
    result = parse("HLA-B39:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:01") to be Allele but got {result}'


def test_HLA_B39_02():
    result = parse("HLA-B39:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:02") to be Allele but got {result}'


def test_HLA_B39_03():
    result = parse("HLA-B39:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:03") to be Allele but got {result}'


def test_HLA_B39_04():
    result = parse("HLA-B39:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:04") to be Allele but got {result}'


def test_HLA_B39_05():
    result = parse("HLA-B39:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:05") to be Allele but got {result}'


def test_HLA_B39_06():
    result = parse("HLA-B39:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:06") to be Allele but got {result}'


def test_HLA_B39_07():
    result = parse("HLA-B39:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:07") to be Allele but got {result}'


def test_HLA_B39_08():
    result = parse("HLA-B39:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:08") to be Allele but got {result}'


def test_HLA_B39_09():
    result = parse("HLA-B39:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:09") to be Allele but got {result}'


def test_HLA_B39_10():
    result = parse("HLA-B39:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:10") to be Allele but got {result}'


def test_HLA_B39_11():
    result = parse("HLA-B39:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:11") to be Allele but got {result}'


def test_HLA_B39_12():
    result = parse("HLA-B39:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:12") to be Allele but got {result}'


def test_HLA_B39_13():
    result = parse("HLA-B39:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:13") to be Allele but got {result}'


def test_HLA_B39_14():
    result = parse("HLA-B39:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:14") to be Allele but got {result}'


def test_HLA_B39_15():
    result = parse("HLA-B39:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:15") to be Allele but got {result}'


def test_HLA_B39_16():
    result = parse("HLA-B39:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:16") to be Allele but got {result}'


def test_HLA_B39_17():
    result = parse("HLA-B39:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:17") to be Allele but got {result}'


def test_HLA_B39_18():
    result = parse("HLA-B39:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:18") to be Allele but got {result}'


def test_HLA_B39_19():
    result = parse("HLA-B39:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:19") to be Allele but got {result}'


def test_HLA_B39_20():
    result = parse("HLA-B39:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:20") to be Allele but got {result}'


def test_HLA_B39_22():
    result = parse("HLA-B39:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:22") to be Allele but got {result}'


def test_HLA_B39_23():
    result = parse("HLA-B39:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:23") to be Allele but got {result}'


def test_HLA_B39_24():
    result = parse("HLA-B39:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:24") to be Allele but got {result}'


def test_HLA_B39_26():
    result = parse("HLA-B39:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:26") to be Allele but got {result}'


def test_HLA_B39_27():
    result = parse("HLA-B39:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:27") to be Allele but got {result}'


def test_HLA_B39_28():
    result = parse("HLA-B39:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:28") to be Allele but got {result}'


def test_HLA_B39_29():
    result = parse("HLA-B39:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:29") to be Allele but got {result}'


def test_HLA_B39_30():
    result = parse("HLA-B39:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:30") to be Allele but got {result}'


def test_HLA_B39_31():
    result = parse("HLA-B39:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:31") to be Allele but got {result}'


def test_HLA_B39_32():
    result = parse("HLA-B39:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:32") to be Allele but got {result}'


def test_HLA_B39_33():
    result = parse("HLA-B39:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:33") to be Allele but got {result}'


def test_HLA_B39_34():
    result = parse("HLA-B39:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:34") to be Allele but got {result}'


def test_HLA_B39_35():
    result = parse("HLA-B39:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:35") to be Allele but got {result}'


def test_HLA_B39_36():
    result = parse("HLA-B39:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:36") to be Allele but got {result}'


def test_HLA_B39_37():
    result = parse("HLA-B39:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:37") to be Allele but got {result}'


def test_HLA_B39_39():
    result = parse("HLA-B39:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:39") to be Allele but got {result}'


def test_HLA_B39_41():
    result = parse("HLA-B39:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:41") to be Allele but got {result}'


def test_HLA_B39_42():
    result = parse("HLA-B39:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:42") to be Allele but got {result}'


def test_HLA_B39_43():
    result = parse("HLA-B39:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:43") to be Allele but got {result}'


def test_HLA_B39_44():
    result = parse("HLA-B39:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:44") to be Allele but got {result}'


def test_HLA_B39_45():
    result = parse("HLA-B39:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:45") to be Allele but got {result}'


def test_HLA_B39_46():
    result = parse("HLA-B39:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:46") to be Allele but got {result}'


def test_HLA_B39_47():
    result = parse("HLA-B39:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:47") to be Allele but got {result}'


def test_HLA_B39_48():
    result = parse("HLA-B39:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:48") to be Allele but got {result}'


def test_HLA_B39_49():
    result = parse("HLA-B39:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:49") to be Allele but got {result}'


def test_HLA_B39_50():
    result = parse("HLA-B39:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:50") to be Allele but got {result}'


def test_HLA_B39_51():
    result = parse("HLA-B39:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:51") to be Allele but got {result}'


def test_HLA_B39_52():
    result = parse("HLA-B39:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:52") to be Allele but got {result}'


def test_HLA_B39_53():
    result = parse("HLA-B39:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:53") to be Allele but got {result}'


def test_HLA_B39_54():
    result = parse("HLA-B39:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:54") to be Allele but got {result}'


def test_HLA_B39_55():
    result = parse("HLA-B39:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:55") to be Allele but got {result}'


def test_HLA_B39_56():
    result = parse("HLA-B39:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:56") to be Allele but got {result}'


def test_HLA_B39_57():
    result = parse("HLA-B39:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:57") to be Allele but got {result}'


def test_HLA_B39_58():
    result = parse("HLA-B39:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:58") to be Allele but got {result}'


def test_HLA_B39_59():
    result = parse("HLA-B39:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:59") to be Allele but got {result}'


def test_HLA_B39_60():
    result = parse("HLA-B39:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B39:60") to be Allele but got {result}'


def test_HLA_B40_01():
    result = parse("HLA-B40:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:01") to be Allele but got {result}'


def test_HLA_B40_02():
    result = parse("HLA-B40:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:02") to be Allele but got {result}'


def test_HLA_B40_03():
    result = parse("HLA-B40:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:03") to be Allele but got {result}'


def test_HLA_B40_04():
    result = parse("HLA-B40:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:04") to be Allele but got {result}'


def test_HLA_B40_05():
    result = parse("HLA-B40:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:05") to be Allele but got {result}'


def test_HLA_B40_06():
    result = parse("HLA-B40:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:06") to be Allele but got {result}'


def test_HLA_B40_07():
    result = parse("HLA-B40:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:07") to be Allele but got {result}'


def test_HLA_B40_08():
    result = parse("HLA-B40:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:08") to be Allele but got {result}'


def test_HLA_B40_09():
    result = parse("HLA-B40:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:09") to be Allele but got {result}'


def test_HLA_B40_10():
    result = parse("HLA-B40:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:10") to be Allele but got {result}'


def test_HLA_B40_11():
    result = parse("HLA-B40:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:11") to be Allele but got {result}'


def test_HLA_B40_12():
    result = parse("HLA-B40:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:12") to be Allele but got {result}'


def test_HLA_B40_13():
    result = parse("HLA-B40:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:13") to be Allele but got {result}'


def test_HLA_B40_14():
    result = parse("HLA-B40:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:14") to be Allele but got {result}'


def test_HLA_B40_15():
    result = parse("HLA-B40:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:15") to be Allele but got {result}'


def test_HLA_B40_16():
    result = parse("HLA-B40:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:16") to be Allele but got {result}'


def test_HLA_B40_18():
    result = parse("HLA-B40:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:18") to be Allele but got {result}'


def test_HLA_B40_19():
    result = parse("HLA-B40:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:19") to be Allele but got {result}'


def test_HLA_B40_20():
    result = parse("HLA-B40:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:20") to be Allele but got {result}'


def test_HLA_B40_21():
    result = parse("HLA-B40:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:21") to be Allele but got {result}'


def test_HLA_B40_23():
    result = parse("HLA-B40:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:23") to be Allele but got {result}'


def test_HLA_B40_24():
    result = parse("HLA-B40:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:24") to be Allele but got {result}'


def test_HLA_B40_25():
    result = parse("HLA-B40:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:25") to be Allele but got {result}'


def test_HLA_B40_26():
    result = parse("HLA-B40:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:26") to be Allele but got {result}'


def test_HLA_B40_27():
    result = parse("HLA-B40:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:27") to be Allele but got {result}'


def test_HLA_B40_28():
    result = parse("HLA-B40:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:28") to be Allele but got {result}'


def test_HLA_B40_29():
    result = parse("HLA-B40:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:29") to be Allele but got {result}'


def test_HLA_B40_30():
    result = parse("HLA-B40:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:30") to be Allele but got {result}'


def test_HLA_B40_31():
    result = parse("HLA-B40:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:31") to be Allele but got {result}'


def test_HLA_B40_32():
    result = parse("HLA-B40:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:32") to be Allele but got {result}'


def test_HLA_B40_33():
    result = parse("HLA-B40:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:33") to be Allele but got {result}'


def test_HLA_B40_34():
    result = parse("HLA-B40:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:34") to be Allele but got {result}'


def test_HLA_B40_35():
    result = parse("HLA-B40:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:35") to be Allele but got {result}'


def test_HLA_B40_36():
    result = parse("HLA-B40:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:36") to be Allele but got {result}'


def test_HLA_B40_37():
    result = parse("HLA-B40:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:37") to be Allele but got {result}'


def test_HLA_B40_38():
    result = parse("HLA-B40:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:38") to be Allele but got {result}'


def test_HLA_B40_39():
    result = parse("HLA-B40:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:39") to be Allele but got {result}'


def test_HLA_B40_40():
    result = parse("HLA-B40:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:40") to be Allele but got {result}'


def test_HLA_B40_42():
    result = parse("HLA-B40:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:42") to be Allele but got {result}'


def test_HLA_B40_43():
    result = parse("HLA-B40:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:43") to be Allele but got {result}'


def test_HLA_B40_44():
    result = parse("HLA-B40:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:44") to be Allele but got {result}'


def test_HLA_B40_45():
    result = parse("HLA-B40:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:45") to be Allele but got {result}'


def test_HLA_B40_46():
    result = parse("HLA-B40:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:46") to be Allele but got {result}'


def test_HLA_B40_47():
    result = parse("HLA-B40:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:47") to be Allele but got {result}'


def test_HLA_B40_48():
    result = parse("HLA-B40:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:48") to be Allele but got {result}'


def test_HLA_B40_49():
    result = parse("HLA-B40:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:49") to be Allele but got {result}'


def test_HLA_B40_50():
    result = parse("HLA-B40:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:50") to be Allele but got {result}'


def test_HLA_B40_51():
    result = parse("HLA-B40:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:51") to be Allele but got {result}'


def test_HLA_B40_52():
    result = parse("HLA-B40:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:52") to be Allele but got {result}'


def test_HLA_B40_53():
    result = parse("HLA-B40:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:53") to be Allele but got {result}'


def test_HLA_B40_54():
    result = parse("HLA-B40:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:54") to be Allele but got {result}'


def test_HLA_B40_55():
    result = parse("HLA-B40:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:55") to be Allele but got {result}'


def test_HLA_B40_56():
    result = parse("HLA-B40:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:56") to be Allele but got {result}'


def test_HLA_B40_57():
    result = parse("HLA-B40:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:57") to be Allele but got {result}'


def test_HLA_B40_58():
    result = parse("HLA-B40:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:58") to be Allele but got {result}'


def test_HLA_B40_59():
    result = parse("HLA-B40:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:59") to be Allele but got {result}'


def test_HLA_B40_60():
    result = parse("HLA-B40:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:60") to be Allele but got {result}'


def test_HLA_B40_61():
    result = parse("HLA-B40:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:61") to be Allele but got {result}'


def test_HLA_B40_62():
    result = parse("HLA-B40:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:62") to be Allele but got {result}'


def test_HLA_B40_63():
    result = parse("HLA-B40:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:63") to be Allele but got {result}'


def test_HLA_B40_64():
    result = parse("HLA-B40:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:64") to be Allele but got {result}'


def test_HLA_B40_65():
    result = parse("HLA-B40:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:65") to be Allele but got {result}'


def test_HLA_B40_66():
    result = parse("HLA-B40:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:66") to be Allele but got {result}'


def test_HLA_B40_67():
    result = parse("HLA-B40:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:67") to be Allele but got {result}'


def test_HLA_B40_68():
    result = parse("HLA-B40:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:68") to be Allele but got {result}'


def test_HLA_B40_69():
    result = parse("HLA-B40:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:69") to be Allele but got {result}'


def test_HLA_B40_70():
    result = parse("HLA-B40:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:70") to be Allele but got {result}'


def test_HLA_B40_71():
    result = parse("HLA-B40:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:71") to be Allele but got {result}'


def test_HLA_B40_72():
    result = parse("HLA-B40:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:72") to be Allele but got {result}'


def test_HLA_B40_73():
    result = parse("HLA-B40:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:73") to be Allele but got {result}'


def test_HLA_B40_74():
    result = parse("HLA-B40:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:74") to be Allele but got {result}'


def test_HLA_B40_75():
    result = parse("HLA-B40:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:75") to be Allele but got {result}'


def test_HLA_B40_76():
    result = parse("HLA-B40:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:76") to be Allele but got {result}'


def test_HLA_B40_77():
    result = parse("HLA-B40:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:77") to be Allele but got {result}'


def test_HLA_B40_78():
    result = parse("HLA-B40:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:78") to be Allele but got {result}'


def test_HLA_B40_79():
    result = parse("HLA-B40:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:79") to be Allele but got {result}'


def test_HLA_B40_80():
    result = parse("HLA-B40:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:80") to be Allele but got {result}'


def test_HLA_B40_81():
    result = parse("HLA-B40:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:81") to be Allele but got {result}'


def test_HLA_B40_82():
    result = parse("HLA-B40:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:82") to be Allele but got {result}'


def test_HLA_B40_83():
    result = parse("HLA-B40:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:83") to be Allele but got {result}'


def test_HLA_B40_84():
    result = parse("HLA-B40:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:84") to be Allele but got {result}'


def test_HLA_B40_85():
    result = parse("HLA-B40:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:85") to be Allele but got {result}'


def test_HLA_B40_86():
    result = parse("HLA-B40:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:86") to be Allele but got {result}'


def test_HLA_B40_87():
    result = parse("HLA-B40:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:87") to be Allele but got {result}'


def test_HLA_B40_88():
    result = parse("HLA-B40:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:88") to be Allele but got {result}'


def test_HLA_B40_89():
    result = parse("HLA-B40:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:89") to be Allele but got {result}'


def test_HLA_B40_90():
    result = parse("HLA-B40:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:90") to be Allele but got {result}'


def test_HLA_B40_91():
    result = parse("HLA-B40:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:91") to be Allele but got {result}'


def test_HLA_B40_92():
    result = parse("HLA-B40:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:92") to be Allele but got {result}'


def test_HLA_B40_93():
    result = parse("HLA-B40:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:93") to be Allele but got {result}'


def test_HLA_B40_94():
    result = parse("HLA-B40:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:94") to be Allele but got {result}'


def test_HLA_B40_95():
    result = parse("HLA-B40:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:95") to be Allele but got {result}'


def test_HLA_B40_96():
    result = parse("HLA-B40:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:96") to be Allele but got {result}'


def test_HLA_B40_97():
    result = parse("HLA-B40:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:97") to be Allele but got {result}'


def test_HLA_B40_98():
    result = parse("HLA-B40:98")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:98") to be Allele but got {result}'


def test_HLA_B40_99():
    result = parse("HLA-B40:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-B40:99") to be Allele but got {result}'


def test_HLA_B40_100():
    result = parse("HLA-B40:100")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:100") to be Allele but got {result}'
    )


def test_HLA_B40_101():
    result = parse("HLA-B40:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:101") to be Allele but got {result}'
    )


def test_HLA_B40_102():
    result = parse("HLA-B40:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:102") to be Allele but got {result}'
    )


def test_HLA_B40_103():
    result = parse("HLA-B40:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:103") to be Allele but got {result}'
    )


def test_HLA_B40_104():
    result = parse("HLA-B40:104")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:104") to be Allele but got {result}'
    )


def test_HLA_B40_105():
    result = parse("HLA-B40:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:105") to be Allele but got {result}'
    )


def test_HLA_B40_106():
    result = parse("HLA-B40:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:106") to be Allele but got {result}'
    )


def test_HLA_B40_107():
    result = parse("HLA-B40:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:107") to be Allele but got {result}'
    )


def test_HLA_B40_108():
    result = parse("HLA-B40:108")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:108") to be Allele but got {result}'
    )


def test_HLA_B40_109():
    result = parse("HLA-B40:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:109") to be Allele but got {result}'
    )


def test_HLA_B40_110():
    result = parse("HLA-B40:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:110") to be Allele but got {result}'
    )


def test_HLA_B40_111():
    result = parse("HLA-B40:111")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:111") to be Allele but got {result}'
    )


def test_HLA_B40_112():
    result = parse("HLA-B40:112")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:112") to be Allele but got {result}'
    )


def test_HLA_B40_113():
    result = parse("HLA-B40:113")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:113") to be Allele but got {result}'
    )


def test_HLA_B40_114():
    result = parse("HLA-B40:114")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:114") to be Allele but got {result}'
    )


def test_HLA_B40_115():
    result = parse("HLA-B40:115")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:115") to be Allele but got {result}'
    )


def test_HLA_B40_116():
    result = parse("HLA-B40:116")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:116") to be Allele but got {result}'
    )


def test_HLA_B40_117():
    result = parse("HLA-B40:117")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:117") to be Allele but got {result}'
    )


def test_HLA_B40_119():
    result = parse("HLA-B40:119")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:119") to be Allele but got {result}'
    )


def test_HLA_B40_120():
    result = parse("HLA-B40:120")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:120") to be Allele but got {result}'
    )


def test_HLA_B40_121():
    result = parse("HLA-B40:121")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:121") to be Allele but got {result}'
    )


def test_HLA_B40_122():
    result = parse("HLA-B40:122")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:122") to be Allele but got {result}'
    )


def test_HLA_B40_123():
    result = parse("HLA-B40:123")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:123") to be Allele but got {result}'
    )


def test_HLA_B40_124():
    result = parse("HLA-B40:124")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:124") to be Allele but got {result}'
    )


def test_HLA_B40_125():
    result = parse("HLA-B40:125")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:125") to be Allele but got {result}'
    )


def test_HLA_B40_126():
    result = parse("HLA-B40:126")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:126") to be Allele but got {result}'
    )


def test_HLA_B40_127():
    result = parse("HLA-B40:127")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:127") to be Allele but got {result}'
    )


def test_HLA_B40_128():
    result = parse("HLA-B40:128")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:128") to be Allele but got {result}'
    )


def test_HLA_B40_129():
    result = parse("HLA-B40:129")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:129") to be Allele but got {result}'
    )


def test_HLA_B40_130():
    result = parse("HLA-B40:130")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:130") to be Allele but got {result}'
    )


def test_HLA_B40_131():
    result = parse("HLA-B40:131")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:131") to be Allele but got {result}'
    )


def test_HLA_B40_132():
    result = parse("HLA-B40:132")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:132") to be Allele but got {result}'
    )


def test_HLA_B40_134():
    result = parse("HLA-B40:134")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:134") to be Allele but got {result}'
    )


def test_HLA_B40_135():
    result = parse("HLA-B40:135")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:135") to be Allele but got {result}'
    )


def test_HLA_B40_136():
    result = parse("HLA-B40:136")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:136") to be Allele but got {result}'
    )


def test_HLA_B40_137():
    result = parse("HLA-B40:137")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:137") to be Allele but got {result}'
    )


def test_HLA_B40_138():
    result = parse("HLA-B40:138")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:138") to be Allele but got {result}'
    )


def test_HLA_B40_139():
    result = parse("HLA-B40:139")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:139") to be Allele but got {result}'
    )


def test_HLA_B40_140():
    result = parse("HLA-B40:140")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:140") to be Allele but got {result}'
    )


def test_HLA_B40_141():
    result = parse("HLA-B40:141")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:141") to be Allele but got {result}'
    )


def test_HLA_B40_143():
    result = parse("HLA-B40:143")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:143") to be Allele but got {result}'
    )


def test_HLA_B40_145():
    result = parse("HLA-B40:145")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:145") to be Allele but got {result}'
    )


def test_HLA_B40_146():
    result = parse("HLA-B40:146")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:146") to be Allele but got {result}'
    )


def test_HLA_B40_147():
    result = parse("HLA-B40:147")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B40:147") to be Allele but got {result}'
    )


def test_HLA_B41_01():
    result = parse("HLA-B41:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:01") to be Allele but got {result}'


def test_HLA_B41_02():
    result = parse("HLA-B41:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:02") to be Allele but got {result}'


def test_HLA_B41_03():
    result = parse("HLA-B41:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:03") to be Allele but got {result}'


def test_HLA_B41_04():
    result = parse("HLA-B41:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:04") to be Allele but got {result}'


def test_HLA_B41_05():
    result = parse("HLA-B41:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:05") to be Allele but got {result}'


def test_HLA_B41_06():
    result = parse("HLA-B41:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:06") to be Allele but got {result}'


def test_HLA_B41_07():
    result = parse("HLA-B41:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:07") to be Allele but got {result}'


def test_HLA_B41_08():
    result = parse("HLA-B41:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:08") to be Allele but got {result}'


def test_HLA_B41_09():
    result = parse("HLA-B41:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:09") to be Allele but got {result}'


def test_HLA_B41_10():
    result = parse("HLA-B41:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:10") to be Allele but got {result}'


def test_HLA_B41_11():
    result = parse("HLA-B41:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:11") to be Allele but got {result}'


def test_HLA_B41_12():
    result = parse("HLA-B41:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B41:12") to be Allele but got {result}'


def test_HLA_B42_01():
    result = parse("HLA-B42:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:01") to be Allele but got {result}'


def test_HLA_B42_02():
    result = parse("HLA-B42:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:02") to be Allele but got {result}'


def test_HLA_B42_04():
    result = parse("HLA-B42:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:04") to be Allele but got {result}'


def test_HLA_B42_05():
    result = parse("HLA-B42:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:05") to be Allele but got {result}'


def test_HLA_B42_06():
    result = parse("HLA-B42:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:06") to be Allele but got {result}'


def test_HLA_B42_07():
    result = parse("HLA-B42:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:07") to be Allele but got {result}'


def test_HLA_B42_08():
    result = parse("HLA-B42:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:08") to be Allele but got {result}'


def test_HLA_B42_09():
    result = parse("HLA-B42:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:09") to be Allele but got {result}'


def test_HLA_B42_10():
    result = parse("HLA-B42:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:10") to be Allele but got {result}'


def test_HLA_B42_11():
    result = parse("HLA-B42:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:11") to be Allele but got {result}'


def test_HLA_B42_12():
    result = parse("HLA-B42:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:12") to be Allele but got {result}'


def test_HLA_B42_13():
    result = parse("HLA-B42:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:13") to be Allele but got {result}'


def test_HLA_B42_14():
    result = parse("HLA-B42:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B42:14") to be Allele but got {result}'


def test_HLA_B44_02():
    result = parse("HLA-B44:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:02") to be Allele but got {result}'


def test_HLA_B44_03():
    result = parse("HLA-B44:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:03") to be Allele but got {result}'


def test_HLA_B44_04():
    result = parse("HLA-B44:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:04") to be Allele but got {result}'


def test_HLA_B44_05():
    result = parse("HLA-B44:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:05") to be Allele but got {result}'


def test_HLA_B44_06():
    result = parse("HLA-B44:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:06") to be Allele but got {result}'


def test_HLA_B44_07():
    result = parse("HLA-B44:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:07") to be Allele but got {result}'


def test_HLA_B44_08():
    result = parse("HLA-B44:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:08") to be Allele but got {result}'


def test_HLA_B44_09():
    result = parse("HLA-B44:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:09") to be Allele but got {result}'


def test_HLA_B44_10():
    result = parse("HLA-B44:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:10") to be Allele but got {result}'


def test_HLA_B44_11():
    result = parse("HLA-B44:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:11") to be Allele but got {result}'


def test_HLA_B44_12():
    result = parse("HLA-B44:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:12") to be Allele but got {result}'


def test_HLA_B44_13():
    result = parse("HLA-B44:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:13") to be Allele but got {result}'


def test_HLA_B44_14():
    result = parse("HLA-B44:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:14") to be Allele but got {result}'


def test_HLA_B44_15():
    result = parse("HLA-B44:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:15") to be Allele but got {result}'


def test_HLA_B44_16():
    result = parse("HLA-B44:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:16") to be Allele but got {result}'


def test_HLA_B44_17():
    result = parse("HLA-B44:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:17") to be Allele but got {result}'


def test_HLA_B44_18():
    result = parse("HLA-B44:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:18") to be Allele but got {result}'


def test_HLA_B44_20():
    result = parse("HLA-B44:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:20") to be Allele but got {result}'


def test_HLA_B44_21():
    result = parse("HLA-B44:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:21") to be Allele but got {result}'


def test_HLA_B44_22():
    result = parse("HLA-B44:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:22") to be Allele but got {result}'


def test_HLA_B44_24():
    result = parse("HLA-B44:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:24") to be Allele but got {result}'


def test_HLA_B44_25():
    result = parse("HLA-B44:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:25") to be Allele but got {result}'


def test_HLA_B44_26():
    result = parse("HLA-B44:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:26") to be Allele but got {result}'


def test_HLA_B44_27():
    result = parse("HLA-B44:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:27") to be Allele but got {result}'


def test_HLA_B44_28():
    result = parse("HLA-B44:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:28") to be Allele but got {result}'


def test_HLA_B44_29():
    result = parse("HLA-B44:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:29") to be Allele but got {result}'


def test_HLA_B44_30():
    result = parse("HLA-B44:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:30") to be Allele but got {result}'


def test_HLA_B44_31():
    result = parse("HLA-B44:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:31") to be Allele but got {result}'


def test_HLA_B44_32():
    result = parse("HLA-B44:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:32") to be Allele but got {result}'


def test_HLA_B44_33():
    result = parse("HLA-B44:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:33") to be Allele but got {result}'


def test_HLA_B44_34():
    result = parse("HLA-B44:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:34") to be Allele but got {result}'


def test_HLA_B44_35():
    result = parse("HLA-B44:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:35") to be Allele but got {result}'


def test_HLA_B44_36():
    result = parse("HLA-B44:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:36") to be Allele but got {result}'


def test_HLA_B44_37():
    result = parse("HLA-B44:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:37") to be Allele but got {result}'


def test_HLA_B44_38():
    result = parse("HLA-B44:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:38") to be Allele but got {result}'


def test_HLA_B44_39():
    result = parse("HLA-B44:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:39") to be Allele but got {result}'


def test_HLA_B44_40():
    result = parse("HLA-B44:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:40") to be Allele but got {result}'


def test_HLA_B44_41():
    result = parse("HLA-B44:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:41") to be Allele but got {result}'


def test_HLA_B44_42():
    result = parse("HLA-B44:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:42") to be Allele but got {result}'


def test_HLA_B44_43():
    result = parse("HLA-B44:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:43") to be Allele but got {result}'


def test_HLA_B44_44():
    result = parse("HLA-B44:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:44") to be Allele but got {result}'


def test_HLA_B44_45():
    result = parse("HLA-B44:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:45") to be Allele but got {result}'


def test_HLA_B44_46():
    result = parse("HLA-B44:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:46") to be Allele but got {result}'


def test_HLA_B44_47():
    result = parse("HLA-B44:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:47") to be Allele but got {result}'


def test_HLA_B44_48():
    result = parse("HLA-B44:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:48") to be Allele but got {result}'


def test_HLA_B44_49():
    result = parse("HLA-B44:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:49") to be Allele but got {result}'


def test_HLA_B44_50():
    result = parse("HLA-B44:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:50") to be Allele but got {result}'


def test_HLA_B44_51():
    result = parse("HLA-B44:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:51") to be Allele but got {result}'


def test_HLA_B44_53():
    result = parse("HLA-B44:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:53") to be Allele but got {result}'


def test_HLA_B44_54():
    result = parse("HLA-B44:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:54") to be Allele but got {result}'


def test_HLA_B44_55():
    result = parse("HLA-B44:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:55") to be Allele but got {result}'


def test_HLA_B44_57():
    result = parse("HLA-B44:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:57") to be Allele but got {result}'


def test_HLA_B44_59():
    result = parse("HLA-B44:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:59") to be Allele but got {result}'


def test_HLA_B44_60():
    result = parse("HLA-B44:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:60") to be Allele but got {result}'


def test_HLA_B44_62():
    result = parse("HLA-B44:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:62") to be Allele but got {result}'


def test_HLA_B44_63():
    result = parse("HLA-B44:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:63") to be Allele but got {result}'


def test_HLA_B44_64():
    result = parse("HLA-B44:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:64") to be Allele but got {result}'


def test_HLA_B44_65():
    result = parse("HLA-B44:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:65") to be Allele but got {result}'


def test_HLA_B44_66():
    result = parse("HLA-B44:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:66") to be Allele but got {result}'


def test_HLA_B44_67():
    result = parse("HLA-B44:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:67") to be Allele but got {result}'


def test_HLA_B44_68():
    result = parse("HLA-B44:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:68") to be Allele but got {result}'


def test_HLA_B44_69():
    result = parse("HLA-B44:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:69") to be Allele but got {result}'


def test_HLA_B44_70():
    result = parse("HLA-B44:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:70") to be Allele but got {result}'


def test_HLA_B44_71():
    result = parse("HLA-B44:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:71") to be Allele but got {result}'


def test_HLA_B44_72():
    result = parse("HLA-B44:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:72") to be Allele but got {result}'


def test_HLA_B44_73():
    result = parse("HLA-B44:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:73") to be Allele but got {result}'


def test_HLA_B44_74():
    result = parse("HLA-B44:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:74") to be Allele but got {result}'


def test_HLA_B44_75():
    result = parse("HLA-B44:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:75") to be Allele but got {result}'


def test_HLA_B44_76():
    result = parse("HLA-B44:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:76") to be Allele but got {result}'


def test_HLA_B44_77():
    result = parse("HLA-B44:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:77") to be Allele but got {result}'


def test_HLA_B44_78():
    result = parse("HLA-B44:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:78") to be Allele but got {result}'


def test_HLA_B44_79():
    result = parse("HLA-B44:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:79") to be Allele but got {result}'


def test_HLA_B44_80():
    result = parse("HLA-B44:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:80") to be Allele but got {result}'


def test_HLA_B44_81():
    result = parse("HLA-B44:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:81") to be Allele but got {result}'


def test_HLA_B44_82():
    result = parse("HLA-B44:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:82") to be Allele but got {result}'


def test_HLA_B44_83():
    result = parse("HLA-B44:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:83") to be Allele but got {result}'


def test_HLA_B44_84():
    result = parse("HLA-B44:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:84") to be Allele but got {result}'


def test_HLA_B44_85():
    result = parse("HLA-B44:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:85") to be Allele but got {result}'


def test_HLA_B44_86():
    result = parse("HLA-B44:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:86") to be Allele but got {result}'


def test_HLA_B44_87():
    result = parse("HLA-B44:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:87") to be Allele but got {result}'


def test_HLA_B44_88():
    result = parse("HLA-B44:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:88") to be Allele but got {result}'


def test_HLA_B44_89():
    result = parse("HLA-B44:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:89") to be Allele but got {result}'


def test_HLA_B44_90():
    result = parse("HLA-B44:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:90") to be Allele but got {result}'


def test_HLA_B44_91():
    result = parse("HLA-B44:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:91") to be Allele but got {result}'


def test_HLA_B44_92():
    result = parse("HLA-B44:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:92") to be Allele but got {result}'


def test_HLA_B44_93():
    result = parse("HLA-B44:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:93") to be Allele but got {result}'


def test_HLA_B44_94():
    result = parse("HLA-B44:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:94") to be Allele but got {result}'


def test_HLA_B44_95():
    result = parse("HLA-B44:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:95") to be Allele but got {result}'


def test_HLA_B44_96():
    result = parse("HLA-B44:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:96") to be Allele but got {result}'


def test_HLA_B44_97():
    result = parse("HLA-B44:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:97") to be Allele but got {result}'


def test_HLA_B44_98():
    result = parse("HLA-B44:98")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:98") to be Allele but got {result}'


def test_HLA_B44_99():
    result = parse("HLA-B44:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-B44:99") to be Allele but got {result}'


def test_HLA_B44_100():
    result = parse("HLA-B44:100")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:100") to be Allele but got {result}'
    )


def test_HLA_B44_101():
    result = parse("HLA-B44:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:101") to be Allele but got {result}'
    )


def test_HLA_B44_102():
    result = parse("HLA-B44:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:102") to be Allele but got {result}'
    )


def test_HLA_B44_103():
    result = parse("HLA-B44:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:103") to be Allele but got {result}'
    )


def test_HLA_B44_104():
    result = parse("HLA-B44:104")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:104") to be Allele but got {result}'
    )


def test_HLA_B44_105():
    result = parse("HLA-B44:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:105") to be Allele but got {result}'
    )


def test_HLA_B44_106():
    result = parse("HLA-B44:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:106") to be Allele but got {result}'
    )


def test_HLA_B44_107():
    result = parse("HLA-B44:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:107") to be Allele but got {result}'
    )


def test_HLA_B44_109():
    result = parse("HLA-B44:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:109") to be Allele but got {result}'
    )


def test_HLA_B44_110():
    result = parse("HLA-B44:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-B44:110") to be Allele but got {result}'
    )


def test_HLA_B45_01():
    result = parse("HLA-B45:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:01") to be Allele but got {result}'


def test_HLA_B45_02():
    result = parse("HLA-B45:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:02") to be Allele but got {result}'


def test_HLA_B45_03():
    result = parse("HLA-B45:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:03") to be Allele but got {result}'


def test_HLA_B45_04():
    result = parse("HLA-B45:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:04") to be Allele but got {result}'


def test_HLA_B45_05():
    result = parse("HLA-B45:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:05") to be Allele but got {result}'


def test_HLA_B45_06():
    result = parse("HLA-B45:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:06") to be Allele but got {result}'


def test_HLA_B45_07():
    result = parse("HLA-B45:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:07") to be Allele but got {result}'


def test_HLA_B45_08():
    result = parse("HLA-B45:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:08") to be Allele but got {result}'


def test_HLA_B45_09():
    result = parse("HLA-B45:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:09") to be Allele but got {result}'


def test_HLA_B45_10():
    result = parse("HLA-B45:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:10") to be Allele but got {result}'


def test_HLA_B45_11():
    result = parse("HLA-B45:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:11") to be Allele but got {result}'


def test_HLA_B45_12():
    result = parse("HLA-B45:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B45:12") to be Allele but got {result}'


def test_HLA_B46_01():
    result = parse("HLA-B46:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:01") to be Allele but got {result}'


def test_HLA_B46_02():
    result = parse("HLA-B46:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:02") to be Allele but got {result}'


def test_HLA_B46_03():
    result = parse("HLA-B46:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:03") to be Allele but got {result}'


def test_HLA_B46_04():
    result = parse("HLA-B46:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:04") to be Allele but got {result}'


def test_HLA_B46_05():
    result = parse("HLA-B46:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:05") to be Allele but got {result}'


def test_HLA_B46_06():
    result = parse("HLA-B46:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:06") to be Allele but got {result}'


def test_HLA_B46_08():
    result = parse("HLA-B46:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:08") to be Allele but got {result}'


def test_HLA_B46_09():
    result = parse("HLA-B46:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:09") to be Allele but got {result}'


def test_HLA_B46_10():
    result = parse("HLA-B46:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:10") to be Allele but got {result}'


def test_HLA_B46_11():
    result = parse("HLA-B46:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:11") to be Allele but got {result}'


def test_HLA_B46_12():
    result = parse("HLA-B46:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:12") to be Allele but got {result}'


def test_HLA_B46_13():
    result = parse("HLA-B46:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:13") to be Allele but got {result}'


def test_HLA_B46_14():
    result = parse("HLA-B46:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:14") to be Allele but got {result}'


def test_HLA_B46_16():
    result = parse("HLA-B46:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:16") to be Allele but got {result}'


def test_HLA_B46_17():
    result = parse("HLA-B46:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:17") to be Allele but got {result}'


def test_HLA_B46_18():
    result = parse("HLA-B46:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:18") to be Allele but got {result}'


def test_HLA_B46_19():
    result = parse("HLA-B46:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:19") to be Allele but got {result}'


def test_HLA_B46_20():
    result = parse("HLA-B46:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:20") to be Allele but got {result}'


def test_HLA_B46_21():
    result = parse("HLA-B46:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:21") to be Allele but got {result}'


def test_HLA_B46_22():
    result = parse("HLA-B46:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:22") to be Allele but got {result}'


def test_HLA_B46_23():
    result = parse("HLA-B46:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:23") to be Allele but got {result}'


def test_HLA_B46_24():
    result = parse("HLA-B46:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B46:24") to be Allele but got {result}'


def test_HLA_B47_01():
    result = parse("HLA-B47:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B47:01") to be Allele but got {result}'


def test_HLA_B47_02():
    result = parse("HLA-B47:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B47:02") to be Allele but got {result}'


def test_HLA_B47_03():
    result = parse("HLA-B47:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B47:03") to be Allele but got {result}'


def test_HLA_B47_04():
    result = parse("HLA-B47:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B47:04") to be Allele but got {result}'


def test_HLA_B47_05():
    result = parse("HLA-B47:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B47:05") to be Allele but got {result}'


def test_HLA_B47_06():
    result = parse("HLA-B47:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B47:06") to be Allele but got {result}'


def test_HLA_B47_07():
    result = parse("HLA-B47:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B47:07") to be Allele but got {result}'


def test_HLA_B48_01():
    result = parse("HLA-B48:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:01") to be Allele but got {result}'


def test_HLA_B48_02():
    result = parse("HLA-B48:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:02") to be Allele but got {result}'


def test_HLA_B48_03():
    result = parse("HLA-B48:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:03") to be Allele but got {result}'


def test_HLA_B48_04():
    result = parse("HLA-B48:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:04") to be Allele but got {result}'


def test_HLA_B48_05():
    result = parse("HLA-B48:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:05") to be Allele but got {result}'


def test_HLA_B48_06():
    result = parse("HLA-B48:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:06") to be Allele but got {result}'


def test_HLA_B48_07():
    result = parse("HLA-B48:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:07") to be Allele but got {result}'


def test_HLA_B48_08():
    result = parse("HLA-B48:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:08") to be Allele but got {result}'


def test_HLA_B48_09():
    result = parse("HLA-B48:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:09") to be Allele but got {result}'


def test_HLA_B48_10():
    result = parse("HLA-B48:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:10") to be Allele but got {result}'


def test_HLA_B48_11():
    result = parse("HLA-B48:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:11") to be Allele but got {result}'


def test_HLA_B48_12():
    result = parse("HLA-B48:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:12") to be Allele but got {result}'


def test_HLA_B48_13():
    result = parse("HLA-B48:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:13") to be Allele but got {result}'


def test_HLA_B48_14():
    result = parse("HLA-B48:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:14") to be Allele but got {result}'


def test_HLA_B48_15():
    result = parse("HLA-B48:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:15") to be Allele but got {result}'


def test_HLA_B48_16():
    result = parse("HLA-B48:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:16") to be Allele but got {result}'


def test_HLA_B48_17():
    result = parse("HLA-B48:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:17") to be Allele but got {result}'


def test_HLA_B48_18():
    result = parse("HLA-B48:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:18") to be Allele but got {result}'


def test_HLA_B48_19():
    result = parse("HLA-B48:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:19") to be Allele but got {result}'


def test_HLA_B48_20():
    result = parse("HLA-B48:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:20") to be Allele but got {result}'


def test_HLA_B48_21():
    result = parse("HLA-B48:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:21") to be Allele but got {result}'


def test_HLA_B48_22():
    result = parse("HLA-B48:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:22") to be Allele but got {result}'


def test_HLA_B48_23():
    result = parse("HLA-B48:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B48:23") to be Allele but got {result}'


def test_HLA_B49_01():
    result = parse("HLA-B49:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:01") to be Allele but got {result}'


def test_HLA_B49_02():
    result = parse("HLA-B49:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:02") to be Allele but got {result}'


def test_HLA_B49_03():
    result = parse("HLA-B49:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:03") to be Allele but got {result}'


def test_HLA_B49_04():
    result = parse("HLA-B49:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:04") to be Allele but got {result}'


def test_HLA_B49_05():
    result = parse("HLA-B49:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:05") to be Allele but got {result}'


def test_HLA_B49_06():
    result = parse("HLA-B49:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:06") to be Allele but got {result}'


def test_HLA_B49_07():
    result = parse("HLA-B49:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:07") to be Allele but got {result}'


def test_HLA_B49_08():
    result = parse("HLA-B49:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:08") to be Allele but got {result}'


def test_HLA_B49_09():
    result = parse("HLA-B49:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:09") to be Allele but got {result}'


def test_HLA_B49_10():
    result = parse("HLA-B49:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B49:10") to be Allele but got {result}'


def test_HLA_B50_01():
    result = parse("HLA-B50:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:01") to be Allele but got {result}'


def test_HLA_B50_02():
    result = parse("HLA-B50:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:02") to be Allele but got {result}'


def test_HLA_B50_04():
    result = parse("HLA-B50:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:04") to be Allele but got {result}'


def test_HLA_B50_05():
    result = parse("HLA-B50:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:05") to be Allele but got {result}'


def test_HLA_B50_06():
    result = parse("HLA-B50:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:06") to be Allele but got {result}'


def test_HLA_B50_07():
    result = parse("HLA-B50:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:07") to be Allele but got {result}'


def test_HLA_B50_08():
    result = parse("HLA-B50:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:08") to be Allele but got {result}'


def test_HLA_B50_09():
    result = parse("HLA-B50:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B50:09") to be Allele but got {result}'


def test_HLA_B51_01():
    result = parse("HLA-B51:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:01") to be Allele but got {result}'


def test_HLA_B51_02():
    result = parse("HLA-B51:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:02") to be Allele but got {result}'


def test_HLA_B51_03():
    result = parse("HLA-B51:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:03") to be Allele but got {result}'


def test_HLA_B51_04():
    result = parse("HLA-B51:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:04") to be Allele but got {result}'


def test_HLA_B51_05():
    result = parse("HLA-B51:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:05") to be Allele but got {result}'


def test_HLA_B51_06():
    result = parse("HLA-B51:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:06") to be Allele but got {result}'


def test_HLA_B51_07():
    result = parse("HLA-B51:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:07") to be Allele but got {result}'


def test_HLA_B51_08():
    result = parse("HLA-B51:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:08") to be Allele but got {result}'


def test_HLA_B51_09():
    result = parse("HLA-B51:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:09") to be Allele but got {result}'


def test_HLA_B51_12():
    result = parse("HLA-B51:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:12") to be Allele but got {result}'


def test_HLA_B51_13():
    result = parse("HLA-B51:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:13") to be Allele but got {result}'


def test_HLA_B51_14():
    result = parse("HLA-B51:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:14") to be Allele but got {result}'


def test_HLA_B51_15():
    result = parse("HLA-B51:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:15") to be Allele but got {result}'


def test_HLA_B51_16():
    result = parse("HLA-B51:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:16") to be Allele but got {result}'


def test_HLA_B51_17():
    result = parse("HLA-B51:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:17") to be Allele but got {result}'


def test_HLA_B51_18():
    result = parse("HLA-B51:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:18") to be Allele but got {result}'


def test_HLA_B51_19():
    result = parse("HLA-B51:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:19") to be Allele but got {result}'


def test_HLA_B51_20():
    result = parse("HLA-B51:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:20") to be Allele but got {result}'


def test_HLA_B51_21():
    result = parse("HLA-B51:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:21") to be Allele but got {result}'


def test_HLA_B51_22():
    result = parse("HLA-B51:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:22") to be Allele but got {result}'


def test_HLA_B51_23():
    result = parse("HLA-B51:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:23") to be Allele but got {result}'


def test_HLA_B51_24():
    result = parse("HLA-B51:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:24") to be Allele but got {result}'


def test_HLA_B51_26():
    result = parse("HLA-B51:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:26") to be Allele but got {result}'


def test_HLA_B51_28():
    result = parse("HLA-B51:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:28") to be Allele but got {result}'


def test_HLA_B51_29():
    result = parse("HLA-B51:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:29") to be Allele but got {result}'


def test_HLA_B51_30():
    result = parse("HLA-B51:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:30") to be Allele but got {result}'


def test_HLA_B51_31():
    result = parse("HLA-B51:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:31") to be Allele but got {result}'


def test_HLA_B51_32():
    result = parse("HLA-B51:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:32") to be Allele but got {result}'


def test_HLA_B51_33():
    result = parse("HLA-B51:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:33") to be Allele but got {result}'


def test_HLA_B51_34():
    result = parse("HLA-B51:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:34") to be Allele but got {result}'


def test_HLA_B51_35():
    result = parse("HLA-B51:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:35") to be Allele but got {result}'


def test_HLA_B51_36():
    result = parse("HLA-B51:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:36") to be Allele but got {result}'


def test_HLA_B51_37():
    result = parse("HLA-B51:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:37") to be Allele but got {result}'


def test_HLA_B51_38():
    result = parse("HLA-B51:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:38") to be Allele but got {result}'


def test_HLA_B51_39():
    result = parse("HLA-B51:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:39") to be Allele but got {result}'


def test_HLA_B51_40():
    result = parse("HLA-B51:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:40") to be Allele but got {result}'


def test_HLA_B51_42():
    result = parse("HLA-B51:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:42") to be Allele but got {result}'


def test_HLA_B51_43():
    result = parse("HLA-B51:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:43") to be Allele but got {result}'


def test_HLA_B51_45():
    result = parse("HLA-B51:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:45") to be Allele but got {result}'


def test_HLA_B51_46():
    result = parse("HLA-B51:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:46") to be Allele but got {result}'


def test_HLA_B51_48():
    result = parse("HLA-B51:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:48") to be Allele but got {result}'


def test_HLA_B51_49():
    result = parse("HLA-B51:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:49") to be Allele but got {result}'


def test_HLA_B51_50():
    result = parse("HLA-B51:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:50") to be Allele but got {result}'


def test_HLA_B51_51():
    result = parse("HLA-B51:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:51") to be Allele but got {result}'


def test_HLA_B51_52():
    result = parse("HLA-B51:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:52") to be Allele but got {result}'


def test_HLA_B51_53():
    result = parse("HLA-B51:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:53") to be Allele but got {result}'


def test_HLA_B51_54():
    result = parse("HLA-B51:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:54") to be Allele but got {result}'


def test_HLA_B51_55():
    result = parse("HLA-B51:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:55") to be Allele but got {result}'


def test_HLA_B51_56():
    result = parse("HLA-B51:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:56") to be Allele but got {result}'


def test_HLA_B51_57():
    result = parse("HLA-B51:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:57") to be Allele but got {result}'


def test_HLA_B51_58():
    result = parse("HLA-B51:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:58") to be Allele but got {result}'


def test_HLA_B51_59():
    result = parse("HLA-B51:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:59") to be Allele but got {result}'


def test_HLA_B51_60():
    result = parse("HLA-B51:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:60") to be Allele but got {result}'


def test_HLA_B51_61():
    result = parse("HLA-B51:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:61") to be Allele but got {result}'


def test_HLA_B51_62():
    result = parse("HLA-B51:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:62") to be Allele but got {result}'


def test_HLA_B51_63():
    result = parse("HLA-B51:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:63") to be Allele but got {result}'


def test_HLA_B51_64():
    result = parse("HLA-B51:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:64") to be Allele but got {result}'


def test_HLA_B51_65():
    result = parse("HLA-B51:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:65") to be Allele but got {result}'


def test_HLA_B51_66():
    result = parse("HLA-B51:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:66") to be Allele but got {result}'


def test_HLA_B51_67():
    result = parse("HLA-B51:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:67") to be Allele but got {result}'


def test_HLA_B51_68():
    result = parse("HLA-B51:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:68") to be Allele but got {result}'


def test_HLA_B51_69():
    result = parse("HLA-B51:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:69") to be Allele but got {result}'


def test_HLA_B51_70():
    result = parse("HLA-B51:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:70") to be Allele but got {result}'


def test_HLA_B51_71():
    result = parse("HLA-B51:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:71") to be Allele but got {result}'


def test_HLA_B51_72():
    result = parse("HLA-B51:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:72") to be Allele but got {result}'


def test_HLA_B51_73():
    result = parse("HLA-B51:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:73") to be Allele but got {result}'


def test_HLA_B51_74():
    result = parse("HLA-B51:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:74") to be Allele but got {result}'


def test_HLA_B51_75():
    result = parse("HLA-B51:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:75") to be Allele but got {result}'


def test_HLA_B51_76():
    result = parse("HLA-B51:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:76") to be Allele but got {result}'


def test_HLA_B51_77():
    result = parse("HLA-B51:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:77") to be Allele but got {result}'


def test_HLA_B51_78():
    result = parse("HLA-B51:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:78") to be Allele but got {result}'


def test_HLA_B51_79():
    result = parse("HLA-B51:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:79") to be Allele but got {result}'


def test_HLA_B51_80():
    result = parse("HLA-B51:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:80") to be Allele but got {result}'


def test_HLA_B51_81():
    result = parse("HLA-B51:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:81") to be Allele but got {result}'


def test_HLA_B51_82():
    result = parse("HLA-B51:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:82") to be Allele but got {result}'


def test_HLA_B51_83():
    result = parse("HLA-B51:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:83") to be Allele but got {result}'


def test_HLA_B51_84():
    result = parse("HLA-B51:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:84") to be Allele but got {result}'


def test_HLA_B51_85():
    result = parse("HLA-B51:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:85") to be Allele but got {result}'


def test_HLA_B51_86():
    result = parse("HLA-B51:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:86") to be Allele but got {result}'


def test_HLA_B51_87():
    result = parse("HLA-B51:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:87") to be Allele but got {result}'


def test_HLA_B51_88():
    result = parse("HLA-B51:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:88") to be Allele but got {result}'


def test_HLA_B51_89():
    result = parse("HLA-B51:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:89") to be Allele but got {result}'


def test_HLA_B51_90():
    result = parse("HLA-B51:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:90") to be Allele but got {result}'


def test_HLA_B51_91():
    result = parse("HLA-B51:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:91") to be Allele but got {result}'


def test_HLA_B51_92():
    result = parse("HLA-B51:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:92") to be Allele but got {result}'


def test_HLA_B51_93():
    result = parse("HLA-B51:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:93") to be Allele but got {result}'


def test_HLA_B51_94():
    result = parse("HLA-B51:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:94") to be Allele but got {result}'


def test_HLA_B51_95():
    result = parse("HLA-B51:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:95") to be Allele but got {result}'


def test_HLA_B51_96():
    result = parse("HLA-B51:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-B51:96") to be Allele but got {result}'


def test_HLA_B52_01():
    result = parse("HLA-B52:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:01") to be Allele but got {result}'


def test_HLA_B52_02():
    result = parse("HLA-B52:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:02") to be Allele but got {result}'


def test_HLA_B52_03():
    result = parse("HLA-B52:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:03") to be Allele but got {result}'


def test_HLA_B52_04():
    result = parse("HLA-B52:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:04") to be Allele but got {result}'


def test_HLA_B52_05():
    result = parse("HLA-B52:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:05") to be Allele but got {result}'


def test_HLA_B52_06():
    result = parse("HLA-B52:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:06") to be Allele but got {result}'


def test_HLA_B52_07():
    result = parse("HLA-B52:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:07") to be Allele but got {result}'


def test_HLA_B52_08():
    result = parse("HLA-B52:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:08") to be Allele but got {result}'


def test_HLA_B52_09():
    result = parse("HLA-B52:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:09") to be Allele but got {result}'


def test_HLA_B52_10():
    result = parse("HLA-B52:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:10") to be Allele but got {result}'


def test_HLA_B52_11():
    result = parse("HLA-B52:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:11") to be Allele but got {result}'


def test_HLA_B52_12():
    result = parse("HLA-B52:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:12") to be Allele but got {result}'


def test_HLA_B52_13():
    result = parse("HLA-B52:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:13") to be Allele but got {result}'


def test_HLA_B52_14():
    result = parse("HLA-B52:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:14") to be Allele but got {result}'


def test_HLA_B52_15():
    result = parse("HLA-B52:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:15") to be Allele but got {result}'


def test_HLA_B52_16():
    result = parse("HLA-B52:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:16") to be Allele but got {result}'


def test_HLA_B52_17():
    result = parse("HLA-B52:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:17") to be Allele but got {result}'


def test_HLA_B52_18():
    result = parse("HLA-B52:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:18") to be Allele but got {result}'


def test_HLA_B52_19():
    result = parse("HLA-B52:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:19") to be Allele but got {result}'


def test_HLA_B52_20():
    result = parse("HLA-B52:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:20") to be Allele but got {result}'


def test_HLA_B52_21():
    result = parse("HLA-B52:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B52:21") to be Allele but got {result}'


def test_HLA_B53_01():
    result = parse("HLA-B53:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:01") to be Allele but got {result}'


def test_HLA_B53_02():
    result = parse("HLA-B53:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:02") to be Allele but got {result}'


def test_HLA_B53_03():
    result = parse("HLA-B53:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:03") to be Allele but got {result}'


def test_HLA_B53_04():
    result = parse("HLA-B53:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:04") to be Allele but got {result}'


def test_HLA_B53_05():
    result = parse("HLA-B53:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:05") to be Allele but got {result}'


def test_HLA_B53_06():
    result = parse("HLA-B53:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:06") to be Allele but got {result}'


def test_HLA_B53_07():
    result = parse("HLA-B53:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:07") to be Allele but got {result}'


def test_HLA_B53_08():
    result = parse("HLA-B53:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:08") to be Allele but got {result}'


def test_HLA_B53_09():
    result = parse("HLA-B53:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:09") to be Allele but got {result}'


def test_HLA_B53_10():
    result = parse("HLA-B53:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:10") to be Allele but got {result}'


def test_HLA_B53_11():
    result = parse("HLA-B53:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:11") to be Allele but got {result}'


def test_HLA_B53_12():
    result = parse("HLA-B53:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:12") to be Allele but got {result}'


def test_HLA_B53_13():
    result = parse("HLA-B53:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:13") to be Allele but got {result}'


def test_HLA_B53_14():
    result = parse("HLA-B53:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:14") to be Allele but got {result}'


def test_HLA_B53_15():
    result = parse("HLA-B53:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:15") to be Allele but got {result}'


def test_HLA_B53_16():
    result = parse("HLA-B53:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:16") to be Allele but got {result}'


def test_HLA_B53_17():
    result = parse("HLA-B53:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:17") to be Allele but got {result}'


def test_HLA_B53_18():
    result = parse("HLA-B53:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:18") to be Allele but got {result}'


def test_HLA_B53_19():
    result = parse("HLA-B53:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:19") to be Allele but got {result}'


def test_HLA_B53_20():
    result = parse("HLA-B53:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:20") to be Allele but got {result}'


def test_HLA_B53_21():
    result = parse("HLA-B53:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:21") to be Allele but got {result}'


def test_HLA_B53_22():
    result = parse("HLA-B53:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:22") to be Allele but got {result}'


def test_HLA_B53_23():
    result = parse("HLA-B53:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B53:23") to be Allele but got {result}'


def test_HLA_B54_01():
    result = parse("HLA-B54:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:01") to be Allele but got {result}'


def test_HLA_B54_02():
    result = parse("HLA-B54:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:02") to be Allele but got {result}'


def test_HLA_B54_03():
    result = parse("HLA-B54:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:03") to be Allele but got {result}'


def test_HLA_B54_04():
    result = parse("HLA-B54:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:04") to be Allele but got {result}'


def test_HLA_B54_06():
    result = parse("HLA-B54:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:06") to be Allele but got {result}'


def test_HLA_B54_07():
    result = parse("HLA-B54:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:07") to be Allele but got {result}'


def test_HLA_B54_09():
    result = parse("HLA-B54:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:09") to be Allele but got {result}'


def test_HLA_B54_10():
    result = parse("HLA-B54:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:10") to be Allele but got {result}'


def test_HLA_B54_11():
    result = parse("HLA-B54:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:11") to be Allele but got {result}'


def test_HLA_B54_12():
    result = parse("HLA-B54:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:12") to be Allele but got {result}'


def test_HLA_B54_13():
    result = parse("HLA-B54:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:13") to be Allele but got {result}'


def test_HLA_B54_14():
    result = parse("HLA-B54:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:14") to be Allele but got {result}'


def test_HLA_B54_15():
    result = parse("HLA-B54:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:15") to be Allele but got {result}'


def test_HLA_B54_16():
    result = parse("HLA-B54:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:16") to be Allele but got {result}'


def test_HLA_B54_17():
    result = parse("HLA-B54:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:17") to be Allele but got {result}'


def test_HLA_B54_18():
    result = parse("HLA-B54:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:18") to be Allele but got {result}'


def test_HLA_B54_19():
    result = parse("HLA-B54:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:19") to be Allele but got {result}'


def test_HLA_B54_20():
    result = parse("HLA-B54:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:20") to be Allele but got {result}'


def test_HLA_B54_21():
    result = parse("HLA-B54:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:21") to be Allele but got {result}'


def test_HLA_B54_22():
    result = parse("HLA-B54:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:22") to be Allele but got {result}'


def test_HLA_B54_23():
    result = parse("HLA-B54:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B54:23") to be Allele but got {result}'


def test_HLA_B55_01():
    result = parse("HLA-B55:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:01") to be Allele but got {result}'


def test_HLA_B55_02():
    result = parse("HLA-B55:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:02") to be Allele but got {result}'


def test_HLA_B55_03():
    result = parse("HLA-B55:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:03") to be Allele but got {result}'


def test_HLA_B55_04():
    result = parse("HLA-B55:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:04") to be Allele but got {result}'


def test_HLA_B55_05():
    result = parse("HLA-B55:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:05") to be Allele but got {result}'


def test_HLA_B55_07():
    result = parse("HLA-B55:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:07") to be Allele but got {result}'


def test_HLA_B55_08():
    result = parse("HLA-B55:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:08") to be Allele but got {result}'


def test_HLA_B55_09():
    result = parse("HLA-B55:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:09") to be Allele but got {result}'


def test_HLA_B55_10():
    result = parse("HLA-B55:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:10") to be Allele but got {result}'


def test_HLA_B55_11():
    result = parse("HLA-B55:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:11") to be Allele but got {result}'


def test_HLA_B55_12():
    result = parse("HLA-B55:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:12") to be Allele but got {result}'


def test_HLA_B55_13():
    result = parse("HLA-B55:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:13") to be Allele but got {result}'


def test_HLA_B55_14():
    result = parse("HLA-B55:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:14") to be Allele but got {result}'


def test_HLA_B55_15():
    result = parse("HLA-B55:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:15") to be Allele but got {result}'


def test_HLA_B55_16():
    result = parse("HLA-B55:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:16") to be Allele but got {result}'


def test_HLA_B55_17():
    result = parse("HLA-B55:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:17") to be Allele but got {result}'


def test_HLA_B55_18():
    result = parse("HLA-B55:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:18") to be Allele but got {result}'


def test_HLA_B55_19():
    result = parse("HLA-B55:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:19") to be Allele but got {result}'


def test_HLA_B55_20():
    result = parse("HLA-B55:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:20") to be Allele but got {result}'


def test_HLA_B55_21():
    result = parse("HLA-B55:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:21") to be Allele but got {result}'


def test_HLA_B55_22():
    result = parse("HLA-B55:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:22") to be Allele but got {result}'


def test_HLA_B55_23():
    result = parse("HLA-B55:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:23") to be Allele but got {result}'


def test_HLA_B55_24():
    result = parse("HLA-B55:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:24") to be Allele but got {result}'


def test_HLA_B55_25():
    result = parse("HLA-B55:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:25") to be Allele but got {result}'


def test_HLA_B55_26():
    result = parse("HLA-B55:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:26") to be Allele but got {result}'


def test_HLA_B55_27():
    result = parse("HLA-B55:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:27") to be Allele but got {result}'


def test_HLA_B55_28():
    result = parse("HLA-B55:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:28") to be Allele but got {result}'


def test_HLA_B55_29():
    result = parse("HLA-B55:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:29") to be Allele but got {result}'


def test_HLA_B55_30():
    result = parse("HLA-B55:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:30") to be Allele but got {result}'


def test_HLA_B55_31():
    result = parse("HLA-B55:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:31") to be Allele but got {result}'


def test_HLA_B55_32():
    result = parse("HLA-B55:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:32") to be Allele but got {result}'


def test_HLA_B55_33():
    result = parse("HLA-B55:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:33") to be Allele but got {result}'


def test_HLA_B55_34():
    result = parse("HLA-B55:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:34") to be Allele but got {result}'


def test_HLA_B55_35():
    result = parse("HLA-B55:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:35") to be Allele but got {result}'


def test_HLA_B55_36():
    result = parse("HLA-B55:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:36") to be Allele but got {result}'


def test_HLA_B55_37():
    result = parse("HLA-B55:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:37") to be Allele but got {result}'


def test_HLA_B55_38():
    result = parse("HLA-B55:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:38") to be Allele but got {result}'


def test_HLA_B55_39():
    result = parse("HLA-B55:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:39") to be Allele but got {result}'


def test_HLA_B55_40():
    result = parse("HLA-B55:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:40") to be Allele but got {result}'


def test_HLA_B55_41():
    result = parse("HLA-B55:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:41") to be Allele but got {result}'


def test_HLA_B55_42():
    result = parse("HLA-B55:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:42") to be Allele but got {result}'


def test_HLA_B55_43():
    result = parse("HLA-B55:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-B55:43") to be Allele but got {result}'


def test_HLA_B56_01():
    result = parse("HLA-B56:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:01") to be Allele but got {result}'


def test_HLA_B56_02():
    result = parse("HLA-B56:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:02") to be Allele but got {result}'


def test_HLA_B56_03():
    result = parse("HLA-B56:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:03") to be Allele but got {result}'


def test_HLA_B56_04():
    result = parse("HLA-B56:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:04") to be Allele but got {result}'


def test_HLA_B56_05():
    result = parse("HLA-B56:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:05") to be Allele but got {result}'


def test_HLA_B56_06():
    result = parse("HLA-B56:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:06") to be Allele but got {result}'


def test_HLA_B56_07():
    result = parse("HLA-B56:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:07") to be Allele but got {result}'


def test_HLA_B56_08():
    result = parse("HLA-B56:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:08") to be Allele but got {result}'


def test_HLA_B56_09():
    result = parse("HLA-B56:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:09") to be Allele but got {result}'


def test_HLA_B56_10():
    result = parse("HLA-B56:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:10") to be Allele but got {result}'


def test_HLA_B56_11():
    result = parse("HLA-B56:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:11") to be Allele but got {result}'


def test_HLA_B56_12():
    result = parse("HLA-B56:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:12") to be Allele but got {result}'


def test_HLA_B56_13():
    result = parse("HLA-B56:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:13") to be Allele but got {result}'


def test_HLA_B56_14():
    result = parse("HLA-B56:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:14") to be Allele but got {result}'


def test_HLA_B56_15():
    result = parse("HLA-B56:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:15") to be Allele but got {result}'


def test_HLA_B56_16():
    result = parse("HLA-B56:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:16") to be Allele but got {result}'


def test_HLA_B56_17():
    result = parse("HLA-B56:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:17") to be Allele but got {result}'


def test_HLA_B56_18():
    result = parse("HLA-B56:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:18") to be Allele but got {result}'


def test_HLA_B56_20():
    result = parse("HLA-B56:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:20") to be Allele but got {result}'


def test_HLA_B56_21():
    result = parse("HLA-B56:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:21") to be Allele but got {result}'


def test_HLA_B56_22():
    result = parse("HLA-B56:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:22") to be Allele but got {result}'


def test_HLA_B56_23():
    result = parse("HLA-B56:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:23") to be Allele but got {result}'


def test_HLA_B56_24():
    result = parse("HLA-B56:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:24") to be Allele but got {result}'


def test_HLA_B56_25():
    result = parse("HLA-B56:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:25") to be Allele but got {result}'


def test_HLA_B56_26():
    result = parse("HLA-B56:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:26") to be Allele but got {result}'


def test_HLA_B56_27():
    result = parse("HLA-B56:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:27") to be Allele but got {result}'


def test_HLA_B56_29():
    result = parse("HLA-B56:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B56:29") to be Allele but got {result}'


def test_HLA_B57_01():
    result = parse("HLA-B57:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:01") to be Allele but got {result}'


def test_HLA_B57_02():
    result = parse("HLA-B57:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:02") to be Allele but got {result}'


def test_HLA_B57_03():
    result = parse("HLA-B57:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:03") to be Allele but got {result}'


def test_HLA_B57_04():
    result = parse("HLA-B57:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:04") to be Allele but got {result}'


def test_HLA_B57_05():
    result = parse("HLA-B57:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:05") to be Allele but got {result}'


def test_HLA_B57_06():
    result = parse("HLA-B57:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:06") to be Allele but got {result}'


def test_HLA_B57_07():
    result = parse("HLA-B57:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:07") to be Allele but got {result}'


def test_HLA_B57_08():
    result = parse("HLA-B57:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:08") to be Allele but got {result}'


def test_HLA_B57_09():
    result = parse("HLA-B57:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:09") to be Allele but got {result}'


def test_HLA_B57_10():
    result = parse("HLA-B57:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:10") to be Allele but got {result}'


def test_HLA_B57_11():
    result = parse("HLA-B57:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:11") to be Allele but got {result}'


def test_HLA_B57_12():
    result = parse("HLA-B57:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:12") to be Allele but got {result}'


def test_HLA_B57_13():
    result = parse("HLA-B57:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:13") to be Allele but got {result}'


def test_HLA_B57_14():
    result = parse("HLA-B57:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:14") to be Allele but got {result}'


def test_HLA_B57_15():
    result = parse("HLA-B57:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:15") to be Allele but got {result}'


def test_HLA_B57_16():
    result = parse("HLA-B57:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:16") to be Allele but got {result}'


def test_HLA_B57_17():
    result = parse("HLA-B57:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:17") to be Allele but got {result}'


def test_HLA_B57_18():
    result = parse("HLA-B57:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:18") to be Allele but got {result}'


def test_HLA_B57_19():
    result = parse("HLA-B57:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:19") to be Allele but got {result}'


def test_HLA_B57_20():
    result = parse("HLA-B57:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:20") to be Allele but got {result}'


def test_HLA_B57_21():
    result = parse("HLA-B57:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:21") to be Allele but got {result}'


def test_HLA_B57_22():
    result = parse("HLA-B57:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:22") to be Allele but got {result}'


def test_HLA_B57_23():
    result = parse("HLA-B57:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:23") to be Allele but got {result}'


def test_HLA_B57_24():
    result = parse("HLA-B57:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:24") to be Allele but got {result}'


def test_HLA_B57_25():
    result = parse("HLA-B57:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:25") to be Allele but got {result}'


def test_HLA_B57_26():
    result = parse("HLA-B57:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:26") to be Allele but got {result}'


def test_HLA_B57_27():
    result = parse("HLA-B57:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:27") to be Allele but got {result}'


def test_HLA_B57_29():
    result = parse("HLA-B57:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:29") to be Allele but got {result}'


def test_HLA_B57_30():
    result = parse("HLA-B57:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:30") to be Allele but got {result}'


def test_HLA_B57_31():
    result = parse("HLA-B57:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:31") to be Allele but got {result}'


def test_HLA_B57_32():
    result = parse("HLA-B57:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-B57:32") to be Allele but got {result}'


def test_HLA_B58_01():
    result = parse("HLA-B58:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:01") to be Allele but got {result}'


def test_HLA_B58_02():
    result = parse("HLA-B58:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:02") to be Allele but got {result}'


def test_HLA_B58_04():
    result = parse("HLA-B58:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:04") to be Allele but got {result}'


def test_HLA_B58_05():
    result = parse("HLA-B58:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:05") to be Allele but got {result}'


def test_HLA_B58_06():
    result = parse("HLA-B58:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:06") to be Allele but got {result}'


def test_HLA_B58_07():
    result = parse("HLA-B58:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:07") to be Allele but got {result}'


def test_HLA_B58_08():
    result = parse("HLA-B58:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:08") to be Allele but got {result}'


def test_HLA_B58_09():
    result = parse("HLA-B58:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:09") to be Allele but got {result}'


def test_HLA_B58_11():
    result = parse("HLA-B58:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:11") to be Allele but got {result}'


def test_HLA_B58_12():
    result = parse("HLA-B58:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:12") to be Allele but got {result}'


def test_HLA_B58_13():
    result = parse("HLA-B58:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:13") to be Allele but got {result}'


def test_HLA_B58_14():
    result = parse("HLA-B58:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:14") to be Allele but got {result}'


def test_HLA_B58_15():
    result = parse("HLA-B58:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:15") to be Allele but got {result}'


def test_HLA_B58_16():
    result = parse("HLA-B58:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:16") to be Allele but got {result}'


def test_HLA_B58_18():
    result = parse("HLA-B58:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:18") to be Allele but got {result}'


def test_HLA_B58_19():
    result = parse("HLA-B58:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:19") to be Allele but got {result}'


def test_HLA_B58_20():
    result = parse("HLA-B58:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:20") to be Allele but got {result}'


def test_HLA_B58_21():
    result = parse("HLA-B58:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:21") to be Allele but got {result}'


def test_HLA_B58_22():
    result = parse("HLA-B58:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:22") to be Allele but got {result}'


def test_HLA_B58_23():
    result = parse("HLA-B58:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:23") to be Allele but got {result}'


def test_HLA_B58_24():
    result = parse("HLA-B58:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:24") to be Allele but got {result}'


def test_HLA_B58_25():
    result = parse("HLA-B58:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:25") to be Allele but got {result}'


def test_HLA_B58_26():
    result = parse("HLA-B58:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:26") to be Allele but got {result}'


def test_HLA_B58_27():
    result = parse("HLA-B58:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:27") to be Allele but got {result}'


def test_HLA_B58_28():
    result = parse("HLA-B58:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:28") to be Allele but got {result}'


def test_HLA_B58_29():
    result = parse("HLA-B58:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:29") to be Allele but got {result}'


def test_HLA_B58_30():
    result = parse("HLA-B58:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-B58:30") to be Allele but got {result}'


def test_HLA_B59_01():
    result = parse("HLA-B59:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B59:01") to be Allele but got {result}'


def test_HLA_B59_02():
    result = parse("HLA-B59:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B59:02") to be Allele but got {result}'


def test_HLA_B59_03():
    result = parse("HLA-B59:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B59:03") to be Allele but got {result}'


def test_HLA_B59_04():
    result = parse("HLA-B59:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B59:04") to be Allele but got {result}'


def test_HLA_B59_05():
    result = parse("HLA-B59:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B59:05") to be Allele but got {result}'


def test_HLA_B67_01():
    result = parse("HLA-B67:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B67:01") to be Allele but got {result}'


def test_HLA_B67_02():
    result = parse("HLA-B67:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B67:02") to be Allele but got {result}'


def test_HLA_B73_01():
    result = parse("HLA-B73:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B73:01") to be Allele but got {result}'


def test_HLA_B73_02():
    result = parse("HLA-B73:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B73:02") to be Allele but got {result}'


def test_HLA_B78_01():
    result = parse("HLA-B78:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B78:01") to be Allele but got {result}'


def test_HLA_B78_02():
    result = parse("HLA-B78:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B78:02") to be Allele but got {result}'


def test_HLA_B78_03():
    result = parse("HLA-B78:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B78:03") to be Allele but got {result}'


def test_HLA_B78_04():
    result = parse("HLA-B78:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-B78:04") to be Allele but got {result}'


def test_HLA_B78_05():
    result = parse("HLA-B78:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B78:05") to be Allele but got {result}'


def test_HLA_B78_06():
    result = parse("HLA-B78:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-B78:06") to be Allele but got {result}'


def test_HLA_B78_07():
    result = parse("HLA-B78:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-B78:07") to be Allele but got {result}'


def test_HLA_B81_01():
    result = parse("HLA-B81:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B81:01") to be Allele but got {result}'


def test_HLA_B81_02():
    result = parse("HLA-B81:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B81:02") to be Allele but got {result}'


def test_HLA_B81_03():
    result = parse("HLA-B81:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B81:03") to be Allele but got {result}'


def test_HLA_B81_05():
    result = parse("HLA-B81:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-B81:05") to be Allele but got {result}'


def test_HLA_B82_01():
    result = parse("HLA-B82:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B82:01") to be Allele but got {result}'


def test_HLA_B82_02():
    result = parse("HLA-B82:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-B82:02") to be Allele but got {result}'


def test_HLA_B82_03():
    result = parse("HLA-B82:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-B82:03") to be Allele but got {result}'


def test_HLA_B83_01():
    result = parse("HLA-B83:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-B83:01") to be Allele but got {result}'


def test_HLA_C01_02():
    result = parse("HLA-C01:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:02") to be Allele but got {result}'


def test_HLA_C01_03():
    result = parse("HLA-C01:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:03") to be Allele but got {result}'


def test_HLA_C01_04():
    result = parse("HLA-C01:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:04") to be Allele but got {result}'


def test_HLA_C01_05():
    result = parse("HLA-C01:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:05") to be Allele but got {result}'


def test_HLA_C01_06():
    result = parse("HLA-C01:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:06") to be Allele but got {result}'


def test_HLA_C01_07():
    result = parse("HLA-C01:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:07") to be Allele but got {result}'


def test_HLA_C01_08():
    result = parse("HLA-C01:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:08") to be Allele but got {result}'


def test_HLA_C01_09():
    result = parse("HLA-C01:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:09") to be Allele but got {result}'


def test_HLA_C01_10():
    result = parse("HLA-C01:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:10") to be Allele but got {result}'


def test_HLA_C01_11():
    result = parse("HLA-C01:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:11") to be Allele but got {result}'


def test_HLA_C01_12():
    result = parse("HLA-C01:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:12") to be Allele but got {result}'


def test_HLA_C01_13():
    result = parse("HLA-C01:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:13") to be Allele but got {result}'


def test_HLA_C01_14():
    result = parse("HLA-C01:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:14") to be Allele but got {result}'


def test_HLA_C01_15():
    result = parse("HLA-C01:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:15") to be Allele but got {result}'


def test_HLA_C01_16():
    result = parse("HLA-C01:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:16") to be Allele but got {result}'


def test_HLA_C01_17():
    result = parse("HLA-C01:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:17") to be Allele but got {result}'


def test_HLA_C01_18():
    result = parse("HLA-C01:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:18") to be Allele but got {result}'


def test_HLA_C01_19():
    result = parse("HLA-C01:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:19") to be Allele but got {result}'


def test_HLA_C01_20():
    result = parse("HLA-C01:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:20") to be Allele but got {result}'


def test_HLA_C01_21():
    result = parse("HLA-C01:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:21") to be Allele but got {result}'


def test_HLA_C01_22():
    result = parse("HLA-C01:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:22") to be Allele but got {result}'


def test_HLA_C01_23():
    result = parse("HLA-C01:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:23") to be Allele but got {result}'


def test_HLA_C01_24():
    result = parse("HLA-C01:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:24") to be Allele but got {result}'


def test_HLA_C01_25():
    result = parse("HLA-C01:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:25") to be Allele but got {result}'


def test_HLA_C01_26():
    result = parse("HLA-C01:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:26") to be Allele but got {result}'


def test_HLA_C01_27():
    result = parse("HLA-C01:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:27") to be Allele but got {result}'


def test_HLA_C01_28():
    result = parse("HLA-C01:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:28") to be Allele but got {result}'


def test_HLA_C01_29():
    result = parse("HLA-C01:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:29") to be Allele but got {result}'


def test_HLA_C01_30():
    result = parse("HLA-C01:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:30") to be Allele but got {result}'


def test_HLA_C01_31():
    result = parse("HLA-C01:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:31") to be Allele but got {result}'


def test_HLA_C01_32():
    result = parse("HLA-C01:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:32") to be Allele but got {result}'


def test_HLA_C01_33():
    result = parse("HLA-C01:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:33") to be Allele but got {result}'


def test_HLA_C01_34():
    result = parse("HLA-C01:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:34") to be Allele but got {result}'


def test_HLA_C01_35():
    result = parse("HLA-C01:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:35") to be Allele but got {result}'


def test_HLA_C01_36():
    result = parse("HLA-C01:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:36") to be Allele but got {result}'


def test_HLA_C01_38():
    result = parse("HLA-C01:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:38") to be Allele but got {result}'


def test_HLA_C01_39():
    result = parse("HLA-C01:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:39") to be Allele but got {result}'


def test_HLA_C01_40():
    result = parse("HLA-C01:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C01:40") to be Allele but got {result}'


def test_HLA_C02_02():
    result = parse("HLA-C02:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:02") to be Allele but got {result}'


def test_HLA_C02_03():
    result = parse("HLA-C02:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:03") to be Allele but got {result}'


def test_HLA_C02_04():
    result = parse("HLA-C02:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:04") to be Allele but got {result}'


def test_HLA_C02_05():
    result = parse("HLA-C02:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:05") to be Allele but got {result}'


def test_HLA_C02_06():
    result = parse("HLA-C02:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:06") to be Allele but got {result}'


def test_HLA_C02_07():
    result = parse("HLA-C02:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:07") to be Allele but got {result}'


def test_HLA_C02_08():
    result = parse("HLA-C02:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:08") to be Allele but got {result}'


def test_HLA_C02_09():
    result = parse("HLA-C02:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:09") to be Allele but got {result}'


def test_HLA_C02_10():
    result = parse("HLA-C02:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:10") to be Allele but got {result}'


def test_HLA_C02_11():
    result = parse("HLA-C02:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:11") to be Allele but got {result}'


def test_HLA_C02_12():
    result = parse("HLA-C02:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:12") to be Allele but got {result}'


def test_HLA_C02_13():
    result = parse("HLA-C02:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:13") to be Allele but got {result}'


def test_HLA_C02_14():
    result = parse("HLA-C02:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:14") to be Allele but got {result}'


def test_HLA_C02_15():
    result = parse("HLA-C02:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:15") to be Allele but got {result}'


def test_HLA_C02_16():
    result = parse("HLA-C02:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:16") to be Allele but got {result}'


def test_HLA_C02_17():
    result = parse("HLA-C02:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:17") to be Allele but got {result}'


def test_HLA_C02_18():
    result = parse("HLA-C02:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:18") to be Allele but got {result}'


def test_HLA_C02_19():
    result = parse("HLA-C02:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:19") to be Allele but got {result}'


def test_HLA_C02_20():
    result = parse("HLA-C02:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:20") to be Allele but got {result}'


def test_HLA_C02_21():
    result = parse("HLA-C02:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:21") to be Allele but got {result}'


def test_HLA_C02_22():
    result = parse("HLA-C02:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:22") to be Allele but got {result}'


def test_HLA_C02_23():
    result = parse("HLA-C02:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:23") to be Allele but got {result}'


def test_HLA_C02_24():
    result = parse("HLA-C02:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:24") to be Allele but got {result}'


def test_HLA_C02_26():
    result = parse("HLA-C02:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:26") to be Allele but got {result}'


def test_HLA_C02_27():
    result = parse("HLA-C02:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:27") to be Allele but got {result}'


def test_HLA_C02_28():
    result = parse("HLA-C02:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:28") to be Allele but got {result}'


def test_HLA_C02_29():
    result = parse("HLA-C02:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:29") to be Allele but got {result}'


def test_HLA_C02_30():
    result = parse("HLA-C02:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:30") to be Allele but got {result}'


def test_HLA_C02_31():
    result = parse("HLA-C02:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:31") to be Allele but got {result}'


def test_HLA_C02_32():
    result = parse("HLA-C02:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:32") to be Allele but got {result}'


def test_HLA_C02_33():
    result = parse("HLA-C02:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:33") to be Allele but got {result}'


def test_HLA_C02_34():
    result = parse("HLA-C02:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:34") to be Allele but got {result}'


def test_HLA_C02_35():
    result = parse("HLA-C02:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:35") to be Allele but got {result}'


def test_HLA_C02_36():
    result = parse("HLA-C02:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:36") to be Allele but got {result}'


def test_HLA_C02_37():
    result = parse("HLA-C02:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:37") to be Allele but got {result}'


def test_HLA_C02_39():
    result = parse("HLA-C02:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:39") to be Allele but got {result}'


def test_HLA_C02_40():
    result = parse("HLA-C02:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C02:40") to be Allele but got {result}'


def test_HLA_C03_01():
    result = parse("HLA-C03:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:01") to be Allele but got {result}'


def test_HLA_C03_02():
    result = parse("HLA-C03:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:02") to be Allele but got {result}'


def test_HLA_C03_03():
    result = parse("HLA-C03:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:03") to be Allele but got {result}'


def test_HLA_C03_04():
    result = parse("HLA-C03:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:04") to be Allele but got {result}'


def test_HLA_C03_05():
    result = parse("HLA-C03:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:05") to be Allele but got {result}'


def test_HLA_C03_06():
    result = parse("HLA-C03:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:06") to be Allele but got {result}'


def test_HLA_C03_07():
    result = parse("HLA-C03:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:07") to be Allele but got {result}'


def test_HLA_C03_08():
    result = parse("HLA-C03:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:08") to be Allele but got {result}'


def test_HLA_C03_09():
    result = parse("HLA-C03:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:09") to be Allele but got {result}'


def test_HLA_C03_10():
    result = parse("HLA-C03:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:10") to be Allele but got {result}'


def test_HLA_C03_11():
    result = parse("HLA-C03:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:11") to be Allele but got {result}'


def test_HLA_C03_12():
    result = parse("HLA-C03:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:12") to be Allele but got {result}'


def test_HLA_C03_13():
    result = parse("HLA-C03:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:13") to be Allele but got {result}'


def test_HLA_C03_14():
    result = parse("HLA-C03:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:14") to be Allele but got {result}'


def test_HLA_C03_15():
    result = parse("HLA-C03:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:15") to be Allele but got {result}'


def test_HLA_C03_16():
    result = parse("HLA-C03:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:16") to be Allele but got {result}'


def test_HLA_C03_17():
    result = parse("HLA-C03:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:17") to be Allele but got {result}'


def test_HLA_C03_18():
    result = parse("HLA-C03:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:18") to be Allele but got {result}'


def test_HLA_C03_19():
    result = parse("HLA-C03:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:19") to be Allele but got {result}'


def test_HLA_C03_21():
    result = parse("HLA-C03:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:21") to be Allele but got {result}'


def test_HLA_C03_23():
    result = parse("HLA-C03:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:23") to be Allele but got {result}'


def test_HLA_C03_24():
    result = parse("HLA-C03:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:24") to be Allele but got {result}'


def test_HLA_C03_25():
    result = parse("HLA-C03:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:25") to be Allele but got {result}'


def test_HLA_C03_26():
    result = parse("HLA-C03:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:26") to be Allele but got {result}'


def test_HLA_C03_27():
    result = parse("HLA-C03:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:27") to be Allele but got {result}'


def test_HLA_C03_28():
    result = parse("HLA-C03:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:28") to be Allele but got {result}'


def test_HLA_C03_29():
    result = parse("HLA-C03:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:29") to be Allele but got {result}'


def test_HLA_C03_30():
    result = parse("HLA-C03:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:30") to be Allele but got {result}'


def test_HLA_C03_31():
    result = parse("HLA-C03:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:31") to be Allele but got {result}'


def test_HLA_C03_32():
    result = parse("HLA-C03:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:32") to be Allele but got {result}'


def test_HLA_C03_33():
    result = parse("HLA-C03:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:33") to be Allele but got {result}'


def test_HLA_C03_34():
    result = parse("HLA-C03:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:34") to be Allele but got {result}'


def test_HLA_C03_35():
    result = parse("HLA-C03:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:35") to be Allele but got {result}'


def test_HLA_C03_36():
    result = parse("HLA-C03:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:36") to be Allele but got {result}'


def test_HLA_C03_37():
    result = parse("HLA-C03:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:37") to be Allele but got {result}'


def test_HLA_C03_38():
    result = parse("HLA-C03:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:38") to be Allele but got {result}'


def test_HLA_C03_39():
    result = parse("HLA-C03:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:39") to be Allele but got {result}'


def test_HLA_C03_40():
    result = parse("HLA-C03:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:40") to be Allele but got {result}'


def test_HLA_C03_41():
    result = parse("HLA-C03:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:41") to be Allele but got {result}'


def test_HLA_C03_42():
    result = parse("HLA-C03:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:42") to be Allele but got {result}'


def test_HLA_C03_43():
    result = parse("HLA-C03:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:43") to be Allele but got {result}'


def test_HLA_C03_44():
    result = parse("HLA-C03:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:44") to be Allele but got {result}'


def test_HLA_C03_45():
    result = parse("HLA-C03:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:45") to be Allele but got {result}'


def test_HLA_C03_46():
    result = parse("HLA-C03:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:46") to be Allele but got {result}'


def test_HLA_C03_47():
    result = parse("HLA-C03:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:47") to be Allele but got {result}'


def test_HLA_C03_48():
    result = parse("HLA-C03:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:48") to be Allele but got {result}'


def test_HLA_C03_49():
    result = parse("HLA-C03:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:49") to be Allele but got {result}'


def test_HLA_C03_50():
    result = parse("HLA-C03:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:50") to be Allele but got {result}'


def test_HLA_C03_51():
    result = parse("HLA-C03:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:51") to be Allele but got {result}'


def test_HLA_C03_52():
    result = parse("HLA-C03:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:52") to be Allele but got {result}'


def test_HLA_C03_53():
    result = parse("HLA-C03:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:53") to be Allele but got {result}'


def test_HLA_C03_54():
    result = parse("HLA-C03:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:54") to be Allele but got {result}'


def test_HLA_C03_55():
    result = parse("HLA-C03:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:55") to be Allele but got {result}'


def test_HLA_C03_56():
    result = parse("HLA-C03:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:56") to be Allele but got {result}'


def test_HLA_C03_57():
    result = parse("HLA-C03:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:57") to be Allele but got {result}'


def test_HLA_C03_58():
    result = parse("HLA-C03:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:58") to be Allele but got {result}'


def test_HLA_C03_59():
    result = parse("HLA-C03:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:59") to be Allele but got {result}'


def test_HLA_C03_60():
    result = parse("HLA-C03:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:60") to be Allele but got {result}'


def test_HLA_C03_61():
    result = parse("HLA-C03:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:61") to be Allele but got {result}'


def test_HLA_C03_62():
    result = parse("HLA-C03:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:62") to be Allele but got {result}'


def test_HLA_C03_63():
    result = parse("HLA-C03:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:63") to be Allele but got {result}'


def test_HLA_C03_64():
    result = parse("HLA-C03:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:64") to be Allele but got {result}'


def test_HLA_C03_65():
    result = parse("HLA-C03:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:65") to be Allele but got {result}'


def test_HLA_C03_66():
    result = parse("HLA-C03:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:66") to be Allele but got {result}'


def test_HLA_C03_67():
    result = parse("HLA-C03:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:67") to be Allele but got {result}'


def test_HLA_C03_68():
    result = parse("HLA-C03:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:68") to be Allele but got {result}'


def test_HLA_C03_69():
    result = parse("HLA-C03:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:69") to be Allele but got {result}'


def test_HLA_C03_70():
    result = parse("HLA-C03:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:70") to be Allele but got {result}'


def test_HLA_C03_71():
    result = parse("HLA-C03:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:71") to be Allele but got {result}'


def test_HLA_C03_72():
    result = parse("HLA-C03:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:72") to be Allele but got {result}'


def test_HLA_C03_73():
    result = parse("HLA-C03:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:73") to be Allele but got {result}'


def test_HLA_C03_74():
    result = parse("HLA-C03:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:74") to be Allele but got {result}'


def test_HLA_C03_75():
    result = parse("HLA-C03:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:75") to be Allele but got {result}'


def test_HLA_C03_76():
    result = parse("HLA-C03:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:76") to be Allele but got {result}'


def test_HLA_C03_77():
    result = parse("HLA-C03:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:77") to be Allele but got {result}'


def test_HLA_C03_78():
    result = parse("HLA-C03:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:78") to be Allele but got {result}'


def test_HLA_C03_79():
    result = parse("HLA-C03:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:79") to be Allele but got {result}'


def test_HLA_C03_80():
    result = parse("HLA-C03:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:80") to be Allele but got {result}'


def test_HLA_C03_81():
    result = parse("HLA-C03:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:81") to be Allele but got {result}'


def test_HLA_C03_82():
    result = parse("HLA-C03:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:82") to be Allele but got {result}'


def test_HLA_C03_83():
    result = parse("HLA-C03:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:83") to be Allele but got {result}'


def test_HLA_C03_84():
    result = parse("HLA-C03:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:84") to be Allele but got {result}'


def test_HLA_C03_85():
    result = parse("HLA-C03:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:85") to be Allele but got {result}'


def test_HLA_C03_86():
    result = parse("HLA-C03:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:86") to be Allele but got {result}'


def test_HLA_C03_87():
    result = parse("HLA-C03:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:87") to be Allele but got {result}'


def test_HLA_C03_88():
    result = parse("HLA-C03:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:88") to be Allele but got {result}'


def test_HLA_C03_89():
    result = parse("HLA-C03:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:89") to be Allele but got {result}'


def test_HLA_C03_90():
    result = parse("HLA-C03:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:90") to be Allele but got {result}'


def test_HLA_C03_91():
    result = parse("HLA-C03:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:91") to be Allele but got {result}'


def test_HLA_C03_92():
    result = parse("HLA-C03:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:92") to be Allele but got {result}'


def test_HLA_C03_93():
    result = parse("HLA-C03:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:93") to be Allele but got {result}'


def test_HLA_C03_94():
    result = parse("HLA-C03:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-C03:94") to be Allele but got {result}'


def test_HLA_C04_01():
    result = parse("HLA-C04:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:01") to be Allele but got {result}'


def test_HLA_C04_03():
    result = parse("HLA-C04:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:03") to be Allele but got {result}'


def test_HLA_C04_04():
    result = parse("HLA-C04:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:04") to be Allele but got {result}'


def test_HLA_C04_05():
    result = parse("HLA-C04:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:05") to be Allele but got {result}'


def test_HLA_C04_06():
    result = parse("HLA-C04:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:06") to be Allele but got {result}'


def test_HLA_C04_07():
    result = parse("HLA-C04:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:07") to be Allele but got {result}'


def test_HLA_C04_08():
    result = parse("HLA-C04:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:08") to be Allele but got {result}'


def test_HLA_C04_10():
    result = parse("HLA-C04:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:10") to be Allele but got {result}'


def test_HLA_C04_11():
    result = parse("HLA-C04:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:11") to be Allele but got {result}'


def test_HLA_C04_12():
    result = parse("HLA-C04:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:12") to be Allele but got {result}'


def test_HLA_C04_13():
    result = parse("HLA-C04:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:13") to be Allele but got {result}'


def test_HLA_C04_14():
    result = parse("HLA-C04:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:14") to be Allele but got {result}'


def test_HLA_C04_15():
    result = parse("HLA-C04:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:15") to be Allele but got {result}'


def test_HLA_C04_16():
    result = parse("HLA-C04:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:16") to be Allele but got {result}'


def test_HLA_C04_17():
    result = parse("HLA-C04:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:17") to be Allele but got {result}'


def test_HLA_C04_18():
    result = parse("HLA-C04:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:18") to be Allele but got {result}'


def test_HLA_C04_19():
    result = parse("HLA-C04:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:19") to be Allele but got {result}'


def test_HLA_C04_20():
    result = parse("HLA-C04:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:20") to be Allele but got {result}'


def test_HLA_C04_23():
    result = parse("HLA-C04:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:23") to be Allele but got {result}'


def test_HLA_C04_24():
    result = parse("HLA-C04:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:24") to be Allele but got {result}'


def test_HLA_C04_25():
    result = parse("HLA-C04:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:25") to be Allele but got {result}'


def test_HLA_C04_26():
    result = parse("HLA-C04:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:26") to be Allele but got {result}'


def test_HLA_C04_27():
    result = parse("HLA-C04:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:27") to be Allele but got {result}'


def test_HLA_C04_28():
    result = parse("HLA-C04:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:28") to be Allele but got {result}'


def test_HLA_C04_29():
    result = parse("HLA-C04:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:29") to be Allele but got {result}'


def test_HLA_C04_30():
    result = parse("HLA-C04:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:30") to be Allele but got {result}'


def test_HLA_C04_31():
    result = parse("HLA-C04:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:31") to be Allele but got {result}'


def test_HLA_C04_32():
    result = parse("HLA-C04:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:32") to be Allele but got {result}'


def test_HLA_C04_33():
    result = parse("HLA-C04:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:33") to be Allele but got {result}'


def test_HLA_C04_34():
    result = parse("HLA-C04:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:34") to be Allele but got {result}'


def test_HLA_C04_35():
    result = parse("HLA-C04:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:35") to be Allele but got {result}'


def test_HLA_C04_36():
    result = parse("HLA-C04:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:36") to be Allele but got {result}'


def test_HLA_C04_37():
    result = parse("HLA-C04:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:37") to be Allele but got {result}'


def test_HLA_C04_38():
    result = parse("HLA-C04:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:38") to be Allele but got {result}'


def test_HLA_C04_39():
    result = parse("HLA-C04:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:39") to be Allele but got {result}'


def test_HLA_C04_40():
    result = parse("HLA-C04:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:40") to be Allele but got {result}'


def test_HLA_C04_41():
    result = parse("HLA-C04:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:41") to be Allele but got {result}'


def test_HLA_C04_42():
    result = parse("HLA-C04:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:42") to be Allele but got {result}'


def test_HLA_C04_43():
    result = parse("HLA-C04:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:43") to be Allele but got {result}'


def test_HLA_C04_44():
    result = parse("HLA-C04:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:44") to be Allele but got {result}'


def test_HLA_C04_45():
    result = parse("HLA-C04:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:45") to be Allele but got {result}'


def test_HLA_C04_46():
    result = parse("HLA-C04:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:46") to be Allele but got {result}'


def test_HLA_C04_47():
    result = parse("HLA-C04:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:47") to be Allele but got {result}'


def test_HLA_C04_48():
    result = parse("HLA-C04:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:48") to be Allele but got {result}'


def test_HLA_C04_49():
    result = parse("HLA-C04:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:49") to be Allele but got {result}'


def test_HLA_C04_50():
    result = parse("HLA-C04:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:50") to be Allele but got {result}'


def test_HLA_C04_51():
    result = parse("HLA-C04:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:51") to be Allele but got {result}'


def test_HLA_C04_52():
    result = parse("HLA-C04:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:52") to be Allele but got {result}'


def test_HLA_C04_53():
    result = parse("HLA-C04:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:53") to be Allele but got {result}'


def test_HLA_C04_54():
    result = parse("HLA-C04:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:54") to be Allele but got {result}'


def test_HLA_C04_55():
    result = parse("HLA-C04:55")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:55") to be Allele but got {result}'


def test_HLA_C04_56():
    result = parse("HLA-C04:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:56") to be Allele but got {result}'


def test_HLA_C04_57():
    result = parse("HLA-C04:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:57") to be Allele but got {result}'


def test_HLA_C04_58():
    result = parse("HLA-C04:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:58") to be Allele but got {result}'


def test_HLA_C04_60():
    result = parse("HLA-C04:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:60") to be Allele but got {result}'


def test_HLA_C04_61():
    result = parse("HLA-C04:61")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:61") to be Allele but got {result}'


def test_HLA_C04_62():
    result = parse("HLA-C04:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:62") to be Allele but got {result}'


def test_HLA_C04_63():
    result = parse("HLA-C04:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:63") to be Allele but got {result}'


def test_HLA_C04_64():
    result = parse("HLA-C04:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:64") to be Allele but got {result}'


def test_HLA_C04_65():
    result = parse("HLA-C04:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:65") to be Allele but got {result}'


def test_HLA_C04_66():
    result = parse("HLA-C04:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:66") to be Allele but got {result}'


def test_HLA_C04_67():
    result = parse("HLA-C04:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:67") to be Allele but got {result}'


def test_HLA_C04_68():
    result = parse("HLA-C04:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:68") to be Allele but got {result}'


def test_HLA_C04_69():
    result = parse("HLA-C04:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:69") to be Allele but got {result}'


def test_HLA_C04_70():
    result = parse("HLA-C04:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-C04:70") to be Allele but got {result}'


def test_HLA_C05_01():
    result = parse("HLA-C05:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:01") to be Allele but got {result}'


def test_HLA_C05_03():
    result = parse("HLA-C05:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:03") to be Allele but got {result}'


def test_HLA_C05_04():
    result = parse("HLA-C05:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:04") to be Allele but got {result}'


def test_HLA_C05_05():
    result = parse("HLA-C05:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:05") to be Allele but got {result}'


def test_HLA_C05_06():
    result = parse("HLA-C05:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:06") to be Allele but got {result}'


def test_HLA_C05_08():
    result = parse("HLA-C05:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:08") to be Allele but got {result}'


def test_HLA_C05_09():
    result = parse("HLA-C05:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:09") to be Allele but got {result}'


def test_HLA_C05_10():
    result = parse("HLA-C05:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:10") to be Allele but got {result}'


def test_HLA_C05_11():
    result = parse("HLA-C05:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:11") to be Allele but got {result}'


def test_HLA_C05_12():
    result = parse("HLA-C05:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:12") to be Allele but got {result}'


def test_HLA_C05_13():
    result = parse("HLA-C05:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:13") to be Allele but got {result}'


def test_HLA_C05_14():
    result = parse("HLA-C05:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:14") to be Allele but got {result}'


def test_HLA_C05_15():
    result = parse("HLA-C05:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:15") to be Allele but got {result}'


def test_HLA_C05_16():
    result = parse("HLA-C05:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:16") to be Allele but got {result}'


def test_HLA_C05_17():
    result = parse("HLA-C05:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:17") to be Allele but got {result}'


def test_HLA_C05_18():
    result = parse("HLA-C05:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:18") to be Allele but got {result}'


def test_HLA_C05_19():
    result = parse("HLA-C05:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:19") to be Allele but got {result}'


def test_HLA_C05_20():
    result = parse("HLA-C05:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:20") to be Allele but got {result}'


def test_HLA_C05_21():
    result = parse("HLA-C05:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:21") to be Allele but got {result}'


def test_HLA_C05_22():
    result = parse("HLA-C05:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:22") to be Allele but got {result}'


def test_HLA_C05_23():
    result = parse("HLA-C05:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:23") to be Allele but got {result}'


def test_HLA_C05_24():
    result = parse("HLA-C05:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:24") to be Allele but got {result}'


def test_HLA_C05_25():
    result = parse("HLA-C05:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:25") to be Allele but got {result}'


def test_HLA_C05_26():
    result = parse("HLA-C05:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:26") to be Allele but got {result}'


def test_HLA_C05_27():
    result = parse("HLA-C05:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:27") to be Allele but got {result}'


def test_HLA_C05_28():
    result = parse("HLA-C05:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:28") to be Allele but got {result}'


def test_HLA_C05_29():
    result = parse("HLA-C05:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:29") to be Allele but got {result}'


def test_HLA_C05_30():
    result = parse("HLA-C05:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:30") to be Allele but got {result}'


def test_HLA_C05_31():
    result = parse("HLA-C05:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:31") to be Allele but got {result}'


def test_HLA_C05_32():
    result = parse("HLA-C05:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:32") to be Allele but got {result}'


def test_HLA_C05_33():
    result = parse("HLA-C05:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:33") to be Allele but got {result}'


def test_HLA_C05_34():
    result = parse("HLA-C05:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:34") to be Allele but got {result}'


def test_HLA_C05_35():
    result = parse("HLA-C05:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:35") to be Allele but got {result}'


def test_HLA_C05_36():
    result = parse("HLA-C05:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:36") to be Allele but got {result}'


def test_HLA_C05_37():
    result = parse("HLA-C05:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:37") to be Allele but got {result}'


def test_HLA_C05_38():
    result = parse("HLA-C05:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:38") to be Allele but got {result}'


def test_HLA_C05_39():
    result = parse("HLA-C05:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:39") to be Allele but got {result}'


def test_HLA_C05_40():
    result = parse("HLA-C05:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:40") to be Allele but got {result}'


def test_HLA_C05_41():
    result = parse("HLA-C05:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:41") to be Allele but got {result}'


def test_HLA_C05_42():
    result = parse("HLA-C05:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:42") to be Allele but got {result}'


def test_HLA_C05_43():
    result = parse("HLA-C05:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:43") to be Allele but got {result}'


def test_HLA_C05_44():
    result = parse("HLA-C05:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:44") to be Allele but got {result}'


def test_HLA_C05_45():
    result = parse("HLA-C05:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-C05:45") to be Allele but got {result}'


def test_HLA_C06_02():
    result = parse("HLA-C06:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:02") to be Allele but got {result}'


def test_HLA_C06_03():
    result = parse("HLA-C06:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:03") to be Allele but got {result}'


def test_HLA_C06_04():
    result = parse("HLA-C06:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:04") to be Allele but got {result}'


def test_HLA_C06_05():
    result = parse("HLA-C06:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:05") to be Allele but got {result}'


def test_HLA_C06_06():
    result = parse("HLA-C06:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:06") to be Allele but got {result}'


def test_HLA_C06_07():
    result = parse("HLA-C06:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:07") to be Allele but got {result}'


def test_HLA_C06_08():
    result = parse("HLA-C06:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:08") to be Allele but got {result}'


def test_HLA_C06_09():
    result = parse("HLA-C06:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:09") to be Allele but got {result}'


def test_HLA_C06_10():
    result = parse("HLA-C06:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:10") to be Allele but got {result}'


def test_HLA_C06_11():
    result = parse("HLA-C06:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:11") to be Allele but got {result}'


def test_HLA_C06_12():
    result = parse("HLA-C06:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:12") to be Allele but got {result}'


def test_HLA_C06_13():
    result = parse("HLA-C06:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:13") to be Allele but got {result}'


def test_HLA_C06_14():
    result = parse("HLA-C06:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:14") to be Allele but got {result}'


def test_HLA_C06_15():
    result = parse("HLA-C06:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:15") to be Allele but got {result}'


def test_HLA_C06_17():
    result = parse("HLA-C06:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:17") to be Allele but got {result}'


def test_HLA_C06_18():
    result = parse("HLA-C06:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:18") to be Allele but got {result}'


def test_HLA_C06_19():
    result = parse("HLA-C06:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:19") to be Allele but got {result}'


def test_HLA_C06_20():
    result = parse("HLA-C06:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:20") to be Allele but got {result}'


def test_HLA_C06_21():
    result = parse("HLA-C06:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:21") to be Allele but got {result}'


def test_HLA_C06_22():
    result = parse("HLA-C06:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:22") to be Allele but got {result}'


def test_HLA_C06_23():
    result = parse("HLA-C06:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:23") to be Allele but got {result}'


def test_HLA_C06_24():
    result = parse("HLA-C06:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:24") to be Allele but got {result}'


def test_HLA_C06_25():
    result = parse("HLA-C06:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:25") to be Allele but got {result}'


def test_HLA_C06_26():
    result = parse("HLA-C06:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:26") to be Allele but got {result}'


def test_HLA_C06_27():
    result = parse("HLA-C06:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:27") to be Allele but got {result}'


def test_HLA_C06_28():
    result = parse("HLA-C06:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:28") to be Allele but got {result}'


def test_HLA_C06_29():
    result = parse("HLA-C06:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:29") to be Allele but got {result}'


def test_HLA_C06_30():
    result = parse("HLA-C06:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:30") to be Allele but got {result}'


def test_HLA_C06_31():
    result = parse("HLA-C06:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:31") to be Allele but got {result}'


def test_HLA_C06_32():
    result = parse("HLA-C06:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:32") to be Allele but got {result}'


def test_HLA_C06_33():
    result = parse("HLA-C06:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:33") to be Allele but got {result}'


def test_HLA_C06_34():
    result = parse("HLA-C06:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:34") to be Allele but got {result}'


def test_HLA_C06_35():
    result = parse("HLA-C06:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:35") to be Allele but got {result}'


def test_HLA_C06_36():
    result = parse("HLA-C06:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:36") to be Allele but got {result}'


def test_HLA_C06_37():
    result = parse("HLA-C06:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:37") to be Allele but got {result}'


def test_HLA_C06_38():
    result = parse("HLA-C06:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:38") to be Allele but got {result}'


def test_HLA_C06_39():
    result = parse("HLA-C06:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:39") to be Allele but got {result}'


def test_HLA_C06_40():
    result = parse("HLA-C06:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:40") to be Allele but got {result}'


def test_HLA_C06_41():
    result = parse("HLA-C06:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:41") to be Allele but got {result}'


def test_HLA_C06_42():
    result = parse("HLA-C06:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:42") to be Allele but got {result}'


def test_HLA_C06_43():
    result = parse("HLA-C06:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:43") to be Allele but got {result}'


def test_HLA_C06_44():
    result = parse("HLA-C06:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:44") to be Allele but got {result}'


def test_HLA_C06_45():
    result = parse("HLA-C06:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-C06:45") to be Allele but got {result}'


def test_HLA_C07_01():
    result = parse("HLA-C07:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:01") to be Allele but got {result}'


def test_HLA_C07_02():
    result = parse("HLA-C07:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:02") to be Allele but got {result}'


def test_HLA_C07_03():
    result = parse("HLA-C07:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:03") to be Allele but got {result}'


def test_HLA_C07_04():
    result = parse("HLA-C07:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:04") to be Allele but got {result}'


def test_HLA_C07_05():
    result = parse("HLA-C07:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:05") to be Allele but got {result}'


def test_HLA_C07_06():
    result = parse("HLA-C07:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:06") to be Allele but got {result}'


def test_HLA_C07_07():
    result = parse("HLA-C07:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:07") to be Allele but got {result}'


def test_HLA_C07_08():
    result = parse("HLA-C07:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:08") to be Allele but got {result}'


def test_HLA_C07_09():
    result = parse("HLA-C07:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:09") to be Allele but got {result}'


def test_HLA_C07_10():
    result = parse("HLA-C07:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:10") to be Allele but got {result}'


def test_HLA_C07_11():
    result = parse("HLA-C07:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:11") to be Allele but got {result}'


def test_HLA_C07_12():
    result = parse("HLA-C07:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:12") to be Allele but got {result}'


def test_HLA_C07_13():
    result = parse("HLA-C07:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:13") to be Allele but got {result}'


def test_HLA_C07_14():
    result = parse("HLA-C07:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:14") to be Allele but got {result}'


def test_HLA_C07_15():
    result = parse("HLA-C07:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:15") to be Allele but got {result}'


def test_HLA_C07_16():
    result = parse("HLA-C07:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:16") to be Allele but got {result}'


def test_HLA_C07_17():
    result = parse("HLA-C07:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:17") to be Allele but got {result}'


def test_HLA_C07_18():
    result = parse("HLA-C07:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:18") to be Allele but got {result}'


def test_HLA_C07_19():
    result = parse("HLA-C07:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:19") to be Allele but got {result}'


def test_HLA_C07_20():
    result = parse("HLA-C07:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:20") to be Allele but got {result}'


def test_HLA_C07_21():
    result = parse("HLA-C07:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:21") to be Allele but got {result}'


def test_HLA_C07_22():
    result = parse("HLA-C07:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:22") to be Allele but got {result}'


def test_HLA_C07_23():
    result = parse("HLA-C07:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:23") to be Allele but got {result}'


def test_HLA_C07_24():
    result = parse("HLA-C07:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:24") to be Allele but got {result}'


def test_HLA_C07_25():
    result = parse("HLA-C07:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:25") to be Allele but got {result}'


def test_HLA_C07_26():
    result = parse("HLA-C07:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:26") to be Allele but got {result}'


def test_HLA_C07_27():
    result = parse("HLA-C07:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:27") to be Allele but got {result}'


def test_HLA_C07_28():
    result = parse("HLA-C07:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:28") to be Allele but got {result}'


def test_HLA_C07_29():
    result = parse("HLA-C07:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:29") to be Allele but got {result}'


def test_HLA_C07_30():
    result = parse("HLA-C07:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:30") to be Allele but got {result}'


def test_HLA_C07_31():
    result = parse("HLA-C07:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:31") to be Allele but got {result}'


def test_HLA_C07_35():
    result = parse("HLA-C07:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:35") to be Allele but got {result}'


def test_HLA_C07_36():
    result = parse("HLA-C07:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:36") to be Allele but got {result}'


def test_HLA_C07_37():
    result = parse("HLA-C07:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:37") to be Allele but got {result}'


def test_HLA_C07_38():
    result = parse("HLA-C07:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:38") to be Allele but got {result}'


def test_HLA_C07_39():
    result = parse("HLA-C07:39")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:39") to be Allele but got {result}'


def test_HLA_C07_40():
    result = parse("HLA-C07:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:40") to be Allele but got {result}'


def test_HLA_C07_41():
    result = parse("HLA-C07:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:41") to be Allele but got {result}'


def test_HLA_C07_42():
    result = parse("HLA-C07:42")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:42") to be Allele but got {result}'


def test_HLA_C07_43():
    result = parse("HLA-C07:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:43") to be Allele but got {result}'


def test_HLA_C07_44():
    result = parse("HLA-C07:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:44") to be Allele but got {result}'


def test_HLA_C07_45():
    result = parse("HLA-C07:45")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:45") to be Allele but got {result}'


def test_HLA_C07_46():
    result = parse("HLA-C07:46")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:46") to be Allele but got {result}'


def test_HLA_C07_47():
    result = parse("HLA-C07:47")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:47") to be Allele but got {result}'


def test_HLA_C07_48():
    result = parse("HLA-C07:48")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:48") to be Allele but got {result}'


def test_HLA_C07_49():
    result = parse("HLA-C07:49")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:49") to be Allele but got {result}'


def test_HLA_C07_50():
    result = parse("HLA-C07:50")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:50") to be Allele but got {result}'


def test_HLA_C07_51():
    result = parse("HLA-C07:51")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:51") to be Allele but got {result}'


def test_HLA_C07_52():
    result = parse("HLA-C07:52")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:52") to be Allele but got {result}'


def test_HLA_C07_53():
    result = parse("HLA-C07:53")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:53") to be Allele but got {result}'


def test_HLA_C07_54():
    result = parse("HLA-C07:54")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:54") to be Allele but got {result}'


def test_HLA_C07_56():
    result = parse("HLA-C07:56")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:56") to be Allele but got {result}'


def test_HLA_C07_57():
    result = parse("HLA-C07:57")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:57") to be Allele but got {result}'


def test_HLA_C07_58():
    result = parse("HLA-C07:58")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:58") to be Allele but got {result}'


def test_HLA_C07_59():
    result = parse("HLA-C07:59")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:59") to be Allele but got {result}'


def test_HLA_C07_60():
    result = parse("HLA-C07:60")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:60") to be Allele but got {result}'


def test_HLA_C07_62():
    result = parse("HLA-C07:62")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:62") to be Allele but got {result}'


def test_HLA_C07_63():
    result = parse("HLA-C07:63")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:63") to be Allele but got {result}'


def test_HLA_C07_64():
    result = parse("HLA-C07:64")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:64") to be Allele but got {result}'


def test_HLA_C07_65():
    result = parse("HLA-C07:65")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:65") to be Allele but got {result}'


def test_HLA_C07_66():
    result = parse("HLA-C07:66")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:66") to be Allele but got {result}'


def test_HLA_C07_67():
    result = parse("HLA-C07:67")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:67") to be Allele but got {result}'


def test_HLA_C07_68():
    result = parse("HLA-C07:68")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:68") to be Allele but got {result}'


def test_HLA_C07_69():
    result = parse("HLA-C07:69")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:69") to be Allele but got {result}'


def test_HLA_C07_70():
    result = parse("HLA-C07:70")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:70") to be Allele but got {result}'


def test_HLA_C07_71():
    result = parse("HLA-C07:71")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:71") to be Allele but got {result}'


def test_HLA_C07_72():
    result = parse("HLA-C07:72")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:72") to be Allele but got {result}'


def test_HLA_C07_73():
    result = parse("HLA-C07:73")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:73") to be Allele but got {result}'


def test_HLA_C07_74():
    result = parse("HLA-C07:74")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:74") to be Allele but got {result}'


def test_HLA_C07_75():
    result = parse("HLA-C07:75")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:75") to be Allele but got {result}'


def test_HLA_C07_76():
    result = parse("HLA-C07:76")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:76") to be Allele but got {result}'


def test_HLA_C07_77():
    result = parse("HLA-C07:77")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:77") to be Allele but got {result}'


def test_HLA_C07_78():
    result = parse("HLA-C07:78")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:78") to be Allele but got {result}'


def test_HLA_C07_79():
    result = parse("HLA-C07:79")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:79") to be Allele but got {result}'


def test_HLA_C07_80():
    result = parse("HLA-C07:80")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:80") to be Allele but got {result}'


def test_HLA_C07_81():
    result = parse("HLA-C07:81")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:81") to be Allele but got {result}'


def test_HLA_C07_82():
    result = parse("HLA-C07:82")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:82") to be Allele but got {result}'


def test_HLA_C07_83():
    result = parse("HLA-C07:83")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:83") to be Allele but got {result}'


def test_HLA_C07_84():
    result = parse("HLA-C07:84")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:84") to be Allele but got {result}'


def test_HLA_C07_85():
    result = parse("HLA-C07:85")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:85") to be Allele but got {result}'


def test_HLA_C07_86():
    result = parse("HLA-C07:86")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:86") to be Allele but got {result}'


def test_HLA_C07_87():
    result = parse("HLA-C07:87")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:87") to be Allele but got {result}'


def test_HLA_C07_88():
    result = parse("HLA-C07:88")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:88") to be Allele but got {result}'


def test_HLA_C07_89():
    result = parse("HLA-C07:89")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:89") to be Allele but got {result}'


def test_HLA_C07_90():
    result = parse("HLA-C07:90")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:90") to be Allele but got {result}'


def test_HLA_C07_91():
    result = parse("HLA-C07:91")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:91") to be Allele but got {result}'


def test_HLA_C07_92():
    result = parse("HLA-C07:92")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:92") to be Allele but got {result}'


def test_HLA_C07_93():
    result = parse("HLA-C07:93")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:93") to be Allele but got {result}'


def test_HLA_C07_94():
    result = parse("HLA-C07:94")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:94") to be Allele but got {result}'


def test_HLA_C07_95():
    result = parse("HLA-C07:95")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:95") to be Allele but got {result}'


def test_HLA_C07_96():
    result = parse("HLA-C07:96")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:96") to be Allele but got {result}'


def test_HLA_C07_97():
    result = parse("HLA-C07:97")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:97") to be Allele but got {result}'


def test_HLA_C07_99():
    result = parse("HLA-C07:99")
    assert result.__class__ is Allele, f'Expected parse("HLA-C07:99") to be Allele but got {result}'


def test_HLA_C07_100():
    result = parse("HLA-C07:100")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:100") to be Allele but got {result}'
    )


def test_HLA_C07_101():
    result = parse("HLA-C07:101")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:101") to be Allele but got {result}'
    )


def test_HLA_C07_102():
    result = parse("HLA-C07:102")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:102") to be Allele but got {result}'
    )


def test_HLA_C07_103():
    result = parse("HLA-C07:103")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:103") to be Allele but got {result}'
    )


def test_HLA_C07_105():
    result = parse("HLA-C07:105")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:105") to be Allele but got {result}'
    )


def test_HLA_C07_106():
    result = parse("HLA-C07:106")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:106") to be Allele but got {result}'
    )


def test_HLA_C07_107():
    result = parse("HLA-C07:107")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:107") to be Allele but got {result}'
    )


def test_HLA_C07_108():
    result = parse("HLA-C07:108")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:108") to be Allele but got {result}'
    )


def test_HLA_C07_109():
    result = parse("HLA-C07:109")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:109") to be Allele but got {result}'
    )


def test_HLA_C07_110():
    result = parse("HLA-C07:110")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:110") to be Allele but got {result}'
    )


def test_HLA_C07_111():
    result = parse("HLA-C07:111")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:111") to be Allele but got {result}'
    )


def test_HLA_C07_112():
    result = parse("HLA-C07:112")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:112") to be Allele but got {result}'
    )


def test_HLA_C07_113():
    result = parse("HLA-C07:113")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:113") to be Allele but got {result}'
    )


def test_HLA_C07_114():
    result = parse("HLA-C07:114")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:114") to be Allele but got {result}'
    )


def test_HLA_C07_115():
    result = parse("HLA-C07:115")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:115") to be Allele but got {result}'
    )


def test_HLA_C07_116():
    result = parse("HLA-C07:116")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:116") to be Allele but got {result}'
    )


def test_HLA_C07_117():
    result = parse("HLA-C07:117")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:117") to be Allele but got {result}'
    )


def test_HLA_C07_118():
    result = parse("HLA-C07:118")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:118") to be Allele but got {result}'
    )


def test_HLA_C07_119():
    result = parse("HLA-C07:119")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:119") to be Allele but got {result}'
    )


def test_HLA_C07_120():
    result = parse("HLA-C07:120")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:120") to be Allele but got {result}'
    )


def test_HLA_C07_122():
    result = parse("HLA-C07:122")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:122") to be Allele but got {result}'
    )


def test_HLA_C07_123():
    result = parse("HLA-C07:123")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:123") to be Allele but got {result}'
    )


def test_HLA_C07_124():
    result = parse("HLA-C07:124")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:124") to be Allele but got {result}'
    )


def test_HLA_C07_125():
    result = parse("HLA-C07:125")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:125") to be Allele but got {result}'
    )


def test_HLA_C07_126():
    result = parse("HLA-C07:126")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:126") to be Allele but got {result}'
    )


def test_HLA_C07_127():
    result = parse("HLA-C07:127")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:127") to be Allele but got {result}'
    )


def test_HLA_C07_128():
    result = parse("HLA-C07:128")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:128") to be Allele but got {result}'
    )


def test_HLA_C07_129():
    result = parse("HLA-C07:129")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:129") to be Allele but got {result}'
    )


def test_HLA_C07_130():
    result = parse("HLA-C07:130")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:130") to be Allele but got {result}'
    )


def test_HLA_C07_131():
    result = parse("HLA-C07:131")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:131") to be Allele but got {result}'
    )


def test_HLA_C07_132():
    result = parse("HLA-C07:132")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:132") to be Allele but got {result}'
    )


def test_HLA_C07_133():
    result = parse("HLA-C07:133")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:133") to be Allele but got {result}'
    )


def test_HLA_C07_134():
    result = parse("HLA-C07:134")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:134") to be Allele but got {result}'
    )


def test_HLA_C07_135():
    result = parse("HLA-C07:135")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:135") to be Allele but got {result}'
    )


def test_HLA_C07_136():
    result = parse("HLA-C07:136")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:136") to be Allele but got {result}'
    )


def test_HLA_C07_137():
    result = parse("HLA-C07:137")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:137") to be Allele but got {result}'
    )


def test_HLA_C07_138():
    result = parse("HLA-C07:138")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:138") to be Allele but got {result}'
    )


def test_HLA_C07_139():
    result = parse("HLA-C07:139")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:139") to be Allele but got {result}'
    )


def test_HLA_C07_140():
    result = parse("HLA-C07:140")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:140") to be Allele but got {result}'
    )


def test_HLA_C07_141():
    result = parse("HLA-C07:141")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:141") to be Allele but got {result}'
    )


def test_HLA_C07_142():
    result = parse("HLA-C07:142")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:142") to be Allele but got {result}'
    )


def test_HLA_C07_143():
    result = parse("HLA-C07:143")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:143") to be Allele but got {result}'
    )


def test_HLA_C07_144():
    result = parse("HLA-C07:144")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:144") to be Allele but got {result}'
    )


def test_HLA_C07_145():
    result = parse("HLA-C07:145")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:145") to be Allele but got {result}'
    )


def test_HLA_C07_146():
    result = parse("HLA-C07:146")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:146") to be Allele but got {result}'
    )


def test_HLA_C07_147():
    result = parse("HLA-C07:147")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:147") to be Allele but got {result}'
    )


def test_HLA_C07_148():
    result = parse("HLA-C07:148")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:148") to be Allele but got {result}'
    )


def test_HLA_C07_149():
    result = parse("HLA-C07:149")
    assert result.__class__ is Allele, (
        f'Expected parse("HLA-C07:149") to be Allele but got {result}'
    )


def test_HLA_C08_01():
    result = parse("HLA-C08:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:01") to be Allele but got {result}'


def test_HLA_C08_02():
    result = parse("HLA-C08:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:02") to be Allele but got {result}'


def test_HLA_C08_03():
    result = parse("HLA-C08:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:03") to be Allele but got {result}'


def test_HLA_C08_04():
    result = parse("HLA-C08:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:04") to be Allele but got {result}'


def test_HLA_C08_05():
    result = parse("HLA-C08:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:05") to be Allele but got {result}'


def test_HLA_C08_06():
    result = parse("HLA-C08:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:06") to be Allele but got {result}'


def test_HLA_C08_07():
    result = parse("HLA-C08:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:07") to be Allele but got {result}'


def test_HLA_C08_08():
    result = parse("HLA-C08:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:08") to be Allele but got {result}'


def test_HLA_C08_09():
    result = parse("HLA-C08:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:09") to be Allele but got {result}'


def test_HLA_C08_10():
    result = parse("HLA-C08:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:10") to be Allele but got {result}'


def test_HLA_C08_11():
    result = parse("HLA-C08:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:11") to be Allele but got {result}'


def test_HLA_C08_12():
    result = parse("HLA-C08:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:12") to be Allele but got {result}'


def test_HLA_C08_13():
    result = parse("HLA-C08:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:13") to be Allele but got {result}'


def test_HLA_C08_14():
    result = parse("HLA-C08:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:14") to be Allele but got {result}'


def test_HLA_C08_15():
    result = parse("HLA-C08:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:15") to be Allele but got {result}'


def test_HLA_C08_16():
    result = parse("HLA-C08:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:16") to be Allele but got {result}'


def test_HLA_C08_17():
    result = parse("HLA-C08:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:17") to be Allele but got {result}'


def test_HLA_C08_18():
    result = parse("HLA-C08:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:18") to be Allele but got {result}'


def test_HLA_C08_19():
    result = parse("HLA-C08:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:19") to be Allele but got {result}'


def test_HLA_C08_20():
    result = parse("HLA-C08:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:20") to be Allele but got {result}'


def test_HLA_C08_21():
    result = parse("HLA-C08:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:21") to be Allele but got {result}'


def test_HLA_C08_22():
    result = parse("HLA-C08:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:22") to be Allele but got {result}'


def test_HLA_C08_23():
    result = parse("HLA-C08:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:23") to be Allele but got {result}'


def test_HLA_C08_24():
    result = parse("HLA-C08:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:24") to be Allele but got {result}'


def test_HLA_C08_25():
    result = parse("HLA-C08:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:25") to be Allele but got {result}'


def test_HLA_C08_27():
    result = parse("HLA-C08:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:27") to be Allele but got {result}'


def test_HLA_C08_28():
    result = parse("HLA-C08:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:28") to be Allele but got {result}'


def test_HLA_C08_29():
    result = parse("HLA-C08:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:29") to be Allele but got {result}'


def test_HLA_C08_30():
    result = parse("HLA-C08:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:30") to be Allele but got {result}'


def test_HLA_C08_31():
    result = parse("HLA-C08:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:31") to be Allele but got {result}'


def test_HLA_C08_32():
    result = parse("HLA-C08:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:32") to be Allele but got {result}'


def test_HLA_C08_33():
    result = parse("HLA-C08:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:33") to be Allele but got {result}'


def test_HLA_C08_34():
    result = parse("HLA-C08:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:34") to be Allele but got {result}'


def test_HLA_C08_35():
    result = parse("HLA-C08:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C08:35") to be Allele but got {result}'


def test_HLA_C12_02():
    result = parse("HLA-C12:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:02") to be Allele but got {result}'


def test_HLA_C12_03():
    result = parse("HLA-C12:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:03") to be Allele but got {result}'


def test_HLA_C12_04():
    result = parse("HLA-C12:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:04") to be Allele but got {result}'


def test_HLA_C12_05():
    result = parse("HLA-C12:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:05") to be Allele but got {result}'


def test_HLA_C12_06():
    result = parse("HLA-C12:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:06") to be Allele but got {result}'


def test_HLA_C12_07():
    result = parse("HLA-C12:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:07") to be Allele but got {result}'


def test_HLA_C12_08():
    result = parse("HLA-C12:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:08") to be Allele but got {result}'


def test_HLA_C12_09():
    result = parse("HLA-C12:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:09") to be Allele but got {result}'


def test_HLA_C12_10():
    result = parse("HLA-C12:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:10") to be Allele but got {result}'


def test_HLA_C12_11():
    result = parse("HLA-C12:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:11") to be Allele but got {result}'


def test_HLA_C12_12():
    result = parse("HLA-C12:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:12") to be Allele but got {result}'


def test_HLA_C12_13():
    result = parse("HLA-C12:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:13") to be Allele but got {result}'


def test_HLA_C12_14():
    result = parse("HLA-C12:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:14") to be Allele but got {result}'


def test_HLA_C12_15():
    result = parse("HLA-C12:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:15") to be Allele but got {result}'


def test_HLA_C12_16():
    result = parse("HLA-C12:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:16") to be Allele but got {result}'


def test_HLA_C12_17():
    result = parse("HLA-C12:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:17") to be Allele but got {result}'


def test_HLA_C12_18():
    result = parse("HLA-C12:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:18") to be Allele but got {result}'


def test_HLA_C12_19():
    result = parse("HLA-C12:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:19") to be Allele but got {result}'


def test_HLA_C12_20():
    result = parse("HLA-C12:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:20") to be Allele but got {result}'


def test_HLA_C12_21():
    result = parse("HLA-C12:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:21") to be Allele but got {result}'


def test_HLA_C12_22():
    result = parse("HLA-C12:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:22") to be Allele but got {result}'


def test_HLA_C12_23():
    result = parse("HLA-C12:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:23") to be Allele but got {result}'


def test_HLA_C12_24():
    result = parse("HLA-C12:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:24") to be Allele but got {result}'


def test_HLA_C12_25():
    result = parse("HLA-C12:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:25") to be Allele but got {result}'


def test_HLA_C12_26():
    result = parse("HLA-C12:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:26") to be Allele but got {result}'


def test_HLA_C12_27():
    result = parse("HLA-C12:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:27") to be Allele but got {result}'


def test_HLA_C12_28():
    result = parse("HLA-C12:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:28") to be Allele but got {result}'


def test_HLA_C12_29():
    result = parse("HLA-C12:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:29") to be Allele but got {result}'


def test_HLA_C12_30():
    result = parse("HLA-C12:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:30") to be Allele but got {result}'


def test_HLA_C12_31():
    result = parse("HLA-C12:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:31") to be Allele but got {result}'


def test_HLA_C12_32():
    result = parse("HLA-C12:32")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:32") to be Allele but got {result}'


def test_HLA_C12_33():
    result = parse("HLA-C12:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:33") to be Allele but got {result}'


def test_HLA_C12_34():
    result = parse("HLA-C12:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:34") to be Allele but got {result}'


def test_HLA_C12_35():
    result = parse("HLA-C12:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:35") to be Allele but got {result}'


def test_HLA_C12_36():
    result = parse("HLA-C12:36")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:36") to be Allele but got {result}'


def test_HLA_C12_37():
    result = parse("HLA-C12:37")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:37") to be Allele but got {result}'


def test_HLA_C12_38():
    result = parse("HLA-C12:38")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:38") to be Allele but got {result}'


def test_HLA_C12_40():
    result = parse("HLA-C12:40")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:40") to be Allele but got {result}'


def test_HLA_C12_41():
    result = parse("HLA-C12:41")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:41") to be Allele but got {result}'


def test_HLA_C12_43():
    result = parse("HLA-C12:43")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:43") to be Allele but got {result}'


def test_HLA_C12_44():
    result = parse("HLA-C12:44")
    assert result.__class__ is Allele, f'Expected parse("HLA-C12:44") to be Allele but got {result}'


def test_HLA_C14_02():
    result = parse("HLA-C14:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:02") to be Allele but got {result}'


def test_HLA_C14_03():
    result = parse("HLA-C14:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:03") to be Allele but got {result}'


def test_HLA_C14_04():
    result = parse("HLA-C14:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:04") to be Allele but got {result}'


def test_HLA_C14_05():
    result = parse("HLA-C14:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:05") to be Allele but got {result}'


def test_HLA_C14_06():
    result = parse("HLA-C14:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:06") to be Allele but got {result}'


def test_HLA_C14_08():
    result = parse("HLA-C14:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:08") to be Allele but got {result}'


def test_HLA_C14_09():
    result = parse("HLA-C14:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:09") to be Allele but got {result}'


def test_HLA_C14_10():
    result = parse("HLA-C14:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:10") to be Allele but got {result}'


def test_HLA_C14_11():
    result = parse("HLA-C14:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:11") to be Allele but got {result}'


def test_HLA_C14_12():
    result = parse("HLA-C14:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:12") to be Allele but got {result}'


def test_HLA_C14_13():
    result = parse("HLA-C14:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:13") to be Allele but got {result}'


def test_HLA_C14_14():
    result = parse("HLA-C14:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:14") to be Allele but got {result}'


def test_HLA_C14_15():
    result = parse("HLA-C14:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:15") to be Allele but got {result}'


def test_HLA_C14_16():
    result = parse("HLA-C14:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:16") to be Allele but got {result}'


def test_HLA_C14_17():
    result = parse("HLA-C14:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:17") to be Allele but got {result}'


def test_HLA_C14_18():
    result = parse("HLA-C14:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:18") to be Allele but got {result}'


def test_HLA_C14_19():
    result = parse("HLA-C14:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:19") to be Allele but got {result}'


def test_HLA_C14_20():
    result = parse("HLA-C14:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C14:20") to be Allele but got {result}'


def test_HLA_C15_02():
    result = parse("HLA-C15:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:02") to be Allele but got {result}'


def test_HLA_C15_03():
    result = parse("HLA-C15:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:03") to be Allele but got {result}'


def test_HLA_C15_04():
    result = parse("HLA-C15:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:04") to be Allele but got {result}'


def test_HLA_C15_05():
    result = parse("HLA-C15:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:05") to be Allele but got {result}'


def test_HLA_C15_06():
    result = parse("HLA-C15:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:06") to be Allele but got {result}'


def test_HLA_C15_07():
    result = parse("HLA-C15:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:07") to be Allele but got {result}'


def test_HLA_C15_08():
    result = parse("HLA-C15:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:08") to be Allele but got {result}'


def test_HLA_C15_09():
    result = parse("HLA-C15:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:09") to be Allele but got {result}'


def test_HLA_C15_10():
    result = parse("HLA-C15:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:10") to be Allele but got {result}'


def test_HLA_C15_11():
    result = parse("HLA-C15:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:11") to be Allele but got {result}'


def test_HLA_C15_12():
    result = parse("HLA-C15:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:12") to be Allele but got {result}'


def test_HLA_C15_13():
    result = parse("HLA-C15:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:13") to be Allele but got {result}'


def test_HLA_C15_15():
    result = parse("HLA-C15:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:15") to be Allele but got {result}'


def test_HLA_C15_16():
    result = parse("HLA-C15:16")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:16") to be Allele but got {result}'


def test_HLA_C15_17():
    result = parse("HLA-C15:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:17") to be Allele but got {result}'


def test_HLA_C15_18():
    result = parse("HLA-C15:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:18") to be Allele but got {result}'


def test_HLA_C15_19():
    result = parse("HLA-C15:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:19") to be Allele but got {result}'


def test_HLA_C15_20():
    result = parse("HLA-C15:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:20") to be Allele but got {result}'


def test_HLA_C15_21():
    result = parse("HLA-C15:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:21") to be Allele but got {result}'


def test_HLA_C15_22():
    result = parse("HLA-C15:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:22") to be Allele but got {result}'


def test_HLA_C15_23():
    result = parse("HLA-C15:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:23") to be Allele but got {result}'


def test_HLA_C15_24():
    result = parse("HLA-C15:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:24") to be Allele but got {result}'


def test_HLA_C15_25():
    result = parse("HLA-C15:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:25") to be Allele but got {result}'


def test_HLA_C15_26():
    result = parse("HLA-C15:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:26") to be Allele but got {result}'


def test_HLA_C15_27():
    result = parse("HLA-C15:27")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:27") to be Allele but got {result}'


def test_HLA_C15_28():
    result = parse("HLA-C15:28")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:28") to be Allele but got {result}'


def test_HLA_C15_29():
    result = parse("HLA-C15:29")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:29") to be Allele but got {result}'


def test_HLA_C15_30():
    result = parse("HLA-C15:30")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:30") to be Allele but got {result}'


def test_HLA_C15_31():
    result = parse("HLA-C15:31")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:31") to be Allele but got {result}'


def test_HLA_C15_33():
    result = parse("HLA-C15:33")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:33") to be Allele but got {result}'


def test_HLA_C15_34():
    result = parse("HLA-C15:34")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:34") to be Allele but got {result}'


def test_HLA_C15_35():
    result = parse("HLA-C15:35")
    assert result.__class__ is Allele, f'Expected parse("HLA-C15:35") to be Allele but got {result}'


def test_HLA_C16_01():
    result = parse("HLA-C16:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:01") to be Allele but got {result}'


def test_HLA_C16_02():
    result = parse("HLA-C16:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:02") to be Allele but got {result}'


def test_HLA_C16_04():
    result = parse("HLA-C16:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:04") to be Allele but got {result}'


def test_HLA_C16_06():
    result = parse("HLA-C16:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:06") to be Allele but got {result}'


def test_HLA_C16_07():
    result = parse("HLA-C16:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:07") to be Allele but got {result}'


def test_HLA_C16_08():
    result = parse("HLA-C16:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:08") to be Allele but got {result}'


def test_HLA_C16_09():
    result = parse("HLA-C16:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:09") to be Allele but got {result}'


def test_HLA_C16_10():
    result = parse("HLA-C16:10")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:10") to be Allele but got {result}'


def test_HLA_C16_11():
    result = parse("HLA-C16:11")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:11") to be Allele but got {result}'


def test_HLA_C16_12():
    result = parse("HLA-C16:12")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:12") to be Allele but got {result}'


def test_HLA_C16_13():
    result = parse("HLA-C16:13")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:13") to be Allele but got {result}'


def test_HLA_C16_14():
    result = parse("HLA-C16:14")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:14") to be Allele but got {result}'


def test_HLA_C16_15():
    result = parse("HLA-C16:15")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:15") to be Allele but got {result}'


def test_HLA_C16_17():
    result = parse("HLA-C16:17")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:17") to be Allele but got {result}'


def test_HLA_C16_18():
    result = parse("HLA-C16:18")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:18") to be Allele but got {result}'


def test_HLA_C16_19():
    result = parse("HLA-C16:19")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:19") to be Allele but got {result}'


def test_HLA_C16_20():
    result = parse("HLA-C16:20")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:20") to be Allele but got {result}'


def test_HLA_C16_21():
    result = parse("HLA-C16:21")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:21") to be Allele but got {result}'


def test_HLA_C16_22():
    result = parse("HLA-C16:22")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:22") to be Allele but got {result}'


def test_HLA_C16_23():
    result = parse("HLA-C16:23")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:23") to be Allele but got {result}'


def test_HLA_C16_24():
    result = parse("HLA-C16:24")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:24") to be Allele but got {result}'


def test_HLA_C16_25():
    result = parse("HLA-C16:25")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:25") to be Allele but got {result}'


def test_HLA_C16_26():
    result = parse("HLA-C16:26")
    assert result.__class__ is Allele, f'Expected parse("HLA-C16:26") to be Allele but got {result}'


def test_HLA_C17_01():
    result = parse("HLA-C17:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C17:01") to be Allele but got {result}'


def test_HLA_C17_02():
    result = parse("HLA-C17:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C17:02") to be Allele but got {result}'


def test_HLA_C17_03():
    result = parse("HLA-C17:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C17:03") to be Allele but got {result}'


def test_HLA_C17_04():
    result = parse("HLA-C17:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-C17:04") to be Allele but got {result}'


def test_HLA_C17_05():
    result = parse("HLA-C17:05")
    assert result.__class__ is Allele, f'Expected parse("HLA-C17:05") to be Allele but got {result}'


def test_HLA_C17_06():
    result = parse("HLA-C17:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-C17:06") to be Allele but got {result}'


def test_HLA_C17_07():
    result = parse("HLA-C17:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-C17:07") to be Allele but got {result}'


def test_HLA_C18_01():
    result = parse("HLA-C18:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-C18:01") to be Allele but got {result}'


def test_HLA_C18_02():
    result = parse("HLA-C18:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-C18:02") to be Allele but got {result}'


def test_HLA_C18_03():
    result = parse("HLA-C18:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-C18:03") to be Allele but got {result}'


def test_Mamu_A1_00101():
    result = parse("Mamu-A1:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00101") to be Allele but got {result}'
    )


def test_Mamu_A1_00102():
    result = parse("Mamu-A1:00102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00102") to be Allele but got {result}'
    )


def test_Mamu_A1_00103():
    result = parse("Mamu-A1:00103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00103") to be Allele but got {result}'
    )


def test_Mamu_A1_00104():
    result = parse("Mamu-A1:00104")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00104") to be Allele but got {result}'
    )


def test_Mamu_A1_00105():
    result = parse("Mamu-A1:00105")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00105") to be Allele but got {result}'
    )


def test_Mamu_A1_00201():
    result = parse("Mamu-A1:00201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00201") to be Allele but got {result}'
    )


def test_Mamu_A1_00301():
    result = parse("Mamu-A1:00301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00301") to be Allele but got {result}'
    )


def test_Mamu_A1_00302():
    result = parse("Mamu-A1:00302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00302") to be Allele but got {result}'
    )


def test_Mamu_A1_00303():
    result = parse("Mamu-A1:00303")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00303") to be Allele but got {result}'
    )


def test_Mamu_A1_00304():
    result = parse("Mamu-A1:00304")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00304") to be Allele but got {result}'
    )


def test_Mamu_A1_00305():
    result = parse("Mamu-A1:00305")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00305") to be Allele but got {result}'
    )


def test_Mamu_A1_00306():
    result = parse("Mamu-A1:00306")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00306") to be Allele but got {result}'
    )


def test_Mamu_A1_00307():
    result = parse("Mamu-A1:00307")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00307") to be Allele but got {result}'
    )


def test_Mamu_A1_00308():
    result = parse("Mamu-A1:00308")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00308") to be Allele but got {result}'
    )


def test_Mamu_A1_00310():
    result = parse("Mamu-A1:00310")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00310") to be Allele but got {result}'
    )


def test_Mamu_A1_00401():
    result = parse("Mamu-A1:00401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00401") to be Allele but got {result}'
    )


def test_Mamu_A1_00402():
    result = parse("Mamu-A1:00402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00402") to be Allele but got {result}'
    )


def test_Mamu_A1_00403():
    result = parse("Mamu-A1:00403")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00403") to be Allele but got {result}'
    )


def test_Mamu_A1_00601():
    result = parse("Mamu-A1:00601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00601") to be Allele but got {result}'
    )


def test_Mamu_A1_00602():
    result = parse("Mamu-A1:00602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00602") to be Allele but got {result}'
    )


def test_Mamu_A1_00701():
    result = parse("Mamu-A1:00701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00701") to be Allele but got {result}'
    )


def test_Mamu_A1_00702():
    result = parse("Mamu-A1:00702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00702") to be Allele but got {result}'
    )


def test_Mamu_A1_00703():
    result = parse("Mamu-A1:00703")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00703") to be Allele but got {result}'
    )


def test_Mamu_A1_00704():
    result = parse("Mamu-A1:00704")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00704") to be Allele but got {result}'
    )


def test_Mamu_A1_00705():
    result = parse("Mamu-A1:00705")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00705") to be Allele but got {result}'
    )


def test_Mamu_A1_00801():
    result = parse("Mamu-A1:00801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:00801") to be Allele but got {result}'
    )


def test_Mamu_A1_01001():
    result = parse("Mamu-A1:01001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01001") to be Allele but got {result}'
    )


def test_Mamu_A1_01002():
    result = parse("Mamu-A1:01002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01002") to be Allele but got {result}'
    )


def test_Mamu_A1_01101():
    result = parse("Mamu-A1:01101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01101") to be Allele but got {result}'
    )


def test_Mamu_A1_01102():
    result = parse("Mamu-A1:01102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01102") to be Allele but got {result}'
    )


def test_Mamu_A1_01103():
    result = parse("Mamu-A1:01103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01103") to be Allele but got {result}'
    )


def test_Mamu_A1_01104():
    result = parse("Mamu-A1:01104")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01104") to be Allele but got {result}'
    )


def test_Mamu_A1_01201():
    result = parse("Mamu-A1:01201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01201") to be Allele but got {result}'
    )


def test_Mamu_A1_01601():
    result = parse("Mamu-A1:01601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01601") to be Allele but got {result}'
    )


def test_Mamu_A1_01801():
    result = parse("Mamu-A1:01801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01801") to be Allele but got {result}'
    )


def test_Mamu_A1_01802():
    result = parse("Mamu-A1:01802")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01802") to be Allele but got {result}'
    )


def test_Mamu_A1_01803():
    result = parse("Mamu-A1:01803")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01803") to be Allele but got {result}'
    )


def test_Mamu_A1_01804():
    result = parse("Mamu-A1:01804")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01804") to be Allele but got {result}'
    )


def test_Mamu_A1_01805():
    result = parse("Mamu-A1:01805")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01805") to be Allele but got {result}'
    )


def test_Mamu_A1_01806():
    result = parse("Mamu-A1:01806")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01806") to be Allele but got {result}'
    )


def test_Mamu_A1_01807():
    result = parse("Mamu-A1:01807")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01807") to be Allele but got {result}'
    )


def test_Mamu_A1_01808():
    result = parse("Mamu-A1:01808")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01808") to be Allele but got {result}'
    )


def test_Mamu_A1_01901():
    result = parse("Mamu-A1:01901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01901") to be Allele but got {result}'
    )


def test_Mamu_A1_01902():
    result = parse("Mamu-A1:01902")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01902") to be Allele but got {result}'
    )


def test_Mamu_A1_01903():
    result = parse("Mamu-A1:01903")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01903") to be Allele but got {result}'
    )


def test_Mamu_A1_01904():
    result = parse("Mamu-A1:01904")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01904") to be Allele but got {result}'
    )


def test_Mamu_A1_01905():
    result = parse("Mamu-A1:01905")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01905") to be Allele but got {result}'
    )


def test_Mamu_A1_01906():
    result = parse("Mamu-A1:01906")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01906") to be Allele but got {result}'
    )


def test_Mamu_A1_01907():
    result = parse("Mamu-A1:01907")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:01907") to be Allele but got {result}'
    )


def test_Mamu_A1_02201():
    result = parse("Mamu-A1:02201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02201") to be Allele but got {result}'
    )


def test_Mamu_A1_02202():
    result = parse("Mamu-A1:02202")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02202") to be Allele but got {result}'
    )


def test_Mamu_A1_02203():
    result = parse("Mamu-A1:02203")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02203") to be Allele but got {result}'
    )


def test_Mamu_A1_02301():
    result = parse("Mamu-A1:02301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02301") to be Allele but got {result}'
    )


def test_Mamu_A1_02302():
    result = parse("Mamu-A1:02302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02302") to be Allele but got {result}'
    )


def test_Mamu_A1_02501():
    result = parse("Mamu-A1:02501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02501") to be Allele but got {result}'
    )


def test_Mamu_A1_02502():
    result = parse("Mamu-A1:02502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02502") to be Allele but got {result}'
    )


def test_Mamu_A1_02601():
    result = parse("Mamu-A1:02601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02601") to be Allele but got {result}'
    )


def test_Mamu_A1_02602():
    result = parse("Mamu-A1:02602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02602") to be Allele but got {result}'
    )


def test_Mamu_A1_02603():
    result = parse("Mamu-A1:02603")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02603") to be Allele but got {result}'
    )


def test_Mamu_A1_02801():
    result = parse("Mamu-A1:02801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02801") to be Allele but got {result}'
    )


def test_Mamu_A1_02802():
    result = parse("Mamu-A1:02802")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02802") to be Allele but got {result}'
    )


def test_Mamu_A1_02803():
    result = parse("Mamu-A1:02803")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02803") to be Allele but got {result}'
    )


def test_Mamu_A1_02804():
    result = parse("Mamu-A1:02804")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02804") to be Allele but got {result}'
    )


def test_Mamu_A1_02805():
    result = parse("Mamu-A1:02805")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02805") to be Allele but got {result}'
    )


def test_Mamu_A1_02806():
    result = parse("Mamu-A1:02806")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:02806") to be Allele but got {result}'
    )


def test_Mamu_A1_03201():
    result = parse("Mamu-A1:03201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:03201") to be Allele but got {result}'
    )


def test_Mamu_A1_03202():
    result = parse("Mamu-A1:03202")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:03202") to be Allele but got {result}'
    )


def test_Mamu_A1_03203():
    result = parse("Mamu-A1:03203")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:03203") to be Allele but got {result}'
    )


def test_Mamu_A1_03301():
    result = parse("Mamu-A1:03301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:03301") to be Allele but got {result}'
    )


def test_Mamu_A1_04001():
    result = parse("Mamu-A1:04001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04001") to be Allele but got {result}'
    )


def test_Mamu_A1_04002():
    result = parse("Mamu-A1:04002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04002") to be Allele but got {result}'
    )


def test_Mamu_A1_04003():
    result = parse("Mamu-A1:04003")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04003") to be Allele but got {result}'
    )


def test_Mamu_A1_04101():
    result = parse("Mamu-A1:04101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04101") to be Allele but got {result}'
    )


def test_Mamu_A1_04102():
    result = parse("Mamu-A1:04102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04102") to be Allele but got {result}'
    )


def test_Mamu_A1_04201():
    result = parse("Mamu-A1:04201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04201") to be Allele but got {result}'
    )


def test_Mamu_A1_04301():
    result = parse("Mamu-A1:04301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04301") to be Allele but got {result}'
    )


def test_Mamu_A1_04501():
    result = parse("Mamu-A1:04501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04501") to be Allele but got {result}'
    )


def test_Mamu_A1_04801():
    result = parse("Mamu-A1:04801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04801") to be Allele but got {result}'
    )


def test_Mamu_A1_04901():
    result = parse("Mamu-A1:04901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04901") to be Allele but got {result}'
    )


def test_Mamu_A1_04902():
    result = parse("Mamu-A1:04902")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04902") to be Allele but got {result}'
    )


def test_Mamu_A1_04903():
    result = parse("Mamu-A1:04903")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04903") to be Allele but got {result}'
    )


def test_Mamu_A1_04904():
    result = parse("Mamu-A1:04904")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:04904") to be Allele but got {result}'
    )


def test_Mamu_A1_05001():
    result = parse("Mamu-A1:05001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05001") to be Allele but got {result}'
    )


def test_Mamu_A1_05101():
    result = parse("Mamu-A1:05101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05101") to be Allele but got {result}'
    )


def test_Mamu_A1_05201():
    result = parse("Mamu-A1:05201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05201") to be Allele but got {result}'
    )


def test_Mamu_A1_05301():
    result = parse("Mamu-A1:05301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05301") to be Allele but got {result}'
    )


def test_Mamu_A1_05302():
    result = parse("Mamu-A1:05302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05302") to be Allele but got {result}'
    )


def test_Mamu_A1_05401():
    result = parse("Mamu-A1:05401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05401") to be Allele but got {result}'
    )


def test_Mamu_A1_05402():
    result = parse("Mamu-A1:05402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05402") to be Allele but got {result}'
    )


def test_Mamu_A1_05501():
    result = parse("Mamu-A1:05501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05501") to be Allele but got {result}'
    )


def test_Mamu_A1_05601():
    result = parse("Mamu-A1:05601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05601") to be Allele but got {result}'
    )


def test_Mamu_A1_05602():
    result = parse("Mamu-A1:05602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05602") to be Allele but got {result}'
    )


def test_Mamu_A1_05603():
    result = parse("Mamu-A1:05603")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05603") to be Allele but got {result}'
    )


def test_Mamu_A1_05701():
    result = parse("Mamu-A1:05701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05701") to be Allele but got {result}'
    )


def test_Mamu_A1_05702():
    result = parse("Mamu-A1:05702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05702") to be Allele but got {result}'
    )


def test_Mamu_A1_05901():
    result = parse("Mamu-A1:05901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:05901") to be Allele but got {result}'
    )


def test_Mamu_A1_06001():
    result = parse("Mamu-A1:06001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:06001") to be Allele but got {result}'
    )


def test_Mamu_A1_06101():
    result = parse("Mamu-A1:06101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:06101") to be Allele but got {result}'
    )


def test_Mamu_A1_06301():
    result = parse("Mamu-A1:06301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:06301") to be Allele but got {result}'
    )


def test_Mamu_A1_06501():
    result = parse("Mamu-A1:06501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:06501") to be Allele but got {result}'
    )


def test_Mamu_A1_06601():
    result = parse("Mamu-A1:06601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:06601") to be Allele but got {result}'
    )


def test_Mamu_A1_07301():
    result = parse("Mamu-A1:07301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:07301") to be Allele but got {result}'
    )


def test_Mamu_A1_07401():
    result = parse("Mamu-A1:07401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:07401") to be Allele but got {result}'
    )


def test_Mamu_A1_07402():
    result = parse("Mamu-A1:07402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:07402") to be Allele but got {result}'
    )


def test_Mamu_A1_07403():
    result = parse("Mamu-A1:07403")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:07403") to be Allele but got {result}'
    )


def test_Mamu_A1_08101():
    result = parse("Mamu-A1:08101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:08101") to be Allele but got {result}'
    )


def test_Mamu_A1_08501():
    result = parse("Mamu-A1:08501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:08501") to be Allele but got {result}'
    )


def test_Mamu_A1_09101():
    result = parse("Mamu-A1:09101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:09101") to be Allele but got {result}'
    )


def test_Mamu_A1_09201():
    result = parse("Mamu-A1:09201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:09201") to be Allele but got {result}'
    )


def test_Mamu_A1_10501():
    result = parse("Mamu-A1:10501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10501") to be Allele but got {result}'
    )


def test_Mamu_A1_10502():
    result = parse("Mamu-A1:10502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10502") to be Allele but got {result}'
    )


def test_Mamu_A1_10503():
    result = parse("Mamu-A1:10503")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10503") to be Allele but got {result}'
    )


def test_Mamu_A1_10504():
    result = parse("Mamu-A1:10504")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10504") to be Allele but got {result}'
    )


def test_Mamu_A1_10601():
    result = parse("Mamu-A1:10601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10601") to be Allele but got {result}'
    )


def test_Mamu_A1_10701():
    result = parse("Mamu-A1:10701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10701") to be Allele but got {result}'
    )


def test_Mamu_A1_10801():
    result = parse("Mamu-A1:10801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10801") to be Allele but got {result}'
    )


def test_Mamu_A1_10901():
    result = parse("Mamu-A1:10901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:10901") to be Allele but got {result}'
    )


def test_Mamu_A1_11001():
    result = parse("Mamu-A1:11001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:11001") to be Allele but got {result}'
    )


def test_Mamu_A1_11101():
    result = parse("Mamu-A1:11101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:11101") to be Allele but got {result}'
    )


def test_Mamu_A1_11201():
    result = parse("Mamu-A1:11201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:11201") to be Allele but got {result}'
    )


def test_Mamu_A1_11301():
    result = parse("Mamu-A1:11301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A1:11301") to be Allele but got {result}'
    )


def test_Mamu_A2_0101():
    result = parse("Mamu-A2:0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0101") to be Allele but got {result}'
    )


def test_Mamu_A2_0102():
    result = parse("Mamu-A2:0102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0102") to be Allele but got {result}'
    )


def test_Mamu_A2_0103():
    result = parse("Mamu-A2:0103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0103") to be Allele but got {result}'
    )


def test_Mamu_A2_0501():
    result = parse("Mamu-A2:0501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0501") to be Allele but got {result}'
    )


def test_Mamu_A2_05020():
    result = parse("Mamu-A2:05020")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:05020") to be Allele but got {result}'
    )


def test_Mamu_A2_05030():
    result = parse("Mamu-A2:05030")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:05030") to be Allele but got {result}'
    )


def test_Mamu_A2_05040():
    result = parse("Mamu-A2:05040")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:05040") to be Allele but got {result}'
    )


def test_Mamu_A2_0505():
    result = parse("Mamu-A2:0505")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0505") to be Allele but got {result}'
    )


def test_Mamu_A2_0506():
    result = parse("Mamu-A2:0506")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0506") to be Allele but got {result}'
    )


def test_Mamu_A2_0507():
    result = parse("Mamu-A2:0507")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0507") to be Allele but got {result}'
    )


def test_Mamu_A2_0509():
    result = parse("Mamu-A2:0509")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0509") to be Allele but got {result}'
    )


def test_Mamu_A2_0510():
    result = parse("Mamu-A2:0510")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0510") to be Allele but got {result}'
    )


def test_Mamu_A2_0511():
    result = parse("Mamu-A2:0511")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0511") to be Allele but got {result}'
    )


def test_Mamu_A2_0512():
    result = parse("Mamu-A2:0512")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0512") to be Allele but got {result}'
    )


def test_Mamu_A2_0513():
    result = parse("Mamu-A2:0513")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0513") to be Allele but got {result}'
    )


def test_Mamu_A2_0514():
    result = parse("Mamu-A2:0514")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0514") to be Allele but got {result}'
    )


def test_Mamu_A2_05150():
    result = parse("Mamu-A2:05150")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:05150") to be Allele but got {result}'
    )


def test_Mamu_A2_05160():
    result = parse("Mamu-A2:05160")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:05160") to be Allele but got {result}'
    )


def test_Mamu_A2_0517():
    result = parse("Mamu-A2:0517")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0517") to be Allele but got {result}'
    )


def test_Mamu_A2_0518():
    result = parse("Mamu-A2:0518")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0518") to be Allele but got {result}'
    )


def test_Mamu_A2_0519():
    result = parse("Mamu-A2:0519")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0519") to be Allele but got {result}'
    )


def test_Mamu_A2_0520():
    result = parse("Mamu-A2:0520")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0520") to be Allele but got {result}'
    )


def test_Mamu_A2_0521():
    result = parse("Mamu-A2:0521")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0521") to be Allele but got {result}'
    )


def test_Mamu_A2_0522():
    result = parse("Mamu-A2:0522")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0522") to be Allele but got {result}'
    )


def test_Mamu_A2_0523():
    result = parse("Mamu-A2:0523")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0523") to be Allele but got {result}'
    )


def test_Mamu_A2_0524():
    result = parse("Mamu-A2:0524")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0524") to be Allele but got {result}'
    )


def test_Mamu_A2_0525():
    result = parse("Mamu-A2:0525")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0525") to be Allele but got {result}'
    )


def test_Mamu_A2_0526():
    result = parse("Mamu-A2:0526")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0526") to be Allele but got {result}'
    )


def test_Mamu_A2_0527():
    result = parse("Mamu-A2:0527")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0527") to be Allele but got {result}'
    )


def test_Mamu_A2_0528():
    result = parse("Mamu-A2:0528")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0528") to be Allele but got {result}'
    )


def test_Mamu_A2_0529():
    result = parse("Mamu-A2:0529")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0529") to be Allele but got {result}'
    )


def test_Mamu_A2_0531():
    result = parse("Mamu-A2:0531")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0531") to be Allele but got {result}'
    )


def test_Mamu_A2_05320():
    result = parse("Mamu-A2:05320")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:05320") to be Allele but got {result}'
    )


def test_Mamu_A2_0533():
    result = parse("Mamu-A2:0533")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0533") to be Allele but got {result}'
    )


def test_Mamu_A2_0534():
    result = parse("Mamu-A2:0534")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0534") to be Allele but got {result}'
    )


def test_Mamu_A2_0535():
    result = parse("Mamu-A2:0535")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0535") to be Allele but got {result}'
    )


def test_Mamu_A2_0536():
    result = parse("Mamu-A2:0536")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0536") to be Allele but got {result}'
    )


def test_Mamu_A2_0537():
    result = parse("Mamu-A2:0537")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0537") to be Allele but got {result}'
    )


def test_Mamu_A2_0538():
    result = parse("Mamu-A2:0538")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0538") to be Allele but got {result}'
    )


def test_Mamu_A2_0539():
    result = parse("Mamu-A2:0539")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0539") to be Allele but got {result}'
    )


def test_Mamu_A2_0540():
    result = parse("Mamu-A2:0540")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0540") to be Allele but got {result}'
    )


def test_Mamu_A2_0541():
    result = parse("Mamu-A2:0541")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0541") to be Allele but got {result}'
    )


def test_Mamu_A2_0542():
    result = parse("Mamu-A2:0542")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0542") to be Allele but got {result}'
    )


def test_Mamu_A2_0543():
    result = parse("Mamu-A2:0543")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0543") to be Allele but got {result}'
    )


def test_Mamu_A2_0544():
    result = parse("Mamu-A2:0544")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0544") to be Allele but got {result}'
    )


def test_Mamu_A2_0545():
    result = parse("Mamu-A2:0545")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0545") to be Allele but got {result}'
    )


def test_Mamu_A2_0546():
    result = parse("Mamu-A2:0546")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0546") to be Allele but got {result}'
    )


def test_Mamu_A2_0547():
    result = parse("Mamu-A2:0547")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:0547") to be Allele but got {result}'
    )


def test_Mamu_A2_2401():
    result = parse("Mamu-A2:2401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:2401") to be Allele but got {result}'
    )


def test_Mamu_A2_2402():
    result = parse("Mamu-A2:2402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:2402") to be Allele but got {result}'
    )


def test_Mamu_A2_2403():
    result = parse("Mamu-A2:2403")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A2:2403") to be Allele but got {result}'
    )


def test_Mamu_A3_1301():
    result = parse("Mamu-A3:1301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1301") to be Allele but got {result}'
    )


def test_Mamu_A3_1302():
    result = parse("Mamu-A3:1302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1302") to be Allele but got {result}'
    )


def test_Mamu_A3_1303():
    result = parse("Mamu-A3:1303")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1303") to be Allele but got {result}'
    )


def test_Mamu_A3_1304():
    result = parse("Mamu-A3:1304")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1304") to be Allele but got {result}'
    )


def test_Mamu_A3_1305():
    result = parse("Mamu-A3:1305")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1305") to be Allele but got {result}'
    )


def test_Mamu_A3_1306():
    result = parse("Mamu-A3:1306")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1306") to be Allele but got {result}'
    )


def test_Mamu_A3_1307():
    result = parse("Mamu-A3:1307")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1307") to be Allele but got {result}'
    )


def test_Mamu_A3_1308():
    result = parse("Mamu-A3:1308")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1308") to be Allele but got {result}'
    )


def test_Mamu_A3_1309():
    result = parse("Mamu-A3:1309")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1309") to be Allele but got {result}'
    )


def test_Mamu_A3_1310():
    result = parse("Mamu-A3:1310")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1310") to be Allele but got {result}'
    )


def test_Mamu_A3_1311():
    result = parse("Mamu-A3:1311")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1311") to be Allele but got {result}'
    )


def test_Mamu_A3_1312():
    result = parse("Mamu-A3:1312")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1312") to be Allele but got {result}'
    )


def test_Mamu_A3_1313():
    result = parse("Mamu-A3:1313")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A3:1313") to be Allele but got {result}'
    )


def test_Mamu_A4_0101():
    result = parse("Mamu-A4:0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:0101") to be Allele but got {result}'
    )


def test_Mamu_A4_01020():
    result = parse("Mamu-A4:01020")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:01020") to be Allele but got {result}'
    )


def test_Mamu_A4_0103():
    result = parse("Mamu-A4:0103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:0103") to be Allele but got {result}'
    )


def test_Mamu_A4_0202():
    result = parse("Mamu-A4:0202")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:0202") to be Allele but got {result}'
    )


def test_Mamu_A4_0203():
    result = parse("Mamu-A4:0203")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:0203") to be Allele but got {result}'
    )


def test_Mamu_A4_0205():
    result = parse("Mamu-A4:0205")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:0205") to be Allele but got {result}'
    )


def test_Mamu_A4_0301():
    result = parse("Mamu-A4:0301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:0301") to be Allele but got {result}'
    )


def test_Mamu_A4_0302():
    result = parse("Mamu-A4:0302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:0302") to be Allele but got {result}'
    )


def test_Mamu_A4_1402():
    result = parse("Mamu-A4:1402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:1402") to be Allele but got {result}'
    )


def test_Mamu_A4_14030():
    result = parse("Mamu-A4:14030")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:14030") to be Allele but got {result}'
    )


def test_Mamu_A4_1404():
    result = parse("Mamu-A4:1404")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:1404") to be Allele but got {result}'
    )


def test_Mamu_A4_1405():
    result = parse("Mamu-A4:1405")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:1405") to be Allele but got {result}'
    )


def test_Mamu_A4_1406():
    result = parse("Mamu-A4:1406")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:1406") to be Allele but got {result}'
    )


def test_Mamu_A4_1407():
    result = parse("Mamu-A4:1407")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:1407") to be Allele but got {result}'
    )


def test_Mamu_A4_1408():
    result = parse("Mamu-A4:1408")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:1408") to be Allele but got {result}'
    )


def test_Mamu_A4_1409():
    result = parse("Mamu-A4:1409")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A4:1409") to be Allele but got {result}'
    )


def test_Mamu_A5_30010():
    result = parse("Mamu-A5:30010")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5:30010") to be Allele but got {result}'
    )


def test_Mamu_A5_3002():
    result = parse("Mamu-A5:3002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5:3002") to be Allele but got {result}'
    )


def test_Mamu_A5_3003():
    result = parse("Mamu-A5:3003")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5:3003") to be Allele but got {result}'
    )


def test_Mamu_A5_3004():
    result = parse("Mamu-A5:3004")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5:3004") to be Allele but got {result}'
    )


def test_Mamu_A5_3005():
    result = parse("Mamu-A5:3005")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5:3005") to be Allele but got {result}'
    )


def test_Mamu_A5_3006():
    result = parse("Mamu-A5:3006")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A5:3006") to be Allele but got {result}'
    )


def test_Mamu_A6_0101():
    result = parse("Mamu-A6:0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6:0101") to be Allele but got {result}'
    )


def test_Mamu_A6_0102():
    result = parse("Mamu-A6:0102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6:0102") to be Allele but got {result}'
    )


def test_Mamu_A6_0103():
    result = parse("Mamu-A6:0103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6:0103") to be Allele but got {result}'
    )


def test_Mamu_A6_0104():
    result = parse("Mamu-A6:0104")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6:0104") to be Allele but got {result}'
    )


def test_Mamu_A6_0105():
    result = parse("Mamu-A6:0105")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A6:0105") to be Allele but got {result}'
    )


def test_Mamu_A7_0101():
    result = parse("Mamu-A7:0101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7:0101") to be Allele but got {result}'
    )


def test_Mamu_A7_0102():
    result = parse("Mamu-A7:0102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7:0102") to be Allele but got {result}'
    )


def test_Mamu_A7_0103():
    result = parse("Mamu-A7:0103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7:0103") to be Allele but got {result}'
    )


def test_Mamu_A7_0201():
    result = parse("Mamu-A7:0201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-A7:0201") to be Allele but got {result}'
    )


def test_Mamu_AG_01():
    result = parse("Mamu-AG:01")
    assert result.__class__ is Allele, f'Expected parse("Mamu-AG:01") to be Allele but got {result}'


def test_Mamu_AG_02011():
    result = parse("Mamu-AG:02011")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG:02011") to be Allele but got {result}'
    )


def test_Mamu_AG_02012():
    result = parse("Mamu-AG:02012")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG:02012") to be Allele but got {result}'
    )


def test_Mamu_AG_0202():
    result = parse("Mamu-AG:0202")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG:0202") to be Allele but got {result}'
    )


def test_Mamu_AG_03011():
    result = parse("Mamu-AG:03011")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG:03011") to be Allele but got {result}'
    )


def test_Mamu_AG_0302():
    result = parse("Mamu-AG:0302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-AG:0302") to be Allele but got {result}'
    )


def test_Mamu_B_00101():
    result = parse("Mamu-B:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00101") to be Allele but got {result}'
    )


def test_Mamu_B_00102():
    result = parse("Mamu-B:00102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00102") to be Allele but got {result}'
    )


def test_Mamu_B_00201():
    result = parse("Mamu-B:00201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00201") to be Allele but got {result}'
    )


def test_Mamu_B_00202():
    result = parse("Mamu-B:00202")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00202") to be Allele but got {result}'
    )


def test_Mamu_B_00301():
    result = parse("Mamu-B:00301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00301") to be Allele but got {result}'
    )


def test_Mamu_B_00302():
    result = parse("Mamu-B:00302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00302") to be Allele but got {result}'
    )


def test_Mamu_B_00401():
    result = parse("Mamu-B:00401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00401") to be Allele but got {result}'
    )


def test_Mamu_B_00501():
    result = parse("Mamu-B:00501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00501") to be Allele but got {result}'
    )


def test_Mamu_B_00502():
    result = parse("Mamu-B:00502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00502") to be Allele but got {result}'
    )


def test_Mamu_B_00601():
    result = parse("Mamu-B:00601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00601") to be Allele but got {result}'
    )


def test_Mamu_B_00602():
    result = parse("Mamu-B:00602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00602") to be Allele but got {result}'
    )


def test_Mamu_B_00701():
    result = parse("Mamu-B:00701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00701") to be Allele but got {result}'
    )


def test_Mamu_B_00702():
    result = parse("Mamu-B:00702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00702") to be Allele but got {result}'
    )


def test_Mamu_B_00703():
    result = parse("Mamu-B:00703")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00703") to be Allele but got {result}'
    )


def test_Mamu_B_00704():
    result = parse("Mamu-B:00704")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00704") to be Allele but got {result}'
    )


def test_Mamu_B_00801():
    result = parse("Mamu-B:00801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:00801") to be Allele but got {result}'
    )


def test_Mamu_B_01001():
    result = parse("Mamu-B:01001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01001") to be Allele but got {result}'
    )


def test_Mamu_B_01101():
    result = parse("Mamu-B:01101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01101") to be Allele but got {result}'
    )


def test_Mamu_B_01201():
    result = parse("Mamu-B:01201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01201") to be Allele but got {result}'
    )


def test_Mamu_B_01301():
    result = parse("Mamu-B:01301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01301") to be Allele but got {result}'
    )


def test_Mamu_B_01401():
    result = parse("Mamu-B:01401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01401") to be Allele but got {result}'
    )


def test_Mamu_B_01501():
    result = parse("Mamu-B:01501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01501") to be Allele but got {result}'
    )


def test_Mamu_B_01502():
    result = parse("Mamu-B:01502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01502") to be Allele but got {result}'
    )


def test_Mamu_B_01601():
    result = parse("Mamu-B:01601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01601") to be Allele but got {result}'
    )


def test_Mamu_B_01701():
    result = parse("Mamu-B:01701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01701") to be Allele but got {result}'
    )


def test_Mamu_B_01702():
    result = parse("Mamu-B:01702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01702") to be Allele but got {result}'
    )


def test_Mamu_B_01703():
    result = parse("Mamu-B:01703")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01703") to be Allele but got {result}'
    )


def test_Mamu_B_01801():
    result = parse("Mamu-B:01801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01801") to be Allele but got {result}'
    )


def test_Mamu_B_01901():
    result = parse("Mamu-B:01901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01901") to be Allele but got {result}'
    )


def test_Mamu_B_01902():
    result = parse("Mamu-B:01902")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01902") to be Allele but got {result}'
    )


def test_Mamu_B_01903():
    result = parse("Mamu-B:01903")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:01903") to be Allele but got {result}'
    )


def test_Mamu_B_02001():
    result = parse("Mamu-B:02001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02001") to be Allele but got {result}'
    )


def test_Mamu_B_02101():
    result = parse("Mamu-B:02101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02101") to be Allele but got {result}'
    )


def test_Mamu_B_02102():
    result = parse("Mamu-B:02102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02102") to be Allele but got {result}'
    )


def test_Mamu_B_02103():
    result = parse("Mamu-B:02103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02103") to be Allele but got {result}'
    )


def test_Mamu_B_02201():
    result = parse("Mamu-B:02201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02201") to be Allele but got {result}'
    )


def test_Mamu_B_02301():
    result = parse("Mamu-B:02301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02301") to be Allele but got {result}'
    )


def test_Mamu_B_02401():
    result = parse("Mamu-B:02401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02401") to be Allele but got {result}'
    )


def test_Mamu_B_02501():
    result = parse("Mamu-B:02501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02501") to be Allele but got {result}'
    )


def test_Mamu_B_02601():
    result = parse("Mamu-B:02601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02601") to be Allele but got {result}'
    )


def test_Mamu_B_02602():
    result = parse("Mamu-B:02602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02602") to be Allele but got {result}'
    )


def test_Mamu_B_02701():
    result = parse("Mamu-B:02701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02701") to be Allele but got {result}'
    )


def test_Mamu_B_02702():
    result = parse("Mamu-B:02702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02702") to be Allele but got {result}'
    )


def test_Mamu_B_02703():
    result = parse("Mamu-B:02703")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02703") to be Allele but got {result}'
    )


def test_Mamu_B_02801():
    result = parse("Mamu-B:02801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02801") to be Allele but got {result}'
    )


def test_Mamu_B_02802():
    result = parse("Mamu-B:02802")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02802") to be Allele but got {result}'
    )


def test_Mamu_B_02803():
    result = parse("Mamu-B:02803")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02803") to be Allele but got {result}'
    )


def test_Mamu_B_02901():
    result = parse("Mamu-B:02901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02901") to be Allele but got {result}'
    )


def test_Mamu_B_02902():
    result = parse("Mamu-B:02902")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:02902") to be Allele but got {result}'
    )


def test_Mamu_B_03001():
    result = parse("Mamu-B:03001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03001") to be Allele but got {result}'
    )


def test_Mamu_B_03002():
    result = parse("Mamu-B:03002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03002") to be Allele but got {result}'
    )


def test_Mamu_B_03003():
    result = parse("Mamu-B:03003")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03003") to be Allele but got {result}'
    )


def test_Mamu_B_03004():
    result = parse("Mamu-B:03004")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03004") to be Allele but got {result}'
    )


def test_Mamu_B_03005():
    result = parse("Mamu-B:03005")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03005") to be Allele but got {result}'
    )


def test_Mamu_B_03101():
    result = parse("Mamu-B:03101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03101") to be Allele but got {result}'
    )


def test_Mamu_B_03102():
    result = parse("Mamu-B:03102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03102") to be Allele but got {result}'
    )


def test_Mamu_B_03103():
    result = parse("Mamu-B:03103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03103") to be Allele but got {result}'
    )


def test_Mamu_B_03201():
    result = parse("Mamu-B:03201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03201") to be Allele but got {result}'
    )


def test_Mamu_B_03301():
    result = parse("Mamu-B:03301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03301") to be Allele but got {result}'
    )


def test_Mamu_B_03401():
    result = parse("Mamu-B:03401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03401") to be Allele but got {result}'
    )


def test_Mamu_B_03501():
    result = parse("Mamu-B:03501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03501") to be Allele but got {result}'
    )


def test_Mamu_B_03601():
    result = parse("Mamu-B:03601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03601") to be Allele but got {result}'
    )


def test_Mamu_B_03602():
    result = parse("Mamu-B:03602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03602") to be Allele but got {result}'
    )


def test_Mamu_B_03701():
    result = parse("Mamu-B:03701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03701") to be Allele but got {result}'
    )


def test_Mamu_B_03801():
    result = parse("Mamu-B:03801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03801") to be Allele but got {result}'
    )


def test_Mamu_B_03802():
    result = parse("Mamu-B:03802")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03802") to be Allele but got {result}'
    )


def test_Mamu_B_03901():
    result = parse("Mamu-B:03901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:03901") to be Allele but got {result}'
    )


def test_Mamu_B_04001():
    result = parse("Mamu-B:04001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04001") to be Allele but got {result}'
    )


def test_Mamu_B_04002():
    result = parse("Mamu-B:04002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04002") to be Allele but got {result}'
    )


def test_Mamu_B_04101():
    result = parse("Mamu-B:04101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04101") to be Allele but got {result}'
    )


def test_Mamu_B_04201():
    result = parse("Mamu-B:04201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04201") to be Allele but got {result}'
    )


def test_Mamu_B_04301():
    result = parse("Mamu-B:04301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04301") to be Allele but got {result}'
    )


def test_Mamu_B_04401():
    result = parse("Mamu-B:04401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04401") to be Allele but got {result}'
    )


def test_Mamu_B_04402():
    result = parse("Mamu-B:04402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04402") to be Allele but got {result}'
    )


def test_Mamu_B_04403():
    result = parse("Mamu-B:04403")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04403") to be Allele but got {result}'
    )


def test_Mamu_B_04404():
    result = parse("Mamu-B:04404")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04404") to be Allele but got {result}'
    )


def test_Mamu_B_04405():
    result = parse("Mamu-B:04405")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04405") to be Allele but got {result}'
    )


def test_Mamu_B_04501():
    result = parse("Mamu-B:04501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04501") to be Allele but got {result}'
    )


def test_Mamu_B_04502():
    result = parse("Mamu-B:04502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04502") to be Allele but got {result}'
    )


def test_Mamu_B_04503():
    result = parse("Mamu-B:04503")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04503") to be Allele but got {result}'
    )


def test_Mamu_B_04504():
    result = parse("Mamu-B:04504")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04504") to be Allele but got {result}'
    )


def test_Mamu_B_04601():
    result = parse("Mamu-B:04601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04601") to be Allele but got {result}'
    )


def test_Mamu_B_04602():
    result = parse("Mamu-B:04602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04602") to be Allele but got {result}'
    )


def test_Mamu_B_04603():
    result = parse("Mamu-B:04603")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04603") to be Allele but got {result}'
    )


def test_Mamu_B_04604():
    result = parse("Mamu-B:04604")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04604") to be Allele but got {result}'
    )


def test_Mamu_B_04605():
    result = parse("Mamu-B:04605")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04605") to be Allele but got {result}'
    )


def test_Mamu_B_04607():
    result = parse("Mamu-B:04607")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04607") to be Allele but got {result}'
    )


def test_Mamu_B_04608():
    result = parse("Mamu-B:04608")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04608") to be Allele but got {result}'
    )


def test_Mamu_B_04609():
    result = parse("Mamu-B:04609")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04609") to be Allele but got {result}'
    )


def test_Mamu_B_04610():
    result = parse("Mamu-B:04610")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04610") to be Allele but got {result}'
    )


def test_Mamu_B_04611():
    result = parse("Mamu-B:04611")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04611") to be Allele but got {result}'
    )


def test_Mamu_B_04612():
    result = parse("Mamu-B:04612")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04612") to be Allele but got {result}'
    )


def test_Mamu_B_04613():
    result = parse("Mamu-B:04613")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04613") to be Allele but got {result}'
    )


def test_Mamu_B_04614():
    result = parse("Mamu-B:04614")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04614") to be Allele but got {result}'
    )


def test_Mamu_B_04615():
    result = parse("Mamu-B:04615")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04615") to be Allele but got {result}'
    )


def test_Mamu_B_04616():
    result = parse("Mamu-B:04616")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04616") to be Allele but got {result}'
    )


def test_Mamu_B_04617():
    result = parse("Mamu-B:04617")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04617") to be Allele but got {result}'
    )


def test_Mamu_B_04701():
    result = parse("Mamu-B:04701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04701") to be Allele but got {result}'
    )


def test_Mamu_B_04702():
    result = parse("Mamu-B:04702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04702") to be Allele but got {result}'
    )


def test_Mamu_B_04703():
    result = parse("Mamu-B:04703")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04703") to be Allele but got {result}'
    )


def test_Mamu_B_04704():
    result = parse("Mamu-B:04704")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04704") to be Allele but got {result}'
    )


def test_Mamu_B_04705():
    result = parse("Mamu-B:04705")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04705") to be Allele but got {result}'
    )


def test_Mamu_B_04801():
    result = parse("Mamu-B:04801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04801") to be Allele but got {result}'
    )


def test_Mamu_B_04802():
    result = parse("Mamu-B:04802")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04802") to be Allele but got {result}'
    )


def test_Mamu_B_04901():
    result = parse("Mamu-B:04901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:04901") to be Allele but got {result}'
    )


def test_Mamu_B_05002():
    result = parse("Mamu-B:05002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05002") to be Allele but got {result}'
    )


def test_Mamu_B_05101():
    result = parse("Mamu-B:05101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05101") to be Allele but got {result}'
    )


def test_Mamu_B_05102():
    result = parse("Mamu-B:05102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05102") to be Allele but got {result}'
    )


def test_Mamu_B_05103():
    result = parse("Mamu-B:05103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05103") to be Allele but got {result}'
    )


def test_Mamu_B_05104():
    result = parse("Mamu-B:05104")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05104") to be Allele but got {result}'
    )


def test_Mamu_B_05105():
    result = parse("Mamu-B:05105")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05105") to be Allele but got {result}'
    )


def test_Mamu_B_05201():
    result = parse("Mamu-B:05201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05201") to be Allele but got {result}'
    )


def test_Mamu_B_05301():
    result = parse("Mamu-B:05301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05301") to be Allele but got {result}'
    )


def test_Mamu_B_05302():
    result = parse("Mamu-B:05302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05302") to be Allele but got {result}'
    )


def test_Mamu_B_05401():
    result = parse("Mamu-B:05401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05401") to be Allele but got {result}'
    )


def test_Mamu_B_05501():
    result = parse("Mamu-B:05501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05501") to be Allele but got {result}'
    )


def test_Mamu_B_05601():
    result = parse("Mamu-B:05601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05601") to be Allele but got {result}'
    )


def test_Mamu_B_05602():
    result = parse("Mamu-B:05602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05602") to be Allele but got {result}'
    )


def test_Mamu_B_05701():
    result = parse("Mamu-B:05701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05701") to be Allele but got {result}'
    )


def test_Mamu_B_05702():
    result = parse("Mamu-B:05702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05702") to be Allele but got {result}'
    )


def test_Mamu_B_05802():
    result = parse("Mamu-B:05802")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05802") to be Allele but got {result}'
    )


def test_Mamu_B_05901():
    result = parse("Mamu-B:05901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:05901") to be Allele but got {result}'
    )


def test_Mamu_B_06001():
    result = parse("Mamu-B:06001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06001") to be Allele but got {result}'
    )


def test_Mamu_B_06002():
    result = parse("Mamu-B:06002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06002") to be Allele but got {result}'
    )


def test_Mamu_B_06003():
    result = parse("Mamu-B:06003")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06003") to be Allele but got {result}'
    )


def test_Mamu_B_06101():
    result = parse("Mamu-B:06101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06101") to be Allele but got {result}'
    )


def test_Mamu_B_06102():
    result = parse("Mamu-B:06102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06102") to be Allele but got {result}'
    )


def test_Mamu_B_06103():
    result = parse("Mamu-B:06103")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06103") to be Allele but got {result}'
    )


def test_Mamu_B_06301():
    result = parse("Mamu-B:06301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06301") to be Allele but got {result}'
    )


def test_Mamu_B_06302():
    result = parse("Mamu-B:06302")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06302") to be Allele but got {result}'
    )


def test_Mamu_B_06401():
    result = parse("Mamu-B:06401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06401") to be Allele but got {result}'
    )


def test_Mamu_B_06402():
    result = parse("Mamu-B:06402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06402") to be Allele but got {result}'
    )


def test_Mamu_B_06501():
    result = parse("Mamu-B:06501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06501") to be Allele but got {result}'
    )


def test_Mamu_B_06502():
    result = parse("Mamu-B:06502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06502") to be Allele but got {result}'
    )


def test_Mamu_B_06503():
    result = parse("Mamu-B:06503")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06503") to be Allele but got {result}'
    )


def test_Mamu_B_06601():
    result = parse("Mamu-B:06601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06601") to be Allele but got {result}'
    )


def test_Mamu_B_06701():
    result = parse("Mamu-B:06701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06701") to be Allele but got {result}'
    )


def test_Mamu_B_06702():
    result = parse("Mamu-B:06702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06702") to be Allele but got {result}'
    )


def test_Mamu_B_06801():
    result = parse("Mamu-B:06801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06801") to be Allele but got {result}'
    )


def test_Mamu_B_06802():
    result = parse("Mamu-B:06802")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06802") to be Allele but got {result}'
    )


def test_Mamu_B_06803():
    result = parse("Mamu-B:06803")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06803") to be Allele but got {result}'
    )


def test_Mamu_B_06804():
    result = parse("Mamu-B:06804")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06804") to be Allele but got {result}'
    )


def test_Mamu_B_06805():
    result = parse("Mamu-B:06805")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06805") to be Allele but got {result}'
    )


def test_Mamu_B_06901():
    result = parse("Mamu-B:06901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06901") to be Allele but got {result}'
    )


def test_Mamu_B_06902():
    result = parse("Mamu-B:06902")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06902") to be Allele but got {result}'
    )


def test_Mamu_B_06903():
    result = parse("Mamu-B:06903")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06903") to be Allele but got {result}'
    )


def test_Mamu_B_06904():
    result = parse("Mamu-B:06904")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:06904") to be Allele but got {result}'
    )


def test_Mamu_B_07001():
    result = parse("Mamu-B:07001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07001") to be Allele but got {result}'
    )


def test_Mamu_B_07002():
    result = parse("Mamu-B:07002")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07002") to be Allele but got {result}'
    )


def test_Mamu_B_07101():
    result = parse("Mamu-B:07101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07101") to be Allele but got {result}'
    )


def test_Mamu_B_07201():
    result = parse("Mamu-B:07201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07201") to be Allele but got {result}'
    )


def test_Mamu_B_07202():
    result = parse("Mamu-B:07202")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07202") to be Allele but got {result}'
    )


def test_Mamu_B_07301():
    result = parse("Mamu-B:07301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07301") to be Allele but got {result}'
    )


def test_Mamu_B_07401():
    result = parse("Mamu-B:07401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07401") to be Allele but got {result}'
    )


def test_Mamu_B_07402():
    result = parse("Mamu-B:07402")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07402") to be Allele but got {result}'
    )


def test_Mamu_B_07501():
    result = parse("Mamu-B:07501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07501") to be Allele but got {result}'
    )


def test_Mamu_B_07502():
    result = parse("Mamu-B:07502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07502") to be Allele but got {result}'
    )


def test_Mamu_B_07601():
    result = parse("Mamu-B:07601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07601") to be Allele but got {result}'
    )


def test_Mamu_B_07602():
    result = parse("Mamu-B:07602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07602") to be Allele but got {result}'
    )


def test_Mamu_B_07701():
    result = parse("Mamu-B:07701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07701") to be Allele but got {result}'
    )


def test_Mamu_B_07702():
    result = parse("Mamu-B:07702")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07702") to be Allele but got {result}'
    )


def test_Mamu_B_07801():
    result = parse("Mamu-B:07801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07801") to be Allele but got {result}'
    )


def test_Mamu_B_07901():
    result = parse("Mamu-B:07901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07901") to be Allele but got {result}'
    )


def test_Mamu_B_07902():
    result = parse("Mamu-B:07902")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07902") to be Allele but got {result}'
    )


def test_Mamu_B_07903():
    result = parse("Mamu-B:07903")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:07903") to be Allele but got {result}'
    )


def test_Mamu_B_08001():
    result = parse("Mamu-B:08001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08001") to be Allele but got {result}'
    )


def test_Mamu_B_08101():
    result = parse("Mamu-B:08101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08101") to be Allele but got {result}'
    )


def test_Mamu_B_08102():
    result = parse("Mamu-B:08102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08102") to be Allele but got {result}'
    )


def test_Mamu_B_08201():
    result = parse("Mamu-B:08201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08201") to be Allele but got {result}'
    )


def test_Mamu_B_08202():
    result = parse("Mamu-B:08202")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08202") to be Allele but got {result}'
    )


def test_Mamu_B_08301():
    result = parse("Mamu-B:08301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08301") to be Allele but got {result}'
    )


def test_Mamu_B_08401():
    result = parse("Mamu-B:08401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08401") to be Allele but got {result}'
    )


def test_Mamu_B_08501():
    result = parse("Mamu-B:08501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08501") to be Allele but got {result}'
    )


def test_Mamu_B_08502():
    result = parse("Mamu-B:08502")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08502") to be Allele but got {result}'
    )


def test_Mamu_B_08601():
    result = parse("Mamu-B:08601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08601") to be Allele but got {result}'
    )


def test_Mamu_B_08602():
    result = parse("Mamu-B:08602")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08602") to be Allele but got {result}'
    )


def test_Mamu_B_08603():
    result = parse("Mamu-B:08603")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08603") to be Allele but got {result}'
    )


def test_Mamu_B_08701():
    result = parse("Mamu-B:08701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08701") to be Allele but got {result}'
    )


def test_Mamu_B_08801():
    result = parse("Mamu-B:08801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08801") to be Allele but got {result}'
    )


def test_Mamu_B_08901():
    result = parse("Mamu-B:08901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:08901") to be Allele but got {result}'
    )


def test_Mamu_B_09001():
    result = parse("Mamu-B:09001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09001") to be Allele but got {result}'
    )


def test_Mamu_B_09101():
    result = parse("Mamu-B:09101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09101") to be Allele but got {result}'
    )


def test_Mamu_B_09102():
    result = parse("Mamu-B:09102")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09102") to be Allele but got {result}'
    )


def test_Mamu_B_09201():
    result = parse("Mamu-B:09201")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09201") to be Allele but got {result}'
    )


def test_Mamu_B_09301():
    result = parse("Mamu-B:09301")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09301") to be Allele but got {result}'
    )


def test_Mamu_B_09401():
    result = parse("Mamu-B:09401")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09401") to be Allele but got {result}'
    )


def test_Mamu_B_09501():
    result = parse("Mamu-B:09501")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09501") to be Allele but got {result}'
    )


def test_Mamu_B_09601():
    result = parse("Mamu-B:09601")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09601") to be Allele but got {result}'
    )


def test_Mamu_B_09701():
    result = parse("Mamu-B:09701")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09701") to be Allele but got {result}'
    )


def test_Mamu_B_09801():
    result = parse("Mamu-B:09801")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09801") to be Allele but got {result}'
    )


def test_Mamu_B_09901():
    result = parse("Mamu-B:09901")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:09901") to be Allele but got {result}'
    )


def test_Mamu_B_10001():
    result = parse("Mamu-B:10001")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:10001") to be Allele but got {result}'
    )


def test_Mamu_B_10101():
    result = parse("Mamu-B:10101")
    assert result.__class__ is Allele, (
        f'Expected parse("Mamu-B:10101") to be Allele but got {result}'
    )


def test_Patr_A0101():
    result = parse("Patr-A0101")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0101"): {result}'
    )


def test_Patr_A0201():
    result = parse("Patr-A0201")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0201"): {result}'
    )


def test_Patr_A0301():
    result = parse("Patr-A0301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0301"): {result}'
    )


def test_Patr_A0302():
    result = parse("Patr-A0302")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0302"): {result}'
    )


def test_Patr_A0401():
    result = parse("Patr-A0401")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0401"): {result}'
    )


def test_Patr_A0402():
    result = parse("Patr-A0402")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0402"): {result}'
    )


def test_Patr_A0404():
    result = parse("Patr-A0404")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0404"): {result}'
    )


def test_Patr_A0501():
    result = parse("Patr-A0501")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0501"): {result}'
    )


def test_Patr_A0601():
    result = parse("Patr-A0601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0601"): {result}'
    )


def test_Patr_A0602():
    result = parse("Patr-A0602")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0602"): {result}'
    )


def test_Patr_A0701():
    result = parse("Patr-A0701")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0701"): {result}'
    )


def test_Patr_A0801():
    result = parse("Patr-A0801")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0801"): {result}'
    )


def test_Patr_A0802():
    result = parse("Patr-A0802")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0802"): {result}'
    )


def test_Patr_A0803():
    result = parse("Patr-A0803")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0803"): {result}'
    )


def test_Patr_A0901():
    result = parse("Patr-A0901")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0901"): {result}'
    )


def test_Patr_A0902():
    result = parse("Patr-A0902")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A0902"): {result}'
    )


def test_Patr_A1001():
    result = parse("Patr-A1001")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1001"): {result}'
    )


def test_Patr_A1101():
    result = parse("Patr-A1101")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1101"): {result}'
    )


def test_Patr_A1201():
    result = parse("Patr-A1201")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1201"): {result}'
    )


def test_Patr_A1301():
    result = parse("Patr-A1301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1301"): {result}'
    )


def test_Patr_A1401():
    result = parse("Patr-A1401")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1401"): {result}'
    )


def test_Patr_A1501():
    result = parse("Patr-A1501")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1501"): {result}'
    )


def test_Patr_A1502():
    result = parse("Patr-A1502")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1502"): {result}'
    )


def test_Patr_A1601():
    result = parse("Patr-A1601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1601"): {result}'
    )


def test_Patr_A1701():
    result = parse("Patr-A1701")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1701"): {result}'
    )


def test_Patr_A1702():
    result = parse("Patr-A1702")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1702"): {result}'
    )


def test_Patr_A1703():
    result = parse("Patr-A1703")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1703"): {result}'
    )


def test_Patr_A1801():
    result = parse("Patr-A1801")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A1801"): {result}'
    )


def test_Patr_A2301():
    result = parse("Patr-A2301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A2301"): {result}'
    )


def test_Patr_A2401():
    result = parse("Patr-A2401")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-A2401"): {result}'
    )


def test_Patr_B0101():
    result = parse("Patr-B0101")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0101"): {result}'
    )


def test_Patr_B0102():
    result = parse("Patr-B0102")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0102"): {result}'
    )


def test_Patr_B0201():
    result = parse("Patr-B0201")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0201"): {result}'
    )


def test_Patr_B0203():
    result = parse("Patr-B0203")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0203"): {result}'
    )


def test_Patr_B0301():
    result = parse("Patr-B0301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0301"): {result}'
    )


def test_Patr_B0302():
    result = parse("Patr-B0302")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0302"): {result}'
    )


def test_Patr_B0401():
    result = parse("Patr-B0401")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0401"): {result}'
    )


def test_Patr_B0402():
    result = parse("Patr-B0402")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0402"): {result}'
    )


def test_Patr_B0501():
    result = parse("Patr-B0501")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0501"): {result}'
    )


def test_Patr_B0502():
    result = parse("Patr-B0502")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0502"): {result}'
    )


def test_Patr_B0601():
    result = parse("Patr-B0601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0601"): {result}'
    )


def test_Patr_B0701():
    result = parse("Patr-B0701")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0701"): {result}'
    )


def test_Patr_B0702():
    result = parse("Patr-B0702")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0702"): {result}'
    )


def test_Patr_B0801():
    result = parse("Patr-B0801")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0801"): {result}'
    )


def test_Patr_B0802():
    result = parse("Patr-B0802")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0802"): {result}'
    )


def test_Patr_B0901():
    result = parse("Patr-B0901")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B0901"): {result}'
    )


def test_Patr_B1001():
    result = parse("Patr-B1001")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1001"): {result}'
    )


def test_Patr_B1101():
    result = parse("Patr-B1101")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1101"): {result}'
    )


def test_Patr_B1102():
    result = parse("Patr-B1102")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1102"): {result}'
    )


def test_Patr_B1202():
    result = parse("Patr-B1202")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1202"): {result}'
    )


def test_Patr_B1301():
    result = parse("Patr-B1301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1301"): {result}'
    )


def test_Patr_B1401():
    result = parse("Patr-B1401")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1401"): {result}'
    )


def test_Patr_B1601():
    result = parse("Patr-B1601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1601"): {result}'
    )


def test_Patr_B1602():
    result = parse("Patr-B1602")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1602"): {result}'
    )


def test_Patr_B1701():
    result = parse("Patr-B1701")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1701"): {result}'
    )


def test_Patr_B1702():
    result = parse("Patr-B1702")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1702"): {result}'
    )


def test_Patr_B1703():
    result = parse("Patr-B1703")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1703"): {result}'
    )


def test_Patr_B1801():
    result = parse("Patr-B1801")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1801"): {result}'
    )


def test_Patr_B1901():
    result = parse("Patr-B1901")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B1901"): {result}'
    )


def test_Patr_B2001():
    result = parse("Patr-B2001")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2001"): {result}'
    )


def test_Patr_B2101():
    result = parse("Patr-B2101")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2101"): {result}'
    )


def test_Patr_B2201():
    result = parse("Patr-B2201")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2201"): {result}'
    )


def test_Patr_B2202():
    result = parse("Patr-B2202")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2202"): {result}'
    )


def test_Patr_B2301():
    result = parse("Patr-B2301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2301"): {result}'
    )


def test_Patr_B2302():
    result = parse("Patr-B2302")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2302"): {result}'
    )


def test_Patr_B2303():
    result = parse("Patr-B2303")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2303"): {result}'
    )


def test_Patr_B2401():
    result = parse("Patr-B2401")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2401"): {result}'
    )


def test_Patr_B2402():
    result = parse("Patr-B2402")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2402"): {result}'
    )


def test_Patr_B2501():
    result = parse("Patr-B2501")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2501"): {result}'
    )


def test_Patr_B2601():
    result = parse("Patr-B2601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2601"): {result}'
    )


def test_Patr_B2701():
    result = parse("Patr-B2701")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2701"): {result}'
    )


def test_Patr_B2801():
    result = parse("Patr-B2801")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2801"): {result}'
    )


def test_Patr_B2901():
    result = parse("Patr-B2901")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B2901"): {result}'
    )


def test_Patr_B3001():
    result = parse("Patr-B3001")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B3001"): {result}'
    )


def test_Patr_B3501():
    result = parse("Patr-B3501")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B3501"): {result}'
    )


def test_Patr_B3601():
    result = parse("Patr-B3601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B3601"): {result}'
    )


def test_Patr_B3701():
    result = parse("Patr-B3701")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-B3701"): {result}'
    )


def test_Patr_C0201():
    result = parse("Patr-C0201")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0201"): {result}'
    )


def test_Patr_C0202():
    result = parse("Patr-C0202")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0202"): {result}'
    )


def test_Patr_C0203():
    result = parse("Patr-C0203")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0203"): {result}'
    )


def test_Patr_C0204():
    result = parse("Patr-C0204")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0204"): {result}'
    )


def test_Patr_C0205():
    result = parse("Patr-C0205")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0205"): {result}'
    )


def test_Patr_C0206():
    result = parse("Patr-C0206")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0206"): {result}'
    )


def test_Patr_C0301():
    result = parse("Patr-C0301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0301"): {result}'
    )


def test_Patr_C0302():
    result = parse("Patr-C0302")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0302"): {result}'
    )


def test_Patr_C0303():
    result = parse("Patr-C0303")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0303"): {result}'
    )


def test_Patr_C0304():
    result = parse("Patr-C0304")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0304"): {result}'
    )


def test_Patr_C0401():
    result = parse("Patr-C0401")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0401"): {result}'
    )


def test_Patr_C0501():
    result = parse("Patr-C0501")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0501"): {result}'
    )


def test_Patr_C0502():
    result = parse("Patr-C0502")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0502"): {result}'
    )


def test_Patr_C0601():
    result = parse("Patr-C0601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0601"): {result}'
    )


def test_Patr_C0701():
    result = parse("Patr-C0701")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0701"): {result}'
    )


def test_Patr_C0801():
    result = parse("Patr-C0801")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0801"): {result}'
    )


def test_Patr_C0901():
    result = parse("Patr-C0901")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0901"): {result}'
    )


def test_Patr_C0902():
    result = parse("Patr-C0902")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0902"): {result}'
    )


def test_Patr_C0903():
    result = parse("Patr-C0903")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0903"): {result}'
    )


def test_Patr_C0904():
    result = parse("Patr-C0904")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0904"): {result}'
    )


def test_Patr_C0905():
    result = parse("Patr-C0905")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C0905"): {result}'
    )


def test_Patr_C1001():
    result = parse("Patr-C1001")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C1001"): {result}'
    )


def test_Patr_C1101():
    result = parse("Patr-C1101")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C1101"): {result}'
    )


def test_Patr_C1201():
    result = parse("Patr-C1201")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C1201"): {result}'
    )


def test_Patr_C1301():
    result = parse("Patr-C1301")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C1301"): {result}'
    )


def test_Patr_C1302():
    result = parse("Patr-C1302")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C1302"): {result}'
    )


def test_Patr_C1501():
    result = parse("Patr-C1501")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C1501"): {result}'
    )


def test_Patr_C1601():
    result = parse("Patr-C1601")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Patr-C1601"): {result}'
    )


def test_SLA_1_0101():
    result = parse("SLA-1:0101")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0101") to be Allele but got {result}'


def test_SLA_1_0201():
    result = parse("SLA-1:0201")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0201") to be Allele but got {result}'


def test_SLA_1_0202():
    result = parse("SLA-1:0202")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0202") to be Allele but got {result}'


def test_SLA_1_0401():
    result = parse("SLA-1:0401")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0401") to be Allele but got {result}'


def test_SLA_1_0501():
    result = parse("SLA-1:0501")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0501") to be Allele but got {result}'


def test_SLA_1_0601():
    result = parse("SLA-1:0601")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0601") to be Allele but got {result}'


def test_SLA_1_0701():
    result = parse("SLA-1:0701")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0701") to be Allele but got {result}'


def test_SLA_1_0702():
    result = parse("SLA-1:0702")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0702") to be Allele but got {result}'


def test_SLA_1_0801():
    result = parse("SLA-1:0801")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:0801") to be Allele but got {result}'


def test_SLA_1_1101():
    result = parse("SLA-1:1101")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:1101") to be Allele but got {result}'


def test_SLA_1_1201():
    result = parse("SLA-1:1201")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:1201") to be Allele but got {result}'


def test_SLA_1_1301():
    result = parse("SLA-1:1301")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:1301") to be Allele but got {result}'


def test_SLA_2_0101():
    result = parse("SLA-2:0101")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0101") to be Allele but got {result}'


def test_SLA_2_0102():
    result = parse("SLA-2:0102")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0102") to be Allele but got {result}'


def test_SLA_2_0201():
    result = parse("SLA-2:0201")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0201") to be Allele but got {result}'


def test_SLA_2_0202():
    result = parse("SLA-2:0202")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0202") to be Allele but got {result}'


def test_SLA_2_0301():
    result = parse("SLA-2:0301")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0301") to be Allele but got {result}'


def test_SLA_2_0302():
    result = parse("SLA-2:0302")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0302") to be Allele but got {result}'


def test_SLA_2_0401():
    result = parse("SLA-2:0401")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0401") to be Allele but got {result}'


def test_SLA_2_0402():
    result = parse("SLA-2:0402")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0402") to be Allele but got {result}'


def test_SLA_2_0501():
    result = parse("SLA-2:0501")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0501") to be Allele but got {result}'


def test_SLA_2_0502():
    result = parse("SLA-2:0502")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0502") to be Allele but got {result}'


def test_SLA_2_0601():
    result = parse("SLA-2:0601")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0601") to be Allele but got {result}'


def test_SLA_2_0701():
    result = parse("SLA-2:0701")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:0701") to be Allele but got {result}'


def test_SLA_2_1001():
    result = parse("SLA-2:1001")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:1001") to be Allele but got {result}'


def test_SLA_2_1002():
    result = parse("SLA-2:1002")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:1002") to be Allele but got {result}'


def test_SLA_2_1101():
    result = parse("SLA-2:1101")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:1101") to be Allele but got {result}'


def test_SLA_2_1201():
    result = parse("SLA-2:1201")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:1201") to be Allele but got {result}'


def test_SLA_3_0101():
    result = parse("SLA-3:0101")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0101") to be Allele but got {result}'


def test_SLA_3_0301():
    result = parse("SLA-3:0301")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0301") to be Allele but got {result}'


def test_SLA_3_0302():
    result = parse("SLA-3:0302")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0302") to be Allele but got {result}'


def test_SLA_3_0303():
    result = parse("SLA-3:0303")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0303") to be Allele but got {result}'


def test_SLA_3_0304():
    result = parse("SLA-3:0304")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0304") to be Allele but got {result}'


def test_SLA_3_0401():
    result = parse("SLA-3:0401")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0401") to be Allele but got {result}'


def test_SLA_3_0501():
    result = parse("SLA-3:0501")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0501") to be Allele but got {result}'


def test_SLA_3_0502():
    result = parse("SLA-3:0502")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0502") to be Allele but got {result}'


def test_SLA_3_0503():
    result = parse("SLA-3:0503")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0503") to be Allele but got {result}'


def test_SLA_3_0601():
    result = parse("SLA-3:0601")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0601") to be Allele but got {result}'


def test_SLA_3_0602():
    result = parse("SLA-3:0602")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0602") to be Allele but got {result}'


def test_SLA_3_0701():
    result = parse("SLA-3:0701")
    assert result.__class__ is Allele, f'Expected parse("SLA-3:0701") to be Allele but got {result}'


def test_SLA_6_0101():
    result = parse("SLA-6:0101")
    assert result.__class__ is Allele, f'Expected parse("SLA-6:0101") to be Allele but got {result}'


def test_SLA_6_0102():
    result = parse("SLA-6:0102")
    assert result.__class__ is Allele, f'Expected parse("SLA-6:0102") to be Allele but got {result}'


def test_SLA_6_0103():
    result = parse("SLA-6:0103")
    assert result.__class__ is Allele, f'Expected parse("SLA-6:0103") to be Allele but got {result}'


def test_SLA_6_0104():
    result = parse("SLA-6:0104")
    assert result.__class__ is Allele, f'Expected parse("SLA-6:0104") to be Allele but got {result}'


def test_SLA_6_0105():
    result = parse("SLA-6:0105")
    assert result.__class__ is Allele, f'Expected parse("SLA-6:0105") to be Allele but got {result}'


def test_SLA_1_CHANGDA():
    result = parse("SLA-1-CHANGDA")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-CHANGDA"): {result}'
    )


def test_SLA_1_HB01():
    result = parse("SLA-1-HB01")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-HB01"): {result}'
    )


def test_SLA_1_HB02():
    result = parse("SLA-1-HB02")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-HB02"): {result}'
    )


def test_SLA_1_HB03():
    result = parse("SLA-1-HB03")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-HB03"): {result}'
    )


def test_SLA_1_HB04():
    result = parse("SLA-1-HB04")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-HB04"): {result}'
    )


def test_SLA_1_LWH():
    result = parse("SLA-1-LWH")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-LWH"): {result}'
    )


def test_SLA_1_TPK():
    result = parse("SLA-1-TPK")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-TPK"): {result}'
    )


def test_SLA_1_YC():
    result = parse("SLA-1-YC")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-YC"): {result}'
    )


def test_SLA_1_YDL01():
    result = parse("SLA-1-YDL01")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-YDL01"): {result}'
    )


def test_SLA_1_YTH():
    result = parse("SLA-1-YTH")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-1-YTH"): {result}'
    )


def test_SLA_1_es11():
    result = parse("SLA-1:es11")
    assert result.__class__ is Allele, f'Expected parse("SLA-1:es11") to be Allele but got {result}'


def test_SLA_2_YDL02():
    result = parse("SLA-2-YDL02")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-2-YDL02"): {result}'
    )


def test_SLA_2_CDY_AA():
    result = parse("SLA-2:CDY.AA")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2:CDY.AA") to be Allele but got {result}'
    )


def test_SLA_2_HB01():
    result = parse("SLA-2:HB01")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:HB01") to be Allele but got {result}'


def test_SLA_2_LWH_AA():
    result = parse("SLA-2:LWH.AA")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2:LWH.AA") to be Allele but got {result}'
    )


def test_SLA_2_TPK_AA():
    result = parse("SLA-2:TPK.AA")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2:TPK.AA") to be Allele but got {result}'
    )


def test_SLA_2_YC_AA():
    result = parse("SLA-2:YC.AA")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2:YC.AA") to be Allele but got {result}'
    )


def test_SLA_2_YDL_AA():
    result = parse("SLA-2:YDL.AA")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2:YDL.AA") to be Allele but got {result}'
    )


def test_SLA_2_YDY_AA():
    result = parse("SLA-2:YDY.AA")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2:YDY.AA") to be Allele but got {result}'
    )


def test_SLA_2_YTH_AA():
    result = parse("SLA-2:YTH.AA")
    assert result.__class__ is Allele, (
        f'Expected parse("SLA-2:YTH.AA") to be Allele but got {result}'
    )


def test_SLA_2_es22():
    result = parse("SLA-2:es22")
    assert result.__class__ is Allele, f'Expected parse("SLA-2:es22") to be Allele but got {result}'


def test_SLA_3_CDY():
    result = parse("SLA-3-CDY")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-CDY"): {result}'
    )


def test_SLA_3_HB01():
    result = parse("SLA-3-HB01")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-HB01"): {result}'
    )


def test_SLA_3_LWH():
    result = parse("SLA-3-LWH")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-LWH"): {result}'
    )


def test_SLA_3_TPK():
    result = parse("SLA-3-TPK")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-TPK"): {result}'
    )


def test_SLA_3_YC():
    result = parse("SLA-3-YC")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-YC"): {result}'
    )


def test_SLA_3_YDL():
    result = parse("SLA-3-YDL")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-YDL"): {result}'
    )


def test_SLA_3_YDY01():
    result = parse("SLA-3-YDY01")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-YDY01"): {result}'
    )


def test_SLA_3_YDY02():
    result = parse("SLA-3-YDY02")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-YDY02"): {result}'
    )


def test_SLA_3_YTH():
    result = parse("SLA-3-YTH")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("SLA-3-YTH"): {result}'
    )


def test_H_2_Db():
    result = parse("H-2-Db")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Db"): {result}'
    )


def test_H_2_Dd():
    result = parse("H-2-Dd")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Dd"): {result}'
    )


def test_H_2_Kb():
    result = parse("H-2-Kb")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Kb"): {result}'
    )


def test_H_2_Kd():
    result = parse("H-2-Kd")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Kd"): {result}'
    )


def test_H_2_Kk():
    result = parse("H-2-Kk")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Kk"): {result}'
    )


def test_H_2_Ld():
    result = parse("H-2-Ld")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Ld"): {result}'
    )


def test_H_2_Qa2():
    result = parse("H-2-Qa2")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Qa2"): {result}'
    )


def test_H_2_Qa1():
    result = parse("H-2-Qa1")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("H-2-Qa1"): {result}'
    )


def test_Gogo_B0101():
    result = parse("Gogo-B0101")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("Gogo-B0101"): {result}'
    )


def test_HLA_G01_01():
    result = parse("HLA-G01:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:01") to be Allele but got {result}'


def test_HLA_G01_02():
    result = parse("HLA-G01:02")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:02") to be Allele but got {result}'


def test_HLA_G01_03():
    result = parse("HLA-G01:03")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:03") to be Allele but got {result}'


def test_HLA_G01_04():
    result = parse("HLA-G01:04")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:04") to be Allele but got {result}'


def test_HLA_G01_06():
    result = parse("HLA-G01:06")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:06") to be Allele but got {result}'


def test_HLA_G01_07():
    result = parse("HLA-G01:07")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:07") to be Allele but got {result}'


def test_HLA_G01_08():
    result = parse("HLA-G01:08")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:08") to be Allele but got {result}'


def test_HLA_G01_09():
    result = parse("HLA-G01:09")
    assert result.__class__ is Allele, f'Expected parse("HLA-G01:09") to be Allele but got {result}'


def test_HLA_E01_01():
    result = parse("HLA-E01:01")
    assert result.__class__ is Allele, f'Expected parse("HLA-E01:01") to be Allele but got {result}'


def test_BoLA_N_00101():
    result = parse("BoLA-N:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00101") to be Allele but got {result}'
    )


def test_BoLA_N_00102():
    result = parse("BoLA-N:00102")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00102") to be Allele but got {result}'
    )


def test_BoLA_N_00103():
    result = parse("BoLA-N:00103")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00103") to be Allele but got {result}'
    )


def test_BoLA_N_00201():
    result = parse("BoLA-N:00201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00201") to be Allele but got {result}'
    )


def test_BoLA_N_00301():
    result = parse("BoLA-N:00301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00301") to be Allele but got {result}'
    )


def test_BoLA_N_00401():
    result = parse("BoLA-N:00401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00401") to be Allele but got {result}'
    )


def test_BoLA_N_00402():
    result = parse("BoLA-N:00402")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00402") to be Allele but got {result}'
    )


def test_BoLA_N_00501():
    result = parse("BoLA-N:00501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00501") to be Allele but got {result}'
    )


def test_BoLA_N_00601():
    result = parse("BoLA-N:00601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00601") to be Allele but got {result}'
    )


def test_BoLA_N_00602():
    result = parse("BoLA-N:00602")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00602") to be Allele but got {result}'
    )


def test_BoLA_N_00801():
    result = parse("BoLA-N:00801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00801") to be Allele but got {result}'
    )


def test_BoLA_N_00802():
    result = parse("BoLA-N:00802")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00802") to be Allele but got {result}'
    )


def test_BoLA_N_00901():
    result = parse("BoLA-N:00901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00901") to be Allele but got {result}'
    )


def test_BoLA_N_00902():
    result = parse("BoLA-N:00902")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:00902") to be Allele but got {result}'
    )


def test_BoLA_N_01001():
    result = parse("BoLA-N:01001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01001") to be Allele but got {result}'
    )


def test_BoLA_N_01101():
    result = parse("BoLA-N:01101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01101") to be Allele but got {result}'
    )


def test_BoLA_N_01201():
    result = parse("BoLA-N:01201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01201") to be Allele but got {result}'
    )


def test_BoLA_N_01301():
    result = parse("BoLA-N:01301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01301") to be Allele but got {result}'
    )


def test_BoLA_N_01302():
    result = parse("BoLA-N:01302")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01302") to be Allele but got {result}'
    )


def test_BoLA_N_01401():
    result = parse("BoLA-N:01401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01401") to be Allele but got {result}'
    )


def test_BoLA_N_01402():
    result = parse("BoLA-N:01402")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01402") to be Allele but got {result}'
    )


def test_BoLA_N_01501():
    result = parse("BoLA-N:01501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01501") to be Allele but got {result}'
    )


def test_BoLA_N_01502():
    result = parse("BoLA-N:01502")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01502") to be Allele but got {result}'
    )


def test_BoLA_N_01601():
    result = parse("BoLA-N:01601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01601") to be Allele but got {result}'
    )


def test_BoLA_N_01602():
    result = parse("BoLA-N:01602")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01602") to be Allele but got {result}'
    )


def test_BoLA_N_01701():
    result = parse("BoLA-N:01701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01701") to be Allele but got {result}'
    )


def test_BoLA_N_01702():
    result = parse("BoLA-N:01702")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01702") to be Allele but got {result}'
    )


def test_BoLA_N_01801():
    result = parse("BoLA-N:01801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01801") to be Allele but got {result}'
    )


def test_BoLA_N_01802():
    result = parse("BoLA-N:01802")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01802") to be Allele but got {result}'
    )


def test_BoLA_N_01901():
    result = parse("BoLA-N:01901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:01901") to be Allele but got {result}'
    )


def test_BoLA_N_02001():
    result = parse("BoLA-N:02001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02001") to be Allele but got {result}'
    )


def test_BoLA_N_02101():
    result = parse("BoLA-N:02101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02101") to be Allele but got {result}'
    )


def test_BoLA_N_02201():
    result = parse("BoLA-N:02201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02201") to be Allele but got {result}'
    )


def test_BoLA_N_02301():
    result = parse("BoLA-N:02301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02301") to be Allele but got {result}'
    )


def test_BoLA_N_02401():
    result = parse("BoLA-N:02401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02401") to be Allele but got {result}'
    )


def test_BoLA_N_02402():
    result = parse("BoLA-N:02402")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02402") to be Allele but got {result}'
    )


def test_BoLA_N_02501():
    result = parse("BoLA-N:02501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02501") to be Allele but got {result}'
    )


def test_BoLA_N_02601():
    result = parse("BoLA-N:02601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02601") to be Allele but got {result}'
    )


def test_BoLA_N_02602():
    result = parse("BoLA-N:02602")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02602") to be Allele but got {result}'
    )


def test_BoLA_N_02701():
    result = parse("BoLA-N:02701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02701") to be Allele but got {result}'
    )


def test_BoLA_N_02702():
    result = parse("BoLA-N:02702")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02702") to be Allele but got {result}'
    )


def test_BoLA_N_02801():
    result = parse("BoLA-N:02801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02801") to be Allele but got {result}'
    )


def test_BoLA_N_02901():
    result = parse("BoLA-N:02901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:02901") to be Allele but got {result}'
    )


def test_BoLA_N_03001():
    result = parse("BoLA-N:03001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03001") to be Allele but got {result}'
    )


def test_BoLA_N_03101():
    result = parse("BoLA-N:03101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03101") to be Allele but got {result}'
    )


def test_BoLA_N_03401():
    result = parse("BoLA-N:03401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03401") to be Allele but got {result}'
    )


def test_BoLA_N_03501():
    result = parse("BoLA-N:03501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03501") to be Allele but got {result}'
    )


def test_BoLA_N_03601():
    result = parse("BoLA-N:03601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03601") to be Allele but got {result}'
    )


def test_BoLA_N_03701():
    result = parse("BoLA-N:03701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03701") to be Allele but got {result}'
    )


def test_BoLA_N_03801():
    result = parse("BoLA-N:03801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03801") to be Allele but got {result}'
    )


def test_BoLA_N_03901():
    result = parse("BoLA-N:03901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:03901") to be Allele but got {result}'
    )


def test_BoLA_N_04001():
    result = parse("BoLA-N:04001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04001") to be Allele but got {result}'
    )


def test_BoLA_N_04101():
    result = parse("BoLA-N:04101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04101") to be Allele but got {result}'
    )


def test_BoLA_N_04201():
    result = parse("BoLA-N:04201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04201") to be Allele but got {result}'
    )


def test_BoLA_N_04301():
    result = parse("BoLA-N:04301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04301") to be Allele but got {result}'
    )


def test_BoLA_N_04401():
    result = parse("BoLA-N:04401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04401") to be Allele but got {result}'
    )


def test_BoLA_N_04501():
    result = parse("BoLA-N:04501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04501") to be Allele but got {result}'
    )


def test_BoLA_N_04601():
    result = parse("BoLA-N:04601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04601") to be Allele but got {result}'
    )


def test_BoLA_N_04701():
    result = parse("BoLA-N:04701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04701") to be Allele but got {result}'
    )


def test_BoLA_N_04801():
    result = parse("BoLA-N:04801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04801") to be Allele but got {result}'
    )


def test_BoLA_N_04901():
    result = parse("BoLA-N:04901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:04901") to be Allele but got {result}'
    )


def test_BoLA_N_05001():
    result = parse("BoLA-N:05001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:05001") to be Allele but got {result}'
    )


def test_BoLA_N_05101():
    result = parse("BoLA-N:05101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:05101") to be Allele but got {result}'
    )


def test_BoLA_N_05201():
    result = parse("BoLA-N:05201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:05201") to be Allele but got {result}'
    )


def test_BoLA_N_05301():
    result = parse("BoLA-N:05301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:05301") to be Allele but got {result}'
    )


def test_BoLA_N_05401():
    result = parse("BoLA-N:05401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:05401") to be Allele but got {result}'
    )


def test_BoLA_N_05501():
    result = parse("BoLA-N:05501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:05501") to be Allele but got {result}'
    )


def test_BoLA_N_05601():
    result = parse("BoLA-N:05601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-N:05601") to be Allele but got {result}'
    )


def test_BoLA_NC1_00101():
    result = parse("BoLA-NC1:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1:00101") to be Allele but got {result}'
    )


def test_BoLA_NC1_00201():
    result = parse("BoLA-NC1:00201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1:00201") to be Allele but got {result}'
    )


def test_BoLA_NC1_00301():
    result = parse("BoLA-NC1:00301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1:00301") to be Allele but got {result}'
    )


def test_BoLA_NC1_00401():
    result = parse("BoLA-NC1:00401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC1:00401") to be Allele but got {result}'
    )


def test_BoLA_NC2_00101():
    result = parse("BoLA-NC2:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC2:00101") to be Allele but got {result}'
    )


def test_BoLA_NC2_00102():
    result = parse("BoLA-NC2:00102")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC2:00102") to be Allele but got {result}'
    )


def test_BoLA_NC3_00101():
    result = parse("BoLA-NC3:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC3:00101") to be Allele but got {result}'
    )


def test_BoLA_NC4_00101():
    result = parse("BoLA-NC4:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC4:00101") to be Allele but got {result}'
    )


def test_BoLA_NC4_00201():
    result = parse("BoLA-NC4:00201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-NC4:00201") to be Allele but got {result}'
    )


def test_BoLA_HD6():
    result = parse("BoLA-HD6")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-HD6"): {result}'
    )


def test_BoLA_JSP_1():
    result = parse("BoLA-JSP.1")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-JSP.1"): {result}'
    )


def test_BoLA_T2c():
    result = parse("BoLA-T2c")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-T2c"): {result}'
    )


def test_BoLA_T2b():
    result = parse("BoLA-T2b")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-T2b"): {result}'
    )


def test_BoLA_T2a():
    result = parse("BoLA-T2a")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-T2a"): {result}'
    )


def test_BoLA_T7():
    result = parse("BoLA-T7")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-T7"): {result}'
    )


def test_BoLA_D18_4():
    result = parse("BoLA-D18.4")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-D18.4"): {result}'
    )


def test_BoLA_AW10():
    result = parse("BoLA-AW10")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-AW10"): {result}'
    )


def test_BoLA_T5():
    result = parse("BoLA-T5")
    assert result.__class__ in (Gene, Allele, AlleleWithoutGene), (
        f'Unexpected type for parse("BoLA-T5"): {result}'
    )


def test_BoLA_1_00901():
    result = parse("BoLA-1:00901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:00901") to be Allele but got {result}'
    )


def test_BoLA_1_00902():
    result = parse("BoLA-1:00902")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:00902") to be Allele but got {result}'
    )


def test_BoLA_1_01901():
    result = parse("BoLA-1:01901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:01901") to be Allele but got {result}'
    )


def test_BoLA_1_02001():
    result = parse("BoLA-1:02001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:02001") to be Allele but got {result}'
    )


def test_BoLA_1_02101():
    result = parse("BoLA-1:02101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:02101") to be Allele but got {result}'
    )


def test_BoLA_1_02301():
    result = parse("BoLA-1:02301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:02301") to be Allele but got {result}'
    )


def test_BoLA_1_02801():
    result = parse("BoLA-1:02801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:02801") to be Allele but got {result}'
    )


def test_BoLA_1_02901():
    result = parse("BoLA-1:02901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:02901") to be Allele but got {result}'
    )


def test_BoLA_1_03101():
    result = parse("BoLA-1:03101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:03101") to be Allele but got {result}'
    )


def test_BoLA_1_03102():
    result = parse("BoLA-1:03102")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:03102") to be Allele but got {result}'
    )


def test_BoLA_1_04201():
    result = parse("BoLA-1:04201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:04201") to be Allele but got {result}'
    )


def test_BoLA_1_04901():
    result = parse("BoLA-1:04901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:04901") to be Allele but got {result}'
    )


def test_BoLA_1_06101():
    result = parse("BoLA-1:06101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:06101") to be Allele but got {result}'
    )


def test_BoLA_1_06701():
    result = parse("BoLA-1:06701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:06701") to be Allele but got {result}'
    )


def test_BoLA_1_07401():
    result = parse("BoLA-1:07401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-1:07401") to be Allele but got {result}'
    )


def test_BoLA_2_00501():
    result = parse("BoLA-2:00501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:00501") to be Allele but got {result}'
    )


def test_BoLA_2_00601():
    result = parse("BoLA-2:00601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:00601") to be Allele but got {result}'
    )


def test_BoLA_2_00602():
    result = parse("BoLA-2:00602")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:00602") to be Allele but got {result}'
    )


def test_BoLA_2_00801():
    result = parse("BoLA-2:00801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:00801") to be Allele but got {result}'
    )


def test_BoLA_2_00802():
    result = parse("BoLA-2:00802")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:00802") to be Allele but got {result}'
    )


def test_BoLA_2_01201():
    result = parse("BoLA-2:01201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:01201") to be Allele but got {result}'
    )


def test_BoLA_2_01601():
    result = parse("BoLA-2:01601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:01601") to be Allele but got {result}'
    )


def test_BoLA_2_01602():
    result = parse("BoLA-2:01602")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:01602") to be Allele but got {result}'
    )


def test_BoLA_2_01801():
    result = parse("BoLA-2:01801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:01801") to be Allele but got {result}'
    )


def test_BoLA_2_01802():
    result = parse("BoLA-2:01802")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:01802") to be Allele but got {result}'
    )


def test_BoLA_2_02201():
    result = parse("BoLA-2:02201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:02201") to be Allele but got {result}'
    )


def test_BoLA_2_02501():
    result = parse("BoLA-2:02501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:02501") to be Allele but got {result}'
    )


def test_BoLA_2_02601():
    result = parse("BoLA-2:02601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:02601") to be Allele but got {result}'
    )


def test_BoLA_2_02602():
    result = parse("BoLA-2:02602")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:02602") to be Allele but got {result}'
    )


def test_BoLA_2_02603():
    result = parse("BoLA-2:02603")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:02603") to be Allele but got {result}'
    )


def test_BoLA_2_03001():
    result = parse("BoLA-2:03001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:03001") to be Allele but got {result}'
    )


def test_BoLA_2_03202():
    result = parse("BoLA-2:03202")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:03202") to be Allele but got {result}'
    )


def test_BoLA_2_04301():
    result = parse("BoLA-2:04301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:04301") to be Allele but got {result}'
    )


def test_BoLA_2_04401():
    result = parse("BoLA-2:04401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:04401") to be Allele but got {result}'
    )


def test_BoLA_2_04402():
    result = parse("BoLA-2:04402")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:04402") to be Allele but got {result}'
    )


def test_BoLA_2_04501():
    result = parse("BoLA-2:04501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:04501") to be Allele but got {result}'
    )


def test_BoLA_2_04601():
    result = parse("BoLA-2:04601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:04601") to be Allele but got {result}'
    )


def test_BoLA_2_04701():
    result = parse("BoLA-2:04701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:04701") to be Allele but got {result}'
    )


def test_BoLA_2_04801():
    result = parse("BoLA-2:04801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:04801") to be Allele but got {result}'
    )


def test_BoLA_2_05401():
    result = parse("BoLA-2:05401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:05401") to be Allele but got {result}'
    )


def test_BoLA_2_05501():
    result = parse("BoLA-2:05501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:05501") to be Allele but got {result}'
    )


def test_BoLA_2_05601():
    result = parse("BoLA-2:05601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:05601") to be Allele but got {result}'
    )


def test_BoLA_2_05701():
    result = parse("BoLA-2:05701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:05701") to be Allele but got {result}'
    )


def test_BoLA_2_06001():
    result = parse("BoLA-2:06001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:06001") to be Allele but got {result}'
    )


def test_BoLA_2_06201():
    result = parse("BoLA-2:06201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:06201") to be Allele but got {result}'
    )


def test_BoLA_2_06901():
    result = parse("BoLA-2:06901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:06901") to be Allele but got {result}'
    )


def test_BoLA_2_07001():
    result = parse("BoLA-2:07001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:07001") to be Allele but got {result}'
    )


def test_BoLA_2_07101():
    result = parse("BoLA-2:07101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-2:07101") to be Allele but got {result}'
    )


def test_BoLA_3_00101():
    result = parse("BoLA-3:00101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:00101") to be Allele but got {result}'
    )


def test_BoLA_3_00102():
    result = parse("BoLA-3:00102")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:00102") to be Allele but got {result}'
    )


def test_BoLA_3_00103():
    result = parse("BoLA-3:00103")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:00103") to be Allele but got {result}'
    )


def test_BoLA_3_00201():
    result = parse("BoLA-3:00201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:00201") to be Allele but got {result}'
    )


def test_BoLA_3_00401():
    result = parse("BoLA-3:00401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:00401") to be Allele but got {result}'
    )


def test_BoLA_3_00402():
    result = parse("BoLA-3:00402")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:00402") to be Allele but got {result}'
    )


def test_BoLA_3_00403():
    result = parse("BoLA-3:00403")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:00403") to be Allele but got {result}'
    )


def test_BoLA_3_01001():
    result = parse("BoLA-3:01001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:01001") to be Allele but got {result}'
    )


def test_BoLA_3_01101():
    result = parse("BoLA-3:01101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:01101") to be Allele but got {result}'
    )


def test_BoLA_3_01701():
    result = parse("BoLA-3:01701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:01701") to be Allele but got {result}'
    )


def test_BoLA_3_01702():
    result = parse("BoLA-3:01702")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:01702") to be Allele but got {result}'
    )


def test_BoLA_3_01703():
    result = parse("BoLA-3:01703")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:01703") to be Allele but got {result}'
    )


def test_BoLA_3_02701():
    result = parse("BoLA-3:02701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:02701") to be Allele but got {result}'
    )


def test_BoLA_3_02702():
    result = parse("BoLA-3:02702")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:02702") to be Allele but got {result}'
    )


def test_BoLA_3_03501():
    result = parse("BoLA-3:03501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:03501") to be Allele but got {result}'
    )


def test_BoLA_3_03601():
    result = parse("BoLA-3:03601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:03601") to be Allele but got {result}'
    )


def test_BoLA_3_03701():
    result = parse("BoLA-3:03701")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:03701") to be Allele but got {result}'
    )


def test_BoLA_3_03801():
    result = parse("BoLA-3:03801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:03801") to be Allele but got {result}'
    )


def test_BoLA_3_05001():
    result = parse("BoLA-3:05001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:05001") to be Allele but got {result}'
    )


def test_BoLA_3_05002():
    result = parse("BoLA-3:05002")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:05002") to be Allele but got {result}'
    )


def test_BoLA_3_05101():
    result = parse("BoLA-3:05101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:05101") to be Allele but got {result}'
    )


def test_BoLA_3_05201():
    result = parse("BoLA-3:05201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:05201") to be Allele but got {result}'
    )


def test_BoLA_3_05301():
    result = parse("BoLA-3:05301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:05301") to be Allele but got {result}'
    )


def test_BoLA_3_05801():
    result = parse("BoLA-3:05801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:05801") to be Allele but got {result}'
    )


def test_BoLA_3_05901():
    result = parse("BoLA-3:05901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:05901") to be Allele but got {result}'
    )


def test_BoLA_3_06501():
    result = parse("BoLA-3:06501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:06501") to be Allele but got {result}'
    )


def test_BoLA_3_06601():
    result = parse("BoLA-3:06601")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:06601") to be Allele but got {result}'
    )


def test_BoLA_3_06602():
    result = parse("BoLA-3:06602")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:06602") to be Allele but got {result}'
    )


def test_BoLA_3_06801():
    result = parse("BoLA-3:06801")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:06801") to be Allele but got {result}'
    )


def test_BoLA_3_07301():
    result = parse("BoLA-3:07301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-3:07301") to be Allele but got {result}'
    )


def test_BoLA_4_02401():
    result = parse("BoLA-4:02401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-4:02401") to be Allele but got {result}'
    )


def test_BoLA_4_02402():
    result = parse("BoLA-4:02402")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-4:02402") to be Allele but got {result}'
    )


def test_BoLA_4_06301():
    result = parse("BoLA-4:06301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-4:06301") to be Allele but got {result}'
    )


def test_BoLA_5_00301():
    result = parse("BoLA-5:00301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5:00301") to be Allele but got {result}'
    )


def test_BoLA_5_03901():
    result = parse("BoLA-5:03901")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5:03901") to be Allele but got {result}'
    )


def test_BoLA_5_06401():
    result = parse("BoLA-5:06401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5:06401") to be Allele but got {result}'
    )


def test_BoLA_5_07201():
    result = parse("BoLA-5:07201")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-5:07201") to be Allele but got {result}'
    )


def test_BoLA_6_01301():
    result = parse("BoLA-6:01301")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:01301") to be Allele but got {result}'
    )


def test_BoLA_6_01302():
    result = parse("BoLA-6:01302")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:01302") to be Allele but got {result}'
    )


def test_BoLA_6_01401():
    result = parse("BoLA-6:01401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:01401") to be Allele but got {result}'
    )


def test_BoLA_6_01402():
    result = parse("BoLA-6:01402")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:01402") to be Allele but got {result}'
    )


def test_BoLA_6_01501():
    result = parse("BoLA-6:01501")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:01501") to be Allele but got {result}'
    )


def test_BoLA_6_01502():
    result = parse("BoLA-6:01502")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:01502") to be Allele but got {result}'
    )


def test_BoLA_6_03401():
    result = parse("BoLA-6:03401")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:03401") to be Allele but got {result}'
    )


def test_BoLA_6_04001():
    result = parse("BoLA-6:04001")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:04001") to be Allele but got {result}'
    )


def test_BoLA_6_04101():
    result = parse("BoLA-6:04101")
    assert result.__class__ is Allele, (
        f'Expected parse("BoLA-6:04101") to be Allele but got {result}'
    )
